@@ -21,7 +21,12 @@ else
 	echo "Running acr..."
 	acr -p
 fi
+V=`./configure -qV | cut -d - -f -1`
+meson rewrite kwargs set project / version "$V"
 if [ -n "$1" ]; then
 	echo "./configure $*"
 	./configure $*
 fi
+
+[ -z "$EDITOR" ] && EDITOR=vim
+$EDITOR README.md
