@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2014-2016 - pancake */
+/* radare - LGPL - Copyright 2014-2020 - pancake */
 
 #include <r_core.h>
 
@@ -30,17 +30,18 @@ static void sigusr1(int s) {
 }
 
 static void sigusr2(int s) {
-	(void)openself();
+	(void)openself ();
 	r_core_cmd0 (core, "=H&");
 }
 
 static void _libwrap_init() __attribute__ ((constructor));
-static void _libwrap_init() {
+static void _libwrap_init(void) {
 	char *web;
 	r_sys_signal (SIGUSR1, sigusr1);
 	r_sys_signal (SIGUSR2, sigusr2);
-	printf ("libr2 initialized. send SIGUSR1 to %d in order to reach the r2 prompt\n", getpid ());
-	printf ("kill -USR1 %d\n", getpid ());
+	int pid = r_sys_getpid ();
+	printf ("libr2 initialized. send SIGUSR1 to %d in order to reach the r2 prompt\n", pid);
+	printf ("kill -USR1 %d\n", pid);
 	fflush (stdout);
 	web = r_sys_getenv ("RARUN2_WEB");
 	core = r_core_new ();
@@ -54,7 +55,7 @@ static void _libwrap_init() {
 	// TODO: open io_self
 }
 #elif __WINDOWS__
-void alloc_console() {
+void alloc_console(void) {
 	CONSOLE_SCREEN_BUFFER_INFO coninfo;
 	HANDLE hStdin = GetStdHandle (STD_INPUT_HANDLE);
 	DWORD lpMode;
@@ -71,7 +72,7 @@ void alloc_console() {
 	freopen ("conout$", "w", stderr);
 }
 
-static void start_r2() {
+static void start_r2(void) {
 	core = r_core_new ();
 	r_core_loadlibs (core, R_CORE_LOADLIBS_ALL, NULL);
 	RCoreFile *fd = r_core_file_open (core, "self://", R_PERM_RW, 0);
