@@ -1,9 +1,9 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include <r_main.h>
 #include <r_util.h>
 
-#if EMSCRIPTEN
+#if EMSCRIPTEN__TODO
 #include <emscripten.h>
 static RCore *core = NULL;
 
@@ -61,35 +61,35 @@ static void r2cmd(int in, int out, const char *cmd) {
                         break;
                 }
                 write (1, buf, n);
-		if (n != bufsz) {
-			break;
-		}
+			if (n != bufsz) {
+				break;
+			}
         }
         free (buf);
         write (1, "\n", 1);
 }
 
-static int r_main_r2pipe(int argc, char **argv) {
+static int r_main_r2pipe(int argc, const char **argv) {
         int i, rc = 0;
         char *_in = r_sys_getenv ("R2PIPE_IN");
         char *_out = r_sys_getenv ("R2PIPE_OUT");
-        if (_in && _out) {
+        if (R_STR_ISNOTEMPTY (_in) && R_STR_ISNOTEMPTY (_out)) {
 		int in = atoi (_in);
 		int out = atoi (_out);
 		for (i = 1; i < argc; i++) {
 			r2cmd (in, out, argv[i]);
 		}
-        } else {
+	} else {
 		eprintf ("Error: R2PIPE_(IN|OUT) environment not set\n");
 		eprintf ("Usage: r2 -c '!*r2p x' # run commands via r2pipe\n");
-                rc = 1;
+		rc = 1;
 	}
 	free (_in);
 	free (_out);
-        return rc;
+	return rc;
 }
 
-int main(int argc, char **argv) {
+int main(int argc, const char **argv) {
 	if (argc > 0 && strstr (argv[0], "r2p")) {
 		return r_main_r2pipe (argc, argv);
 	}
