@@ -1,10 +1,11 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2021 - pancake, nibble */
 
 #include <r_anal.h>
 #include <r_util.h>
 #include <r_list.h>
 #include <r_io.h>
 #include <config.h>
+#include "../config.h"
 
 R_LIB_VERSION(r_anal);
 
@@ -64,6 +65,18 @@ static void zign_rename_for(REvent *ev, int type, void *user, void *data) {
 void r_anal_hint_storage_init(RAnal *a);
 void r_anal_hint_storage_fini(RAnal *a);
 
+static void r_meta_item_fini(RAnalMetaItem *item) {
+	free (item->str);
+}
+
+static void r_meta_item_free(void *_item) {
+	if (_item) {
+		RAnalMetaItem *item = _item;
+		r_meta_item_fini (item);
+		free (item);
+	}
+}
+
 R_API RAnal *r_anal_new(void) {
 	int i;
 	RAnal *anal = R_NEW0 (RAnal);
@@ -94,9 +107,8 @@ R_API RAnal *r_anal_new(void) {
 	r_event_hook (anal->zign_spaces.event, R_SPACE_EVENT_UNSET, zign_unset_for, NULL);
 	r_event_hook (anal->zign_spaces.event, R_SPACE_EVENT_COUNT, zign_count_for, NULL);
 	r_event_hook (anal->zign_spaces.event, R_SPACE_EVENT_RENAME, zign_rename_for, NULL);
-	anal->sdb_fcns = sdb_ns (anal->sdb, "fcns", 1);
-	anal->sdb_meta = sdb_ns (anal->sdb, "meta", 1);
 	r_anal_hint_storage_init (anal);
+	r_interval_tree_init (&anal->meta, r_meta_item_free);
 	anal->sdb_types = sdb_ns (anal->sdb, "types", 1);
 	anal->sdb_fmts = sdb_ns (anal->sdb, "spec", 1);
 	anal->sdb_cc = sdb_ns (anal->sdb, "cc", 1);
@@ -105,6 +117,7 @@ R_API RAnal *r_anal_new(void) {
 	anal->sdb_classes_attrs = sdb_ns (anal->sdb_classes, "attrs", 1);
 	anal->zign_path = strdup ("");
 	anal->cb_printf = (PrintfCallback) printf;
+	anal->esil = NULL; // nul on purpose, otherwise many analysisi fail O_O
 	(void)r_anal_pin_init (anal);
 	(void)r_anal_xrefs_init (anal);
 	anal->diff_thbb = R_ANAL_THRESHOLDBB;
@@ -116,9 +129,9 @@ R_API RAnal *r_anal_new(void) {
 	anal->last_disasm_reg = NULL;
 	anal->stackptr = 0;
 	anal->lineswidth = 0;
-	anal->fcns = r_list_newf (r_anal_function_free);
-	anal->refs = r_anal_ref_list_new ();
+	anal->fcns = r_list_newf ((RListFree)r_anal_function_free);
 	anal->leaddrs = NULL;
+	anal->imports = r_list_newf (free);
 	r_anal_set_bits (anal, 32);
 	anal->plugins = r_list_newf ((RListFree) r_anal_plugin_free);
 	if (anal->plugins) {
@@ -126,7 +139,6 @@ R_API RAnal *r_anal_new(void) {
 			r_anal_add (anal, anal_static_plugins[i]);
 		}
 	}
-	anal->cmdtail = r_strbuf_new (NULL);
 	return anal;
 }
 
@@ -143,11 +155,13 @@ R_API RAnal *r_anal_free(RAnal *a) {
 		return NULL;
 	}
 	/* TODO: Free anals here */
+	free (a->pincmd);
 	r_list_free (a->fcns);
 	ht_up_free (a->ht_addr_fun);
 	ht_pp_free (a->ht_name_fun);
 	set_u_free (a->visited);
 	r_anal_hint_storage_fini (a);
+	r_interval_tree_fini (&a->meta);
 	free (a->cpu);
 	free (a->os);
 	free (a->zign_path);
@@ -156,10 +170,8 @@ R_API RAnal *r_anal_free(RAnal *a) {
 	r_spaces_fini (&a->meta_spaces);
 	r_spaces_fini (&a->zign_spaces);
 	r_anal_pin_fini (a);
-	r_list_free (a->refs);
 	r_syscall_free (a->syscall);
 	r_reg_free (a->reg);
-	r_anal_op_free (a->queued);
 	ht_up_free (a->dict_refs);
 	ht_up_free (a->dict_xrefs);
 	r_list_free (a->leaddrs);
@@ -169,7 +181,7 @@ R_API RAnal *r_anal_free(RAnal *a) {
 		a->esil = NULL;
 	}
 	free (a->last_disasm_reg);
-	r_strbuf_free (a->cmdtail);
+	r_list_free (a->imports);
 	r_str_constpool_fini (&a->constpool);
 	free (a);
 	return NULL;
@@ -179,6 +191,22 @@ R_API void r_anal_set_user_ptr(RAnal *anal, void *user) {
 	anal->user = user;
 }
 
+R_API bool r_anal_esil_use(RAnal *anal, const char *name) {
+	RListIter *it;
+	RAnalEsilPlugin *h;
+
+	if (anal) {
+		r_list_foreach (anal->esil_plugins, it, h) {
+			if (!h->name || strcmp (h->name, name)) {
+				continue;
+			}
+			anal->esil_cur = h;
+			return true;
+		}
+	}
+	return false;
+}
+
 R_API int r_anal_add(RAnal *anal, RAnalPlugin *foo) {
 	if (foo->init) {
 		foo->init (anal->user);
@@ -192,7 +220,6 @@ R_API bool r_anal_use(RAnal *anal, const char *name) {
 	RAnalPlugin *h;
 
 	if (anal) {
-		bool change = anal->cur && strcmp (anal->cur->name, name);
 		r_list_foreach (anal->plugins, it, h) {
 			if (!h->name || strcmp (h->name, name)) {
 				continue;
@@ -204,10 +231,7 @@ R_API bool r_anal_use(RAnal *anal, const char *name) {
 			}
 #endif
 			anal->cur = h;
-			r_anal_set_reg_profile (anal);
-			if (change) {
-				r_anal_set_fcnsign (anal, NULL);
-			}
+			r_anal_set_reg_profile (anal, NULL);
 			return true;
 		}
 	}
@@ -220,7 +244,10 @@ R_API char *r_anal_get_reg_profile(RAnal *anal) {
 }
 
 // deprecate.. or at least reuse get_reg_profile...
-R_API bool r_anal_set_reg_profile(RAnal *anal) {
+R_API bool r_anal_set_reg_profile(RAnal *anal, const char *p) {
+	if (p) {
+		return r_reg_set_profile_string (anal->reg, p);
+	}
 	bool ret = false;
 	if (anal && anal->cur && anal->cur->set_reg_profile) {
 		ret = anal->cur->set_reg_profile (anal);
@@ -235,27 +262,6 @@ R_API bool r_anal_set_reg_profile(RAnal *anal) {
 	return ret;
 }
 
-R_API bool r_anal_set_fcnsign(RAnal *anal, const char *name) {
-	const char *dirPrefix = r_sys_prefix (NULL);
-	const char *arch = (anal->cur && anal->cur->arch) ? anal->cur->arch : R_SYS_ARCH;
-	const char *file = (name && *name)
-		? sdb_fmt (R_JOIN_3_PATHS ("%s", R2_SDB_FCNSIGN, "%s.sdb"), dirPrefix, name)
-		: sdb_fmt (R_JOIN_3_PATHS ("%s", R2_SDB_FCNSIGN, "%s-%s-%d.sdb"), dirPrefix,
-			anal->os, arch, anal->bits);
-	if (r_file_exists (file)) {
-		sdb_close (anal->sdb_fcnsign);
-		sdb_free (anal->sdb_fcnsign);
-		anal->sdb_fcnsign = sdb_new (0, file, 0);
-		sdb_ns_set (anal->sdb, "fcnsign", anal->sdb_fcnsign);
-		return (anal->sdb_fcnsign != NULL);
-	}
-	return false;
-}
-
-R_API const char *r_anal_get_fcnsign(RAnal *anal, const char *sym) {
-	return sdb_const_get (anal->sdb_fcnsign, sym, 0);
-}
-
 R_API bool r_anal_set_triplet(RAnal *anal, const char *os, const char *arch, int bits) {
 	r_return_val_if_fail (anal, false);
 	if (!os || !*os) {
@@ -284,11 +290,24 @@ static void sdb_concat_by_path(Sdb *s, const char *path) {
 R_API bool r_anal_set_os(RAnal *anal, const char *os) {
 	Sdb *types = anal->sdb_types;
 	const char *dir_prefix = r_sys_prefix (NULL);
-	const char *dbpath = sdb_fmt (R_JOIN_3_PATHS ("%s", R2_SDB_FCNSIGN, "types-%s.sdb"),
-		dir_prefix, os);
+	SdbGperf *gp = r_anal_get_gperf_types (os);
+	if (gp) {
+		Sdb *gd = sdb_new0 ();
+		sdb_open_gperf (gd, gp);
+		sdb_reset (anal->sdb_types);
+		sdb_merge (anal->sdb_types, gd);
+		sdb_close (gd);
+		sdb_free (gd);
+		return r_anal_set_triplet (anal, os, NULL, -1);
+	}
+
+	// char *ff = r_str_newf ("types-%s.sdb", os);
+	// char *dbpath = r_file_new (dir_prefix, r2_sdb_fcnsign, ff);
+	char *dbpath = r_str_newf ("%s/%s/types-%s.sdb", dir_prefix, R2_SDB_FCNSIGN, os);
 	if (r_file_exists (dbpath)) {
 		sdb_concat_by_path (types, dbpath);
 	}
+	free (dbpath);
 	return r_anal_set_triplet (anal, os, NULL, -1);
 }
 
@@ -301,8 +320,7 @@ R_API bool r_anal_set_bits(RAnal *anal, int bits) {
 	case 64:
 		if (anal->bits != bits) {
 			anal->bits = bits;
-			r_anal_set_fcnsign (anal, NULL);
-			r_anal_set_reg_profile (anal);
+			r_anal_set_reg_profile (anal, NULL);
 		}
 		return true;
 	}
@@ -318,10 +336,12 @@ R_API void r_anal_set_cpu(RAnal *anal, const char *cpu) {
 	}
 }
 
-R_API int r_anal_set_big_endian(RAnal *anal, int bigend) {
+R_API void r_anal_set_big_endian(RAnal *anal, int bigend) {
+	r_return_if_fail (anal);
 	anal->big_endian = bigend;
-	anal->reg->big_endian = bigend;
-	return true;
+	if (anal->reg) {
+		anal->reg->big_endian = bigend;
+	}
 }
 
 R_API ut8 *r_anal_mask(RAnal *anal, int size, const ut8 *data, ut64 at) {
@@ -365,10 +385,10 @@ R_API ut8 *r_anal_mask(RAnal *anal, int size, const ut8 *data, ut64 at) {
 }
 
 R_API void r_anal_trace_bb(RAnal *anal, ut64 addr) {
+	r_return_if_fail (anal);
 	RAnalBlock *bbi;
-	RAnalFunction *fcni;
 	RListIter *iter2;
-	fcni = r_anal_get_fcn_in (anal, addr, 0);
+	RAnalFunction *fcni = r_anal_get_fcn_in (anal, addr, 0);
 	if (fcni) {
 		r_list_foreach (fcni->bbs, iter2, bbi) {
 			if (addr >= bbi->addr && addr < (bbi->addr + bbi->size)) {
@@ -379,14 +399,6 @@ R_API void r_anal_trace_bb(RAnal *anal, ut64 addr) {
 	}
 }
 
-R_API void r_anal_colorize_bb(RAnal *anal, ut64 addr, ut32 color) {
-	RAnalBlock *bbi;
-	bbi = r_anal_bb_from_offset (anal, addr);
-	if (bbi) {
-		bbi->colorize = color;
-	}
-}
-
 R_API RList* r_anal_get_fcns (RAnal *anal) {
 	// avoid received to free this thing
 	anal->fcns->free = NULL;
@@ -428,19 +440,20 @@ R_API bool r_anal_op_is_eob(RAnalOp *op) {
 	}
 }
 
-R_API int r_anal_purge (RAnal *anal) {
-	sdb_reset (anal->sdb_fcns);
-	sdb_reset (anal->sdb_meta);
+R_API void r_anal_purge(RAnal *anal) {
 	r_anal_hint_clear (anal);
+	r_interval_tree_fini (&anal->meta);
+	r_interval_tree_init (&anal->meta, r_meta_item_free);
 	sdb_reset (anal->sdb_types);
 	sdb_reset (anal->sdb_zigns);
 	sdb_reset (anal->sdb_classes);
 	sdb_reset (anal->sdb_classes_attrs);
+	r_anal_pin_fini (anal);
+	r_anal_pin_init (anal);
+	sdb_reset (anal->sdb_cc);
 	r_list_free (anal->fcns);
-	anal->fcns = r_list_newf (r_anal_function_free);
-	r_list_free (anal->refs);
-	anal->refs = r_anal_ref_list_new ();
-	return 0;
+	anal->fcns = r_list_newf ((RListFree)r_anal_function_free);
+	r_anal_purge_imports (anal);
 }
 
 R_API int r_anal_archinfo(RAnal *anal, int query) {
@@ -448,6 +461,7 @@ R_API int r_anal_archinfo(RAnal *anal, int query) {
 	switch (query) {
 	case R_ANAL_ARCHINFO_MIN_OP_SIZE:
 	case R_ANAL_ARCHINFO_MAX_OP_SIZE:
+	case R_ANAL_ARCHINFO_INV_OP_SIZE:
 	case R_ANAL_ARCHINFO_ALIGN:
 		if (anal->cur && anal->cur->archinfo) {
 			return anal->cur->archinfo (anal, query);
@@ -457,7 +471,7 @@ R_API int r_anal_archinfo(RAnal *anal, int query) {
 	return -1;
 }
 
-static int __nonreturn_print_commands(void *p, const char *k, const char *v) {
+static bool __nonreturn_print_commands(void *p, const char *k, const char *v) {
 	RAnal *anal = (RAnal *)p;
 	if (!strncmp (v, "func", strlen ("func") + 1)) {
 		char *query = sdb_fmt ("func.%s.noreturn", k);
@@ -468,10 +482,10 @@ static int __nonreturn_print_commands(void *p, const char *k, const char *v) {
 	if (!strncmp (k, "addr.", 5)) {
 		anal->cb_printf ("tna 0x%s %s\n", k + 5, v);
 	}
-	return 1;
+	return true;
 }
 
-static int __nonreturn_print(void *p, const char *k, const char *v) {
+static bool __nonreturn_print(void *p, const char *k, const char *v) {
 	RAnal *anal = (RAnal *)p;
 	if (!strncmp (k, "func.", 5) && strstr (k, ".noreturn")) {
 		char *s = strdup (k + 5);
@@ -494,7 +508,7 @@ static int __nonreturn_print(void *p, const char *k, const char *v) {
 		}
 		free (off);
 	}
-	return 1;
+	return true;
 }
 
 R_API void r_anal_noreturn_list(RAnal *anal, int mode) {
@@ -519,6 +533,10 @@ R_API bool r_anal_noreturn_add(RAnal *anal, const char *name, ut64 addr) {
 	char *fnl_name = NULL;
 	if (addr != UT64_MAX) {
 		if (sdb_bool_set (TDB, K_NORET_ADDR (addr), true, 0)) {
+			RAnalFunction *fcn = r_anal_get_function_at (anal, addr);
+			if (fcn) {
+				fcn->is_noreturn = true;
+			}
 			return true;
 		}
 	}
@@ -532,6 +550,9 @@ R_API bool r_anal_noreturn_add(RAnal *anal, const char *name, ut64 addr) {
 			return false;
 		}
 		tmp_name = fcn ? fcn->name: fi->name;
+		if (fcn) {
+			fcn->is_noreturn = true;
+		}
 	}
 	if (r_type_func_exist (TDB, tmp_name)) {
 		fnl_name = strdup (tmp_name);
@@ -676,6 +697,9 @@ R_API void r_anal_bind(RAnal *anal, RAnalBind *b) {
 		b->anal = anal;
 		b->get_fcn_in = r_anal_get_fcn_in;
 		b->get_hint = r_anal_hint_get;
+		b->decode = (RAnalDecode)r_anal_op;
+		b->opinit = r_anal_op_init;
+		b->opfini = r_anal_op_fini;
 	}
 }
 
@@ -702,3 +726,33 @@ R_API bool r_anal_is_prelude(RAnal *anal, const ut8 *data, int len) {
 	}
 	return false;
 }
+
+R_API void r_anal_add_import(RAnal *anal, const char *imp) {
+	RListIter *it;
+	const char *eimp;
+	r_list_foreach (anal->imports, it, eimp) {
+		if (!strcmp (eimp, imp)) {
+			return;
+		}
+	}
+	char *cimp = strdup (imp);
+	if (!cimp) {
+		return;
+	}
+	r_list_push (anal->imports, cimp);
+}
+
+R_API void r_anal_remove_import(RAnal *anal, const char *imp) {
+	RListIter *it;
+	const char *eimp;
+	r_list_foreach (anal->imports, it, eimp) {
+		if (!strcmp (eimp, imp)) {
+			r_list_delete (anal->imports, it);
+			return;
+		}
+	}
+}
+
+R_API void r_anal_purge_imports(RAnal *anal) {
+	r_list_purge (anal->imports);
+}
