@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2011-2019 - pancake, Oddcoder */
+/* radare - LGPL - Copyright 2011-2021 - pancake, Oddcoder */
 
 /* Universal calling convention implementation based on sdb */
 
@@ -6,65 +6,125 @@
 #define DB anal->sdb_cc
 
 R_API void r_anal_cc_del(RAnal *anal, const char *name) {
-	int i;
-	sdb_unset (DB, sdb_fmt ("%s", name), 0);
-	sdb_unset (DB, sdb_fmt ("cc.%s.ret", name), 0);
-	sdb_unset (DB, sdb_fmt ("cc.%s.argn", name), 0);
+	r_return_if_fail (anal && name);
+	size_t i;
+	RStrBuf sb;
+	sdb_unset (DB, r_strbuf_initf (&sb, "%s", name), 0);
+	sdb_unset (DB, r_strbuf_setf (&sb, "cc.%s.ret", name), 0);
+	sdb_unset (DB, r_strbuf_setf (&sb, "cc.%s.argn", name), 0);
 	for (i = 0; i < R_ANAL_CC_MAXARG; i++) {
-		sdb_unset (DB, sdb_fmt ("cc.%s.arg%d", name, i), 0);
+		sdb_unset (DB, r_strbuf_setf (&sb, "cc.%s.arg%zu", name, i), 0);
 	}
+	sdb_unset (DB, r_strbuf_setf (&sb, "cc.%s.self", name), 0);
+	sdb_unset (DB, r_strbuf_setf (&sb, "cc.%s.error", name), 0);
+	r_strbuf_fini (&sb);
 }
 
-R_API void r_anal_cc_set(RAnal *anal, const char *expr) {
+R_API bool r_anal_cc_set(RAnal *anal, const char *expr) {
+	r_return_val_if_fail (anal && expr, false);
 	char *e = strdup (expr);
 	char *p = strchr (e, '(');
-	if (p) {
-		*p++ = 0;
+	if (!p) {
+		free (e);
+		return false;
 	}
+	*p++ = 0;
 	char *args = strdup (p);
+	r_str_trim (p);
 	char *end = strchr (args, ')');
-	if (end) {
-		*end++ = 0;
-		RList *retName = r_str_split_list (e, " ", 0);
-		RList *ccArgs = r_str_split_list (args, ",", 0);
-		if (r_list_length (retName) == 2) {
-			const char *ret = r_list_get_n (retName, 0);
-			const char *name = r_list_get_n (retName, 1);
-			sdb_set (DB, name, "cc", 0);
-			sdb_set (DB, sdb_fmt ("cc.%s.ret", name), ret, 0);
-			RListIter *iter;
-			const char *arg;
-			int n = 0;
-			r_list_foreach (ccArgs, iter, arg) {
-				if (!strcmp (arg, "stack")) {
-					sdb_set (DB, sdb_fmt ("cc.%s.argn", name), arg, 0);
-				} else {
-					sdb_set (DB, sdb_fmt ("cc.%s.arg%d", name, n), arg, 0);
-					n++;
-				}
-			}
-		}
-		r_list_free (retName);
-		r_list_free (ccArgs);
+	if (!end) {
+		free (args);
+		free (e);
+		return false;
 	}
+	*end++ = 0;
+	r_str_trim (p);
+	r_str_trim (e);
+	char *ccname = strchr (e, ' ');
+	if (ccname) {
+		*ccname++ = 0;
+		r_str_trim (ccname);
+	} else {
+		free (args);
+		free (e);
+		return false;
+	}
+	sdb_set (DB, ccname, "cc", 0);
+	sdb_set (DB, sdb_fmt ("cc.%s.ret", ccname), e, 0);
+
+	RList *ccArgs = r_str_split_list (args, ",", 0);
+	RListIter *iter;
+	const char *arg;
+	int n = 0;
+	r_list_foreach (ccArgs, iter, arg) {
+		if (!strcmp (arg, "stack")) {
+			sdb_set (DB, sdb_fmt ("cc.%s.argn", ccname), arg, 0);
+		} else {
+			sdb_set (DB, sdb_fmt ("cc.%s.arg%d", ccname, n), arg, 0);
+			n++;
+		}
+	}
+	r_list_free (ccArgs);
 	free (e);
 	free (args);
+	return true;
 }
 
-R_API char *r_anal_cc_get(RAnal *anal, const char *name) {
+R_API bool r_anal_cc_once(RAnal *anal) {
+	return sdb_add (DB, "warn", "once", 0);
+}
+
+R_API void r_anal_cc_get_json(RAnal *anal, PJ *pj, const char *name) {
+	r_return_if_fail (anal && pj && name);
 	int i;
 	// get cc by name and print the expr
 	if (r_str_cmp (sdb_const_get (DB, name, 0), "cc", -1)) {
-		eprintf ("This is not a valid calling convention name\n");
+		return;
+	}
+	const char *ret = sdb_const_get (DB, sdb_fmt ("cc.%s.ret", name), 0);
+	if (!ret) {
+		return;
+	}
+	pj_ks (pj, "ret", ret);
+	char *sig = r_anal_cc_get (anal, name);
+	pj_ks (pj, "signature", sig);
+	free (sig);
+	pj_ka (pj, "args");
+	for (i = 0; i < R_ANAL_CC_MAXARG; i++) {
+		const char *k = sdb_fmt ("cc.%s.arg%d", name, i);
+		const char *arg = sdb_const_get (DB, k, 0);
+		if (!arg) {
+			break;
+		}
+		pj_s (pj, arg);
+	}
+	pj_end (pj);
+	const char *argn = sdb_const_get (DB, sdb_fmt ("cc.%s.argn", name), 0);
+	if (argn) {
+		pj_ks (pj, "argn", argn);
+	}
+	const char *error = r_anal_cc_error (anal, name);
+	if (error) {
+		pj_ks (pj, "error", error);
+	}
+}
+
+R_API char *r_anal_cc_get(RAnal *anal, const char *name) {
+	r_return_val_if_fail (anal && name, NULL);
+	int i;
+	// get cc by name and print the expr
+	if (r_str_cmp (sdb_const_get (DB, name, 0), "cc", -1)) {
+		eprintf ("This is not a valid calling convention name (%s)\n", name);
 		return NULL;
 	}
 	const char *ret = sdb_const_get (DB, sdb_fmt ("cc.%s.ret", name), 0);
 	if (!ret) {
-		eprintf ("Cannot find return key\n");
+		eprintf ("Cannot find return type for %s\n", name);
 		return NULL;
 	}
 	RStrBuf *sb = r_strbuf_new (NULL);
-	r_strbuf_appendf (sb, "%s %s (", ret, name);
+	const char *self = r_anal_cc_self (anal, name);
+	r_strbuf_appendf (sb, "%s %s%s%s (", ret, r_str_get (self), self? ".": "", name);
 	bool isFirst = true;
 	for (i = 0; i < R_ANAL_CC_MAXARG; i++) {
 		const char *k = sdb_fmt ("cc.%s.arg%d", name, i);
@@ -79,20 +139,17 @@ R_API char *r_anal_cc_get(RAnal *anal, const char *name) {
 	if (argn) {
 		r_strbuf_appendf (sb, "%s%s", isFirst? "": ", ", argn);
 	}
-	const char *self = r_anal_cc_self (anal, name);
-	if (self) {
-		r_strbuf_appendf (sb, "%s%s", isFirst? "": ", ", self);
-	}
+	r_strbuf_append (sb, ")");
+
 	const char *error = r_anal_cc_error (anal, name);
 	if (error) {
-		r_strbuf_appendf (sb, "%s%s", isFirst? "": ", ", error);
+		r_strbuf_appendf (sb, " %s", error);
 	}
 
-	r_strbuf_appendf (sb, ");");
+	r_strbuf_append (sb, ";");
 	return r_strbuf_drain (sb);
 }
 
-
 R_API bool r_anal_cc_exist(RAnal *anal, const char *convention) {
 	r_return_val_if_fail (anal && convention, false);
 	const char *x = sdb_const_get (DB, convention, 0);
@@ -100,10 +157,12 @@ R_API bool r_anal_cc_exist(RAnal *anal, const char *convention) {
 }
 
 R_API const char *r_anal_cc_arg(RAnal *anal, const char *convention, int n) {
-	r_return_val_if_fail (anal && convention, NULL);
-	if (n < 0) {
+	r_return_val_if_fail (anal, NULL);
+	r_return_val_if_fail (n >= 0, NULL);
+	if (!convention) {
 		return NULL;
 	}
+
 	const char *query = sdb_fmt ("cc.%s.arg%d", convention, n);
 	const char *ret = sdb_const_get (DB, query, 0);
 	if (!ret) {
@@ -120,6 +179,16 @@ R_API const char *r_anal_cc_self(RAnal *anal, const char *convention) {
 	return self? r_str_constpool_get (&anal->constpool, self): NULL;
 }
 
+R_API void r_anal_cc_set_self(RAnal *anal, const char *convention, const char *self) {
+	r_return_if_fail (anal && convention && self);
+	if (!r_anal_cc_exist (anal, convention)) {
+		return;
+	}
+	RStrBuf sb;
+	sdb_set (DB, r_strbuf_initf (&sb, "cc.%s.self", convention), self, 0);
+	r_strbuf_fini (&sb);
+}
+
 R_API const char *r_anal_cc_error(RAnal *anal, const char *convention) {
 	r_return_val_if_fail (anal && convention, NULL);
 	const char *query = sdb_fmt ("cc.%s.error", convention);
@@ -127,6 +196,16 @@ R_API const char *r_anal_cc_error(RAnal *anal, const char *convention) {
 	return error? r_str_constpool_get (&anal->constpool, error): NULL;
 }
 
+R_API void r_anal_cc_set_error(RAnal *anal, const char *convention, const char *error) {
+	r_return_if_fail (anal && convention && error);
+	if (!r_anal_cc_exist (anal, convention)) {
+		return;
+	}
+	RStrBuf sb;
+	sdb_set (DB, r_strbuf_initf (&sb, "cc.%s.error", convention), error, 0);
+	r_strbuf_fini (&sb);
+}
+
 R_API int r_anal_cc_max_arg(RAnal *anal, const char *cc) {
 	int i = 0;
 	r_return_val_if_fail (anal && DB && cc, 0);
@@ -161,6 +240,21 @@ R_API const char *r_anal_cc_default(RAnal *anal) {
 	return sdb_const_get (DB, "default.cc", 0);
 }
 
+R_API void r_anal_set_cc_default(RAnal *anal, const char *cc) {
+	r_return_if_fail (anal && cc);
+	sdb_set (DB, "default.cc", cc, 0);
+}
+
+R_API const char *r_anal_syscc_default(RAnal *anal) {
+	r_return_val_if_fail (anal, NULL);
+	return sdb_const_get (DB, "default.syscc", 0);
+}
+
+R_API void r_anal_set_syscc_default(RAnal *anal, const char *cc) {
+	r_return_if_fail (anal && cc);
+	sdb_set (DB, "default.syscc", cc, 0);
+}
+
 R_API const char *r_anal_cc_func(RAnal *anal, const char *func_name) {
 	r_return_val_if_fail (anal && func_name, NULL);
 	const char *query = sdb_fmt ("func.%s.cc", func_name);
