@@ -99,7 +99,6 @@
 
 #include <r_types.h>
 #include <r_lib.h>
-#include <r_cmd.h>
 #include <r_sign.h>
 #include <signal.h>
 
@@ -282,9 +281,6 @@ static int header_size = 0;
  */
 
 
-// XXX need more infos on compression of version 5 sigs
-// r_inflate doesn't work with them
-
 #define R_FLIRT_NAME_MAX 1024
 
 typedef struct RFlirtTailByte {
@@ -594,7 +590,7 @@ static int module_match_buffer(RAnal *anal, const RFlirtModule *module,
 						r_anal_function_delete (fcn);
 					}
 				}
-				r_anal_fcn_resize (anal, next_module_function, flirt_fcn_size);
+				r_anal_function_resize (next_module_function, flirt_fcn_size);
 				next_module_function_size = r_anal_function_linear_size (next_module_function);
 				r_anal_trim_jmprefs ((RAnal *)anal, next_module_function);
 			}
@@ -673,10 +669,6 @@ static int node_match_functions(RAnal *anal, const RFlirtNode *root_node) {
 	RListIter *it_func;
 	RAnalFunction *func;
 	r_list_foreach (anal->fcns, it_func, func) {
-		if (func->type != R_ANAL_FCN_TYPE_FCN && func->type != R_ANAL_FCN_TYPE_LOC) { // scan only for unknown functions
-			continue;
-		}
-
 		ut64 func_size = r_anal_function_linear_size (func);
 		ut8 *func_buf = malloc (func_size);
 		if (!func_buf) {
@@ -928,7 +920,7 @@ static ut8 parse_leaf(const RAnal *anal, RBuffer *b, RFlirtNode *node) {
 		}
 #if DEBUG
 		if (crc_length == 0x00 && crc16 != 0x0000) {
-			eprintf ("WARNING non zero crc of zero length @ %04X\n",
+			eprintf ("Warning: non zero crc of zero length @ %04X\n",
 				r_buf_tell (b) + header_size);
 		}
 		eprintf ("crc_len: %02X crc16: %04X\n", crc_length, crc16);
@@ -1226,7 +1218,8 @@ static void print_header(idasig_v5_t *header) {
 #endif
 
 static int parse_header(RBuffer *buf, idasig_v5_t *header) {
-	if (r_buf_read_at (buf, 0, header->magic, sizeof(header->magic)) != sizeof(header->magic)) {
+	r_buf_seek (buf, 0, R_BUF_SET);
+	if (r_buf_read (buf, header->magic, sizeof(header->magic)) != sizeof(header->magic)) {
 		return false;
 	}
 	if (r_buf_read (buf, &header->version, sizeof(header->version)) != sizeof(header->version)) {
@@ -1292,7 +1285,7 @@ static int parse_v10_header(RBuffer *buf, idasig_v10_t *header) {
 
 static RFlirtNode *flirt_parse(const RAnal *anal, RBuffer *flirt_buf) {
 	ut8 *name = NULL;
-	ut8 *buf = NULL, *decompressed_buf = NULL;
+	ut8 *buf = NULL, *dbuf = NULL;
 	RBuffer *r_buf = NULL;
 	int size, decompressed_size;
 	RFlirtNode *node = NULL;
@@ -1365,23 +1358,31 @@ static RFlirtNode *flirt_parse(const RAnal *anal, RBuffer *flirt_buf) {
 #endif
 
 	size = r_buf_size (flirt_buf) - r_buf_tell (flirt_buf);
-	buf = malloc (size);
+	if (!(buf = malloc (size))) {
+		goto exit;
+	}
 	if (r_buf_read (flirt_buf, buf, size) != size) {
 		goto exit;
 	}
 
 	if (header->features & IDASIG__FEATURE__COMPRESSED) {
-		if (version == 5) {
-			eprintf ("Sorry we do not support the signatures version 5 compression.\n");
-			goto exit;
-		}
-		if (!(decompressed_buf = r_inflate (buf, size, NULL, &decompressed_size))) {
-			eprintf ("Decompressing failed.\n");
+		if (version >= 5 && version < 7) {
+			if (!(dbuf = r_inflate_raw (buf, size, NULL, &decompressed_size))) {
+				eprintf ("Decompression failed.\n");
+				goto exit;
+			}
+		} else if (version >= 7) {
+			if (!(dbuf = r_inflate (buf, size, NULL, &decompressed_size))) {
+				eprintf ("Decompression failed.\n");
+				goto exit;
+			}
+		} else {
+			eprintf ("FLIRT signatures version %d is not supported.\n", version);
 			goto exit;
 		}
 
 		R_FREE (buf);
-		buf = decompressed_buf;
+		buf = dbuf;
 		size = decompressed_size;
 	}
 
@@ -1390,7 +1391,7 @@ static RFlirtNode *flirt_parse(const RAnal *anal, RBuffer *flirt_buf) {
 	}
 	r_buf = r_buf_new_with_pointers (buf, size, false);
 #if DEBUG
-	r_file_dump ("sig_dump", r_buf->buf, r_buf_size (r_buf));
+	r_file_dump ("sig_dump", buf, size, false);
 #endif
 	if (parse_tree (anal, r_buf, node)) {
 		ret = node;
