@@ -1,24 +1,16 @@
-/* radare - LGPL - Copyright 2010-2019 - pancake, nibble */
+/* radare - LGPL - Copyright 2010-2020 - pancake, nibble */
 
 #include <r_anal.h>
 #include <r_util.h>
 #include <r_list.h>
 
-#define SDB_VARUSED_FMT "qzdq"
-struct VarUsedType {
-	ut64 fcn_addr;
-	char *type;
-	ut32 scope;
-	st64 delta;
-};
-
-R_API RAnalOp *r_anal_op_new() {
+R_API RAnalOp *r_anal_op_new(void) {
 	RAnalOp *op = R_NEW (RAnalOp);
 	r_anal_op_init (op);
 	return op;
 }
 
-R_API RList *r_anal_op_list_new() {
+R_API RList *r_anal_op_list_new(void) {
 	RList *list = r_list_new ();
 	if (list) {
 		list->free = &r_anal_op_free;
@@ -39,12 +31,10 @@ R_API void r_anal_op_init(RAnalOp *op) {
 	}
 }
 
-R_API bool r_anal_op_fini(RAnalOp *op) {
+R_API void r_anal_op_fini(RAnalOp *op) {
 	if (!op) {
-		return false;
+		return;
 	}
-	r_anal_var_free (op->var);
-	op->var = NULL;
 	r_anal_value_free (op->src[0]);
 	r_anal_value_free (op->src[1]);
 	r_anal_value_free (op->src[2]);
@@ -53,12 +43,13 @@ R_API bool r_anal_op_fini(RAnalOp *op) {
 	op->src[2] = NULL;
 	r_anal_value_free (op->dst);
 	op->dst = NULL;
+	r_list_free (op->access);
+	op->access = NULL;
 	r_strbuf_fini (&op->opex);
 	r_strbuf_fini (&op->esil);
 	r_anal_switch_op_free (op->switch_op);
 	op->switch_op = NULL;
 	R_FREE (op->mnemonic);
-	return true;
 }
 
 R_API void r_anal_op_free(void *_op) {
@@ -70,43 +61,6 @@ R_API void r_anal_op_free(void *_op) {
 	free (_op);
 }
 
-R_API RAnalVar *get_link_var(RAnal *anal, ut64 faddr, RAnalVar *var) {
-	const char *var_local = sdb_fmt ("var.0x%"PFMT64x".%d.%d.%s",
-			faddr, 1, var->delta, "reads");
-	const char *xss = sdb_const_get (anal->sdb_fcns, var_local, 0);
-	ut64 addr = r_num_math (NULL, xss);
-	char *inst_key = r_str_newf ("inst.0x%"PFMT64x".lvar", addr);
-	char *var_def = sdb_get (anal->sdb_fcns, inst_key, 0);
-
-	if (!var_def) {
-		free (inst_key);
-		return NULL;
-	}
-	struct VarUsedType vut;
-	RAnalVar *res = NULL;
-	if (sdb_fmt_tobin (var_def, SDB_VARUSED_FMT, &vut) == 4) {
-		res = r_anal_var_get (anal, vut.fcn_addr, vut.type[0], vut.scope, vut.delta);
-		sdb_fmt_free (&vut, SDB_VARUSED_FMT);
-	}
-	free (inst_key);
-	free (var_def);
-	return res;
-}
-
-static RAnalVar *get_used_var(RAnal *anal, RAnalOp *op) {
-	char *inst_key = r_str_newf ("inst.0x%"PFMT64x".vars", op->addr);
-	char *var_def = sdb_get (anal->sdb_fcns, inst_key, 0);
-	struct VarUsedType vut;
-	RAnalVar *res = NULL;
-	if (sdb_fmt_tobin (var_def, SDB_VARUSED_FMT, &vut) == 4) {
-		res = r_anal_var_get (anal, vut.fcn_addr, vut.type[0], vut.scope, vut.delta);
-		sdb_fmt_free (&vut, SDB_VARUSED_FMT);
-	}
-	free (inst_key);
-	free (var_def);
-	return res;
-}
-
 static int defaultCycles(RAnalOp *op) {
 	switch (op->type) {
 	case R_ANAL_OP_TYPE_PUSH:
@@ -160,14 +114,6 @@ R_API int r_anal_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int le
 		if (op->nopcode < 1) {
 			op->nopcode = 1;
 		}
-		if (mask & R_ANAL_OP_MASK_VAL) {
-			//free the previous var in op->var
-			RAnalVar *tmp = get_used_var (anal, op);
-			if (tmp) {
-				r_anal_var_free (op->var);
-				op->var = tmp;
-			}
-		}
 	} else if (!memcmp (data, "\xff\xff\xff\xff", R_MIN (4, len))) {
 		op->type = R_ANAL_OP_TYPE_ILL;
 	} else {
@@ -210,6 +156,15 @@ R_API RAnalOp *r_anal_op_copy(RAnalOp *op) {
 	nop->src[1] = r_anal_value_copy (op->src[1]);
 	nop->src[2] = r_anal_value_copy (op->src[2]);
 	nop->dst = r_anal_value_copy (op->dst);
+	if (op->access) {
+		RListIter *it;
+		RAnalValue *val;
+		RList *naccess = r_list_newf ((RListFree)r_anal_value_free);
+		r_list_foreach (op->access, it, val) {
+			r_list_append (naccess, r_anal_value_copy (val));
+		}
+		nop->access = naccess;
+	}
 	r_strbuf_init (&nop->esil);
 	r_strbuf_copy (&nop->esil, &op->esil);
 	return nop;
@@ -388,15 +343,15 @@ repeat:
 	case R_ANAL_OP_TYPE_SWITCH: return "switch";
 	case R_ANAL_OP_TYPE_TRAP  : return "trap";
 	case R_ANAL_OP_TYPE_UCALL : return "ucall";
-	case R_ANAL_OP_TYPE_RCALL : return "rcall"; // needs to be changed
-	case R_ANAL_OP_TYPE_ICALL : return "ucall"; // needs to be changed
-	case R_ANAL_OP_TYPE_IRCALL: return "ucall"; // needs to be changed
+	case R_ANAL_OP_TYPE_RCALL : return "rcall";
+	case R_ANAL_OP_TYPE_ICALL : return "icall";
+	case R_ANAL_OP_TYPE_IRCALL: return "ircall";
 	case R_ANAL_OP_TYPE_UCCALL: return "uccall";
 	case R_ANAL_OP_TYPE_UCJMP : return "ucjmp";
 	case R_ANAL_OP_TYPE_UJMP  : return "ujmp";
-	case R_ANAL_OP_TYPE_RJMP  : return "rjmp"; // needs to be changed
-	case R_ANAL_OP_TYPE_IJMP  : return "ujmp"; // needs to be changed
-	case R_ANAL_OP_TYPE_IRJMP : return "ujmp"; // needs to be changed
+	case R_ANAL_OP_TYPE_RJMP  : return "rjmp";
+	case R_ANAL_OP_TYPE_IJMP  : return "ijmp";
+	case R_ANAL_OP_TYPE_IRJMP : return "irjmp";
 	case R_ANAL_OP_TYPE_UNK   : return "unk";
 	case R_ANAL_OP_TYPE_UPUSH : return "upush";
 	case R_ANAL_OP_TYPE_XCHG  : return "xchg";
@@ -627,7 +582,7 @@ R_API const char *r_anal_op_family_to_string(int n) {
 	switch (n) {
 	case R_ANAL_OP_FAMILY_UNKNOWN: return "unk";
 	case R_ANAL_OP_FAMILY_CPU: return "cpu";
-	case R_ANAL_OP_FAMILY_PAC: return "pac";
+	case R_ANAL_OP_FAMILY_SECURITY: return "sec";
 	case R_ANAL_OP_FAMILY_FPU: return "fpu";
 	case R_ANAL_OP_FAMILY_MMX: return "mmx";
 	case R_ANAL_OP_FAMILY_SSE: return "sse";
@@ -654,7 +609,7 @@ R_API int r_anal_op_family_from_string(const char *f) {
 		{"virt", R_ANAL_OP_FAMILY_VIRT},
 		{"crpt", R_ANAL_OP_FAMILY_CRYPTO},
 		{"io", R_ANAL_OP_FAMILY_IO},
-		{"pac", R_ANAL_OP_FAMILY_PAC},
+		{"sec", R_ANAL_OP_FAMILY_SECURITY},
 		{"thread", R_ANAL_OP_FAMILY_THREAD},
 	};
 
