@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2019 - condret, riq */
+/* radare - LGPL - Copyright 2019-2020 - condret, riq */
 
 /* 6502 info taken from http://unusedino.de/ec64/technical/aay/c64/bchrt651.htm
  *
@@ -187,7 +187,7 @@ static void _6502_anal_esil_ccall(RAnalOp *op, ut8 data0) {
 		flag = "unk";
 		break;
 	}
-	r_strbuf_setf (&op->esil, "%s,?{,0x%04x,pc,=,}", flag, (op->jump & 0xffff));
+	r_strbuf_setf (&op->esil, "%s,?{,0x%04x,pc,=,}", flag, (ut32)(op->jump & 0xffff));
 }
 
 // inc register
@@ -312,7 +312,6 @@ static int _6502_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int le
 		return -1;
 	}
 
-	memset (op, '\0', sizeof (RAnalOp));
 	op->size = snes_op_get_size (1, 1, &snes_op[data[0]]);	//snes-arch is similar to nes/6502
 	op->addr = addr;
 	op->type = R_ANAL_OP_TYPE_UNK;
@@ -744,21 +743,21 @@ static int _6502_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int le
 		// JSR pushes the address-1 of the next operation on to the stack before transferring program
 		// control to the following address
 		// stack is on page one and sp is an 8-bit reg: operations must be done like: sp + 0x100
-		r_strbuf_setf (&op->esil, "1,pc,-,0xff,sp,+,=[2],0x%04x,pc,=,2,sp,-=", op->jump);
+		r_strbuf_setf (&op->esil, "1,pc,-,0xff,sp,+,=[2],0x%04" PFMT64x ",pc,=,2,sp,-=", op->jump);
 		break;
 	// JMP
 	case 0x4c: // jmp $ffff
 		op->cycles = 3;
 		op->type = R_ANAL_OP_TYPE_JMP;
 		op->jump = (len > 2)? data[1] | data[2] << 8: 0;
-		r_strbuf_setf (&op->esil, "0x%04x,pc,=", op->jump);
+		r_strbuf_setf (&op->esil, "0x%04" PFMT64x ",pc,=", op->jump);
 		break;
 	case 0x6c: // jmp ($ffff)
 		op->cycles = 5;
 		op->type = R_ANAL_OP_TYPE_UJMP;
 		// FIXME: how to read memory?
 		// op->jump = data[1] | data[2] << 8;
-		r_strbuf_setf (&op->esil, "0x%04x,[2],pc,=", data[1] | data[2] << 8);
+		r_strbuf_setf (&op->esil, "0x%04x,[2],pc,=", len > 2? data[1] | data[2] << 8: 0);
 		break;
 	// RTS
 	case 0x60: // rts
@@ -908,10 +907,12 @@ static int _6502_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int le
 	return op->size;
 }
 
-static int set_reg_profile(RAnal *anal) {
+static bool set_reg_profile(RAnal *anal) {
 	char *p =
 		"=PC	pc\n"
 		"=SP	sp\n"
+		"=A0	y\n"
+		"=A1	y\n"
 		"gpr	a	.8	0	0\n"
 		"gpr	x	.8	1	0\n"
 		"gpr	y	.8	2	0\n"
