@@ -43,8 +43,6 @@ static int op_thumb(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int le
 	arm_set_thumb (arminsn, true);
 	arm_set_input_buffer (arminsn, data);
 	arm_set_pc (arminsn, addr);
-	op->jump = op->fail = -1;
-	op->ptr = op->val = -1;
 	op->delay = 0;
 	op->size = arm_disasm_one_insn (arminsn);
 	op->jump = arminsn->jmp;
@@ -169,7 +167,7 @@ static int op_cond(const ut8 *data) {
 
 static int arm_op32(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len) {
 	const ut8 *b = (ut8 *) data;
-	ut8 ndata[4];
+	ut8 ndata[4] = {0};
 	ut32 branch_dst_addr, i = 0;
 	ut32 *code = (ut32 *) data;
 	struct winedbg_arm_insn *arminsn;
@@ -177,22 +175,20 @@ static int arm_op32(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int le
 	if (!data) {
 		return 0;
 	}
-	memset (op, '\0', sizeof (RAnalOp));
 	arminsn = arm_new ();
 	arm_set_thumb (arminsn, false);
 	arm_set_input_buffer (arminsn, data);
 	arm_set_pc (arminsn, addr);
-	op->jump = op->fail = -1;
-	op->ptr = op->val = -1;
 	op->addr = addr;
 	op->type = R_ANAL_OP_TYPE_UNK;
 
 	if (anal->big_endian) {
 		b = data = ndata;
+		ut8 tmp = data[3];
 		ndata[0] = data[3];
 		ndata[1] = data[2];
 		ndata[2] = data[1];
-		ndata[3] = data[0];
+		ndata[3] = tmp;
 	}
 	if (anal->bits == 16) {
 		arm_free (arminsn);
@@ -358,7 +354,6 @@ static ut64 getaddr(ut64 addr, const ut8 *d) {
 }
 
 static int arm_op64(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *d, int len) {
-	memset (op, 0, sizeof (RAnalOp));
 	if (d[3] == 0) {
 		return -1;      // invalid
 	}
@@ -414,7 +409,7 @@ static int arm_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len,
 	return arm_op32 (anal, op, addr, data, len);
 }
 
-static int set_reg_profile(RAnal *anal) {
+static bool set_reg_profile(RAnal *anal) {
 	// TODO: support 64bit profile
 	const char *p32 =
 		"=PC	r15\n"
