@@ -5,7 +5,7 @@
 
 static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len, RAnalOpMask mask) {
 	int opsize = -1;
-        op->type = -1;
+	op->type = -1;
 	opsize = 2;
 	switch (buf[0]) {
 	case 0x3f:
@@ -237,7 +237,7 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len, RAn
 	return opsize;
 }
 
-static int set_reg_profile(RAnal *anal) {
+static bool set_reg_profile(RAnal *anal) {
 	const char *p =
 		"=PC	pc\n"
 		"=SP	r14\n" // XXX
