@@ -188,11 +188,11 @@ static void gb_anal_mov_imm (RReg *reg, RAnalOp *op, const ut8 *data) {
 	if (data[0] & 1) {
 		op->dst->reg = r_reg_get (reg, regs_16[data[0]>>4], R_REG_TYPE_GPR);
 		op->src[0]->imm = GB_SOFTCAST (data[1], data[2]);
-		r_strbuf_setf (&op->esil, "0x%04x,%s,=", op->src[0]->imm, regs_16[data[0]>>4]);
+		r_strbuf_setf (&op->esil, "0x%04" PFMT64x ",%s,=", op->src[0]->imm, regs_16[data[0]>>4]);
 	} else {
 		op->dst->reg = r_reg_get (reg, regs_8[data[0]>>3], R_REG_TYPE_GPR);
 		op->src[0]->imm = data[1];
-		r_strbuf_setf (&op->esil, "0x%02x,%s,=", op->src[0]->imm, regs_8[data[0]>>3]);
+		r_strbuf_setf (&op->esil, "0x%02" PFMT64x ",%s,=", op->src[0]->imm, regs_8[data[0]>>3]);
 	}
 	op->src[0]->absolute = true;
 	op->val = op->src[0]->imm;
@@ -310,9 +310,9 @@ static inline void gb_anal_and_res (RAnal *anal, RAnalOp *op, const ut8 data) {
 	op->dst->memref = ((data & 7) == 6);
 	op->dst->reg = r_reg_get (anal->reg, regs_x[data & 7], R_REG_TYPE_GPR);
 	if (op->dst->memref) {
-		r_strbuf_setf (&op->esil, "0x%02x,%s,[1],&,%s,=[1]", op->src[0]->imm, regs_x[data & 7], regs_x[data & 7]);
+		r_strbuf_setf (&op->esil, "0x%02" PFMT64x ",%s,[1],&,%s,=[1]", op->src[0]->imm, regs_x[data & 7], regs_x[data & 7]);
 	} else {
-		r_strbuf_setf (&op->esil, "0x%02x,%s,&=", op->src[0]->imm, regs_x[data & 7]);
+		r_strbuf_setf (&op->esil, "0x%02" PFMT64x ",%s,&=", op->src[0]->imm, regs_x[data & 7]);
 	}
 }
 
@@ -323,9 +323,9 @@ static inline void gb_anal_and_bit (RReg *reg, RAnalOp *op, const ut8 data) {
 	op->dst->memref = ((data & 7) == 6);
 	op->dst->reg = r_reg_get (reg, regs_x[data & 7], R_REG_TYPE_GPR);
 	if (op->dst->memref) {
-		r_strbuf_setf (&op->esil, "%i,%s,[1],&,0,==,$z,Z,:=,0,N,:=,1,H,:=", op->src[0]->imm, regs_x[data & 7]);
+		r_strbuf_setf (&op->esil, "%" PFMT64d ",%s,[1],&,0,==,$z,Z,:=,0,N,:=,1,H,:=", op->src[0]->imm, regs_x[data & 7]);
 	} else {
-		r_strbuf_setf (&op->esil, "%i,%s,&,0,==,$z,Z,:=,0,N,:=,1,H,:=", op->src[0]->imm, regs_x[data & 7]);
+		r_strbuf_setf (&op->esil, "%" PFMT64d ",%s,&,0,==,$z,Z,:=,0,N,:=,1,H,:=", op->src[0]->imm, regs_x[data & 7]);
 	}
 }
 
@@ -336,9 +336,9 @@ static inline void gb_anal_or_set (RAnal *anal, RAnalOp *op, const ut8 data) {
 	op->dst->memref = ((data & 7) == 6);
 	op->dst->reg = r_reg_get (anal->reg, regs_x[data & 7], R_REG_TYPE_GPR);
 	if (op->dst->memref) {
-		r_strbuf_setf (&op->esil, "0x%02x,%s,[1],|,%s,=[1]", op->src[0]->imm, regs_x[data & 7], regs_x[data & 7], op->src[0]->imm);
+		r_strbuf_setf (&op->esil, "0x%02" PFMT64x ",%s,[1],|,%s,=[1]", op->src[0]->imm, regs_x[data & 7], regs_x[data & 7]);
 	} else {
-		r_strbuf_setf (&op->esil, "0x%02x,%s,|=", op->src[0]->imm, regs_x[data & 7]);
+		r_strbuf_setf (&op->esil, "0x%02" PFMT64x ",%s,|=", op->src[0]->imm, regs_x[data & 7]);
 	}
 }
 
@@ -489,7 +489,7 @@ static inline void gb_anal_load (RReg *reg, RAnalOp *op, const ut8 *data) {
 	switch (data[0]) {
 	case 0xf0:
 		op->src[0]->base = 0xff00 + data[1];
-		r_strbuf_setf (&op->esil, "0x%04x,[1],a,=", op->src[0]->base);
+		r_strbuf_setf (&op->esil, "0x%04" PFMT64x ",[1],a,=", op->src[0]->base);
 		break;
 	case 0xf2:
 		op->src[0]->base = 0xff00;
@@ -505,7 +505,7 @@ static inline void gb_anal_load (RReg *reg, RAnalOp *op, const ut8 *data) {
 				op->ptr = op->src[0]->base + (op->addr & 0xffffffffffff0000LL);
 			}
 		}
-		r_strbuf_setf (&op->esil, "0x%04x,[1],a,=", op->src[0]->base);
+		r_strbuf_setf (&op->esil, "0x%04" PFMT64x ",[1],a,=", op->src[0]->base);
 		break;
 	default:
 		op->src[0]->reg = r_reg_get (reg, regs_16[(data[0] & 0xf0) >> 4], R_REG_TYPE_GPR);
@@ -546,11 +546,11 @@ static void gb_anal_store (RReg *reg, RAnalOp *op, const ut8 *data)
 			op->dst->memref = 2;
 			op->dst->base = GB_SOFTCAST (data[1], data[2]);
 			op->src[0]->reg = r_reg_get (reg, "sp", R_REG_TYPE_GPR);
-			r_strbuf_setf (&op->esil, "sp,0x%04x,=[2]", op->dst->base);
+			r_strbuf_setf (&op->esil, "sp,0x%04" PFMT64x ",=[2]", op->dst->base);
 			break;
 		case 0xe0:
 			op->dst->base = 0xff00 + data[1];
-			r_strbuf_setf (&op->esil, "a,0x%04x,=[1]", op->dst->base);
+			r_strbuf_setf (&op->esil, "a,0x%04" PFMT64x ",=[1]", op->dst->base);
 			break;
 		case 0xe2:
 			op->dst->base = 0xff00;
@@ -559,7 +559,7 @@ static void gb_anal_store (RReg *reg, RAnalOp *op, const ut8 *data)
 			break;
 		case 0xea:
 			op->dst->base = GB_SOFTCAST (data[1], data[2]);
-			r_strbuf_setf (&op->esil, "a,0x%04x,=[1]", op->dst->base);
+			r_strbuf_setf (&op->esil, "a,0x%04" PFMT64x ",=[1]", op->dst->base);
 			break;
 		default:
 			op->dst->reg = r_reg_get (reg, regs_16[(data[0] & 0xf0)>>4], R_REG_TYPE_GPR);
@@ -714,9 +714,11 @@ static bool gb_custom_daa (RAnalEsil *esil) {
 static int gb_anop(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len, RAnalOpMask mask) {
 	int ilen = gbOpLength (gb_op[data[0]].type);
 	if (ilen > len) {
-		ilen = 0;
-	} else if (mask & R_ANAL_OP_MASK_DISASM) {
-		memset (op, '\0', sizeof (RAnalOp));
+		op->type = R_ANAL_OP_TYPE_ILL;
+		op->size = 0;
+		return 0;
+	} 
+	if (mask & R_ANAL_OP_MASK_DISASM) {
 		char mn[32];
 		memset (mn, '\0', sizeof (char) * sizeof (mn));
 		char reg[32];
@@ -740,14 +742,11 @@ static int gb_anop(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len
 			break;
 		}
 		op->mnemonic = strdup (mn);
-	} else {
-		memset (op, '\0', sizeof (RAnalOp));
 	}
 	op->addr = addr;
 	op->type = R_ANAL_OP_TYPE_UNK;
 	op->size = ilen;
 	op->nopcode = 1;
-	r_strbuf_init (&op->esil);
 	switch (data[0])
 	{
 		case 0x00:
@@ -1496,7 +1495,7 @@ static int gb_anop(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len
 	the mbc can be seen as a register but it isn't. For the Gameboy the mbc is invisble.
 */
 
-static int set_reg_profile(RAnal *anal) {
+static bool set_reg_profile(RAnal *anal) {
 	const char *p =
 		"=PC	mpc\n"
 		"=SP	sp\n"
@@ -1566,6 +1565,20 @@ static int esil_gb_fini (RAnalEsil *esil) {
 	return true;
 }
 
+static int archinfo(RAnal *anal, int q) {
+	switch (q) {
+	case R_ANAL_ARCHINFO_ALIGN:
+		return 0;
+	case R_ANAL_ARCHINFO_MAX_OP_SIZE:
+		return 3;
+	case R_ANAL_ARCHINFO_INV_OP_SIZE:
+		return 1;
+	case R_ANAL_ARCHINFO_MIN_OP_SIZE:
+		return 1;
+	}
+	return 1;
+}
+
 RAnalPlugin r_anal_plugin_gb = {
 	.name = "gb",
 	.desc = "Gameboy CPU code analysis plugin",
@@ -1575,6 +1588,7 @@ RAnalPlugin r_anal_plugin_gb = {
 	.bits = 16,
 	.op = &gb_anop,
 	.set_reg_profile = &set_reg_profile,
+	.archinfo = archinfo,
 	.esil_init = esil_gb_init,
 	.esil_fini = esil_gb_fini,
 };
