@@ -10,9 +10,11 @@
 #include "hexagon_anal.h"
 
 static int hexagon_v6_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *buf, int len, RAnalOpMask mask) {
-	HexInsn hi = {0};;
+	HexInsn hi = {0};
 	ut32 data = 0;
-	memset (op, 0, sizeof (RAnalOp));
+	if (len < 4) {
+		return 0;
+	}
 	data = r_read_le32 (buf);
 	int size = hexagon_disasm_instruction (data, &hi, (ut32) addr);
 	op->size = size;
@@ -21,16 +23,20 @@ static int hexagon_v6_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *buf, in
 	}
 
 	op->addr = addr;
-	op->jump = op->fail = -1;
-	op->ptr = op->val = -1;
 	return hexagon_anal_instruction (&hi, op);
 }
 
-static int set_reg_profile(RAnal *anal) {
+static bool set_reg_profile(RAnal *anal) {
 	// TODO: Add missing registers
 	const char *p =
 		"=PC	pc\n"
 		"=SP	r29\n"
+		"=A0	r0\n"
+		"=A1	r1\n"
+		"=A2	r2\n"
+		"=A3	r3\n"
+		"=A4	r4\n"
+		"=A5	r5\n"
 		"=BP	r30\n"
 		"=LR	r31\n"
 		"=SN	r6\n"
