@@ -11,15 +11,10 @@
 #include "../../../shlr/java/code.h"
 #include "../../../shlr/java/class.h"
 
-#ifdef IFDBG
-#define dprintf eprintf
-#endif
-
 #define DO_THE_DBG 0
 #define IFDBG  if(DO_THE_DBG)
-#define IFINT  if(0)
 
-ut64 METHOD_START = 0;
+static ut64 METHOD_START = 0;
 
 static void java_update_anal_types (RAnal *anal, RBinJavaObj *bin_obj);
 
@@ -36,14 +31,14 @@ static RBinJavaObj * get_java_bin_obj(RAnal *anal) {
 	return is_java ? b->cur->o->bin_obj : NULL;
 }
 
-static ut64 java_get_method_start () {
+static ut64 java_get_method_start(void) {
 	return METHOD_START;
 }
 
 static int java_switch_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len) {
 	ut8 op_byte = data[0];
 	ut64 offset = addr - java_get_method_start ();
-	ut8 pos = (offset+1)%4 ? 1 + 4 - (offset+1)%4 : 1;
+	ut8 pos = (offset + 1)%4 ? 1 + 4 - (offset+1)%4 : 1;
 
 	if (op_byte == 0xaa) {
 		// handle a table switch condition
@@ -54,8 +49,7 @@ static int java_switch_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data,
 		const int max_val = (ut32)(UINT (data, pos + 8));
 
 		ut32 default_loc = (ut32) (UINT (data, pos)), cur_case = 0;
-		op->switch_op = r_anal_switch_op_new (addr, min_val, default_loc);
-		RAnalCaseOp *caseop = NULL;
+		op->switch_op = r_anal_switch_op_new (addr, min_val, max_val, default_loc);
 		pos += 12;
 		if (max_val > min_val && ((max_val - min_val)<(UT16_MAX/4))) {
 			//caseop = r_anal_switch_op_add_case(op->switch_op, addr+default_loc, -1, addr+offset);
@@ -66,12 +60,7 @@ static int java_switch_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data,
 					break;
 				}
 				int offset = (int)(ut32)(R_BIN_JAVA_UINT (data, pos));
-				caseop = r_anal_switch_op_add_case (op->switch_op,
-					addr + pos, cur_case + min_val, addr + offset);
-				if (caseop) {
-					caseop->bb_ref_to = addr+offset;
-					caseop->bb_ref_from = addr; // TODO figure this one out
-				}
+				r_anal_switch_op_add_case (op->switch_op, addr + pos, cur_case + min_val, addr + offset);
 			}
 		} else {
 			eprintf ("Invalid switch boundaries at 0x%"PFMT64x"\n", addr);
