@@ -5,12 +5,11 @@
 #include <r_lib.h>
 
 static int null_anal(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len, RAnalOpMask mask) {
-	memset (op, '\0', sizeof(RAnalOp));
 	/* This should better follow the disassembler */
 	return op->size = 1;
 }
 
-static int null_set_reg_profile(RAnal* anal){
+static bool null_set_reg_profile(RAnal* anal){
 	return r_reg_set_profile_string(anal->reg, "");
 }
 
