@@ -167,7 +167,7 @@ static RList *rtti_msvc_read_base_class_array(RVTableContext *context, ut32 num_
 
 	if (num_base_classes > BASE_CLASSES_MAX) {
 		if (context->anal->verbose) {
-			eprintf ("WARNING: Length of base class array at 0x%08"PFMT64x" exceeds %d.\n", addr, BASE_CLASSES_MAX);
+			eprintf ("Warning: Length of base class array at 0x%08"PFMT64x" exceeds %d.\n", addr, BASE_CLASSES_MAX);
 		}
 		num_base_classes = BASE_CLASSES_MAX;
 	}
@@ -298,11 +298,15 @@ static void rtti_msvc_print_complete_object_locator(rtti_complete_object_locator
 				   prefix, col->object_base);
 }
 
-static void rtti_msvc_print_complete_object_locator_json(rtti_complete_object_locator *col) {
-	r_cons_printf ("{\"signature\":%"PFMT32u",\"vftable_offset\":%"PFMT32u",\"cd_offset\":%"PFMT32u","
-				   "\"type_desc_addr\":%"PFMT32u",\"class_desc_addr\":%"PFMT32u",\"object_base\":%"PFMT32u"}",
-				   col->signature, col->vtable_offset, col->cd_offset, col->type_descriptor_addr,
-				   col->class_descriptor_addr, col->object_base);
+static void rtti_msvc_print_complete_object_locator_json(PJ *pj, rtti_complete_object_locator *col) {
+	pj_o (pj);
+	pj_kn (pj, "signature", col->signature);
+	pj_kn (pj, "vftable_offset", col->vtable_offset);
+	pj_kn (pj, "cd_offset", col->cd_offset);
+	pj_kn (pj, "type_desc_addr", col->type_descriptor_addr);
+	pj_kn (pj, "class_desc_addr", col->class_descriptor_addr);
+	pj_kn (pj, "object_base", col->object_base);
+	pj_end (pj);
 }
 
 static void rtti_msvc_print_type_descriptor(rtti_type_descriptor *td, ut64 addr, const char *prefix) {
@@ -316,9 +320,12 @@ static void rtti_msvc_print_type_descriptor(rtti_type_descriptor *td, ut64 addr,
 				   prefix, td->name);
 }
 
-static void rtti_msvc_print_type_descriptor_json(rtti_type_descriptor *td) {
-	r_cons_printf ("{\"vtable_addr\":%"PFMT32u",\"spare\":%"PFMT32u",\"name\":\"%s\"}",
-				   td->vtable_addr, td->spare, td->name);
+static void rtti_msvc_print_type_descriptor_json(PJ *pj, rtti_type_descriptor *td) {
+	pj_o (pj);
+	pj_kn (pj, "vtable_addr", td->vtable_addr);
+	pj_kn (pj, "spare", td->spare);
+	pj_ks (pj, "name", td->name);
+	pj_end (pj);
 }
 
 static void rtti_msvc_print_class_hierarchy_descriptor(rtti_class_hierarchy_descriptor *chd, ut64 addr, const char *prefix) {
@@ -334,10 +341,13 @@ static void rtti_msvc_print_class_hierarchy_descriptor(rtti_class_hierarchy_desc
 				   prefix, chd->base_class_array_addr);
 }
 
-static void rtti_msvc_print_class_hierarchy_descriptor_json(rtti_class_hierarchy_descriptor *chd) {
-	r_cons_printf ("{\"signature\":%"PFMT32u",\"attributes\":%"PFMT32u",\"num_base_classes\":%"PFMT32u","
-				   "\"base_class_array_addr\":%"PFMT32u"}",
-				   chd->signature, chd->attributes, chd->num_base_classes, chd->base_class_array_addr);
+static void rtti_msvc_print_class_hierarchy_descriptor_json(PJ *pj, rtti_class_hierarchy_descriptor *chd) {
+	pj_o (pj);
+	pj_kn (pj, "signature", chd->signature);
+	pj_kn (pj, "attributes", chd->attributes);
+	pj_kn (pj, "num_base_classes", chd->num_base_classes);
+	pj_kn (pj, "base_class_array_addr", chd->base_class_array_addr);
+	pj_end (pj);
 }
 
 static void rtti_msvc_print_base_class_descriptor(rtti_base_class_descriptor *bcd, const char *prefix) {
@@ -359,12 +369,17 @@ static void rtti_msvc_print_base_class_descriptor(rtti_base_class_descriptor *bc
 				   prefix, bcd->attributes);
 }
 
-static void rtti_msvc_print_base_class_descriptor_json(rtti_base_class_descriptor *bcd) {
-	r_cons_printf ("{\"type_desc_addr\":%"PFMT32u",\"num_contained_bases\":%"PFMT32u","
-				   "\"where\":{\"mdisp\":%"PFMT32d",\"pdisp\":%"PFMT32d",\"vdisp\":%"PFMT32d"},"
-				   "\"attributes\":%"PFMT32u"}",
-				   bcd->type_descriptor_addr, bcd->num_contained_bases,
-				   bcd->where.mdisp, bcd->where.pdisp, bcd->where.vdisp, bcd->attributes);
+static void rtti_msvc_print_base_class_descriptor_json(PJ *pj, rtti_base_class_descriptor *bcd) {
+	pj_o (pj);
+	pj_kn (pj, "type_desc_addr", bcd->type_descriptor_addr);
+	pj_kn (pj, "num_contained_bases", bcd->num_contained_bases);
+	pj_ko (pj, "where");
+	pj_ki (pj, "mdisp", bcd->where.mdisp);
+	pj_ki (pj, "pdisp", bcd->where.pdisp);
+	pj_ki (pj, "vdisp", bcd->where.vdisp);
+	pj_end (pj);
+	pj_kn (pj, "attributes", bcd->attributes);
+	pj_end (pj);
 }
 
 
@@ -411,7 +426,13 @@ R_API void r_anal_rtti_msvc_print_complete_object_locator(RVTableContext *contex
 	}
 
 	if (mode == 'j') {
-		rtti_msvc_print_complete_object_locator_json (&col);
+		PJ *pj = pj_new ();
+		if (!pj) {
+			return;
+		}
+		rtti_msvc_print_complete_object_locator_json (pj, &col);
+		r_cons_print (pj_string (pj));
+		pj_free (pj);
 	} else {
 		rtti_msvc_print_complete_object_locator (&col, addr, "");
 	}
@@ -425,7 +446,13 @@ R_API void r_anal_rtti_msvc_print_type_descriptor(RVTableContext *context, ut64
 	}
 
 	if (mode == 'j') {
-		rtti_msvc_print_type_descriptor_json (&td);
+		PJ *pj = pj_new ();
+		if (!pj) {
+			return;
+		}
+		rtti_msvc_print_type_descriptor_json (pj, &td);
+		r_cons_print (pj_string (pj));
+		pj_free (pj);
 	} else {
 		rtti_msvc_print_type_descriptor (&td, addr, "");
 	}
@@ -441,7 +468,13 @@ R_API void r_anal_rtti_msvc_print_class_hierarchy_descriptor(RVTableContext *con
 	}
 
 	if (mode == 'j') {
-		rtti_msvc_print_class_hierarchy_descriptor_json (&chd);
+		PJ *pj = pj_new ();
+		if (!pj) {
+			return;
+		}
+		rtti_msvc_print_class_hierarchy_descriptor_json (pj, &chd);
+		r_cons_print (pj_string (pj));
+		pj_free (pj);
 	} else {
 		rtti_msvc_print_class_hierarchy_descriptor (&chd, addr, "");
 	}
@@ -455,7 +488,13 @@ R_API void r_anal_rtti_msvc_print_base_class_descriptor(RVTableContext *context,
 	}
 
 	if (mode == 'j') {
-		rtti_msvc_print_base_class_descriptor_json (&bcd);
+		PJ *pj = pj_new ();
+		if (!pj) {
+			return;
+		}
+		rtti_msvc_print_base_class_descriptor_json (pj, &bcd);
+		r_cons_print (pj_string (pj));
+		pj_free (pj);
 	} else {
 		rtti_msvc_print_base_class_descriptor (&bcd, "");
 	}
@@ -463,6 +502,7 @@ R_API void r_anal_rtti_msvc_print_base_class_descriptor(RVTableContext *context,
 
 static bool rtti_msvc_print_complete_object_locator_recurse(RVTableContext *context, ut64 atAddress, int mode, bool strict) {
 	bool use_json = mode == 'j';
+	PJ *pj = NULL;
 
 	ut64 colRefAddr = atAddress - context->word_size;
 	ut64 colAddr;
@@ -519,13 +559,18 @@ static bool rtti_msvc_print_complete_object_locator_recurse(RVTableContext *cont
 
 	// print
 	if (use_json) {
-		r_cons_print ("{\"complete_object_locator\":");
-		rtti_msvc_print_complete_object_locator_json (&col);
-		r_cons_print (",\"type_desc\":");
-		rtti_msvc_print_type_descriptor_json (&td);
-		r_cons_print (",\"class_hierarchy_desc\":");
-		rtti_msvc_print_class_hierarchy_descriptor_json (&chd);
-		r_cons_print (",\"base_classes\":[");
+		pj = pj_new ();
+		if (!pj) {
+			return false;
+		}
+		pj_o (pj);
+		pj_k (pj, "complete_object_locator");
+		rtti_msvc_print_complete_object_locator_json (pj, &col);
+		pj_k (pj, "type_desc");
+		rtti_msvc_print_type_descriptor_json (pj, &td);
+		pj_k (pj, "class_hierarchy_desc");
+		rtti_msvc_print_class_hierarchy_descriptor_json (pj, &chd);
+		pj_ka (pj, "base_classes");
 	} else {
 		rtti_msvc_print_complete_object_locator (&col, colAddr, "");
 		rtti_msvc_print_type_descriptor (&td, typeDescriptorAddr, "\t");
@@ -534,21 +579,13 @@ static bool rtti_msvc_print_complete_object_locator_recurse(RVTableContext *cont
 
 
 	// base classes
-	bool json_first = true;
 	RListIter *bcdIter;
 	rtti_base_class_descriptor *bcd;
 	r_list_foreach (baseClassArray, bcdIter, bcd) {
 		if (use_json) {
-			if (json_first) {
-				r_cons_print ("{\"desc\":");
-				json_first = false;
-			} else {
-				r_cons_print (",{\"desc\":");
-			}
-		}
-
-		if (use_json) {
-			rtti_msvc_print_base_class_descriptor_json (bcd);
+			pj_o (pj);
+			pj_k (pj, "desc");
+			rtti_msvc_print_base_class_descriptor_json (pj, bcd);
 		} else {
 			rtti_msvc_print_base_class_descriptor (bcd, "\t\t");
 		}
@@ -557,8 +594,8 @@ static bool rtti_msvc_print_complete_object_locator_recurse(RVTableContext *cont
 		rtti_type_descriptor btd = { 0 };
 		if (rtti_msvc_read_type_descriptor (context, baseTypeDescriptorAddr, &btd)) {
 			if (use_json) {
-				r_cons_print (",\"type_desc\":");
-				rtti_msvc_print_type_descriptor_json (&btd);
+				pj_k (pj, "type_desc");
+				rtti_msvc_print_type_descriptor_json (pj, &btd);
 			} else {
 				rtti_msvc_print_type_descriptor (&btd, baseTypeDescriptorAddr, "\t\t\t");
 			}
@@ -569,16 +606,15 @@ static bool rtti_msvc_print_complete_object_locator_recurse(RVTableContext *cont
 			}
 		}
 
-		if(use_json) {
-			r_cons_print ("}");
+		if (use_json) {
+			pj_end (pj);
 		}
 	}
 	if (use_json) {
-		r_cons_print ("]");
-	}
-
-	if (use_json) {
-		r_cons_print ("}");
+		pj_end (pj);
+		pj_end (pj);
+		r_cons_print (pj_string (pj));
+		pj_free (pj);
 	}
 
 	rtti_type_descriptor_fini (&td);
@@ -768,7 +804,6 @@ RecoveryTypeDescriptor *recovery_anal_type_descriptor(RRTTIMSVCAnalContext *cont
 	return td;
 }
 
-
 static char *unique_class_name(RAnal *anal, const char *original_name) {
 	if (!r_anal_class_exists (anal, original_name)) {
 		return strdup (original_name);
@@ -796,9 +831,7 @@ static void recovery_apply_vtable(RAnal *anal, const char *class_name, RVTableIn
 		return;
 	}
 
-	RAnalVTable vtable;
-	vtable.id = NULL;
-	vtable.offset = 0;
+	RAnalVTable vtable = {0};
 	vtable.addr = vtable_info->saddr;
 	r_anal_class_vtable_set (anal, class_name, &vtable);
 	r_anal_class_vtable_fini (&vtable);
@@ -808,7 +841,7 @@ static void recovery_apply_vtable(RAnal *anal, const char *class_name, RVTableIn
 		RAnalMethod meth;
 		meth.addr = vmeth->addr;
 		meth.vtable_offset = vmeth->vtable_offset;
-		meth.name = r_str_newf ("virtual_%d", meth.vtable_offset);
+		meth.name = r_str_newf ("virtual_%" PFMT64d, meth.vtable_offset);
 		r_anal_class_method_set (anal, class_name, &meth);
 		r_anal_class_method_fini (&meth);
 	}
@@ -937,7 +970,7 @@ static const char *recovery_apply_type_descriptor(RRTTIMSVCAnalContext *context,
 	return name;
 }
 
-void str_value_free(HtPPKv *kv) {
+void str_value_free(HtUPKv *kv) {
 	free (kv->value);
 }
 
