@@ -2,7 +2,7 @@
 
 #include <r_anal.h>
 
-static RAnalSwitchOp *__switch_op_new() {
+static RAnalSwitchOp *__switch_op_new(void) {
 	RAnalSwitchOp * swop = R_NEW0 (RAnalSwitchOp);
 	if (swop) {
 		swop->cases = r_list_new ();
@@ -16,13 +16,13 @@ static RAnalSwitchOp *__switch_op_new() {
 	return swop;
 }
 
-R_API RAnalSwitchOp * r_anal_switch_op_new(ut64 addr, ut64 min_val, ut64 def_val) {
+R_API RAnalSwitchOp *r_anal_switch_op_new(ut64 addr, ut64 min_val, ut64 max_val, ut64 def_val) {
 	RAnalSwitchOp *swop = __switch_op_new ();
 	if (swop) {
 		swop->addr = addr;
 		swop->min_val = min_val;
-		swop->def_val = min_val;
-		swop->max_val = min_val;
+		swop->def_val = def_val;
+		swop->max_val = max_val;
 	}
 	return swop;
 }
