@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2018 - pancake, maijin, thestr4ng3r */
+/* radare - LGPL - Copyright 2009-2020 - pancake, maijin, thestr4ng3r */
 
 #include "r_util.h"
 #include "r_anal.h"
@@ -6,13 +6,13 @@
 #define VTABLE_BUFF_SIZE 10
 
 #define VTABLE_READ_ADDR_FUNC(fname, read_fname, sz) \
-	static bool fname(RAnal *anal, ut64 addr, ut64 *buf) { \
-		ut8 tmp[sz]; \
-		if(!anal->iob.read_at(anal->iob.io, addr, tmp, sz)) { \
-			return false; \
-		} \
-		*buf = read_fname(tmp); \
-		return true; \
+	static bool fname(RAnal *anal, ut64 addr, ut64 *buf) {\
+		ut8 tmp[sz];\
+		if (!anal->iob.read_at (anal->iob.io, addr, tmp, sz)) {\
+			return false;\
+		}\
+		*buf = read_fname (tmp);\
+		return true;\
 	}
 VTABLE_READ_ADDR_FUNC (vtable_read_addr_le8, r_read_le8, 1)
 VTABLE_READ_ADDR_FUNC (vtable_read_addr_le16, r_read_le16, 2)
@@ -23,9 +23,6 @@ VTABLE_READ_ADDR_FUNC (vtable_read_addr_be16, r_read_be16, 2)
 VTABLE_READ_ADDR_FUNC (vtable_read_addr_be32, r_read_be32, 4)
 VTABLE_READ_ADDR_FUNC (vtable_read_addr_be64, r_read_be64, 8)
 
-
-
-
 R_API void r_anal_vtable_info_free(RVTableInfo *vtable) {
 	if (!vtable) {
 		return;
@@ -38,35 +35,38 @@ R_API ut64 r_anal_vtable_info_get_size(RVTableContext *context, RVTableInfo *vta
 	return (ut64)vtable->methods.len * context->word_size;
 }
 
-
 R_API bool r_anal_vtable_begin(RAnal *anal, RVTableContext *context) {
 	context->anal = anal;
 	context->abi = anal->cpp_abi;
-	context->word_size = (ut8)(anal->bits / 8);
-	switch (anal->bits) {
-		case 8:
-			context->read_addr = anal->big_endian ? vtable_read_addr_be8 : vtable_read_addr_le8;
-			break;
-		case 16:
-			context->read_addr = anal->big_endian ? vtable_read_addr_be16 : vtable_read_addr_le16;
-			break;
-		case 32:
-			context->read_addr = anal->big_endian ? vtable_read_addr_be32 : vtable_read_addr_le32;
-			break;
-		case 64:
-			context->read_addr = anal->big_endian ? vtable_read_addr_be64 : vtable_read_addr_le64;
-			break;
-		default:
-			return false;
+	context->word_size = (ut8) (anal->bits / 8);
+	const bool is_arm = anal->cur->arch && r_str_startswith (anal->cur->arch, "arm");
+	if (is_arm && context->word_size < 4) {
+		context->word_size = 4;
+	}
+	switch (context->word_size) {
+	case 1:
+		context->read_addr = anal->big_endian ? vtable_read_addr_be8 : vtable_read_addr_le8;
+		break;
+	case 2:
+		context->read_addr = anal->big_endian ? vtable_read_addr_be16 : vtable_read_addr_le16;
+		break;
+	case 4:
+		context->read_addr = anal->big_endian ? vtable_read_addr_be32 : vtable_read_addr_le32;
+		break;
+	case 8:
+		context->read_addr = anal->big_endian ? vtable_read_addr_be64 : vtable_read_addr_le64;
+		break;
+	default:
+		return false;
 	}
 	return true;
 }
 
 static bool vtable_addr_in_text_section(RVTableContext *context, ut64 curAddress) {
 	//section of the curAddress
-	RBinSection* value = context->anal->binb.get_vsect_at (context->anal->binb.bin, curAddress);
+	RBinSection *value = context->anal->binb.get_vsect_at (context->anal->binb.bin, curAddress);
 	//If the pointed value lies in .text section
-	return value && !strcmp (value->name, ".text");
+	return value && strstr (value->name, "text") && (value->perm & 1) != 0;
 }
 
 static bool vtable_is_value_in_text_section(RVTableContext *context, ut64 curAddress, ut64 *value) {
@@ -83,18 +83,54 @@ static bool vtable_is_value_in_text_section(RVTableContext *context, ut64 curAdd
 	return ret;
 }
 
-static bool vtable_section_can_contain_vtables(RVTableContext *context, RBinSection *section) {
+static bool vtable_section_can_contain_vtables(RBinSection *section) {
 	if (section->is_segment) {
 		return false;
 	}
 	return !strcmp (section->name, ".rodata") ||
 		!strcmp (section->name, ".rdata") ||
 		!strcmp (section->name, ".data.rel.ro") ||
+		!strcmp (section->name, ".data.rel.ro.local") ||
 		r_str_endswith (section->name, "__const");
 }
 
+static bool section_can_contain_rtti(RBinSection *section) {
+	if (!section) {
+		return false;
+	}
+	if (section->is_data) {
+		return true;
+	}
+	return !strcmp (section->name, ".data.rel.ro") ||
+		!strcmp (section->name, ".data.rel.ro.local") ||
+		r_str_endswith (section->name, "__const");
+}
 
-static int vtable_is_addr_vtable_start(RVTableContext *context, ut64 curAddress) {
+static bool vtable_is_addr_vtable_start_itanium(RVTableContext *context, RBinSection *section, ut64 curAddress) {
+	ut64 value;
+	if (!curAddress || curAddress == UT64_MAX) {
+		return false;
+	}
+	if (curAddress && !vtable_is_value_in_text_section (context, curAddress, NULL)) { // Vtable beginning referenced from the code
+		return false;
+	}
+	if (!context->read_addr (context->anal, curAddress - context->word_size, &value)) { // get the RTTI pointer
+		return false;
+	}
+	RBinSection *rtti_section = context->anal->binb.get_vsect_at (context->anal->binb.bin, value);
+	if (value && !section_can_contain_rtti (rtti_section)) { // RTTI ptr must point somewhere in the data section
+		return false;
+	}
+	if (!context->read_addr (context->anal, curAddress - 2 * context->word_size, &value)) { // Offset to top
+		return false;
+	}
+	if ((st32)value > 0) { // Offset to top has to be negative
+		return false;
+	}
+	return true;
+}
+
+static bool vtable_is_addr_vtable_start_msvc(RVTableContext *context, ut64 curAddress) {
 	RAnalRef *xref;
 	RListIter *xrefIter;
 
@@ -133,12 +169,30 @@ static int vtable_is_addr_vtable_start(RVTableContext *context, ut64 curAddress)
 	return false;
 }
 
+static bool vtable_is_addr_vtable_start(RVTableContext *context, RBinSection *section, ut64 curAddress) {
+	if (context->abi == R_ANAL_CPP_ABI_MSVC) {
+		return vtable_is_addr_vtable_start_msvc (context, curAddress);
+	}
+	if (context->abi == R_ANAL_CPP_ABI_ITANIUM) {
+		return vtable_is_addr_vtable_start_itanium (context, section, curAddress);
+	}
+	r_return_val_if_reached (false);
+	return false;
+}
+
 R_API RVTableInfo *r_anal_vtable_parse_at(RVTableContext *context, ut64 addr) {
-	RVTableInfo *vtable = calloc (1, sizeof(RVTableInfo));
+	ut64 offset_to_top;
+	if (!context->read_addr (context->anal, addr - 2 * context->word_size, &offset_to_top)) {
+		return NULL;
+	}
+
+	RVTableInfo *vtable = calloc (1, sizeof (RVTableInfo));
 	if (!vtable) {
 		return NULL;
 	}
+
 	vtable->saddr = addr;
+
 	r_vector_init (&vtable->methods, sizeof (RVTableMethodInfo), NULL, NULL);
 
 	RVTableMethodInfo meth;
@@ -187,21 +241,25 @@ R_API RList *r_anal_vtable_search(RVTableContext *context) {
 			break;
 		}
 
-		if (!vtable_section_can_contain_vtables (context, section)) {
+		if (!vtable_section_can_contain_vtables (section)) {
 			continue;
 		}
 
-		// ut8 *segBuff = calloc (1, section->vsize);
-		// r_io_read_at (core->io, section->vaddr, segBuff, section->vsize);
-
 		ut64 startAddress = section->vaddr;
 		ut64 endAddress = startAddress + (section->vsize) - context->word_size;
+		ut64 ss = endAddress - startAddress;
+		if (ss > ST32_MAX) {
+			break;
+		}
 		while (startAddress <= endAddress) {
 			if (r_cons_is_breaked ()) {
 				break;
 			}
+			if (!anal->iob.is_valid_offset (anal->iob.io, startAddress, 0)) {
+				break;
+			}
 
-			if (vtable_is_addr_vtable_start (context, startAddress)) {
+			if (vtable_is_addr_vtable_start (context, section, startAddress)) {
 				RVTableInfo *vtable = r_anal_vtable_parse_at (context, startAddress);
 				if (vtable) {
 					r_list_append (vtables, vtable);
@@ -212,7 +270,7 @@ R_API RList *r_anal_vtable_search(RVTableContext *context) {
 					}
 				}
 			}
-			startAddress += 1;
+			startAddress += context->word_size;
 		}
 	}
 
@@ -232,34 +290,35 @@ R_API void r_anal_list_vtables(RAnal *anal, int rad) {
 
 	const char *noMethodName = "No Name found";
 	RVTableMethodInfo *curMethod;
-	RListIter* vtableIter;
-	RVTableInfo* table;
+	RListIter *vtableIter;
+	RVTableInfo *table;
 
-	RList* vtables = r_anal_vtable_search (&context);
+	RList *vtables = r_anal_vtable_search (&context);
 
 	if (rad == 'j') {
-		bool isFirstElement = true;
-		r_cons_print ("[");
-		r_list_foreach (vtables, vtableIter, table) {
-			if (!isFirstElement) {
-				r_cons_print (",");
-			}
-			bool isFirstMethod = true;
-			r_cons_printf ("{\"offset\":%"PFMT64d",\"methods\":[", table->saddr);
-			r_vector_foreach (&table->methods, curMethod) {
-				if (!isFirstMethod) {
-					r_cons_print (",");
-				}
-				RAnalFunction *fcn = r_anal_get_fcn_in (anal, curMethod->addr, 0);
-				const char* const name = fcn ? fcn->name : NULL;
-				r_cons_printf ("{\"offset\":%"PFMT64d",\"name\":\"%s\"}",
-						curMethod->addr, name ? name : noMethodName);
-				isFirstMethod = false;
-			}
-			r_cons_print ("]}");
-			isFirstElement = false;
+		PJ *pj = pj_new ();
+		if (!pj) {
+			return;
 		}
-		r_cons_println ("]");
+		pj_a (pj);
+		r_list_foreach (vtables, vtableIter, table) {
+			pj_o (pj);
+			pj_kN (pj, "offset", table->saddr);
+			pj_ka (pj, "methods");
+			r_vector_foreach (&table->methods, curMethod) {
+				RAnalFunction *fcn = r_anal_get_fcn_in (anal, curMethod->addr, 0);
+				const char *const name = fcn ? fcn->name : NULL;
+				pj_o (pj);
+				pj_kN (pj, "offset", curMethod->addr);
+				pj_ks (pj, "name", r_str_get_fail (name, noMethodName));
+				pj_end (pj);
+			}
+			pj_end (pj);
+			pj_end (pj);
+		}
+		pj_end (pj);
+		r_cons_println (pj_string (pj));
+		pj_free (pj);
 	} else if (rad == '*') {
 		r_list_foreach (vtables, vtableIter, table) {
 			r_cons_printf ("f vtable.0x%08"PFMT64x" %"PFMT64d" @ 0x%08"PFMT64x"\n",
@@ -283,8 +342,8 @@ R_API void r_anal_list_vtables(RAnal *anal, int rad) {
 			r_cons_printf ("\nVtable Found at 0x%08"PFMT64x"\n", vtableStartAddress);
 			r_vector_foreach (&table->methods, curMethod) {
 				RAnalFunction *fcn = r_anal_get_fcn_in (anal, curMethod->addr, 0);
-				const char* const name = fcn ? fcn->name : NULL;
-				r_cons_printf ("0x%08"PFMT64x" : %s\n", vtableStartAddress, name ? name : noMethodName);
+				const char *const name = fcn ? fcn->name : NULL;
+				r_cons_printf ("0x%08"PFMT64x" : %s\n", vtableStartAddress, r_str_get_fail (name, noMethodName));
 				vtableStartAddress += context.word_size;
 			}
 			r_cons_newline ();
@@ -292,5 +351,3 @@ R_API void r_anal_list_vtables(RAnal *anal, int rad) {
 	}
 	r_list_free (vtables);
 }
-
-
