@@ -11,7 +11,9 @@ typedef enum optype_t {
 	ARM_GPR = 1,
 	ARM_CONSTANT = 2,
 	ARM_FP = 4,
-	ARM_MEM_OPT = 8
+	ARM_MEM_OPT = 8,
+	ARM_SHIFT = 16,
+	ARM_EXTEND = 32
 } OpType;
 
 typedef enum regtype_t {
@@ -24,12 +26,27 @@ typedef enum regtype_t {
 } RegType;
 
 typedef enum shifttype_t {
-	ARM_NO_SHIFT = -1,
 	ARM_LSL = 0,
 	ARM_LSR = 1,
-	ARM_ASR = 2
+	ARM_ASR = 2,
+	ARM_ROR = 3,
+	ARM_UXTB,
+	ARM_UXTH,
+	ARM_UXTW,
+	ARM_UXTX,
+	ARM_SXTB,
+	ARM_SXTH,
+	ARM_SXTW,
+	ARM_SXTX
 } ShiftType;
 
+typedef enum logicalop_t {
+	ARM_AND = 0,
+	ARM_ORR = 1,
+	ARM_EOR = 2,
+	ARM_ANDS = 3
+} LogicalOp;
+
 typedef struct operand_t {
 	OpType type;
 	union {
@@ -40,11 +57,13 @@ typedef struct operand_t {
 		};
 		struct {
 			ut64 immediate;
-			int sign;
+			bool sign;
+			bool preindex;
 		};
 		struct {
 			ut64 shift_amount;
 			ShiftType shift;
+			bool amount_present;
 		};
 		struct {
 			ut32 mem_option;
@@ -59,10 +78,13 @@ typedef struct Opcode_t {
 	ut32 op[3];
 	size_t op_len;
 	ut8 opcode[3];
+	bool writeback;
 	int operands_count;
 	Operand operands[MAX_OPERANDS];
 } ArmOp;
 
+#define check_cond(cond) if (!(cond)) { return data; }
+
 static int get_mem_option(char *token) {
 	// values 4, 8, 12, are unused. XXX to adjust
 	const char *options[] = {"sy", "st", "ld", "xxx", "ish", "ishst",
@@ -78,8 +100,8 @@ static int get_mem_option(char *token) {
 	return -1;
 }
 
-static int countLeadingZeros(ut32 x) {
-	int count = 0;
+static int countLeadingZeros(ut64 x) {
+	int count = 64;
 	while (x) {
 		x >>= 1;
 		--count;
@@ -87,15 +109,11 @@ static int countLeadingZeros(ut32 x) {
 	return count;
 }
 
-static int countTrailingZeros(ut32 x) {
+static int countTrailingZeros(ut64 x) {
 	int count = 0;
-	while (x > 0) {
-		if ((x & 1) == 1) {
-			break;
-		} else {
-			count ++;
-			x = x >> 1;
-		}
+	while (x && !(x & 1)) {
+		count++;
+		x >>= 1;
 	}
 	return count;
 }
@@ -120,35 +138,49 @@ static int calcNegOffset(int n, int shift) {
 	return 0xff & (0xff - a);
 }
 
-static int countLeadingOnes(ut32 x) {
+static int countLeadingOnes(ut64 x) {
 	return countLeadingZeros (~x);
 }
 
-static int countTrailingOnes(ut32 x) {
+static int countTrailingOnes(ut64 x) {
 	return countTrailingZeros (~x);
 }
 
-static bool isMask(ut32 value) {
+static bool isMask(ut64 value) {
   return value && ((value + 1) & value) == 0;
 }
 
-static bool isShiftedMask (ut32 value) {
+static bool isShiftedMask (ut64 value) {
   return value && isMask ((value - 1) | value);
 }
 
-static ut32 decodeBitMasks(ut32 imm) {
+// https://llvm.org/doxygen/AArch64AddressingModes_8h_source.html
+static ut32 encodeBitMasksWithSize(ut64 imm, ut32 reg_size) {
+	if (imm == 0 || imm == UT64_MAX || (reg_size != 64
+		&& (imm >> reg_size != 0 || imm == (~0ULL >> (64 - reg_size))))) {
+		return UT32_MAX;
+	}
 	// get element size
-	int size = 32;
+	ut32 size = reg_size;
+	do {
+		size >>= 1;
+		ut64 mask = (1ull << size) - 1;
+		if ((imm & mask) != ((imm >> size) & mask)) {
+			size <<= 1;
+			break;
+		}
+   } while (size > 2);
 	// determine rot to make element be 0^m 1^n
 	ut32 cto, i;
-	ut32 mask = ((ut64) - 1LL) >> (64 - size);
+	ut64 mask = UT64_MAX >> (64 - size);
+	imm &= mask;
 
 	if (isShiftedMask (imm)) {
 		i = countTrailingZeros (imm);
 		cto = countTrailingOnes (imm >> i);
 	} else {
 		imm |= ~mask;
-		if (!isShiftedMask (imm)) {
+		if (!isShiftedMask (~imm)) {
 			return UT32_MAX;
 		}
 
@@ -169,67 +201,107 @@ static ut32 decodeBitMasks(ut32 imm) {
 	nimms |= (cto - 1);
 	// Extract and toggle seventh bit to make N field.
 	ut32 n = ((nimms >> 6) & 1) ^ 1;
-	ut64 encoding = (n << 12) | (immr << 6) | (nimms & 0x3f);
+	ut32 encoding = (n << 12) | (immr << 6) | (nimms & 0x3f);
 	return encoding;
 }
 
+static inline ut32 encode1reg(ArmOp *op) {
+	return op->operands[0].reg << 24;
+}
+
+static inline ut32 encode2regs(ArmOp *op) {
+	return (op->operands[1].reg & 0x7) << 29 | (op->operands[1].reg & 0x18) << 13
+		| encode1reg (op);
+}
+
+static inline ut32 encodeImm9(ut32 n) {
+	return (n & 0x1f0) << 4 | (n & 0xf) << 20;
+}
+
 static ut32 mov(ArmOp *op) {
-	int k = 0;
+	ut32 k = 0;
 	ut32 data = UT32_MAX;
-	if (!strncmp (op->mnemonic, "movz", 4)) {
-		if (op->operands[0].reg_type & ARM_REG64) {
-			k = 0x80d2;
-		} else if (op->operands[0].reg_type & ARM_REG32) {
-			k = 0x8052;
+	check_cond (op->operands_count >= 2);
+	check_cond (op->operands[0].type == ARM_GPR);
+	int bits = (op->operands[0].reg_type & ARM_REG64) ? 64 : 32;
+	if (bits == 64) {
+		k = 0x0080;
+	}
+	k |= encode1reg (op);
+	if (!strcmp (op->mnemonic, "mov")) {
+		check_cond (op->operands_count == 2);
+		if (op->operands[1].type == ARM_GPR) {
+			check_cond (bits == ((op->operands[1].reg_type & ARM_REG64) ? 64 : 32));
+			if (op->operands[0].reg_type & ARM_SP || op->operands[1].reg_type & ARM_SP) { // alias of add
+				k |= 0x0011;
+				data = k | encode2regs (op);
+				return data;
+			}
+			k |= 0xe003002a; // alias of orr
+			data = k | op->operands[1].reg << 8;
+			return data;
 		}
-	} else if (!strncmp (op->mnemonic, "movk", 4)) {
-		if (op->operands[0].reg_type & ARM_REG32) {
-			k = 0x8072;
-		} else if (op->operands[0].reg_type & ARM_REG64) {
-			k = 0x80f2;
+		check_cond (op->operands[1].type & ARM_CONSTANT);
+		ut64 imm = op->operands[1].immediate;
+		ut64 imm_inverse = ~imm;
+		if (bits == 32) {
+			check_cond (imm <= 0xffffffff || imm_inverse <= 0xffffffff);
+			imm &= 0xffffffff;
+			imm_inverse &= 0xffffffff;
 		}
-	} else if (!strncmp (op->mnemonic, "movn", 4)) {
-		if (op->operands[0].reg_type & ARM_REG32) {
-			k = 0x8012;
-		} else if (op->operands[0].reg_type & ARM_REG64) {
-			k = 0x8092;
+		int shift;
+		ut64 mask = 0xffff;
+		for (shift = 0; shift < bits; shift += 16, mask <<= 16) {
+			if (imm == (imm & mask)) { // movz
+				data = k | 0x00008052;
+				imm >>= shift;
+				data |= (imm & 7) << 29 | (imm & 0x7f8) << 13 | (imm & 0xf800) >> 3;
+				data |= shift << 9;
+				return data;
+			}
 		}
-	} else if (!strncmp (op->mnemonic, "mov", 3)) {
-		//printf ("%d - %d [%d]\n", op->operands[0].type, op->operands[1].type, ARM_GPR);
-		if (op->operands[0].reg_type & ARM_REG64) {
-            if (op->operands[1].reg_type & ARM_REG64) {
-                k = 0xe00300aa;
-            } else if (op->operands[1].type & ARM_CONSTANT) {
-                k = 0x80d2;
-            } else {
-                return data;
-            }
-        } else if (op->operands[0].reg_type & ARM_REG32) {
-            if (op->operands[1].reg_type & ARM_REG32) {
-                k = 0xe003002a;
-            } else if (op->operands[1].type & ARM_CONSTANT) {
-                k = 0x8052;
-            } else {
-                return data;
-            }
-        }
-		if (op->operands[0].type & ARM_GPR) {
-            if (op->operands[1].type & ARM_GPR) {
-                data = k | op->operands[1].reg << 8;
-            } else if (op->operands[1].type & ARM_CONSTANT) {
-                ut32 imm = op->operands[1].immediate << 1;
-                data = k | ((imm & 0xf) << 28) | ((imm & 0x1f0) << 12) ;
-            }
-			data |=  op->operands[0].reg << 24;
-        }
+		mask = 0xffff;
+		for (shift = 0; shift < bits; shift += 16, mask <<= 16) {
+			if (imm_inverse == (imm_inverse & mask)) { // movn
+				data = k | 0x00008012;
+				imm_inverse >>= shift;
+				data |= (imm_inverse & 7) << 29 | (imm_inverse & 0x7f8) << 13 | (imm_inverse & 0xf800) >> 3;
+				data |= shift << 9;
+				return data;
+			}
+		}
+		ut32 bitmask = encodeBitMasksWithSize (op->operands[1].immediate, bits); // orr
+		check_cond (bitmask != UT32_MAX);
+		data = k | 0xe0030032;
+		data |= (bitmask & 0x3f) << 18 | (bitmask & 0x1fc0) << 2;
 		return data;
 	}
-
+	if (!strcmp (op->mnemonic, "movz")) {
+		k |= 0x8052;
+	} else if (!strcmp (op->mnemonic, "movk")) {
+		k |= 0x8072;
+	} else if (!strcmp (op->mnemonic, "movn")) {
+		k |= 0x8012;
+	} else {
+		return data;
+	}
+	check_cond (op->operands[1].type == ARM_CONSTANT);
+	ut64 imm = op->operands[1].immediate;
+	check_cond (imm <= 0xffff);
+	int shift = 0;
+	if (op->operands_count >= 3) {
+		check_cond (op->operands_count == 3);
+		check_cond (op->operands[2].type == ARM_SHIFT);
+		check_cond (op->operands[2].shift == ARM_LSL);
+		shift = op->operands[2].shift_amount;
+		check_cond (!(shift & 0xf));
+		check_cond (shift < bits);
+	}
 	data = k;
-	data |= (op->operands[0].reg << 24); // arg(0)
-	data |= ((op->operands[1].immediate & 7) << 29); // arg(1)
-	data |= (((op->operands[1].immediate >> 3) & 0xff) << 16); // arg(1)
-	data |= ((op->operands[1].immediate >> 10) << 7); // arg(1)
+	data |= (imm & 7) << 29; // arg(1)
+	data |= (imm & 0x7f8) << 13; // arg(1)
+	data |= (imm & 0xf800) >> 3; // arg(1)
+	data |= shift << 9; // arg(2)
 	return data;
 }
 
@@ -257,7 +329,7 @@ static ut32 cb(ArmOp *op) {
     }
     //printf ("%s %d, %llu\n", op->mnemonic, op->operands[0].reg, op->operands[1].immediate);
     ut32 imm = op->operands[1].immediate;
-	data = k | (op->operands[0].reg << 24) | ((imm & 0x1c) << 27) | ((imm & 0x1fe0) << 11);
+	data = k | encode1reg (op) | ((imm & 0x1c) << 27) | ((imm & 0x1fe0) << 11);
     data = data | ((imm & 0x1fe000) >> 5);
 
 	return data;
@@ -274,13 +346,24 @@ static ut32 cmp(ArmOp *op) {
 			return UT32_MAX;
 		}
 		k =  0x1f00006b;
+	} else if (op->operands[0].type & ARM_GPR  && op->operands[1].type & ARM_CONSTANT) {
+		if (op->operands[0].reg_type & ARM_REG64) {
+			k =  0x1f0000f1;
+		} else { 
+			k =  0x1f000071;
+		}
+		k |= ( op->operands[1].immediate  * 4 ) << 16 ;
 	} else {
 		return UT32_MAX;
 	}
 
-	data = k | (op->operands[0].reg & 0x18) << 13 | op->operands[0].reg << 29 | op->operands[1].reg << 8;
+	data = k | (op->operands[0].reg & 0x18) << 13 | op->operands[0].reg << 29;
 
-	if (op->operands[2].shift != ARM_NO_SHIFT) {
+	if (op->operands[1].type &  ARM_GPR) {
+		data |= op->operands[1].reg << 8;
+	}
+
+	if (op->operands[2].type != ARM_SHIFT) {
 		data |= op->operands[2].shift_amount << 18 | op->operands[2].shift << 14;
 	}
 	return data;
@@ -305,7 +388,7 @@ static ut32 regsluop(ArmOp *op, int k) {
 		return data;
 	}
 
-	data = k | op->operands[0].reg << 24 | op->operands[1].reg << 29 | (op->operands[1].reg & 56) << 13;
+	data = k | encode2regs (op);
 
 	if (n < 0) {
 		n *= -1;
@@ -341,7 +424,7 @@ static ut32 reglsop(ArmOp *op, int k) {
 	}
 	if (op->operands[2].type & ARM_GPR) {
 		k += 0x00682000;
-		data = k | op->operands[0].reg << 24 | op->operands[1].reg << 29 | (op->operands[1].reg & 56) << 13;
+		data = k | encode2regs (op);
 		data |= op->operands[2].reg << 8;
 	} else {
 		int n = op->operands[2].immediate;
@@ -352,7 +435,7 @@ static ut32 reglsop(ArmOp *op, int k) {
 		if (n == 0 || (n > 0 && countTrailingZeros(n) >= 4)) {
 			k ++;
 		}
-		data = k | op->operands[0].reg << 24 | op->operands[1].reg << 29 | (op->operands[1].reg & 56) << 13;
+		data = k | encode2regs (op);
 
 		if (n < 0) {
 			n *= -1;
@@ -386,81 +469,126 @@ static ut32 reglsop(ArmOp *op, int k) {
 	return data;
 }
 
-// Byte load/store ops
-static ut32 bytelsop(ArmOp *op, int k) {
+// load/store ops
+static ut32 lsop(ArmOp *op, int k, ut64 addr) {
 	ut32 data = UT32_MAX;
-
-	if (op->operands[0].reg_type & ARM_REG64) {
-		return data;
+	int op_count = op->operands_count;
+	if (k == 0x00000098) { // ldrsw
+		if (op->operands[0].type & ARM_GPR
+			&& op->operands[1].type & ARM_CONSTANT) { // (literal)
+			st64 offset = op->operands[1].immediate - addr;
+			check_cond (op->operands[0].reg_type & ARM_REG64);
+			check_cond (!(offset & 0x3));
+			check_cond (-0x100000 <= offset && offset < 0x100000);
+			offset >>= 2;
+			data = k | (offset & 0x7f800) >> 3 | (offset & 0x7f8) << 13
+				| (offset & 0x7) << 29 | encode1reg (op);
+			return data;
+		}
+		k = 0x000080b8;
 	}
-	if (op->operands[1].reg_type & ARM_REG32) {
-		return data;
+	check_cond (op->operands[0].type == ARM_GPR);
+	check_cond (op->operands[1].type == ARM_GPR);
+	check_cond (op->operands[1].reg_type & ARM_REG64);
+	k |= encode2regs (op);
+	if (!strcmp (op->mnemonic, "ldrb") || !strcmp (op->mnemonic, "ldrh")
+		|| !strcmp (op->mnemonic, "strb") || !strcmp (op->mnemonic, "strh")) {
+		check_cond (op->operands[0].reg_type & ARM_REG32);
+	} else if (!strcmp (op->mnemonic, "ldrsw")) {
+		check_cond (op->operands[0].reg_type & ARM_REG64);
+	} else { // ldrsh, ldrsb
+		if (op->operands[0].reg_type & ARM_REG32) {
+			k |= 0x00004000;
+		}
 	}
+	char width = op->mnemonic[strlen (op->mnemonic) - 1];
 	if (op->operands[2].type & ARM_GPR) {
-		if ((k & 0xf) != 8) {
-			k--;
+		k |= 0x00482000;
+		if (op->operands[3].type == ARM_EXTEND) {
+			switch (op->operands[3].shift) {
+			case ARM_SXTW:
+				k |= 0x00800000;
+			// fall through
+			case ARM_UXTW:
+				check_cond (op->operands[2].reg_type & ARM_REG32);
+				break;
+			case ARM_SXTX:
+				k |= 0x00a00000;
+				check_cond (op->operands[2].reg_type & ARM_REG64);
+				break;
+			default:
+				return data;
+			}
+		} else if (op->operands[3].type == ARM_SHIFT) {
+			check_cond (op->operands[3].shift == ARM_LSL);
+			check_cond (op->operands[2].reg_type & ARM_REG64);
+			k |= 0x00200000;
 		}
-		k += 0x00682000;
-		data = k | op->operands[0].reg << 24 | op->operands[1].reg << 29 | (op->operands[1].reg & 56) << 13;
-		data |= op->operands[2].reg << 8;
+		if (op->operands[3].type == ARM_EXTEND || op->operands[3].type == ARM_SHIFT) {
+			if (width == 'b') {
+				check_cond (op->operands[3].shift_amount == 0);
+				if (op->operands[3].amount_present) {
+					k |= 0x00100000;
+				}
+			} else if (width == 'h') {
+				switch (op->operands[3].shift_amount) {
+				case 1:
+					k |= 0x00100000;
+				// fall through
+				case 0:
+					break;
+				default:
+					return data;
+				}
+			} else { // w
+				switch (op->operands[3].shift_amount) {
+				case 2:
+					k |= 0x00100000;
+				// fall through
+				case 0:
+					break;
+				default:
+					return data;
+				}
+			}
+		} else { // lsl 0 by default
+			check_cond (op->operands[2].reg_type & ARM_REG64);
+			k |= 0x00200000;
+		}
+		data = k | op->operands[2].reg << 8;
 		return data;
 	}
-
-	int n = op->operands[2].immediate;
-	if (n > 0xfff || n < -0x100) {
-		return UT32_MAX;
-	}
-	// Half ops
-	int halfop = false;
-	if ((k & 0xf) == 8) {
-		halfop = true;
-		if (n == 0 || (countTrailingZeros(n) && n > 0)) {
-			k++;
-		}
-	} else {
-		if (n < 0) {
-			k--;
+	check_cond (op_count == 2 || op->operands[2].type == ARM_CONSTANT);
+	check_cond (!op->writeback || op->operands[2].preindex);
+	int n = op_count == 2 ? 0 : op->operands[2].immediate;
+	if (!op->writeback && (op_count == 2 || op->operands[2].preindex)) { // unsigned offset
+		check_cond (n >= 0);
+		if (width == 'b') {
+			check_cond (n <= 0xfff);
+		} else if (width == 'h') {
+			check_cond (n <= 0x1ffe && !(n & 1))
+			n >>= 1;
+		} else { // w
+			check_cond (n <= 0x3ffc && !(n & 3));
+			n >>= 2;
 		}
+		data = k | (n & 0x3f) << 18 | (n & 0xfc0) << 2 | 1;
+		return data;
 	}
-
-	data = k | op->operands[0].reg << 24 | op->operands[1].reg << 29 | (op->operands[1].reg & 56) << 13;
-
-	int imm = n;
-	int low_shift = 20;
-	int high_shift = 8;
-	int top_shift = 10;
-	if (n < 0) {
-		imm = 0xfff + (n + 1);
-	}
-	if (halfop) {
-		if (imm & 0x1 || n < 0) {
-			data |= (0xf & imm) << low_shift ;
-			data |= (0x7 & (imm >> 4)) << high_shift;
-			data |= (0x7 & (imm >> 6)) << top_shift;
-		} else {
-			data |= (0xf & imm) << (low_shift - 3);
-			data |= (0x7 & (imm >> 4)) << (high_shift + 13);
-			data |= (0x7 & (imm >> 7)) << (top_shift  - 2);
-		}
-	} else {
-		if (n < 0) {
-			data |= (0xf & imm) << 20;
-			data |= (0x1f & (imm >> 4)) << 8;
-		} else {
-			data |= (0xf & imm) << 18;
-			data |= (0x3 & (imm >> 4)) << 22;
-			data |= (0x7 & (imm >> 6)) << 8;
-		}
+	check_cond (-0x100 <= n && n < 0x100)
+	if (op->operands[2].preindex) {
+		k |= 0x00080000;
 	}
+	data = k | encodeImm9 (n) | 0x00040000;
 	return data;
 }
 
 static ut32 branch(ArmOp *op, ut64 addr, int k) {
 	ut32 data = UT32_MAX;
-	int n = 0;
+	ut64 n = 0;
 	if (op->operands[0].type & ARM_CONSTANT) {
 		n = op->operands[0].immediate;
-		if (!(n & 0x3 || n > 0x7ffffff)) {
+		if (!(n & 0x3)) {
 			if (n >= addr) {
 				n -= addr;
 			} else {
@@ -469,8 +597,8 @@ static ut32 branch(ArmOp *op, ut64 addr, int k) {
 				k |= 3;
 			}
 			n = n >> 2;
-			int t = n >> 24;
-			int h = n >> 16;
+			int t = (n & 0xff000000) >> 24;
+			int h = (n & 0xff0000) >> 16;
 			int m = (n & 0xff00) >> 8;
 			n &= 0xff;
 			data = k;
@@ -481,7 +609,7 @@ static ut32 branch(ArmOp *op, ut64 addr, int k) {
 		}
 	} else {
 		n = op->operands[0].reg;
-		if (n < 0 || n > 31) {
+		if (n >= 31) {
 			return -1;
 		}
 		n = n << 5;
@@ -524,7 +652,7 @@ static ut32 bdot(ArmOp *op, ut64 addr, int k) {
 	return data;
 }
 
-static ut32 mem_barrier (ArmOp *op, ut64 addr, int k) {
+static ut32 mem_barrier(ArmOp *op, ut64 addr, int k) {
 	ut32 data = UT32_MAX;
 	data = k;
 	if (!strncmp (op->mnemonic, "isb", 3)) {
@@ -624,65 +752,78 @@ if (op->operands[1].reg_type == ARM_REG64) {
 	return seq_data;
 }
 
-static ut32 orr(ArmOp *op, int addr) {
+static ut32 logical(ArmOp *op, bool invert, LogicalOp opc) {
 	ut32 data = UT32_MAX;
+	RegType reg_type = op->operands[0].reg_type;
 
-	if (op->operands[2].type & ARM_GPR) {
-		// All operands need to be the same
-		if (!(op->operands[0].reg_type == op->operands[1].reg_type &&
-	 	    op->operands[1].reg_type == op->operands[2].reg_type)) {
-		 	   return data;
-		}
-		if (op->operands[0].reg_type & ARM_REG64) {
-			data = 0x000000aa;
-		} else {
-			data = 0x0000002a;
-		}
-		data += op->operands[0].reg << 24;
-		data += op->operands[1].reg << 29;
-		data += (op->operands[1].reg >> 3)  << 16;
-		data += op->operands[2].reg << 8;
-	} else if (op->operands[2].type & ARM_CONSTANT) {
-		// Reg types need to match
-		if (!(op->operands[0].reg_type == op->operands[1].reg_type)) {
-			return data;
-		}
-		if (op->operands[0].reg_type & ARM_REG64) {
-			data = 0x000040b2;
-		} else {
-			data = 0x00000032;
-		}
-
-		data += op->operands[0].reg << 24;
-		data += op->operands[1].reg << 29;
-		data += (op->operands[1].reg >> 3)  << 16;
-
-		ut32 imm = decodeBitMasks (op->operands[2].immediate);
-		if (imm == -1) {
-			return imm;
-		}
-		int low = imm & 0xF;
-		if (op->operands[0].reg_type & ARM_REG64) {
-			imm = ((imm >> 6) | 0x78);
-			if (imm > 120) {
-				data |= imm << 8;
-			}
-		} else {
-			imm = ((imm >> 2));
-			if (imm > 120) {
-				data |= imm << 4;
-			}
-		}
-		data |= (4 * low) << 16;
+	// Reg types need to match
+	if (!(reg_type == op->operands[1].reg_type)) {
+		return data;
 	}
-	return data;
+
+	OpType op2_type = op->operands[2].type;
+	if (op2_type == ARM_CONSTANT) {
+		if (invert) {
+			/* there aren't inverted immediates in arm64 */
+			return UT32_MAX;
+		}
+		if (reg_type & ARM_REG64) {
+			data = 0x92000000;
+		} else if (reg_type & ARM_REG32) {
+			data = 0x12000000;
+		} else {
+			return UT32_MAX;
+		}
+
+		bool is64bit = reg_type & ARM_REG64;
+
+		data |= op->operands[0].reg;
+		data |= op->operands[1].reg << 5;
+		data |= (opc & 3) << 29;
+
+		ut32 imm_orig = op->operands[2].immediate;
+		ut32 imm = encodeBitMasksWithSize (invert? ~imm_orig: imm_orig, is64bit? 64: 32);
+		if (imm == UT32_MAX) {
+			return UT32_MAX;
+		}
+		data |= (imm & 0x1fff) << 10;
+	} else if (op2_type == ARM_GPR) {
+		if (reg_type & ARM_REG64) {
+			data = 0x8a000000;
+		} else if (reg_type & ARM_REG32) {
+			data = 0x0a000000;
+		} else {
+			return UT32_MAX;
+		}
+
+		data |= op->operands[0].reg;
+		data |= op->operands[1].reg << 5;
+		data |= op->operands[2].reg << 16;
+		data |= (opc & 3) << 29;
+
+		if (op->operands_count == 4) {
+			Operand shift_op = op->operands[3];
+			if (shift_op.type == ARM_SHIFT) {
+				data |= (shift_op.shift_amount & 0x3f) << 10;
+				data |= (shift_op.shift & 0x3) << 22;
+			}
+		}
+
+		if (invert) {
+			data |= 1 << 21;
+		}
+	} else {
+		return UT32_MAX;
+	}
+
+	return r_read_be32 (&data);
 }
 
 static ut32 adrp(ArmOp *op, ut64 addr, ut32 k) { //, int reg, ut64 dst) {
 	ut64 at = 0LL;
 	ut32 data = k;
 	if (op->operands[0].type == ARM_GPR) {
-		data += ((op->operands[0].reg & 0xff) << 24);
+		data |= encode1reg (op);
 	} else {
 		eprintf ("Usage: adrp x0, addr\n");
 		return UT32_MAX;
@@ -719,7 +860,7 @@ static ut32 adr(ArmOp *op, int addr) {
 		at /= 4;
 	}
 	data = 0x00000030;
-	data += 0x01000000 * op->operands[0].reg;
+	data |= encode1reg (op);
 	ut8 b0 = at;
 	ut8 b1 = (at >> 3) & 0xff;
 	ut8 b2 = (at >> (8 + 7)) & 0xff;
@@ -740,10 +881,7 @@ static ut32 stp(ArmOp *op, int k) {
 	}
 
 	data = k;
-	data += op->operands[0].reg << 24;
-	data += op->operands[1].reg << 18;
-	data += (op->operands[2].reg & 0x7) << 29;
-	data += (op->operands[2].reg >> 3) << 16;
+	data |= encode2regs (op);
 	data += (op->operands[3].immediate & 0x8) << 20;
 	data += (op->operands[3].immediate >> 4) << 8;
 	return data;
@@ -761,7 +899,7 @@ static ut32 exception(ArmOp *op, ut32 k) {
 	return data;
 }
 
-static ut32 arithmetic (ArmOp *op, int k) {
+static ut32 arithmetic(ArmOp *op, int k) {
 	ut32 data = UT32_MAX;
 	if (op->operands_count < 3) {
 		return data;
@@ -774,9 +912,8 @@ static ut32 arithmetic (ArmOp *op, int k) {
 	if (op->operands[2].type & ARM_GPR) {
 		k -= 6;
 	}
-
 	data = k;
-	data += op->operands[0].reg << 24;
+	data += encode1reg (op);
 	data += (op->operands[1].reg & 7) << (24 + 5);
 	data += (op->operands[1].reg >> 3) << 16;
 	if (op->operands[2].type & ARM_GPR) {
@@ -785,6 +922,28 @@ static ut32 arithmetic (ArmOp *op, int k) {
 		data += (op->operands[2].reg & 0x3f) << 18;
 		data += (op->operands[2].reg >> 6) << 8;
 	}
+
+	if (op->operands[2].type & ARM_CONSTANT  && op->operands[3].type & ARM_SHIFT) {
+		if ((op->operands[3].shift == ARM_LSL) && (op->operands[3].shift_amount == 12)) {
+			data |= (0x4000);
+		}
+	}
+
+	if (op->operands[2].type & ARM_GPR  && op->operands[3].type & ARM_SHIFT) {
+		switch (op->operands[3].shift) {
+		case ARM_LSL:
+			data |= (0x00040000 * op->operands[3].shift_amount);
+			break;
+		case ARM_LSR:
+			data |= (0x00040000 * op->operands[3].shift_amount) | (0x4000);
+			break;
+		case ARM_ASR:
+			data |= (0x00040000 * op->operands[3].shift_amount) | (0x8000);
+			break;
+		default:
+			return data;
+		}
+	}
 	return data;
 }
 
@@ -799,6 +958,35 @@ static ut32 neg(ArmOp *op) {
 	return arithmetic (op, 0xd1); // sub reg0, xzr, reg1
 }
 
+static ut32 bitfield(ArmOp *op, int k) {
+	ut32 data = UT32_MAX;
+	check_cond (op->operands_count == 4);
+	check_cond (op->operands[0].type == ARM_GPR);
+	check_cond (op->operands[1].type == ARM_GPR);
+	check_cond (op->operands[0].reg_type == op->operands[1].reg_type);
+	check_cond (op->operands[2].type == ARM_CONSTANT);
+	check_cond (op->operands[3].type == ARM_CONSTANT);
+	int bits = (op->operands[0].reg_type & ARM_REG64) ? 64 : 32;
+	// unalias
+	if (!strcmp (op->mnemonic, "sbfx") || !strcmp (op->mnemonic, "ubfx")) {
+		op->operands[3].immediate += op->operands[2].immediate - 1;
+	} else if (!strcmp (op->mnemonic, "sbfiz") || !strcmp (op->mnemonic, "ubfiz")) {
+		check_cond (op->operands[2].immediate < bits);
+		int temp = bits - op->operands[2].immediate;
+		check_cond (op->operands[3].immediate <= temp);
+		op->operands[2].immediate = temp & (bits - 1);
+		op->operands[3].immediate--;
+	}
+	check_cond (op->operands[2].immediate < bits);
+	check_cond (op->operands[3].immediate < bits);
+	if (bits == 64) {
+		k |= 0x00004080;
+	}
+	k |= op->operands[2].immediate << 8 | op->operands[3].immediate << 18;
+	data = k | encode2regs (op);
+	return data;
+}
+
 static bool parseOperands(char* str, ArmOp *op) {
 	char *t = strdup (str);
 	int operand = 0;
@@ -807,6 +995,7 @@ static bool parseOperands(char* str, ArmOp *op) {
 	int imm_count = 0;
 	int mem_opt = 0;
 	int msr_op_index = 0;
+	size_t index_bound = strcspn (t, "]");
 	if (!token) {
 		return false;
 	}
@@ -827,7 +1016,6 @@ static bool parseOperands(char* str, ArmOp *op) {
 		}
 		op->operands[operand].type = ARM_NOTYPE;
 		op->operands[operand].reg_type = ARM_UNDEFINED;
-		op->operands[operand].shift = ARM_NO_SHIFT;
 
 		//parse MSR (immediate) operand 1
 		if (strcmp (op->mnemonic, "msr") == 0 && operand == 1) {
@@ -866,15 +1054,56 @@ static bool parseOperands(char* str, ArmOp *op) {
 		}
 
 		if (!strncmp (token, "lsl", 3)) {
+			op->operands[operand].type = ARM_SHIFT;
 			op->operands[operand].shift = ARM_LSL;
 		} else if (!strncmp (token, "lsr", 3)) {
+			op->operands[operand].type = ARM_SHIFT;
 			op->operands[operand].shift = ARM_LSR;
 		} else if (!strncmp (token, "asr", 3)) {
+			op->operands[operand].type = ARM_SHIFT;
 			op->operands[operand].shift = ARM_ASR;
+		} else if (!strncmp (token, "ror", 3)) {
+			op->operands[operand].type = ARM_SHIFT;
+			op->operands[operand].shift = ARM_ROR;
+		} else if (!strncmp (token, "uxtb", 4)) {
+			op->operands[operand].type = ARM_EXTEND;
+			op->operands[operand].shift = ARM_UXTB;
+		} else if (!strncmp (token, "uxth", 4)) {
+			op->operands[operand].type = ARM_EXTEND;
+			op->operands[operand].shift = ARM_UXTH;
+		} else if (!strncmp (token, "uxtw", 4)) {
+			op->operands[operand].type = ARM_EXTEND;
+			op->operands[operand].shift = ARM_UXTW;
+		} else if (!strncmp (token, "uxtx", 4)) {
+			op->operands[operand].type = ARM_EXTEND;
+			op->operands[operand].shift = ARM_UXTX;
+		} else if (!strncmp (token, "sxtb", 4)) {
+			op->operands[operand].type = ARM_EXTEND;
+			op->operands[operand].shift = ARM_SXTB;
+		} else if (!strncmp (token, "sxth", 4)) {
+			op->operands[operand].type = ARM_EXTEND;
+			op->operands[operand].shift = ARM_SXTH;
+		} else if (!strncmp (token, "sxtw", 4)) {
+			op->operands[operand].type = ARM_EXTEND;
+			op->operands[operand].shift = ARM_SXTW;
+		} else if (!strncmp (token, "sxtx", 4)) {
+			op->operands[operand].type = ARM_EXTEND;
+			op->operands[operand].shift = ARM_SXTX;
 		}
-		if (strlen (token) > 4 && op->operands[operand].shift != ARM_NO_SHIFT) {
+		if (op->operands[operand].type == ARM_SHIFT) {
 			op->operands_count ++;
-			op->operands[operand].shift_amount = r_num_math (NULL, token + 4);
+			token += 3;
+			while (*token && *token == ' ') {
+				token++;
+			}
+			if (*token == '#') {
+				token++;
+			}
+			if (!*token || !isdigit((unsigned char)*token)) {
+				return false;
+			}
+			op->operands[operand].shift_amount = r_num_math (NULL, token);
+			op->operands[operand].amount_present = true;
 			if (op->operands[operand].shift_amount > 63) {
 				free (t);
 				return false;
@@ -883,6 +1112,31 @@ static bool parseOperands(char* str, ArmOp *op) {
 			token = next;
 			continue;
 		}
+		if (op->operands[operand].type == ARM_EXTEND) {
+			op->operands_count++;
+			token += 4;
+			while (*token && *token == ' ') {
+				token++;
+			}
+			bool present = false;
+			if (*token == '#') {
+				present = true;
+				++token;
+			}
+			if (!*token || !isdigit((unsigned char)*token)) {
+				if (present) {
+					return false;
+				}
+				op->operands[operand].shift_amount = 0;
+				op->operands[operand].amount_present = false;
+			} else {
+				op->operands[operand].shift_amount = r_num_math (NULL, token);
+				op->operands[operand].amount_present = true;
+			}
+			operand++;
+			token = next;
+			continue;
+		}
 
 		switch (token[0]) {
 		case 'x':
@@ -917,6 +1171,7 @@ static bool parseOperands(char* str, ArmOp *op) {
 			} else if (!strncmp(token + 1, "sp", 2)) {
 				// WSP
 				op->operands[operand].reg = 31;
+				op->operands[operand].reg_type |= ARM_SP;
 			} else {
 				op->operands[operand].reg = r_num_math (NULL, token + 1);
 			}
@@ -978,6 +1233,7 @@ static bool parseOperands(char* str, ArmOp *op) {
 			op->operands_count ++;
 			op->operands[operand].type = ARM_CONSTANT;
 			op->operands[operand].immediate = r_num_math (NULL, token + 1);
+			op->operands[operand].preindex = token - t < index_bound;
 			imm_count++;
 			break;
 		case '-':
@@ -987,6 +1243,7 @@ static bool parseOperands(char* str, ArmOp *op) {
 			op->operands_count ++;
 			op->operands[operand].type = ARM_CONSTANT;
 			op->operands[operand].immediate = r_num_math (NULL, token);
+			op->operands[operand].preindex = token - t < index_bound;
 			imm_count++;
 			break;
 		}
@@ -1013,6 +1270,7 @@ static bool parseOpcode(const char *str, ArmOp *op) {
 	space[0] = '\0';
 	op->mnemonic = in;
 	space ++;
+	op->writeback = strstr (space, "]!") != NULL;
 	return parseOperands (space, op);
 }
 
@@ -1059,184 +1317,122 @@ static bool handlePAC(ut32 *op, const char *str) {
 bool arm64ass(const char *str, ut64 addr, ut32 *op) {
 	ArmOp ops = {0};
 	if (!parseOpcode (str, &ops)) {
+		free (ops.mnemonic);
 		return false;
 	}
 	/* TODO: write tests for this and move out the regsize logic into the mov */
 	if (!strncmp (str, "mov", 3)) {
 		*op = mov (&ops);
-		return *op != -1;
-	}
-	if (!strncmp (str, "cb", 2)) {
+	} else if (!strncmp (str, "cb", 2)) {
 		*op = cb (&ops);
-		return *op != -1;
-	}
-	if (!strncmp (str, "cmp", 3)) {
+	} else if (!strncmp (str, "cmp", 3)) {
 		*op = cmp (&ops);
-		return *op != -1;
-	}
-	if (!strncmp (str, "ldrb", 4)) {
-		*op = bytelsop (&ops, 0x00004039);
-		return *op != -1;
-	}
-	if (!strncmp (str, "ldrh", 4)) {
-		*op = bytelsop (&ops, 0x00004078);
-		return *op != -1;
-	}
-	if (!strncmp (str, "ldrsh", 5)) {
-		*op = bytelsop (&ops, 0x0000c078);
-		return *op != -1;
-	}
-	if (!strncmp (str, "ldrsw", 5)) {
-		*op = bytelsop (&ops, 0x000080b8);
-		return *op != -1;
-	}
-	if (!strncmp (str, "ldrsb", 5)) {
-		*op = bytelsop (&ops, 0x0000c039);
-		return *op != -1;
-	}
-	if (!strncmp (str, "strb", 4)) {
-		*op = bytelsop (&ops, 0x00000039);
-		return *op != -1;
-	}
-	if (!strncmp (str, "strh", 4)) {
-		*op = bytelsop (&ops, 0x00000078);
-		return *op != -1;
-	}
-	if (!strncmp (str, "ldr", 3)) {
+	} else if (!strncmp (str, "ldrb", 4)) {
+		*op = lsop (&ops, 0x00004038, -1);
+	} else if (!strncmp (str, "ldrh", 4)) {
+		*op = lsop (&ops, 0x00004078, -1);
+	} else if (!strncmp (str, "ldrsh", 5)) {
+		*op = lsop (&ops, 0x00008078, -1);
+	} else if (!strncmp (str, "ldrsw", 5)) {
+		*op = lsop (&ops, 0x00000098, addr);
+	} else if (!strncmp (str, "ldrsb", 5)) {
+		*op = lsop (&ops, 0x00008038, -1);
+	} else if (!strncmp (str, "strb", 4)) {
+		*op = lsop (&ops, 0x00000038, -1);
+	} else if (!strncmp (str, "strh", 4)) {
+		*op = lsop (&ops, 0x00000078, -1);
+	} else if (!strncmp (str, "ldr", 3)) {
 		*op = reglsop (&ops, 0x000040f8);
-		return *op != -1;
-	}
-	if (!strncmp (str, "stur", 4)) {
+	} else if (!strncmp (str, "stur", 4)) {
 		*op = regsluop (&ops, 0x000000f8);
-		return *op != -1;
-	}
-	if (!strncmp (str, "ldur", 4)) {
+	} else if (!strncmp (str, "ldur", 4)) {
 		*op = regsluop (&ops, 0x000040f8);
-		return *op != -1;
-	}
-	if (!strncmp (str, "str", 3)) {
+	} else if (!strncmp (str, "str", 3)) {
 		*op = reglsop (&ops, 0x000000f8);
-		return *op != -1;
-	}
-	if (!strncmp (str, "stp", 3)) {
+	} else if (!strncmp (str, "stp", 3)) {
 		*op = stp (&ops, 0x000000a9);
-		return *op != -1;
-	}
-	if (!strncmp (str, "ldp", 3)) {
+	} else if (!strncmp (str, "ldp", 3)) {
 		*op = stp (&ops, 0x000040a9);
-		return *op != -1;
-	}
-	if (!strncmp (str, "sub", 3)) { // w
+	} else if (!strncmp (str, "sub", 3)) { // w
 		*op = arithmetic (&ops, 0xd1);
-		return *op != -1;
-	}
-	if (!strncmp (str, "add x", 5)) {
+	} else if (!strncmp (str, "add x", 5)) {
 		*op = arithmetic (&ops, 0x91);
-		return *op != -1;
-	}
-	if (!strncmp (str, "add w", 5)) {
+	} else if (!strncmp (str, "add w", 5)) {
 		*op = arithmetic (&ops, 0x11);
-		return *op != -1;
-	}
-	if (!strncmp (str, "adr x", 5)) { // w
+	} else if (!strncmp (str, "adr x", 5)) { // w
 		*op = adr (&ops, addr);
-		return *op != -1;
-	}
-	if (!strncmp (str, "adrp x", 6)) {
+	} else if (!strncmp (str, "adrp x", 6)) {
 		*op = adrp (&ops, addr, 0x00000090);
-		return *op != -1;
-	}
-	if (!strncmp (str, "neg", 3)) {
+	} else if (!strncmp (str, "neg", 3)) {
 		*op = neg (&ops);
-		return *op != -1;
-	}
-	if (!strcmp (str, "isb")) {
+	} else if (!strcmp (str, "isb")) {
 		*op = 0xdf3f03d5;
-		return *op != -1;
-	}
-	// PAC
-	if (handlePAC (op, str)) {
+	} else if (handlePAC (op, str)) { // PAC
+		free (ops.mnemonic);
 		return true;
-	}
-	if (!strcmp (str, "nop")) {
+	} else if (!strcmp (str, "nop")) {
 		*op = 0x1f2003d5;
-		return *op != -1;
-	}
-	if (!strcmp (str, "ret")) {
+	} else if (!strcmp (str, "ret")) {
 		*op = 0xc0035fd6;
-		return true;
-	}
-	if (!strncmp (str, "msr ", 4)) {
+	} else if (!strncmp (str, "msr ", 4)) {
 		*op = msr (&ops, 0);
-		if (*op != UT32_MAX) {
-			return true;
-		}
-	}
-	if (!strncmp (str, "mrs ", 4)) {
+	} else if (!strncmp (str, "mrs ", 4)) {
 		*op = msr (&ops, 1);
-		if (*op != UT32_MAX) {
-			return true;
-		}
-	}
-	if (!strncmp (str, "orr ", 4)) {
-		*op = orr (&ops, addr);
-		return *op != UT32_MAX;
-	}
-	if (!strncmp (str, "svc ", 4)) { // system level exception
+	} else if (!strncmp (str, "ands ", 5)) {
+		*op = logical (&ops, false, ARM_ANDS);
+	} else if (!strncmp (str, "and ", 4)) {
+		*op = logical (&ops, false, ARM_AND);
+	} else if (!strncmp (str, "bics ", 5)) {
+		*op = logical (&ops, true, ARM_ANDS);
+	} else if (!strncmp (str, "bic ", 4)) {
+		*op = logical (&ops, true, ARM_AND);
+	} else if (!strncmp (str, "eon ", 4)) {
+		*op = logical (&ops, true, ARM_EOR);
+	} else if (!strncmp (str, "eor ", 4)) {
+		*op = logical (&ops, false, ARM_EOR);
+	} else if (!strncmp (str, "orn ", 4)) {
+		*op = logical (&ops, true, ARM_ORR);
+	} else if (!strncmp (str, "orr ", 4)) {
+		*op = logical (&ops, false, ARM_ORR);
+	} else if (!strncmp (str, "svc ", 4)) { // system level exception
 		*op = exception (&ops, 0x010000d4);
-		return *op != -1;
-	}
-	if (!strncmp (str, "hvc ", 4)) { // hypervisor level exception
+	} else if (!strncmp (str, "hvc ", 4)) { // hypervisor level exception
 		*op = exception (&ops, 0x020000d4);
-		return *op != -1;
-	}
-	if (!strncmp (str, "smc ", 4)) { // secure monitor exception
+	} else if (!strncmp (str, "smc ", 4)) { // secure monitor exception
 		*op = exception (&ops, 0x030000d4);
-		return *op != -1;
-	}
-	if (!strncmp (str, "brk ", 4)) { // breakpoint
+	} else if (!strncmp (str, "brk ", 4)) { // breakpoint
 		*op = exception (&ops, 0x000020d4);
-		return *op != -1;
-	}
-	if (!strncmp (str, "hlt ", 4)) { // halt
+	} else if (!strncmp (str, "hlt ", 4)) { // halt
 		*op = exception (&ops, 0x000040d4);
-		return *op != -1;
-	}
-	if (!strncmp (str, "b ", 2)) {
+	} else if (!strncmp (str, "b ", 2)) {
 		*op = branch (&ops, addr, 0x14);
-		return *op != -1;
-	}
-	if (!strncmp (str, "b.eq ", 5)) {
+	} else if (!strncmp (str, "b.eq ", 5) || !strncmp (str, "beq ", 4)) {
 		*op = bdot (&ops, addr, 0x00000054);
-		return *op != -1;
-	}
-	if (!strncmp (str, "b.hs ", 5)) {
+	} else if (!strncmp (str, "b.ne ", 5) || !strncmp (str, "bne ", 4)) {
+		*op = bdot (&ops, addr, 0x01000054);
+	} else if (!strncmp (str, "b.hs ", 5)) {
 		*op = bdot (&ops, addr, 0x02000054);
-		return *op != -1;
-	}
-	if (!strncmp (str, "bl ", 3)) {
+	} else if (!strncmp (str, "bl ", 3)) {
 		*op = branch (&ops, addr, 0x94);
-		return *op != -1;
-	}
-	if (!strncmp (str, "br x", 4)) {
+	} else if (!strncmp (str, "br x", 4)) {
 		*op = branch (&ops, addr, 0x1fd6);
-		return *op != -1;
-	}
-	if (!strncmp (str, "blr x", 5)) {
+	} else if (!strncmp (str, "blr x", 5)) {
 		*op = branch (&ops, addr, 0x3fd6);
-		return *op != -1;
-	}
-	if (!strncmp (str, "dmb ", 4)) {
+	} else if (!strncmp (str, "dmb ", 4)) {
 		*op = mem_barrier (&ops, addr, 0xbf3003d5);
-		return *op != -1;
-	}
-	if (!strncmp (str, "dsb ", 4)) {
+	} else if (!strncmp (str, "dsb ", 4)) {
 		*op = mem_barrier (&ops, addr, 0x9f3003d5);
-		return *op != -1;
-	}
-	if (!strncmp (str, "isb", 3)) {
+	} else if (!strncmp (str, "isb", 3)) {
 		*op = mem_barrier (&ops, addr, 0xdf3f03d5);
-		return *op != -1;
+	} else if (!strncmp (str, "sbfiz ", 6) || !strncmp (str, "sbfm ", 5)
+		|| !strncmp (str, "sbfx ", 5)) {
+		*op = bitfield (&ops, 0x00000013);
+	} else if (!strncmp (str, "ubfiz ", 6) || !strncmp (str, "ubfm ", 5)
+		|| !strncmp (str, "ubfx ", 5)) {
+		*op = bitfield (&ops, 0x00000053);
+	} else {
+		*op = UT32_MAX;
 	}
-	return false;
+	free (ops.mnemonic);
+	return *op != UT32_MAX;
 }
