@@ -258,7 +258,7 @@ static gb_opcode gb_op[] = {
 	{"sbc 0x%02x"		,GB_8BIT+ARG_8},
 	{"rst 24"		,GB_8BIT},
 
-	{"ld [0x%04x], a"	,GB_8BIT+ARG_8+GB_IO},	//0xe0
+	{"ld [%s], a"		,GB_8BIT+ARG_8+GB_IO},	//0xe0
 	{"pop hl"		,GB_8BIT},
 	{"ld [0xff00 + c], a"	,GB_8BIT},
 	{"invalid"		,GB_8BIT},
@@ -275,7 +275,7 @@ static gb_opcode gb_op[] = {
 	{"xor 0x%02x"		,GB_8BIT+ARG_8},
 	{"rst 40"		,GB_8BIT},
 
-	{"ld a, [0x%04x]"	,GB_8BIT+ARG_8+GB_IO},	//0xf0
+	{"ld a, [%s]"		,GB_8BIT+ARG_8+GB_IO},	//0xf0
 	{"pop af"		,GB_8BIT},
 	{"ld a, [0xff00 + c]"	,GB_8BIT},
 	{"di"			,GB_8BIT},
