@@ -951,143 +951,104 @@ char *get_reg_name_4(ut32 idx) {
 }
 
 char *get_opers(ut8 oper_byte) {
-	char *res = NULL;
-	ut8 oper_type = 0x00;
-	char *reg_name = NULL;
-
 	switch (oper_byte) {
 	case 0xE0u:
-		res = strdup ("overflow(ac0)");
-		break;
+		return strdup ("overflow(ac0)");
 	case 0xE1u:
-		res = strdup ("overflow(ac1)");
-		break;
+		return strdup ("overflow(ac1)");
 	case 0xE2u:
-		res = strdup ("overflow(ac2)");
-		break;
+		return strdup ("overflow(ac2)");
 	case 0xE3u:
-		res = strdup ("overflow(ac3)");
-		break;
+		return strdup ("overflow(ac3)");
 	case 0xE4u:
-		res = strdup ("tc1");
-		break;
+		return strdup ("tc1");
 	case 0xE5u:
-		res = strdup ("tc2");
-		break;
+		return strdup ("tc2");
 	case 0xE6u:
-		res = strdup ("carry");
-		break;
+		return strdup ("carry");
 	case 0xE7u:
-		res = strdup ("overflow(govf)");
-		break;
+		return strdup ("overflow(govf)");
 	case 0xE8u:
-		res = strdup ("tc1 & tc2");
-		break;
+		return strdup ("tc1 & tc2");
 	case 0xE9u:
-		res = strdup ("tc1 & !tc2");
-		break;
+		return strdup ("tc1 & !tc2");
 	case 0xEAu:
-		res = strdup ("!tc1 & tc2");
-		break;
+		return strdup ("!tc1 & tc2");
 	case 0xEBu:
-		res = strdup ("!tc1 & !tc2");
-		break;
+		return strdup ("!tc1 & !tc2");
 	case 0xECu:
-		res = strdup ("word_mode");
-		break;
+		return strdup ("word_mode");
 	case 0xEDu:
-		res = strdup ("byte_mode");
-		break;
+		return strdup ("byte_mode");
 	case 0xF0u:
-		res = strdup ("!overflow(ac0)");
-		break;
+		return strdup ("!overflow(ac0)");
 	case 0xF1u:
-		res = strdup ("!overflow(ac1)");
-		break;
+		return strdup ("!overflow(ac1)");
 	case 0xF2u:
-		res = strdup ("!overflow(ac2)");
-		break;
+		return strdup ("!overflow(ac2)");
 	case 0xF3u:
-		res = strdup ("!overflow(ac3)");
-		break;
+		return strdup ("!overflow(ac3)");
 	case 0xF4u:
-		res = strdup ("!tc1");
-		break;
+		return strdup ("!tc1");
 	case 0xF5u:
-		res = strdup ("!tc2");
-		break;
+		return strdup ("!tc2");
 	case 0xF6u:
-		res = strdup ("!carry");
-		break;
+		return strdup ("!carry");
 	case 0xF7u:
-		res = strdup ("!overflow(govf)");
-		break;
+		return strdup ("!overflow(govf)");
 	case 0xF8u:
-		res = strdup ("tc1 | tc2");
-		break;
+		return strdup ("tc1 | tc2");
 	case 0xF9u:
-		res = strdup ("tc1 | !tc2");
-		break;
+		return strdup ("tc1 | !tc2");
 	case 0xFAu:
-		res = strdup ("!tc1 | tc2");
-		break;
+		return strdup ("!tc1 | tc2");
 	case 0xFBu:
-		res = strdup ("!tc1 | !tc2");
-		break;
+		return strdup ("!tc1 | !tc2");
 	case 0xFCu:
-		res = strdup ("tc1 ^ tc2");
-		break;
+		return strdup ("tc1 ^ tc2");
 	case 0xFDu:
-		res = strdup ("tc1 ^ !tc2");
-		break;
+		return strdup ("tc1 ^ !tc2");
 	case 0xFEu:
-		res = strdup ("!tc1 ^ tc2");
-		break;
+		return strdup ("!tc1 ^ tc2");
 	case 0xFFu:
-		res = strdup("!tc1 ^ !tc2");
-		break;
-	default:
-		oper_type = oper_byte >> 5;
-		if (oper_type != 6 ) {
-			reg_name = get_reg_name_4 (oper_byte & 0x1F);
+		return strdup("!tc1 ^ !tc2");
+	default: {
+		ut8 oper_type = oper_byte >> 5;
+		if (oper_type != 6) {
+			char *reg_name = get_reg_name_4 (oper_byte & 0x1F);
 			switch (oper_type) {
 			case 1u:
-				res = strcat_dup (reg_name, " != #0", 1);
-				break;
+				return strcat_dup (reg_name, " != #0", 1);
 			case 0u:
-				res = strcat_dup (reg_name, " == #0", 1);
-				break;
+				return strcat_dup (reg_name, " == #0", 1);
 			case 2u:
-				res =  strcat_dup (reg_name, " < #0", 1);
-				break;
+				return strcat_dup (reg_name, " < #0", 1);
 			case 3u:
-				res =  strcat_dup (reg_name, " >= #0", 1);
-				break;
+				return strcat_dup (reg_name, " >= #0", 1);
 			case 4u:
-				res =  strcat_dup (reg_name, " > #0", 1);
-				break;
+				return strcat_dup (reg_name, " > #0", 1);
 			case 5u:
-				res =  strcat_dup (reg_name, " <= #0", 1);
+				return strcat_dup (reg_name, " <= #0", 1);
+			default:
+				free (reg_name);
+				return NULL;
 			}
-			//TODO: still can leak
-			return res;
 		}
-		reg_name = get_reg_name_1 ((oper_byte & 0xF) + 128);
+		char *reg_name = get_reg_name_1 ((oper_byte & 0xF) + 128);
 		oper_type = (oper_byte >> 4) - 12;
 		if (oper_type) {
 			if (oper_type != 1) {
 				free (reg_name);
 				return NULL;
 			}
-			res = strcat_dup (reg_name, " != #0", 1);
+			return strcat_dup (reg_name, " != #0", 1);
 		} else {
 			// coverity may complain but strcat_dup set to null
 			// reg_name when free
-			res = strcat_dup (reg_name, " == #0", 1);
+			return strcat_dup (reg_name, " == #0", 1);
 		}
 	}
-	free (reg_name);
-	return res;
+	}
 }
 
 char *get_cmp_op(ut32 idx) {
