@@ -1,7 +1,7 @@
 /* c55plus - LGPL - Copyright 2013 - th0rpe */
 
 #include "ins.h"
-#include "hashvector.c"
+#include "hashvector.h"
 #include <r_types.h>
 
 static const st32 hash_const_01 = 0x2474f685;
@@ -2281,6 +2281,521 @@ st32 get_hashfunc_91(st32 arg1, st32 arg2) {
 	return result;
 }
 
+static HASHCODE_ENTRY_T ins_hash[] = {
+	{ 0x223, get_hashfunc_02 },
+	{ 0x223, get_hashfunc_03 },
+	{ 0x223, get_hashfunc_04 },
+	{ 0x223, get_hashfunc_05 },
+	{ 0xDF, get_hashfunc_01 },
+	{ 0xE0, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_06 },
+	{ 0x223, get_hashfunc_07 },
+	{ 0xEE, get_hashfunc_01 },
+	{ 0xEE, get_hashfunc_01 },
+	{ 0xEF, get_hashfunc_01 },
+	{ 0xEF, get_hashfunc_01 },
+	{ 0xF0, get_hashfunc_01 },
+	{ 0xF0, get_hashfunc_01 },
+	{ 0xF1, get_hashfunc_01 },
+	{ 0xF1, get_hashfunc_01 },
+	{ 0x1, get_hashfunc_01 },
+	{ 0x1, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_08 },
+	{ 0x223, get_hashfunc_09 },
+	{ 0x1E7, get_hashfunc_01 },
+	{ 0x1E7, get_hashfunc_01 },
+	{ 0x6B, get_hashfunc_01 },
+	{ 0x6B, get_hashfunc_01 },
+	{ 0x71, get_hashfunc_01 },
+	{ 0x70, get_hashfunc_01 },
+	{ 0x72, get_hashfunc_01 },
+	{ 0x72, get_hashfunc_01 },
+	{ 0x6E, get_hashfunc_01 },
+	{ 0x6E, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x3A, get_hashfunc_01 },
+	{ 0x3A, get_hashfunc_01 },
+	{ 0x65, get_hashfunc_01 },
+	{ 0x65, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0xE7, get_hashfunc_01 },
+	{ 0xE7, get_hashfunc_01 },
+	{ 0xE8, get_hashfunc_01 },
+	{ 0xE8, get_hashfunc_01 },
+	{ 0x1A8, get_hashfunc_01 },
+	{ 0x1A8, get_hashfunc_01 },
+	{ 0xEC, get_hashfunc_01 },
+	{ 0xEC, get_hashfunc_01 },
+	{ 0xED, get_hashfunc_01 },
+	{ 0xED, get_hashfunc_01 },
+	{ 0xE9, get_hashfunc_01 },
+	{ 0xE9, get_hashfunc_01 },
+	{ 0xEA, get_hashfunc_01 },
+	{ 0xEA, get_hashfunc_01 },
+	{ 0x1E3, get_hashfunc_01 },
+	{ 0x1E3, get_hashfunc_01 },
+	{ 0x1E4, get_hashfunc_01 },
+	{ 0x1E4, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x1DC, get_hashfunc_01 },
+	{ 0x1DC, get_hashfunc_01 },
+	{ 0x1E1, get_hashfunc_01 },
+	{ 0x1E1, get_hashfunc_01 },
+	{ 0x1E2, get_hashfunc_01 },
+	{ 0x1E2, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x160, get_hashfunc_01 },
+	{ 0x160, get_hashfunc_01 },
+	{ 0x160, get_hashfunc_01 },
+	{ 0x160, get_hashfunc_01 },
+	{ 0x160, get_hashfunc_01 },
+	{ 0x160, get_hashfunc_01 },
+	{ 0x160, get_hashfunc_01 },
+	{ 0x160, get_hashfunc_01 },
+	{ 0x1DF, get_hashfunc_01 },
+	{ 0x1DF, get_hashfunc_01 },
+	{ 0x1DF, get_hashfunc_01 },
+	{ 0x1DF, get_hashfunc_01 },
+	{ 0x1DF, get_hashfunc_01 },
+	{ 0x1DF, get_hashfunc_01 },
+	{ 0x1DF, get_hashfunc_01 },
+	{ 0x1DF, get_hashfunc_01 },
+	{ 0x1DD, get_hashfunc_01 },
+	{ 0x1DD, get_hashfunc_01 },
+	{ 0x1DD, get_hashfunc_01 },
+	{ 0x1DD, get_hashfunc_01 },
+	{ 0x1DD, get_hashfunc_01 },
+	{ 0x1DD, get_hashfunc_01 },
+	{ 0x1DD, get_hashfunc_01 },
+	{ 0x1DD, get_hashfunc_01 },
+	{ 0x1D0, get_hashfunc_01 },
+	{ 0x1D0, get_hashfunc_01 },
+	{ 0x1D0, get_hashfunc_01 },
+	{ 0x1D0, get_hashfunc_01 },
+	{ 0x1D0, get_hashfunc_01 },
+	{ 0x1D0, get_hashfunc_01 },
+	{ 0x1D0, get_hashfunc_01 },
+	{ 0x1D0, get_hashfunc_01 },
+	{ 0x135, get_hashfunc_01 },
+	{ 0x135, get_hashfunc_01 },
+	{ 0xF3, get_hashfunc_01 },
+	{ 0xF3, get_hashfunc_01 },
+	{ 0x1CE, get_hashfunc_01 },
+	{ 0x1CE, get_hashfunc_01 },
+	{ 0x1CF, get_hashfunc_01 },
+	{ 0x1CF, get_hashfunc_01 },
+	{ 0x17B, get_hashfunc_01 },
+	{ 0x17B, get_hashfunc_01 },
+	{ 0x16E, get_hashfunc_01 },
+	{ 0x16E, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0xF6, get_hashfunc_01 },
+	{ 0xF6, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_10 },
+	{ 0x223, get_hashfunc_10 },
+	{ 0x223, get_hashfunc_11 },
+	{ 0x223, get_hashfunc_11 },
+	{ 0x223, get_hashfunc_12 },
+	{ 0x223, get_hashfunc_12 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x3, get_hashfunc_01 },
+	{ 0x3, get_hashfunc_01 },
+	{ 0x4, get_hashfunc_01 },
+	{ 0x4, get_hashfunc_01 },
+	{ 0x2, get_hashfunc_01 },
+	{ 0x2, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x6, get_hashfunc_01 },
+	{ 0x6, get_hashfunc_01 },
+	{ 0x0, get_hashfunc_01 },
+	{ 0x0, get_hashfunc_01 },
+	{ 0x69, get_hashfunc_01 },
+	{ 0x69, get_hashfunc_01 },
+	{ 0x7, get_hashfunc_01 },
+	{ 0x7, get_hashfunc_01 },
+	{ 0x46, get_hashfunc_01 },
+	{ 0x46, get_hashfunc_01 },
+	{ 0x47, get_hashfunc_01 },
+	{ 0x47, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_13 },
+	{ 0x1F, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_14 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_15 },
+	{ 0x223, get_hashfunc_16 },
+	{ 0x223, get_hashfunc_17 },
+	{ 0x223, get_hashfunc_18 },
+	{ 0x223, get_hashfunc_19 },
+	{ 0x19D, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x7B, get_hashfunc_01 },
+	{ 0x80, get_hashfunc_01 },
+	{ 0x81, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_20 },
+	{ 0x223, get_hashfunc_21 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x109, get_hashfunc_01 },
+	{ 0x109, get_hashfunc_01 },
+	{ 0x10A, get_hashfunc_01 },
+	{ 0x10A, get_hashfunc_01 },
+	{ 0x10B, get_hashfunc_01 },
+	{ 0x10B, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x10C, get_hashfunc_01 },
+	{ 0x10C, get_hashfunc_01 },
+	{ 0x10D, get_hashfunc_01 },
+	{ 0x10D, get_hashfunc_01 },
+	{ 0x10E, get_hashfunc_01 },
+	{ 0x10E, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_22 },
+	{ 0x223, get_hashfunc_22 },
+	{ 0x223, get_hashfunc_23 },
+	{ 0x223, get_hashfunc_23 },
+	{ 0x223, get_hashfunc_24 },
+	{ 0x223, get_hashfunc_24 },
+	{ 0x223, get_hashfunc_25 },
+	{ 0x223, get_hashfunc_25 },
+	{ 0x223, get_hashfunc_26 },
+	{ 0x223, get_hashfunc_26 },
+	{ 0x223, get_hashfunc_27 },
+	{ 0x223, get_hashfunc_27 },
+	{ 0x223, get_hashfunc_28 },
+	{ 0x223, get_hashfunc_28 },
+	{ 0x223, get_hashfunc_29 },
+	{ 0x223, get_hashfunc_29 },
+	{ 0x223, get_hashfunc_30 },
+	{ 0x223, get_hashfunc_30 },
+	{ 0x223, get_hashfunc_31 },
+	{ 0x223, get_hashfunc_31 },
+	{ 0x223, get_hashfunc_32 },
+	{ 0x223, get_hashfunc_32 },
+	{ 0x223, get_hashfunc_32 },
+	{ 0x223, get_hashfunc_32 },
+	{ 0x223, get_hashfunc_33 },
+	{ 0x223, get_hashfunc_33 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_34 },
+	{ 0x223, get_hashfunc_34 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x94, get_hashfunc_01 },
+	{ 0x94, get_hashfunc_01 },
+	{ 0x95, get_hashfunc_01 },
+	{ 0x95, get_hashfunc_01 },
+	{ 0x92, get_hashfunc_01 },
+	{ 0x92, get_hashfunc_01 },
+	{ 0x93, get_hashfunc_01 },
+	{ 0x93, get_hashfunc_01 },
+	{ 0x92, get_hashfunc_01 },
+	{ 0x92, get_hashfunc_01 },
+	{ 0x93, get_hashfunc_01 },
+	{ 0x93, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_35 },
+	{ 0x223, get_hashfunc_35 },
+	{ 0x223, get_hashfunc_36 },
+	{ 0x223, get_hashfunc_37 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_38 },
+	{ 0x223, get_hashfunc_39 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_40 },
+	{ 0x223, get_hashfunc_41 },
+	{ 0x223, get_hashfunc_42 },
+	{ 0x223, get_hashfunc_43 },
+	{ 0x223, get_hashfunc_44 },
+	{ 0x223, get_hashfunc_44 },
+	{ 0x223, get_hashfunc_45 },
+	{ 0x223, get_hashfunc_46 },
+	{ 0x223, get_hashfunc_47 },
+	{ 0x223, get_hashfunc_48 },
+	{ 0x223, get_hashfunc_47 },
+	{ 0x223, get_hashfunc_48 },
+	{ 0x9F, get_hashfunc_01 },
+	{ 0x9F, get_hashfunc_01 },
+	{ 0x1D1, get_hashfunc_01 },
+	{ 0x1D1, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_49 },
+	{ 0xA0, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x19C, get_hashfunc_01 },
+	{ 0x19C, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_50 },
+	{ 0x223, get_hashfunc_50 },
+	{ 0x223, get_hashfunc_51 },
+	{ 0x223, get_hashfunc_51 },
+	{ 0x223, get_hashfunc_52 },
+	{ 0x223, get_hashfunc_52 },
+	{ 0x223, get_hashfunc_53 },
+	{ 0x223, get_hashfunc_53 },
+	{ 0x223, get_hashfunc_54 },
+	{ 0x223, get_hashfunc_54 },
+	{ 0x223, get_hashfunc_55 },
+	{ 0x223, get_hashfunc_55 },
+	{ 0x223, get_hashfunc_56 },
+	{ 0x223, get_hashfunc_56 },
+	{ 0x223, get_hashfunc_57 },
+	{ 0x223, get_hashfunc_57 },
+	{ 0x223, get_hashfunc_57 },
+	{ 0x223, get_hashfunc_57 },
+	{ 0x223, get_hashfunc_58 },
+	{ 0x223, get_hashfunc_58 },
+	{ 0x223, get_hashfunc_58 },
+	{ 0x223, get_hashfunc_58 },
+	{ 0x223, get_hashfunc_59 },
+	{ 0x223, get_hashfunc_59 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_60 },
+	{ 0xB1, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_61 },
+	{ 0xB0, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_62 },
+	{ 0x9C, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_63 },
+	{ 0x9B, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_64 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_65 },
+	{ 0xB7, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_66 },
+	{ 0x223, get_hashfunc_66 },
+	{ 0x223, get_hashfunc_67 },
+	{ 0x223, get_hashfunc_67 },
+	{ 0x223, get_hashfunc_68 },
+	{ 0x223, get_hashfunc_69 },
+	{ 0x223, get_hashfunc_68 },
+	{ 0x223, get_hashfunc_69 },
+	{ 0x223, get_hashfunc_70 },
+	{ 0x223, get_hashfunc_71 },
+	{ 0x223, get_hashfunc_70 },
+	{ 0x223, get_hashfunc_71 },
+	{ 0xDB, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_72 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_73 },
+	{ 0x223, get_hashfunc_74 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x1DE, get_hashfunc_01 },
+	{ 0x1DE, get_hashfunc_01 },
+	{ 0x1D2, get_hashfunc_01 },
+	{ 0x1D2, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_75 },
+	{ 0x1CA, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_76 },
+	{ 0x223, get_hashfunc_77 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x90, get_hashfunc_01 },
+	{ 0x90, get_hashfunc_01 },
+	{ 0x91, get_hashfunc_01 },
+	{ 0x91, get_hashfunc_01 },
+	{ 0x1CD, get_hashfunc_01 },
+	{ 0x1CD, get_hashfunc_01 },
+	{ 0x1CD, get_hashfunc_01 },
+	{ 0x1CD, get_hashfunc_01 },
+	{ 0x96, get_hashfunc_01 },
+	{ 0x96, get_hashfunc_01 },
+	{ 0x96, get_hashfunc_01 },
+	{ 0x96, get_hashfunc_01 },
+	{ 0x96, get_hashfunc_01 },
+	{ 0x96, get_hashfunc_01 },
+	{ 0x96, get_hashfunc_01 },
+	{ 0x96, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_78 },
+	{ 0x223, get_hashfunc_78 },
+	{ 0x223, get_hashfunc_78 },
+	{ 0x223, get_hashfunc_78 },
+	{ 0x223, get_hashfunc_79 },
+	{ 0x223, get_hashfunc_79 },
+	{ 0x223, get_hashfunc_80 },
+	{ 0x223, get_hashfunc_80 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_81 },
+	{ 0x223, get_hashfunc_82 },
+	{ 0x223, get_hashfunc_81 },
+	{ 0x223, get_hashfunc_82 },
+	{ 0x223, get_hashfunc_83 },
+	{ 0xCC, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_83 },
+	{ 0xCC, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_84 },
+	{ 0x223, get_hashfunc_85 },
+	{ 0x223, get_hashfunc_84 },
+	{ 0x223, get_hashfunc_85 },
+	{ 0x223, get_hashfunc_86 },
+	{ 0x223, get_hashfunc_86 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_87 },
+	{ 0x223, get_hashfunc_87 },
+	{ 0x223, get_hashfunc_88 },
+	{ 0x223, get_hashfunc_88 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_89 },
+	{ 0x223, get_hashfunc_90 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_91 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 },
+	{ 0x223, get_hashfunc_01 }
+};
+
 //get hashcode from instruction bytecode
 st32 get_hash_code(ut32 ins_pos) {
 	ut32 len, ins_part1;
