@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2017-2019 - pancake, cgvwzq */
+/* radare - LGPL - Copyright 2017-2020 - pancake, cgvwzq */
 
 // http://webassembly.org/docs/binary-encoding/#module-structure
 
@@ -252,13 +252,190 @@ static WasmOpDef opcodes_threads[256] = {
 	[WASM_OP_I64ATOMICRMW32UCMPXCHG] = { "i64.atomic.rmw32_u.cmpxchg" , 3, 3 }
 };
 
+static WasmOpDef opcodes_simd[256] = {
+	[WASM_OP_V128LOAD] = { "v128.load", 6, 6 },
+	[WASM_OP_V128STORE] = { "v128.store", 2, 6 },
+	[WASM_OP_V128CONST] = { "v128.const", 2, 6 },
+	[WASM_OP_I8X16SPLAT] = { "i8x16.splat", 2, 6 },
+	[WASM_OP_I8X16EXTRACTLANES] = { "i8x16.extract_lane_s", 2, 6 },
+	[WASM_OP_I8X16EXTRACTLANEU] = { "i8x16.extract_lane_u", 2, 6 },
+	[WASM_OP_I8X16REPLACELANE] = { "i8x16.replace_lane", 2, 6 },
+	[WASM_OP_I16X8SPLAT] = { "i16x8.splat", 2, 6 },
+	[WASM_OP_I16X8EXTRACTLANES] = { "i16x8.extract_lane_s", 2, 6 },
+	[WASM_OP_I16X8EXTRACTLANEU] = { "i16x8.extract_lane_u", 2, 6 },
+	[WASM_OP_I16X8REPLACELANE] = { "i16x8.replace_lane", 2, 6 },
+	[WASM_OP_I32X4SPLAT] = { "i32x4.splat", 2, 6 },
+	[WASM_OP_I32X4EXTRACTLANE] = { "i32x4.extract_lane", 2, 6 },
+	[WASM_OP_I32X4REPLACELANE] = { "i32x4.replace_lane", 2, 6 },
+	[WASM_OP_I64X2SPLAT] = { "i64x2.splat", 2, 6 },
+	[WASM_OP_I64X2EXTRACTLANE] = { "i64x2.extract_lane", 2, 6 },
+	[WASM_OP_I64X2REPLACELANE] = { "i64x2.replace_lane", 2, 6 },
+	[WASM_OP_F32X4SPLAT] = { "f32x4.splat", 2, 6 },
+	[WASM_OP_F32X4EXTRACTLANE] = { "f32x4.extract_lane", 2, 6 },
+	[WASM_OP_F32X4REPLACELANE] = { "f32x4.replace_lane", 2, 6 },
+	[WASM_OP_F64X2SPLAT] = { "f64x2.splat", 2, 6 },
+	[WASM_OP_F64X2EXTRACTLANE] = { "f64x2.extract_lane", 2, 6 },
+	[WASM_OP_F64X2REPLACELANE] = { "f64x2.replace_lane", 2, 6 },
+	[WASM_OP_I8X16EQ] = { "i8x16.eq", 2, 6 },
+	[WASM_OP_I8X16NE] = { "i8x16.ne", 2, 6 },
+	[WASM_OP_I8X16LTS] = { "i8x16.lt_s", 2, 6 },
+	[WASM_OP_I8X16LTU] = { "i8x16.lt_u", 2, 6 },
+	[WASM_OP_I8X16GTS] = { "i8x16.gt_s", 2, 6 },
+	[WASM_OP_I8X16GTU] = { "i8x16.gt_u", 2, 6 },
+	[WASM_OP_I8X16LES] = { "i8x16.le_s", 2, 6 },
+	[WASM_OP_I8X16LEU] = { "i8x16.le_u", 2, 6 },
+	[WASM_OP_I8X16GES] = { "i8x16.ge_s", 2, 6 },
+	[WASM_OP_I8X16GEU] = { "i8x16.ge_u", 2, 6 },
+	[WASM_OP_I16X8EQ] = { "i16x8.eq", 2, 6 },
+	[WASM_OP_I16X8NE] = { "i16x8.ne", 2, 6 },
+	[WASM_OP_I16X8LTS] = { "i16x8.lt_s", 2, 6 },
+	[WASM_OP_I16X8LTU] = { "i16x8.lt_u", 2, 6 },
+	[WASM_OP_I16X8GTS] = { "i16x8.gt_s", 2, 6 },
+	[WASM_OP_I16X8GTU] = { "i16x8.gt_u", 2, 6 },
+	[WASM_OP_I16X8LES] = { "i16x8.le_s", 2, 6 },
+	[WASM_OP_I16X8LEU] = { "i16x8.le_u", 2, 6 },
+	[WASM_OP_I16X8GES] = { "i16x8.ge_s", 2, 6 },
+	[WASM_OP_I16X8GEU] = { "i16x8.ge_u", 2, 6 },
+	[WASM_OP_I32X4EQ] = { "i32x4.eq", 2, 6 },
+	[WASM_OP_I32X4NE] = { "i32x4.ne", 2, 6 },
+	[WASM_OP_I32X4LTS] = { "i32x4.lt_s", 2, 6 },
+	[WASM_OP_I32X4LTU] = { "i32x4.lt_u", 2, 6 },
+	[WASM_OP_I32X4GTS] = { "i32x4.gt_s", 2, 6 },
+	[WASM_OP_I32X4GTU] = { "i32x4.gt_u", 2, 6 },
+	[WASM_OP_I32X4LES] = { "i32x4.le_s", 2, 6 },
+	[WASM_OP_I32X4LEU] = { "i32x4.le_u", 2, 6 },
+	[WASM_OP_I32X4GES] = { "i32x4.ge_s", 2, 6 },
+	[WASM_OP_I32X4GEU] = { "i32x4.ge_u", 2, 6 },
+	[WASM_OP_F32X4EQ] = { "f32x4.eq", 2, 6 },
+	[WASM_OP_F32X4NE] = { "f32x4.ne", 2, 6 },
+	[WASM_OP_F32X4LT] = { "f32x4.lt", 2, 6 },
+	[WASM_OP_F32X4GT] = { "f32x4.gt", 2, 6 },
+	[WASM_OP_F32X4LE] = { "f32x4.le", 2, 6 },
+	[WASM_OP_F32X4GE] = { "f32x4.ge", 2, 6 },
+	[WASM_OP_F64X2EQ] = { "f64x2.eq", 2, 6 },
+	[WASM_OP_F64X2NE] = { "f64x2.ne", 2, 6 },
+	[WASM_OP_F64X2LT] = { "f64x2.lt", 2, 6 },
+	[WASM_OP_F64X2GT] = { "f64x2.gt", 2, 6 },
+	[WASM_OP_F64X2LE] = { "f64x2.le", 2, 6 },
+	[WASM_OP_F64X2GE] = { "f64x2.ge", 2, 6 },
+	[WASM_OP_V128NOT] = { "v128.not", 2, 6 },
+	[WASM_OP_V128AND] = { "v128.and", 2, 6 },
+	[WASM_OP_V128OR] = { "v128.or", 2, 6 },
+	[WASM_OP_V128XOR] = { "v128.xor", 2, 6 },
+	[WASM_OP_V128BITSELECT] = { "v128.bitselect", 2, 6 },
+	[WASM_OP_I8X16NEG] = { "i8x16.neg", 2, 6 },
+	[WASM_OP_I8X16ANYTRUE] = { "i8x16.any_true", 2, 6 },
+	[WASM_OP_I8X16ALLTRUE] = { "i8x16.all_true", 2, 6 },
+	[WASM_OP_I8X16SHL] = { "i8x16.shl", 2, 6 },
+	[WASM_OP_I8X16SHRS] = { "i8x16.shr_s", 2, 6 },
+	[WASM_OP_I8X16SHRU] = { "i8x16.shr_u", 2, 6 },
+	[WASM_OP_I8X16ADD] = { "i8x16.add", 2, 6 },
+	[WASM_OP_I8X16ADDSATURATES] = { "i8x16.add_saturate_s", 2, 6 },
+	[WASM_OP_I8X16ADDSATURATEU] = { "i8x16.add_saturate_u", 2, 6 },
+	[WASM_OP_I8X16SUB] = { "i8x16.sub", 2, 6 },
+	[WASM_OP_I8X16SUBSATURATES] = { "i8x16.sub_saturate_s", 2, 6 },
+	[WASM_OP_I8X16SUBSATURATEU] = { "i8x16.sub_saturate_u", 2, 6 },
+	[WASM_OP_I8X16MINS] = { "i8x16.min_s", 2, 6 },
+	[WASM_OP_I8X16MINU] = { "i8x16.min_u", 2, 6 },
+	[WASM_OP_I8X16MAXS] = { "i8x16.max_s", 2, 6 },
+	[WASM_OP_I8X16MAXU] = { "i8x16.max_u", 2, 6 },
+	[WASM_OP_I16X8NEG] = { "i16x8.neg", 2, 6 },
+	[WASM_OP_I16X8ANYTRUE] = { "i16x8.any_true", 2, 6 },
+	[WASM_OP_I16X8ALLTRUE] = { "i16x8.all_true", 2, 6 },
+	[WASM_OP_I16X8SHL] = { "i16x8.shl", 2, 6 },
+	[WASM_OP_I16X8SHRS] = { "i16x8.shr_s", 2, 6 },
+	[WASM_OP_I16X8SHRU] = { "i16x8.shr_u", 2, 6 },
+	[WASM_OP_I16X8ADD] = { "i16x8.add", 2, 6 },
+	[WASM_OP_I16X8ADDSATURATES] = { "i16x8.add_saturate_s", 2, 6 },
+	[WASM_OP_I16X8ADDSATURATEU] = { "i16x8.add_saturate_u", 2, 6 },
+	[WASM_OP_I16X8SUB] = { "i16x8.sub", 2, 6 },
+	[WASM_OP_I16X8SUBSATURATES] = { "i16x8.sub_saturate_s", 2, 6 },
+	[WASM_OP_I16X8SUBSATURATEU] = { "i16x8.sub_saturate_u", 2, 6 },
+	[WASM_OP_I16X8MUL] = { "i16x8.mul", 2, 6 },
+	[WASM_OP_I16X8MINS] = { "i16x8.min_s", 2, 6 },
+	[WASM_OP_I16X8MINU] = { "i16x8.min_u", 2, 6 },
+	[WASM_OP_I16X8MAXS] = { "i16x8.max_s", 2, 6 },
+	[WASM_OP_I16X8MAXU] = { "i16x8.max_u", 2, 6 },
+	[WASM_OP_I32X4NEG] = { "i32x4.neg", 2, 6 },
+	[WASM_OP_I32X4ANYTRUE] = { "i32x4.any_true", 2, 6 },
+	[WASM_OP_I32X4ALLTRUE] = { "i32x4.all_true", 2, 6 },
+	[WASM_OP_I32X4SHL] = { "i32x4.shl", 2, 6 },
+	[WASM_OP_I32X4SHRS] = { "i32x4.shr_s", 2, 6 },
+	[WASM_OP_I32X4SHRU] = { "i32x4.shr_u", 2, 6 },
+	[WASM_OP_I32X4ADD] = { "i32x4.add", 2, 6 },
+	[WASM_OP_I32X4SUB] = { "i32x4.sub", 2, 6 },
+	[WASM_OP_I32X4MUL] = { "i32x4.mul", 2, 6 },
+	[WASM_OP_I32X4MINS] = { "i32x4.min_s", 2, 6 },
+	[WASM_OP_I32X4MINU] = { "i32x4.min_u", 2, 6 },
+	[WASM_OP_I32X4MAXS] = { "i32x4.max_s", 2, 6 },
+	[WASM_OP_I32X4MAXU] = { "i32x4.max_u", 2, 6 },
+	[WASM_OP_I64X2NEG] = { "i64x2.neg", 2, 6 },
+	[WASM_OP_I64X2SHL] = { "i64x2.shl", 2, 6 },
+	[WASM_OP_I64X2SHRS] = { "i64x2.shr_s", 2, 6 },
+	[WASM_OP_I64X2SHRU] = { "i64x2.shr_u", 2, 6 },
+	[WASM_OP_I64X2ADD] = { "i64x2.add", 2, 6 },
+	[WASM_OP_I64X2SUB] = { "i64x2.sub", 2, 6 },
+	[WASM_OP_I64X2MUL] = { "i64x2.mul", 2, 6 },
+	[WASM_OP_F32X4ABS] = { "f32x4.abs", 2, 6 },
+	[WASM_OP_F32X4NEG] = { "f32x4.neg", 2, 6 },
+	[WASM_OP_F32X4SQRT] = { "f32x4.sqrt", 2, 6 },
+	[WASM_OP_F32X4ADD] = { "f32x4.add", 2, 6 },
+	[WASM_OP_F32X4SUB] = { "f32x4.sub", 2, 6 },
+	[WASM_OP_F32X4MUL] = { "f32x4.mul", 2, 6 },
+	[WASM_OP_F32X4DIV] = { "f32x4.div", 2, 6 },
+	[WASM_OP_F32X4MIN] = { "f32x4.min", 2, 6 },
+	[WASM_OP_F32X4MAX] = { "f32x4.max", 2, 6 },
+	[WASM_OP_F64X2ABS] = { "f64x2.abs", 2, 6 },
+	[WASM_OP_F64X2NEG] = { "f64x2.neg", 2, 6 },
+	[WASM_OP_F64X2SQRT] = { "f64x2.sqrt", 2, 6 },
+	[WASM_OP_F64X2ADD] = { "f64x2.add", 2, 6 },
+	[WASM_OP_F64X2SUB] = { "f64x2.sub", 2, 6 },
+	[WASM_OP_F64X2MUL] = { "f64x2.mul", 2, 6 },
+	[WASM_OP_F64X2DIV] = { "f64x2.div", 2, 6 },
+	[WASM_OP_F64X2MIN] = { "f64x2.min", 2, 6 },
+	[WASM_OP_F64X2MAX] = { "f64x2.max", 2, 6 },
+	[WASM_OP_I32X4TRUNCSATF32X4S] = { "i32x4.trunc_sat_f32x4_s", 2, 6 },
+	[WASM_OP_I32X4TRUNCSATF32X4U] = { "i32x4.trunc_sat_f32x4_u", 2, 6 },
+	[WASM_OP_F32X4CONVERTI32X4S] = { "f32x4.convert_i32x4_s", 2, 6 },
+	[WASM_OP_F32X4CONVERTI32X4U] = { "f32x4.convert_i32x4_u", 2, 6 },
+	[WASM_OP_V8X16SWIZZLE] = { "v8x16.swizzle", 2, 6 },
+	[WASM_OP_V8X16SHUFFLE] = { "v8x16.shuffle", 2, 6 },
+	[WASM_OP_V8X16LOADSPLAT] = { "v8x16.load_splat", 2, 6 },
+	[WASM_OP_V16X8LOADSPLAT] = { "v16x8.load_splat", 2, 6 },
+	[WASM_OP_V32X4LOADSPLAT] = { "v32x4.load_splat", 2, 6 },
+	[WASM_OP_V64X2LOADSPLAT] = { "v64x2.load_splat", 2, 6 },
+	[WASM_OP_I8X16NARROWI16X8S] = { "i8x16.narrow_i16x8_s", 2, 6 },
+	[WASM_OP_I8X16NARROWI16X8U] = { "i8x16.narrow_i16x8_u", 2, 6 },
+	[WASM_OP_I16X8NARROWI32X4S] = { "i16x8.narrow_i32x4_s", 2, 6 },
+	[WASM_OP_I16X8NARROWI32X4U] = { "i16x8.narrow_i32x4_u", 2, 6 },
+	[WASM_OP_I16X8WIDENLOWI8X16S] = { "i16x8.widen_low_i8x16_s", 2, 6 },
+	[WASM_OP_I16X8WIDENHIGHI8X16S] = { "i16x8.widen_high_i8x16_s", 2, 6 },
+	[WASM_OP_I16X8WIDENLOWI8X16U] = { "i16x8.widen_low_i8x16_u", 2, 6 },
+	[WASM_OP_I16X8WIDENHIGHI8X16U] = { "i16x8.widen_high_i8x16_u", 2, 6 },
+	[WASM_OP_I32X4WIDENLOWI16X8S] = { "i32x4.widen_low_i16x8_s", 2, 6 },
+	[WASM_OP_I32X4WIDENHIGHI16X8S] = { "i32x4.widen_high_i16x8_s", 2, 6 },
+	[WASM_OP_I32X4WIDENLOWI16X8U] = { "i32x4.widen_low_i16x8_u", 2, 6 },
+	[WASM_OP_I32X4WIDENHIGHI16X8U] = { "i32x4.widen_high_i16x8_u", 2, 6 },
+	[WASM_OP_I16X8LOAD8X8S] = { "i16x8.load8x8_s", 2, 6 },
+	[WASM_OP_I16X8LOAD8X8U] = { "i16x8.load8x8_u", 2, 6 },
+	[WASM_OP_I32X4LOAD16X4S] = { "i32x4.load16x4_s", 2, 6 },
+	[WASM_OP_I32X4LOAD16X4U] = { "i32x4.load16x4_u", 2, 6 },
+	[WASM_OP_I64X2LOAD32X2S] = { "i64x2.load32x2_s", 2, 6 },
+	[WASM_OP_I64X2LOAD32X2U] = { "i64x2.load32x2_u", 2, 6 },
+	[WASM_OP_V128ANDNOT] = { "v128.andnot", 2, 6 },
+	[WASM_OP_I8X16AVGRU] = { "i8x16.avgr_u", 2, 6 },
+	[WASM_OP_I16X8AVGRU] = { "i16x8.avgr_u", 2, 6 },
+	[WASM_OP_I8X16ABS] = { "i8x16.abs", 2, 6 },
+	[WASM_OP_I16X8ABS] = { "i16x8.abs", 2, 6 },
+};
+
 #ifndef WASM_NO_ASM
 // assembles the given line of wasm assembly.
 R_IPI int wasm_asm(const char *str, unsigned char *buf, int buf_len) {
 	int i = 0, len = 0;
 	char tmp[256];
 	WasmOpDef *opdef = NULL;
-	while (i < sizeof (tmp) && str[i] && str[i] != ' ') {
+	while (i < (sizeof (tmp) - 1) && str[i] && str[i] != ' ') {
 		tmp[i] = str[i];
 		i++;
 	}
@@ -290,8 +467,8 @@ R_IPI int wasm_asm(const char *str, unsigned char *buf, int buf_len) {
 	if (len == 0) goto err;
 	// TODO: parse immediates
 	return len;
-	err:
-		return -1;
+  err:
+	return -1;
 }
 #endif
 
@@ -464,6 +641,9 @@ R_IPI int wasm_dis(WasmOp *op, const unsigned char *buf, int buf_len) {
 				case R_BIN_WASM_VALUETYPE_f64:
 					r_strbuf_setf (sb, "%s (result f64)", opdef->txt);
 					break;
+				case R_BIN_WASM_VALUETYPE_v128:
+					r_strbuf_setf (sb, "%s (result v128)", opdef->txt);
+					break;
 				default:
 					r_strbuf_setf (sb, "%s (result ?)", opdef->txt);
 					break;
@@ -606,7 +786,6 @@ R_IPI int wasm_dis(WasmOp *op, const unsigned char *buf, int buf_len) {
 				op->len += n;
 			}
 			break;
-
 		case WASM_OP_I32CONST:
 			{
 				st32 val = 0;
@@ -736,18 +915,255 @@ R_IPI int wasm_dis(WasmOp *op, const unsigned char *buf, int buf_len) {
 				r_strbuf_setf (sb, "%s %d %d", opdef->txt, flag, offset);
 				op->len += n;
 			}
-		break;
+			break;
 		default:
 			goto err;
 		}
+	} else if (id == 0xfd) {
+		if (buf_len < 2) {
+			goto err;
+		}
+		ut32 simdop;
+		op->type = WASM_TYPE_OP_SIMD;
+		size_t simdop_size = read_u32_leb128 (buf + 1, buf + buf_len, &simdop);
+		if (simdop_size > buf_len) {
+			goto err;
+		}
+		op->len = 1 + simdop_size;
+		op->op.simd = simdop;
+		id = simdop; // this isn't great because we're losing the sign
+		// but it should never cause problems
+		WasmOpDef *opdef = &opcodes_simd[id];
+		switch (id) {
+		case WASM_OP_I8X16SPLAT:
+		case WASM_OP_I16X8SPLAT:
+		case WASM_OP_I32X4SPLAT:
+		case WASM_OP_I64X2SPLAT:
+		case WASM_OP_F32X4SPLAT:
+		case WASM_OP_F64X2SPLAT:
+		case WASM_OP_I8X16EQ:
+		case WASM_OP_I8X16NE:
+		case WASM_OP_I8X16LTS:
+		case WASM_OP_I8X16LTU:
+		case WASM_OP_I8X16GTS:
+		case WASM_OP_I8X16GTU:
+		case WASM_OP_I8X16LES:
+		case WASM_OP_I8X16LEU:
+		case WASM_OP_I8X16GES:
+		case WASM_OP_I8X16GEU:
+		case WASM_OP_I16X8EQ:
+		case WASM_OP_I16X8NE:
+		case WASM_OP_I16X8LTS:
+		case WASM_OP_I16X8LTU:
+		case WASM_OP_I16X8GTS:
+		case WASM_OP_I16X8GTU:
+		case WASM_OP_I16X8LES:
+		case WASM_OP_I16X8LEU:
+		case WASM_OP_I16X8GES:
+		case WASM_OP_I16X8GEU:
+		case WASM_OP_I32X4EQ:
+		case WASM_OP_I32X4NE:
+		case WASM_OP_I32X4LTS:
+		case WASM_OP_I32X4LTU:
+		case WASM_OP_I32X4GTS:
+		case WASM_OP_I32X4GTU:
+		case WASM_OP_I32X4LES:
+		case WASM_OP_I32X4LEU:
+		case WASM_OP_I32X4GES:
+		case WASM_OP_I32X4GEU:
+		case WASM_OP_F32X4EQ:
+		case WASM_OP_F32X4NE:
+		case WASM_OP_F32X4LT:
+		case WASM_OP_F32X4GT:
+		case WASM_OP_F32X4LE:
+		case WASM_OP_F32X4GE:
+		case WASM_OP_F64X2EQ:
+		case WASM_OP_F64X2NE:
+		case WASM_OP_F64X2LT:
+		case WASM_OP_F64X2GT:
+		case WASM_OP_F64X2LE:
+		case WASM_OP_F64X2GE:
+		case WASM_OP_V128NOT:
+		case WASM_OP_V128AND:
+		case WASM_OP_V128OR:
+		case WASM_OP_V128XOR:
+		case WASM_OP_V128BITSELECT:
+		case WASM_OP_I8X16NEG:
+		case WASM_OP_I8X16ANYTRUE:
+		case WASM_OP_I8X16ALLTRUE:
+		case WASM_OP_I8X16SHL:
+		case WASM_OP_I8X16SHRS:
+		case WASM_OP_I8X16SHRU:
+		case WASM_OP_I8X16ADD:
+		case WASM_OP_I8X16ADDSATURATES:
+		case WASM_OP_I8X16ADDSATURATEU:
+		case WASM_OP_I8X16SUB:
+		case WASM_OP_I8X16SUBSATURATES:
+		case WASM_OP_I8X16SUBSATURATEU:
+		case WASM_OP_I8X16MINS:
+		case WASM_OP_I8X16MINU:
+		case WASM_OP_I8X16MAXS:
+		case WASM_OP_I8X16MAXU:
+		case WASM_OP_I16X8NEG:
+		case WASM_OP_I16X8ANYTRUE:
+		case WASM_OP_I16X8ALLTRUE:
+		case WASM_OP_I16X8SHL:
+		case WASM_OP_I16X8SHRS:
+		case WASM_OP_I16X8SHRU:
+		case WASM_OP_I16X8ADD:
+		case WASM_OP_I16X8ADDSATURATES:
+		case WASM_OP_I16X8ADDSATURATEU:
+		case WASM_OP_I16X8SUB:
+		case WASM_OP_I16X8SUBSATURATES:
+		case WASM_OP_I16X8SUBSATURATEU:
+		case WASM_OP_I16X8MUL:
+		case WASM_OP_I16X8MINS:
+		case WASM_OP_I16X8MINU:
+		case WASM_OP_I16X8MAXS:
+		case WASM_OP_I16X8MAXU:
+		case WASM_OP_I32X4NEG:
+		case WASM_OP_I32X4ANYTRUE:
+		case WASM_OP_I32X4ALLTRUE:
+		case WASM_OP_I32X4SHL:
+		case WASM_OP_I32X4SHRS:
+		case WASM_OP_I32X4SHRU:
+		case WASM_OP_I32X4ADD:
+		case WASM_OP_I32X4SUB:
+		case WASM_OP_I32X4MUL:
+		case WASM_OP_I32X4MINS:
+		case WASM_OP_I32X4MINU:
+		case WASM_OP_I32X4MAXS:
+		case WASM_OP_I32X4MAXU:
+		case WASM_OP_I64X2NEG:
+		case WASM_OP_I64X2SHL:
+		case WASM_OP_I64X2SHRS:
+		case WASM_OP_I64X2SHRU:
+		case WASM_OP_I64X2ADD:
+		case WASM_OP_I64X2SUB:
+		case WASM_OP_I64X2MUL:
+		case WASM_OP_F32X4ABS:
+		case WASM_OP_F32X4NEG:
+		case WASM_OP_F32X4SQRT:
+		case WASM_OP_F32X4ADD:
+		case WASM_OP_F32X4SUB:
+		case WASM_OP_F32X4MUL:
+		case WASM_OP_F32X4DIV:
+		case WASM_OP_F32X4MIN:
+		case WASM_OP_F32X4MAX:
+		case WASM_OP_F64X2ABS:
+		case WASM_OP_F64X2NEG:
+		case WASM_OP_F64X2SQRT:
+		case WASM_OP_F64X2ADD:
+		case WASM_OP_F64X2SUB:
+		case WASM_OP_F64X2MUL:
+		case WASM_OP_F64X2DIV:
+		case WASM_OP_F64X2MIN:
+		case WASM_OP_F64X2MAX:
+		case WASM_OP_I32X4TRUNCSATF32X4S:
+		case WASM_OP_I32X4TRUNCSATF32X4U:
+		case WASM_OP_F32X4CONVERTI32X4S:
+		case WASM_OP_F32X4CONVERTI32X4U:
+		case WASM_OP_V8X16SWIZZLE:
+		case WASM_OP_I8X16NARROWI16X8S:
+		case WASM_OP_I8X16NARROWI16X8U:
+		case WASM_OP_I16X8NARROWI32X4S:
+		case WASM_OP_I16X8NARROWI32X4U:
+		case WASM_OP_I16X8WIDENLOWI8X16S:
+		case WASM_OP_I16X8WIDENHIGHI8X16S:
+		case WASM_OP_I16X8WIDENLOWI8X16U:
+		case WASM_OP_I16X8WIDENHIGHI8X16U:
+		case WASM_OP_I32X4WIDENLOWI16X8S:
+		case WASM_OP_I32X4WIDENHIGHI16X8S:
+		case WASM_OP_I32X4WIDENLOWI16X8U:
+		case WASM_OP_I32X4WIDENHIGHI16X8U:
+		case WASM_OP_V128ANDNOT:
+		case WASM_OP_I8X16AVGRU:
+		case WASM_OP_I16X8AVGRU:
+		case WASM_OP_I8X16ABS:
+		case WASM_OP_I16X8ABS:
+		case WASM_OP_I32X4ABS:
+			{
+				r_strbuf_set (sb, opdef->txt);
+			}
+			break;
+		case WASM_OP_V128LOAD:
+		case WASM_OP_I16X8LOAD8X8S:
+		case WASM_OP_I16X8LOAD8X8U:
+		case WASM_OP_I32X4LOAD16X4S:
+		case WASM_OP_I32X4LOAD16X4U:
+		case WASM_OP_I64X2LOAD32X2S:
+		case WASM_OP_I64X2LOAD32X2U:
+		case WASM_OP_V8X16LOADSPLAT:
+		case WASM_OP_V16X8LOADSPLAT:
+		case WASM_OP_V32X4LOADSPLAT:
+		case WASM_OP_V64X2LOADSPLAT:
+		case WASM_OP_V128STORE:
+			{
+				ut32 offset = 0, align = 0;
+				size_t n = read_u32_leb128 (buf + 1 + simdop_size, buf + buf_len, &offset);
+				op->len += n;
+				if (!(n > 0 && n < buf_len)) {
+					goto err;
+				}
+				n = read_u32_leb128 (buf + 1 + simdop_size + n, buf + buf_len, &align);
+				op->len += n;
+				if (!(n > 0 && n < buf_len)) {
+					goto err;
+				}
+				r_strbuf_setf (sb, "%s o:%d a:2^%d", opdef->txt, offset, align);
+			}
+			break;
+		case WASM_OP_V128CONST:
+		case WASM_OP_V8X16SHUFFLE:
+			{
+				if (buf_len < 18) {
+					goto err;
+				}
+				op->len += 16;
+				unsigned char bytes[16] = { 0 };
+				int i;
+				for (i = 0; i < 16; ++i) {
+					bytes[i] = buf[i + 1 + simdop_size];
+				}
+				r_strbuf_setf (sb, "%s %02x %02x %02x %02x %02x %02x %02x " \
+						"%02x %02x %02x %02x %02x %02x %02x %02x %02x",
+						opdef->txt, bytes[0], bytes[1], bytes[2], bytes[3],
+						bytes[4], bytes[5], bytes[6], bytes[7], bytes[8],
+						bytes[9], bytes[10], bytes[11], bytes[12], bytes[13],
+						bytes[14], bytes[15]);
+			}
+			break;
+		case WASM_OP_I8X16EXTRACTLANES:
+		case WASM_OP_I8X16EXTRACTLANEU:
+		case WASM_OP_I8X16REPLACELANE:
+		case WASM_OP_I16X8EXTRACTLANES:
+		case WASM_OP_I16X8EXTRACTLANEU:
+		case WASM_OP_I16X8REPLACELANE:
+		case WASM_OP_I32X4EXTRACTLANE:
+		case WASM_OP_I32X4REPLACELANE:
+		case WASM_OP_I64X2EXTRACTLANE:
+		case WASM_OP_I64X2REPLACELANE:
+		case WASM_OP_F32X4EXTRACTLANE:
+		case WASM_OP_F32X4REPLACELANE:
+		case WASM_OP_F64X2EXTRACTLANE:
+		case WASM_OP_F64X2REPLACELANE:
+			{
+				if (buf_len < 3) {
+					goto err;
+				}
+				unsigned char lane = buf[1 + simdop_size];
+				++op->len;
+				r_strbuf_setf (sb, "%s %d", opdef->txt, lane);
+			}
+			break;
+		}
 	} else {
 		goto err;
 	}
-
 	op->txt = r_strbuf_drain (sb);
 	return op->len;
 
-err:
+  err:
 	op->len = 1;
 	r_strbuf_set (sb, "invalid");
 	op->txt = r_strbuf_drain (sb);
