@@ -1,10 +1,11 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2021 - pancake, nibble */
 
 #include <stdio.h>
 #include <r_core.h>
 #include <r_types.h>
 #include <r_util.h>
 #include <r_asm.h>
+#include <r_anal.h> // for RAnalBind
 #define USE_R2 1
 #include <spp/spp.h>
 #include <config.h>
@@ -26,7 +27,8 @@ static void parseHeap(RParse *p, RStrBuf *s) {
 	if (out) {
 		*out = 0;
 		strcpy (out , op_buf_asm);
-	// XXX we shouldn't pad here because we have t orefactor the RParse API to handle boundaries and chunks properly
+		// XXX we shouldn't pad here because we have to refactor
+		// the RParse API to handle boundaries and chunks properly
 		r_parse_parse (p, op_buf_asm, out);
 		r_strbuf_set (s, out);
 		free (out);
@@ -139,15 +141,28 @@ static inline int r_asm_pseudo_byte(RAsmOp *op, char *input) {
 
 static inline int r_asm_pseudo_fill(RAsmOp *op, char *input) {
 	int i, repeat = 0, size=0, value=0;
-	sscanf (input, "%d,%d,%d", &repeat, &size, &value); // use r_num?
+	if (strchr (input, ',')) {
+		int res = sscanf (input, "%d,%d,%d", &repeat, &size, &value); // use r_num?
+		if (res != 3) {
+			eprintf ("Invalid usage of .fill repeat,size,value\n");
+			eprintf ("for example: .fill 1,0x100,0\n");
+			return -1;
+		}
+	} else {
+		ut64 v = r_num_math (NULL, input);
+		size = (int)v;
+		repeat = 1;
+	}
 	size *= (sizeof (value) * repeat);
 	if (size > 0) {
 		ut8 *buf = malloc (size);
-		for (i = 0; i < size; i+= sizeof(value)) {
-			memcpy (&buf[i], &value, sizeof(value));
+		if (buf) {
+			for (i = 0; i < size; i += sizeof (value)) {
+				memcpy (&buf[i], &value, sizeof (value));
+			}
+			r_asm_op_set_buf (op, buf, size);
+			free (buf);
 		}
-		r_asm_op_set_buf (op, buf, size);
-		free (buf);
 	} else {
 		size = 0;
 	}
@@ -155,13 +170,13 @@ static inline int r_asm_pseudo_fill(RAsmOp *op, char *input) {
 }
 
 static inline int r_asm_pseudo_incbin(RAsmOp *op, char *input) {
-	int bytes_read = 0;
+	size_t bytes_read = 0;
 	r_str_replace_char (input, ',', ' ');
 	// int len = r_str_word_count (input);
 	r_str_word_set0 (input);
 	//const char *filename = r_str_word_get0 (input, 0);
-	int skip = (int)r_num_math (NULL, r_str_word_get0 (input, 1));
-	int count = (int)r_num_math (NULL,r_str_word_get0 (input, 2));
+	size_t skip = (size_t)r_num_math (NULL, r_str_word_get0 (input, 1));
+	size_t count = (size_t)r_num_math (NULL,r_str_word_get0 (input, 2));
 	char *content = r_file_slurp (input, &bytes_read);
 	if (!content) {
 		eprintf ("Could not open '%s'.\n", input);
@@ -189,7 +204,7 @@ static void plugin_free(RAsmPlugin *p) {
 	}
 }
 
-R_API RAsm *r_asm_new() {
+R_API RAsm *r_asm_new(void) {
 	int i;
 	RAsm *a = R_NEW0 (RAsm);
 	if (!a) {
@@ -217,7 +232,7 @@ R_API bool r_asm_setup(RAsm *a, const char *arch, int bits, int big_endian) {
 }
 
 // TODO: spagueti
-R_API int r_asm_filter_input(RAsm *a, const char *f) {
+R_API int r_asm_sub_names_input(RAsm *a, const char *f) {
 	r_return_val_if_fail (a && f, false);
 	if (!a->ifilter) {
 		a->ifilter = r_parse_new ();
@@ -230,7 +245,7 @@ R_API int r_asm_filter_input(RAsm *a, const char *f) {
 	return true;
 }
 
-R_API int r_asm_filter_output(RAsm *a, const char *f) {
+R_API int r_asm_sub_names_output(RAsm *a, const char *f) {
 	if (!a->ofilter) {
 		a->ofilter = r_parse_new ();
 	}
@@ -266,19 +281,14 @@ R_API void r_asm_set_user_ptr(RAsm *a, void *user) {
 }
 
 R_API bool r_asm_add(RAsm *a, RAsmPlugin *foo) {
-	RListIter *iter;
-	RAsmPlugin *h;
-	// TODO: cache foo->name length and use memcmp instead of strcmp
 	if (!foo->name) {
 		return false;
 	}
 	if (foo->init) {
 		foo->init (a->user);
 	}
-	r_list_foreach (a->plugins, iter, h) {
-		if (!strcmp (h->name, foo->name)) {
-			return false;
-		}
+	if (r_asm_is_valid (a, foo->name)) {
+		return false;
 	}
 	r_list_append (a->plugins, foo);
 	return true;
@@ -289,7 +299,7 @@ R_API int r_asm_del(RAsm *a, const char *name) {
 	return false;
 }
 
-R_API int r_asm_is_valid(RAsm *a, const char *name) {
+R_API bool r_asm_is_valid(RAsm *a, const char *name) {
 	RAsmPlugin *h;
 	RListIter *iter;
 	if (!name || !*name) {
@@ -320,6 +330,34 @@ R_API bool r_asm_use_assembler(RAsm *a, const char *name) {
 	return false;
 }
 
+static void load_asm_descriptions(RAsm *a, RAsmPlugin *p) {
+	const char *arch = p->arch;
+	if (!arch || !strcmp (p->name, "r2ghidra")) {
+		if (a->cpu) {
+			arch = a->cpu;
+		} else {
+			arch = p->name;
+		}
+	}
+#if HAVE_GPERF
+	SdbGperf *gp = r_asm_get_gperf (arch); // p->name);
+	if (gp) {
+		sdb_free (a->pair);
+		a->pair = sdb_new0 ();
+		sdb_open_gperf (a->pair, gp);
+		return;
+	}
+#endif
+	char *r2prefix = r_str_r2_prefix (R2_SDB_OPCODES);
+	char *file = r_str_newf ("%s/%s.sdb", r_str_getf (r2prefix), arch);
+	if (file) {
+		sdb_free (a->pair);
+		a->pair = sdb_new (NULL, file, 0);
+		free (file);
+	}
+	free (r2prefix);
+}
+
 // TODO: this can be optimized using r_str_hash()
 R_API bool r_asm_use(RAsm *a, const char *name) {
 	RAsmPlugin *h;
@@ -328,20 +366,33 @@ R_API bool r_asm_use(RAsm *a, const char *name) {
 		return false;
 	}
 	r_list_foreach (a->plugins, iter, h) {
-		if (!strcmp (h->name, name) && h->arch) {
-			if (!a->cur || (a->cur && strcmp (a->cur->arch, h->arch))) {
-				char *r2prefix = r_str_r2_prefix (R2_SDB_OPCODES);
-				char *file = r_str_newf ("%s/%s.sdb", r_str_get (r2prefix), h->arch);
-				if (file) {
+		if (h->arch) {
+			if (!strcmp (h->name, name)) {
+				if (!a->cur || (a->cur && strcmp (a->cur->arch, h->arch))) {
+					load_asm_descriptions (a, h);
 					r_asm_set_cpu (a, NULL);
-					sdb_free (a->pair);
-					a->pair = sdb_new (NULL, file, 0);
-					free (file);
 				}
-				free (r2prefix);
+				a->cur = h;
+				return true;
+			}
+		} else {
+			char *vv = strchr (name, '.');
+			if (vv) {
+				if (!strcmp (vv + 1, h->name)) {
+					char *cpu = r_str_ndup (name, vv - name);
+					r_asm_set_cpu (a, cpu);
+					a->cur = h;
+					free (cpu);
+					return true;
+				}
+			} else {
+				if (!strcmp (name, h->name)) {
+					h->arch = name;
+					r_asm_set_cpu (a, NULL);
+					a->cur = h;
+					return true;
+				}
 			}
-			a->cur = h;
-			return true;
 		}
 	}
 	sdb_free (a->pair);
@@ -349,18 +400,18 @@ R_API bool r_asm_use(RAsm *a, const char *name) {
 	return false;
 }
 
-static int has_bits(RAsmPlugin *h, int bits) {
-	return (h && h->bits && (bits & h->bits));
-}
-
-R_API void r_asm_set_cpu(RAsm *a, const char *cpu) {
+R_DEPRECATE R_API void r_asm_set_cpu(RAsm *a, const char *cpu) {
 	if (a) {
 		free (a->cpu);
 		a->cpu = cpu? strdup (cpu): NULL;
 	}
 }
 
-R_API int r_asm_set_bits(RAsm *a, int bits) {
+static bool has_bits(RAsmPlugin *h, int bits) {
+	return (h && h->bits && (bits & h->bits));
+}
+
+R_DEPRECATE R_API int r_asm_set_bits(RAsm *a, int bits) {
 	if (has_bits (a->cur, bits)) {
 		a->bits = bits; // TODO : use OR? :)
 		return true;
@@ -370,11 +421,11 @@ R_API int r_asm_set_bits(RAsm *a, int bits) {
 
 R_API bool r_asm_set_big_endian(RAsm *a, bool b) {
 	r_return_val_if_fail (a && a->cur, false);
-	a->big_endian = false; //little endian by default
+	a->big_endian = false; // little endian by default
 	switch (a->cur->endian) {
 	case R_SYS_ENDIAN_NONE:
 	case R_SYS_ENDIAN_BI:
-		// let user select
+		// TODO: not yet implemented
 		a->big_endian = b;
 		break;
 	case R_SYS_ENDIAN_LITTLE:
@@ -390,7 +441,8 @@ R_API bool r_asm_set_big_endian(RAsm *a, bool b) {
 	return a->big_endian;
 }
 
-R_API int r_asm_set_syntax(RAsm *a, int syntax) {
+R_API bool r_asm_set_syntax(RAsm *a, int syntax) {
+	// TODO: move into r_arch ?
 	switch (syntax) {
 	case R_ASM_SYNTAX_REGNUM:
 	case R_ASM_SYNTAX_INTEL:
@@ -445,6 +497,14 @@ R_API int r_asm_disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 		} else {
 			ret = a->cur->disassemble (a, op, buf, len);
 		}
+	} else if (a->analb.anal) {
+		// disassemble using the analysis plugin if found
+		RAnalOp aop;
+		a->analb.opinit (&aop);
+		ret = a->analb.decode (a->analb.anal, &aop, a->pc, buf, len, R_ANAL_OP_MASK_DISASM);
+		op->size = aop.size;
+		r_strbuf_set (&op->buf_asm, aop.mnemonic);
+		a->analb.opfini (&aop);
 	}
 	if (ret < 0) {
 		ret = 0;
@@ -487,7 +547,7 @@ static bool assemblerMatches(RAsm *a, RAsmPlugin *h) {
 	if (!a || !h->arch || !h->assemble || !has_bits (h, a->bits)) {
 		return false;
 	}
-	return (!strncmp (a->cur->arch, h->arch, strlen (a->cur->arch)));
+	return (a->cur->arch && !strncmp (a->cur->arch, h->arch, strlen (a->cur->arch)));
 }
 
 static Ase findAssembler(RAsm *a, const char *kw) {
@@ -558,7 +618,7 @@ static char *replace_directives(char *str) {
 	return o;
 }
 
-R_API void r_asm_list_directives() {
+R_API void r_asm_list_directives(void) {
 	int i = 0;
 	char *dir = directives[i++];
 	while (dir) {
@@ -567,6 +627,7 @@ R_API void r_asm_list_directives() {
 	}
 }
 
+// returns instruction size
 R_API int r_asm_assemble(RAsm *a, RAsmOp *op, const char *buf) {
 	r_return_val_if_fail (a && op && buf, 0);
 	int ret = 0;
@@ -617,8 +678,8 @@ R_API RAsmCode* r_asm_mdisassemble(RAsm *a, const ut8 *buf, int len) {
 	ut64 pc = a->pc;
 	RAsmOp op;
 	ut64 idx;
-	int ret, slen;
-	const int addrbytes = a->user ? ((RCore *)a->user)->io->addrbytes : 1;
+	size_t ret;
+	const size_t addrbytes = a->user? ((RCore *)a->user)->io->addrbytes: 1;
 
 	if (!(acode = r_asm_code_new ())) {
 		return NULL;
@@ -630,7 +691,7 @@ R_API RAsmCode* r_asm_mdisassemble(RAsm *a, const ut8 *buf, int len) {
 	if (!(buf_asm = r_strbuf_new (NULL))) {
 		return r_asm_code_free (acode);
 	}
-	for (idx = ret = slen = 0; idx + addrbytes <= len; idx += addrbytes * ret) {
+	for (idx = 0; idx + addrbytes <= len; idx += (addrbytes * ret)) {
 		r_asm_set_pc (a, pc + idx);
 		ret = r_asm_disassemble (a, &op, buf + idx, len - idx);
 		if (ret < 1) {
@@ -676,7 +737,7 @@ static void *__dup_val(const void *v) {
 }
 
 R_API RAsmCode *r_asm_massemble(RAsm *a, const char *assembly) {
-	int num, stage, ret, idx, ctr, i, j, linenum = 0;
+	int num, stage, ret, idx, ctr, i, linenum = 0;
 	char *lbuf = NULL, *ptr2, *ptr = NULL, *ptr_start = NULL;
 	const char *asmcpu = NULL;
 	RAsmCode *acode = NULL;
@@ -793,7 +854,7 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *assembly) {
 		}
 		inComment = false;
 		r_asm_set_pc (a, pc);
-		for (idx = ret = i = j = 0, off = a->pc; i <= ctr; i++, idx += ret) {
+		for (idx = ret = i = 0, off = a->pc; i <= ctr; i++, idx += ret) {
 			buf_token = tokens[i];
 			if (!buf_token) {
 				continue;
@@ -805,7 +866,7 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *assembly) {
 				continue;
 			}
 			// XXX TODO remove arch-specific hacks
-			if (!strncmp (a->cur->arch, "avr", 3)) {
+			if (a->cur->arch && !strncmp (a->cur->arch, "avr", 3)) {
 				for (ptr_start = buf_token; *ptr_start && isavrseparator (*ptr_start); ptr_start++);
 			} else {
 				for (ptr_start = buf_token; *ptr_start && IS_SEPARATOR (*ptr_start); ptr_start++);
@@ -825,7 +886,7 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *assembly) {
 				if (cptr && ptr && cptr < ptr) {
 					likely_comment = false;
 					for (cptr += 1; cptr < ptr ; cptr += 1) {
-						if ( ! isspace ( *cptr)) {
+						if ( ! isspace ((unsigned char) *cptr)) {
 							likely_comment = true;
 							break;
 						}
@@ -922,6 +983,9 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *assembly) {
 					ret = r_asm_pseudo_bits (a, ptr+6);
 				} else if (!strncmp (ptr, ".fill ", 6)) {
 					ret = r_asm_pseudo_fill (&op, ptr+6);
+					if (ret == -1) {
+						goto fail;
+					}
 				} else if (!strncmp (ptr, ".kernel ", 8)) {
 					r_syscall_setup (a->syscall, a->cur->arch, a->bits, asmcpu, ptr + 8);
 				} else if (!strncmp (ptr, ".cpu ", 5)) {
@@ -1022,6 +1086,7 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *assembly) {
 				if (op.buf_inc && r_buf_size (op.buf_inc) > 1) {
 					char *inc = r_buf_to_string (op.buf_inc);
 					r_buf_free (op.buf_inc);
+					op.buf_inc = NULL;
 					if (inc) {
 						ret += r_hex_str2bin (inc, acode->bytes + idx + ret);
 						free (inc);
