@@ -11,11 +11,13 @@
 #include <r_lib.h>
 #include <r_asm.h>
 
-#include "../arch/avr/disasm.c"
+#include "../arch/avr/avr_disasm.h"
+#include "../arch/avr/avr_instructionset.h"
+#include "../arch/avr/disasm.h"
 
 static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	char buf_asm[32] = {0};
-	op->size = avrdis (buf_asm, a->pc, buf, len);
+	op->size = avr_decode (buf_asm, a->pc, buf, len);
 	if (*buf_asm == '.') {
 		*buf_asm = 0;
 	}
