@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2019 - pancake */
+/* radare - LGPL - Copyright 2019-2021 - pancake */
 
 #include <stdio.h>
 #include <string.h>
@@ -6,12 +6,14 @@
 #include <r_lib.h>
 #include <r_asm.h>
 
+#if 0
 static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	int opsz = 0;
 	r_strbuf_set (&op->buf_asm, "");
 	op->size = opsz;
 	return opsz;
 }
+#endif
 
 static int assemble(RAsm *a, RAsmOp *op, const char *buf) {
 	return 0;
@@ -26,7 +28,7 @@ RAsmPlugin r_asm_plugin_null = {
 	.bits = 16 | 32 | 64,
 	.endian = R_SYS_ENDIAN_NONE,
 	.desc = "no disassemble",
-	.disassemble = &disassemble,
+	.disassemble = NULL,// &disassemble,
 	.assemble = &assemble
 };
 
