@@ -1,24 +1,21 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2020 - pancake */
 
-#include <r_types.h>
-#include <r_util.h>
 #include <r_lib.h>
 #include <r_asm.h>
 
 static int assemble(RAsm *a, RAsmOp *op, const char *buf) {
 	char *ipath, *opath;
-	int ifd, ofd;
 	if (a->syntax != R_ASM_SYNTAX_INTEL) {
 		eprintf ("asm.x86.nasm does not support non-intel syntax\n");
 		return -1;
 	}
 
-	ifd = r_file_mkstemp ("r_nasm", &ipath);
+	int ifd = r_file_mkstemp ("r_nasm", &ipath);
 	if (ifd == -1) {
 		return -1;
 	}
 
-	ofd = r_file_mkstemp ("r_nasm", &opath);
+	int ofd = r_file_mkstemp ("r_nasm", &opath);
 	if (ofd == -1) {
 		free (ipath);
 		return -1;
@@ -26,7 +23,7 @@ static int assemble(RAsm *a, RAsmOp *op, const char *buf) {
 
 	char *asm_buf = r_str_newf ("[BITS %i]\nORG 0x%"PFMT64x"\n%s\n", a->bits, a->pc, buf);
 	if (asm_buf) {
-		write (ifd, asm_buf, strlen (asm_buf));
+		(void)write (ifd, asm_buf, strlen (asm_buf));
 		free (asm_buf);
 	}
 
@@ -55,7 +52,7 @@ RAsmPlugin r_asm_plugin_x86_nasm = {
 	.license = "LGPL3",
 	.arch = "x86",
 	// NOTE: 64bits is not supported on OSX's nasm :(
-	.bits = 16|32|64,
+	.bits = 16 | 32 | 64,
 	.endian = R_SYS_ENDIAN_LITTLE,
 	.assemble = &assemble
 };
