@@ -6,11 +6,11 @@ static char *mnemonics(RAsm *a, int id, bool json) {
 		if (json) {
 			return name? r_str_newf ("[\"%s\"]\n", name): NULL;
 		}
-		return name? r_str_newf ("%s", name): NULL;
+		return name? strdup (name): NULL;
 	}
 	RStrBuf *buf = r_strbuf_new ("");
 	if (json) {
-			r_strbuf_append (buf, "[");
+		r_strbuf_append (buf, "[");
 	}
 	for (i = 1; ; i++) {
 		const char *op = cs_insn_name (cd, i);
