@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2009-2019 - pancake, nibble, dso */
+/* radare2 - LGPL - Copyright 2009-2021 - pancake, nibble, dso */
 
 #include <r_bin.h>
 #include <r_hash.h>
@@ -82,23 +82,19 @@ static void print_string(RBinFile *bf, RBinString *string, int raw, PJ *pj) {
 		}
 		break;
 	case R_MODE_RADARE: {
-		char *f_name, *nstr;
-		f_name = strdup (string->string);
-		r_name_filter (f_name, 512);
+		char *f_name = strdup (string->string);
+		r_name_filter (f_name, -1);
 		if (bin->prefix) {
-			nstr = r_str_newf ("%s.str.%s", bin->prefix, f_name);
 			io->cb_printf ("f %s.str.%s %u @ 0x%08"PFMT64x"\n"
 					"Cs %u @ 0x%08"PFMT64x"\n",
 					bin->prefix, f_name, string->size, addr,
 					string->size, addr);
 		} else {
-			nstr = r_str_newf ("str.%s", f_name);
 			io->cb_printf ("f str.%s %u @ 0x%08"PFMT64x"\n"
 					"Cs %u @ 0x%08"PFMT64x"\n",
 					f_name, string->size, addr,
 					string->size, addr);
 		}
-		free (nstr);
 		free (f_name);
 		break;
 		}
@@ -150,6 +146,37 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 		}
 	}
 	r_buf_read_at (bf->buf, from, buf, len);
+	char *charset = r_sys_getenv ("RABIN2_CHARSET");
+	if (!R_STR_ISEMPTY (charset)) {
+		RCharset *ch = r_charset_new ();
+		if (r_charset_use (ch, charset)) {
+			int outlen = len * 4;
+			ut8 *out = calloc (len, 4);
+			if (out) {
+				int res = r_charset_encode_str (ch, out, outlen, buf, len);
+				int i;
+				// TODO unknown chars should be translated to null bytes
+				for (i = 0; i < res; i++) {
+					if (out[i] == '?') {
+						out[i] = 0;
+					}
+				}
+				len = res;
+				free (buf);
+#if 1
+				buf = out;
+#else
+				// buf = realloc (out, len + 1);
+#endif
+			} else {
+				eprintf ("Cannot allocate\n");
+			}
+		} else {
+			eprintf ("Invalid value for RABIN2_CHARSET.\n");
+		}
+		r_charset_free (ch);
+	}
+	free (charset);
 	// may oobread
 	while (needle < to) {
 		if (bin && bin->consb.is_breaked) {
@@ -169,12 +196,14 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 				if (is_wide32) {
 					str_type = R_STRING_TYPE_WIDE32;
 				} else {
-					bool is_wide = needle + rc + 2 < to && !w[0] && w[1] && !w[2];
+					bool is_wide = needle + rc + 4 < to && !w[0] && w[1] && !w[2] && w[3] && !w[4];
 					str_type = is_wide? R_STRING_TYPE_WIDE: R_STRING_TYPE_ASCII;
 				}
 			} else {
 				str_type = R_STRING_TYPE_ASCII;
 			}
+		} else if (type == R_STRING_TYPE_UTF8) {
+			str_type = R_STRING_TYPE_ASCII; // initial assumption
 		} else {
 			str_type = type;
 		}
@@ -353,9 +382,11 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 	free (buf);
 	if (pj) {
 		pj_end (pj);
-		RIO *io = bin->iob.io;
-		if (io) {
-			io->cb_printf ("%s", pj_string (pj));
+		if (bin) {
+			RIO *io = bin->iob.io;
+			if (io) {
+				io->cb_printf ("%s", pj_string (pj));
+			}
 		}
 		pj_free (pj);
 	}
@@ -382,18 +413,25 @@ static void get_strings_range(RBinFile *bf, RList *list, int min, int raw, ut64
 		min = plugin? plugin->minstrlen: 4;
 	}
 	/* Some plugins return zero, fix it up */
-	if (!min) {
-		min = 4;
-	}
 	if (min < 0) {
 		return;
 	}
-	if (!bf->rbin->is_debugger) {
-		if (!to || to > r_buf_size (bf->buf)) {
-			to = r_buf_size (bf->buf);
-		}
-		if (!to) {
-			return;
+	if (!min) {
+		min = 4;
+	}
+	{
+		RIO *io = bf->rbin->iob.io;
+		RCoreBind *cb = &io->corebind;
+		if (cb && cb->cfgGet) {
+			const bool cfg_debug = cb->cfgGet (cb->core, "cfg.debug");
+			if (!cfg_debug) {
+				if (!to || to > r_buf_size (bf->buf)) {
+					to = r_buf_size (bf->buf);
+				}
+				if (!to) {
+					return;
+				}
+			}
 		}
 	}
 	if (raw != 2) {
@@ -401,13 +439,29 @@ static void get_strings_range(RBinFile *bf, RList *list, int min, int raw, ut64
 		// in case of dump ignore here
 		if (bf->rbin->maxstrbuf && size && size > bf->rbin->maxstrbuf) {
 			if (bf->rbin->verbose) {
-				eprintf ("WARNING: bin_strings buffer is too big (0x%08" PFMT64x "). Use -zzz or set bin.maxstrbuf (RABIN2_MAXSTRBUF) in r2 (rabin2)\n",
+				eprintf ("Warning: bin_strings buffer is too big (0x%08" PFMT64x "). Use -zzz or set bin.maxstrbuf (RABIN2_MAXSTRBUF) in r2 (rabin2)\n",
 					size);
 			}
 			return;
 		}
 	}
-	string_scan_range (list, bf, min, from, to, -1, raw, section);
+	int type;
+	const char *enc = bf->rbin->strenc;
+	if (!enc) {
+		type = R_STRING_TYPE_DETECT;
+	} else if (!strcmp (enc, "latin1")) {
+		type = R_STRING_TYPE_ASCII;
+	} else if (!strcmp (enc, "utf8")) {
+		type = R_STRING_TYPE_UTF8;
+	} else if (!strcmp (enc, "utf16le")) {
+		type = R_STRING_TYPE_WIDE;
+	} else if (!strcmp (enc, "utf32le")) {
+		type = R_STRING_TYPE_WIDE32;
+	} else { // TODO utf16be, utf32be
+		eprintf ("ERROR: encoding %s not supported\n", enc);
+		return;
+	}
+	string_scan_range (list, bf, min, from, to, type, raw, section);
 }
 
 R_IPI RBinFile *r_bin_file_new(RBin *bin, const char *file, ut64 file_sz, int rawstr, int fd, const char *xtrname, Sdb *sdb, bool steal_ptr) {
@@ -434,7 +488,7 @@ R_IPI RBinFile *r_bin_file_new(RBin *bin, const char *file, ut64 file_sz, int ra
 	return bf;
 }
 
-static RBinPlugin *get_plugin_from_buffer(RBin *bin, const char *pluginname, RBuffer *buf) {
+static RBinPlugin *get_plugin_from_buffer(RBin *bin, RBinFile *bf, const char *pluginname, RBuffer *buf) {
 	RBinPlugin *plugin = bin->force? r_bin_get_binplugin_by_name (bin, bin->force): NULL;
 	if (plugin) {
 		return plugin;
@@ -443,7 +497,7 @@ static RBinPlugin *get_plugin_from_buffer(RBin *bin, const char *pluginname, RBu
 	if (plugin) {
 		return plugin;
 	}
-	plugin = r_bin_get_binplugin_by_buffer (bin, buf);
+	plugin = r_bin_get_binplugin_by_buffer (bin, bf, buf);
 	if (plugin) {
 		return plugin;
 	}
@@ -456,7 +510,7 @@ R_API bool r_bin_file_object_new_from_xtr_data(RBin *bin, RBinFile *bf, ut64 bas
 	ut64 offset = data->offset;
 	ut64 sz = data->size;
 
-	RBinPlugin *plugin = get_plugin_from_buffer (bin, NULL, data->buf);
+	RBinPlugin *plugin = get_plugin_from_buffer (bin, bf, NULL, data->buf);
 	bf->buf = r_buf_ref (data->buf);
 
 	RBinObject *o = r_bin_object_new (bf, plugin, baseaddr, loadaddr, offset, sz);
@@ -500,11 +554,12 @@ R_IPI RBinFile *r_bin_file_new_from_buffer(RBin *bin, const char *file, RBuffer
 
 	RBinFile *bf = r_bin_file_new (bin, file, r_buf_size (buf), rawstr, fd, pluginname, NULL, false);
 	if (bf) {
+		RListIter *item = r_list_append (bin->binfiles, bf);
 		bf->buf = r_buf_ref (buf);
-		RBinPlugin *plugin = get_plugin_from_buffer (bin, pluginname, bf->buf);
+		RBinPlugin *plugin = get_plugin_from_buffer (bin, bf, pluginname, bf->buf);
 		RBinObject *o = r_bin_object_new (bf, plugin, baseaddr, loadaddr, 0, r_buf_size (bf->buf));
 		if (!o) {
-			r_bin_file_free (bf);
+			r_list_delete (bin->binfiles, item);
 			return NULL;
 		}
 		// size is set here because the reported size of the object depends on
@@ -512,7 +567,6 @@ R_IPI RBinFile *r_bin_file_new_from_buffer(RBin *bin, const char *file, RBuffer
 		if (!o->size) {
 			o->size = r_buf_size (buf);
 		}
-		r_list_append (bin->binfiles, bf);
 	}
 	return bf;
 }
@@ -611,26 +665,6 @@ R_API RBinFile *r_bin_file_find_by_name(RBin *bin, const char *name) {
 	return NULL;
 }
 
-R_IPI RBinFile *r_bin_file_find_by_name_n(RBin *bin, const char *name, int idx) {
-	RListIter *iter;
-	RBinFile *bf = NULL;
-	int i = 0;
-	if (!bin) {
-		return bf;
-	}
-
-	r_list_foreach (bin->binfiles, iter, bf) {
-		if (bf && bf->file && !strcmp (bf->file, name)) {
-			if (i == idx) {
-				break;
-			}
-			i++;
-		}
-		bf = NULL;
-	}
-	return bf;
-}
-
 R_API bool r_bin_file_set_cur_by_id(RBin *bin, ut32 bin_id) {
 	RBinFile *bf = r_bin_file_find_by_id (bin, bin_id);
 	return bf? r_bin_file_set_cur_binfile (bin, bf): false;
@@ -856,55 +890,34 @@ R_API bool r_bin_file_close(RBin *bin, int bd) {
 	return false;
 }
 
-R_API bool r_bin_file_hash(RBin *bin, ut64 limit, const char *file, RList/*<RBinFileHash>*/ **old_file_hashes) {
-	r_return_val_if_fail (bin, false);
-
-	char hash[128];
-	RHash *ctx;
+R_API RList *r_bin_file_compute_hashes(RBin *bin, ut64 limit) {
+	r_return_val_if_fail (bin && bin->cur && bin->cur->o, NULL);
 	ut64 buf_len = 0, r = 0;
 	RBinFile *bf = bin->cur;
-	if (!bf) {
-		return false;
-	}
 	RBinObject *o = bf->o;
-	if (!o || !o->info) {
-		return false;
-	}
+
 	RIODesc *iod = r_io_desc_get (bin->iob.io, bf->fd);
 	if (!iod) {
-		return false;
-	}
-	if (!file && iod) {
-		file = iod->name;
+		return NULL;
 	}
+
 	buf_len = r_io_desc_size (iod);
 	// By SLURP_LIMIT normally cannot compute ...
 	if (buf_len > limit) {
-		if (old_file_hashes) {
-			//	if (bin->verbose) {
+		if (bin->verbose) {
 			eprintf ("Warning: r_bin_file_hash: file exceeds bin.hashlimit\n");
-			//	}
 		}
-		return false;
+		return NULL;
 	}
 	const size_t blocksize = 64000;
 	ut8 *buf = malloc (blocksize);
 	if (!buf) {
 		eprintf ("Cannot allocate computation buffer\n");
-		return false;
+		return NULL;
 	}
-	if (old_file_hashes) {
-		*old_file_hashes = NULL;
-	}
-	if (!r_list_empty (o->info->file_hashes)) {
-		if (old_file_hashes && o->info->file_hashes) {
-			*old_file_hashes = o->info->file_hashes;
-		} else {
-			r_list_free (o->info->file_hashes);
-		}
-		o->info->file_hashes = NULL;
-	}
-	ctx = r_hash_new (false, R_HASH_MD5 | R_HASH_SHA1 | R_HASH_SHA256);
+
+	char hash[128];
+	RHash *ctx = r_hash_new (false, R_HASH_MD5 | R_HASH_SHA1 | R_HASH_SHA256);
 	while (r + blocksize < buf_len) {
 		r_io_desc_seek (iod, r, R_IO_SEEK_SET);
 		int b = r_io_desc_read (iod, buf, blocksize);
@@ -928,12 +941,12 @@ R_API bool r_bin_file_hash(RBin *bin, ut64 limit, const char *file, RList/*<RBin
 	r_hash_do_end (ctx, R_HASH_MD5);
 	r_hex_bin2str (ctx->digest, R_HASH_SIZE_MD5, hash);
 
-	o->info->file_hashes = r_list_newf ((RListFree) r_bin_file_hash_free);
+	RList *file_hashes = r_list_newf ((RListFree) r_bin_file_hash_free);
 	RBinFileHash *md5h = R_NEW0 (RBinFileHash);
 	if (md5h) {
 		md5h->type = strdup ("md5");
 		md5h->hex = strdup (hash);
-		r_list_push (o->info->file_hashes, md5h);
+		r_list_push (file_hashes, md5h);
 	}
 	r_hash_do_end (ctx, R_HASH_SHA1);
 	r_hex_bin2str (ctx->digest, R_HASH_SIZE_SHA1, hash);
@@ -942,7 +955,7 @@ R_API bool r_bin_file_hash(RBin *bin, ut64 limit, const char *file, RList/*<RBin
 	if (sha1h) {
 		sha1h->type = strdup ("sha1");
 		sha1h->hex = strdup (hash);
-		r_list_push (o->info->file_hashes, sha1h);
+		r_list_push (file_hashes, sha1h);
 	}
 	r_hash_do_end (ctx, R_HASH_SHA256);
 	r_hex_bin2str (ctx->digest, R_HASH_SIZE_SHA256, hash);
@@ -951,13 +964,31 @@ R_API bool r_bin_file_hash(RBin *bin, ut64 limit, const char *file, RList/*<RBin
 	if (sha256h) {
 		sha256h->type = strdup ("sha256");
 		sha256h->hex = strdup (hash);
-		r_list_push (o->info->file_hashes, sha256h);
+		r_list_push (file_hashes, sha256h);
+	}
+
+	if (o->plugin && o->plugin->hashes) {
+		RList *plugin_hashes = o->plugin->hashes (bf);
+		r_list_join (file_hashes, plugin_hashes);
+		free (plugin_hashes);
 	}
 	// TODO: add here more rows
 
 	free (buf);
 	r_hash_free (ctx);
-	return true;
+	return file_hashes;
+}
+
+// Set new hashes to current RBinInfo, caller should free the returned RList
+R_API RList *r_bin_file_set_hashes(RBin *bin, RList/*<RBinFileHash*/ *new_hashes) {
+	r_return_val_if_fail (bin && bin->cur && bin->cur->o && bin->cur->o->info, NULL);
+	RBinFile *bf = bin->cur;
+	RBinInfo *info = bf->o->info;
+
+	RList *prev_hashes = info->file_hashes;
+	info->file_hashes = new_hashes;
+
+	return prev_hashes;
 }
 
 R_IPI RBinClass *r_bin_class_new(const char *name, const char *super, int view) {
@@ -977,6 +1008,7 @@ R_IPI void r_bin_class_free(RBinClass *k) {
 	if (k) {
 		free (k->name);
 		free (k->super);
+		free (k->visibility_str);
 		r_list_free (k->methods);
 		r_list_free (k->fields);
 		free (k);
