@@ -1,3 +1,6 @@
+#ifndef DEX_H
+#define DEX_H
+
 #include <r_types.h>
 #include <r_util.h>
 #include <r_lib.h>
@@ -5,24 +8,53 @@
 
 #define R_BIN_DEX_MAXSTR 256
 #define DEX_CLASS_SIZE (32)
+#define LEB_MAX_SIZE 6
 
-/* method flags */
+/* method flags (ACC_ things) */
 #define R_DEX_METH_PUBLIC 0x0001
 #define R_DEX_METH_PRIVATE 0x0002
 #define R_DEX_METH_PROTECTED 0x0004
 #define R_DEX_METH_STATIC 0x0008
 #define R_DEX_METH_FINAL 0x0010
 #define R_DEX_METH_SYNCHRONIZED 0x0020
+#define R_DEX_METH_VOLATILE 0x0040
 #define R_DEX_METH_BRIDGE 0x0040
+#define R_DEX_METH_TRANSIENT 0x0080
 #define R_DEX_METH_VARARGS 0x0080
 #define R_DEX_METH_NATIVE 0x0100
+#define R_DEX_METH_INTERFACE 0x0200
 #define R_DEX_METH_ABSTRACT 0x0400
 #define R_DEX_METH_STRICT 0x0800
 #define R_DEX_METH_SYNTHETIC 0x1000
+#define R_DEX_METH_ANNOTATION 0x2000
+#define R_DEX_METH_ENUM 0x4000
 #define R_DEX_METH_MIRANDA 0x8000
 #define R_DEX_METH_CONSTRUCTOR 0x10000
 #define R_DEX_METH_DECLARED_SYNCHRONIZED 0x20000
 
+// encoded value types
+#define R_DEX_ENCVAL_BYTE 0x00
+#define R_DEX_ENCVAL_SHORT 0x02
+#define R_DEX_ENCVAL_CHAR 0x03
+#define R_DEX_ENCVAL_INT 0x04
+#define R_DEX_ENCVAL_LONG 0x06
+#define R_DEX_ENCVAL_FLOAT 0x10
+#define R_DEX_ENCVAL_DOUBLE 0x11
+#define R_DEX_ENCVAL_STRING 0x17
+#define R_DEX_ENCVAL_TYPE 0x18
+#define R_DEX_ENCVAL_FIELD 0x19
+#define R_DEX_ENCVAL_ENUM 0x1b
+#define R_DEX_ENCVAL_METHOD 0x1a
+#define R_DEX_ENCVAL_ARRAY 0x1c
+#define R_DEX_ENCVAL_ANNOTATION 0x1d
+#define R_DEX_ENCVAL_NULL 0x1e
+#define R_DEX_ENCVAL_BOOLEAN 0x1f
+
+// visibilities
+#define R_DEX_VISIBILITY_BUILD 0
+#define R_DEX_VISIBILITY_RUNTIME 1
+#define R_DEX_VISIBILITY_SYSTEM 2
+
 R_PACKED(
 typedef struct dex_header_t {
 	ut8 magic[8];
@@ -116,6 +148,8 @@ typedef struct r_bin_dex_obj_t {
 	ut64 code_to;
 	char *version;
 	Sdb *kv;
+	char **cal_strings;
+	bool verbose;
 } RBinDexObj;
 
 struct r_bin_dex_str_t {
@@ -154,9 +188,8 @@ struct dex_debug_local_t {
 };
 
 char* r_bin_dex_get_version(struct r_bin_dex_obj_t* bin);
-struct r_bin_dex_obj_t *r_bin_dex_new_buf(RBuffer *buf);
-struct r_bin_dex_str_t *r_bin_dex_get_strings (struct r_bin_dex_obj_t *bin);
+void r_bin_dex_free(struct r_bin_dex_obj_t *bin);
+struct r_bin_dex_obj_t *r_bin_dex_new_buf(RBuffer *buf, bool verbose);
+struct r_bin_dex_str_t *r_bin_dex_get_strings(struct r_bin_dex_obj_t *bin);
 
-int dex_read_uleb128 (const ut8 *ptr, int size);
-int dex_read_sleb128 (const char *ptr, int size);
-int dex_uleb128_len (const ut8 *ptr, int size);
+#endif
