@@ -18,6 +18,23 @@
 #define R_BIN_ELF_ALL_SYMBOLS (R_BIN_ELF_SYMTAB_SYMBOLS | R_BIN_ELF_DYNSYM_SYMBOLS)
 #define ELFOBJ struct Elf_(r_bin_elf_obj_t)
 
+#if R_BIN_ELF64
+#define R_BIN_ELF_WORDSIZE 0x8
+#define R_BIN_ELF_WORD_MAX UT64_MAX
+#define R_BIN_ELF_READWORD(x, i) READ64 (x, i)
+#define R_BIN_ELF_BREADWORD(x, i) BREAD64 (x, i)
+#define R_BIN_ELF_ADDR_MAX UT64_MAX
+#define R_BIN_ELF_XWORD_MAX UT64_MAX
+#else
+#define R_BIN_ELF_WORDSIZE 0x4
+#define R_BIN_ELF_WORD_MAX UT32_MAX
+#define R_BIN_ELF_READWORD(x, i) READ32 (x, i)
+#define R_BIN_ELF_BREADWORD(x, i) BREAD32 (x, i)
+#define R_BIN_ELF_ADDR_MAX UT32_MAX
+#define R_BIN_ELF_XWORD_MAX UT64_MAX
+#endif
+
+
 typedef struct r_bin_elf_section_t {
 	ut64 offset;
 	ut64 rva;
@@ -49,7 +66,7 @@ typedef struct r_bin_elf_symbol_t {
 typedef struct r_bin_elf_reloc_t {
 	int sym;
 	int type;
-	int is_rela;
+	Elf_(Xword) rel_mode;
 	st64 addend;
 	ut64 offset;
 	ut64 rva;
@@ -72,6 +89,32 @@ typedef struct r_bin_elf_string_t {
 	int last;
 } RBinElfString;
 
+typedef struct Elf_(r_bin_elf_dynamic_info) {
+	Elf_(Xword) dt_pltrelsz;
+	Elf_(Addr) dt_pltgot;
+	Elf_(Addr) dt_hash;
+	Elf_(Addr) dt_strtab;
+	Elf_(Addr) dt_symtab;
+	Elf_(Addr) dt_rela;
+	Elf_(Xword) dt_relasz;
+	Elf_(Xword) dt_relaent;
+	Elf_(Xword) dt_strsz;
+	Elf_(Xword) dt_syment;
+	Elf_(Addr) dt_fini;
+	Elf_(Addr) dt_rel;
+	Elf_(Xword) dt_relsz;
+	Elf_(Xword) dt_relent;
+	Elf_(Xword) dt_pltrel;
+	Elf_(Addr) dt_jmprel;
+	Elf_(Addr) dt_mips_pltgot;
+	bool dt_bind_now;
+	Elf_(Xword) dt_flags;
+	Elf_(Xword) dt_flags_1;
+	Elf_(Xword) dt_rpath;
+	Elf_(Xword) dt_runpath;
+	RVector dt_needed;
+} RBinElfDynamicInfo;
+
 typedef struct r_bin_elf_lib_t {
 	char name[ELF_STRING_LENGTH];
 	int last;
@@ -79,21 +122,18 @@ typedef struct r_bin_elf_lib_t {
 
 struct Elf_(r_bin_elf_obj_t) {
 	Elf_(Ehdr) ehdr;
-	Elf_(Phdr)* phdr;
-	Elf_(Shdr)* shdr;
+	Elf_(Phdr) *phdr;
+	Elf_(Shdr) *shdr;
 
 	Elf_(Shdr) *strtab_section;
 	ut64 strtab_size;
-	char* strtab;
+	char *strtab;
 
 	Elf_(Shdr) *shstrtab_section;
 	ut64 shstrtab_size;
-	char* shstrtab;
+	char *shstrtab;
 
-	Elf_(Dyn) *dyn_buf;
-	int dyn_entries;
-	int is_rela;
-	ut32 reloc_num;
+	RBinElfDynamicInfo dyn_info;
 
 	ut64 version_info[DT_VERSIONTAGNUM];
 
@@ -118,6 +158,8 @@ struct Elf_(r_bin_elf_obj_t) {
 	RBinElfSection *g_sections;
 	RBinElfSymbol *g_symbols;
 	RBinElfSymbol *g_imports;
+	RBinElfReloc *g_relocs;
+	ut32 g_reloc_num;
 	RBinElfSymbol *phdr_symbols;
 	RBinElfSymbol *phdr_imports;
 	HtUP *rel_cache;
@@ -143,6 +185,9 @@ bool Elf_(r_bin_elf_is_static)(struct Elf_(r_bin_elf_obj_t) *bin);
 char* Elf_(r_bin_elf_get_data_encoding)(struct Elf_(r_bin_elf_obj_t) *bin);
 char* Elf_(r_bin_elf_get_arch)(struct Elf_(r_bin_elf_obj_t) *bin);
 char* Elf_(r_bin_elf_get_machine_name)(struct Elf_(r_bin_elf_obj_t) *bin);
+char* Elf_(r_bin_elf_get_head_flag)(ELFOBJ *bin); //yin
+char* Elf_(r_bin_elf_get_abi)(ELFOBJ *bin);
+char* Elf_(r_bin_elf_get_cpu)(ELFOBJ *bin); 
 char* Elf_(r_bin_elf_get_file_type)(struct Elf_(r_bin_elf_obj_t) *bin);
 char* Elf_(r_bin_elf_get_elf_class)(struct Elf_(r_bin_elf_obj_t) *bin);
 int Elf_(r_bin_elf_get_bits)(struct Elf_(r_bin_elf_obj_t) *bin);
@@ -165,6 +210,8 @@ bool Elf_(r_bin_elf_section_perms)(RBinFile *bf, const char *name, int perms);
 bool Elf_(r_bin_elf_entry_write)(RBinFile *bf, ut64 addr);
 bool Elf_(r_bin_elf_del_rpath)(RBinFile *bf);
 
+ut64 Elf_(r_bin_elf_get_phnum)(ELFOBJ *bin);
+bool Elf_(r_bin_elf_is_executable)(ELFOBJ *bin);
 int Elf_(r_bin_elf_has_relro)(struct Elf_(r_bin_elf_obj_t) *bin);
 int Elf_(r_bin_elf_has_nx)(struct Elf_(r_bin_elf_obj_t) *bin);
 ut8 *Elf_(r_bin_elf_grab_regstate)(struct Elf_(r_bin_elf_obj_t) *bin, int *len);
