@@ -76,7 +76,8 @@ enum HeaderFileType {
 	MH_BUNDLE      = 0x8u,
 	MH_DYLIB_STUB  = 0x9u,
 	MH_DSYM        = 0xAu,
-	MH_KEXT_BUNDLE = 0xBu
+	MH_KEXT_BUNDLE = 0xBu,
+	MH_FILESET     = 0xCu
 };
 
 enum {
@@ -167,7 +168,10 @@ enum LoadCommandType {
 	LC_VERSION_MIN_TVOS     = 0x0000002Fu,
 	LC_VERSION_MIN_WATCHOS  = 0x00000030u,
 	LC_NOTE                 = 0x00000031u,
-	LC_BUILD_VERSION        = 0x00000032u
+	LC_BUILD_VERSION        = 0x00000032u,
+	LC_DYLD_EXPORTS_TRIE    = 0x80000033u,
+	LC_DYLD_CHAINED_FIXUPS  = 0x80000034u,
+	LC_KEXT  = 0x80000035u, /* TODO: get the right name */
 /*
 Load command 9
        cmd LC_BUILD_VERSION
@@ -1435,4 +1439,109 @@ sizeof(struct x86_exception_state_t) / sizeof(uint32_t);
 #define EXPORT_SYMBOL_FLAGS_REEXPORT 0x08
 #define EXPORT_SYMBOL_FLAGS_STUB_AND_RESOLVER 0x10
 
+struct dyld_chained_fixups_header
+{
+	uint32_t fixups_version;
+	uint32_t starts_offset;
+	uint32_t imports_offset;
+	uint32_t symbols_offset;
+	uint32_t imports_count;
+	uint32_t imports_format;
+	uint32_t symbols_format;
+};
+
+struct dyld_chained_starts_in_image {
+	uint32_t seg_count;
+};
+
+struct dyld_chained_starts_in_segment {
+	uint32_t size;
+	uint16_t page_size;
+	uint16_t pointer_format;
+	uint64_t segment_offset;
+	uint32_t max_valid_pointer;
+	uint16_t page_count;
+};
+
+struct r_dyld_chained_starts_in_segment {
+	uint32_t size;
+	uint16_t page_size;
+	uint16_t pointer_format;
+	uint64_t segment_offset;
+	uint32_t max_valid_pointer;
+	uint16_t page_count;
+	ut16 * page_start;
+};
+
+enum {
+	DYLD_CHAINED_PTR_START_NONE   = 0xFFFF,
+	DYLD_CHAINED_PTR_START_MULTI  = 0x8000,
+	DYLD_CHAINED_PTR_START_LAST   = 0x8000,
+};
+
+enum {
+	DYLD_CHAINED_PTR_ARM64E      = 1,
+	DYLD_CHAINED_PTR_64          = 2,
+	DYLD_CHAINED_PTR_32          = 3,
+	DYLD_CHAINED_PTR_32_CACHE    = 4,
+	DYLD_CHAINED_PTR_32_FIRMWARE = 5,
+	DYLD_CHAINED_PTR_ARM64E_KERNEL = 7,
+	DYLD_CHAINED_PTR_64_KERNEL_CACHE = 8,
+};
+
+struct dyld_chained_ptr_arm64e_rebase {
+	uint64_t target : 43,
+		high8 : 8,
+		next : 11,
+		bind : 1, // == 0
+		auth : 1; // == 0
+};
+
+struct dyld_chained_ptr_arm64e_bind {
+	uint64_t ordinal : 16,
+		zero : 16,
+		addend : 19,
+		next : 11,
+		bind : 1, // == 1
+		auth : 1; // == 0
+};
+
+struct dyld_chained_ptr_arm64e_auth_rebase {
+	uint64_t target : 32,
+		diversity : 16,
+		addrDiv : 1,
+		key : 2,
+		next : 11,
+		bind : 1, // == 0
+		auth : 1; // == 1
+};
+
+struct dyld_chained_ptr_arm64e_auth_bind {
+	uint64_t ordinal : 16,
+		zero : 16,
+		diversity : 16,
+		addrDiv : 1,
+		key : 2,
+		next : 11,
+		bind : 1, // == 1
+		auth : 1; // == 1
+};
+
+/* WARNING: this is guesswork based on trial and error */
+struct dyld_chained_ptr_arm64e_cache_rebase {
+	uint64_t target : 43,
+		high8 : 8,
+		next : 12,
+		auth : 1; // == 0
+};
+
+struct dyld_chained_ptr_arm64e_cache_auth_rebase {
+	uint64_t target : 32,
+		diversity : 16,
+		addrDiv : 1,
+		key : 2,
+		next : 12,
+		auth : 1; // == 1
+};
+
 #endif
