@@ -45,19 +45,15 @@ static int cmp_sections(const void *a, const void *b) {
 	return s_a->vaddr - s_b->vaddr;
 }
 
-static RBinSection *r_bin_mz_init_section(const struct r_bin_mz_obj_t *bin,
-					  ut64 laddr) {
-	RBinSection *section;
-
-	section = R_NEW0 (RBinSection);
+static RBinSection *r_bin_mz_init_section(const struct r_bin_mz_obj_t *bin, ut64 laddr) {
+	RBinSection *section = R_NEW0 (RBinSection);
 	if (section) {
 		section->vaddr = laddr;
 	}
-
 	return section;
 }
 
-RList *r_bin_mz_get_segments (const struct r_bin_mz_obj_t *bin) {
+RList *r_bin_mz_get_segments(const struct r_bin_mz_obj_t *bin) {
 	RList *seg_list;
 	RListIter *iter;
 	RBinSection *section;
@@ -88,8 +84,7 @@ RList *r_bin_mz_get_segments (const struct r_bin_mz_obj_t *bin) {
 	for (i = 0; i < num_relocs; i++) {
 		RBinSection c;
 		ut64 laddr, paddr, section_laddr;
-		ut16 *curr_seg;
-		int left;
+		ut16 curr_seg;
 
 		laddr = r_bin_mz_va_to_la (relocs[i].segment, relocs[i].offset);
 		if ((laddr + 2) >= bin->load_module_size) {
@@ -97,12 +92,12 @@ RList *r_bin_mz_get_segments (const struct r_bin_mz_obj_t *bin) {
 		}
 
 		paddr = r_bin_mz_la_to_pa (bin, laddr);
-		curr_seg = (ut16 *)r_buf_get_at (bin->b, paddr, &left);
-		if (left < 2) {
+		if (r_buf_size (bin->b) < paddr + 2) {
 			continue;
 		}
+		curr_seg = r_buf_read_le16_at (bin->b, paddr);
 
-		section_laddr = r_bin_mz_va_to_la (r_read_le16 (curr_seg), 0);
+		section_laddr = r_bin_mz_va_to_la (curr_seg, 0);
 		if (section_laddr > bin->load_module_size) {
 			continue;
 		}
@@ -212,14 +207,25 @@ static int r_bin_mz_init_hdr(struct r_bin_mz_obj_t *bin) {
 	if (mz->blocks_in_file < 1) {
 		return false;
 	}
-	dos_file_size = ((mz->blocks_in_file - 1) << 9) +
-		mz->bytes_in_last_block;
+	if (mz->bytes_in_last_block == 0) {
+		// last block is full
+		dos_file_size = mz->blocks_in_file << 9;
+	} else {
+		// last block is partially full
+		dos_file_size = ((mz->blocks_in_file - 1) << 9) +
+			mz->bytes_in_last_block;
+	}
 
 	bin->dos_file_size = dos_file_size;
 	if (dos_file_size > bin->size) {
 		return false;
 	}
-	bin->load_module_size = dos_file_size - (mz->header_paragraphs << 4);
+	eprintf ("ii %d %d\n", dos_file_size , (mz->header_paragraphs << 4));
+	if (dos_file_size < (mz->header_paragraphs << 4)) {
+		bin->load_module_size = dos_file_size;
+	} else {
+		bin->load_module_size = dos_file_size - (mz->header_paragraphs << 4);
+	}
 	relocations_size = mz->num_relocs * sizeof (MZ_image_relocation_entry);
 	if ((mz->reloc_table_offset + relocations_size) > bin->size) {
 		return false;
@@ -269,7 +275,7 @@ static int r_bin_mz_init_hdr(struct r_bin_mz_obj_t *bin) {
 	return true;
 }
 
-static int r_bin_mz_init(struct r_bin_mz_obj_t *bin) {
+static bool r_bin_mz_init(struct r_bin_mz_obj_t *bin) {
 	bin->dos_header = NULL;
 	bin->dos_extended_header = NULL;
 	bin->relocation_entries = NULL;
@@ -282,13 +288,15 @@ static int r_bin_mz_init(struct r_bin_mz_obj_t *bin) {
 }
 
 struct r_bin_mz_obj_t *r_bin_mz_new (const char *file) {
-	const ut8 *buf;
 	struct r_bin_mz_obj_t *bin = R_NEW0 (struct r_bin_mz_obj_t);
 	if (!bin) {
 		return NULL;
 	}
 	bin->file = file;
-	if (!(buf = (ut8 *)r_file_slurp (file, &bin->size))) {
+	size_t binsz;
+	ut8 *buf = (ut8*)r_file_slurp (file, &binsz);
+	bin->size = binsz;
+	if (!buf) {
 		return r_bin_mz_free (bin);
 	}
 	bin->b = r_buf_new ();
@@ -303,18 +311,17 @@ struct r_bin_mz_obj_t *r_bin_mz_new (const char *file) {
 	return bin;
 }
 
-struct r_bin_mz_obj_t *r_bin_mz_new_buf (const struct r_buf_t *buf) {
+struct r_bin_mz_obj_t *r_bin_mz_new_buf(RBuffer *buf) {
 	struct r_bin_mz_obj_t *bin = R_NEW0 (struct r_bin_mz_obj_t);
 	if (!bin) {
 		return NULL;
 	}
-	bin->b = r_buf_new ();
-	bin->size = buf->length;
-	if (!r_buf_set_bytes (bin->b, buf->buf, bin->size)) {
+	bin->b = r_buf_new_with_buf (buf);
+	if (!bin->b) {
 		return r_bin_mz_free (bin);
 	}
-
-	return r_bin_mz_init (bin) ? bin : r_bin_mz_free (bin);
+	bin->size = r_buf_size (buf);
+	return r_bin_mz_init (bin)? bin: r_bin_mz_free (bin);
 }
 
 RBinAddr *r_bin_mz_get_main_vaddr (struct r_bin_mz_obj_t *bin) {
