@@ -1,10 +1,23 @@
-/* radare - LGPL - Copyright 2015-2018 - inisider */
+/* radare - LGPL - Copyright 2015-2020 - inisider, pancake */
 
+#include "../../i/private.h"
 #include "mach0_classes.h"
 
 #define RO_META (1 << 0)
 #define MAX_CLASS_NAME_LEN 256
 
+#ifdef R_BIN_MACH064
+#define FAST_DATA_MASK 0x00007ffffffffff8UL
+#else
+#define FAST_DATA_MASK 0xfffffffcUL
+#endif
+
+#define METHOD_LIST_FLAG_IS_SMALL 0x80000000
+#define METHOD_LIST_FLAG_IS_PREOPT 0x3
+#define METHOD_LIST_ENTSIZE_FLAG_MASK 0xffff0003
+
+#define RO_DATA_PTR(x) ((x) & FAST_DATA_MASK)
+
 struct MACH0_(SMethodList) {
 	ut32 entsize;
 	ut32 count;
@@ -82,13 +95,27 @@ struct MACH0_(SObjcPropertyList) {
 	/* struct SObjcProperty first;  These structures follow inline */
 };
 
-static mach0_ut get_pointer(mach0_ut p, ut32 *offset, ut32 *left, RBinFile *bf);
+struct MACH0_(SCategory) {
+	mach0_ut name;
+	mach0_ut targetClass;
+	mach0_ut instanceMethods;
+	mach0_ut classMethods;
+	mach0_ut protocols;
+	mach0_ut properties;
+};
+
+static mach0_ut va2pa(mach0_ut p, ut32 *offset, ut32 *left, RBinFile *bf);
 static void copy_sym_name_with_namespace(char *class_name, char *read_name, RBinSymbol *sym);
 static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass);
 static void get_objc_property_list(mach0_ut p, RBinFile *bf, RBinClass *klass);
-static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinClass *klass, bool is_static);
-static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass);
-static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinClass *klass);
+static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinClass *klass, bool is_static, objc_cache_opt_info *oi);
+static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass, objc_cache_opt_info *oi);
+static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinClass *klass, objc_cache_opt_info *oi);
+static RList *MACH0_(parse_categories)(RBinFile *bf, RSkipList *relocs, objc_cache_opt_info *oi);
+static bool read_ptr_pa(RBinFile *bf, ut64 paddr, mach0_ut *out);
+static bool read_ptr_va(RBinFile *bf, ut64 vaddr, mach0_ut *out);
+static char *read_str(RBinFile *bf, mach0_ut p, ut32 *offset, ut32 *left);
+static char *get_class_name(mach0_ut p, RBinFile *bf);
 static bool is_thumb(RBinFile *bf) {
 	struct MACH0_(obj_t) *bin = (struct MACH0_(obj_t) *)bf->o->bin_obj;
 	if (bin->hdr.cputype == 12) {
@@ -99,20 +126,16 @@ static bool is_thumb(RBinFile *bf) {
 	return false;
 }
 
-static mach0_ut get_pointer(mach0_ut p, ut32 *offset, ut32 *left, RBinFile *bf) {
+static mach0_ut va2pa(mach0_ut p, ut32 *offset, ut32 *left, RBinFile *bf) {
+	r_return_val_if_fail (bf && bf->o && bf->o->bin_obj, 0);
+
 	mach0_ut r;
 	mach0_ut addr;
 
 	static RList *sctns = NULL;
 	RListIter *iter = NULL;
 	RBinSection *s = NULL;
-	RBinObject *obj = bf ? bf->o : NULL;
-	if (!obj) {
-		return 0;
-	}
-	if (!obj->bin_obj) {
-		return 0;
-	}
+	RBinObject *obj = bf->o;
 
 	struct MACH0_(obj_t) *bin = (struct MACH0_(obj_t)*) obj->bin_obj;
 	if (bin->va2pa) {
@@ -160,17 +183,21 @@ static void copy_sym_name_with_namespace(char *class_name, char *read_name, RBin
 	sym->name = strdup (read_name);
 }
 
+static int sort_by_offset(const void *_a , const void *_b) {
+	RBinField *a = (RBinField*)_a;
+	RBinField *b = (RBinField*)_b;
+	return a->offset - b->offset;
+}
+
 static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
-	struct MACH0_(SIVarList) il;
+	struct MACH0_(SIVarList) il = {0};
 	struct MACH0_(SIVar) i;
 	mach0_ut r;
 	ut32 offset, left, j;
-	char *name = NULL;
-	char *type = NULL;
 
 	int len;
 	bool bigendian;
-	mach0_ut ivar_offset_p, ivar_offset;
+	mach0_ut ivar_offset;
 	RBinField *field = NULL;
 	ut8 sivarlist[sizeof (struct MACH0_(SIVarList))] = {0};
 	ut8 sivar[sizeof (struct MACH0_(SIVar))] = {0};
@@ -181,10 +208,9 @@ static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 		return;
 	}
 	bigendian = bf->o->info->big_endian;
-	if (!(r = get_pointer (p, &offset, &left, bf))) {
+	if (!(r = va2pa (p, &offset, &left, bf))) {
 		return;
 	}
-	memset (&il, '\0', sizeof (struct MACH0_(SIVarList)));
 	if (r + left < r || r + sizeof (struct MACH0_(SIVarList)) < r) {
 		return;
 	}
@@ -208,16 +234,13 @@ static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 	il.count = r_read_ble (&sivarlist[4], bigendian, 32);
 	p += sizeof (struct MACH0_(SIVarList));
 	offset += sizeof (struct MACH0_(SIVarList));
+
 	for (j = 0; j < il.count; j++) {
-		r = get_pointer (p, &offset, &left, bf);
+		r = va2pa (p, &offset, &left, bf);
 		if (!r) {
 			return;
 		}
-		if (!(field = R_NEW0 (RBinField))) {
-			// retain just for debug
-			// eprintf ("RBinField allocation error\n");
-			return;
-		}
+		field = R_NEW0 (RBinField);
 		memset (&i, '\0', sizeof (struct MACH0_(SIVar)));
 		if (r + left < r || r + sizeof (struct MACH0_(SIVar)) < r) {
 			goto error;
@@ -238,44 +261,39 @@ static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 				goto error;
 			}
 		}
-		switch (sizeof (mach0_ut)) {
-		case 4:
-			i.offset = r_read_ble (&sivar[0], bigendian, 32);
-			i.name = r_read_ble (&sivar[4], bigendian, 32);
-			i.type = r_read_ble (&sivar[8], bigendian, 32);
-			i.alignment = r_read_ble (&sivar[12], bigendian, 32);
-			i.size = r_read_ble (&sivar[16], bigendian, 32);
-			break;
-		case 8:
-			i.offset = r_read_ble (&sivar[0], bigendian, 64);
-			i.name = r_read_ble (&sivar[8], bigendian, 64);
-			i.type = r_read_ble (&sivar[16], bigendian, 64);
-			i.alignment = r_read_ble (&sivar[24], bigendian, 32);
-			i.size = r_read_ble (&sivar[28], bigendian, 32);
-			break;
-		}
+#if R_BIN_MACH064
+		i.offset = r_read_ble (&sivar[0], bigendian, 64);
+		i.name = r_read_ble (&sivar[8], bigendian, 64);
+		i.type = r_read_ble (&sivar[16], bigendian, 64);
+		i.alignment = r_read_ble (&sivar[24], bigendian, 32);
+		i.size = r_read_ble (&sivar[28], bigendian, 32);
+#else
+		i.offset = r_read_ble (&sivar[0], bigendian, 32);
+		i.name = r_read_ble (&sivar[4], bigendian, 32);
+		i.type = r_read_ble (&sivar[8], bigendian, 32);
+		i.alignment = r_read_ble (&sivar[12], bigendian, 32);
+		i.size = r_read_ble (&sivar[16], bigendian, 32);
+#endif
+		field->vaddr = i.offset;
+		mach0_ut offset_at = va2pa (i.offset, NULL, &left, bf);
 
-
-		ivar_offset_p = get_pointer (i.offset, NULL, &left, bf);
-
-		if (ivar_offset_p > bf->size) {
+		if (offset_at > bf->size) {
 			goto error;
 		}
-		if (ivar_offset_p + sizeof (ivar_offset) > bf->size) {
+		if (offset_at + sizeof (ivar_offset) > bf->size) {
 			goto error;
 		}
-		if (ivar_offset_p != 0 && left >= sizeof (mach0_ut)) {
-			len = r_buf_read_at (bf->buf, ivar_offset_p, offs, sizeof (mach0_ut));
+		if (offset_at != 0 && left >= sizeof (mach0_ut)) {
+			len = r_buf_read_at (bf->buf, offset_at, offs, sizeof (mach0_ut));
 			if (len != sizeof (mach0_ut)) {
 				eprintf ("Error reading\n");
 				goto error;
 			}
 			ivar_offset = r_read_ble (offs, bigendian, 8 * sizeof (mach0_ut));
-			field->vaddr = ivar_offset;
+			field->offset = ivar_offset;
 		}
-
-		r = get_pointer (i.name, NULL, &left, bf);
-		if (r != 0) {
+		r = va2pa (i.name, NULL, &left, bf);
+		if (r) {
 			struct MACH0_(obj_t) *bin = (struct MACH0_(obj_t) *)bf->o->bin_obj;
 			if (r + left < r) {
 				goto error;
@@ -283,6 +301,7 @@ static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 			if (r > bf->size || r + left > bf->size) {
 				goto error;
 			}
+			char *name;
 			if (bin->has_crypto) {
 				name = strdup ("some_encrypted_data");
 				left = strlen (name) + 1;
@@ -297,12 +316,13 @@ static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 				}
 				name[name_len] = 0;
 			}
+			// XXX the field name shouldnt contain the class name
 			field->name = r_str_newf ("%s::%s%s", klass->name, "(ivar)", name);
 			R_FREE (name);
 		}
 
-		r = get_pointer (i.type, NULL, &left, bf);
-		if (r != 0) {
+		r = va2pa (i.type, NULL, &left, bf);
+		if (r) {
 			struct MACH0_(obj_t) *bin = (struct MACH0_(obj_t) *) bf->o->bin_obj;
 			int is_crypted = bin->has_crypto;
 			if (r + left < r) {
@@ -311,25 +331,42 @@ static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 			if (r > bf->size || r + left > bf->size) {
 				goto error;
 			}
+			char *type = NULL;
 			if (is_crypted == 1) {
 				type = strdup ("some_encrypted_data");
 			// 	left = strlen (name) + 1;
 			} else {
 				int type_len = R_MIN (MAX_CLASS_NAME_LEN, left);
 				type = calloc (1, type_len + 1);
-				r_buf_read_at (bf->buf, r, (ut8 *)type, type_len);
-				type[type_len] = 0;
+				if (type) {
+					r_buf_read_at (bf->buf, r, (ut8 *)type, type_len);
+					type[type_len] = 0;
+				}
 			}
-      			field->type = strdup (type);
-			R_FREE (type);
+			if (type) {
+				field->type = type;
+				type = NULL;
+			} else {
+				field->type = NULL;
+			}
+			r_list_append (klass->fields, field);
+		} else {
+			goto error;
 		}
-
-		r_list_append (klass->fields, field);
 		p += sizeof (struct MACH0_(SIVar));
 		offset += sizeof (struct MACH0_(SIVar));
 	}
+	if (!r_list_empty (klass->fields)) {
+		r_list_sort (klass->fields, sort_by_offset);
+	}
+	RBinField *isa_field = R_NEW0 (RBinField);
+	isa_field->name = strdup ("isa");
+	isa_field->size = sizeof (mach0_ut);
+	isa_field->type = strdup ("struct objc_class *");
+	isa_field->vaddr = 0;
+	isa_field->offset = 0;
+	r_list_prepend (klass->fields, isa_field);
 	return;
-
 error:
 	r_bin_field_free (field);
 }
@@ -352,7 +389,7 @@ static void get_objc_property_list(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 		return;
 	}
 	bigendian = bf->o->info->big_endian;
-	r = get_pointer (p, &offset, &left, bf);
+	r = va2pa (p, &offset, &left, bf);
 	if (!r) {
 		return;
 	}
@@ -383,7 +420,7 @@ static void get_objc_property_list(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 	p += sizeof (struct MACH0_(SObjcPropertyList));
 	offset += sizeof (struct MACH0_(SObjcPropertyList));
 	for (j = 0; j < opl.count; j++) {
-		r = get_pointer (p, &offset, &left, bf);
+		r = va2pa (p, &offset, &left, bf);
 		if (!r) {
 			return;
 		}
@@ -418,7 +455,7 @@ static void get_objc_property_list(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 		}
 		op.name = r_read_ble (&sop[0], bigendian, 8 * sizeof (mach0_ut));
 		op.attributes = r_read_ble (&sop[sizeof (mach0_ut)], bigendian, 8 * sizeof (mach0_ut));
-		r = get_pointer (op.name, NULL, &left, bf);
+		r = va2pa (op.name, NULL, &left, bf);
 		if (r) {
 			struct MACH0_(obj_t) *bin = (struct MACH0_(obj_t) *)bf->o->bin_obj;
 			if (r > bf->size || r + left > bf->size) {
@@ -445,7 +482,7 @@ static void get_objc_property_list(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 			R_FREE (name);
 		}
 #if 0
-		r = get_pointer (op.attributes, NULL, &left, bf);
+		r = va2pa (op.attributes, NULL, &left, bf);
 		if (r != 0) {
 			struct MACH0_(obj_t) *bin = (struct MACH0_(obj_t) *) bf->o->bin_obj;
 			int is_crypted = bin->has_crypto;
@@ -478,9 +515,8 @@ error:
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinClass *klass, bool is_static) {
+static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinClass *klass, bool is_static, objc_cache_opt_info *oi) {
 	struct MACH0_(SMethodList) ml;
-	struct MACH0_(SMethod) m;
 	mach0_ut r;
 	ut32 offset, left, i;
 	char *name = NULL;
@@ -496,7 +532,7 @@ static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinCl
 		return;
 	}
 	bigendian = bf->o->info->big_endian;
-	r = get_pointer (p, &offset, &left, bf);
+	r = va2pa (p, &offset, &left, bf);
 	if (!r) {
 		return;
 	}
@@ -505,7 +541,7 @@ static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinCl
 	if (r + left < r || r + sizeof (struct MACH0_(SMethodList)) < r) {
 		return;
 	}
-	if (r > bf->size || r + left > bf->size) {
+	if (r > bf->size) {
 		return;
 	}
 	if (r + sizeof (struct MACH0_(SMethodList)) > bf->size) {
@@ -523,12 +559,25 @@ static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinCl
 	}
 	ml.entsize = r_read_ble (&sml[0], bigendian, 32);
 	ml.count = r_read_ble (&sml[4], bigendian, 32);
+	if (ml.count < 1 || ml.count > ST32_MAX) {
+		return;
+	}
+	if (r + (ml.count * (ml.entsize & ~METHOD_LIST_ENTSIZE_FLAG_MASK)) > bf->size) {
+		return;
+	}
+
+	bool is_small = (ml.entsize & METHOD_LIST_FLAG_IS_SMALL) != 0;
+	ut8 mlflags = ml.entsize & 0x3;
 
 	p += sizeof (struct MACH0_(SMethodList));
 	offset += sizeof (struct MACH0_(SMethodList));
+
+	size_t read_size = is_small ? 3 * sizeof (ut32) :
+			sizeof (struct MACH0_(SMethod));
+
 	for (i = 0; i < ml.count; i++) {
-		r = get_pointer (p, &offset, &left, bf);
-		if (!r) {
+		r = va2pa (p, &offset, &left, bf);
+		if (!r || r == -1) {
 			return;
 		}
 
@@ -537,37 +586,65 @@ static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinCl
 			// eprintf ("RBinClass allocation error\n");
 			return;
 		}
+		struct MACH0_(SMethod) m;
 		memset (&m, '\0', sizeof (struct MACH0_(SMethod)));
-		if (r + left < r || r + sizeof (struct MACH0_(SMethod)) < r) {
+		if (r + left < r || r + read_size < r) {
 			goto error;
 		}
-		if (r > bf->size || r + left > bf->size) {
+		if (r > bf->size) {
 			goto error;
 		}
-		if (r + sizeof (struct MACH0_(SMethod)) > bf->size) {
+		if (r + read_size > bf->size) {
 			goto error;
 		}
-		if (left < sizeof (struct MACH0_(SMethod))) {
+		if (left < read_size) {
 			if (r_buf_read_at (bf->buf, r, sm, left) != left) {
 				goto error;
 			}
 		} else {
-			len = r_buf_read_at (bf->buf, r, sm, sizeof (struct MACH0_(SMethod)));
-			if (len != sizeof (struct MACH0_(SMethod))) {
+			len = r_buf_read_at (bf->buf, r, sm, read_size);
+			if (len != read_size) {
 				goto error;
 			}
 		}
-		m.name = r_read_ble (&sm[0], bigendian, 8 * sizeof (mach0_ut));
-		m.types = r_read_ble (&sm[sizeof (mach0_ut)], bigendian, 8 * sizeof (mach0_ut));
-		m.imp = r_read_ble (&sm[2 * sizeof (mach0_ut)], bigendian, 8 * sizeof (mach0_ut));
+		if (!is_small) {
+			m.name = r_read_ble (&sm[0], bigendian, 8 * sizeof (mach0_ut));
+			m.types = r_read_ble (&sm[sizeof (mach0_ut)], bigendian, 8 * sizeof (mach0_ut));
+			m.imp = r_read_ble (&sm[2 * sizeof (mach0_ut)], bigendian, 8 * sizeof (mach0_ut));
+		} else {
+			st64 name_offset = (st32) r_read_ble (&sm[0], bigendian, 8 * sizeof (ut32));
+			mach0_ut name;
+			if (oi && oi->sel_string_base) {
+				name = oi->sel_string_base + name_offset;
+			} else {
+				name = p + name_offset;
+			}
+			if (mlflags != METHOD_LIST_FLAG_IS_PREOPT) {
+				r = va2pa (name, &offset, &left, bf);
+				if (!r) {
+					goto error;
+				}
+				ut8 tmp[8];
+				if (r_buf_read_at (bf->buf, r, tmp, sizeof (mach0_ut)) != sizeof (mach0_ut)) {
+					goto error;
+				}
+				m.name = r_read_ble (tmp, bigendian, 8 * sizeof (mach0_ut));
+			} else {
+				m.name = name;
+			}
+			st64 types_offset = (st32) r_read_ble (&sm[sizeof (ut32)], bigendian, 8 * sizeof (ut32));
+			m.types = p + types_offset + 4;
+			st64 imp_offset = (st32) r_read_ble (&sm[2 * sizeof (ut32)], bigendian, 8 * sizeof (ut32));
+			m.imp = p + imp_offset + 8;
+		}
 
-		r = get_pointer (m.name, NULL, &left, bf);
+		r = va2pa (m.name, NULL, &left, bf);
 		if (r) {
 			struct MACH0_(obj_t) *bin = (struct MACH0_(obj_t) *)bf->o->bin_obj;
 			if (r + left < r) {
 				goto error;
 			}
-			if (r > bf->size || r + left > bf->size) {
+			if (r > bf->size || r + MAX_CLASS_NAME_LEN > bf->size) {
 				goto error;
 			}
 			if (bin->has_crypto) {
@@ -586,10 +663,12 @@ static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinCl
 			R_FREE (name);
 		}
 
-		r = get_pointer (m.types, NULL, &left, bf);
+		r = va2pa (m.types, NULL, &left, bf);
 		if (r != 0) {
 			struct MACH0_(obj_t) *bin = (struct MACH0_(obj_t) *)bf->o->bin_obj;
-
+			if (r + left > bf->size) {
+				left = bf->size - r;
+			}
 			if (r + left < r || r > bf->size || r + left > bf->size) {
 				goto error;
 			}
@@ -613,7 +692,11 @@ static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinCl
 		}
 
 		method->vaddr = m.imp;
-		method->type = is_static ? R_BIN_TYPE_FUNC_STR : "METH";
+		if (!method->vaddr) {
+			R_FREE (method);
+			goto next;
+		}
+		method->type = is_static? R_BIN_TYPE_FUNC_STR: R_BIN_TYPE_METH_STR;
 		if (is_static) {
 			method->method_flags |= R_BIN_METH_CLASS;
 		}
@@ -625,8 +708,9 @@ static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinCl
 			}
 		}
 		r_list_append (klass->methods, method);
-		p += sizeof (struct MACH0_(SMethod));
-		offset += sizeof (struct MACH0_(SMethod));
+next:
+		p += read_size;
+		offset += read_size;
 	}
 	return;
 error:
@@ -636,7 +720,7 @@ error:
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
+static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass, objc_cache_opt_info *oi) {
 	struct MACH0_(SProtocolList) pl = { 0 };
 	struct MACH0_(SProtocol) pc;
 	char *class_name = NULL;
@@ -653,7 +737,7 @@ static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 		return;
 	}
 	bigendian = bf->o->info->big_endian;
-	if (!(r = get_pointer (p, &offset, &left, bf))) {
+	if (!(r = va2pa (p, &offset, &left, bf))) {
 		return;
 	}
 	if (r + left < r || r + sizeof (struct MACH0_(SProtocolList)) < r) {
@@ -680,7 +764,7 @@ static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 	p += sizeof (struct MACH0_(SProtocolList));
 	offset += sizeof (struct MACH0_(SProtocolList));
 	for (i = 0; i < pl.count; i++) {
-		if (!(r = get_pointer (p, &offset, &left, bf))) {
+		if (!(r = va2pa (p, &offset, &left, bf))) {
 			return;
 		}
 		if (r + left < r || r + sizeof (mach0_ut) < r) {
@@ -703,7 +787,7 @@ static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 			}
 		}
 		q = r_read_ble (&sptr[0], bigendian, 8 * sizeof (mach0_ut));
-		if (!(r = get_pointer (q, &offset, &left, bf))) {
+		if (!(r = va2pa (q, &offset, &left, bf))) {
 			return;
 		}
 		memset (&pc, '\0', sizeof (struct MACH0_(SProtocol)));
@@ -743,7 +827,7 @@ static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 		pc.optionalClassMethods = r_read_ble (&spc[j], bigendian, 8 * sizeof (mach0_ut));
 		j += sizeof (mach0_ut);
 		pc.instanceProperties = r_read_ble (&spc[j], bigendian, 8 * sizeof (mach0_ut));
-		r = get_pointer (pc.name, NULL, &left, bf);
+		r = va2pa (pc.name, NULL, &left, bf);
 		if (r != 0) {
 			char *name = NULL;
 			struct MACH0_(obj_t) *bin = (struct MACH0_(obj_t) *)bf->o->bin_obj;
@@ -773,10 +857,10 @@ static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 		}
 
 		if (pc.instanceMethods > 0) {
-			get_method_list_t (pc.instanceMethods, bf, class_name, klass, false);
+			get_method_list_t (pc.instanceMethods, bf, class_name, klass, false, oi);
 		}
 		if (pc.classMethods > 0) {
-			get_method_list_t (pc.classMethods, bf, class_name, klass, true);
+			get_method_list_t (pc.classMethods, bf, class_name, klass, true, oi);
 		}
 		R_FREE (class_name);
 		p += sizeof (ut32);
@@ -785,7 +869,9 @@ static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 }
 
 static const char *skipnum(const char *s) {
-	while (IS_DIGIT (*s)) s++;
+	while (IS_DIGIT (*s)) {
+		s++;
+	}
 	return s;
 }
 
@@ -795,16 +881,26 @@ static char *demangle_classname(const char *s) {
 	const char *kstr;
 	char *ret, *klass, *module;
 	if (!strncmp (s, "_TtC", 4)) {
-		len = atoi (s + 4);
-		modlen = strlen (s + 4);
-		if (len >= modlen) {
+		int off = 4;
+		while (s[off] && (s[off] < '0' || s[off] > '9')) {
+			off++;
+		}
+		len = atoi (s + off);
+		modlen = strlen (s + off);
+		if (!len || len >= modlen) {
 			return strdup (s);
 		}
-		module = r_str_ndup (skipnum (s + 4), len);
-		kstr = skipnum (s + 4) + len;
+		module = r_str_ndup (skipnum (s + off), len);
+		int skip = (skipnum (s + off) - s) + len;
+		if (s[skip] == 'P') {
+			skip++;
+			len = atoi (s + skip);
+			skip = (skipnum (s + skip) - s) + len;
+		}
+		kstr = s + skip;
 		len = atoi (kstr);
 		modlen = strlen (kstr);
-		if (len >= modlen) {
+		if (!len || len >= modlen) {
 			free (module);
 			return strdup (s);
 		}
@@ -818,9 +914,103 @@ static char *demangle_classname(const char *s) {
 	return ret;
 }
 
+static char *get_class_name(mach0_ut p, RBinFile *bf) {
+	struct MACH0_(obj_t) *bin;
+	ut32 offset, left;
+	ut64 r;
+	int len;
+	bool bigendian;
+	ut8 sc[sizeof (mach0_ut)] = {0};
+	const ut32 ptr_size = sizeof (mach0_ut);
+
+	if (!bf || !bf->o || !bf->o->bin_obj || !bf->o->info) {
+		eprintf ("Invalid RBinFile pointer\n");
+		return NULL;
+	}
+	if (!p) {
+		return NULL;
+	}
+	bigendian = bf->o->info->big_endian;
+	bin = (struct MACH0_(obj_t) *)bf->o->bin_obj;
+
+	if (!(r = va2pa (p, &offset, &left, bf))) {
+		return NULL;
+	}
+	if ((r + left) < r || (r + sizeof (sc)) < r) {
+		return NULL;
+	}
+	if (r > bf->size) {
+		return NULL;
+	}
+	if (r + sizeof (sc) > bf->size) {
+		return NULL;
+	}
+	if (left < sizeof (sc)) {
+		return NULL;
+	}
+	len = r_buf_read_at (bf->buf, r + 4 * ptr_size, sc, sizeof (sc));
+	if (len != sizeof (sc)) {
+		return NULL;
+	}
+
+	ut64 rodata = r_read_ble (sc, bigendian, 8 * ptr_size);
+	if (!(r = va2pa (rodata, &offset, &left, bf))) {
+		return NULL;
+	}
+	if (r + left < r || r + sizeof (sc) < r) {
+		return NULL;
+	}
+	if (r > bf->size) {
+		return NULL;
+	}
+	if (r + sizeof (sc) > bf->size) {
+		return NULL;
+	}
+	if (left < sizeof (sc)) {
+		return NULL;
+	}
+
+#ifdef R_BIN_MACH064
+	len = r_buf_read_at (bf->buf, r + 4 * sizeof (ut32) + ptr_size, sc, sizeof (sc));
+#else
+	len = r_buf_read_at (bf->buf, r + 3 * sizeof (ut32) + ptr_size, sc, sizeof (sc));
+#endif
+	if (len != sizeof (sc)) {
+		return NULL;
+	}
+	ut64 name = r_read_ble (sc, bigendian, 8 * ptr_size);
+
+	if ((r = va2pa (name, NULL, &left, bf))) {
+		if (left < 1 || r + left < r) {
+			return NULL;
+		}
+		if (r > bf->size || r + MAX_CLASS_NAME_LEN > bf->size) {
+			return NULL;
+		}
+		if (bin->has_crypto) {
+			return strdup ("some_encrypted_data");
+		} else {
+			int name_len = R_MIN (MAX_CLASS_NAME_LEN, left);
+			char *name = malloc (name_len + 1);
+			if (name) {
+				int rc = r_buf_read_at (bf->buf, r, (ut8 *)name, name_len);
+				if (rc != name_len) {
+					rc = 0;
+				}
+				name[rc] = 0;
+				char * result = demangle_classname (name);
+				free (name);
+				return result;
+			}
+		}
+	}
+
+	return NULL;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
-static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinClass *klass) {
-	struct MACH0_(obj_t) * bin;
+static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinClass *klass, objc_cache_opt_info *oi) {
+	struct MACH0_(obj_t) *bin;
 	struct MACH0_(SClassRoT) cro = { 0 };
 	ut32 offset, left, i;
 	ut64 r, s;
@@ -834,7 +1024,7 @@ static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinCl
 	}
 	bigendian = bf->o->info->big_endian;
 	bin = (struct MACH0_(obj_t) *)bf->o->bin_obj;
-	if (!(r = get_pointer (p, &offset, &left, bf))) {
+	if (!(r = va2pa (p, &offset, &left, bf))) {
 		// eprintf ("No pointer\n");
 		return;
 	}
@@ -842,7 +1032,7 @@ static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinCl
 	if (r + left < r || r + sizeof (cro) < r) {
 		return;
 	}
-	if (r > bf->size || r + left > bf->size) {
+	if (r > bf->size || r + sizeof (cro) >= bf->size) {
 		return;
 	}
 	if (r + sizeof (cro) > bf->size) {
@@ -884,7 +1074,7 @@ static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinCl
 	cro.baseProperties = r_read_ble (&scro[i], bigendian, 8 * sizeof (mach0_ut));
 
 	s = r;
-	if ((r = get_pointer (cro.name, NULL, &left, bf))) {
+	if ((r = va2pa (cro.name, NULL, &left, bf))) {
 		if (left < 1 || r + left < r) {
 			return;
 		}
@@ -911,17 +1101,17 @@ static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinCl
 		sdb_num_set (bin->kv, sdb_fmt ("objc_class_%s.offset", klass->name), s, 0);
 	}
 #ifdef R_BIN_MACH064
-	sdb_set (bin->kv, sdb_fmt ("objc_class.format", 0), "lllll isa super cache vtable data", 0);
+	sdb_set (bin->kv, sdb_fmt ("objc_class.format"), "lllll isa super cache vtable data", 0);
 #else
-	sdb_set (bin->kv, sdb_fmt ("objc_class.format", 0), "xxxxx isa super cache vtable data", 0);
+	sdb_set (bin->kv, sdb_fmt ("objc_class.format"), "xxxxx isa super cache vtable data", 0);
 #endif
 
 	if (cro.baseMethods > 0) {
-		get_method_list_t (cro.baseMethods, bf, klass->name, klass, (cro.flags & RO_META) ? true : false);
+		get_method_list_t (cro.baseMethods, bf, klass->name, klass, (cro.flags & RO_META) ? true : false, oi);
 	}
 
 	if (cro.baseProtocols > 0) {
-		get_protocol_list_t (cro.baseProtocols, bf, klass);
+		get_protocol_list_t (cro.baseProtocols, bf, klass, oi);
 	}
 
 	if (cro.ivars > 0) {
@@ -937,12 +1127,12 @@ static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinCl
 	}
 }
 
-static mach0_ut get_isa_value() {
+static mach0_ut get_isa_value(void) {
 	// TODO: according to otool sources this is taken from relocs
 	return 0;
 }
 
-void MACH0_(get_class_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, bool dupe) {
+void MACH0_(get_class_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, bool dupe, RSkipList *relocs, objc_cache_opt_info *oi) {
 	struct MACH0_(SClass) c = { 0 };
 	const int size = sizeof (struct MACH0_(SClass));
 	mach0_ut r = 0;
@@ -957,13 +1147,13 @@ void MACH0_(get_class_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, bool dupe)
 		return;
 	}
 	bigendian = bf->o->info->big_endian;
-	if (!(r = get_pointer (p, &offset, &left, bf))) {
+	if (!(r = va2pa (p, &offset, &left, bf))) {
 		return;
 	}
 	if ((r + left) < r || (r + size) < r) {
 		return;
 	}
-	if (r > bf->size || r + left > bf->size) {
+	if (r > bf->size) {
 		return;
 	}
 	if (r + size > bf->size) {
@@ -990,7 +1180,23 @@ void MACH0_(get_class_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, bool dupe)
 	c.data = r_read_ble (&sc[i], bigendian, 8 * sizeof (mach0_ut));
 
 	klass->addr = c.isa;
-	get_class_ro_t (c.data & ~0x3, bf, &is_meta_class, klass);
+	if (c.superclass) {
+		klass->super = get_class_name (c.superclass, bf);
+	} else if (relocs) {
+		struct reloc_t reloc_at_class_addr;
+		reloc_at_class_addr.addr = p + sizeof (mach0_ut);
+		RSkipListNode *found = r_skiplist_find (relocs, &reloc_at_class_addr);
+		if (found) {
+			const char *_objc_class = "_OBJC_CLASS_$_";
+			const int _objc_class_len = strlen (_objc_class);
+			char *target_class_name = (char*) ((struct reloc_t*) found->data)->name;
+			if (r_str_startswith (target_class_name, _objc_class)) {
+				target_class_name += _objc_class_len;
+				klass->super = strdup (target_class_name);
+			}
+		}
+	}
+	get_class_ro_t (RO_DATA_PTR (c.data), bf, &is_meta_class, klass, oi);
 
 #if SWIFT_SUPPORT
 	if (q (c.data + n_value) & 7) {
@@ -1000,7 +1206,7 @@ void MACH0_(get_class_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, bool dupe)
 	if (!is_meta_class && !dupe) {
 		mach0_ut isa_n_value = get_isa_value ();
 		ut64 tmp = klass->addr;
-		MACH0_(get_class_t) (c.isa + isa_n_value, bf, klass, true);
+		MACH0_(get_class_t) (c.isa + isa_n_value, bf, klass, true, relocs, oi);
 		klass->addr = tmp;
 	}
 }
@@ -1041,11 +1247,10 @@ static RList *parse_swift_classes(RBinFile *bf) {
 }
 #endif
 
-RList *MACH0_(parse_classes)(RBinFile *bf) {
+RList *MACH0_(parse_classes)(RBinFile *bf, objc_cache_opt_info *oi) {
 	RList /*<RBinClass>*/ *ret = NULL;
 	ut64 num_of_unnamed_class = 0;
 	RBinClass *klass = NULL;
-	RBinObject *obj = bf ? bf->o : NULL;
 	ut32 i = 0, size = 0;
 	RList *sctns = NULL;
 	bool is_found = false;
@@ -1057,10 +1262,16 @@ RList *MACH0_(parse_classes)(RBinFile *bf) {
 	bool bigendian;
 	ut8 pp[sizeof (mach0_ut)] = {0};
 
-	if (!bf || !obj || !obj->bin_obj || !obj->info) {
+	r_return_val_if_fail (bf && bf->o, NULL);
+
+	if (!bf->o->bin_obj || !bf->o->info) {
 		return NULL;
 	}
-	bigendian = obj->info->big_endian;
+	bigendian = bf->o->info->big_endian;
+
+	RSkipList *relocs = MACH0_(get_relocs) (bf->o->bin_obj);
+
+	ret = MACH0_(parse_categories) (bf, relocs, oi);
 
 	/* check if it's Swift */
 	// ret = parse_swift_classes (bf);
@@ -1068,7 +1279,8 @@ RList *MACH0_(parse_classes)(RBinFile *bf) {
 	// sebfing of section with name __objc_classlist
 
 	struct section_t *sections = NULL;
-	if (!(sections = MACH0_(get_sections) (obj->bin_obj))) {
+	if (!(sections = MACH0_(get_sections) (bf->o->bin_obj))) {
+		r_skiplist_free (relocs);
 		return ret;
 	}
 
@@ -1129,7 +1341,7 @@ RList *MACH0_(parse_classes)(RBinFile *bf) {
 			goto get_classes_error;
 		}
 		p = r_read_ble (&pp[0], bigendian, 8 * sizeof (mach0_ut));
-		MACH0_(get_class_t) (p, bf, klass, false);
+		MACH0_(get_class_t) (p, bf, klass, false, relocs, oi);
 		if (!klass->name) {
 			klass->name = r_str_newf ("UnnamedClass%" PFMT64d, num_of_unnamed_class);
 			if (!klass->name) {
@@ -1139,11 +1351,262 @@ RList *MACH0_(parse_classes)(RBinFile *bf) {
 		}
 		r_list_append (ret, klass);
 	}
+	r_skiplist_free (relocs);
 	return ret;
 
 get_classes_error:
 	r_list_free (sctns);
 	r_list_free (ret);
+	r_skiplist_free (relocs);
 	// XXX DOUBLE FREE r_bin_class_free (klass);
 	return NULL;
 }
+
+static RList *MACH0_(parse_categories)(RBinFile *bf, RSkipList *relocs, objc_cache_opt_info *oi) {
+	r_return_val_if_fail (bf && bf->o && bf->o->bin_obj && bf->o->info, NULL);
+
+	RList /*<RBinClass>*/ *ret = NULL;
+	RBinObject *obj = bf->o;
+	const ut32 ptr_size = sizeof (mach0_ut);
+	bool is_found = false;
+	ut64 paddr;
+	ut64 s_size;
+
+	struct section_t *sections = NULL;
+	if (!(sections = MACH0_(get_sections) (obj->bin_obj))) {
+		return ret;
+	}
+
+	ut32 i = 0;
+	for (; !sections[i].last; i++) {
+		if (strstr (sections[i].name, "__objc_catlist")) {
+			is_found = true;
+			paddr = sections[i].offset;
+			s_size = sections[i].size;
+			break;
+		}
+	}
+
+	R_FREE (sections);
+
+	if (!is_found) {
+		goto error;
+	}
+
+	if (!ret && !(ret = r_list_newf ((RListFree)r_bin_class_free))) {
+		goto error;
+	}
+
+	if (!relocs) {
+		goto error;
+	}
+
+	for (i = 0; i < s_size; i += ptr_size) {
+		RBinClass *klass;
+		mach0_ut p;
+
+		if ((s_size - i) < ptr_size) {
+			eprintf ("Chopped catlist data\n");
+			break;
+		}
+		if (!(klass = R_NEW0 (RBinClass))) {
+			goto error;
+		}
+		if (!(klass->methods = r_list_new ())) {
+			R_FREE (klass);
+			goto error;
+		}
+		if (!(klass->fields = r_list_new ())) {
+			R_FREE (klass);
+			goto error;
+		}
+		if (!read_ptr_pa (bf, paddr + i, &p)) {
+			R_FREE (klass);
+			goto error;
+		}
+		MACH0_(get_category_t) (p, bf, klass, relocs, oi);
+		if (!klass->name) {
+			R_FREE (klass);
+			continue;
+		}
+		r_list_append (ret, klass);
+	}
+	return ret;
+
+error:
+	r_list_free (ret);
+	return NULL;
+}
+
+void MACH0_(get_category_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, RSkipList *relocs, objc_cache_opt_info *oi) {
+	r_return_if_fail (bf && bf->o && bf->o->info);
+
+	struct MACH0_(SCategory) c = { 0 };
+	const int size = sizeof (struct MACH0_(SCategory));
+	mach0_ut r = 0;
+	ut32 offset = 0, left = 0;
+	int len;
+	bool bigendian = bf->o->info->big_endian;
+	ut8 sc[sizeof (struct MACH0_(SCategory))] = {0};
+	ut32 i;
+
+	if (!(r = va2pa (p, &offset, &left, bf))) {
+		return;
+	}
+	if ((r + left) < r || (r + size) < r) {
+		return;
+	}
+	if (r > bf->size || r + left > bf->size) {
+		return;
+	}
+	if (r + size > bf->size) {
+		return;
+	}
+	if (left < size) {
+		eprintf ("Cannot parse obj category info out of bounds\n");
+		return;
+	}
+	len = r_buf_read_at (bf->buf, r, sc, size);
+	if (len != size) {
+		return;
+	}
+
+	ut32 ptr_size = sizeof (mach0_ut);
+	ut32 bits = 8 * ptr_size;
+
+	i = 0;
+	c.name = r_read_ble (&sc[i], bigendian, bits);
+	i += ptr_size;
+	c.targetClass = r_read_ble (&sc[i], bigendian, bits);
+	i += ptr_size;
+	c.instanceMethods = r_read_ble (&sc[i], bigendian, bits);
+	i += ptr_size;
+	c.classMethods = r_read_ble (&sc[i], bigendian, bits);
+	i += ptr_size;
+	c.protocols = r_read_ble (&sc[i], bigendian, bits);
+	i += ptr_size;
+	c.properties = r_read_ble (&sc[i], bigendian, bits);
+
+	char *category_name = read_str (bf, c.name, &offset, &left);
+	if (!category_name) {
+		return;
+	}
+
+	char *target_class_name = NULL;
+	if (c.targetClass == 0) {
+		if (!relocs) {
+			R_FREE (category_name);
+			return;
+		}
+		struct reloc_t reloc_at_class_addr;
+		reloc_at_class_addr.addr = p + ptr_size;
+		RSkipListNode *found = r_skiplist_find (relocs, &reloc_at_class_addr);
+		if (!found) {
+			R_FREE (category_name);
+			return;
+		}
+
+		const char *_objc_class = "_OBJC_CLASS_$_";
+		const int _objc_class_len = strlen (_objc_class);
+		target_class_name = (char*) ((struct reloc_t*) found->data)->name;
+		if (!r_str_startswith (target_class_name, _objc_class)) {
+			R_FREE (category_name);
+			return;
+		}
+		target_class_name += _objc_class_len;
+		klass->name = r_str_newf ("%s(%s)", target_class_name, category_name);
+	} else {
+		mach0_ut ro_data_field = c.targetClass + 4 * ptr_size;
+		mach0_ut ro_data;
+		if (!read_ptr_va (bf, ro_data_field, &ro_data)) {
+			R_FREE (category_name);
+			return;
+		}
+		mach0_ut name_field = RO_DATA_PTR (ro_data) + 3 * 4 + ptr_size;
+#ifdef R_BIN_MACH064
+		name_field += 4;
+#endif
+		mach0_ut name_at;
+		if (!read_ptr_va (bf, name_field & ~1, &name_at)) {
+			R_FREE (category_name);
+			return;
+		}
+
+		target_class_name = read_str (bf, name_at, &offset, &left);
+		char *demangled = NULL;
+		if (target_class_name) {
+			demangled = demangle_classname (target_class_name);
+		}
+		klass->name = r_str_newf ("%s(%s)", r_str_getf (demangled), category_name);
+		R_FREE (target_class_name);
+		R_FREE (demangled);
+	}
+
+	klass->addr = p;
+
+	R_FREE (category_name);
+
+	if (c.instanceMethods > 0) {
+		get_method_list_t (c.instanceMethods, bf, klass->name, klass, false, oi);
+	}
+
+	if (c.classMethods > 0) {
+		get_method_list_t (c.classMethods, bf, klass->name, klass, true, oi);
+	}
+
+	if (c.protocols > 0) {
+		get_protocol_list_t (c.protocols, bf, klass, oi);
+	}
+
+	if (c.properties > 0) {
+		get_objc_property_list (c.properties, bf, klass);
+	}
+}
+
+static bool read_ptr_pa(RBinFile *bf, ut64 paddr, mach0_ut *out) {
+	r_return_val_if_fail (out, false);
+	r_return_val_if_fail (bf && bf->o && bf->o->info, false);
+
+	size_t ptr_size = sizeof (mach0_ut);
+	ut8 pp[sizeof (mach0_ut)] = {0};
+
+	int len = r_buf_read_at (bf->buf, paddr, pp, ptr_size);
+	if (len != ptr_size) {
+		return false;
+	}
+
+	mach0_ut p = r_read_ble (&pp[0], bf->o->info->big_endian, 8 * ptr_size);
+	*out = p;
+
+	return true;
+}
+
+static bool read_ptr_va(RBinFile *bf, ut64 vaddr, mach0_ut *out) {
+	r_return_val_if_fail (bf, false);
+	ut32 offset = 0, left = 0;
+	mach0_ut paddr = va2pa (vaddr, &offset, &left, bf);
+	if (paddr == 0 || left < sizeof (mach0_ut)) {
+		return false;
+	}
+	return read_ptr_pa (bf, paddr, out);
+}
+
+static char *read_str(RBinFile *bf, mach0_ut p, ut32 *offset, ut32 *left) {
+	r_return_val_if_fail (bf && offset && left, NULL);
+
+	mach0_ut paddr = va2pa (p, offset, left, bf);
+	if (paddr == 0 || *left <= 1) {
+		return NULL;
+	}
+
+	int name_len = R_MIN (MAX_CLASS_NAME_LEN, *left);
+	char *name = calloc (1, name_len + 1);
+	int len = r_buf_read_at (bf->buf, paddr, (ut8 *)name, name_len);
+	if (len < name_len) {
+		eprintf ("Error reading\n");
+		R_FREE (name);
+		return NULL;
+	}
+
+	return name;
+}
