@@ -6,6 +6,8 @@
 #undef PE_Word
 #undef PE_DWord
 #undef PE_VWord
+#undef R_BUF_READ_PE_DWORD_AT
+#undef PE_DWORD_MAX
 
 #ifdef R_BIN_PE64
 #define PE_(name) Pe64_ ## name
@@ -14,6 +16,8 @@
 #define PE_Word ut16
 #define PE_DWord ut64
 #define PE_VWord ut32
+#define R_BUF_READ_PE_DWORD_AT r_buf_read_le64_at
+#define PE_DWORD_MAX UT64_MAX
 #else
 #define PE_(name) Pe32_ ## name
 #define ILT_MASK1 0x80000000
@@ -21,6 +25,8 @@
 #define PE_Word ut16
 #define PE_DWord ut32
 #define PE_VWord ut32
+#define R_BUF_READ_PE_DWORD_AT r_buf_read_le32_at
+#define PE_DWORD_MAX UT32_MAX
 #endif
 
 #ifndef _INCLUDE_R_BIN_PE_SPECS_H_
@@ -85,6 +91,9 @@ typedef struct {
 #define PE_IMAGE_FILE_MACHINE_THUMB            0x01c2
 #define PE_IMAGE_FILE_MACHINE_TRICORE          0x0520
 #define PE_IMAGE_FILE_MACHINE_WCEMIPSV2        0x0169
+#define PE_IMAGE_FILE_MACHINE_RISCV32          0x5032
+#define PE_IMAGE_FILE_MACHINE_RISCV64          0x5064
+#define PE_IMAGE_FILE_MACHINE_RISCV128         0x5128
 
 #define PE_IMAGE_FILE_RELOCS_STRIPPED          0x0001
 #define PE_IMAGE_FILE_EXECUTABLE_IMAGE         0x0002
@@ -442,6 +451,26 @@ typedef struct {
 	ut32 Characteristics;
 } Pe32_image_tls_directory, Pe64_image_tls_directory;
 
+typedef struct {
+	ut32 dwLength;
+	ut16 wRevision;
+	ut16 wCertificateType;
+	ut8 *bCertificate;
+} Pe_certificate;
+
+typedef struct {
+	ut32 length;
+	Pe_certificate **certificates;
+} Pe_image_security_directory;
+
+#define PE_WIN_CERT_REVISION_1_0	0x0100
+#define PE_WIN_CERT_REVISION_2_0	0x0200
+
+#define PE_WIN_CERT_TYPE_X509			0x0001
+#define PE_WIN_CERT_TYPE_PKCS_SIGNED_DATA	0x0002
+#define PE_WIN_CERT_TYPE_RESERVED_1		0x0003
+#define PE_WIN_CERT_TYPE_TS_STACK_SIGNED	0x0004
+
 typedef struct {
 	ut32 Signature;
 	Pe32_image_file_header file_header;
@@ -476,19 +505,19 @@ typedef struct {
 
 typedef struct {
 	union {
-		struct {
-			ut32 NameOffset: 31;
-			ut32 NameIsString: 1;
-		} s;
+		// struct {
+		// 	ut32 NameOffset: 31;
+		// 	ut32 NameIsString: 1;
+		// } s;
+		// ut16 Id;
 		ut32 Name;
-		ut16 Id;
 	} u1;
 	union {
+		// struct {
+		// 	ut32 OffsetToDirectory: 31;
+		// 	ut32 DataIsDirectory: 1;
+		// } s;
 		ut32 OffsetToData;
-		struct {
-			ut32 OffsetToDirectory: 31;
-			ut32 DataIsDirectory: 1;
-		} s;
 	} u2;
 } Pe_image_resource_directory_entry;
 
@@ -756,4 +785,14 @@ typedef struct {
 	PE64_SCOPE_RECORD ScopeRecord[];
 } PE64_SCOPE_TABLE;
 
+int Pe32_read_dos_header(RBuffer *b, Pe32_image_dos_header *header);
+int Pe32_read_nt_headers(RBuffer *b, ut64 addr, Pe32_image_nt_headers *headers);
+int Pe32_read_image_section_header(RBuffer *b, ut64 addr, Pe32_image_section_header *section_header);
+void Pe32_write_image_section_header(RBuffer *b, ut64 addr, Pe32_image_section_header *section_header);
+
+int Pe64_read_dos_header(RBuffer *b, Pe64_image_dos_header *header);
+int Pe64_read_nt_headers(RBuffer *b, ut64 addr, Pe64_image_nt_headers *headers);
+int Pe64_read_image_section_header(RBuffer *b, ut64 addr, Pe64_image_section_header *section_header);
+void Pe64_write_image_section_header(RBuffer *b, ut64 addr, Pe64_image_section_header *section_header);
+
 #endif
