@@ -35,6 +35,7 @@ typedef enum {
 	R_BIN_WASM_VALUETYPE_i64 = 0x2,
 	R_BIN_WASM_VALUETYPE_f32 = 0x3,
 	R_BIN_WASM_VALUETYPE_f64 = 0x4,
+	R_BIN_WASM_VALUETYPE_v128 = 0x5,
 	R_BIN_WASM_VALUETYPE_ANYFUNC = 0x10,
 	R_BIN_WASM_VALUETYPE_FUNC = 0x20,
 	R_BIN_WASM_VALUETYPE_EMPTY = 0x40,
@@ -64,16 +65,16 @@ struct r_bin_wasm_resizable_limits_t {
 	ut32 maximum;
 };
 
-struct r_bin_wasm_name_t {
+typedef struct r_bin_wasm_name_t {
 	ut32 len;
-	ut8 name[R_BIN_WASM_STRING_LENGTH];
-};
+	ut8 *name;
+} RBinWasmName;
 
 typedef struct r_bin_wasm_section_t {
 	ut8 id;
 	ut32 size;
 	ut32 name_len;
-	char name[R_BIN_WASM_STRING_LENGTH];
+	char *name;
 	ut32 offset;
 	ut32 payload_data;
 	ut32 payload_len;
@@ -168,7 +169,7 @@ typedef struct r_bin_wasm_code_t {
 	ut32 code; // offset
 	ut32 len; // real bytecode length
 	ut8 byte; // 0xb, indicating end of the body
-	char name[R_BIN_WASM_STRING_LENGTH];
+	char *name;
 	char *signature;
 } RBinWasmCodeEntry;
 
@@ -237,21 +238,21 @@ typedef struct r_bin_wasm_obj_t {
 
 } RBinWasmObj;
 
-RBinWasmObj *r_bin_wasm_init (RBinFile *bf, RBuffer *buf);
-void r_bin_wasm_destroy (RBinFile *bf);
-RList *r_bin_wasm_get_sections (RBinWasmObj *bin);
-RList *r_bin_wasm_get_types (RBinWasmObj *bin);
-RList *r_bin_wasm_get_imports (RBinWasmObj *bin);
-RList *r_bin_wasm_get_exports (RBinWasmObj *bin);
-RList *r_bin_wasm_get_tables (RBinWasmObj *bin);
-RList *r_bin_wasm_get_memories (RBinWasmObj *bin);
-RList *r_bin_wasm_get_globals (RBinWasmObj *bin);
-RList *r_bin_wasm_get_elements (RBinWasmObj *bin);
-RList *r_bin_wasm_get_codes (RBinWasmObj *bin);
-RList *r_bin_wasm_get_datas (RBinWasmObj *bin);
-RList *r_bin_wasm_get_custom_names (RBinWasmObj *bin);
-ut32 r_bin_wasm_get_entrypoint (RBinWasmObj *bin);
-const char *r_bin_wasm_get_function_name (RBinWasmObj *bin, ut32 idx);
-const char *r_bin_wasm_valuetype_to_string (r_bin_wasm_value_type_t type);
+RBinWasmObj *r_bin_wasm_init(RBinFile *bf, RBuffer *buf);
+void r_bin_wasm_destroy(RBinFile *bf);
+RList *r_bin_wasm_get_sections(RBinWasmObj *bin);
+RList *r_bin_wasm_get_types(RBinWasmObj *bin);
+RList *r_bin_wasm_get_imports(RBinWasmObj *bin);
+RList *r_bin_wasm_get_exports(RBinWasmObj *bin);
+RList *r_bin_wasm_get_tables(RBinWasmObj *bin);
+RList *r_bin_wasm_get_memories(RBinWasmObj *bin);
+RList *r_bin_wasm_get_globals(RBinWasmObj *bin);
+RList *r_bin_wasm_get_elements(RBinWasmObj *bin);
+RList *r_bin_wasm_get_codes(RBinWasmObj *bin);
+RList *r_bin_wasm_get_datas(RBinWasmObj *bin);
+RList *r_bin_wasm_get_custom_names(RBinWasmObj *bin);
+ut32 r_bin_wasm_get_entrypoint(RBinWasmObj *bin);
+const char *r_bin_wasm_get_function_name(RBinWasmObj *bin, ut32 idx);
+const char *r_bin_wasm_valuetype_to_string(r_bin_wasm_value_type_t type);
 
 #endif
