@@ -65,7 +65,7 @@ typedef struct  {
 }) xbe_lib;
 
 typedef struct {
-	xbe_header *header;
+	xbe_header header;
 	int kt_key;
 	int ep_key;
 } r_bin_xbe_obj_t; 
