@@ -9,10 +9,6 @@
 #include <stdlib.h>
 #include <r_cons.h>
 
-#ifndef HAS_MAIN
-#define HAS_MAIN 0
-#endif
-
 #define IFDBG if(0)
 
 // $ echo "..." | xcrun swift-demangle
@@ -25,7 +21,7 @@ struct Type {
 static struct Type types[] = {
 	/* basic types */
 	{ "Sb", "Bool" },
-	{ "SS", "String" },
+	{ "SS", "Swift.String" },
 	{ "FS", "String" },
 	{ "GV", "mutableAddressor" }, // C_ARGC
 	{ "Ss", "generic" }, // C_ARGC
@@ -40,7 +36,6 @@ static struct Type types[] = {
 	{ "SQ", "ImplicitlyUnwrappedOptional" },
 	{ "Sc", "UnicodeScalar" },
 	{ "Sd", "Double" },
-	{ "SS", "String" }, // Swift.String
 	/* builtin */
 	{ "Bi1", "Builtin.Int1" },
 	{ "Bp", "Builtin.RawPointer" },
@@ -175,7 +170,7 @@ static char *swift_demangle_lib(const char *s) {
 	return NULL;
 }
 
-R_API char *r_bin_demangle_swift(const char *s, bool syscmd) {
+R_API char *r_bin_demangle_swift(const char *s, bool syscmd, bool trylib) {
 #define STRCAT_BOUNDS(x) if (((x) + 2 + strlen (out)) > sizeof (out)) break;
 	char out[1024];
 	int i, len, is_generic = 0;
@@ -195,18 +190,15 @@ R_API char *r_bin_demangle_swift(const char *s, bool syscmd) {
 			return NULL;
 		}
 	}
-
 	if (!strncmp (s, "__", 2)) {
 		s = s + 2;
 	}
-#if 0
-	const char *element[] = {
-		"module", "class", "method", NULL
-	};
-#endif
-	char *res = swift_demangle_lib (s);
-	if (res) {
-		return res;
+	char *res = NULL;
+	if (trylib) {
+		res = swift_demangle_lib (s);
+		if (res) {
+			return res;
+		}
 	}
 	const char *attr = NULL;
 	const char *attr2 = NULL;
@@ -265,7 +257,7 @@ R_API char *r_bin_demangle_swift(const char *s, bool syscmd) {
 			break;
 		case 'I': // interfaces
 			/* TODO */
-			break;
+			return NULL; // Fix __TIFF demangling
 		}
 	}
 	if (tail) {
@@ -283,8 +275,8 @@ R_API char *r_bin_demangle_swift(const char *s, bool syscmd) {
 	
 	// _TF or __TW
 	if (IS_DIGIT (*p) || *p == 'v' || *p == 'I' || *p == 'o' || *p == 'T' || *p == 'V' || *p == 'M' || *p == 'C' || *p == 'F' || *p == 'W') {
-		if (!strncmp (p+1, "SS", 2)) {
-			strcat (out, "Swift.String.init (");
+		if (!strncmp (p + 1, "SS", 2)) {
+			strcat (out, "Swift.String.init(");
 			p += 3;
 		}
 		if (!strncmp (p, "vdv", 3)) {
@@ -429,7 +421,6 @@ R_API char *r_bin_demangle_swift(const char *s, bool syscmd) {
 						if (attr) {
 							strcat (out, attr);
 						}
-						//p = q + 7;
 						q = p = q + 1;
 						attr = "";
 						break;
@@ -520,27 +511,29 @@ R_API char *r_bin_demangle_swift(const char *s, bool syscmd) {
 						const char *s = getstring (q, len);
 						if (s && *s) {
 							if (is_first) {	
-								strcat (out, is_generic?"<":"(");
+								strcat (out, is_generic?"<":": ");
 								is_first = 0;
 							}
 							//printf ("ISLAST (%s)\n", q+len);
-							is_last = q[len];
+							is_last = strlen (q+len) < 5;
 							if (attr) {
 								STRCAT_BOUNDS (strlen (attr));
 								strcat (out, attr);
-								strcat (out, " ");
+								if (!is_last) {
+									strcat (out, ", ");
+								}
 							}
 							STRCAT_BOUNDS (strlen (s));
-							strcat (out, s);
-							if (is_last) {
-								strcat (out, is_generic?">":")");
-								is_first = (*s != '_');
-								if (is_generic && !is_first) {
-									break;
+								if (strcmp (s, "_")) {
+									strcat (out, s);
+									strcat (out, is_generic?">":"");
+									is_first = (*s != '_');
+									if (is_generic && !is_first) {
+										break;
+									}
+								} else {
+									strcat(out, ")");
 								}
-							} else {
-								strcat (out, ", ");
-							}
 						} else {
 							if (attr) {
 								strcat (out, " -> ");
@@ -589,167 +582,15 @@ R_API char *r_bin_demangle_swift(const char *s, bool syscmd) {
 		p = outstr;
 		for (;;) {
 			p = strstr (p, ")(");
-			if (p) {
-				p[0] = '_';
-				p[1] = '_';
-				p+=2;
-			} else {
+			if (!p) {
 				break;
 			}
+			p[0] = '_';
+			p[1] = '_';
+			p += 2;
 		}
 		return outstr;
 #endif
 	}
 	return NULL;
 }
-#define MAXMAIN
-
-#if HAS_MAIN
-
-typedef struct {
-	const char *sym;
-	const char *dem;
-} Test;
-
-Test swift_tests[] = {
-{
-	"_TWPu0_Rq_Ss14CollectionType_GVSs17MapCollectionViewq_q0__Ss23_CollectionDefaultsTypeSs_8",
-	"<generic _CollectionDefaultsType>"
-},
-{
-	"_TWPurGVSs15CollectionOfOneq__Ss14CollectionTypeSs_248",
-	"CollectionOfOne<generic CollectionType><generic S>"
-},
-{
-	"_TFSSCfT21_builtinStringLiteralBp8byteSizeBw7isASCIIBi1__SS"
-	,"Swift.String.init (_builtinStringLiteral(Builtin.RawPointer byteSize__Builtin.Word isASCII__Builtin.Int1 _) -> String"
-	//, "Swift.String.init (Swift.String.Type) -> (_builtinStringLiteral : Builtin.RawPointer, byteSize : Builtin.Word, isASCII : Builtin.Int1) -> Swift.String
-},{
-	"_TFC10swifthello5Hellog5WorldSS" // getter
-	,"swifthello.Hello.World.getter__String"
-	// swifthello.Hello.World.getter : Swift.String
-},{
-	"_TFC10swifthello5Hellom5WorldSS" // getter
-	,"swifthello.Hello.World.method__String"
-},{
-	"_TFC10swifthello5Hellos5WorldSS" // getter
-	,"swifthello.Hello.World.setter__String"
-},{
-	"_TFSSCfMSSFT21_builtinStringLiteralBp8byteSizeBw7isASCIIBi1__SS"
-	,"Swift.String.init (_builtinStringLiteral(Builtin.RawPointer byteSize__Builtin.Word isASCII__Builtin.Int1 _) -> String"
-},{
-	"_TF10swifthello3norFT_Si"
-	,"swifthello.nor () -> Swift.Int"
-},{
-	"_TFSs7printlnU__FQ_T_"
-	,"println.<A>(A) -> ()"
-},{
-	"_TFSsa6C_ARGVGVSs20UnsafeMutablePointerGS_VSs4Int8__"
-	,"C_ARGV<generic UnsafeMutablePointer><generic Int8>"
-},{
-	"_TFC10FlappyBird9GameScene10resetScenefS0_FT_T_"
-	,"FlappyBird.GameScene.resetScene (self) -> (__ _) ()" // XXX this is not correct
-},{
-	"__TFC4main8BarClass8sayHellofT_T_"
-	,"main.BarClass.sayHello"
-},{
-	"__TFC4main4TostCfT_S0_"
-	,"main.Tost.allocator"
-},{
-	"__TFC4main4TostD"
-	,"main.Tost.deallocator"
-},{
-	"__TFC4main4TostcfT_S0_"
-	,"main.Tost.constructor"
-},{
-	"__TF4main4moinFT_Si"
-	,"main.moin () -> Swift.Int"
-},{
-	"__TFC4main4Tostg3msgSS"
-	,"main.Tost.msg.getter__String"
-},{
-	"__TMC4main4Tost"
-	,"main.Tost..metadata"
-},{
-	"__TMLC4main4Tost"
-	,"main.Tost..lazy.metadata"
-
-},{
-	"__TMaC4main4Tost"
-	,"main.Tost..accessor.metadata"
-	//,"_lazy cache variable for type metadata for main.Tost"
-},{
-	"__TMmC4main4Tost"
-	,"main.Tost..metaclass"
-},{
-	"__TFV4main7Balanceg5widthSd"
-	,"main.Balance.width.getter__Double"
-},{
-	"__TWoFC4main4TostCfT_S0_"
-	,"Tost.allocator..init.witnesstable"
-},{
-	"__TMBi256_"
-	,"_..metadata"
-},{
-	"__TWvdvC4main4Tost3msgSS"
-	,"main.Tost.msg__String..field"
-},{
-	"__TIFC10Moscapsule10MQTTClient11unsubscribeFTSS17requestCompletionGSqFTOS_10MosqResultSi_T___T_A0_"
-	,"Moscapsule.MQTTClient.unsubscribe ()"
-////imp._TIFC10Moscapsule10MQTTClient11unsubscribeFTSS17requestCompletionGSqFTOS_10MosqResultSi_T___T_A0_
-},{
-	"__TWaC4main8FooClassS_9FoodClassS_"
-	,"main.FooClass..FoodClass..protocol"
-},{
-	"__TFe4mainRxCS_8FooClassxS_9FoodClassrS1_8sayHellofT_T_"
-	,"main..FooClass..FoodClass..sayHello..extension"
-},{
-	"_TTSg5P____TFs27_allocateUninitializedArrayurFBwTGSax_Bp_",
-	"P____(GenericSpec F)_allocateUninitializedArray -> Builtin.RawPointer" // TODO poor translation
-	// "generic specialization <protocol<>> of Swift._allocateUninitializedArray <A> (Builtin.Word) -> ([A], Builtin.RawPointer)"
-},{
-	"_TIFs5printFTGSaP__9separatorSS10terminatorSS_T_A0_",
-	"print (__Array P)", // TODO: poor translation
-	//"Swift.(print ([protocol<>], separator : Swift.String, terminator : Swift.String) -> ()).(default argument 1)"
-},{
-	"__TZFsoi2eeuRxs9EquatablerFTGSqx_GSqx__Sb",
-	"Swift..Equatable () -> Bool"
-},{
-	// _direct field offset for main.Tost.msg : Swift.String
-	NULL, NULL
-}};
-
-int main(int argc, char **argv) {
-	int rc = 0;
-	char *ret;
-	if (argc > 1) {
-		ret = r_bin_demangle_swift (argv[1], 0);
-		if (ret) {
-			printf ("%s\n", ret);
-			free (ret);
-		}
-	} else {
-		int i = 0;
-		for (i = 0; swift_tests[i].sym; i++) {
-			Test *test = &swift_tests[i];
-			printf ("[>>] %s\n", test->sym);
-			ret = r_bin_demangle_swift (test->sym, 0);
-			if (ret) {
-				if (test->dem && !strcmp (ret, test->dem)) {
-					printf (Color_GREEN"[OK]"Color_RESET"  %s\n", ret);
-				} else {
-					printf (Color_RED"[XX]"Color_RESET"  %s\n", ret);
-					printf (Color_YELLOW"[MUSTBE]"Color_RESET"  %s\n", test->dem);
-					rc = 1;
-				}
-				free (ret);
-			} else {
-				printf (Color_RED"[XX]"Color_RESET"  \"(null)\"\n");
-				printf (Color_YELLOW"[MUSTBE]"Color_RESET"  %s\n", test->dem);
-				rc = 1;
-			}
-		}
-	}
-	return rc;
-}
-#endif
