@@ -1,9 +1,9 @@
-/* radare - LGPL - Copyright 2009-2016 - nibble, montekki, pancake */
+/* radare - LGPL - Copyright 2009-2020 - nibble, montekki, pancake */
 
 #include <r_types.h>
 #include <r_bin.h>
 
-static int get_line(RBinFile *bf, ut64 addr, char *file, int len, int *line) {
+static bool get_line(RBinFile *bf, ut64 addr, char *file, int len, int *line) {
 	if (bf->sdb_addrinfo) {
 		char offset[64];
 		char *offset_ptr = sdb_itoa (addr, offset, 16);
@@ -21,6 +21,6 @@ static int get_line(RBinFile *bf, ut64 addr, char *file, int len, int *line) {
 	return false;
 }
 
-struct r_bin_dbginfo_t r_bin_dbginfo_dex = {
+RBinDbgInfo r_bin_dbginfo_dex = {
 	.get_line = &get_line,
 };
