@@ -23,7 +23,7 @@ RBinLdrPlugin r_bin_ldr_plugin_ldr_linux = {
 	.load = &load,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN_LDR,
 	.data = &r_bin_ldr_plugin_ldr_linux,
