@@ -7,7 +7,7 @@
 #include <string.h>
 #include "../format/nin/gba.h"
 
-static bool check_buffer(RBuffer *b) {
+static bool check_buffer(RBinFile *bf, RBuffer *b) {
 	ut8 lict[156];
 	r_return_val_if_fail (b, false);
 	r_buf_read_at (b, 4, (ut8*)lict, sizeof (lict));
@@ -15,7 +15,7 @@ static bool check_buffer(RBuffer *b) {
 }
 
 static bool load_buffer(RBinFile * bf, void **bin_obj, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
-	return check_buffer (buf);
+	return check_buffer (bf, buf);
 }
 
 static RList *entries(RBinFile *bf) {
