@@ -7,7 +7,7 @@
 #include "../i/private.h"
 #include "psxexe/psxexe.h"
 
-static bool check_buffer(RBuffer *b) {
+static bool check_buffer(RBinFile *bf, RBuffer *b) {
 	ut8 magic[PSXEXE_ID_LEN];
 	if (r_buf_read_at (b, 0, magic, sizeof (magic)) == PSXEXE_ID_LEN) {
 		return !memcmp (magic, PSXEXE_ID, PSXEXE_ID_LEN);
@@ -16,7 +16,7 @@ static bool check_buffer(RBuffer *b) {
 }
 
 static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *b, ut64 loadaddr, Sdb *sdb) {
-	return check_buffer (b);
+	return check_buffer (bf, b);
 }
 
 static RBinInfo* info(RBinFile* bf) {
