@@ -1,4 +1,4 @@
-/* radare - LGPL3 - 2015 - riq */
+/* radare - LGPL3 - 2015-2019 - riq */
 
 /* VICE Snapshot File loader: http://vice-emu.sourceforge.net/ */
 
@@ -6,7 +6,7 @@
 #include "vsf/vsf_specs.h"
 
 static const char VICE_MAGIC[] = "VICE Snapshot File\032";
-static const int VICE_MAGIC_LEN = sizeof(VICE_MAGIC)-1;
+#define VICE_MAGIC_LEN sizeof (VICE_MAGIC) - 1
 static const char VICE_MAINCPU[] = "MAINCPU";
 static const char VICE_C64MEM[] = "C64MEM";
 static const char VICE_C64ROM[] = "C64ROM";
@@ -25,41 +25,45 @@ static const struct {
 static const int MACHINES_MAX = sizeof(_machines) / sizeof(_machines[0]);
 
 static Sdb* get_sdb (RBinFile *bf) {
-	if (!bf || !bf->o || !bf->o->bin_obj) {
-		return NULL;
-	}
+	r_return_val_if_fail (bf && bf->o && bf->o->bin_obj, NULL);
 	struct r_bin_vsf_obj* bin = (struct r_bin_vsf_obj*) bf->o->bin_obj;
-	return bin? bin->kv: NULL;
+	return bin->kv;
 }
 
-static bool check_bytes(const ut8 *buf, ut64 length) {
-	if (!buf || length < VICE_MAGIC_LEN) {
-		return false;
+static bool check_buffer(RBinFile *bf, RBuffer *b) {
+	ut8 magic[VICE_MAGIC_LEN];
+	if (r_buf_read_at (b, 0, magic, VICE_MAGIC_LEN) == VICE_MAGIC_LEN) {
+		return !memcmp (magic, VICE_MAGIC, VICE_MAGIC_LEN);
 	}
-	return (!memcmp (buf, VICE_MAGIC, VICE_MAGIC_LEN));
+	return false;
 }
 
-static void * load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb) {
+// XXX b vs bf->buf
+static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *b, ut64 loadaddr, Sdb *sdb) {
 	ut64 offset = 0;
 	struct r_bin_vsf_obj* res = NULL;
-	if (check_bytes (buf, sz)) {
+	if (check_buffer (bf, bf->buf)) {
 		int i = 0;
 		if (!(res = R_NEW0 (struct r_bin_vsf_obj))) {
-		    return NULL;
+		    return false;
 		}
 		offset = r_offsetof(struct vsf_hdr, machine);
 		if (offset > bf->size) {
 			free (res);
-			return NULL;
+			return false;
+		}
+		char machine[20];
+		int l = r_buf_read_at (bf->buf, offset, (ut8 *)machine, sizeof (machine));
+		if (l < 0) {
+			free (res);
+			return false;
 		}
-		const unsigned char* machine = bf->buf->buf + offset;
 		for (; i < MACHINES_MAX; i++) {
 			if (offset + strlen (_machines[i].name) > bf->size) {
 				free (res);
-				return NULL;
+				return false;
 			}
-			if (!strncmp ((const char *)machine, _machines[i].name,
-				      strlen (_machines[i].name))) {
+			if (!strncmp (machine, _machines[i].name, strlen (_machines[i].name))) {
 				res->machine_idx = i;
 				break;
 			}
@@ -67,40 +71,46 @@ static void * load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, S
 		if (i >= MACHINES_MAX) {
 			eprintf ("Unsupported machine type\n");
 			free (res);
-			return NULL;
+			return false;
 		}
 		// read all VSF modules
 		offset = sizeof (struct vsf_hdr);
+		ut64 sz = r_buf_size (bf->buf);
 		while (offset < sz) {
 			struct vsf_module module;
 			int read = r_buf_fread_at (bf->buf, offset, (ut8*)&module, "16ccci", 1);
 			if (read != sizeof(module)) {
 				eprintf ("Truncated Header\n");
 				free (res);
-				return NULL;
+				return false;
 			}
 #define CMP_MODULE(x) memcmp (module.module_name, x, sizeof (x) - 1)
 			if (!CMP_MODULE (VICE_C64MEM) && !module.major) {
-				res->mem = &bf->buf->buf[offset + read];
+				res->mem = offset + read;
 			} else if (!CMP_MODULE (VICE_C64ROM) && !module.major) {
-				res->rom = &bf->buf->buf[offset + read];
+				res->rom = offset + read;
 			} else if (!CMP_MODULE (VICE_C128MEM) && !module.major) {
-				res->mem = &bf->buf->buf[offset + read];
+				res->mem = offset + read;
 			} else if (!CMP_MODULE (VICE_C128ROM) && !module.major) {
-				res->rom = &bf->buf->buf[offset + read];
+				res->rom = offset + read;
 			} else if (!CMP_MODULE (VICE_MAINCPU) && module.major == 1) {
-				res->maincpu = (struct vsf_maincpu*)&bf->buf->buf[offset + read];
+				res->maincpu = R_NEW (struct vsf_maincpu);
+				r_buf_read_at (bf->buf, offset + read, (ut8 *)res->maincpu, sizeof (*res->maincpu));
 			}
 #undef CMP_MODULE
 			offset += module.length;
+			if (module.length == 0) {
+				eprintf ("Malformed VSF module with length 0\n");
+				break;
+			}
 		}
 	}
 	if (res) {
 		res->kv = sdb_new0 ();
 		sdb_ns_set (sdb, "info", res->kv);
 	}
-	// res will be assigned to bf->o->bin_obj by the callee
-	return res;
+	*bin_obj = res;
+	return true;
 }
 
 static RList *mem(RBinFile *bf) {
@@ -111,7 +121,7 @@ static RList *mem(RBinFile *bf) {
 	}
 	RList *ret;
 	RBinMem *m;
-	if (!(ret = r_list_new())) {
+	if (!(ret = r_list_new ())) {
 		return NULL;
 	}
 	ret->free = free;
@@ -150,14 +160,12 @@ static RList* sections(RBinFile* bf) {
 			if (!(ptr = R_NEW0 (RBinSection))) {
 				return ret;
 			}
-			strcpy (ptr->name, "BASIC");
-			ptr->paddr = ((char *)vsf_obj->rom +
-				      r_offsetof (struct vsf_c64rom, basic)) -
-				     (char *)bf->buf->buf;
+			ptr->name = strdup ("BASIC");
+			ptr->paddr = vsf_obj->rom + r_offsetof (struct vsf_c64rom, basic);
 			ptr->size = 1024 * 8; // (8k)
 			ptr->vaddr = 0xa000;
 			ptr->vsize = 1024 * 8;	// BASIC size (8k)
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
+			ptr->perm = R_PERM_RX;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -165,14 +173,12 @@ static RList* sections(RBinFile* bf) {
 			if (!(ptr = R_NEW0 (RBinSection))) {
 				return ret;
 			}
-			strcpy (ptr->name, "KERNAL");
-			ptr->paddr = ((char *)vsf_obj->rom +
-				      r_offsetof (struct vsf_c64rom, kernal)) -
-				     (char *)bf->buf->buf;
+			ptr->name = strdup ("KERNAL");
+			ptr->paddr = vsf_obj->rom + r_offsetof (struct vsf_c64rom, kernal);
 			ptr->size = 1024 * 8; // (8k)
 			ptr->vaddr = 0xe000;
 			ptr->vsize = 1024 * 8;	// KERNAL size (8k)
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
+			ptr->perm = R_PERM_RX;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -183,14 +189,12 @@ static RList* sections(RBinFile* bf) {
 			if (!(ptr = R_NEW0 (RBinSection))) {
 				return ret;
 			}
-			strcpy (ptr->name, "BASIC");
-			ptr->paddr = ((char *)vsf_obj->rom +
-				      r_offsetof (struct vsf_c128rom, basic)) -
-				     (char *)bf->buf->buf;
+			ptr->name = strdup ("BASIC");
+			ptr->paddr = vsf_obj->rom + r_offsetof (struct vsf_c128rom, basic);
 			ptr->size = 1024 * 28; // (28k)
 			ptr->vaddr = 0x4000;
 			ptr->vsize = 1024 * 28;	// BASIC size (28k)
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
+			ptr->perm = R_PERM_RX;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -198,15 +202,13 @@ static RList* sections(RBinFile* bf) {
 			if (!(ptr = R_NEW0 (RBinSection))) {
 				return ret;
 			}
-			strcpy (ptr->name, "MONITOR");
+			ptr->name = strdup ("MONITOR");
 			// skip first 28kb  since "BASIC" and "MONITOR" share the same section in VSF
-			ptr->paddr = ((char *)vsf_obj->rom +
-				      r_offsetof (struct vsf_c128rom, basic)) +
-				     1024 * 28 - (char *)bf->buf->buf;
+			ptr->paddr = vsf_obj->rom + r_offsetof (struct vsf_c128rom, basic) + 1024 * 28;
 			ptr->size = 1024 * 4; // (4k)
 			ptr->vaddr = 0xb000;
 			ptr->vsize = 1024 * 4;	// BASIC size (4k)
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
+			ptr->perm = R_PERM_RX;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -214,14 +216,12 @@ static RList* sections(RBinFile* bf) {
 			if (!(ptr = R_NEW0 (RBinSection))) {
 				return ret;
 			}
-			strcpy (ptr->name, "EDITOR");
-			ptr->paddr = ((char *)vsf_obj->rom +
-				      r_offsetof (struct vsf_c128rom, editor)) -
-				     (char *)bf->buf->buf;
+			ptr->name = strdup ("EDITOR");
+			ptr->paddr = vsf_obj->rom + r_offsetof (struct vsf_c128rom, editor);
 			ptr->size = 1024 * 4; // (4k)
 			ptr->vaddr = 0xc000;
 			ptr->vsize = 1024 * 4;	// BASIC size (4k)
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
+			ptr->perm = R_PERM_RX;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -229,14 +229,12 @@ static RList* sections(RBinFile* bf) {
 			if (!(ptr = R_NEW0 (RBinSection))) {
 				return ret;
 			}
-			strcpy (ptr->name, "KERNAL");
-			ptr->paddr = ((char *)vsf_obj->rom +
-				      r_offsetof (struct vsf_c128rom, kernal)) -
-				     (char *)bf->buf->buf;
+			ptr->name = strdup ("KERNAL");
+			ptr->paddr = vsf_obj->rom + r_offsetof (struct vsf_c128rom, kernal);
 			ptr->size = 1024 * 8; // (8k)
 			ptr->vaddr = 0xe000;
 			ptr->vsize = 1024 * 8;	// KERNAL size (8k)
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
+			ptr->perm = R_PERM_RX;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -252,12 +250,12 @@ static RList* sections(RBinFile* bf) {
 			if (!(ptr = R_NEW0 (RBinSection))) {
 				return ret;
 			}
-			strcpy (ptr->name, "RAM");
-			ptr->paddr = ((char *)vsf_obj->mem + offset) - (char*) bf->buf->buf;
+			ptr->name = strdup ("RAM");
+			ptr->paddr = vsf_obj->mem + offset;
 			ptr->size = size;
 			ptr->vaddr = 0x0;
 			ptr->vsize = size;
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_WRITABLE | R_BIN_SCN_EXECUTABLE;
+			ptr->perm = R_PERM_RWX;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 		} else {
@@ -269,24 +267,24 @@ static RList* sections(RBinFile* bf) {
 			if (!(ptr = R_NEW0 (RBinSection))) {
 				return ret;
 			}
-			strcpy (ptr->name, "RAM BANK 0");
-			ptr->paddr = ((char *)vsf_obj->mem + offset) - (char*) bf->buf->buf;
+			ptr->name = strdup ("RAM BANK 0");
+			ptr->paddr = vsf_obj->mem + offset;
 			ptr->size = size;
 			ptr->vaddr = 0x0;
 			ptr->vsize = size;
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_WRITABLE | R_BIN_SCN_EXECUTABLE;
+			ptr->perm = R_PERM_RWX;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
 			if (!(ptr = R_NEW0 (RBinSection))) {
 				return ret;
 			}
-			strcpy (ptr->name, "RAM BANK 1");
-			ptr->paddr = ((char *)vsf_obj->mem + offset) + size - (char*) bf->buf->buf;
+			ptr->name = strdup ("RAM BANK 1");
+			ptr->paddr = vsf_obj->mem + offset + size;
 			ptr->size = size;
 			ptr->vaddr = 0x0;
 			ptr->vsize = size;
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_WRITABLE | R_BIN_SCN_EXECUTABLE;
+			ptr->perm = R_PERM_RWX;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 		}
@@ -475,12 +473,14 @@ static RList* symbols(RBinFile *bf) {
 	};
 	static const int SYMBOLS_MAX = sizeof(_symbols) / sizeof(_symbols[0]);
 	struct r_bin_vsf_obj* vsf_obj = (struct r_bin_vsf_obj*) bf->o->bin_obj;
-	if (!vsf_obj) return NULL;
+	if (!vsf_obj) {
+		return NULL;
+	}
 	const int m_idx = vsf_obj->machine_idx;
 	int offset = _machines[m_idx].offset_mem;
 	RList *ret = NULL;
 	RBinSymbol *ptr;
-	if (!(ret = r_list_new())) {
+	if (!(ret = r_list_new ())) {
 		return NULL;
 	}
 	ret->free = free;
@@ -488,15 +488,16 @@ static RList* symbols(RBinFile *bf) {
 	int i;
 	for (i = 0; i < SYMBOLS_MAX; i++)
 	{
-		if (!(ptr = R_NEW0 (RBinSymbol))) return ret;
+		if (!(ptr = R_NEW0 (RBinSymbol))) {
+			return ret;
+		}
 		if (!ptr->name) {
 			ptr->name = calloc(1, R_BIN_SIZEOF_STRINGS);
 		}
 		strncpy (ptr->name, _symbols[i].symbol_name, R_BIN_SIZEOF_STRINGS);
 		ptr->vaddr = _symbols[i].address;
 		ptr->size = 2;
-		ptr->paddr = ((char *)vsf_obj->mem + offset) - (char *)bf->buf->buf +
-			     _symbols[i].address;
+		ptr->paddr = vsf_obj->mem + offset + _symbols[i].address;
 		ptr->ordinal = i;
 		r_list_append (ret, ptr);
 	}
@@ -504,14 +505,17 @@ static RList* symbols(RBinFile *bf) {
 	return ret;
 }
 
-static int destroy(RBinFile *bf) {
-	free(bf->o->bin_obj);
-	return true;
+static void destroy(RBinFile *bf) {
+	struct r_bin_vsf_obj *obj = (struct r_bin_vsf_obj *)bf->o->bin_obj;
+	free (obj->maincpu);
+	free (obj);
 }
 
 static RList* entries(RBinFile *bf) {
 	struct r_bin_vsf_obj* vsf_obj = (struct r_bin_vsf_obj*) bf->o->bin_obj;
-	if (!vsf_obj) return NULL;
+	if (!vsf_obj) {
+		return NULL;
+	}
 	const int m_idx = vsf_obj->machine_idx;
 
 	RList *ret;
@@ -524,7 +528,7 @@ static RList* entries(RBinFile *bf) {
 	if (!(ptr = R_NEW0 (RBinAddr))) {
 		return ret;
 	}
-	ptr->paddr = ((char *)vsf_obj->mem + offset) - (char*) bf->buf->buf;
+	ptr->paddr = vsf_obj->mem + offset;
 	ptr->vaddr = vsf_obj->maincpu ? vsf_obj->maincpu->pc : 0;
 	r_list_append (ret, ptr);
 
@@ -543,8 +547,8 @@ RBinPlugin r_bin_plugin_vsf = {
 	.desc = "VICE Snapshot File",
 	.license = "LGPL3",
 	.get_sdb = &get_sdb,
-	.load_bytes = &load_bytes,
-	.check_bytes = &check_bytes,
+	.load_buffer = &load_buffer,
+	.check_buffer = &check_buffer,
 	.entries = &entries,
 	.sections = sections,
 	.symbols = &symbols,
@@ -553,8 +557,8 @@ RBinPlugin r_bin_plugin_vsf = {
 	.mem = &mem,
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_vsf,
 	.version = R2_VERSION
