@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2017 - pancake, cgvwzq */
+/* radare2 - LGPL - Copyright 2017-2021 - pancake, cgvwzq */
 
 // http://webassembly.org/docs/binary-encoding/#module-structure
 
@@ -8,46 +8,32 @@
 #include <r_bin.h>
 
 #include "wasm/wasm.h"
+#include "../format/wasm/wasm.h"
 
-static bool check_bytes(const ut8 *buf, ut64 length) {
-	return (buf && length >= 4 && !memcmp (buf, R_BIN_WASM_MAGIC_BYTES, 4));
+static bool check_buffer(RBinFile *bf, RBuffer *rbuf) {
+	ut8 buf[4] = { 0 };
+	return rbuf && r_buf_read_at (rbuf, 0, buf, 4) == 4 && !memcmp (buf, R_BIN_WASM_MAGIC_BYTES, 4);
 }
 
-static bool check_bytes_buf(RBuffer* rbuf) {
-	ut8 buf[4] = {0};
-	return rbuf && r_buf_read_at (rbuf, R_BUF_CUR, buf, 4) == 4 && !memcmp (buf, R_BIN_WASM_MAGIC_BYTES, 4);
-}
-
-static bool find_symbol(const ut32 *p, const RBinWasmSymbol* q) {
-	return q->id != (*p);
-}
-
-static bool find_export(const ut32 *p, const RBinWasmExportEntry* q) {
+static bool find_export(const ut32 *p, const RBinWasmExportEntry *q) {
 	if (q->kind != R_BIN_WASM_EXTERNALKIND_Function) {
-		return false;
+		return true;
 	}
-	return q->index != (*p);
+	return q->index != *p;
 }
 
-static void * load_buffer(RBinFile *bf, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
-	r_return_val_if_fail (bf && buf && r_buf_size (buf) != UT64_MAX, NULL);
+static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
+	r_return_val_if_fail (bf && buf && r_buf_size (buf) != UT64_MAX, false);
 
-	if (!check_bytes_buf (buf)) {
-		return NULL;
+	if (check_buffer (bf, buf)) {
+		*bin_obj = r_bin_wasm_init (bf, buf);
+		return true;
 	}
-	return r_bin_wasm_init (bf);
+	return false;
 }
 
-static bool load(RBinFile *bf) {
-	r_return_val_if_fail (bf && bf->o, false);
-
-	bf->o->bin_obj = load_buffer (bf, bf->buf, bf->o->loadaddr, bf->sdb);
-	return bf->o->bin_obj != NULL;
-}
-
-static int destroy(RBinFile *bf) {
+static void destroy(RBinFile *bf) {
 	r_bin_wasm_destroy (bf);
-	return true;
 }
 
 static ut64 baddr(RBinFile *bf) {
@@ -65,13 +51,12 @@ static RList *entries(RBinFile *bf) {
 	// TODO
 	RList *ret = NULL;
 	RBinAddr *ptr = NULL;
-	ut64 addr = 0x0;
 
 	if (!(ret = r_list_newf ((RListFree)free))) {
 		return NULL;
 	}
 
-	addr = (ut64) r_bin_wasm_get_entrypoint (bin);
+	ut64 addr = (ut64)r_bin_wasm_get_entrypoint (bin);
 	if (!addr) {
 		RList *codes = r_bin_wasm_get_codes (bin);
 		if (codes) {
@@ -95,7 +80,7 @@ static RList *entries(RBinFile *bf) {
 	return ret;
 }
 
-static RList *sections(RBinFile *bf) {
+static RList *sections (RBinFile *bf) {
 	RBinWasmObj *bin = bf && bf->o ? bf->o->bin_obj : NULL;
 	RList *ret = NULL;
 	RList *secs = NULL;
@@ -116,7 +101,7 @@ static RList *sections(RBinFile *bf) {
 			r_list_free (ret);
 			return NULL;
 		}
-		ptr->name = strdup ((char*)sec->name);
+		ptr->name = strdup ((char *)sec->name);
 		if (sec->id == R_BIN_WASM_SECTION_DATA || sec->id == R_BIN_WASM_SECTION_MEMORY) {
 			ptr->is_data = true;
 		}
@@ -133,14 +118,13 @@ static RList *sections(RBinFile *bf) {
 }
 
 static RList *symbols(RBinFile *bf) {
-	RBinWasmObj *bin = NULL;
-	RList *ret = NULL, *codes = NULL, *imports = NULL, *symtab = NULL, *exports = NULL;
+	RList *ret = NULL, *codes = NULL, *imports = NULL, *exports = NULL;
 	RBinSymbol *ptr = NULL;
 
 	if (!bf || !bf->o || !bf->o->bin_obj) {
 		return NULL;
 	}
-	bin = bf->o->bin_obj;
+	RBinWasmObj *bin = bf->o->bin_obj;
 	if (!(ret = r_list_newf ((RListFree)free))) {
 		return NULL;
 	}
@@ -150,13 +134,15 @@ static RList *symbols(RBinFile *bf) {
 	if (!(imports = r_bin_wasm_get_imports (bin))) {
 		goto bad_alloc;
 	}
-	if (!(symtab = r_bin_wasm_get_symtab (bin))) {
-		goto bad_alloc;
-	}
 	if (!(exports = r_bin_wasm_get_exports (bin))) {
 		goto bad_alloc;
 	}
 
+	ut32 fcn_idx = 0,
+	     table_idx = 0,
+	     mem_idx = 0,
+	     global_idx = 0;
+
 	ut32 i = 0;
 	RBinWasmImportEntry *imp;
 	RListIter *iter;
@@ -164,14 +150,28 @@ static RList *symbols(RBinFile *bf) {
 		if (!(ptr = R_NEW0 (RBinSymbol))) {
 			goto bad_alloc;
 		}
-		ptr->name = r_str_newf ("imp.%s.%s", imp->module_str, imp->field_str);
-		ptr->forwarder = r_str_const ("NONE");
-		ptr->bind = r_str_const ("NONE");
+		ptr->name = strdup (imp->field_str);
+		ptr->libname = strdup (imp->module_str);
+		ptr->is_imported = true;
+		ptr->forwarder = "NONE";
+		ptr->bind = "NONE";
 		switch (imp->kind) {
-		case 0: ptr->type = r_str_const (R_BIN_TYPE_FUNC_STR); break;
-		case 1: ptr->type = r_str_const ("TABLE"); break;
-		case 2: ptr->type = r_str_const ("MEMORY"); break;
-		case 3: ptr->type = r_str_const (R_BIN_BIND_GLOBAL_STR); break;
+		case R_BIN_WASM_EXTERNALKIND_Function:
+			ptr->type = R_BIN_TYPE_FUNC_STR;
+			fcn_idx++;
+			break;
+		case R_BIN_WASM_EXTERNALKIND_Table:
+			ptr->type = "TABLE";
+			table_idx++;
+			break;
+		case R_BIN_WASM_EXTERNALKIND_Memory:
+			ptr->type = "MEMORY";
+			mem_idx++;
+			break;
+		case R_BIN_WASM_EXTERNALKIND_Global:
+			ptr->type = R_BIN_BIND_GLOBAL_STR;
+			global_idx++;
+			break;
 		}
 		ptr->size = 0;
 		ptr->vaddr = -1;
@@ -181,41 +181,36 @@ static RList *symbols(RBinFile *bf) {
 		r_list_append (ret, ptr);
 	}
 
-	ut32 fcn_id = 0;
-	RListIter *sym_it = NULL;
+	RListIter *is_exp = NULL;
 	RBinWasmCodeEntry *func;
-	RBinWasmExportEntry *export = NULL;
-	RBinWasmSymbol *wasm_sym = NULL;
-	fcn_id = 0;
 	r_list_foreach (codes, iter, func) {
 		if (!(ptr = R_NEW0 (RBinSymbol))) {
 			goto bad_alloc;
 		}
+		const char *fcn_name = r_bin_wasm_get_function_name (bin, fcn_idx);
+		if (fcn_name) {
+			ptr->name = strdup (fcn_name);
 
-		sym_it = r_list_find (symtab, &fcn_id, (RListComparator) find_symbol);
-		if (sym_it) {
-			wasm_sym = (RBinWasmSymbol *) r_list_iter_get_data (sym_it);
-			ptr->name = strdup (wasm_sym->name);
-		} else {
-			sym_it = r_list_find (exports, &fcn_id, (RListComparator) find_export);
-			if (sym_it) {
-				export = (RBinWasmExportEntry *) r_list_iter_get_data (sym_it);
-				ptr->name = strdup (export->field_str);
-			} else {
-				// fallback if symbol is not found.
-				ptr->name = r_str_newf ("fcn.%d", fcn_id);
+			is_exp = r_list_find (exports, &fcn_idx, (RListComparator)find_export);
+			if (is_exp) {
+				ptr->bind = R_BIN_BIND_GLOBAL_STR;
 			}
+		} else {
+			// fallback if symbol is not found.
+			ptr->name = r_str_newf ("fcn.%d", fcn_idx);
 		}
 
-		ptr->forwarder = r_str_const ("NONE");
-		ptr->bind = r_str_const ("NONE");
-		ptr->type = r_str_const (R_BIN_TYPE_FUNC_STR);
+		ptr->forwarder = "NONE";
+		if (!ptr->bind) {
+			ptr->bind = "NONE";
+		}
+		ptr->type = R_BIN_TYPE_FUNC_STR;
 		ptr->size = func->len;
 		ptr->vaddr = (ut64)func->code;
 		ptr->paddr = (ut64)func->code;
 		ptr->ordinal = i;
 		i++;
-		fcn_id++;
+		fcn_idx++;
 		r_list_append (ret, ptr);
 	}
 
@@ -224,13 +219,12 @@ static RList *symbols(RBinFile *bf) {
 bad_alloc:
 	// not so sure if imports should be freed.
 	r_list_free (exports);
-	r_list_free (symtab);
 	r_list_free (codes);
 	r_list_free (ret);
 	return NULL;
 }
 
-static RList *imports(RBinFile *bf) {
+static RList *imports (RBinFile *bf) {
 	RBinWasmObj *bin = NULL;
 	RList *imports = NULL;
 	RBinImport *ptr = NULL;
@@ -240,7 +234,7 @@ static RList *imports(RBinFile *bf) {
 		return NULL;
 	}
 	bin = bf->o->bin_obj;
-	if (!(ret = r_list_newf (r_bin_import_free))) {
+	if (!(ret = r_list_newf ((RListFree)r_bin_import_free))) {
 		return NULL;
 	}
 	if (!(imports = r_bin_wasm_get_imports (bin))) {
@@ -257,19 +251,19 @@ static RList *imports(RBinFile *bf) {
 		ptr->name = strdup (import->field_str);
 		ptr->classname = strdup (import->module_str);
 		ptr->ordinal = i;
-		ptr->bind = r_str_const ("NONE");
-		switch(import->kind) {
+		ptr->bind = "NONE";
+		switch (import->kind) {
 		case R_BIN_WASM_EXTERNALKIND_Function:
-			ptr->type = r_str_const ("FUNC");
+			ptr->type = "FUNC";
 			break;
 		case R_BIN_WASM_EXTERNALKIND_Table:
-			ptr->type = r_str_const ("TABLE");
+			ptr->type = "TABLE";
 			break;
 		case R_BIN_WASM_EXTERNALKIND_Memory:
-			ptr->type = r_str_const ("MEM");
+			ptr->type = "MEM";
 			break;
 		case R_BIN_WASM_EXTERNALKIND_Global:
-			ptr->type = r_str_const ("GLOBAL");
+			ptr->type = "GLOBAL";
 			break;
 		}
 		r_list_append (ret, ptr);
@@ -281,11 +275,11 @@ bad_alloc:
 	return NULL;
 }
 
-static RList *libs(RBinFile *bf) {
+static RList *libs (RBinFile *bf) {
 	return NULL;
 }
 
-static RBinInfo *info(RBinFile *bf) {
+static RBinInfo *info (RBinFile *bf) {
 	RBinInfo *ret = NULL;
 
 	if (!(ret = R_NEW0 (RBinInfo))) {
@@ -306,32 +300,63 @@ static RBinInfo *info(RBinFile *bf) {
 	return ret;
 }
 
-static ut64 size(RBinFile *bf) {
+static ut64 size (RBinFile *bf) {
 	if (!bf || !bf->buf) {
 		return 0;
 	}
-	return bf->buf->length;
+	return r_buf_size (bf->buf);
 }
 
 /* inspired in http://www.phreedom.org/solar/code/tinype/tiny.97/tiny.asm */
-static RBuffer *create(RBin *bin, const ut8 *code, int codelen, const ut8 *data, int datalen, RBinArchOptions *opt) {
+static RBuffer *create (RBin *bin, const ut8 *code, int codelen, const ut8 *data, int datalen, RBinArchOptions *opt) {
 	RBuffer *buf = r_buf_new ();
-#define B(x, y) r_buf_append_bytes (buf, (const ut8 *) (x), y)
+#define B(x, y) r_buf_append_bytes (buf, (const ut8 *)(x), y)
 #define D(x) r_buf_append_ut32 (buf, x)
 	B ("\x00" "asm", 4);
 	B ("\x01\x00\x00\x00", 4);
 	return buf;
 }
 
+static int get_fcn_offset_from_id(RBinFile *bf, int fcn_idx) {
+	RBinWasmObj *bin = bf->o->bin_obj;
+	RList *codes = r_bin_wasm_get_codes (bin);
+	if (codes) {
+		RBinWasmCodeEntry *func = r_list_get_n (codes, fcn_idx);
+		if (func) {
+			return func->code;
+		}
+	}
+	return -1;
+}
+
+static int getoffset(RBinFile *bf, int type, int idx) {
+	switch (type) {
+	case 'f': // fcnid -> fcnaddr
+		return get_fcn_offset_from_id (bf, idx);
+	}
+	return -1;
+}
+
+static const char *getname(RBinFile *bf, int type, int idx, bool sd) {
+	RBinWasmObj *bin = bf->o->bin_obj;
+        switch (type) {
+        case 'f': // fcnidx
+		{
+			const char *r = r_bin_wasm_get_function_name (bin, idx);
+			return r? strdup (r): NULL;
+		}
+	}
+	return NULL;
+}
+
 RBinPlugin r_bin_plugin_wasm = {
 	.name = "wasm",
 	.desc = "WebAssembly bin plugin",
 	.license = "MIT",
-	.load = &load,
 	.load_buffer = &load_buffer,
 	.size = &size,
 	.destroy = &destroy,
-	.check_bytes = &check_bytes,
+	.check_buffer = &check_buffer,
 	.baddr = &baddr,
 	.binsym = &binsym,
 	.entries = &entries,
@@ -340,10 +365,12 @@ RBinPlugin r_bin_plugin_wasm = {
 	.imports = &imports,
 	.info = &info,
 	.libs = &libs,
+	.get_offset = &getoffset,
+	.get_name = &getname,
 	.create = &create,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_wasm,
