@@ -1,7 +1,7 @@
 /* radare - LGPL - Copyright 2009-2016 pancake */
 
 #define R_BIN_ELF64 1
-#include "bin_write_elf.c"
+#include "bin_write_elf.inc"
 
 RBinWrite r_bin_write_elf64 = {
 	.scn_resize = &scn_resize,
