@@ -11,7 +11,7 @@ static const char *kt_name[] = {
 #include "../format/xbe/kernel.h"
 };
 
-static bool check_buffer(RBuffer *b) {
+static bool check_buffer(RBinFile *bf, RBuffer *b) {
 	ut8 magic[4];
 	if (r_buf_read_at (b, 0, magic, sizeof (magic)) == 4) {
 		return !memcmp (magic, "XBEH", 4);
@@ -297,9 +297,9 @@ static RList *symbols(RBinFile *bf) {
 		}
 		const ut32 thunk_index = thunk_addr[i] ^ 0x80000000;
 		// Basic sanity checks
-		if (thunk_addr[i] & 0x80000000 && thunk_index < XBE_MAX_THUNK) {
-			eprintf ("%d\n", thunk_index);
-			sym->name = r_str_newf ("kt.%s", kt_name[thunk_index]);
+		if (thunk_addr[i] & 0x80000000 && thunk_index > 0 && thunk_index <= XBE_MAX_THUNK) {
+			eprintf ("thunk_index %d\n", thunk_index);
+			sym->name = r_str_newf ("kt.%s", kt_name[thunk_index - 1]);
 			sym->vaddr = (h->kernel_thunk_addr ^ obj->kt_key) + (4 * i);
 			sym->paddr = sym->vaddr - h->base;
 			sym->size = 4;
