@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2019 - mrmacete */
+/* radare2 - LGPL - Copyright 2019-2021 - mrmacete */
 
 #include <r_types.h>
 #include <r_util.h>
@@ -120,6 +120,9 @@ typedef struct _RKmodInfo {
 #define K_PPTR(p) p_ptr (p, obj)
 #define K_RPTR(buf) r_ptr (buf, obj)
 
+#define IS_PTR_AUTH(x) ((x & (1ULL << 63)) != 0)
+#define IS_PTR_BIND(x) ((x & (1ULL << 62)) != 0)
+
 static ut64 p_ptr (ut64 decorated_addr, RKernelCacheObj *obj);
 static ut64 r_ptr (ut8 *buf, RKernelCacheObj *obj);
 
@@ -132,11 +135,13 @@ static void swizzle_io_read(RKernelCacheObj *obj, RIO *io);
 static int kernelcache_io_read(RIO *io, RIODesc *fd, ut8 *buf, int count);
 static bool r_parse_pointer(RParsedPointer *ptr, ut64 decorated_addr, RKernelCacheObj *obj);
 static bool on_rebase_pointer (ut64 offset, ut64 decorated_addr, RRebaseCtx *ctx);
-static void rebase_buffer(RKernelCacheObj *obj, RIO *io, RIODesc *fd, ut8 *buf, int count);
+static void rebase_buffer(RKernelCacheObj *obj, ut64 off, RIODesc *fd, ut8 *buf, int count);
+static void rebase_buffer_fixup(RKernelCacheObj *kobj, ut64 off, RIODesc *fd, ut8 *buf, int count);
 
 static RPrelinkRange *get_prelink_info_range_from_mach0(struct MACH0_(obj_t) *mach0);
-static RList *filter_kexts(RKernelCacheObj *obj);
-static RList *carve_kexts(RKernelCacheObj *obj);
+static RList *filter_kexts(RKernelCacheObj *obj, RBinFile *bf);
+static RList *carve_kexts(RKernelCacheObj *obj, RBinFile *bf);
+static RList *kexts_from_load_commands(RKernelCacheObj *obj, RBinFile *bf);
 
 static void sections_from_mach0(RList *ret, struct MACH0_(obj_t) *mach0, RBinFile *bf, ut64 paddr, char *prefix, RKernelCacheObj *obj);
 static void handle_data_sections(RBinSection *sect);
@@ -150,7 +155,8 @@ static int prot2perm (int x);
 static void r_kext_free(RKext *kext);
 static void r_kext_fill_text_range(RKext *kext);
 static int kexts_sort_vaddr_func(const void *a, const void *b);
-static struct MACH0_(obj_t) *create_kext_mach0(RKernelCacheObj *obj, RKext *kext);
+static struct MACH0_(obj_t) *create_kext_mach0(RKernelCacheObj *obj, RKext *kext, RBinFile *bf);
+static struct MACH0_(obj_t) *create_kext_shared_mach0(RKernelCacheObj *obj, RKext *kext, RBinFile *bf);
 
 #define r_kext_index_foreach(index, i, item)\
 	if (index)\
@@ -164,7 +170,7 @@ static void process_kmod_init_term(RKernelCacheObj *obj, RKext *kext, RList *ret
 static void create_initterm_syms(RKext *kext, RList *ret, int type, ut64 *pointers);
 static void process_constructors(RKernelCacheObj *obj, struct MACH0_(obj_t) *mach0, RList *ret, ut64 paddr, bool is_first, int mode, const char *prefix);
 static RBinAddr *newEntry(ut64 haddr, ut64 vaddr, int type);
-static void ensure_kexts_initialized(RKernelCacheObj *obj);
+static void ensure_kexts_initialized(RKernelCacheObj *obj, RBinFile *bf);
 
 static void r_kernel_cache_free(RKernelCacheObj *obj);
 
@@ -180,24 +186,28 @@ static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *buf, ut64 loadadd
 	}
 
 	RRebaseInfo *rebase_info = r_rebase_info_new_from_mach0 (fbuf, main_mach0);
+	RKernelCacheObj *obj = NULL;
 
 	RPrelinkRange *prelink_range = get_prelink_info_range_from_mach0 (main_mach0);
 	if (!prelink_range) {
 		goto beach;
 	}
 
-	RKernelCacheObj *obj = R_NEW0 (RKernelCacheObj);
+	obj = R_NEW0 (RKernelCacheObj);
 	if (!obj) {
 		R_FREE (prelink_range);
 		goto beach;
 	}
 
-	RCFValueDict *prelink_info = r_cf_value_dict_parse (fbuf, prelink_range->range.offset,
-		prelink_range->range.size, R_CF_OPTION_SKIP_NSDATA);
-	if (!prelink_info) {
-		R_FREE (prelink_range);
-		R_FREE (obj);
-		goto beach;
+	RCFValueDict *prelink_info = NULL;
+	if (main_mach0->hdr.filetype != MH_FILESET && prelink_range->range.size) {
+		prelink_info = r_cf_value_dict_parse (fbuf, prelink_range->range.offset,
+				prelink_range->range.size, R_CF_OPTION_SKIP_NSDATA);
+		if (!prelink_info) {
+			R_FREE (prelink_range);
+			R_FREE (obj);
+			goto beach;
+		}
 	}
 
 	if (!pending_bin_files) {
@@ -217,11 +227,13 @@ static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *buf, ut64 loadadd
 	obj->pa2va_exec = prelink_range->pa2va_exec;
 	obj->pa2va_data = prelink_range->pa2va_data;
 
+	R_FREE (prelink_range);
+
 	*bin_obj = obj;
 
 	r_list_push (pending_bin_files, bf);
 
-	if (rebase_info) {
+	if (rebase_info || main_mach0->chained_starts) {
 		RIO *io = bf->rbin->iob.io;
 		swizzle_io_read (obj, io);
 	}
@@ -230,17 +242,22 @@ static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *buf, ut64 loadadd
 
 beach:
 	r_buf_free (fbuf);
+	obj->cache_buf = NULL;
 	MACH0_(mach0_free) (main_mach0);
 	return false;
 }
 
-static void ensure_kexts_initialized(RKernelCacheObj *obj) {
+static void ensure_kexts_initialized(RKernelCacheObj *obj, RBinFile *bf) {
 	if (obj->kexts_initialized) {
 		return;
 	}
 	obj->kexts_initialized = true;
 
-	RList *kexts = filter_kexts (obj);
+	RList *kexts = NULL;
+
+	if (obj->prelink_info) {
+		kexts = filter_kexts (obj, bf);
+	}
 
 	if (kexts && !r_list_length (kexts)) {
 		r_list_free (kexts);
@@ -248,10 +265,24 @@ static void ensure_kexts_initialized(RKernelCacheObj *obj) {
 	}
 
 	if (!kexts) {
-		kexts = carve_kexts (obj);
+		kexts = kexts_from_load_commands (obj, bf);
+	}
+
+	if (kexts && !r_list_length (kexts)) {
+		r_list_free (kexts);
+		kexts = NULL;
+	}
+
+	if (!kexts) {
+		kexts = carve_kexts (obj, bf);
 	}
 
 	obj->kexts = r_kext_index_new (kexts);
+
+	if (kexts) {
+		kexts->free = NULL;
+		r_list_free (kexts);
+	}
 }
 
 static RPrelinkRange *get_prelink_info_range_from_mach0(struct MACH0_(obj_t) *mach0) {
@@ -294,6 +325,20 @@ static RPrelinkRange *get_prelink_info_range_from_mach0(struct MACH0_(obj_t) *ma
 
 	R_FREE (sections);
 
+	if (incomplete == 1 && !prelink_range->pa2va_data) {
+		struct MACH0_(segment_command) *seg;
+		int nsegs = R_MIN (mach0->nsegs, 128);
+		size_t i;
+		for (i = 0; i < nsegs; i++) {
+			seg = &mach0->segs[i];
+			if (!strcmp (seg->segname, "__DATA")) {
+				prelink_range->pa2va_data = seg->vmaddr - seg->fileoff;
+				incomplete--;
+				break;
+			}
+		}
+	}
+
 	if (incomplete) {
 		R_FREE (prelink_range);
 	}
@@ -301,7 +346,7 @@ static RPrelinkRange *get_prelink_info_range_from_mach0(struct MACH0_(obj_t) *ma
 	return prelink_range;
 }
 
-static RList *filter_kexts(RKernelCacheObj *obj) {
+static RList *filter_kexts(RKernelCacheObj *obj, RBinFile *bf) {
 	RCFValueArray *kext_array = NULL;
 	RListIter *iter;
 	RCFKeyValue *item;
@@ -388,7 +433,7 @@ static RList *filter_kexts(RKernelCacheObj *obj) {
 		}
 		prev_kext = kext;
 
-		kext->mach0 = create_kext_mach0 (obj, kext);
+		kext->mach0 = create_kext_mach0 (obj, kext, bf);
 		if (!kext->mach0) {
 			r_kext_free (kext);
 			continue;
@@ -417,7 +462,7 @@ static ut64 r_ptr (ut8 *buf, RKernelCacheObj *obj) {
 	return K_PPTR (decorated_addr);
 }
 
-static RList *carve_kexts(RKernelCacheObj *obj) {
+static RList *carve_kexts(RKernelCacheObj *obj, RBinFile *bf) {
 	struct section_t *sections = NULL;
 	if (!(sections = MACH0_(get_sections) (obj->mach0))) {
 		return NULL;
@@ -515,7 +560,7 @@ static RList *carve_kexts(RKernelCacheObj *obj) {
 		kext->vaddr = K_RPTR (bytes);
 		kext->range.offset = kext->vaddr - pa2va_exec;
 
-		kext->mach0 = create_kext_mach0 (obj, kext);
+		kext->mach0 = create_kext_mach0 (obj, kext, bf);
 		if (!kext->mach0) {
 			r_kext_free (kext);
 			continue;
@@ -561,6 +606,80 @@ beach:
 	return NULL;
 }
 
+static RList *kexts_from_load_commands(RKernelCacheObj *obj, RBinFile *bf) {
+	RList *kexts = r_list_newf ((RListFree) &r_kext_free);
+	if (!kexts) {
+		return NULL;
+	}
+
+	ut32 i, ncmds = r_buf_read_le32_at (obj->cache_buf, 16);
+	ut64 length = r_buf_size (obj->cache_buf);
+
+	ut32 cursor = sizeof (struct MACH0_(mach_header));
+	for (i = 0; i < ncmds && cursor < length; i++) {
+		ut32 cmdtype = r_buf_read_le32_at (obj->cache_buf, cursor);
+		ut32 cmdsize = r_buf_read_le32_at (obj->cache_buf, cursor + 4);
+		if (!cmdsize || cmdsize + cursor < cursor) {
+			break;
+		}
+		if (cmdtype != LC_KEXT) {
+			cursor += cmdsize;
+			continue;
+		}
+
+		ut64 vaddr = r_buf_read_le64_at (obj->cache_buf, cursor + 8);
+		ut64 paddr = r_buf_read_le64_at (obj->cache_buf, cursor + 16);
+		st32 padded_name_length = (st32)cmdsize - 32;
+		if (padded_name_length <= 0 || cmdsize - 32 + cursor >= length) {
+			cursor += cmdsize;
+			continue;
+		}
+
+		char *padded_name = calloc (1, padded_name_length);
+		if (!padded_name) {
+			goto beach;
+		}
+		if (r_buf_read_at (obj->cache_buf, cursor + 32, (ut8 *)padded_name, padded_name_length)
+				!= padded_name_length) {
+			free (padded_name);
+			goto early;
+		}
+
+		RKext *kext = R_NEW0 (RKext);
+		if (!kext) {
+			free (padded_name);
+			goto beach;
+		}
+
+		kext->vaddr = vaddr;
+		kext->range.offset = paddr;
+
+		kext->mach0 = create_kext_shared_mach0 (obj, kext, bf);
+		if (!kext->mach0) {
+			free (padded_name);
+			r_kext_free (kext);
+			cursor += cmdsize;
+			continue;
+		}
+
+		r_kext_fill_text_range (kext);
+		kext->vaddr = K_PPTR (kext->vaddr);
+		kext->pa2va_exec = obj->pa2va_exec;
+		kext->pa2va_data = obj->pa2va_data;
+		kext->name = strdup (padded_name);
+		kext->own_name = true;
+		free (padded_name);
+		r_list_push (kexts, kext);
+
+		cursor += cmdsize;
+	}
+early:
+	return kexts;
+beach:
+	r_list_free (kexts);
+	return NULL;
+}
+
 static void r_kext_free(RKext *kext) {
 	if (!kext) {
 		return;
@@ -678,17 +797,25 @@ static RKext *r_kext_index_vget(RKextIndex *index, ut64 vaddr) {
 	return NULL;
 }
 
-static struct MACH0_(obj_t) *create_kext_mach0(RKernelCacheObj *obj, RKext *kext) {
+static struct MACH0_(obj_t) *create_kext_mach0(RKernelCacheObj *obj, RKext *kext, RBinFile *bf) {
 	RBuffer *buf = r_buf_new_slice (obj->cache_buf, kext->range.offset, r_buf_size (obj->cache_buf) - kext->range.offset);
 	struct MACH0_(opts_t) opts;
+	MACH0_(opts_set_default) (&opts, bf);
 	opts.verbose = true;
 	opts.header_at = 0;
 	struct MACH0_(obj_t) *mach0 = MACH0_(new_buf) (buf, &opts);
 	r_buf_free (buf);
-	if (!mach0) {
-		return NULL;
-	}
+	return mach0;
+}
 
+static struct MACH0_(obj_t) *create_kext_shared_mach0(RKernelCacheObj *obj, RKext *kext, RBinFile *bf) {
+	RBuffer *buf = r_buf_ref (obj->cache_buf);
+	struct MACH0_(opts_t) opts;
+	MACH0_(opts_set_default) (&opts, bf);
+	opts.verbose = false;
+	opts.header_at = kext->range.offset;
+	struct MACH0_(obj_t) *mach0 = MACH0_(new_buf) (buf, &opts);
+	r_buf_free (buf);
 	return mach0;
 }
 
@@ -891,7 +1018,7 @@ static RBinAddr *newEntry(ut64 haddr, ut64 vaddr, int type) {
 	return ptr;
 }
 
-static bool check_buffer(RBuffer *b) {
+static bool check_buffer(RBinFile *bf, RBuffer *b) {
 	if (r_buf_size (b) > 4) {
 		ut8 buf[4];
 		r_buf_read_at (b, 0, buf, sizeof (buf));
@@ -911,7 +1038,7 @@ static RList *sections(RBinFile *bf) {
 	}
 
 	RKernelCacheObj *kobj = (RKernelCacheObj*) obj->bin_obj;
-	ensure_kexts_initialized (kobj);
+	ensure_kexts_initialized (kobj, bf);
 
 	int iter;
 	RKext *kext;
@@ -1071,7 +1198,7 @@ static RList *symbols(RBinFile *bf) {
 		r_list_free (subsystem);
 	}
 
-	ensure_kexts_initialized (obj);
+	ensure_kexts_initialized (obj, bf);
 
 	RKext *kext;
 	int kiter;
@@ -1265,6 +1392,7 @@ static RList *resolve_syscalls(RKernelCacheObj *obj, ut64 enosys_addr) {
 		if (item && item->name) {
 			RBinSymbol *sym = R_NEW0 (RBinSymbol);
 			if (!sym) {
+				r_syscall_item_free (item);
 				goto beach;
 			}
 
@@ -1276,10 +1404,8 @@ static RList *resolve_syscalls(RKernelCacheObj *obj, ut64 enosys_addr) {
 			sym->bind = "GLOBAL";
 			sym->type = "FUNC";
 			r_list_append (syscalls, sym);
-
-			r_syscall_item_free (item);
 		}
-
+		r_syscall_item_free (item);
 		cursor += 24;
 		i++;
 	}
@@ -1303,7 +1429,7 @@ beach:
 #define K_MIG_ROUTINE_SIZE (5 * 8)
 #define K_MIG_MAX_ROUTINES 100
 
-static HtPP *mig_hash_new() {
+static HtPP *mig_hash_new(void) {
 	HtPP *hash = sdb_ht_new ();
 	if (!hash) {
 		return NULL;
@@ -1380,13 +1506,13 @@ static RList *resolve_mig_subsystem(RKernelCacheObj *obj) {
 		}
 		ut32 subs_min_idx = r_read_le32 (cursor + 8);
 		ut32 subs_max_idx = r_read_le32 (cursor + 12);
-		ut32 n_routines = (subs_max_idx - subs_min_idx);
 		if (subs_min_idx >= subs_max_idx || (subs_max_idx - subs_min_idx) > K_MIG_MAX_ROUTINES) {
 			cursor += 16;
 			continue;
 		}
 
-		ut64 *routines = (ut64 *) malloc (n_routines * sizeof (ut64));
+		ut32 n_routines = (subs_max_idx - subs_min_idx);
+		ut64 *routines = (ut64 *) calloc (n_routines, sizeof (ut64));
 		if (!routines) {
 			goto beach;
 		}
@@ -1541,7 +1667,7 @@ static void symbols_from_stubs(RList *ret, HtPP *kernel_syms_by_addr, RKernelCac
 			continue;
 		}
 
-		ensure_kexts_initialized (obj);
+		ensure_kexts_initialized (obj, bf);
 		RKext *remote_kext = r_kext_index_vget (obj->kexts, target_addr);
 		if (!remote_kext) {
 			continue;
@@ -1589,6 +1715,7 @@ static RStubsInfo *get_stubs_info(struct MACH0_(obj_t) *mach0, ut64 paddr, RKern
 
 	RStubsInfo *stubs_info = R_NEW0 (RStubsInfo);
 	if (!stubs_info) {
+		free (sections);
 		return NULL;
 	}
 
@@ -1853,20 +1980,15 @@ static ut64 iterate_rebase_list(RBuffer *cache_buf, ut64 multiplier, ut64 start_
 }
 
 static void swizzle_io_read(RKernelCacheObj *obj, RIO *io) {
-	if (!io || !io->desc || !io->desc->plugin) {
-		return;
-	}
-
+	r_return_if_fail (io && io->desc && io->desc->plugin);
 	RIOPlugin *plugin = io->desc->plugin;
 	obj->original_io_read = plugin->read;
 	plugin->read = &kernelcache_io_read;
 }
 
 static int kernelcache_io_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
-	if (!io) {
-		return -1;
-	}
-	RCore *core = (RCore*) io->user;
+	r_return_val_if_fail (io, -1);
+	RCore *core = (RCore*) io->corebind.core;
 
 	if (!core || !core->bin || !core->bin->binfiles) {
 		return -1;
@@ -1902,13 +2024,21 @@ static int kernelcache_io_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	}
 
 	if (!cache || !cache->original_io_read || cache->rebasing_buffer) {
-		if (fd->plugin->read == &kernelcache_io_read) {
-			return -1;
+		if (cache) {
+			if ((!cache->rebasing_buffer && fd->plugin->read == &kernelcache_io_read) ||
+					(cache->rebasing_buffer && !cache->original_io_read)) {
+				return -1;
+			}
+			if (cache->rebasing_buffer) {
+				return cache->original_io_read (io, fd, buf, count);
+			}
 		}
 		return fd->plugin->read (io, fd, buf, count);
 	}
 
-	r_rebase_info_populate (cache->rebase_info, cache);
+	if (cache->rebase_info) {
+		r_rebase_info_populate (cache->rebase_info, cache);
+	}
 
 	static ut8 *internal_buffer = NULL;
 	static int internal_buf_size = 0;
@@ -1921,23 +2051,30 @@ static int kernelcache_io_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 		internal_buf_size = count;
 	}
 
+	if (!cache->original_io_read) {
+		return -1;
+	}
+	ut64 io_off = io->off;
 	int result = cache->original_io_read (io, fd, internal_buffer, count);
 
 	if (result == count) {
-		rebase_buffer (cache, io, fd, internal_buffer, count);
+		if (cache->mach0->chained_starts) {
+			rebase_buffer_fixup (cache, io_off, fd, internal_buffer, count);
+		} else if (cache->rebase_info) {
+			rebase_buffer (cache, io_off, fd, internal_buffer, count);
+		}
 		memcpy (buf, internal_buffer, result);
 	}
 
 	return result;
 }
 
-static void rebase_buffer(RKernelCacheObj *obj, RIO *io, RIODesc *fd, ut8 *buf, int count) {
-	if (obj->rebasing_buffer) {
+static void rebase_buffer(RKernelCacheObj *obj, ut64 off, RIODesc *fd, ut8 *buf, int count) {
+	if (obj->rebasing_buffer || !buf) {
 		return;
 	}
 	obj->rebasing_buffer = true;
 
-	ut64 off = io->off;
 	ut64 eob = off + count;
 	int i = 0;
 	RRebaseCtx ctx;
@@ -1960,6 +2097,99 @@ static void rebase_buffer(RKernelCacheObj *obj, RIO *io, RIODesc *fd, ut8 *buf,
 	obj->rebasing_buffer = false;
 }
 
+static void rebase_buffer_fixup(RKernelCacheObj *kobj, ut64 off, RIODesc *fd, ut8 *buf, int count) {
+	if (kobj->rebasing_buffer) {
+		return;
+	}
+	kobj->rebasing_buffer = true;
+	struct MACH0_(obj_t) *obj = kobj->mach0;
+	ut64 eob = off + count;
+	size_t i = 0;
+	for (; i < obj->nsegs; i++) {
+		if (!obj->chained_starts[i]) {
+			continue;
+		}
+		ut64 page_size = obj->chained_starts[i]->page_size;
+		ut64 start = obj->segs[i].fileoff;
+		ut64 end = start + obj->segs[i].filesize;
+		if (end >= off && start <= eob) {
+			ut64 page_idx = (R_MAX (start, off) - start) / page_size;
+			ut64 page_end_idx = (R_MIN (eob, end) - start) / page_size;
+			for (; page_idx <= page_end_idx; page_idx++) {
+				if (page_idx >= obj->chained_starts[i]->page_count) {
+					break;
+				}
+				ut16 page_start = obj->chained_starts[i]->page_start[page_idx];
+				if (page_start == DYLD_CHAINED_PTR_START_NONE) {
+					continue;
+				}
+				ut64 cursor = start + page_idx * page_size + page_start;
+				while (cursor < eob && cursor < end) {
+					ut8 tmp[8];
+					if (r_buf_read_at (obj->b, cursor, tmp, 8) != 8) {
+						break;
+					}
+					ut64 raw_ptr = r_read_le64 (tmp);
+					ut64 ptr_value = raw_ptr;
+					ut64 delta = 0;
+					ut64 stride = 8;
+					if (obj->chained_starts[i]->pointer_format == DYLD_CHAINED_PTR_ARM64E) {
+						bool is_auth = IS_PTR_AUTH (raw_ptr);
+						bool is_bind = IS_PTR_BIND (raw_ptr);
+						if (is_auth && is_bind) {
+							struct dyld_chained_ptr_arm64e_auth_bind *p =
+									(struct dyld_chained_ptr_arm64e_auth_bind *) &raw_ptr;
+							delta = p->next;
+						} else if (!is_auth && is_bind) {
+							struct dyld_chained_ptr_arm64e_bind *p =
+									(struct dyld_chained_ptr_arm64e_bind *) &raw_ptr;
+							delta = p->next;
+						} else if (is_auth && !is_bind) {
+							struct dyld_chained_ptr_arm64e_auth_rebase *p =
+									(struct dyld_chained_ptr_arm64e_auth_rebase *) &raw_ptr;
+							delta = p->next;
+							ptr_value = p->target + obj->baddr;
+						} else {
+							struct dyld_chained_ptr_arm64e_rebase *p =
+									(struct dyld_chained_ptr_arm64e_rebase *) &raw_ptr;
+							delta = p->next;
+							ptr_value = ((ut64)p->high8 << 56) | p->target;
+							ptr_value += obj->baddr;
+						}
+					} else if (obj->chained_starts[i]->pointer_format == DYLD_CHAINED_PTR_64_KERNEL_CACHE ||
+							obj->chained_starts[i]->pointer_format == DYLD_CHAINED_PTR_ARM64E_KERNEL) {
+						bool is_auth = IS_PTR_AUTH (raw_ptr);
+						stride = 4;
+						if (is_auth) {
+							struct dyld_chained_ptr_arm64e_cache_auth_rebase *p =
+									(struct dyld_chained_ptr_arm64e_cache_auth_rebase *) &raw_ptr;
+							delta = p->next;
+							ptr_value = p->target + obj->baddr;
+						} else {
+							struct dyld_chained_ptr_arm64e_cache_rebase *p =
+									(struct dyld_chained_ptr_arm64e_cache_rebase *) &raw_ptr;
+							delta = p->next;
+							ptr_value = ((ut64)p->high8 << 56) | p->target;
+							ptr_value += obj->baddr;
+						}
+					} else {
+						eprintf ("Unsupported pointer format: %u\n", obj->chained_starts[i]->pointer_format);
+					}
+					ut64 in_buf = cursor - off;
+					if (cursor >= off && cursor <= eob - 8) {
+						r_write_le64 (&buf[in_buf], ptr_value);
+					}
+					cursor += delta * stride;
+					if (!delta) {
+						break;
+					}
+				}
+			}
+		}
+	}
+	kobj->rebasing_buffer = false;
+}
+
 static bool on_rebase_pointer (ut64 offset, ut64 decorated_addr, RRebaseCtx *ctx) {
 	if (offset < ctx->off) {
 		return true;
