@@ -38,10 +38,22 @@ R_API bool r_bp_restore_except(RBreakpoint *bp, bool set, ut64 addr) {
 	RListIter *iter;
 	RBreakpointItem *b;
 
+	if (set && bp->bpinmaps) {
+		bp->corebind.syncDebugMaps (bp->corebind.core);
+	}
+
 	r_list_foreach (bp->bps, iter, b) {
 		if (addr && b->addr == addr) {
 			continue;
 		}
+		// Avoid restoring disabled breakpoints
+		if (set && !b->enabled) {
+			continue;
+		}
+		// Check if the breakpoint is in a valid map
+		if (set && bp->bpinmaps && !r_bp_is_valid (bp, b)) {
+			continue;
+		}
 		if (bp->breakpoint && bp->breakpoint (bp, b, set)) {
 			continue;
 		}
