@@ -17,8 +17,8 @@ struct r_bp_plugin_t r_bp_plugin_ppc = {
 	.bps = r_bp_plugin_ppc_bps,
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BP,
 	.data = &r_bp_plugin_ppc,
 	.version = R2_VERSION
