@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2006-2020 - pancake */
+/* radare - LGPL - Copyright 2006-2021 - pancake */
 
 #include "r_config.h"
 
@@ -9,13 +9,32 @@ R_API RConfigNode* r_config_node_new(const char *name, const char *value) {
 		return NULL;
 	}
 	node->name = strdup (name);
-	node->value = strdup (value? value: "");
+	node->value = strdup (r_str_get (value));
 	node->flags = CN_RW | CN_STR;
 	node->i_value = r_num_get (NULL, value);
-	node->options = r_list_new ();
+	node->options = NULL;
 	return node;
 }
 
+R_API char *r_config_node_to_string(RConfigNode *node) {
+	return (node && node->name)? strdup (node->name): NULL;
+}
+
+R_API void r_config_node_purge_options(RConfigNode *node) {
+	if (node->options) {
+		r_list_purge (node->options);
+	} else {
+		node->options = r_list_newf (free);
+	}
+}
+
+R_API void r_config_node_add_option(RConfigNode *node, const char *option) {
+	if (!node->options) {
+		node->options = r_list_newf (free);
+	}
+	r_list_append (node->options, strdup (option));
+}
+
 R_API RConfigNode* r_config_node_clone(RConfigNode *n) {
 	r_return_val_if_fail (n, NULL);
 	RConfigNode *cn = R_NEW0 (RConfigNode);
@@ -23,12 +42,12 @@ R_API RConfigNode* r_config_node_clone(RConfigNode *n) {
 		return NULL;
 	}
 	cn->name = strdup (n->name);
-	cn->desc = n->desc? strdup (n->desc): NULL;
-	cn->value = strdup (n->value? n->value: "");
+	cn->desc = n->desc ? strdup (n->desc) : NULL;
+	cn->value = strdup (r_str_get (n->value));
 	cn->i_value = n->i_value;
 	cn->flags = n->flags;
 	cn->setter = n->setter;
-	cn->options = r_list_clone (n->options);
+	cn->options = n->options? r_list_clone (n->options): NULL;
 	return cn;
 }
 
@@ -86,7 +105,7 @@ static void config_print_node(RConfig *cfg, RConfigNode *node, const char *pfx,
 				free (es);
 			}
 			cfg->cb_printf ("\"ro\":%s", r_str_bool (r_config_node_is_ro (node)));
-			if (!r_list_empty (node->options)) {
+			if (node->options && !r_list_empty (node->options)) {
 				isFirst = true;
 				cfg->cb_printf (",\"options\":[");
 				r_list_foreach (node->options, iter, option) {
@@ -111,10 +130,10 @@ static void config_print_node(RConfig *cfg, RConfigNode *node, const char *pfx,
 	} else {
 		if (verbose) {
 			cfg->cb_printf ("%s%s = %s%s %s; %s", pfx,
-				node->name, node->value, sfx, 
+				node->name, node->value, sfx,
 				r_config_node_is_ro (node) ? "(ro)" : "",
 				node->desc);
-			if (!r_list_empty (node->options)) {
+			if (node->options && !r_list_empty (node->options)) {
 				isFirst = true;
 				cfg->cb_printf(" [");
 				r_list_foreach (node->options, iter, option) {
@@ -182,7 +201,7 @@ R_API void r_config_list(RConfig *cfg, const char *str, int rad) {
 			if (!str || (str && (!strncmp (str, node->name, len)))) {
 				if (!str || !strncmp (str, node->name, len)) {
 					cfg->cb_printf ("%20s: %s\n", node->name,
-						node->desc? node->desc: "");
+						r_str_get (node->desc));
 				}
 			}
 		}
@@ -233,7 +252,9 @@ R_API void r_config_list(RConfig *cfg, const char *str, int rad) {
 		break;
 	case 'q':
 		r_list_foreach (cfg->nodes, iter, node) {
-			cfg->cb_printf ("%s\n", node->name);
+			if (!str || (str && (!strncmp (str, node->name, len)))) {
+				cfg->cb_printf ("%s\n", node->name);
+			}
 		}
 		break;
 	case 'J':
@@ -272,25 +293,6 @@ R_API RConfigNode* r_config_node_get(RConfig *cfg, const char *name) {
 	return ht_pp_find (cfg->ht, name, NULL);
 }
 
-R_API bool r_config_set_getter(RConfig *cfg, const char *key, RConfigCallback cb) {
-	r_return_val_if_fail (cfg && key, false);
-	RConfigNode *node = r_config_node_get (cfg, key);
-	if (node) {
-		node->getter = cb;
-		return true;
-	}
-	return false;
-}
-
-R_API bool r_config_set_setter(RConfig *cfg, const char *key, RConfigCallback cb) {
-	RConfigNode *node = r_config_node_get (cfg, key);
-	if (node) {
-		node->setter = cb;
-		return true;
-	}
-	return false;
-}
-
 R_API const char* r_config_get(RConfig *cfg, const char *name) {
 	r_return_val_if_fail (cfg && name, NULL);
 	RConfigNode *node = r_config_node_get (cfg, name);
@@ -310,28 +312,44 @@ R_API const char* r_config_get(RConfig *cfg, const char *name) {
 
 R_API bool r_config_toggle(RConfig *cfg, const char *name) {
 	RConfigNode *node = r_config_node_get (cfg, name);
-	if (node && r_config_node_is_bool (node)) {
-		(void)r_config_set_i (cfg, name, !node->i_value);
-		return true;
+	if (!node) {
+		return false;
 	}
-	return false;
+	if (!r_config_node_is_bool (node)) {
+		eprintf ("(error: '%s' is not a boolean variable)\n", name);
+		return false;
+	}
+	if (r_config_node_is_ro (node)) {
+		eprintf ("(error: '%s' config key is read only)\n", name);
+		return false;
+	}
+	(void)r_config_set_b (cfg, name, !node->i_value);
+	return true;
+}
+
+R_API bool r_config_get_b(RConfig *cfg, const char *name) {
+	return r_config_get_i (cfg, name) != 0;
 }
 
 R_API ut64 r_config_get_i(RConfig *cfg, const char *name) {
+	r_return_val_if_fail (cfg, 0ULL);
 	RConfigNode *node = r_config_node_get (cfg, name);
 	if (node) {
 		if (node->getter) {
 			node->getter (cfg->user, node);
 		}
-		if (node->i_value || !strcmp (node->value, "false")) {
+		if (node->i_value) {
 			return node->i_value;
 		}
+		if (!strcmp (node->value, "false")) {
+			return 0;
+		}
 		if (!strcmp (node->value, "true")) {
 			return 1;
 		}
 		return (ut64) r_num_math (cfg->num, node->value);
 	}
-	return (ut64) 0LL;
+	return 0ULL;
 }
 
 R_API const char* r_config_node_type(RConfigNode *node) {
@@ -355,9 +373,7 @@ R_API const char* r_config_node_type(RConfigNode *node) {
 R_API RConfigNode* r_config_set_cb(RConfig *cfg, const char *name, const char *value, RConfigCallback cb) {
 	RConfigNode *node = r_config_set (cfg, name, value);
 	if (node && (node->setter = cb)) {
-		if (!cb (cfg->user, node)) {
-			return NULL;
-		}
+		(void)cb (cfg->user, node);
 	}
 	return node;
 }
@@ -372,7 +388,7 @@ R_API RConfigNode* r_config_set_i_cb(RConfig *cfg, const char *name, int ivalue,
 	return node;
 }
 
-static bool __is_true_or_false(const char *s) {
+static inline bool is_true_or_false(const char *s) {
 	return s && (!r_str_casecmp (s, "true") || !r_str_casecmp (s, "false"));
 }
 
@@ -395,15 +411,14 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 		if (node->value) {
 			ov = strdup (node->value);
 			if (!ov) {
-				goto beach;
+				return node;
 			}
 		} else {
-			free (node->value);
 			node->value = strdup ("");
 		}
 		if (r_config_node_is_bool (node)) {
 			bool b = r_str_is_true (value);
-			node->i_value = b? 1: 0;
+			node->i_value = b;
 			char *value = strdup (r_str_bool (b));
 			if (value) {
 				free (node->value);
@@ -437,7 +452,7 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 		if (!cfg->lock) {
 			node = r_config_node_new (name, value);
 			if (node) {
-				if (__is_true_or_false (value)) {
+				if (is_true_or_false (value)) {
 					node->flags |= CN_BOOL;
 					node->i_value = r_str_is_true (value)? 1: 0;
 				}
@@ -457,8 +472,10 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 				node->i_value = oi;
 			}
 			free (node->value);
-			node->value = strdup (ov? ov: "");
-			free (ov);
+			node->value = strdup (r_str_get (ov));
+			if (ov) {
+				free (ov);
+			}
 			return NULL;
 		}
 	}
@@ -470,18 +487,18 @@ beach:
 /* r_config_desc takes a RConfig and a name,
  * r_config_node_desc takes a RConfigNode
  * Both set and return node->desc */
-R_API const char* r_config_desc(RConfig *cfg, const char *name, const char *desc) {
+R_API RConfigNode * r_config_desc(RConfig *cfg, const char *name, const char *desc) {
 	RConfigNode *node = r_config_node_get (cfg, name);
 	return r_config_node_desc (node, desc);
 }
 
-R_API const char* r_config_node_desc(RConfigNode *node, const char *desc) {
+R_API RConfigNode* r_config_node_desc(RConfigNode *node, const char *desc) {
 	r_return_val_if_fail (node, NULL);
 	if (desc) {
 		free (node->desc);
 		node->desc = strdup (desc);
 	}
-	return node->desc;
+	return node;
 }
 
 R_API bool r_config_rm(RConfig *cfg, const char *name) {
@@ -506,6 +523,14 @@ R_API void r_config_node_value_format_i(char *buf, size_t buf_size, const ut64 i
 	}
 }
 
+R_API RConfigNode* r_config_set_b(RConfig *cfg, const char *name, bool b) {
+	RConfigNode *node = r_config_node_get (cfg, name);
+	if (node && r_config_node_is_bool (node)) {
+		return r_config_set_i (cfg, name, b? 1: 0);
+	}
+	return NULL;
+}
+
 R_API RConfigNode* r_config_set_i(RConfig *cfg, const char *name, const ut64 i) {
 	char buf[128], *ov = NULL;
 	r_return_val_if_fail (cfg && name, NULL);
@@ -620,9 +645,9 @@ static int cmp(RConfigNode *a, RConfigNode *b) {
 	return strcmp (a->name, b->name);
 }
 
-R_API void r_config_lock(RConfig *cfg, int l) {
+R_API void r_config_lock(RConfig *cfg, bool lock) {
 	r_list_sort (cfg->nodes, (RListComparator) cmp);
-	cfg->lock = l;
+	cfg->lock = lock;
 }
 
 R_API bool r_config_readonly(RConfig *cfg, const char *key) {
@@ -647,7 +672,7 @@ R_API RConfig* r_config_new(void *user) {
 	}
 	cfg->user = user;
 	cfg->num = NULL;
-	cfg->lock = 0;
+	cfg->lock = false;
 	cfg->cb_printf = (void *) printf;
 	return cfg;
 }
@@ -701,14 +726,13 @@ R_API void r_config_serialize(R_NONNULL RConfig *config, R_NONNULL Sdb *db) {
 	}
 }
 
-static int load_config_cb(void *user, const char *k, const char *v) {
+static bool load_config_cb(void *user, const char *k, const char *v) {
 	RConfig *config = user;
 	RConfigNode *node = r_config_node_get (config, k);
-	if (!node) {
-		return 1;
+	if (node) {
+		r_config_set (config, k, v);
 	}
-	r_config_set (config, k, v);
-	return 1;
+	return true;
 }
 
 R_API bool r_config_unserialize(R_NONNULL RConfig *config, R_NONNULL Sdb *db, R_NULLABLE char **err) {
