@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2013-2019 - pancake */
+/* radare - LGPL - Copyright 2013-2021 - pancake */
 
 #include <r_cons.h>
 #include <r_util/r_assert.h>
@@ -115,7 +115,7 @@ static bool __expandLine(RConsCanvas *c, int real_len, int utf8_len) {
 
 	if (padding) {
 		if (padding > 0 && c->blen[c->y] + padding > c->bsize[c->y]) {
-			int newsize = R_MAX (c->bsize[c->y] * 1.5, c->blen[c->y] + padding);
+			int newsize = R_MAX ((int)(c->bsize[c->y] * 1.5), c->blen[c->y] + padding);
 			char * newline = realloc (c->b[c->y], sizeof (*c->b[c->y])*(newsize));
 			if (!newline) {
 				return false;
@@ -212,7 +212,7 @@ R_API bool r_cons_canvas_gotoxy(RConsCanvas *c, int x, int y) {
 	if (x < c->blen[y] && x >= 0) {
 		c->x = x;
 	}
-	if (y < c->h && y >= 0) {
+	if (y < c->h) {
 		c->y = y;
 	}
 	return ret;
@@ -265,7 +265,7 @@ R_API RConsCanvas *r_cons_canvas_new(int w, int h) {
 	c->attr = Color_RESET;
 	r_cons_canvas_clear (c);
 	return c;
-beach: {
+beach:
 	r_str_constpool_fini (&c->constpool);
 	int j;
 	for (j = 0; j < i; j++) {
@@ -276,7 +276,6 @@ beach: {
 	free (c->b);
 	free (c);
 	return NULL;
-       }
 }
 
 R_API void r_cons_canvas_write(RConsCanvas *c, const char *s) {
@@ -323,13 +322,13 @@ R_API void r_cons_canvas_write(RConsCanvas *c, const char *s) {
 
 		attr_len = slen <= 0 && s_part != s? 1: utf8_len;
 		if (attr_len > 0 && attr_x < c->blen[c->y]) {
-			__stampAttribute (c, c->y*c->w + attr_x, attr_len);
+			__stampAttribute (c, c->y * c->w + attr_x, attr_len);
 		}
 
 		s = s_part;
 		if (ch == '\n') {
 			c->attr = Color_RESET;
-			__stampAttribute (c, c->y*c->w + attr_x, 0);
+			__stampAttribute (c, c->y * c->w + attr_x, 0);
 			c->y++;
 			s++;
 			if (*s == '\0' || c->y  >= c->h) {
@@ -351,7 +350,7 @@ R_API char *r_cons_canvas_to_string(RConsCanvas *c) {
 	r_return_val_if_fail (c, NULL);
 
 	int x, y, olen = 0, attr_x = 0;
-	int is_first = true;
+	bool is_first = true;
 
 	for (y = 0; y < c->h; y++) {
 		olen += c->blen[y] + 1;
@@ -376,7 +375,7 @@ R_API char *r_cons_canvas_to_string(RConsCanvas *c) {
 			if ((c->b[y][x] & 0xc0) != 0x80) {
 				const char *atr = __attributeAt (c, y * c->w + attr_x);
 				if (atr) {
-					int len = strlen (atr);
+					size_t len = strlen (atr);
 					memcpy (o + olen, atr, len);
 					olen += len;
 				}
@@ -391,8 +390,9 @@ R_API char *r_cons_canvas_to_string(RConsCanvas *c) {
 			}
 			const char *rune = r_cons_get_rune ((const ut8)c->b[y][x]);
 			if (rune) {
-				strcpy (o + olen, rune);
-				olen += strlen (rune);
+				size_t rune_len = strlen (rune);
+				memcpy (o + olen, rune, rune_len + 1);
+				olen += rune_len;
 			} else {
 				o[olen++] = c->b[y][x];
 			}
@@ -452,12 +452,12 @@ R_API int r_cons_canvas_resize(RConsCanvas *c, int w, int h) {
 		if (i < c->h) {
 			newline = realloc (c->b[i], sizeof *c->b[i] * (w + 1));
 		} else {
-			newline = malloc ((w + 1));
+			newline = malloc (w + 1);
 		}
 		c->blen[i] = w;
 		c->bsize[i] = w + 1;
 		if (!newline) {
-			int j;
+			size_t j;
 			for (j = 0; j <= i; j++) {
 				free (c->b[i]);
 			}
@@ -487,15 +487,15 @@ R_API void r_cons_canvas_circle(RConsCanvas *c, int x, int y, int w, int h, cons
 	double xfactor = 1; //(double)w / (double)h;
 	double yfactor = (double)h / 24; // 0.8; // 24  10
 	double size = w;
-	float a = 0.0;
+	double a = 0.0;
 	double s = size / 2;
 	while (a < (2 * PI)) {
 		double sa = r_num_sin (a);
 		double ca = r_num_cos (a);
 		double cx = s * ca + (size / 2);
 		double cy = s * sa + (size / 4);
-		int X = x + (xfactor * cx) - 2;
-		int Y = y + ((yfactor/2) * cy);
+		int X = x + (int)(xfactor * cx) - 2;
+		int Y = y + (int)((yfactor/2) * cy);
 		if (G (X, Y)) {
 			W ("=");
 		}
