@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2019 - pancake */
+/* radare - LGPL - Copyright 2007-2021 - pancake */
 /* dietline is a lightweight and portable library similar to GNU readline */
 
 #include <r_cons.h>
@@ -13,7 +13,9 @@
 static int r_line_readchar_win(ut8 *s, int slen);
 #else
 #include <sys/ioctl.h>
+#ifndef HAVE_PTY
 #include <termios.h>
+#endif
 #include <signal.h>
 #define USE_UTF8 1
 #endif
@@ -46,7 +48,7 @@ static inline bool is_word_break_char(char ch, bool mode) {
 }
 
 /* https://www.gnu.org/software/bash/manual/html_node/Commands-For-Killing.html */
-static void backward_kill_word() {
+static void backward_kill_word(void) {
 	int i, len;
 	if (I.buffer.index > 0) {
 		for (i = I.buffer.index; i > 0 && is_word_break_char (I.buffer.data[i], MINOR_BREAK); i--) {
@@ -74,7 +76,7 @@ static void backward_kill_word() {
 	}
 }
 
-static void backward_kill_Word() {
+static void backward_kill_Word(void) {
 	int i, len;
 	if (I.buffer.index > 0) {
 		for (i = I.buffer.index; i > 0 && is_word_break_char (I.buffer.data[i], MAJOR_BREAK); i--) {
@@ -102,7 +104,7 @@ static void backward_kill_Word() {
 	}
 }
 
-static void kill_word() {
+static void kill_word(void) {
 	int i, len;
 	for (i = I.buffer.index; i < I.buffer.length && is_word_break_char (I.buffer.data[i], MINOR_BREAK); i++) {
 		/* Move the cursor index forward until we hit a non-word-break-character */
@@ -121,7 +123,7 @@ static void kill_word() {
 	I.buffer.length = strlen (I.buffer.data);
 }
 
-static void kill_Word() {
+static void kill_Word(void) {
 	int i, len;
 	for (i = I.buffer.index; i < I.buffer.length && is_word_break_char (I.buffer.data[i], MAJOR_BREAK); i++) {
 		/* Move the cursor index forward until we hit a non-word-break-character */
@@ -140,7 +142,7 @@ static void kill_Word() {
 	I.buffer.length = strlen (I.buffer.data);
 }
 
-static void paste() {
+static void paste(void) {
 	if (I.clipboard) {
 		char *cursor = I.buffer.data + I.buffer.index;
 		int dist = (I.buffer.data + I.buffer.length) - cursor;
@@ -153,7 +155,7 @@ static void paste() {
 	}
 }
 
-static void unix_word_rubout() {
+static void unix_word_rubout(void) {
 	int i, len;
 	if (I.buffer.index > 0) {
 		for (i = I.buffer.index - 1; i > 0 && I.buffer.data[i] == ' '; i--) {
@@ -184,7 +186,7 @@ static void unix_word_rubout() {
 	}
 }
 
-static int inithist() {
+static int inithist(void) {
 	ZERO_FILL (I.history);
 	if ((I.history.size + 1024) * sizeof (char *) < I.history.size) {
 		return false;
@@ -198,7 +200,7 @@ static int inithist() {
 }
 
 /* initialize history stuff */
-R_API int r_line_dietline_init() {
+R_API int r_line_dietline_init(void) {
 	ZERO_FILL (I.completion);
 	if (!inithist ()) {
 		return false;
@@ -259,29 +261,35 @@ static int r_line_readchar_utf8(ut8 *s, int slen) {
 
 #if __WINDOWS__
 static int r_line_readchar_win(ut8 *s, int slen) { // this function handle the input in console mode
-	INPUT_RECORD irInBuf;
-	BOOL ret, bCtrl = FALSE;
+	r_sys_backtrace();
+	INPUT_RECORD irInBuf = { { 0 } };
+	BOOL ret;
 	DWORD mode, out;
 	char buf[5] = {0};
-	HANDLE h;
 	void *bed;
 
+	HANDLE h = GetStdHandle (STD_INPUT_HANDLE);
+	DWORD new_mode = I.vtmode == 2 ? ENABLE_VIRTUAL_TERMINAL_INPUT : 0;
+	GetConsoleMode (h, &mode);
+	SetConsoleMode (h, new_mode);
 	if (I.zerosep) {
 		bed = r_cons_sleep_begin ();
-		int rsz = read (0, s, 1);
+		DWORD rsz = 0;
+		BOOL ret = ReadFile (h, s, 1, &rsz, NULL);
 		r_cons_sleep_end (bed);
-		if (rsz != 1) {
+		SetConsoleMode (h, mode);
+		if (!ret || rsz != 1) {
 			return 0;
 		}
 		return 1;
 	}
-
-	h = GetStdHandle (STD_INPUT_HANDLE);
-	GetConsoleMode (h, &mode);
-	SetConsoleMode (h, 0);	// RAW
 do_it_again:
 	bed = r_cons_sleep_begin ();
-	ret = ReadConsoleInput (h, &irInBuf, 1, &out);
+	if (r_cons_singleton()->term_xterm) {
+		ret = ReadFile (h, buf, 1, &out, NULL);
+	} else {
+		ret = ReadConsoleInput (h, &irInBuf, 1, &out);
+	}
 	r_cons_sleep_end (bed);
 	if (ret < 1) {
 		return 0;
@@ -289,19 +297,19 @@ do_it_again:
 	if (irInBuf.EventType == KEY_EVENT) {
 		if (irInBuf.Event.KeyEvent.bKeyDown) {
 			if (irInBuf.Event.KeyEvent.uChar.UnicodeChar) {
-				char *tmp = r_sys_conv_win_to_utf8_l ((PTCHAR)&irInBuf.Event.KeyEvent.uChar, 1);
-				if (!tmp) {
-					return 0;
+				ut8 chbuf[4] = {0};
+				memcpy (chbuf, &(irInBuf.Event.KeyEvent.uChar), 2);
+				char *tmp = r_sys_conv_win_to_utf8_l ((PTCHAR)&chbuf, 1);
+				if (tmp) {
+					r_str_ncpy (buf, tmp, sizeof (buf));
+					free (tmp);
 				}
-				strncpy_s (buf, sizeof (buf), tmp, strlen (tmp));
-				free (tmp);
 			} else {
 				int idx = 0;
 				buf[idx++] = 27;
 				buf[idx++] = '['; // Simulate escaping
-				bCtrl = irInBuf.Event.KeyEvent.dwControlKeyState & 8;
-				if (bCtrl) {
-					buf[idx++] = 0x31;
+				if (irInBuf.Event.KeyEvent.dwControlKeyState & 8) {
+					buf[idx++] = '1'; // control key
 				}
 				switch (irInBuf.Event.KeyEvent.wVirtualKeyCode) {
 				case VK_UP: buf[idx++] = 'A'; break;
@@ -321,7 +329,7 @@ do_it_again:
 	if (!buf[0]) {
 		goto do_it_again;
 	}
-	strncpy_s ((char *)s, slen, buf, sizeof (buf));
+	r_str_ncpy ((char *)s, buf, slen);
 	SetConsoleMode (h, mode);
 	return strlen ((char *)s);
 }
@@ -337,6 +345,26 @@ R_API int r_line_set_hist_callback(RLine *line, RLineHistoryUpCb up, RLineHistor
 	return 1;
 }
 
+static inline bool match_hist_line(RLine *line, int i) {
+	const char *hist_line = line->history.data[i];
+	char *cur_line = line->history.match;
+	if (!line->histfilter) {
+		return true;
+	}
+	// Starts with but not equal to
+	return r_str_startswith (hist_line, cur_line) && strcmp (hist_line, cur_line);
+}
+
+static void setup_hist_match(RLine *line) {
+	if (line->history.do_setup_match) {
+		R_FREE (line->history.match);
+		if (*line->buffer.data) {
+			line->history.match = strdup (line->buffer.data);
+		}
+	}
+	line->history.do_setup_match = false;
+}
+
 R_API int r_line_hist_cmd_up(RLine *line) {
 	if (line->hist_up) {
 		return line->hist_up (line->user);
@@ -345,7 +373,22 @@ R_API int r_line_hist_cmd_up(RLine *line) {
 		inithist ();
 	}
 	if (line->history.index > 0 && line->history.data) {
-		strncpy (line->buffer.data, line->history.data[--line->history.index], R_LINE_BUFSIZE - 1);
+		setup_hist_match (line);
+		if (line->history.match) {
+			int i;
+			for (i= line->history.index - 1; i >= 0; i--) {
+				if (match_hist_line (line, i)) {
+					line->history.index = i;
+					break;
+				}
+			}
+			if (i < 0) {
+				return false;
+			}
+		} else {
+			line->history.index--;
+		}
+		strncpy (line->buffer.data, line->history.data[line->history.index], R_LINE_BUFSIZE - 1);
 		line->buffer.index = line->buffer.length = strlen (line->buffer.data);
 		return true;
 	}
@@ -356,17 +399,29 @@ R_API int r_line_hist_cmd_down(RLine *line) {
 	if (line->hist_down) {
 		return line->hist_down (line->user);
 	}
-	line->buffer.index = 0;
 	if (!line->history.data) {
 		inithist ();
 	}
-	if (line->history.index == line->history.top) {
-		return false;
+	setup_hist_match (line);
+	if (line->history.match) {
+		int i;
+		for (i = line->history.index + 1; i < line->history.top; i++) {
+			if (match_hist_line (line, i)) {
+				break;
+			}
+		}
+		line->history.index = i;
+	} else {
+		line->history.index++;
 	}
-	line->history.index++;
-	if (line->history.index == line->history.top) {
-		line->buffer.data[0] = '\0';
-		line->buffer.index = line->buffer.length = 0;
+	if (line->history.index >= line->history.top) {
+		line->history.index = line->history.top;
+		if (line->history.match) {
+			strncpy (line->buffer.data, line->history.match, R_LINE_BUFSIZE - 1);
+		} else {
+			line->buffer.data[0] = '\0';
+		}
+		line->buffer.index = line->buffer.length = strlen (line->buffer.data);
 		return false;
 	}
 	if (line->history.data && line->history.data[line->history.index]) {
@@ -404,14 +459,14 @@ R_API int r_line_hist_add(const char *line) {
 	return true;
 }
 
-static int r_line_hist_up() {
+static int r_line_hist_up(void) {
 	if (!I.cb_history_up) {
 		r_line_set_hist_callback (&I, &r_line_hist_cmd_up, &r_line_hist_cmd_down);
 	}
 	return I.cb_history_up (&I);
 }
 
-static int r_line_hist_down() {
+static int r_line_hist_down(void) {
 	if (!I.cb_history_down) {
 		r_line_set_hist_callback (&I, &r_line_hist_cmd_up, &r_line_hist_cmd_down);
 	}
@@ -434,7 +489,7 @@ R_API const char *r_line_hist_get(int n) {
 	return NULL;
 }
 
-R_API int r_line_hist_list() {
+R_API int r_line_hist_list(void) {
 	int i = 0;
 	if (!I.history.data) {
 		inithist ();
@@ -448,7 +503,7 @@ R_API int r_line_hist_list() {
 	return i;
 }
 
-R_API void r_line_hist_free() {
+R_API void r_line_hist_free(void) {
 	int i;
 	if (I.history.data) {
 		for (i = 0; i < I.history.size; i++) {
@@ -480,25 +535,23 @@ R_API int r_line_hist_load(const char *file) {
 	return true;
 }
 
-R_API int r_line_hist_save(const char *file) {
-	FILE *fd;
-	int i, ret = false;
-	if (!file || !*file) {
-		return false;
-	}
+R_API bool r_line_hist_save(const char *file) {
+	r_return_val_if_fail (file && *file, false);
+	int i;
+	bool ret = false;
 	char *p, *path = r_str_home (file);
-	if (path != NULL) {
+	if (path) {
 		p = (char *) r_str_lastbut (path, R_SYS_DIR[0], NULL);	// TODO: use fs
 		if (p) {
 			*p = 0;
 			if (!r_sys_mkdirp (path)) {
-				eprintf ("could not save history into %s\n", path);
+				eprintf ("Could not save history into %s\n", path);
 				goto end;
 			}
 			*p = R_SYS_DIR[0];
 		}
-		fd = r_sandbox_fopen (path, "w");
-		if (fd != NULL) {
+		FILE *fd = r_sandbox_fopen (path, "w");
+		if (fd) {
 			if (I.history.data) {
 				for (i = 0; i < I.history.index; i++) {
 					fputs (I.history.data[i], fd);
@@ -521,7 +574,7 @@ R_API int r_line_hist_chop(const char *file, int limit) {
 	return 0;
 }
 
-static void selection_widget_draw() {
+static void selection_widget_draw(void) {
 	RCons *cons = r_cons_singleton ();
 	RSelWidget *sel_widget = I.sel_widget;
 	int y, pos_y, pos_x = r_str_ansi_len (I.prompt);
@@ -560,14 +613,14 @@ static void selection_widget_draw() {
 		r_cons_printf ("%s", sel_widget->selection == y + scroll ? selected_color : background_color);
 		r_cons_printf ("%-*.*s", sel_widget->w, sel_widget->w, option);
 		if (scrollbar && R_BETWEEN (scrollbar_y, y, scrollbar_y + scrollbar_l)) {
-			r_cons_memcat (Color_INVERT" "Color_INVERT_RESET, 10);
+			r_cons_write (Color_INVERT" "Color_INVERT_RESET, 10);
 		} else {
-			r_cons_memcat (" ", 1);
+			r_cons_write (" ", 1);
 		}
 	}
 
 	r_cons_gotoxy (pos_x + I.buffer.length, pos_y);
-	r_cons_memcat (Color_RESET_BG, 5);
+	r_cons_write (Color_RESET_BG, 5);
 	r_cons_flush ();
 }
 
@@ -615,13 +668,13 @@ static void selection_widget_down(int steps) {
 	}
 }
 
-static void print_rline_task(void *core) {
+static void print_rline_task(void *_core) {
 	r_cons_clear_line (0);
 	r_cons_printf ("%s%s%s", Color_RESET, I.prompt,  I.buffer.data);
 	r_cons_flush ();
 }
 
-static void selection_widget_erase() {
+static void selection_widget_erase(void) {
 	RSelWidget *sel_widget = I.sel_widget;
 	if (sel_widget) {
 		sel_widget->options_len = 0;
@@ -640,7 +693,7 @@ static void selection_widget_erase() {
 	}
 }
 
-static void selection_widget_select() {
+static void selection_widget_select(void) {
 	RSelWidget *sel_widget = I.sel_widget;
 	if (sel_widget && sel_widget->selection < sel_widget->options_len) {
 		char *sp = strchr (I.buffer.data, ' ');
@@ -659,7 +712,7 @@ static void selection_widget_select() {
 	}
 }
 
-static void selection_widget_update() {
+static void selection_widget_update(void) {
 	int argc = r_pvector_len (&I.completion.args);
 	const char **argv = (const char **)r_pvector_data (&I.completion.args);
 	if (argc == 0 || (argc == 1 && I.buffer.length >= strlen (argv[0]))) {
@@ -686,7 +739,7 @@ static void selection_widget_update() {
 	return;
 }
 
-R_API void r_line_autocomplete() {
+R_API void r_line_autocomplete(void) {
 	char *p;
 	const char **argv = NULL;
 	int argc = 0, i, j, plen, len = 0;
@@ -728,9 +781,9 @@ R_API void r_line_autocomplete() {
 			I.buffer.index, strlen (I.buffer.data), ' ');
 		const char *t = end_word != NULL?
 				end_word: I.buffer.data + I.buffer.index;
-		int largv0 = strlen (argv[0]? argv[0]: "");
+		int largv0 = strlen (r_str_get (argv[0]));
 		size_t len_t = strlen (t);
-		p[largv0]='\0';
+		p[largv0] = '\0';
 
 		if ((p - I.buffer.data) + largv0 + 1 + len_t < plen) {
 			if (len_t > 0) {
@@ -826,11 +879,11 @@ R_API void r_line_autocomplete() {
 	fflush (stdout);
 }
 
-R_API const char *r_line_readline() {
+R_API const char *r_line_readline(void) {
 	return r_line_readline_cb (NULL, NULL);
 }
 
-static inline void rotate_kill_ring() {
+static inline void rotate_kill_ring(void) {
 	if (enable_yank_pop) {
 		I.buffer.index -= strlen (r_list_get_n (I.kill_ring, I.kill_ring_ptr));
 		I.buffer.data[I.buffer.index] = 0;
@@ -843,7 +896,7 @@ static inline void rotate_kill_ring() {
 	}
 }
 
-static inline void __delete_next_char() {
+static inline void __delete_next_char(void) {
 	if (I.buffer.index < I.buffer.length) {
 		int len = r_str_utf8_charsize (I.buffer.data + I.buffer.index);
 		memmove (I.buffer.data + I.buffer.index,
@@ -853,10 +906,10 @@ static inline void __delete_next_char() {
 	}
 }
 
-static inline void __delete_prev_char() {
+static inline void __delete_prev_char(void) {
 	if (I.buffer.index < I.buffer.length) {
 		if (I.buffer.index > 0) {
-			int len = r_str_utf8_charsize_prev (I.buffer.data + I.buffer.index, I.buffer.index);
+			size_t len = r_str_utf8_charsize_prev (I.buffer.data + I.buffer.index, I.buffer.index);
 			I.buffer.index -= len;
 			memmove (I.buffer.data + I.buffer.index,
 				I.buffer.data + I.buffer.index + len,
@@ -876,24 +929,29 @@ static inline void __delete_prev_char() {
 	}
 }
 
-static inline void delete_till_end() {
+static inline void delete_till_end(void) {
 	I.buffer.data[I.buffer.index] = '\0';
 	I.buffer.length = I.buffer.index;
 	I.buffer.index = I.buffer.index > 0 ? I.buffer.index - 1 : 0;
 }
 
-static void __print_prompt() {
+static void __print_prompt(void) {
         RCons *cons = r_cons_singleton ();
 	int columns = r_cons_get_size (NULL) - 2;
-	int chars = R_MAX (1, strlen (I.buffer.data));
 	int len, i, cols = R_MAX (1, columns - r_str_ansi_len (I.prompt) - 2);
 	if (cons->line->prompt_type == R_LINE_PROMPT_OFFSET) {
                 r_cons_gotoxy (0,  cons->rows);
                 r_cons_flush ();
 	}
 	r_cons_clear_line (0);
-	printf ("\r%s%s", Color_RESET, I.prompt);
-	fwrite (I.buffer.data, 1, R_MIN (cols, chars), stdout);
+	if (cons->context->color_mode > 0) {
+		printf ("\r%s%s", Color_RESET, I.prompt);
+	} else {
+		printf ("\r%s", I.prompt);
+	}
+	if (I.buffer.length > 0) {
+		fwrite (I.buffer.data, I.buffer.length, 1, stdout);
+	}
 	printf ("\r%s", I.prompt);
 	if (I.buffer.index > cols) {
 		printf ("< ");
@@ -906,24 +964,29 @@ static void __print_prompt() {
 	}
 	len = I.buffer.index - i;
 	if (len > 0 && (i + len) <= I.buffer.length) {
-		fwrite (I.buffer.data + i, 1, len, stdout);
+		if (i < I.buffer.length) {
+			size_t slen = R_MIN (len, (I.buffer.length - i));
+			if (slen > 0 && i < sizeof (I.buffer.data)) {
+				fwrite (I.buffer.data + i, 1, slen, stdout);
+			}
+		}
 	}
 	fflush (stdout);
 }
 
-static inline void __move_cursor_right() {
+static inline void __move_cursor_right(void) {
 	I.buffer.index = I.buffer.index < I.buffer.length
 		? I.buffer.index + r_str_utf8_charsize (I.buffer.data + I.buffer.index)
 		: I.buffer.length;
 }
 
-static inline void __move_cursor_left() {
+static inline void __move_cursor_left(void) {
 	I.buffer.index = I.buffer.index
 		? I.buffer.index - r_str_utf8_charsize_prev (I.buffer.data + I.buffer.index, I.buffer.index)
 		: 0;
 }
 
-static inline void vi_cmd_b() {
+static inline void vi_cmd_b(void) {
 	int i;
 	for (i = I.buffer.index - 2; i >= 0; i--) {
 		if ((is_word_break_char (I.buffer.data[i], MINOR_BREAK)
@@ -939,7 +1002,7 @@ static inline void vi_cmd_b() {
 	}
 }
 
-static inline void vi_cmd_B() {
+static inline void vi_cmd_B(void) {
 	int i;
 	for (i = I.buffer.index - 2; i >= 0; i--) {
 		if ((!is_word_break_char (I.buffer.data[i], MAJOR_BREAK)
@@ -953,7 +1016,7 @@ static inline void vi_cmd_B() {
 	}
 }
 
-static inline void vi_cmd_W() {
+static inline void vi_cmd_W(void) {
 	int i;
 	for (i = I.buffer.index + 1; i < I.buffer.length; i++) {
 		if ((!is_word_break_char (I.buffer.data[i], MAJOR_BREAK)
@@ -967,7 +1030,7 @@ static inline void vi_cmd_W() {
 	}
 }
 
-static inline void vi_cmd_w() {
+static inline void vi_cmd_w(void) {
 	int i;
 	for (i = I.buffer.index + 1; i < I.buffer.length; i++) {
 		if ((!is_word_break_char (I.buffer.data[i], MINOR_BREAK)
@@ -983,7 +1046,7 @@ static inline void vi_cmd_w() {
 	}
 }
 
-static inline void vi_cmd_E() {
+static inline void vi_cmd_E(void) {
 	int i;
 	for (i = I.buffer.index + 1; i < I.buffer.length; i++) {
 		if ((!is_word_break_char (I.buffer.data[i], MAJOR_BREAK)
@@ -997,7 +1060,7 @@ static inline void vi_cmd_E() {
 	}
 }
 
-static inline void vi_cmd_e() {
+static inline void vi_cmd_e(void) {
 	int i;
 	for (i = I.buffer.index + 1; i < I.buffer.length; i++) {
 		if ((!is_word_break_char (I.buffer.data[i], MINOR_BREAK)
@@ -1013,7 +1076,7 @@ static inline void vi_cmd_e() {
 	}
 }
 
-static void __update_prompt_color () {
+static void __update_prompt_color (void) {
 	RCons *cons = r_cons_singleton ();
 	const char *BEGIN = "", *END = "";
 	if (cons->context->color_mode) {
@@ -1037,7 +1100,7 @@ static void __update_prompt_color () {
 	I.prompt = r_str_newf ("%s%s%s", BEGIN, prompt, END);
 }
 
-static void __vi_mode() {
+static void __vi_mode(void) {
 	char ch;
 	I.vi_mode = CONTROL_MODE;
 	__update_prompt_color ();
@@ -1048,10 +1111,12 @@ static void __vi_mode() {
 		if (I.echo) {
 			__print_prompt ();
 		}
-		if (I.vi_mode != CONTROL_MODE) {		// exit if insert mode is selected
+		if (I.vi_mode != CONTROL_MODE) { // exit if insert mode is selected
 			__update_prompt_color ();
 			break;
 		}
+		bool o_do_setup_match = I.history.do_setup_match;
+		I.history.do_setup_match = true;
 		ch = r_cons_readchar ();
 		while (IS_DIGIT (ch)) {			// handle commands like 3b
 			if (ch == '0' && rep == 0) {	// to handle the command 0
@@ -1082,13 +1147,17 @@ static void __vi_mode() {
 		case 'r': {
 			char c =  r_cons_readchar ();
 			I.buffer.data[I.buffer.index] = c;
-			} break;
+			break;
+		}
 		case 'x':
 			while (rep--) {
 				__delete_next_char ();
-			} break;
-		case 'c':
-			I.vi_mode = INSERT_MODE;			// goto insert mode
+			}
+			break;
+		case 'c': {
+			I.vi_mode = INSERT_MODE;	// goto insert mode
+			break;
+		}
 		case 'd': {
 			char c = r_cons_readchar ();
 			while (rep--) {
@@ -1105,7 +1174,8 @@ static void __vi_mode() {
 					if (I.hud) {
 						I.hud->vi = false;
 					}
-					} break;
+					break;
+				}
 				case 'W':
 					kill_Word ();
 					break;
@@ -1134,8 +1204,9 @@ static void __vi_mode() {
 					I.buffer.index = 0;
 					break;
 				} __print_prompt ();
-			}
-			} break;
+			} // end of while (rep--)
+			break;
+		} // end of case 'd'
 		case 'I':
 			if (I.hud) {
 				I.hud->vi = false;
@@ -1163,13 +1234,16 @@ static void __vi_mode() {
 				gcomp = false;
 			} else {
 				I.buffer.index = I.buffer.length;
-			} break;
+			}
+			break;
 		case 'p':
 			while (rep--) {
 				paste ();
-			} break;
+			}
+			break;
 		case 'a':
 			__move_cursor_right ();
+			break;
 		case 'i':
 			I.vi_mode = INSERT_MODE;
 			if (I.hud) {
@@ -1179,42 +1253,52 @@ static void __vi_mode() {
 		case 'h':
 			while (rep--) {
 				__move_cursor_left ();
-			} break;
+			}
+			break;
 		case 'l':
 			while (rep--) {
 				__move_cursor_right ();
-			} break;
+			}
+			break;
 		case 'E':
 			while (rep--) {
 				vi_cmd_E ();
-			} break;
+			}
+			break;
 		case 'e':
 			while (rep--) {
 				vi_cmd_e ();
-			} break;
+			}
+			break;
 		case 'B':
 			while (rep--) {
 				vi_cmd_B ();
-			} break;
+			}
+			break;
 		case 'b':
 			while (rep--) {
 				vi_cmd_b ();
-			} break;
+			}
+			break;
 		case 'W':
 			while (rep--) {
 				vi_cmd_W ();
-			} break;
+			}
+			break;
 		case 'w':
 			while (rep--) {
 				vi_cmd_w ();
-			} break;
+			}
+			break;
 		default:					// escape key
 			ch = tolower (r_cons_arrow_to_hjkl (ch));
 			switch (ch) {
 			case 'k': 			// up
+ 				I.history.do_setup_match = o_do_setup_match;
 				r_line_hist_up ();
 				break;
 			case 'j':			// down
+ 				I.history.do_setup_match = o_do_setup_match;
 				r_line_hist_down ();
 				break;
 			case 'l':			// right
@@ -1232,8 +1316,32 @@ static void __vi_mode() {
 	}
 }
 
+static void dietline_print_risprompt(const char *gcomp_line) {
+	RCons *cons = r_cons_singleton ();
+	if (cons->context->color_mode && *gcomp_line && I.buffer.length > 0) {
+		printf ("\r (ri-search): ");
+		const char *line = gcomp_line;
+		while (line) {
+			char *m = strstr (line, I.buffer.data);
+			if (m) {
+				fwrite (line, m - line, 1, stdout);
+				printf (Color_INVERT);
+				fwrite (m, I.buffer.length, 1, stdout);
+				printf (Color_RESET);
+				line = m + I.buffer.length;
+			} else {
+				printf ("%s", line);
+				line = NULL;
+			}
+		}
+		printf ("\r");
+	} else {
+		printf ("\r(ri-search (%s)): %s\r", I.buffer.data, gcomp_line);
+	}
+}
+
 R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
-	int rows, columns = r_cons_get_size (&rows) - 2;
+	int rows;
 	const char *gcomp_line = "";
 	static int gcomp_idx = 0;
 	static bool yank_flag = 0;
@@ -1242,7 +1350,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 #if USE_UTF8
 	int utflen;
 #endif
-	int ch, key, i = 0;	/* grep completion */
+	int ch = 0, key, i = 0;	/* grep completion */
 	char *tmp_ed_cmd, prev = 0;
 	int prev_buflen = -1;
 	RCons *cons = r_cons_singleton ();
@@ -1319,21 +1427,12 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 		buf[0] = ch;
 #endif
 #endif
-		if (I.echo) {
+		bool o_do_setup_match = I.history.do_setup_match;
+		I.history.do_setup_match = true;
+		if (I.echo && cons->context->color_mode) {
 			r_cons_clear_line (0);
 		}
-		if (columns < 1) {
-			columns = 40;
-		}
-#if __WINDOWS__
-		if (I.echo) {
-			printf ("\r%*c\r", columns, ' ');
-		}
-#else
-		if (I.echo) {
-			printf ("\r\x1b[2K\r");	// %*c\r", columns, ' ');
-		}
-#endif
+		(void)r_cons_get_size (&rows);
 		switch (*buf) {
 		case 0:	// control-space
 			/* ignore atm */
@@ -1390,7 +1489,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 		case 4:	// ^D
 			if (!I.buffer.data[0]) {/* eof */
 				if (I.echo) {
-					printf ("^D\n");
+					eprintf ("^D\n");
 				}
 				r_cons_set_raw (false);
 				r_cons_break_pop ();
@@ -1400,9 +1499,6 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 				__delete_next_char ();
 			}
 			break;
-		case 10:// ^J -- ignore
-			r_cons_break_pop ();
-			return I.buffer.data;
 		case 11:// ^K
 			I.buffer.data[I.buffer.index] = '\0';
 			I.buffer.length = I.buffer.index;
@@ -1478,9 +1574,11 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 			unix_word_rubout ();
 			break;
 		case 24:// ^X
-			strncpy (I.buffer.data, I.buffer.data + I.buffer.index, I.buffer.length);
-			I.buffer.length -= I.buffer.index;
-			I.buffer.index = 0;
+			if (I.buffer.index > 0) {
+				strncpy (I.buffer.data, I.buffer.data + I.buffer.index, I.buffer.length);
+				I.buffer.length -= I.buffer.index;
+				I.buffer.index = 0;
+			}
 			break;
 		case 25:// ^Y - paste
 			paste ();
@@ -1509,6 +1607,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 					gcomp_idx--;
 				}
 			} else {
+				I.history.do_setup_match = o_do_setup_match;
 				r_line_hist_down ();
 			}
 			break;
@@ -1523,15 +1622,28 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 			} else if (gcomp) {
 				gcomp_idx++;
 			} else {
+				I.history.do_setup_match = o_do_setup_match;
 				r_line_hist_up ();
 			}
 			break;
 		case 27: // esc-5b-41-00-00 alt/meta key
 #if __WINDOWS__
-			memmove (buf, buf + 1, strlen (buf));
-			if (!buf[0]) {
-				buf[0] = -1;
+			// always skip escape
+			memmove (buf, buf + 1, strlen ((char *)buf));
+#if 0
+			if (I.vtmode != 2) {
+				if (buf[1] == '[') {
+					memmove (buf, buf + 2, strlen (buf));
+				} else {
+					memmove (buf, buf + 1, strlen (buf));
+				}
+				if (!buf[0]) {
+					buf[0] = -1;
+				}
+			} else {
+				buf[0] = r_cons_readchar_timeout (50);
 			}
+#endif
 #else
 			buf[0] = r_cons_readchar_timeout (50);
 #endif
@@ -1589,28 +1701,49 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 				break;
 			default:
 #if !__WINDOWS__
-				buf[1] = r_cons_readchar_timeout (50);
-				if (buf[1] == -1) {
-					r_cons_break_pop ();
-					return NULL;
+				if (I.vtmode == 2) {
+					buf[1] = r_cons_readchar_timeout (50);
+					if (buf[1] == -1) { // alt+e
+						r_cons_break_pop ();
+						__print_prompt ();
+						continue;
+					}
+				} else {
+					buf[1] = r_cons_readchar_timeout (50);
 				}
 #endif
-				if (buf[0] == 0x5b) {	// [
+				if (buf[0] == '[') { // [
 					switch (buf[1]) {
-					case '3':	// supr
+					case '3': // supr or mouse click
 						__delete_next_char ();
-#if !__WINDOWS__
-						buf[1] = r_cons_readchar ();
-						if (buf[1] == -1) {
-							r_cons_break_pop ();
-							return NULL;
+						if (I.vtmode == 2) {
+							buf[1] = r_cons_readchar ();
+							if (buf[1] == 126) {
+								// handle SUPR key
+								r_cons_break_pop ();
+								__print_prompt ();
+								continue;
+							}
+							if (buf[1] == -1) {
+								r_cons_break_pop ();
+								return NULL;
+							}
+						}
+						for (;;) {
+							ch = r_cons_readchar ();
+							if (ch < 20) {
+								r_cons_break_pop ();
+								return NULL;
+							}
+							if (isupper (ch)) { // read until 'M'
+								break;
+							}
 						}
-#endif
 						break;
 					case '5': // pag up
-#if !__WINDOWS__
-						buf[1] = r_cons_readchar ();
-#endif
+						if (I.vtmode == 2) {
+							buf[1] = r_cons_readchar ();
+						}
 						if (I.hud) {
 							I.hud->top_entry_n -= (rows - 1);
 							if (I.hud->top_entry_n < 0) {
@@ -1623,9 +1756,9 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 						}
 						break;
 					case '6': // pag down
-#if !__WINDOWS__
-						buf[1] = r_cons_readchar ();
-#endif
+						if (I.vtmode == 2) {
+							buf[1] = r_cons_readchar ();
+						}
 						if (I.hud) {
 							I.hud->top_entry_n += (rows - 1);
 							if (I.hud->top_entry_n >= I.hud->current_entry_n) {
@@ -1662,9 +1795,12 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 							selection_widget_draw ();
 						} else if (gcomp) {
 							gcomp_idx++;
-						} else if (r_line_hist_up () == -1) {
-							r_cons_break_pop ();
-							return NULL;
+						} else {
+							I.history.do_setup_match = o_do_setup_match;
+							if (r_line_hist_up () == -1) {
+								r_cons_break_pop ();
+								return NULL;
+							}
 						}
 						break;
 					case 'B':	// down arrow
@@ -1679,9 +1815,12 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 							if (gcomp_idx > 0) {
 								gcomp_idx--;
 							}
-						} else if (r_line_hist_down () == -1) {
-							r_cons_break_pop ();
-							return NULL;
+						} else {
+							I.history.do_setup_match = o_do_setup_match;
+							if (r_line_hist_down () == -1) {
+								r_cons_break_pop ();
+								return NULL;
+							}
 						}
 						break;
 					case 'C':	// right arrow
@@ -1691,18 +1830,21 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 						__move_cursor_left ();
 						break;
 					case 0x31:	// control + arrow
+						if (I.vtmode == 2) {
+							ch = r_cons_readchar ();
+							if (ch == 0x7e) { // HOME in screen/tmux
+								// corresponding END is 0x34 below (the 0x7e is ignored there)
+								I.buffer.index = 0;
+								break;
+							}
+							r_cons_readchar ();
+						}
 #if __WINDOWS__
-						ch = buf[2];
-#else
-						ch = r_cons_readchar ();
-						if (ch == 0x7e) {	// HOME in screen/tmux
-							// corresponding END is 0x34 below (the 0x7e is ignored there)
-							I.buffer.index = 0;
-							break;
+						else {
+							ch = buf[2];
 						}
-						r_cons_readchar ();
-						int fkey = ch - '0';
 #endif
+						int fkey = ch - '0';
 						switch (ch) {
 						case 0x41:
 							// first
@@ -1737,21 +1879,18 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 							}
 							break;
 						default:
-#if __WINDOWS__
-							// ...
-#else
-							{
+							if (I.vtmode == 2) {
 								if (I.cb_fkey) {
 									I.cb_fkey (I.user, fkey);
 								}
 							}
-#endif
 							break;
 						}
 						r_cons_set_raw (1);
 						break;
 					case 0x37:	// HOME xrvt-unicode
 						r_cons_readchar ();
+						break;
 					case 0x48:	// HOME
 						if (I.sel_widget) {
 							selection_widget_up (I.sel_widget->options_len - 1);
@@ -1804,6 +1943,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 				r_line_autocomplete ();
 			}
 			break;
+		case 10: // ^J
 		case 13: // enter
 			if (I.hud) {
 				I.hud->activate = false;
@@ -1885,8 +2025,8 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 			if (gcomp) {
 				gcomp_line = "";
 				int counter = 0;
-				if (I.history.data != NULL) {
-					for (i = I.history.size-1; i >= 0; i--) {
+				if (I.history.data) {
+					for (i = I.history.size - 1; i >= 0; i--) {
 						if (!I.history.data[i]) {
 							continue;
 						}
@@ -1901,7 +2041,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 						}
 					}
 				}
-				printf ("\r (reverse-i-search (%s)): %s\r", I.buffer.data, gcomp_line);
+				dietline_print_risprompt (gcomp_line);
 			} else {
 			        __print_prompt ();
 			}
