@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2008-2018 - pancake */
+/* radare - LGPL - Copyright 2008-2021 - pancake */
 
 #include <r_cons.h>
 #include <ctype.h>
@@ -16,9 +16,52 @@ R_API char *r_cons_hud_file(const char *f) {
 	return NULL;
 }
 
+// Display a buffer in the hud (splitting it line-by-line and ignoring
+// the lines starting with # )
+R_API char *r_cons_hud_line_string(const char *s) {
+	if (!r_cons_is_interactive ()) {
+		eprintf ("Hud mode requires scr.interactive=true.\n");
+		return NULL;
+	}
+	char *os, *track, *ret, *o = strdup (s);
+	if (!o) {
+		return NULL;
+	}
+	r_str_replace_ch (o, '\r', 0, true);
+	r_str_replace_ch (o, '\t', 0, true);
+	RList *fl = r_list_new ();
+	int i;
+	if (!fl) {
+		free (o);
+		return NULL;
+	}
+	fl->free = free;
+	for (os = o, i = 0; o[i]; i++) {
+		if (o[i] == '\n') {
+			o[i] = 0;
+			if (*os && *os != '#') {
+				track = strdup (os);
+				if (!r_list_append (fl, track)) {
+					free (track);
+					break;
+				}
+			}
+			os = o + i + 1;
+		}
+	}
+	ret = r_cons_hud_line (fl, NULL);
+	free (o);
+	r_list_free (fl);
+	return ret;
+}
+
 // Display a buffer in the hud (splitting it line-by-line and ignoring
 // the lines starting with # )
 R_API char *r_cons_hud_string(const char *s) {
+	if (!r_cons_is_interactive ()) {
+		eprintf ("Hud mode requires scr.interactive=true.\n");
+		return NULL;
+	}
 	char *os, *track, *ret, *o = strdup (s);
 	if (!o) {
 		return NULL;
@@ -105,7 +148,7 @@ static bool __matchString(char *entry, char *filter, char *mask, const int mask_
 }
 
 
-static RList *hud_filter(RList *list, char *user_input, int top_entry_n, int *current_entry_n, char **selected_entry) {
+static RList *hud_filter(RList *list, char *user_input, int top_entry_n, int *current_entry_n, char **selected_entry, bool simple) {
 	RListIter *iter;
 	char *current_entry;
 	char mask[HUD_BUF_SIZE];
@@ -132,7 +175,14 @@ static RList *hud_filter(RList *list, char *user_input, int top_entry_n, int *cu
 			}
 			p = strdup (current_entry);
 			// if the filter is empty, print the entry and move on
-			if (!user_input[0]) {
+			if (simple) {
+				for (j = 0; p[j] && user_input[0]; j++) {
+					if (mask[j]) {
+						p[j] = toupper ((unsigned char) p[j]);
+					}
+				}
+				r_list_append (res, strdup (p));
+			} else if (!user_input[0]) {
 				r_list_append (res, r_str_newf (" %c %s", first_line? '-': ' ', p));
 			} else {
 				// otherwise we need to emphasize the matching part
@@ -197,7 +247,8 @@ static void mht_free_kv(HtPPKv *kv) {
 
 #define HUD_CACHE 0
 R_API char *r_cons_hud(RList *list, const char *prompt) {
-	char user_input[HUD_BUF_SIZE], hud_prompt[HUD_BUF_SIZE + 1];
+	bool demo = r_cons_singleton ()->context->demo;
+	char user_input[HUD_BUF_SIZE + 1];
 	char *selected_entry = NULL;
 	RListIter *iter;
 
@@ -207,11 +258,12 @@ R_API char *r_cons_hud(RList *list, const char *prompt) {
 	hud->vi = 0;
 	I(line)->echo = false;
 	I(line)->hud = hud;
-	hud_prompt [0] = 0;
 	user_input [0] = 0;
+	user_input[HUD_BUF_SIZE] = 0;
 	hud->top_entry_n = 0;
 	r_cons_show_cursor (false);
 	r_cons_enable_mouse (false);
+	r_cons_set_raw (true);
 	r_cons_clear ();
 
 	// Repeat until the user exits the hud
@@ -223,10 +275,20 @@ R_API char *r_cons_hud(RList *list, const char *prompt) {
 			hud->top_entry_n = 0;
 		}
 		selected_entry = NULL;
+		char *p = NULL;
 		if (prompt && *prompt) {
-			r_cons_printf (">> %s\n", prompt);
+			p = r_str_appendf (p, ">> %s\n", prompt);
+		}
+		p = r_str_appendf (p, "%d> %s|\n", hud->top_entry_n, user_input);
+		if (p) {
+			if (demo) {
+				char *q = r_str_ss (p, NULL, 0);
+				free (p);
+				p = q;
+			}
+			r_cons_printf ("%s", p);
+			free (p);
 		}
-		r_cons_printf ("%d> %s\n", hud->top_entry_n, hud_prompt);
 		char *row;
 		RList *filtered_list = NULL;
 
@@ -234,7 +296,7 @@ R_API char *r_cons_hud(RList *list, const char *prompt) {
 		filtered_list = ht_pp_find (ht, user_input, &found);
 		if (!found) {
 			filtered_list = hud_filter (list, user_input,
-				hud->top_entry_n, &(hud->current_entry_n), &selected_entry);
+				hud->top_entry_n, &(hud->current_entry_n), &selected_entry, false);
 #if HUD_CACHE
 			ht_pp_insert (ht, user_input, filtered_list);
 #endif
@@ -250,15 +312,94 @@ R_API char *r_cons_hud(RList *list, const char *prompt) {
 #endif
 		r_cons_visual_flush ();
 		(void) r_line_readline ();
-		memset (user_input, 0, HUD_BUF_SIZE);
-		memset (hud_prompt, 0, HUD_BUF_SIZE + 1);
-		strncpy (user_input, I(line)->buffer.data, HUD_BUF_SIZE - 1); 				// to search
-		strcpy (hud_prompt, user_input); 					// to display
-		int i;
-		for (i = I(line)->buffer.length; i > I(line)->buffer.index; i--) {
-			hud_prompt[i] = hud_prompt[i - 1];
+		r_str_ncpy (user_input, I(line)->buffer.data, HUD_BUF_SIZE);
+
+		if (!hud->activate) {
+			hud->top_entry_n = 0;
+			if (hud->current_entry_n >= 1 ) {
+				if (selected_entry) {
+					R_FREE (I(line)->hud);
+					I(line)->echo = true;
+					r_cons_enable_mouse (false);
+					r_cons_show_cursor (true);
+					r_cons_set_raw (false);
+					return strdup (selected_entry);
+				}
+			} else {
+				goto _beach;
+			}
 		}
-		memcpy (hud_prompt + I(line)->buffer.index, "|", 1);
+	}
+_beach:
+	R_FREE (I(line)->hud);
+	I(line)->echo = true;
+	r_cons_show_cursor (true);
+	r_cons_enable_mouse (false);
+	r_cons_set_raw (false);
+	ht_pp_free (ht);
+	return NULL;
+}
+
+R_API char *r_cons_hud_line(RList *list, const char *prompt) {
+	char user_input[HUD_BUF_SIZE + 1];
+	char *selected_entry = NULL;
+	RListIter *iter;
+
+	HtPP *ht = ht_pp_new (NULL, (HtPPKvFreeFunc)mht_free_kv, (HtPPCalcSizeV)strlen);
+	RLineHud *hud = (RLineHud*) R_NEW (RLineHud);
+	hud->activate = 0;
+	hud->vi = 0;
+	I(line)->echo = false;
+	I(line)->hud = hud;
+	user_input [0] = 0;
+	user_input[HUD_BUF_SIZE] = 0;
+	hud->top_entry_n = 0;
+	r_cons_show_cursor (false);
+	r_cons_enable_mouse (false);
+	r_cons_set_raw (true);
+
+	r_cons_reset ();
+	// Repeat until the user exits the hud
+	for (;;) {
+		hud->current_entry_n = 0;
+
+		if (hud->top_entry_n < 0) {
+			hud->top_entry_n = 0;
+		}
+		selected_entry = NULL;
+		r_cons_printf ("\r%s", R_CONS_CLEAR_LINE);
+		if (prompt && *prompt) {
+			r_cons_printf (">> %s [ ", prompt);
+		}
+		char *row;
+
+		bool found = false;
+		RList *filtered_list = ht_pp_find (ht, user_input, &found);
+		if (!found) {
+			filtered_list = hud_filter (list, user_input,
+				hud->top_entry_n, &(hud->current_entry_n), &selected_entry, true);
+#if HUD_CACHE
+			ht_pp_insert (ht, user_input, filtered_list);
+#endif
+		}
+		r_cons_printf ("(%d)> %s [", r_list_length (filtered_list), user_input);
+		int slen = 0;
+		int w = r_cons_get_size (NULL);
+		r_list_foreach (filtered_list, iter, row) {
+			slen += strlen (row);
+			if (slen >= w) {
+				break;
+			}
+			r_cons_printf (" %s,", row);
+		}
+#if !HUD_CACHE
+		r_list_free (filtered_list);
+#endif
+		r_cons_printf ("]");
+		r_cons_flush ();
+		(void) r_line_readline ();
+		r_str_ncpy (user_input, I(line)->buffer.data, HUD_BUF_SIZE);
+
 		if (!hud->activate) {
 			hud->top_entry_n = 0;
 			if (hud->current_entry_n >= 1 ) {
@@ -291,7 +432,7 @@ R_API char *r_cons_hud_path(const char *path, int dir) {
 	RList *files;
 	if (path) {
 		path = r_str_trim_head_ro (path);
-		tmp = strdup (*path? path: "./");
+		tmp = strdup (*path ? path : "./");
 	} else {
 		tmp = strdup ("./");
 	}
