@@ -1,9 +1,9 @@
-/* radare - LGPL - Copyright 2013-2019 - pancake, sghctoma, xarkes */
+/* radare - LGPL - Copyright 2013-2021 - pancake, sghctoma, xarkes */
 
 #include <r_cons.h>
 
-#define RCOLOR_AT(i) (RColor *) (((ut8 *) &(r_cons_singleton ()->context->cpal)) + keys[i].coff)
-#define COLOR_AT(i) (char **) (((ut8 *) &(r_cons_singleton ()->context->pal)) + keys[i].off)
+#define RCOLOR_AT(i) (RColor *) (((ut8 *) &(r_cons_context ()->cpal)) + keys[i].coff)
+#define COLOR_AT(i) (char **) (((ut8 *) &(r_cons_context ()->pal)) + keys[i].off)
 
 static struct {
 	const char *name;
@@ -157,14 +157,14 @@ R_API void r_cons_pal_init(RConsContext *ctx) {
 	ctx->cpal.b0x7f              = (RColor) RColor_CYAN;
 	ctx->cpal.b0xff              = (RColor) RColor_RED;
 	ctx->cpal.args               = (RColor) RColor_YELLOW;
-	ctx->cpal.bin                = (RColor) RColor_CYAN;
+	ctx->cpal.bin                = (RColor) RColor_YELLOW;
 	ctx->cpal.btext              = (RColor) RColor_YELLOW;
 	ctx->cpal.call               = (RColor) RColor_BGREEN;
 	ctx->cpal.call.attr          = R_CONS_ATTR_BOLD;
 	ctx->cpal.ucall              = (RColor) RColor_GREEN;
 	ctx->cpal.ujmp               = (RColor) RColor_GREEN;
 	ctx->cpal.cjmp               = (RColor) RColor_GREEN;
-	ctx->cpal.cmp                = (RColor) RColor_CYAN;
+	ctx->cpal.cmp                = (RColor) RColor_YELLOW;
 	ctx->cpal.comment            = (RColor) RColor_RED;
 	ctx->cpal.usercomment        = (RColor) RColor_WHITE;
 	ctx->cpal.creg               = (RColor) RColor_CYAN;
@@ -181,13 +181,14 @@ R_API void r_cons_pal_init(RConsContext *ctx) {
 	ctx->cpal.jmp                = (RColor) RColor_GREEN;
 	ctx->cpal.label              = (RColor) RColor_CYAN;
 	ctx->cpal.math               = (RColor) RColor_YELLOW;
+	// ctx->cpal.mov                = (RColor) RColor_WHITE;
 	ctx->cpal.mov                = (RColor) RColor_WHITE;
 	ctx->cpal.nop                = (RColor) RColor_BLUE;
 	ctx->cpal.num                = (RColor) RColor_YELLOW;
 	ctx->cpal.offset             = (RColor) RColor_GREEN;
 	ctx->cpal.other              = (RColor) RColor_WHITE;
-	ctx->cpal.pop                = (RColor) RColor_BMAGENTA;
-	ctx->cpal.pop.attr           = R_CONS_ATTR_BOLD;
+	ctx->cpal.pop                = (RColor) RColor_MAGENTA;
+	// ctx->cpal.pop.attr           = R_CONS_ATTR_BOLD;
 	ctx->cpal.prompt             = (RColor) RColor_YELLOW;
 	ctx->cpal.push               = (RColor) RColor_MAGENTA;
 	ctx->cpal.crypto             = (RColor) RColor_BGBLUE;
@@ -224,12 +225,12 @@ R_API void r_cons_pal_init(RConsContext *ctx) {
 	ctx->cpal.widget_sel         = (RColor) RColor_BGRED;
 
 	ctx->cpal.graph_box          = (RColor) RColor_NULL;
-	ctx->cpal.graph_box2         = (RColor) RColor_BLUE;
+	ctx->cpal.graph_box2         = (RColor) RColor_YELLOW;
 	ctx->cpal.graph_box3         = (RColor) RColor_MAGENTA;
 	ctx->cpal.graph_box4         = (RColor) RColor_GRAY;
 	ctx->cpal.graph_true         = (RColor) RColor_GREEN;
 	ctx->cpal.graph_false        = (RColor) RColor_RED;
-	ctx->cpal.graph_trufae       = (RColor) RColor_BLUE; // single jump
+	ctx->cpal.graph_trufae       = (RColor) RColor_CYAN; // single jump
 	ctx->cpal.graph_traced       = (RColor) RColor_YELLOW;
 	ctx->cpal.graph_current      = (RColor) RColor_BLUE;
 	ctx->cpal.graph_diff_unknown = (RColor) RColor_MAGENTA;
@@ -237,7 +238,6 @@ R_API void r_cons_pal_init(RConsContext *ctx) {
 	ctx->cpal.graph_diff_match   =  (RColor) RColor_GRAY;
 	ctx->cpal.graph_diff_unmatch =  (RColor) RColor_YELLOW;
 
-
 	r_cons_pal_free (ctx);
 	ctx->pal.reset = Color_RESET; // reset is not user accessible, const char* is ok
 	__cons_pal_update_event (ctx);
@@ -266,7 +266,7 @@ R_API void r_cons_pal_copy(RConsContext *dst, RConsContext *src) {
 	__cons_pal_update_event (dst);
 }
 
-R_API void r_cons_pal_random() {
+R_API void r_cons_pal_random(void) {
 	int i;
 	RColor *rcolor;
 	for (i = 0; keys[i].name; i++) {
@@ -367,8 +367,8 @@ R_API char *r_cons_pal_parse(const char *str, RColor *outcol) {
 			rcolor.b = colors[i].rcolor.b;
 			rcolor.id16 = colors[i].rcolor.id16;
 			if (!outcol) {
-				strncat (out, colors[i].code,
-					sizeof (out) - strlen (out) - 1);
+				size_t n = strlen (out);
+				snprintf (out + n, sizeof (out) - n, "%s", colors[i].code);
 			}
 		}
 		if (bgcolor && !strcmp (bgcolor, colors[i].name)) {
@@ -378,8 +378,8 @@ R_API char *r_cons_pal_parse(const char *str, RColor *outcol) {
 			rcolor.b2 = colors[i].rcolor.b;
 			rcolor.id16 = colors[i].rcolor.id16;
 			if (!outcol) {
-				strncat (out, colors[i].bgcode,
-					sizeof (out) - strlen (out) - 1);
+				size_t n = strlen (out);
+				snprintf (out + n, sizeof (out) - n, "%s", colors[i].bgcode);
 			}
 		}
 	}
@@ -417,7 +417,7 @@ R_API char *r_cons_pal_parse(const char *str, RColor *outcol) {
 	return (*out && !outcol) ? strdup (out) : NULL;
 }
 
-static void r_cons_pal_show_gs() {
+static void r_cons_pal_show_gs(void) {
 	int i, n;
 	r_cons_print ("\nGreyscale:\n");
 	RColor rcolor = RColor_BLACK;
@@ -442,7 +442,7 @@ static void r_cons_pal_show_gs() {
 	}
 }
 
-static void r_cons_pal_show_256() {
+static void r_cons_pal_show_256(void) {
 	RColor rc = RColor_BLACK;
 	r_cons_print ("\n\nXTerm colors:\n");
 	int r = 0;
@@ -474,7 +474,7 @@ static void r_cons_pal_show_256() {
 	}
 }
 
-static void r_cons_pal_show_rgb() {
+static void r_cons_pal_show_rgb(void) {
 	const int inc = 3;
 	int i, j, k, n = 0;
 	RColor rc = RColor_BLACK;
@@ -500,7 +500,7 @@ static void r_cons_pal_show_rgb() {
 	}
 }
 
-R_API void r_cons_pal_show() {
+R_API void r_cons_pal_show(void) {
 	int i;
 	for (i = 0; colors[i].name; i++) {
 		r_cons_printf ("%s%s__"Color_RESET" %s\n",
@@ -508,7 +508,7 @@ R_API void r_cons_pal_show() {
 			colors[i].bgcode,
 			colors[i].name);
 	}
-	switch (r_cons_singleton ()->context->color_mode) {
+	switch (r_cons_context ()->color_mode) {
 	case COLOR_MODE_256: // 256 color palette
 		r_cons_pal_show_gs ();
 		r_cons_pal_show_256 ();
@@ -544,7 +544,7 @@ R_API void r_cons_pal_list(int rad, const char *arg) {
 				keys[i].name, rcolor->r, rcolor->g, rcolor->b, hasnext);
 			break;
 		case 'c': {
-			const char *prefix = r_str_trim_ro (arg);
+			const char *prefix = r_str_trim_head_ro (arg);
 			if (!prefix) {
 				prefix = "";
 			}
@@ -617,7 +617,8 @@ R_API int r_cons_pal_set(const char *key, const char *val) {
 	for (i = 0; keys[i].name; i++) {
 		if (!strcmp (key, keys[i].name)) {
 			rcolor = RCOLOR_AT (i);
-			r_cons_pal_parse (val, rcolor);
+			char *r = r_cons_pal_parse (val, rcolor);
+			free (r);
 			return true;
 		}
 	}
@@ -648,12 +649,12 @@ R_API const char *r_cons_pal_get_name(int index) {
 	return (index >= 0 && index < keys_len) ? keys[index].name : NULL;
 }
 
-R_API int r_cons_pal_len() {
+R_API int r_cons_pal_len(void) {
 	return keys_len;
 }
 
-R_API void r_cons_pal_update_event() {
-	__cons_pal_update_event (r_cons_singleton ()->context);
+R_API void r_cons_pal_update_event(void) {
+	__cons_pal_update_event (r_cons_context ());
 }
 
 R_API void r_cons_rainbow_new(RConsContext *ctx, int sz) {
