@@ -23,7 +23,7 @@ static int rcoreasm_address_comparator(RCoreAsmHit *a, RCoreAsmHit *b){
 	return 1; /* a->addr > b->addr */
 }
 
-R_API RCoreAsmHit *r_core_asm_hit_new() {
+R_API RCoreAsmHit *r_core_asm_hit_new(void) {
 	RCoreAsmHit *hit = R_NEW0 (RCoreAsmHit);
 	if (!hit) {
 		return NULL;
@@ -33,7 +33,7 @@ R_API RCoreAsmHit *r_core_asm_hit_new() {
 	return hit;
 }
 
-R_API RList *r_core_asm_hit_list_new() {
+R_API RList *r_core_asm_hit_list_new(void) {
 	RList *list = r_list_new ();
 	if (list) {
 		list->free = &r_core_asm_hit_free;
@@ -54,7 +54,7 @@ R_API void r_core_asm_hit_free(void *_hit) {
 R_API char* r_core_asm_search(RCore *core, const char *input) {
 	RAsmCode *acode;
 	char *ret;
-	if (!(acode = r_asm_massemble (core->assembler, input))) {
+	if (!(acode = r_asm_massemble (core->rasm, input))) {
 		return NULL;
 	}
 	ret = r_asm_code_get_hex (acode);
@@ -76,6 +76,7 @@ R_API RList *r_core_asm_strsearch(RCore *core, const char *input, ut64 from, ut6
 	int tokcount, matchcount, count = 0;
 	int matches = 0;
 	const int addrbytes = core->io->addrbytes;
+	 ut64 first_match_addr = 0;
 
 	if (!input || !*input) {
 		return NULL;
@@ -121,7 +122,7 @@ R_API RList *r_core_asm_strsearch(RCore *core, const char *input, ut64 from, ut6
 	tokens[tokcount] = NULL;
 	r_cons_break_push (NULL, NULL);
 	char *opst = NULL;
-	for (at = from, matchcount = 0; at < to; at += core->blocksize) {
+	for (at = from; at < to; at += core->blocksize) {
 		if (r_cons_is_breaked ()) {
 			break;
 		}
@@ -135,7 +136,7 @@ R_API RList *r_core_asm_strsearch(RCore *core, const char *input, ut64 from, ut6
 			if (addr >= to) {
 				break;
 			}
-			r_asm_set_pc (core->assembler, addr);
+			r_asm_set_pc (core->rasm, addr);
 			if (mode == 'i') {
 				RAnalOp analop = {0};
 				ut64 len = R_MIN (15, core->blocksize - idx);
@@ -165,9 +166,10 @@ R_API RList *r_core_asm_strsearch(RCore *core, const char *input, ut64 from, ut6
 						r_core_asm_hit_free (hit);
 						goto beach;
 					}
-					r_asm_disassemble (core->assembler, &op, buf + addrbytes * idx,
+					r_asm_disassemble (core->rasm, &op, buf + addrbytes * idx,
 					      core->blocksize - addrbytes * idx);
-					hit->code = r_str_newf (r_strbuf_get (&op.buf_asm));
+					hit->code = r_str_new (r_strbuf_get (&op.buf_asm));
+					r_asm_op_fini (&op);
 					idx = (matchcount)? tidx + 1: idx + 1;
 					matchcount = 0;
 					r_list_append (hits, hit);
@@ -187,15 +189,17 @@ R_API RList *r_core_asm_strsearch(RCore *core, const char *input, ut64 from, ut6
 				r_anal_op_fini (&analop);
 			} else {
 				if (!(len = r_asm_disassemble (
-					      core->assembler, &op,
+					      core->rasm, &op,
 					      buf + addrbytes * idx,
 					      core->blocksize - addrbytes * idx))) {
 					idx = (matchcount)? tidx + 1: idx + 1;
 					matchcount = 0;
+					r_asm_op_fini (&op);
 					continue;
 				}
 				//opsz = op.size;
 				opst = strdup (r_strbuf_get (&op.buf_asm));
+				r_asm_op_fini (&op);
 			}
 			if (opst) {
 				matches = strcmp (opst, "invalid") && strcmp (opst, "unaligned");
@@ -206,10 +210,8 @@ R_API RList *r_core_asm_strsearch(RCore *core, const char *input, ut64 from, ut6
 				} else if (!regexp) {
 					matches = strstr (opst, tokens[matchcount]) != NULL;
 				} else {
-					rx = r_regex_new (tokens[matchcount], "");
-					if (r_regex_comp (rx, tokens[matchcount], R_REGEX_EXTENDED|R_REGEX_NOSUB) == 0) {
-						matches = r_regex_exec (rx, opst, 0, 0, 0) == 0;
-					}
+					rx = r_regex_new (tokens[matchcount], "es");
+					matches = r_regex_exec (rx, opst, 0, 0, 0) == 0;
 					r_regex_free (rx);
 				}
 			}
@@ -220,6 +222,9 @@ R_API RList *r_core_asm_strsearch(RCore *core, const char *input, ut64 from, ut6
 			}
 			if (matches) {
 				code = r_str_appendf (code, "%s; ", opst);
+				if (matchcount == 0) {
+					first_match_addr = addr;
+				}
 				if (matchcount == tokcount - 1) {
 					if (tokcount == 1) {
 						tidx = idx;
@@ -229,7 +234,7 @@ R_API RList *r_core_asm_strsearch(RCore *core, const char *input, ut64 from, ut6
 						R_FREE (hits);
 						goto beach;
 					}
-					hit->addr = addr;
+					hit->addr = first_match_addr;
 					hit->len = idx + len - tidx;
 					if (hit->len == -1) {
 						r_core_asm_hit_free (hit);
@@ -268,9 +273,8 @@ R_API RList *r_core_asm_strsearch(RCore *core, const char *input, ut64 from, ut6
 			R_FREE (opst);
 		}
 	}
-	r_cons_break_pop ();
-	r_asm_set_pc (core->assembler, toff);
 beach:
+	r_asm_set_pc (core->rasm, toff);
 	free (buf);
 	free (ptr);
 	free (code);
@@ -365,11 +369,11 @@ static int handle_forward_disassemble(RCore* core, RList *hits, ut8* buf, ut64 l
 		return end_addr;
 	}
 
-	r_asm_set_pc (core->assembler, current_instr_addr);
+	r_asm_set_pc (core->rasm, current_instr_addr);
 	while (tmp_current_buf_pos < len && temp_instr_addr < end_addr) {
 		temp_instr_len = len - tmp_current_buf_pos;
 		IFDBG eprintf("Current position: %"PFMT64d" instr_addr: 0x%"PFMT64x"\n", tmp_current_buf_pos, temp_instr_addr);
-		temp_instr_len = r_asm_disassemble (core->assembler, &op, buf+tmp_current_buf_pos, temp_instr_len);
+		temp_instr_len = r_asm_disassemble (core->rasm, &op, buf+tmp_current_buf_pos, temp_instr_len);
 
 		if (temp_instr_len == 0){
 			is_valid = false;
@@ -514,7 +518,7 @@ R_API RList *r_core_asm_bwdisassemble(RCore *core, ut64 addr, int n, int len) {
 		if (r_cons_is_breaked ()) {
 			break;
 		}
-		c = r_asm_mdisassemble (core->assembler, buf + len - idx, idx);
+		c = r_asm_mdisassemble (core->rasm, buf + len - idx, idx);
 		if (strstr (c->assembly, "invalid") || strstr (c->assembly, ".byte")) {
 			r_asm_code_free(c);
 			continue;
@@ -532,9 +536,9 @@ R_API RList *r_core_asm_bwdisassemble(RCore *core, ut64 addr, int n, int len) {
 		}
 	}
 	at = addr - idx / addrbytes;
-	r_asm_set_pc (core->assembler, at);
+	r_asm_set_pc (core->rasm, at);
 	for (hit_count = 0; hit_count < n; hit_count++) {
-		int instrlen = r_asm_disassemble (core->assembler, &op,
+		int instrlen = r_asm_disassemble (core->rasm, &op,
 			buf + len - addrbytes * (addr - at), addrbytes * (addr - at));
 		add_hit_to_hits (hits, at, instrlen, true);
 		at += instrlen;
@@ -581,10 +585,10 @@ static RList * r_core_asm_back_disassemble_all(RCore *core, ut64 addr, ut64 len,
 			break;
 		}
 		// reset assembler
-		r_asm_set_pc (core->assembler, current_instr_addr);
+		r_asm_set_pc (core->rasm, current_instr_addr);
 		current_instr_len = len - current_buf_pos + extra_padding;
 		IFDBG eprintf("current_buf_pos: 0x%"PFMT64x", current_instr_len: %d\n", current_buf_pos, current_instr_len);
-		current_instr_len = r_asm_disassemble (core->assembler, &op, buf+current_buf_pos, current_instr_len);
+		current_instr_len = r_asm_disassemble (core->rasm, &op, buf+current_buf_pos, current_instr_len);
 		hit = r_core_asm_hit_new ();
 		hit->addr = current_instr_addr;
 		hit->len = current_instr_len;
@@ -661,9 +665,9 @@ static RList *r_core_asm_back_disassemble (RCore *core, ut64 addr, int len, ut64
 			break;
 		}
 		// reset assembler
-		r_asm_set_pc (core->assembler, current_instr_addr);
+		r_asm_set_pc (core->rasm, current_instr_addr);
 		current_instr_len = next_buf_pos - current_buf_pos;
-		current_instr_len = r_asm_disassemble (core->assembler, &op, buf+current_buf_pos, current_instr_len);
+		current_instr_len = r_asm_disassemble (core->rasm, &op, buf+current_buf_pos, current_instr_len);
 		IFDBG {
 			ut32 byte_cnt =  current_instr_len ? current_instr_len : 1;
 			eprintf("current_instr_addr: 0x%"PFMT64x", current_buf_pos: 0x%"PFMT64x", current_instr_len: %d \n", current_instr_addr, current_buf_pos, current_instr_len);
@@ -743,7 +747,7 @@ static RList *r_core_asm_back_disassemble (RCore *core, ut64 addr, int len, ut64
 		}
 	} while (((int) current_buf_pos >= 0) && (int)(len - current_buf_pos) >= 0);
 
-	r_asm_set_pc (core->assembler, addr);
+	r_asm_set_pc (core->rasm, addr);
 	free (buf);
 	return hits;
 }
