@@ -1,15 +1,15 @@
-/* radare2 - LGPL - Copyright 2009-2019 - pancake */
+/* radare2 - LGPL - Copyright 2009-2021 - pancake */
 
 #include "r_core.h"
 
 R_API int r_core_setup_debugger (RCore *r, const char *debugbackend, bool attach) {
 	int pid, *p = NULL;
 	bool is_gdb = !strcmp (debugbackend, "gdb");
-	RIODesc * fd = r->file ? r_io_desc_get (r->io, r->file->fd) : NULL;
+	RIODesc * fd = r->io->desc;
 	const char *prompt = NULL;
 
 	p = fd ? fd->data : NULL;
-	r_config_set_i (r->config, "cfg.debug", 1);
+	r_config_set_b (r->config, "cfg.debug", true);
 	if (!p) {
 		eprintf ("Invalid debug io\n");
 		return false;
@@ -19,9 +19,13 @@ R_API int r_core_setup_debugger (RCore *r, const char *debugbackend, bool attach
 	r_core_cmdf (r, "dL %s", debugbackend);
 	if (!is_gdb) {
 		pid = r_io_desc_get_pid (fd);
-		r_core_cmdf (r, "dp=%d", pid);
-		if (attach) {
-			r_core_cmdf (r, "dpa %d", pid);
+		if (pid >= 0) {
+			r_core_cmdf (r, "dp=%d", pid);
+			if (attach) {
+				r_core_cmdf (r, "dpa %d", pid);
+			}
+		} else {
+			eprintf ("Cannot retrieve pid from io.\n");
 		}
 	}
 	//this makes to attach twice showing warnings in the output
@@ -58,7 +62,7 @@ R_API int r_core_setup_debugger (RCore *r, const char *debugbackend, bool attach
 
 R_API int r_core_seek_base (RCore *core, const char *hex) {
 	ut64 addr = r_num_tail (core->num, core->offset, hex);
-	return r_core_seek (core, addr, 1);
+	return r_core_seek (core, addr, true);
 }
 
 R_API bool r_core_dump(RCore *core, const char *file, ut64 addr, ut64 size, int append) {
@@ -141,15 +145,22 @@ R_API ut8* r_core_transform_op(RCore *core, const char *arg, char op) {
 	int i, j;
 	ut64 len;
 	char *str = NULL;
-	ut8 *buf;
-
-	buf = (ut8 *)malloc (core->blocksize);
+	ut8 *buf = (ut8 *)malloc (core->blocksize);
 	if (!buf) {
 		return NULL;
 	}
+	if (op == 'i') { // "woi"
+		int hbs = core->blocksize / 2;
+		int j = core->blocksize - 1;
+		for (i = 0; i < hbs; i++, j--) {
+			buf[i] = core->block[j];
+			buf[j] = core->block[i];
+		}
+		return buf;
+	}
 	memcpy (buf, core->block, core->blocksize);
 
-	if (op!='e') {
+	if (op != 'e') {
 		// fill key buffer either from arg or from clipboard
 		if (arg) {  // parse arg for key
 			// r_hex_str2bin() is guaranteed to output maximum half the
@@ -177,7 +188,7 @@ R_API ut8* r_core_transform_op(RCore *core, const char *arg, char op) {
 	}
 
 	// execute the operand
-	if (op=='e') {
+	if (op == 'e') {
 		int wordsize = 1;
 		char *os, *p, *s = strdup (arg);
 		int n = 0, from = 0, to = UT8_MAX, dif = 0, step = 1;
@@ -243,9 +254,6 @@ R_API ut8* r_core_transform_op(RCore *core, const char *arg, char op) {
 	} else if (op == '2' || op == '4' || op == '8') { // "wo2" "wo4" "wo8"
 		int inc = op - '0';
 		ut8 tmp;
-		if (inc < 1 || inc > 8) {
-			goto beach;
-		}
 		for (i = 0; (i + inc) <= core->blocksize; i += inc) {
 			if (inc == 2) {
 				tmp = buf[i];
@@ -315,7 +323,7 @@ beach:
 }
 
 R_API int r_core_write_op(RCore *core, const char *arg, char op) {
-	ut8 *buf = r_core_transform_op(core, arg, op);
+	ut8 *buf = r_core_transform_op (core, arg, op);
 	if (!buf) {
 		return false;
 	}
@@ -408,7 +416,7 @@ R_API int r_core_seek_delta(RCore *core, st64 addr) {
 		}
 	}
 	core->offset = addr;
-	return r_core_seek (core, addr, 1);
+	return r_core_seek (core, addr, true);
 }
 
 // TODO: kill this wrapper
@@ -424,19 +432,24 @@ R_API bool r_core_write_at(RCore *core, ut64 addr, const ut8 *buf, int size) {
 	return ret;
 }
 
-R_API int r_core_extend_at(RCore *core, ut64 addr, int size) {
-	int ret;
-	if (!core->io || !core->file || size < 1) {
+R_API bool r_core_extend_at(RCore *core, ut64 addr, int size) {
+	if (!core->io || !core->io->desc || size < 1) {
 		return false;
 	}
-	ret = r_io_use_fd (core->io, core->file->fd);
-	if (ret != -1) {
-		ret = r_io_extend_at (core->io, addr, size);
-		if (addr >= core->offset && addr <= core->offset+core->blocksize) {
-			r_core_block_read (core);
+	int io_va = r_config_get_i (core->config, "io.va");
+	if (io_va) {
+		RIOMap *map = r_io_map_get_at (core->io, core->offset);
+		if (map) {
+			addr = addr - r_io_map_begin (map) + map->delta;
 		}
+		r_config_set_i (core->config, "io.va", false);
 	}
-	return ret != -1;
+	int ret = r_io_extend_at (core->io, addr, size);
+	if (addr >= core->offset && addr <= core->offset+core->blocksize) {
+		r_core_block_read (core);
+	}
+	r_config_set_i (core->config, "io.va", io_va);
+	return ret;
 }
 
 R_API int r_core_shift_block(RCore *core, ut64 addr, ut64 b_size, st64 dist) {
@@ -445,12 +458,12 @@ R_API int r_core_shift_block(RCore *core, ut64 addr, ut64 b_size, st64 dist) {
 	ut8 * shift_buf = NULL;
 	int res = false;
 
-	if (!core->io || !core->file) {
+	if (!core->io || !core->io->desc) {
 		return false;
 	}
 
 	if (b_size == 0 || b_size == (ut64) -1) {
-		r_io_use_fd (core->io, core->file->fd);
+		r_io_use_fd (core->io, core->io->desc->fd);
 		file_sz = r_io_size (core->io);
 		if (file_sz == UT64_MAX) {
 			file_sz = 0;
@@ -494,12 +507,11 @@ R_API int r_core_shift_block(RCore *core, ut64 addr, ut64 b_size, st64 dist) {
 	} else if ( (addr) + dist > fend) {
 		res = false;
 	} else {
-		r_io_use_fd (core->io, core->file->fd);
 		r_io_read_at (core->io, addr, shift_buf, b_size);
 		r_io_write_at (core->io, addr + dist, shift_buf, b_size);
 		res = true;
 	}
-	r_core_seek (core, addr, 1);
+	r_core_seek (core, addr, true);
 	free (shift_buf);
 	return res;
 }
@@ -510,12 +522,3 @@ R_API int r_core_block_read(RCore *core) {
 	}
 	return -1;
 }
-
-R_API int r_core_is_valid_offset (RCore *core, ut64 offset) {
-	if (!core) {
-		eprintf ("r_core_is_valid_offset: core is NULL\n");
-		r_sys_backtrace ();
-		return -1;
-	}
-	return r_io_is_valid_offset (core->io, offset, 0);
-}
