@@ -1,7 +1,15 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include "r_core.h"
 
+static const char *help_message_ci[] = {
+	"Usage: ci", "[sil] ([obid])", "Compare two bin objects",
+	"cis", " 0", "Compare symbols with current `ob 1` with given obid (0)",
+	"cii", " 0", "Compare imports",
+	"cil", " 0", "Compare libraries",
+	NULL
+};
+
 static const char *help_msg_c[] = {
 	"Usage:", "c[?dfx] [argument]", " # Compare",
 	"c", " [string]", "Compare a plain with escaped chars string",
@@ -15,10 +23,13 @@ static const char *help_msg_c[] = {
 	"ccc", " [at]", "Same as above, but only showing different lines",
 	"ccd", " [at]", "Compares in two disasm columns of block size",
 	"ccdd", " [at]", "Compares decompiler output (e cmd.pdc=pdg|pdd)",
+	"cd", " [dir]", "chdir",
 	// "cc", " [offset]", "code bindiff current block against offset"
 	// "cD", " [file]", "like above, but using radiff -b",
 	"cf", " [file]", "Compare contents of file at current seek",
 	"cg", "[?] [o] [file]", "Graphdiff current file and [file]",
+	"ci", "[?] [obid] ([obid2])", "Compare two bin-objects (symbols, imports, ...)",
+	"cl|cls|clear", "", "Clear screen, (clear0 to goto 0, 0 only)",
 	"cu", "[?] [addr] @at", "Compare memory hexdumps of $$ and dst in unified diff",
 	"cud", " [addr] @at", "Unified diff disasm from $$ and given address",
 	"cv", "[1248] [hexpairs] @at", "Compare 1,2,4,8-byte (silent return in $?)",
@@ -27,16 +38,9 @@ static const char *help_msg_c[] = {
 	"cx", " [hexpair]", "Compare hexpair string (use '.' as nibble wildcard)",
 	"cx*", " [hexpair]", "Compare hexpair string (output r2 commands)",
 	"cX", " [addr]", "Like 'cc' but using hexdiff output",
-	"", "", "",
-	"cd", " [dir]", "chdir",
-	"cl|cls|clear", "", "Clear screen, (clear0 to goto 0, 0 only)",
 	NULL
 };
 
-static void cmd_cmp_init(RCore *core) {
-	DEFINE_CMD_DESCRIPTOR (core, c);
-}
-
 R_API void r_core_cmpwatch_free(RCoreCmpWatcher *w) {
 	free (w->ndata);
 	free (w->odata);
@@ -108,6 +112,7 @@ R_API int r_core_cmpwatch_show(RCore *core, ut64 addr, int mode) {
 			if (is_diff) {
 				r_cons_printf ("0x%08"PFMT64x " has changed\n", w->addr);
 			}
+			break;
 		case 'o': // old contents
 		// use tmpblocksize
 		default:
@@ -261,8 +266,7 @@ static int radare_compare(RCore *core, const ut8 *f, const ut8 *d, int len, int
 			eq++;
 			continue;
 		}
-		switch (mode)
-		{
+		switch (mode) {
 		case 0:
 			r_cons_printf ("0x%08"PFMT64x " (byte=%.2d)   %02x '%c'  ->  %02x '%c'\n",
 				core->offset + i, i + 1,
@@ -282,7 +286,9 @@ static int radare_compare(RCore *core, const ut8 *f, const ut8 *d, int len, int
 			pj_ki (pj, "cmp_value", (int)d[i]);
 			pj_end (pj);
 			break;
-
+		default:
+			eprintf ("Unknown mode\n");
+			break;
 		}
 	}
 	if (mode == 0) {
@@ -336,19 +342,19 @@ static void cmd_cmp_watcher(RCore *core, const char *input) {
 		r_core_cmpwatch_show (core, UT64_MAX, 0);
 		break;
 	case '?': {
-		const char *help_message[] = {
-			"Usage: cw", "", "Watcher commands",
-			"cw", "", "List all compare watchers",
-			"cw", " addr", "List all compare watchers",
-			"cw", " addr sz cmd", "Add a memory watcher",
-			// "cws", " [addr]", "Show watchers",
-			"cw", "*", "List compare watchers in r2 cmds",
-			"cwr", " [addr]", "Reset/revert watchers",
-			"cwu", " [addr]", "Update watchers",
-			NULL
-		};
-		r_core_cmd_help (core, help_message);
-	}
+			  const char *help_message[] = {
+				  "Usage: cw", "", "Watcher commands",
+				  "cw", "", "List all compare watchers",
+				  "cw", " addr", "List all compare watchers",
+				  "cw", " addr sz cmd", "Add a memory watcher",
+				  // "cws", " [addr]", "Show watchers",
+				  "cw", "*", "List compare watchers in r2 cmds",
+				  "cwr", " [addr]", "Reset/revert watchers",
+				  "cwu", " [addr]", "Update watchers",
+				  NULL
+			  };
+			  r_core_cmd_help (core, help_message);
+		  }
 		  break;
 	}
 }
@@ -389,13 +395,13 @@ static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 	case 'c': // columns
 		for (i = j = 0; i < core->blocksize && j < core->blocksize;) {
 			// dis A
-			r_asm_set_pc (core->assembler, core->offset + i);
-			(void) r_asm_disassemble (core->assembler, &op,
+			r_asm_set_pc (core->rasm, core->offset + i);
+			(void) r_asm_disassemble (core->rasm, &op,
 				core->block + i, core->blocksize - i);
 
 			// dis B
-			r_asm_set_pc (core->assembler, off + i);
-			(void) r_asm_disassemble (core->assembler, &op2,
+			r_asm_set_pc (core->rasm, off + i);
+			(void) r_asm_disassemble (core->rasm, &op2,
 				buf + j, core->blocksize - j);
 
 			// show output
@@ -407,14 +413,14 @@ static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 				colpad[pos] = 0;
 			}
 			if (hascolor) {
-				r_cons_printf (iseq? pal->graph_true: pal->graph_false);
+				r_cons_print (iseq? pal->graph_true: pal->graph_false);
 			}
 			r_cons_printf (" 0x%08"PFMT64x "  %s %s",
 				core->offset + i, r_strbuf_get (&op.buf_asm), colpad);
 			r_cons_printf ("%c 0x%08"PFMT64x "  %s\n",
 				iseq? '=': '!', off + j, r_strbuf_get (&op2.buf_asm));
 			if (hascolor) {
-				r_cons_printf (Color_RESET);
+				r_cons_print (Color_RESET);
 			}
 			if (op.size < 1) {
 				op.size = 1;
@@ -429,13 +435,13 @@ static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 	case 'u': // unified
 		for (i = j = 0; i < core->blocksize && j < core->blocksize;) {
 			// dis A
-			r_asm_set_pc (core->assembler, core->offset + i);
-			(void) r_asm_disassemble (core->assembler, &op,
+			r_asm_set_pc (core->rasm, core->offset + i);
+			(void) r_asm_disassemble (core->rasm, &op,
 				core->block + i, core->blocksize - i);
 
 			// dis B
-			r_asm_set_pc (core->assembler, off + i);
-			(void) r_asm_disassemble (core->assembler, &op2,
+			r_asm_set_pc (core->rasm, off + i);
+			(void) r_asm_disassemble (core->rasm, &op2,
 				buf + j, core->blocksize - j);
 
 			// show output
@@ -445,17 +451,17 @@ static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 					core->offset + i, r_strbuf_get (&op.buf_asm));
 			} else {
 				if (hascolor) {
-					r_cons_printf (pal->graph_false);
+					r_cons_print (pal->graph_false);
 				}
 				r_cons_printf ("-0x%08"PFMT64x "  %s\n",
 					core->offset + i, r_strbuf_get (&op.buf_asm));
 				if (hascolor) {
-					r_cons_printf (pal->graph_true);
+					r_cons_print (pal->graph_true);
 				}
 				r_cons_printf ("+0x%08"PFMT64x "  %s\n",
 					off + j, r_strbuf_get (&op2.buf_asm));
 				if (hascolor) {
-					r_cons_printf (Color_RESET);
+					r_cons_print (Color_RESET);
 				}
 			}
 			if (op.size < 1) {
@@ -542,6 +548,163 @@ static void __core_cmp_bits (RCore *core, ut64 addr) {
 	r_cons_newline ();
 }
 
+static const RList *symbols_of(RCore *core, int id0) {
+	RBinFile *bf = r_bin_file_find_by_id (core->bin, id0);
+	RBinFile *old_bf = core->bin->cur;
+	r_bin_file_set_cur_binfile (core->bin, bf);
+	const RList *list = bf? r_bin_get_symbols (core->bin): NULL;
+	r_bin_file_set_cur_binfile (core->bin, old_bf);
+	return list;
+}
+
+static const RList *imports_of(RCore *core, int id0) {
+	RBinFile *bf = r_bin_file_find_by_id (core->bin, id0);
+	RBinFile *old_bf = core->bin->cur;
+	r_bin_file_set_cur_binfile (core->bin, bf);
+	const RList *list = bf? r_bin_get_imports (core->bin): NULL;
+	r_bin_file_set_cur_binfile (core->bin, old_bf);
+	return list;
+}
+
+static const RList *libs_of(RCore *core, int id0) {
+	RBinFile *bf = r_bin_file_find_by_id (core->bin, id0);
+	RBinFile *old_bf = core->bin->cur;
+	r_bin_file_set_cur_binfile (core->bin, bf);
+	const RList *list = bf? r_bin_get_libs (core->bin): NULL;
+	r_bin_file_set_cur_binfile (core->bin, old_bf);
+	return list;
+}
+
+static void _core_cmp_info_libs(RCore *core, int id0, int id1) {
+	const RList *s0 = libs_of (core, id0);
+	const RList *s1 = libs_of (core, id1);
+	if (!s0 || !s1) {
+		eprintf ("Missing bin object\n");
+		return;
+	}
+	RListIter *iter, *iter2;
+	char *s, *s2;
+	if (id0 == id1) {
+		eprintf ("%d == %d\n", id0, id1);
+		return;
+	}
+	r_list_foreach (s0, iter, s) {
+		bool found = false;
+		r_list_foreach (s1, iter2, s2) {
+			if (!strcmp (s, s2)) {
+				found = true;
+			}
+		}
+		r_cons_printf ("%s%s\n", found? " ": "-", s);
+	}
+	r_list_foreach (s1, iter, s) {
+		bool found = false;
+		r_list_foreach (s0, iter2, s2) {
+			if (!strcmp (s, s2)) {
+				found = true;
+			}
+		}
+		if (!found) {
+			r_cons_printf ("+%s\n", s);
+		}
+	}
+	// r_list_free (s0);
+	// r_list_free (s1);
+}
+
+static void _core_cmp_info_imports(RCore *core, int id0, int id1) {
+	const RList *s0 = imports_of (core, id0);
+	const RList *s1 = imports_of (core, id1);
+	if (!s0 || !s1) {
+		eprintf ("Missing bin object\n");
+		return;
+	}
+	RListIter *iter, *iter2;
+	RBinImport *s, *s2;
+	if (id0 == id1) {
+		eprintf ("%d == %d\n", id0, id1);
+		return;
+	}
+	r_list_foreach (s0, iter, s) {
+		bool found = false;
+		r_list_foreach (s1, iter2, s2) {
+			if (!strcmp (s->name, s2->name)) {
+				found = true;
+			}
+		}
+		r_cons_printf ("%s%s\n", found? " ": "-", s->name);
+	}
+	r_list_foreach (s1, iter, s) {
+		bool found = false;
+		r_list_foreach (s0, iter2, s2) {
+			if (!strcmp (s->name, s2->name)) {
+				found = true;
+			}
+		}
+		if (!found) {
+			r_cons_printf ("+%s\n", s->name);
+		}
+	}
+	// r_list_free (s0);
+	// r_list_free (s1);
+}
+
+static void _core_cmp_info_symbols(RCore *core, int id0, int id1) {
+	const RList *s0 = symbols_of (core, id0);
+	const RList *s1 = symbols_of (core, id1);
+	if (!s0 || !s1) {
+		eprintf ("Missing bin object\n");
+		return;
+	}
+	RListIter *iter, *iter2;
+	RBinSymbol *s, *s2;
+	if (id0 == id1) {
+		eprintf ("%d == %d\n", id0, id1);
+		return;
+	}
+	r_list_foreach (s0, iter, s) {
+		bool found = false;
+		r_list_foreach (s1, iter2, s2) {
+			if (!strcmp (s->name, s2->name)) {
+				found = true;
+			}
+		}
+		r_cons_printf ("%s%s\n", found? " ": "-", s->name);
+	}
+	r_list_foreach (s1, iter, s) {
+		bool found = false;
+		r_list_foreach (s0, iter2, s2) {
+			if (!strcmp (s->name, s2->name)) {
+				found = true;
+			}
+		}
+		if (!found) {
+			r_cons_printf ("+%s\n", s->name);
+		}
+	}
+}
+
+static void _core_cmp_info(RCore *core, const char *input) {
+	RBinFile *cur = core->bin->cur;
+	int id0 = (cur && cur->o) ? cur->id: 0;
+	int id1 = atoi (input + 1);
+	// do the magic
+	switch (input[0]) {
+	case 's':
+		_core_cmp_info_symbols (core, id0, id1);
+		break;
+	case 'l':
+		_core_cmp_info_libs (core, id0, id1);
+		break;
+	case 'i':
+		_core_cmp_info_imports (core, id0, id1);
+		break;
+	default:
+		r_core_cmd_help (core, help_message_ci);
+		break;
+	}
+}
+
 static int cmd_cmp(void *data, const char *input) {
 	static char *oldcwd = NULL;
 	int ret = 0, i, mode = 0;
@@ -561,11 +724,17 @@ static int cmd_cmp(void *data, const char *input) {
 		break;
 	case 'a': // "cat"
 		if (input[1] == 't') {
-			const char *path = r_str_trim_ro (input + 2);
-			if (*path == '$') {
-				const char *oldText = r_cmd_alias_get (core->rcmd, path, 1);
-				if (oldText) {
-					r_cons_printf ("%s\n", oldText + 1);
+			const char *path = r_str_trim_head_ro (input + 2);
+			if (*path == '$' && !path[1]) {
+				eprintf ("No alias name given.\n");
+			} else if (*path == '$') {
+				RCmdAliasVal *v = r_cmd_alias_get (core->rcmd, path+1);
+				if (v) {
+					char *v_str = r_cmd_alias_val_strdup (v);
+					r_cons_println (v_str);
+					free (v_str);
+				} else {
+					eprintf ("No such alias \"$%s\"\n", path+1);
 				}
 			} else {
 				if (r_fs_check (core->fs, path)) {
@@ -654,18 +823,17 @@ static int cmd_cmp(void *data, const char *input) {
 		free (buf);
 		free (filled);
 		break;
-	case 'X':
+	case 'X': // "cX"
 		buf = malloc (core->blocksize);
 		if (buf) {
 			if (!r_io_read_at (core->io, r_num_math (core->num,
 					    input + 1), buf, core->blocksize)) {
 				eprintf ("Cannot read hexdump\n");
 			} else {
-				val = radare_compare (core, block, buf, ret, mode);
+				val = radare_compare (core, block, buf, core->blocksize, mode);
 			}
 			free (buf);
 		}
-		return false;
 		break;
 	case 'f':
 		if (input[1] != ' ') {
@@ -784,6 +952,9 @@ static int cmd_cmp(void *data, const char *input) {
 			core->print->flags = oflags;
 		}
 		break;
+	case 'i': // "ci"
+		_core_cmp_info (core, input + 1);
+		break;
 	case 'g': // "cg"
 	{          // XXX: this is broken
 		int diffops = 0;
@@ -791,8 +962,13 @@ static int cmd_cmp(void *data, const char *input) {
 		char *file2 = NULL;
 		switch (input[1]) {
 		case 'o':         // "cgo"
-			file2 = (char *) r_str_trim_ro (input + 2);
-			r_anal_diff_setup (core->anal, true, -1, -1);
+			file2 = (char *) r_str_trim_head_ro (input + 2);
+			if (*file2) {
+				r_anal_diff_setup (core->anal, true, -1, -1);
+			} else {
+				eprintf ("Usage: cgo [file]\n");
+				return false;
+			}
 			break;
 		case 'f':         // "cgf"
 			eprintf ("TODO: agf is experimental\n");
@@ -801,7 +977,7 @@ static int cmd_cmp(void *data, const char *input) {
 				r_num_math (core->num, input + 2));
 			return false;
 		case ' ':
-			file2 = (char *) r_str_trim_ro (input + 2);
+			file2 = (char *) r_str_trim_head_ro (input + 2);
 			r_anal_diff_setup (core->anal, false, -1, -1);
 			break;
 		default: {
@@ -983,6 +1159,7 @@ static int cmd_cmp(void *data, const char *input) {
 		break;
 	default:
 		r_core_cmd_help (core, help_msg_c);
+		break;
 	}
 	if (val != UT64_MAX) {
 		core->num->value = val;
