@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2009-2020 - pancake */
+/* radare2 - LGPL - Copyright 2009-2021 - pancake */
 
 #include <stddef.h>
 #include <stdbool.h>
@@ -17,7 +17,7 @@ static const char *help_msg_e[] = {
 	"e-", "", "reset config vars",
 	"e*", "", "dump config vars in r commands",
 	"e!", "a", "invert the boolean value of 'a' var",
-	"ec", " [k] [color]", "set color for given key (prompt, offset, ...)",
+	"ec", "[?] [k] [color]", "set color for given key (prompt, offset, ...)",
 	"ee", "var", "open editor to change the value of var",
 	"ed", "", "open editor to change the ~/.radare2rc",
 	"ej", "", "list config vars in JSON",
@@ -42,7 +42,7 @@ static const char *help_msg_ec[] = {
 	"eco", " [theme]", "load theme if provided (list available themes if not)",
 	"ecp", "", "load previous color theme",
 	"ecn", "", "load next color theme",
-	"ecH", " [?]", "highlight word or instruction",
+	"ecH", "[?]", "highlight word or instruction",
 	"ec", " prompt red", "change color of prompt",
 	"ec", " prompt red blue", "change color and background of prompt",
 	"Vars:", "", "",
@@ -64,14 +64,8 @@ static const char *help_msg_eco[] = {
 	NULL
 };
 
-static char *curtheme = "default";
 static bool getNext = false;
 
-static void cmd_eval_init(RCore *core) {
-	DEFINE_CMD_DESCRIPTOR (core, e);
-	DEFINE_CMD_DESCRIPTOR (core, ec);
-}
-
 static bool load_theme(RCore *core, const char *path) {
 	if (!r_file_exists (path)) {
 		return false;
@@ -85,12 +79,12 @@ static bool load_theme(RCore *core, const char *path) {
 	return res;
 }
 
-static bool nextpal_item(RCore *core, int mode, const char *file, int ctr) {
+static bool nextpal_item(RCore *core, PJ *pj, int mode, const char *file) {
 	const char *fn = r_str_lchr (file, '/');
 	if (!fn) fn = file;
 	switch (mode) {
 	case 'j': // json
-		r_cons_printf ("%s\"%s\"", ctr?",":"", fn);
+		pj_s (pj, fn);
 		break;
 	case 'l': // list
 		r_cons_println (fn);
@@ -99,16 +93,20 @@ static bool nextpal_item(RCore *core, int mode, const char *file, int ctr) {
 		// TODO: move logic here
 		break;
 	case 'n': // next
+		if (core->theme && !strcmp (core->theme, "default")) {
+			core->theme = r_str_dup (core->theme, fn);
+			getNext = false;
+		}
 		if (getNext) {
-			curtheme = r_str_dup (curtheme, fn);
+			core->theme = r_str_dup (core->theme, fn);
 			getNext = false;
 			return false;
-		} else if (curtheme) {
-			if (!strcmp (curtheme, fn)) {
+		} else if (core->theme) {
+			if (!strcmp (core->theme, fn)) {
 				getNext = true;
 			}
 		} else {
-			curtheme = r_str_dup (curtheme, fn);
+			core->theme = r_str_dup (core->theme, fn);
 			return false;
 		}
 		break;
@@ -123,7 +121,7 @@ static bool cmd_load_theme(RCore *core, const char *_arg) {
 		return false;
 	}
 	if (!r_str_cmp (_arg, "default", strlen (_arg))) {
-		curtheme = strdup (_arg);
+		core->theme = r_str_dup (core->theme, _arg);
 		r_cons_pal_init (core->cons->context);
 		return true;
 	}
@@ -139,10 +137,10 @@ static bool cmd_load_theme(RCore *core, const char *_arg) {
 
 	if (!load_theme (core, home)) {
 		if (load_theme (core, path)) {
-			curtheme = r_str_dup (curtheme, arg);
+			core->theme = r_str_dup (core->theme, arg);
 		} else {
 			if (load_theme (core, arg)) {
-				curtheme = r_str_dup (curtheme, arg);
+				core->theme = r_str_dup (core->theme, arg);
 			} else {
 				char *absfile = r_file_abspath (arg);
 				eprintf ("eco: cannot open colorscheme profile (%s)\n", absfile);
@@ -169,8 +167,8 @@ static void list_themes_in_path(RList *list, const char *path) {
 	r_list_free (files);
 }
 
-R_API char *r_core_get_theme () {
-	return curtheme;
+R_API char *r_core_get_theme (RCore *core) {
+	return core->theme;
 }
 
 R_API RList *r_core_list_themes(RCore *core) {
@@ -199,68 +197,86 @@ static void nextpal(RCore *core, int mode) {
 	RListIter *iter;
 	const char *fn;
 	char *path = NULL;
-	int ctr = 0;
+	PJ *pj = NULL;
+	if (mode == 'j') {
+		pj = r_core_pj_new (core);
+		if (!pj) {
+			return;
+		}
+		pj_a (pj);
+	}
 	char *home = r_str_home (R2_HOME_THEMES R_SYS_DIR);
 
 	getNext = false;
-	if (mode == 'j') {
-		r_cons_printf ("[");
-	}
+	// spaguetti!
 	if (home) {
 		files = r_sys_dir (home);
-		r_list_foreach (files, iter, fn) {
-			if (*fn && *fn != '.') {
-				if (mode == 'p') {
-					const char *nfn = iter->n? iter->n->data: NULL;
-					if (!curtheme) {
-						free (home);
-						r_list_free (files);
-						return;
-					}
-					eprintf ("%s %s %s\n", nfn, curtheme, fn);
-					if (nfn && !strcmp (nfn, curtheme)) {
-						r_list_free (files);
-						files = NULL;
-						free (curtheme);
-						curtheme = strdup (fn);
-						R_FREE (home);
-						goto done;
-					}
-				} else {
-					if (!nextpal_item (core, mode, fn, ctr++)) {
-						r_list_free (files);
-						files = NULL;
-						R_FREE (home);
-						goto done;
+		if (files) {
+			r_list_sort (files, (RListComparator)strcmp);
+			r_list_foreach (files, iter, fn) {
+				if (*fn && *fn != '.') {
+					if (mode == 'p') {
+						const char *nfn = iter->n? iter->n->data: NULL;
+						if (!core->theme) {
+							free (home);
+							r_list_free (files);
+							return;
+						}
+						eprintf ("%s %s %s\n",
+							r_str_get (nfn),
+							r_str_get (core->theme),
+							r_str_get (fn));
+						if (nfn && !strcmp (nfn, core->theme)) {
+							r_list_free (files);
+							files = NULL;
+							free (core->theme);
+							core->theme = strdup (fn);
+							R_FREE (home);
+							goto done;
+						}
+					} else {
+						if (!nextpal_item (core, pj, mode, fn)) {
+							r_list_free (files);
+							files = NULL;
+							R_FREE (home);
+							goto done;
+						}
 					}
 				}
 			}
 		}
 		r_list_free (files);
+		files = NULL;
 		R_FREE (home);
 	}
 
 	path = r_str_r2_prefix (R2_THEMES R_SYS_DIR);
 	if (path) {
 		files = r_sys_dir (path);
-		r_list_foreach (files, iter, fn) {
-			if (*fn && *fn != '.') {
-				if (mode == 'p') {
-					const char *nfn = iter->n? iter->n->data: NULL;
-					if (!curtheme) {
-						free (home);
-						r_list_free (files);
-						return;
-					}
-					eprintf ("%s %s %s\n", nfn, curtheme, fn);
-					if (nfn && !strcmp (nfn, curtheme)) {
-						free (curtheme);
-						curtheme = strdup (fn);
-						goto done;
-					}
-				} else {
-					if (!nextpal_item (core, mode, fn, ctr++)) {
-						goto done;
+		if (files) {
+			r_list_sort (files, (RListComparator)strcmp);
+			r_list_foreach (files, iter, fn) {
+				if (*fn && *fn != '.') {
+					if (mode == 'p') {
+						const char *nfn = iter->n? iter->n->data: NULL;
+						if (!core->theme) {
+							free (home);
+							r_list_free (files);
+							return;
+						}
+						eprintf ("%s %s %s\n",
+							r_str_get (nfn),
+							r_str_get (core->theme),
+							r_str_get (fn));
+						if (nfn && !strcmp (nfn, core->theme)) {
+							free (core->theme);
+							core->theme = strdup (fn);
+							goto done;
+						}
+					} else { // next
+						if (!nextpal_item (core, pj, mode, fn)) {
+							goto done;
+						}
 					}
 				}
 			}
@@ -270,21 +286,23 @@ static void nextpal(RCore *core, int mode) {
 done:
 	free (path);
 	if (getNext) {
-		R_FREE (curtheme);
+		R_FREE (core->theme);
 		nextpal (core, mode);
 		return;
 	}
-	if (mode == 'l' && !curtheme && !r_list_empty (files)) {
+	if (mode == 'l' && !core->theme && !r_list_empty (files)) {
 		//nextpal (core, mode);
 	} else if (mode == 'n' || mode == 'p') {
-		if (curtheme) {
-			r_core_cmdf (core, "eco %s", curtheme);
+		if (core->theme) {
+			r_core_cmdf (core, "eco %s", core->theme);
 		}
 	}
 	r_list_free (files);
 	files = NULL;
 	if (mode == 'j') {
-		r_cons_printf ("]\n");
+		pj_end (pj);
+		r_cons_println (pj_string (pj));
+		pj_free (pj);
 	}
 }
 
@@ -334,7 +352,7 @@ static int cmd_eval(void *data, const char *input) {
 			eprintf ("Usage: et [varname]  ; show type of eval var\n");
 		}
 		break;
-	case 'n': // "en"
+	case 'n': // "en" "env"
 		if (!strchr (input, '=')) {
 			char *var, *p;
 			var = strchr (input, ' ');
@@ -350,23 +368,30 @@ static int cmd_eval(void *data, const char *input) {
 					e++;
 				}
 			}
-		} else if (strlen (input)>3) {
-			char *v, *k = strdup (input+3);
-			if (!k) break;
+		} else if (strlen (input) > 3) {
+			char *v, *k = strdup (input + 3);
+			if (!k) {
+				break;
+			}
 			v = strchr (k, '=');
-			if (v) {
+			if (*k && v) {
 				*v++ = 0;
 				r_str_trim (k);
 				r_str_trim (v);
+				char *last = k + strlen (k) - 1;
+				if (*last == '%') {
+					*last = 0;
+					r_str_trim (k);
+				}
 				r_sys_setenv (k, v);
 			}
 			free (k);
 		}
 		return true;
-	case 'x': // exit
+	case 'x': // "ex"
 		// XXX we need headers for the cmd_xxx files.
 		return cmd_quit (data, "");
-	case 'j': // json
+	case 'j': // "ej"
 		r_config_list (core->config, NULL, 'j');
 		break;
 	case 'v': // verbose
@@ -389,29 +414,25 @@ static int cmd_eval(void *data, const char *input) {
 			} else if (input[2] == ' ') {
 				cmd_load_theme (core, input + 3);
 			} else if (input[2] == 'o') {
-				cmd_load_theme (core, r_core_get_theme ());
+				cmd_load_theme (core, core->theme);
 			} else if (input[2] == 'c' || input[2] == '.') {
-				r_cons_printf ("%s\n", r_core_get_theme ());
+				r_cons_printf ("%s\n", core->theme);
 			} else if (input[2] == '?') {
 				r_core_cmd_help (core, help_msg_eco);
-			} else if (input[2] == 'q') {
-				RList *themes_list = r_core_list_themes (core);
-				RListIter *th_iter;
-				const char *th;
-				r_list_foreach (themes_list, th_iter, th) {
-					r_cons_printf ("%s\n", th);
-				}
 			} else {
 				RList *themes_list = r_core_list_themes (core);
 				RListIter *th_iter;
 				const char *th;
 				r_list_foreach (themes_list, th_iter, th) {
-					if (curtheme && !strcmp (curtheme, th)) {
-						r_cons_printf ("> %s\n", th);
+					if (input[2] == 'q') {
+						r_cons_printf ("%s\n", th);
+					} else if (core->theme && !strcmp (core->theme, th)) {
+						r_cons_printf ("- %s\n", th);
 					} else {
 						r_cons_printf ("  %s\n", th);
 					}
 				}
+				r_list_free (themes_list);
 			}
 			break;
 		case 's': r_cons_pal_show (); break; // "ecs"
@@ -473,19 +494,19 @@ static int cmd_eval(void *data, const char *input) {
 				r_str_argv_free (argv);
 				return false;
 			case '.':
-				r_meta_list_at (core->anal, R_META_TYPE_HIGHLIGHT, 0, core->offset);
+				r_meta_print_list_in_function (core->anal, R_META_TYPE_HIGHLIGHT, 0, core->offset, NULL);
 				r_str_argv_free (argv);
 				return false;
 			case '\0':
-				r_meta_list (core->anal, R_META_TYPE_HIGHLIGHT, 0);
+				r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, 0, NULL);
 				r_str_argv_free (argv);
 				return false;
 			case 'j':
-				r_meta_list (core->anal, R_META_TYPE_HIGHLIGHT, 'j');
+				r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, 'j', NULL);
 				r_str_argv_free (argv);
 				return false;
 			case '*':
-				r_meta_list (core->anal, R_META_TYPE_HIGHLIGHT, '*');
+				r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, '*', NULL);
 				r_str_argv_free (argv);
 				return false;
 			case ' ':
@@ -526,8 +547,9 @@ static int cmd_eval(void *data, const char *input) {
 				return true;
 			}
 			r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset, "");
-			char *str = r_meta_get_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset);
-			char *dup = r_str_newf ("%s \"%s%s\"", str?str:"", word?word:"", color_code?color_code:r_cons_singleton ()->context->pal.wordhl);
+			const char *str = r_meta_get_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset);
+			char *dup = r_str_newf ("%s \"%s%s\"", r_str_get (str), r_str_get (word),
+				color_code ? color_code : r_cons_singleton ()->context->pal.wordhl);
 			r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset, dup);
 			r_str_argv_free (argv);
 			R_FREE (word);
