@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include <string.h>
 #include "r_config.h"
@@ -11,16 +11,19 @@ static const char *help_msg_L[] = {
 	"L",  "", "show this help",
 	"L", " blah."R_LIB_EXT, "load plugin file",
 	"L-", "duk", "unload core plugin by name",
-	"Ll", "", "list lang plugins (same as #!)",
-	"LL", "", "lock screen",
 	"La", "", "list asm/anal plugins (aL, e asm.arch=" "??" ")",
 	"Lc", "", "list core plugins",
 	"Ld", "", "list debug plugins (same as dL)",
 	"LD", "", "list supported decompilers (e cmd.pdc=?)",
-	"Lm", "", "list fs plugins (same as mL)",
+	"Le", "", "list esil plugins",
+	"Lg", "", "list egg plugins",
 	"Lh", "", "list hash plugins (same as ph)",
 	"Li", "", "list bin plugins (same as iL)",
+	"Ll", "", "list lang plugins (same as #!)",
+	"LL", "", "lock screen",
+	"Lm", "", "list fs plugins (same as mL)",
 	"Lo", "", "list io plugins (same as oL)",
+	"Lp", "", "list parser plugins (e asm.parser=?)",
 	NULL
 };
 
@@ -43,12 +46,6 @@ static const char *help_msg_T[] = {
 	NULL
 };
 
-// TODO #7967 help refactor: move L to another place
-static void cmd_log_init(RCore *core) {
-	DEFINE_CMD_DESCRIPTOR (core, L);
-	DEFINE_CMD_DESCRIPTOR (core, T);
-}
-
 static void screenlock(RCore *core) {
 	//  char *pass = r_cons_input ("Enter new password: ");
 	char *pass = r_cons_password (Color_INVERT "Enter new password:"Color_INVERT_RESET);
@@ -68,9 +65,9 @@ static void screenlock(RCore *core) {
 	}
 	bool running = true;
 	r_cons_clear_buffer ();
-	ut64 begin = r_sys_now ();
+	ut64 begin = r_time_now ();
 	ut64 last = UT64_MAX;
-	ut64 tries = 0;
+	int tries = 0;
 	do {
 		r_cons_clear00 ();
 		r_cons_printf ("Retries: %d\n", tries);
@@ -85,7 +82,7 @@ static void screenlock(RCore *core) {
 			running = false;
 		} else {
 			eprintf ("\nInvalid password.\n");
-			last = r_sys_now ();
+			last = r_time_now ();
 			tries++;
 		}
 		free (msg);
@@ -110,7 +107,7 @@ static int textlog_chat(RCore *core) {
 	for (;;) {
 		r_core_log_list (core, lastmsg, 0, 0);
 		lastmsg = core->log->last;
-		if (r_cons_fgets (buf, sizeof (buf) - 1, 0, NULL) < 0) {
+		if (r_cons_fgets (buf, sizeof (buf), 0, NULL) < 0) {
 			return 1;
 		}
 		if (!*buf) {
@@ -324,10 +321,10 @@ static int cmd_plugins(void *data, const char *input) {
 		// return r_core_cmd0 (core, "Lc");
 		break;
 	case '-':
-		r_lib_close (core->lib, r_str_trim_ro (input + 1));
+		r_lib_close (core->lib, r_str_trim_head_ro (input + 1));
 		break;
 	case ' ':
-		r_lib_open (core->lib, r_str_trim_ro (input + 1));
+		r_lib_open (core->lib, r_str_trim_head_ro (input + 1));
 		break;
 	case '?':
 		r_core_cmd_help (core, help_msg_L);
@@ -335,6 +332,9 @@ static int cmd_plugins(void *data, const char *input) {
 	case 'm': // "Lm"
 		r_core_cmdf (core, "mL%s", input + 1);
 		break;
+	case 'e': // "Le"
+		r_core_cmdf (core, "aeL%s", input + 1);
+		break;
 	case 'd': // "Ld"
 		r_core_cmdf (core, "dL%s", input + 1);
 		break;
@@ -344,9 +344,12 @@ static int cmd_plugins(void *data, const char *input) {
 	case 'a': // "La"
 		r_core_cmd0 (core, "e asm.arch=??");
 		break;
+	case 'p': // "Lp"
+		r_core_cmd0 (core, "e asm.parser=?");
+		break;
 	case 'D': // "LD"
 		if (input[1] == ' ') {
-			r_core_cmdf (core, "e cmd.pdc=%s", r_str_trim_ro (input + 2));
+			r_core_cmdf (core, "e cmd.pdc=%s", r_str_trim_head_ro (input + 2));
 		} else {
 			r_core_cmd0 (core, "e cmd.pdc=?");
 		}
@@ -357,6 +360,9 @@ static int cmd_plugins(void *data, const char *input) {
 	case 'L': // "LL"
 		screenlock (core);
 		break;
+	case 'g': // "Lg"
+		r_core_cmd0 (core, "gL");
+		break;
 	case 'o': // "Lo"
 	case 'i': // "Li"
 		r_core_cmdf (core, "%cL", input[0]);
@@ -366,16 +372,25 @@ static int cmd_plugins(void *data, const char *input) {
 		RCorePlugin *cp;
 		switch (input[1]) {
 		case 'j': {
-			r_cons_printf ("[");
-			bool is_first_element = true;
-			r_list_foreach (core->rcmd->plist, iter, cp) {
-				r_cons_printf ("%s{\"Name\":\"%s\",\"Description\":\"%s\"}",
-					is_first_element? "" : ",", cp->name, cp->desc);
-				is_first_element = false;
+			PJ *pj = r_core_pj_new (core);
+			if (!pj) {
+				return 1;
 			}
-			r_cons_printf ("]\n");
+			pj_a (pj);
+			r_list_foreach (core->rcmd->plist, iter, cp) {
+				pj_o (pj);
+				pj_ks (pj, "Name", cp->name);
+				pj_ks (pj, "Description", cp->desc);
+				pj_end (pj);
+			}
+			pj_end (pj);
+			r_cons_println (pj_string (pj));
+			pj_free (pj);
 			break;
 			}
+		case ' ':
+			r_lib_open (core->lib, r_str_trim_head_ro (input + 2));
+			break;
 		case 0:
 			r_lib_list (core->lib);
 			r_list_foreach (core->rcmd->plist, iter, cp) {
@@ -383,7 +398,7 @@ static int cmd_plugins(void *data, const char *input) {
 			}
 			break;
 		default:
-			eprintf ("oops\n");
+			r_core_cmd_help (core, help_msg_L);
 			break;
 		}
 		}
