@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2016 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include "r_core.h"
 
@@ -15,20 +15,17 @@ static const char *help_msg_q[] = {
 	NULL
 };
 
-static void cmd_quit_init(RCore *core) {
-	DEFINE_CMD_DESCRIPTOR (core, q);
-}
-
 static int cmd_Quit(void *data, const char *input) {
 	RCore *core = (RCore *)data;
 	if (input[0] == '!') {
-		if (input[1] == '!') {
+		if (input[1] == '!' || !input[1]) {
 			if (!r_sandbox_enable (false)) {
+				r_cons_flush ();
 				exit (0);
 			}
 			return -2;
 		}
-		r_config_set (core->config, "scr.histsave", "false");
+		r_config_set (core->config, "scr.hist.save", "false");
 	}
 	if (IS_DIGIT (input[0]) || input[0] == ' ') {
 		core->num->value = r_num_math (core->num, input);
@@ -59,13 +56,11 @@ static int cmd_quit(void *data, const char *input) {
 		} else {
 			core->num->value = 0LL;
 		}
-
 		if (*input == 'y') {
 			core->num->value = 5;
 		} else if (*input == 'n') {
 			core->num->value = 1;
 		}
-
 		if (input[1] == 'y') {
 			core->num->value += 10;	
 		} else if (input[1] == 'n') {
