@@ -1,10 +1,10 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake, oddcoder, Anton Kochkov, Jody Frankowski */
+/* radare - LGPL - Copyright 2009-2020 - pancake, oddcoder, Anton Kochkov, Jody Frankowski */
 
 #include <string.h>
 #include "r_anal.h"
 #include "r_cons.h"
 #include "r_core.h"
-#include "sdb/sdb.h"
+#include <sdb.h>
 
 static const char *help_msg_t[] = {
 	"Usage: t", "", "# cparse types commands",
@@ -15,7 +15,7 @@ static const char *help_msg_t[] = {
 	"t-", " <name>", "Delete types by its name",
 	"t-*", "", "Remove all types",
 	"tail", " [filename]", "Output the last part of files",
-	"tc", "[type.name]", "List all/given types in C output format",
+	"tc", " [type.name]", "List all/given types in C output format",
 	"te", "[?]", "List all loaded enums",
 	"td", "[?] <string>", "Load types from string",
 	"tf", "", "List all loaded functions signatures",
@@ -24,10 +24,12 @@ static const char *help_msg_t[] = {
 	"tn", "[?] [-][addr]", "manage noreturn function attributes and marks",
 	"to", " -", "Open cfg.editor to load types",
 	"to", " <path>", "Load types from C header file",
-	"toe", "[type.name]", "Open cfg.editor to edit types",
+	"toe", " [type.name]", "Open cfg.editor to edit types",
 	"tos", " <path>", "Load types from parsed Sdb database",
-	"tp", "  <type> [addr|varname]", "cast data at <address> to <type> and print it",
-	"tpx", " <type> <hexpairs>", "Show value for type with specified byte sequence",
+	"touch", " <file>", "Create or update timestamp in file",
+	"tp", "  <type> [addr|varname]", "cast data at <address> to <type> and print it (XXX: type can contain spaces)",
+	"tpv", " <type> @ [value]", "Show offset formatted for given type",
+	"tpx", " <type> <hexpairs>", "Show value for type with specified byte sequence (XXX: type can contain spaces)",
 	"ts", "[?]", "Print loaded struct types",
 	"tu", "[?]", "Print loaded union types",
 	"tx", "[f?]", "Type xrefs",
@@ -36,12 +38,13 @@ static const char *help_msg_t[] = {
 };
 
 static const char *help_msg_tcc[] = {
-	"Usage: tfc", "[-name]", "# type function calling conventions",
+	"Usage: tcc", "[-name]", "# type function calling conventions (see also afc? and arcc)",
 	"tcc", "", "List all calling convcentions",
-	"tcc", " r0 pascal(r0,r1,r2)", "Show signature for the 'pascal' calling convention",
+	"tcc", " r0 pascal(r0,r1,r2)", "Define signature for pascall cc (see also arcc)",
 	"tcc", "-pascal", "Remove the pascal cc",
+	"tcc-*", "", "Unregister all the calling conventions",
 	"tcck", "", "List calling conventions in k=v",
-	"tccl", "", "List the cc signatures",
+	"tccl", "", "List cc signatures (return ccname (arg0, arg1, ..) err;)",
 	"tccj", "", "List them in JSON",
 	"tcc*", "", "List them as r2 commands",
 	NULL
@@ -58,8 +61,8 @@ static const char *help_msg_tf[] = {
 	"tf", " <name>", "Show function signature",
 	"tfc", " <name>", "Show function signature in C syntax",
 	"tfcj", " <name>", "Same as above but in JSON",
-	"tfj", "", "List all function definitions in json",
-	"tfj", " <name>", "Show function signature in json",
+	"tfj", "", "List all function definitions in JSON",
+	"tfj", " <name>", "Show function signature in JSON",
 	NULL
 };
 
@@ -68,12 +71,21 @@ static const char *help_msg_to[] = {
 	"to", " -", "Open cfg.editor to load types",
 	"to", " <path>", "Load types from C header file",
 	"tos", " <path>", "Load types from parsed Sdb database",
+	"touch", " <file>", "Create or update timestamp in file",
+	NULL
+};
+
+static const char *help_msg_tp[] = {
+	"Usage: tp[...]", "", "",
+	"tp", "  <type> [addr|varname]", "cast data at <address> to <type> and print it (XXX: type can contain spaces)",
+	"tpv", " <type> @ [value]", "Show offset formatted for given type",
+	"tpx", " <type> <hexpairs>", "Show value for type with specified byte sequence (XXX: type can contain spaces)",
 	NULL
 };
 
 static const char *help_msg_tc[] = {
 	"Usage: tc[...]", " [cctype]", "",
-	"tc", "[type.name]", "List all/given loaded types in C output format with newlines",
+	"tc", " [type.name]", "List all/given loaded types in C output format with newlines",
 	"tcd", "", "List all loaded types in C output format without newlines",
 	"tcc", "?", "Manage calling conventions types",
 	"tc?", "", "show this help",
@@ -144,7 +156,7 @@ static const char *help_msg_ts[] = {
 	"ts*", "", "Show pf.<name> format string for all loaded structs",
 	"ts*", " [type]", "Show pf.<name> format string for given struct",
 	"tsc", "<name>", "List all/given loaded structs in C output format with newlines",
-	"tsd", "", "List all loaded structs in C output format without newlines", 
+	"tsd", "", "List all loaded structs in C output format without newlines",
 	"tss", " [type]", "Display size of struct",
 	"ts?", "", "show this help",
 	NULL
@@ -164,93 +176,104 @@ static const char *help_msg_tu[] = {
 	NULL
 };
 
-static void cmd_type_init(RCore *core) {
-	DEFINE_CMD_DESCRIPTOR (core, t);
-	DEFINE_CMD_DESCRIPTOR_SPECIAL (core, t-, t_minus);
-	DEFINE_CMD_DESCRIPTOR (core, tc);
-	DEFINE_CMD_DESCRIPTOR (core, td);
-	DEFINE_CMD_DESCRIPTOR (core, te);
-	DEFINE_CMD_DESCRIPTOR (core, tl);
-	DEFINE_CMD_DESCRIPTOR (core, tn);
-	DEFINE_CMD_DESCRIPTOR (core, ts);
-	DEFINE_CMD_DESCRIPTOR (core, tu);
-	DEFINE_CMD_DESCRIPTOR (core, tt);
-}
-
 static void show_help(RCore *core) {
 	r_core_cmd_help (core, help_msg_t);
 }
 
-static void __core_cmd_tcc(RCore *core, const char *input) {
+static bool cc_cb(void *p, const char *k, const char *v) {
+	if (!strcmp (v, "cc")) {
+		RList *list = (RList*)p;
+		r_list_append (list, (void*)k);
+	}
+	return true;
+}
+
+static void cmd_afcl(RCore *core, const char *input) {
+	int mode = 0;
+	PJ *pj = NULL;
+	if (input) {
+		mode = *input;
+		if (*input == 'j') {
+			pj = r_core_pj_new (core);
+			pj_o (pj);
+		}
+	}
+	RList *list = r_list_newf (NULL);
+	sdb_foreach (core->anal->sdb_cc, cc_cb, list);
+	char *cc;
+	RListIter *iter;
+	r_list_sort (list, (RListComparator)strcmp);
+	r_list_foreach (list, iter, cc) {
+		if (pj) {
+			pj_ko (pj, cc);
+			r_anal_cc_get_json (core->anal, pj, cc);
+			pj_end (pj);
+		} else if (mode == 'l') {
+			char *sig = r_anal_cc_get (core->anal, cc);
+			r_cons_println (sig);
+			free (sig);
+		} else if (mode == '*') {
+			char *ccexpr = r_anal_cc_get (core->anal, cc);
+			r_cons_printf ("tcc %s\n", ccexpr);
+			free (ccexpr);
+		} else {
+			r_cons_println (cc);
+		}
+	}
+	r_list_free (list);
+	if (pj) {
+		pj_end (pj);
+		char *j = pj_drain (pj);
+		r_cons_println (j);
+		free (j);
+	}
+}
+
+static void cmd_afck(RCore *core, const char *c) {
+	const char *s = "anal/cc/*";
+	char *out = sdb_querys (core->sdb, NULL, 0, s);
+	if (out) {
+		r_cons_print (out);
+	}
+	free (out);
+}
+
+static void cmd_tcc(RCore *core, const char *input) {
 	switch (*input) {
 	case '?':
 		r_core_cmd_help (core, help_msg_tcc);
 		break;
 	case '-':
-		r_anal_cc_del (core->anal, r_str_trim_ro (input + 1));
+		if (input[1] == '*') {
+			sdb_reset (core->anal->sdb_cc);
+		} else {
+			r_anal_cc_del (core->anal, r_str_trim_head_ro (input + 1));
+		}
 		break;
 	case 0:
-		r_core_cmd0 (core, "afcl");
-		break;
-	case 'j':
-		{
-			char *ccs = r_core_cmd_strf (core, "afcl");
-			RList *list = r_str_split_list (ccs, "\n", 0);
-			RListIter *iter;
-			const char *cc;
-			PJ *pj = pj_new ();
-			pj_a (pj);
-			r_list_foreach (list, iter, cc) {
-				char *ccexpr = r_anal_cc_get (core->anal, cc);
-				// TODO: expose this as an object, not just an array of strings
-				pj_s (pj, ccexpr);
-				free (ccexpr);
-			}
-			pj_end (pj);
-			r_cons_printf ("%s\n", pj_string (pj));
-			pj_free (pj);
-			r_list_free (list);
-			free (ccs);
-		}
+		cmd_afcl (core, "");
 		break;
 	case 'l':
-		{
-			char *ccs = r_core_cmd_strf (core, "afcl");
-			RList *list = r_str_split_list (ccs, "\n", 0);
-			RListIter *iter;
-			const char *cc;
-			r_list_foreach (list, iter, cc) {
-				char *ccexpr = r_anal_cc_get (core->anal, cc);
-				r_cons_printf ("%s\n", ccexpr);
-				free (ccexpr);
-			}
-			r_list_free (list);
-			free (ccs);
-		}
+		cmd_afcl (core, "l");
+		break;
+	case 'j':
+		cmd_afcl (core, "j");
+		break;
 		break;
 	case '*':
-		{
-			char *ccs = r_core_cmd_strf (core, "afcl");
-			RList *list = r_str_split_list (ccs, "\n", 0);
-			RListIter *iter;
-			const char *cc;
-			r_list_foreach (list, iter, cc) {
-				char *ccexpr = r_anal_cc_get (core->anal, cc);
-				r_cons_printf ("tfc %s\n", ccexpr);
-				free (ccexpr);
-			}
-			r_list_free (list);
-			free (ccs);
-		}
+		cmd_afcl (core, "*");
 		break;
 	case 'k':
-		r_core_cmd0 (core, "afck");
+		cmd_afck (core, NULL);
 		break;
 	case ' ':
 		if (strchr (input, '(')) {
-			r_anal_cc_set (core->anal, input + 1);
+			if (!r_anal_cc_set (core->anal, input + 1)) {
+				eprintf ("Invalid syntax in cc signature.");
+			}
 		} else {
-			char *cc = r_anal_cc_get (core->anal, input + 1);
+			const char *ccname = r_str_trim_head_ro (input + 1);
+			char *cc = r_anal_cc_get (core->anal, ccname);
 			if (cc) {
 				r_cons_printf ("%s\n", cc);
 				free (cc);
@@ -300,11 +323,11 @@ static int cmd_tail(void *data, const char *_input) { // "tail"
 	char *arg = strchr (input, ' ');
 	char *tmp, *count;
 	if (arg) {
-		arg = (char *)r_str_trim_ro (arg + 1); 	// contains "count filename"
+		arg = (char *)r_str_trim_head_ro (arg + 1); 	// contains "count filename"
 		count = strchr (arg, ' ');
 		if (count) {
 			*count = 0;	// split the count and file name
-			tmp = (char *)r_str_trim_ro (count + 1);
+			tmp = (char *)r_str_trim_head_ro (count + 1);
 			lines = atoi (arg);
 			arg = tmp;
 		}
@@ -336,9 +359,9 @@ static void cmd_type_noreturn(RCore *core, const char *input) {
 	switch (input[0]) {
 	case '-': // "tn-"
 		if (input[1] == '*') {
-			r_core_cmd0 (core, "tn-`tn`");
+			r_core_cmd0 (core, "tn- `tn`");
 		} else {
-			char *s = strdup (r_str_trim_ro (input + 1));
+			char *s = strdup (r_str_trim_head_ro (input + 1));
 			RListIter *iter;
 			char *k;
 			RList *list = r_str_split_list (s, " ", 0);
@@ -351,7 +374,7 @@ static void cmd_type_noreturn(RCore *core, const char *input) {
 		break;
 	case ' ': // "tn"
 		{
-			const char *arg = r_str_trim_ro (input + 1);
+			const char *arg = r_str_trim_head_ro (input + 1);
 			ut64 n = r_num_math (core->num, arg);
 			if (n) {
 				r_anal_noreturn_add (core->anal, arg, n);
@@ -371,7 +394,7 @@ static void cmd_type_noreturn(RCore *core, const char *input) {
 	case 'n': // "tnn"
 		if (input[1] == ' ') {
 			/* do nothing? */
-			r_anal_noreturn_add (core->anal, r_str_trim_ro (input + 2), UT64_MAX);
+			r_anal_noreturn_add (core->anal, r_str_trim_head_ro (input + 2), UT64_MAX);
 		} else {
 			r_core_cmd_help (core, help_msg_tn);
 		}
@@ -392,15 +415,15 @@ static void cmd_type_noreturn(RCore *core, const char *input) {
 
 static Sdb *TDB_ = NULL; // HACK
 
-static int stdifstruct(void *user, const char *k, const char *v) {
+static bool stdifstruct(void *user, const char *k, const char *v) {
 	r_return_val_if_fail (TDB_, false);
-	if (!strcmp (v, "struct")) {
+	if (!strcmp (v, "struct") && !r_str_startswith (k, "typedef")) {
 		return true;
 	}
 	if (!strcmp (v, "typedef")) {
 		const char *typedef_key = sdb_fmt ("typedef.%s", k);
 		const char *type = sdb_const_get (TDB_, typedef_key, NULL);
-		if (type && r_str_startswith (type, "struct ")) {
+		if (type && r_str_startswith (type, "struct")) {
 			return true;
 		}
 	}
@@ -428,19 +451,9 @@ static int print_struct_union_list_json(Sdb *TDB, SdbForeachCallback filter) {
 		if (!k || !*k) {
 			continue;
 		}
-
 		pj_o (pj); // {
-		char *sizecmd = r_str_newf ("%s.%s.!size", sdbkv_value (kv), k);
-		if (!sizecmd) {
-			break;
-		}
-		char *size_s = sdb_querys (TDB, NULL, -1, sizecmd);
 		pj_ks (pj, "type", k); // key value pair of string and string
-		pj_ki (pj, "size", size_s ? atoi (size_s) : 0); // key value pair of string and int
 		pj_end (pj); // }
-
-		free (sizecmd);
-		free (size_s);
 	}
 	pj_end (pj); // ]
 
@@ -450,6 +463,7 @@ static int print_struct_union_list_json(Sdb *TDB, SdbForeachCallback filter) {
 	return 1;
 }
 
+// Rename to char *RAnal.type_to_c() {}
 static void print_struct_union_in_c_format(Sdb *TDB, SdbForeachCallback filter, const char *arg, bool multiline) {
 	char *name = NULL;
 	SdbKv *kv;
@@ -479,22 +493,30 @@ static void print_struct_union_in_c_format(Sdb *TDB, SdbForeachCallback filter,
 			if (var2) {
 				char *val = sdb_array_get (TDB, var2, 0, NULL);
 				if (val) {
+					char *arr = sdb_array_get (TDB, var2, 2, NULL);
+					int arrnum = atoi (arr);
+					free (arr);
 					if (multiline) {
-						r_cons_printf ("\t%s", val);
+						r_cons_printf ("  %s", val);
 						if (p && p[0] != '\0') {
 							r_cons_printf ("%s%s", strstr (val, " *")? "": " ", p);
+							if (arrnum) {
+								r_cons_printf ("[%d]", arrnum);
+							}
 						}
+						r_cons_println (";");
 					} else {
-						r_cons_printf ("%s%s %s;", space, val, p);
+						r_cons_printf ("%s%s %s", space, val, p);
+						if (arrnum) {
+							r_cons_printf ("[%d]", arrnum);
+						}
+						r_cons_print (";");
 						space = " ";
 					}
+					free (val);
 				}
-				if (multiline) {
-					r_cons_println (";");
-				}
-				free (val);
+				free (var2);
 			}
-			free (var2);
 			free (p);
 		}
 		free (var);
@@ -535,7 +557,7 @@ static void print_enum_in_c_format(Sdb *TDB, const char *arg, bool multiline) {
 						RTypeEnum *member;
 						separator = multiline? "\t": "";
 						r_list_foreach (list, iter, member) {
-							r_cons_printf ("%s%s = %d", separator, member->name, r_num_math (NULL, member->val));
+							r_cons_printf ("%s%s = %" PFMT64u, separator, member->name, r_num_math (NULL, member->val));
 							separator = multiline? ",\n\t": ", ";
 						}
 					}
@@ -552,18 +574,21 @@ static void print_enum_in_c_format(Sdb *TDB, const char *arg, bool multiline) {
 	ls_free (l);
 }
 
-static int printkey_cb(void *user, const char *k, const char *v) {
+static bool printkey_cb(void *user, const char *k, const char *v) {
 	r_cons_println (k);
-	return 1;
+	return true;
 }
 
 // maybe dupe?. should return char *instead of print for reusability
 static void printFunctionTypeC(RCore *core, const char *input) {
 	Sdb *TDB = core->anal->sdb_types;
 	char *res = sdb_querys (TDB, NULL, -1, sdb_fmt ("func.%s.args", input));
-	const char *name = r_str_trim_ro (input);
+	const char *name = r_str_trim_head_ro (input);
 	int i, args = sdb_num_get (TDB, sdb_fmt ("func.%s.args", name), 0);
 	const char *ret = sdb_const_get (TDB, sdb_fmt ("func.%s.ret", name), 0);
+	if (!ret) {
+		ret = "void";
+	}
 	if (!ret || !name) {
 		// missing function name specified
 		return;
@@ -590,10 +615,11 @@ static void printFunctionType(RCore *core, const char *input) {
 	}
 	pj_o (pj);
 	char *res = sdb_querys (TDB, NULL, -1, sdb_fmt ("func.%s.args", input));
-	const char *name = r_str_trim_ro (input);
+	const char *name = r_str_trim_head_ro (input);
 	int i, args = sdb_num_get (TDB, sdb_fmt ("func.%s.args", name), 0);
 	pj_ks (pj, "name", name);
-	pj_ks (pj, "ret", sdb_const_get (TDB, sdb_fmt ("func.%s.ret", name), 0));
+	const char *ret_type = sdb_const_get (TDB, sdb_fmt ("func.%s.ret", name), 0);
+	pj_ks (pj, "ret", r_str_get_fail (ret_type, "void"));
 	pj_k (pj, "args");
 	pj_a (pj);
 	for (i = 0; i < args; i++) {
@@ -621,47 +647,48 @@ static void printFunctionType(RCore *core, const char *input) {
 	free (res);
 }
 
-static int printfunc_json_cb(void *user, const char *k, const char *v) {
+static bool printfunc_json_cb(void *user, const char *k, const char *v) {
 	printFunctionType ((RCore *)user, k);
-	return 1;
+	return true;
 }
 
-static int stdiffunc(void *p, const char *k, const char *v) {
+static bool stdiffunc(void *p, const char *k, const char *v) {
 	return !strncmp (v, "func", strlen ("func") + 1);
 }
 
-static int stdifunion(void *p, const char *k, const char *v) {
+static bool stdifunion(void *p, const char *k, const char *v) {
 	return !strncmp (v, "union", strlen ("union") + 1);
 }
 
-static int sdbdeletelink(void *p, const char *k, const char *v) {
+static bool sdbdeletelink(void *p, const char *k, const char *v) {
 	RCore *core = (RCore *)p;
 	if (!strncmp (k, "link.", strlen ("link."))) {
 		r_type_del (core->anal->sdb_types, k);
 	}
-	return 1;
+	return true;
 }
 
-static int stdiflink(void *p, const char *k, const char *v) {
+static bool stdiflink(void *p, const char *k, const char *v) {
 	return !strncmp (k, "link.", strlen ("link."));
 }
 
-static int print_link_cb(void *p, const char *k, const char *v) {
+static bool print_link_cb(void *p, const char *k, const char *v) {
 	r_cons_printf ("0x%s = %s\n", k + strlen ("link."), v);
-	return 1;
+	return true;
 }
 
-static int print_link_json_cb(void *p, const char *k, const char *v) {
+//TODO PJ
+static bool print_link_json_cb(void *p, const char *k, const char *v) {
 	r_cons_printf ("{\"0x%s\":\"%s\"}", k + strlen ("link."), v);
-	return 1;
+	return true;
 }
 
-static int print_link_r_cb(void *p, const char *k, const char *v) {
+static bool print_link_r_cb(void *p, const char *k, const char *v) {
 	r_cons_printf ("tl %s = 0x%s\n", v, k + strlen ("link."));
-	return 1;
+	return true;
 }
 
-static int print_link_readable_cb(void *p, const char *k, const char *v) {
+static bool print_link_readable_cb(void *p, const char *k, const char *v) {
 	RCore *core = (RCore *)p;
 	char *fmt = r_type_format (core->anal->sdb_types, v);
 	if (!fmt) {
@@ -670,32 +697,33 @@ static int print_link_readable_cb(void *p, const char *k, const char *v) {
 	}
 	r_cons_printf ("(%s)\n", v);
 	r_core_cmdf (core, "pf %s @ 0x%s\n", fmt, k + strlen ("link."));
-	return 1;
+	return true;
 }
 
-static int print_link_readable_json_cb(void *p, const char *k, const char *v) {
+//TODO PJ
+static bool print_link_readable_json_cb(void *p, const char *k, const char *v) {
 	RCore *core = (RCore *)p;
 	char *fmt = r_type_format (core->anal->sdb_types, v);
 	if (!fmt) {
 		eprintf ("Can't fint type %s", v);
-		return 1;
+		return true;
 	}
 	r_cons_printf ("{\"%s\":", v);
 	r_core_cmdf (core, "pfj %s @ 0x%s\n", fmt, k + strlen ("link."));
 	r_cons_printf ("}");
-	return 1;
+	return true;
 }
 
-static int stdiftype(void *p, const char *k, const char *v) {
+static bool stdiftype(void *p, const char *k, const char *v) {
 	return !strncmp (v, "type", strlen ("type") + 1);
 }
 
-static int print_typelist_r_cb(void *p, const char *k, const char *v) {
-	r_cons_printf ("tk %s=%s\n", k, v);
-	return 1;
+static bool print_typelist_r_cb(void *p, const char *k, const char *v) {
+	r_cons_printf ("\"tk %s=%s\"\n", k, v);
+	return true;
 }
 
-static int print_typelist_json_cb(void *p, const char *k, const char *v) {
+static bool print_typelist_json_cb(void *p, const char *k, const char *v) {
 	RCore *core = (RCore *)p;
 	PJ *pj = pj_new ();
 	pj_o (pj);
@@ -715,7 +743,7 @@ static int print_typelist_json_cb(void *p, const char *k, const char *v) {
 	free (format_s);
 	free (sizecmd);
 	free (formatcmd);
-	return 1;
+	return true;
 }
 
 static void print_keys(Sdb *TDB, RCore *core, SdbForeachCallback filter, SdbForeachCallback printfn_cb, bool json) {
@@ -769,11 +797,11 @@ static void set_offset_hint(RCore *core, RAnalOp *op, const char *type, ut64 lad
 			r_anal_hint_set_offset (core->anal, at, res);
 		}
 	} else if (cmt && r_anal_op_ismemref (op->type)) {
-			r_meta_set_string (core->anal, R_META_TYPE_VARTYPE, at, cmt);
+		r_meta_set_string (core->anal, R_META_TYPE_VARTYPE, at, cmt);
 	}
 }
 
-R_API int r_core_get_stacksz (RCore *core, ut64 from, ut64 to) {
+R_API int r_core_get_stacksz(RCore *core, ut64 from, ut64 to) {
 	int stack = 0, maxstack = 0;
 	ut64 at = from;
 
@@ -800,7 +828,7 @@ R_API int r_core_get_stacksz (RCore *core, ut64 from, ut64 to) {
 	return maxstack;
 }
 
-static void set_retval (RCore *core, ut64 at) {
+static void set_retval(RCore *core, ut64 at) {
 	RAnal *anal = core->anal;
 	RAnalHint *hint = r_anal_hint_get (anal, at);
 	RAnalFunction *fcn = r_anal_get_fcn_in (anal, at, 0);
@@ -828,7 +856,7 @@ R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
 	RAnalBlock *bb;
 	RListIter *it;
 	RAnalOp aop = {0};
-	bool ioCache = r_config_get_i (core->config, "io.cache");
+	bool ioCache = r_config_get_b (core->config, "io.cache");
 	bool stack_set = false;
 	bool resolved = false;
 	const char *varpfx;
@@ -851,6 +879,7 @@ R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
 	r_anal_esil_setup (esil, core->anal, 0, 0, 0);
 	int i, ret, bsize = R_MAX (64, core->blocksize);
 	const int mininstrsz = r_anal_archinfo (core->anal, R_ANAL_ARCHINFO_MIN_OP_SIZE);
+	const int maxinstrsz = r_anal_archinfo (core->anal, R_ANAL_ARCHINFO_MAX_OP_SIZE);
 	const int minopcode = R_MAX (1, mininstrsz);
 	ut8 *buf = malloc (bsize);
 	if (!buf) {
@@ -875,11 +904,12 @@ R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
 		r_core_cmd0 (core, "aeim");
 		stack_set = true;
 	}
-	r_config_set_i (core->config, "io.cache", 1);
+	r_config_set_b (core->config, "io.cache", true);
 	r_config_set_i (core->config, "dbg.follow", 0);
 	ut64 oldoff = core->offset;
 	r_cons_break_push (NULL, NULL);
-	r_list_sort (fcn->bbs, bb_cmpaddr); // TODO: The algorithm can be more accurate if blocks are followed by their jmp/fail, not just by address
+	// TODO: The algorithm can be more accurate if blocks are followed by their jmp/fail, not just by address
+	r_list_sort (fcn->bbs, bb_cmpaddr);
 	r_list_foreach (fcn->bbs, it, bb) {
 		ut64 at = bb->addr;
 		ut64 to = bb->addr + bb->size;
@@ -891,7 +921,7 @@ R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
 			if (at < bb->addr) {
 				break;
 			}
-			if (i >= (bsize - 32)) {
+			if (i >= (bsize - maxinstrsz)) {
 				i = 0;
 			}
 			if (!i) {
@@ -923,7 +953,12 @@ R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
 				dst_addr = r_reg_getv (esil->anal->reg, aop.dst->reg->name) + index;
 				dst_imm = aop.dst->delta;
 			}
-			RAnalVar *var = aop.var;
+			RAnalVar *var = r_anal_get_used_function_var (core->anal, aop.addr);
+			if (false) { // src_addr != UT64_MAX || dst_addr != UT64_MAX) {
+			//  if (src_addr == UT64_MAX && dst_addr == UT64_MAX) {
+				r_anal_op_fini (&aop);
+				continue;
+			}
 			char *slink = r_type_link_at (TDB, src_addr);
 			char *vlink = r_type_link_at (TDB, src_addr + src_imm);
 			char *dlink = r_type_link_at (TDB, dst_addr);
@@ -938,10 +973,8 @@ R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
 				// var int local_e0h --> var struct foo
 				if (strcmp (var->name , vlink) && !resolved) {
 					resolved = true;
-					r_anal_var_retype (core->anal, fcn->addr, R_ANAL_VAR_SCOPE_LOCAL,
-							-1, var->kind, varpfx, -1, var->isarg, var->name);
-					r_anal_var_rename (core->anal, fcn->addr, R_ANAL_VAR_SCOPE_LOCAL,
-							var->kind, var->name, vlink, false);
+					r_anal_var_set_type (var, varpfx);
+					r_anal_var_rename (var, vlink, false);
 				}
 			} else if (slink) {
 				set_offset_hint (core, &aop, slink, src_addr, at - ret, src_imm);
@@ -962,12 +995,12 @@ R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
 	}
 beach:
 	r_core_cmd0 (core, "wc-*"); // drop cache writes
-	r_config_set_i (core->config, "io.cache", ioCache);
+	r_config_set_b (core->config, "io.cache", ioCache);
 	r_config_set_i (core->config, "dbg.follow", dbg_follow);
 	if (stack_set) {
 		r_core_cmd0 (core, "aeim-");
 	}
-	r_core_seek (core, oldoff, 1);
+	r_core_seek (core, oldoff, true);
 	r_anal_esil_free (esil);
 	r_reg_arena_pop (core->anal->reg);
 	r_core_cmd0 (core, ".ar*");
@@ -993,7 +1026,7 @@ static int cmd_type(void *data, const char *input) {
 			break;
 		case '*':
 			if (input[2] == ' ') {
-				showFormat (core, r_str_trim_ro (input + 2), 1);
+				showFormat (core, r_str_trim_head_ro (input + 2), 1);
 			} else {
 				SdbList *l = sdb_foreach_list_filter (TDB, stdifunion, true);
 				SdbListIter *it;
@@ -1006,26 +1039,27 @@ static int cmd_type(void *data, const char *input) {
 			break;
 		case 'j': // "tuj"
 			if (input[2]) {
-				showFormat (core, r_str_trim_ro (input + 2), 'j');
+				showFormat (core, r_str_trim_head_ro (input + 2), 'j');
 				r_cons_newline ();
 			} else {
 				print_struct_union_list_json (TDB, stdifunion);
 			}
 			break;
 		case 'c':
-			print_struct_union_in_c_format (TDB, stdifunion, r_str_trim_ro (input + 2), true);
+			print_struct_union_in_c_format (TDB, stdifunion, r_str_trim_head_ro (input + 2), true);
 			break;
 		case 'd':
-			print_struct_union_in_c_format (TDB, stdifunion, r_str_trim_ro (input + 2), false);
-			break;	
+			print_struct_union_in_c_format (TDB, stdifunion, r_str_trim_head_ro (input + 2), false);
+			break;
 		case ' ':
-			showFormat (core, r_str_trim_ro (input + 1), 0);
+			showFormat (core, r_str_trim_head_ro (input + 1), 0);
 			break;
 		case 0:
 			print_keys (TDB, core, stdifunion, printkey_cb, false);
 			break;
 		}
-	} break;
+		break;
+	}
 	case 'k': // "tk"
 		res = (input[1] == ' ')
 			? sdb_querys (TDB, NULL, -1, input + 2)
@@ -1038,15 +1072,16 @@ static int cmd_type(void *data, const char *input) {
 	case 'c': // "tc"
 		switch (input[1]) {
 		case 'c': // "tcc" -- calling conventions
-			__core_cmd_tcc (core, input + 2);
+			cmd_tcc (core, input + 2);
 			break;
 		case '?': //"tc?"
 			r_core_cmd_help (core, help_msg_tc);
 			break;
-		case ' ': {
-			const char *type = r_str_trim_ro (input + 1);
-			const char *name = type ? strchr (type, '.') + 1: NULL;
+		case ' ': { // "tcc "
+			const char *type = r_str_trim_head_ro (input + 1);
+			const char *name = type ? strchr (type, '.') : NULL;
 			if (name && type) {
+				name++; // skip the '.'
 				if (r_str_startswith (type, "struct")) {
 					r_core_cmdf (core, "tsc %s", name);
 				} else if (r_str_startswith (type, "union")) {
@@ -1057,14 +1092,19 @@ static int cmd_type(void *data, const char *input) {
 					r_core_cmdf (core, "ttc %s", name);
 				} else if (r_str_startswith (type, "func")) {
 					r_core_cmdf (core, "tfc %s", name);
+				} else {
+					eprintf ("unk\n");
 				}
 			}
 			break;
 		}
-		case 0:
+		case '*': // "tc*"
+			r_core_cmd0 (core, "ts*");
+			break;
+		case 0: // "tc"
 			r_core_cmd0 (core, "tfc;tuc;tsc;ttc;tec");
 			break;
-		case 'd':
+		case 'd': // "tcd"
 			r_core_cmd0 (core, "tud;tsd;ttc;ted");
 			break;
 		default:
@@ -1079,12 +1119,11 @@ static int cmd_type(void *data, const char *input) {
 			break;
 		case '*':
 			if (input[2] == ' ') {
-				showFormat (core, r_str_trim_ro (input + 2), 1);
+				showFormat (core, r_str_trim_head_ro (input + 2), 1);
 			} else {
 				SdbList *l = sdb_foreach_list_filter (TDB, stdifstruct, true);
 				SdbListIter *it;
 				SdbKv *kv;
-
 				ls_foreach (l, it, kv) {
 					showFormat (core, sdbkv_key (kv), 1);
 				}
@@ -1092,11 +1131,11 @@ static int cmd_type(void *data, const char *input) {
 			}
 			break;
 		case ' ':
-			showFormat (core, r_str_trim_ro (input + 1), 0);
+			showFormat (core, r_str_trim_head_ro (input + 1), 0);
 			break;
 		case 's':
 			if (input[2] == ' ') {
-				r_cons_printf ("%d\n", (r_type_get_bitsize (TDB, input + 3) / 8));
+				r_cons_printf ("%" PFMT64u "\n", (r_type_get_bitsize (TDB, input + 3) / 8));
 			} else {
 				r_core_cmd_help (core, help_msg_ts);
 			}
@@ -1104,23 +1143,24 @@ static int cmd_type(void *data, const char *input) {
 		case 0:
 			print_keys (TDB, core, stdifstruct, printkey_cb, false);
 			break;
-		case 'c':
-			print_struct_union_in_c_format (TDB, stdifstruct, r_str_trim_ro (input + 2), true);
+		case 'c': // "tsc"
+			print_struct_union_in_c_format (TDB, stdifstruct, r_str_trim_head_ro (input + 2), true);
 			break;
-		case 'd':
-			print_struct_union_in_c_format (TDB, stdifstruct, r_str_trim_ro (input + 2), false);
+		case 'd': // "tsd"
+			print_struct_union_in_c_format (TDB, stdifstruct, r_str_trim_head_ro (input + 2), false);
 			break;
 		case 'j': // "tsj"
 			// TODO: current output is a bit poor, will be good to improve
 			if (input[2]) {
-				showFormat (core, r_str_trim_ro (input + 2), 'j');
+				showFormat (core, r_str_trim_head_ro (input + 2), 'j');
 				r_cons_newline ();
 			} else {
 				print_struct_union_list_json (TDB, stdifstruct);
 			}
 			break;
-		}
-	} break;
+		} // end of switch (input[1])
+		break;
+	}
 	case 'e': { // "te"
 		char *res = NULL, *temp = strchr (input, ' ');
 		Sdb *TDB = core->anal->sdb_types;
@@ -1204,10 +1244,10 @@ static int cmd_type(void *data, const char *input) {
 			res = r_type_enum_member (TDB, name, member_name, 0);
 			break;
 		case 'c': // "tec"
-			print_enum_in_c_format(TDB, r_str_trim_ro (input + 2), true);
+			print_enum_in_c_format(TDB, r_str_trim_head_ro (input + 2), true);
 			break;
 		case 'd':
-			print_enum_in_c_format(TDB, r_str_trim_ro (input + 2), false);
+			print_enum_in_c_format(TDB, r_str_trim_head_ro (input + 2), false);
 			break;
 		case ' ':
 			if (member_name) {
@@ -1238,15 +1278,17 @@ static int cmd_type(void *data, const char *input) {
 			}
 			free (name);
 			ls_free (l);
-		} break;
+			break;
 		}
+		} // end of switch (input[1])
 		free (name);
 		if (res) {
 			r_cons_println (res);
 		} else if (member_name) {
 			eprintf ("Invalid enum member\n");
 		}
-	} break;
+		break;
+	}
 	case ' ':
 		showFormat (core, input + 1, 0);
 		break;
@@ -1262,7 +1304,7 @@ static int cmd_type(void *data, const char *input) {
 		} else if (!r_sandbox_enable (0)) {
 			if (input[1] == ' ') {
 				const char *dir = r_config_get (core->config, "dir.types");
-				const char *filename = input + 2;
+				const char *filename = r_str_trim_head_ro (input + 2);
 				char *homefile = NULL;
 				if (*filename == '~') {
 					if (filename[1] && filename[2]) {
@@ -1276,7 +1318,7 @@ static int cmd_type(void *data, const char *input) {
 						char *error_msg = NULL;
 						char *out = r_parse_c_string (core->anal, tmp, &error_msg);
 						if (out) {
-							//		r_cons_strcat (out);
+							// r_cons_strcat (out);
 							r_anal_save_parsed_type (core->anal, out);
 							free (out);
 						}
@@ -1300,6 +1342,14 @@ static int cmd_type(void *data, const char *input) {
 					}
 				}
 				free (homefile);
+			} else if (input[1] == 'u') {
+				// "tou" "touch"
+				char *arg = strchr (input, ' ');
+				if (arg) {
+					r_file_touch (arg + 1);
+				} else {
+					eprintf ("Usage: touch [filename]");
+				}
 			} else if (input[1] == 's') {
 				const char *dbpath = input + 3;
 				if (r_file_exists (dbpath)) {
@@ -1355,7 +1405,7 @@ static int cmd_type(void *data, const char *input) {
 				free (out);
 			}
 			if (error_msg) {
-				fprintf (stderr, "%s", error_msg);
+				eprintf ("%s", error_msg);
 				free (error_msg);
 			}
 		} else {
@@ -1365,11 +1415,16 @@ static int cmd_type(void *data, const char *input) {
 	case 'x': {
 		  char *type, *type2;
 		RListIter *iter, *iter2;
-		  RAnalFunction *fcn;
+		RAnalFunction *fcn;
 		switch (input[1]) {
-		case '.': // "tx." type xrefs 
-		case 'f': // "txf" type xrefs 
-			fcn  = r_anal_get_function_at (core->anal, core->offset);
+		case '.': // "tx." type xrefs
+		case 'f': // "txf" type xrefs
+			{
+			ut64 addr = core->offset;
+			if (input[2] == ' ') {
+				addr = r_num_math (core->num, input + 2);
+			}
+			fcn = r_anal_get_function_at (core->anal, addr);
 			if (fcn) {
 				RList *uniq = r_anal_types_from_fcn (core->anal, fcn);
 				r_list_foreach (uniq , iter , type) {
@@ -1377,7 +1432,8 @@ static int cmd_type(void *data, const char *input) {
 				}
 				r_list_free (uniq);
 			} else {
-				eprintf ("cannot find function at 0x%08"PFMT64x"\n", core->offset);
+				eprintf ("cannot find function at 0x%08"PFMT64x"\n", addr);
+			}
 			}
 			break;
 		case 0: // "tx"
@@ -1426,8 +1482,9 @@ static int cmd_type(void *data, const char *input) {
 				r_list_free (uniqList);
 			}
 			break;
+		case 't':
 		case ' ': // "tx " -- show which function use given type
-			type = (char *)r_str_trim_ro (input + 2);
+			type = (char *)r_str_trim_head_ro (input + 2);
 			r_list_foreach (core->anal->fcns, iter, fcn) {
 				RList *uniq = r_anal_types_from_fcn (core->anal, fcn);
 				r_list_foreach (uniq , iter2, type2) {
@@ -1441,13 +1498,16 @@ static int cmd_type(void *data, const char *input) {
 		default:
 			eprintf ("Usage: tx[flg] [...]\n");
 			eprintf (" txf | tx.      list all types used in this function\n");
+			eprintf (" txf 0xaddr     list all types used in function at 0xaddr\n");
 			eprintf (" txl            list all types used by any function\n");
-			eprintf (" txg            render the type xrefs graph\n");
-			eprintf (" tx  int32_t    list functions names using this type\n");
+			eprintf (" txg            render the type xrefs graph (usage .txg;aggv)\n");
+			eprintf (" tx int32_t     list functions names using this type\n");
+			eprintf (" txt int32_t    same as 'tx type'\n");
 			eprintf (" tx             list functions and the types they use\n");
 			break;
 		}
-	} break;
+		break;
+	}
 	// ta: moved to anal hints (aht)- just for tail, at the moment
 	case 'a': // "ta"
 		switch (input[1]) {
@@ -1457,7 +1517,8 @@ static int cmd_type(void *data, const char *input) {
 			} else {
 				eprintf ("Usage: tail [number] [file]\n");
 			}
-		} break;
+			break;
+		}
 		default:
 			eprintf ("[WARNING] \"ta\" is deprecated. Use \"aht\" instead.\n");
 		}
@@ -1471,7 +1532,7 @@ static int cmd_type(void *data, const char *input) {
 		case ' ': {
 			char *type = strdup (input + 2);
 			char *ptr = strchr (type, '=');
-			ut64 addr;
+			ut64 addr = core->offset;
 
 			if (ptr) {
 				*ptr++ = 0;
@@ -1479,21 +1540,25 @@ static int cmd_type(void *data, const char *input) {
 				if (ptr && *ptr) {
 					addr = r_num_math (core->num, ptr);
 				} else {
-					eprintf ("address is unvalid\n");
+					eprintf ("tl: Address is unvalid\n");
 					free (type);
 					break;
 				}
-			} else {
-				addr = core->offset;
 			}
 			r_str_trim (type);
 			char *tmp = sdb_get (TDB, type, 0);
 			if (tmp && *tmp) {
 				r_type_set_link (TDB, type, addr);
-				RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->offset, 0);
-				if (fcn) {
+				RList *fcns = r_anal_get_functions_in (core->anal, core->offset);
+				if (r_list_length (fcns) > 1) {
+					eprintf ("Multiple functions found in here.\n");
+				} else if (r_list_length (fcns) == 1) {
+					RAnalFunction *fcn = r_list_first (fcns);
 					r_core_link_stroff (core, fcn);
+				} else {
+					eprintf ("Cannot find any function here\n");
 				}
+				r_list_free (fcns);
 				free (tmp);
 			} else {
 				eprintf ("unknown type %s\n", type);
@@ -1548,42 +1613,68 @@ static int cmd_type(void *data, const char *input) {
 		break;
 	case 'p':  // "tp"
 		if (input[1] == '?') { // "tp?"
-			r_core_cmd0 (core, "t?~tp\n");
-		} else { // "tp"
-			char *tmp = strdup (input);
-			char *ptr = strchr (tmp, ' ');
-			if (!ptr) {
-				break;
+			r_core_cmd_help (core, help_msg_tp);
+		} else if (input[1] == 'v') { // "tpv"
+			const char *type_name = r_str_trim_head_ro (input + 2);
+			char *fmt = r_type_format (TDB, type_name);
+			if (fmt && *fmt) {
+				ut64 val = core->offset;
+				r_core_cmdf (core, "pf %s @v:0x%08" PFMT64x "\n", fmt, val);
+			} else {
+				eprintf ("Usage: tpv [type] @ [value]\n");
 			}
-			r_str_trim (ptr);
-			int nargs = r_str_word_set0 (ptr);
-			if (nargs > 0) {
-				const char *type = r_str_word_get0 (ptr, 0);
-				const char *arg = (nargs > 1)? r_str_word_get0 (ptr, 1): NULL;
+		} else if (input[1] == ' ' || input[1] == 'x' || !input[1]) {
+			char *tmp = strdup (input);
+			char *type_begin = strchr (tmp, ' ');
+			if (type_begin) {
+				r_str_trim (type_begin);
+				const char *type_end = r_str_rchr (type_begin, NULL, ' ');
+				int type_len = (type_end)
+					? (int)(type_end - type_begin)
+					: strlen (type_begin);
+				char *type = strdup (type_begin);
+				if (!type) {
+					free (tmp);
+					break;
+				}
+				snprintf (type, type_len + 1, "%s", type_begin);
+				const char *arg = (type_end) ? type_end + 1 : NULL;
 				char *fmt = r_type_format (TDB, type);
 				if (!fmt) {
 					eprintf ("Cannot find '%s' type\n", type);
+					free (tmp);
+					free (type);
 					break;
 				}
 				if (input[1] == 'x' && arg) { // "tpx"
 					r_core_cmdf (core, "pf %s @x:%s", fmt, arg);
-					eprintf ("pf %s @x:%s", fmt, arg);
+					// eprintf ("pf %s @x:%s", fmt, arg);
 				} else {
 					ut64 addr = arg ? r_num_math (core->num, arg): core->offset;
+					ut64 original_addr = addr;
 					if (!addr && arg) {
 						RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->offset, -1);
-						addr = r_anal_var_addr (core->anal, fcn, arg);
+						if (fcn) {
+							RAnalVar *var = r_anal_function_get_var_byname (fcn, arg);
+							if (var) {
+								addr = r_anal_var_addr (var);
+							}
+						}
 					}
 					if (addr != UT64_MAX) {
-						r_core_cmdf (core, "pf %s @ 0x%08" PFMT64x "\n", fmt, addr);
+						r_core_cmdf (core, "pf %s @ 0x%08" PFMT64x, fmt, addr);
+					} else if (original_addr == 0) {
+						r_core_cmdf (core, "pf %s @ 0x%08" PFMT64x, fmt, original_addr);
 					}
 				}
 				free (fmt);
+				free (type);
 			} else {
-				eprintf ("see t?\n");
-				break;
+				eprintf ("Usage: tp?\n");
 			}
 			free (tmp);
+		} else { // "tp"
+			eprintf ("Usage: tp?\n");
 		}
 		break;
 	case '-': // "t-"
@@ -1593,8 +1684,7 @@ static int cmd_type(void *data, const char *input) {
 			sdb_reset (TDB);
 			r_parse_c_reset (core->parser);
 		} else {
-			const char *name = input + 1;
-			while (IS_WHITESPACE (*name)) name++;
+			const char *name = r_str_trim_head_ro (input + 1);
 			if (*name) {
 				r_anal_remove_parsed_type (core->anal, name);
 			} else {
@@ -1605,16 +1695,18 @@ static int cmd_type(void *data, const char *input) {
 	// tv - get/set type value linked to a given address
 	case 'f': // "tf"
 		switch (input[1]) {
-		case 0:
+		case 0: // "tf"
 			print_keys (TDB, core, stdiffunc, printkey_cb, false);
 			break;
 		case 'c': // "tfc"
-			printFunctionTypeC (core, input + 2);
+			if (input[2] == ' ') {
+				printFunctionTypeC (core, input + 3);
+			}
 			break;
-		case 'j':
+		case 'j': // "tfj"
 			if (input[2] == ' ') {
 				printFunctionType (core, input + 2);
-				r_cons_printf ("\n");
+				r_cons_newline ();
 			} else {
 				print_keys (TDB, core, stdiffunc, printfunc_json_cb, true);
 			}
@@ -1674,7 +1766,7 @@ static int cmd_type(void *data, const char *input) {
 			SdbKv *kv;
 			SdbListIter *iter;
 			SdbList *l = sdb_foreach_list (TDB, true);
-			const char *arg = r_str_trim_ro (input + 2);
+			const char *arg = r_str_trim_head_ro (input + 2);
 			bool match = false;
 			ls_foreach (l, iter, kv) {
 				if (!strcmp (sdbkv_value (kv), "typedef")) {
@@ -1713,13 +1805,15 @@ static int cmd_type(void *data, const char *input) {
 		if (istypedef && !strncmp (istypedef, "typedef", 7)) {
 			const char *q = sdb_fmt ("typedef.%s", s);
 			const char *res = sdb_const_get (TDB, q, 0);
-			if (res)
+			if (res) {
 				r_cons_println (res);
+			}
 		} else {
-			eprintf ("This is not an typedef\n");
+			eprintf ("This is not a typedef\n");
 		}
 		free (s);
-	} break;
+		break;
+	}
 	case '?':
 		show_help (core);
 		break;
