@@ -1,3 +1,5 @@
+/* radare - LGPL - Copyright 2017-2021 - javierptd, pancake */
+
 #ifndef INCLUDE_HEAP_JEMALLOC_STD_C
 #define INCLUDE_HEAP_JEMALLOC_STD_C
 #define HEAP32 1
@@ -10,6 +12,7 @@
 #undef GHT_MAX
 #undef PFMTx
 
+// FIXME: It should be detected at runtime, not during the compilation stage
 #if HEAP32
 #define GH(x) x##_32
 #define GHT ut32
@@ -25,33 +28,62 @@
 #if __linux__
 // TODO: provide proper api in cbin to resolve symbols and load libraries from debug maps and such
 // this is, provide a programmatic api for the slow dmi command
+#if 0
 static GHT GH(je_get_va_symbol)(const char *path, const char *symname) {
 	RListIter *iter;
 	RBinSymbol *s;
 	RCore *core = r_core_new ();
-	RList * syms = NULL;
 	GHT vaddr = 0LL;
 
 	if (!core) {
 		return GHT_MAX;
 	}
 
-	RBinOptions opt;
-	r_bin_options_init (&opt, -1, 0, 0, false);
-	r_bin_open (core->bin, path, &opt);
-	syms = r_bin_get_symbols (core->bin);
-	if (!syms) {
-		return GHT_MAX;
-	}
-	r_list_foreach (syms, iter, s) {
-		if (!strcmp(s->name, symname)) {
-			vaddr = s->vaddr;
-			break;
+	RBinFileOptions opt;
+	r_bin_file_options_init (&opt, -1, 0, 0, false);
+	if (r_bin_open (core->bin, path, &opt)) {
+		RList *syms = r_bin_get_symbols (core->bin);
+		if (!syms) {
+			r_core_free (core);
+			return GHT_MAX;
+		}
+		r_list_foreach (syms, iter, s) {
+			if (!strcmp(s->name, symname)) {
+				vaddr = s->vaddr;
+				break;
+			}
 		}
 	}
 	r_core_free (core);
 	return vaddr;
 }
+#else
+static GHT GH(je_get_va_symbol)(RCore *core, const char *path, const char *sym_name) {
+	GHT vaddr = GHT_MAX;
+	RBinFileOptions opt = { 0 };
+	r_bin_file_options_init (&opt, -1, 0, 0, false);
+	RBinSymbol *s;
+	RListIter *iter;
+	RBinFile *current_bf = r_bin_cur (core->bin);
+	if (!r_bin_open (core->bin, path, &opt)) {
+		return vaddr;
+	}
+	RBinFile *libc_bf = r_bin_cur (core->bin);
+ 	if (!libc_bf) {
+ 		return vaddr;
+ 	}
+ 	RList *syms = r_bin_get_symbols (core->bin);
+ 	r_list_foreach (syms, iter, s) {
+ 		if (!strcmp (s->name, sym_name)) {
+ 			vaddr = s->vaddr;
+ 			break;
+		}
+	}
+	r_bin_file_delete (core->bin, libc_bf->id);
+ 	r_bin_file_set_cur_binfile (core->bin, current_bf);
+	return vaddr;
+}
+#endif
 
 static int GH(je_matched)(const char *ptr, const char *str) {
         int ret = strncmp (ptr, str, strlen (str) - 1);
@@ -62,7 +94,7 @@ static int GH(je_matched)(const char *ptr, const char *str) {
 static bool GH(r_resolve_jemalloc)(RCore *core, char *symname, ut64 *symbol) {
 	RListIter *iter;
 	RDebugMap *map;
-	const char *jemalloc_ver_end = NULL;
+	const char *path = NULL;
 	ut64 jemalloc_addr = UT64_MAX;
 
 	if (!core || !core->dbg || !core->dbg->maps){
@@ -72,36 +104,33 @@ static bool GH(r_resolve_jemalloc)(RCore *core, char *symname, ut64 *symbol) {
 	r_list_foreach (core->dbg->maps, iter, map) {
 		if (strstr (map->name, "libjemalloc.")) {
 			jemalloc_addr = map->addr;
-			jemalloc_ver_end = map->name;
+			path = map->name;
 			break;
 		}
 	}
-	if (!jemalloc_ver_end) {
-		eprintf ("Warning: Is jemalloc mapped in memory? (see dm command)\n");
+	if (!path) {
+		eprintf ("Warning: cannot find jemalloc mapped in memory (see `dm`)\n");
 		return false;
 	}
 #if __linux__
-	bool is_debug_file = GH(je_matched)(jemalloc_ver_end, "/usr/local/lib");
+	bool is_debug_file = GH(je_matched)(path, "/usr/local/lib");
 
 	if (!is_debug_file) {
-		eprintf ("Warning: Is libjemalloc.so.2 in /usr/local/lib path?\n");
+		eprintf ("Warning: Cannot find libjemalloc.so.2 in /usr/local/lib\n");
 		return false;
 	}
-	char *path = r_str_newf ("%s", jemalloc_ver_end);
 	if (r_file_exists (path)) {
-		ut64 vaddr = GH(je_get_va_symbol)(path, symname);
+		ut64 vaddr = GH(je_get_va_symbol)(core, path, symname);
 		if (jemalloc_addr != GHT_MAX && vaddr != 0) {
 			*symbol = jemalloc_addr + vaddr;
-			free (path);
 			return true;
 		}
 	}
-	free (path);
 	return false;
 #else
 	eprintf ("[*] Resolving %s from libjemalloc.2... ", symname);
 	// this is quite sloooow, we must optimize dmi
-	char *va = r_core_cmd_strf (core, "dmi libjemalloc.2 %s$~[1]", symname, symname);
+	char *va = r_core_cmd_strf (core, "dmi libjemalloc.2 %s$~[1]", symname);
 	ut64 n = r_num_get (NULL, va);
 	if (n && n != UT64_MAX) {
 		*symbol = n;
@@ -123,7 +152,7 @@ static void GH(jemalloc_get_chunks)(RCore *core, const char *input) {
 		return;
 	}
 	r_io_read_at (core->io, cnksz, (ut8 *)&cnksz, sizeof (GHT));
- 
+
 	switch (input[0]) {
 	case '\0':
 		eprintf ("need an arena_t to associate chunks");
@@ -133,7 +162,7 @@ static void GH(jemalloc_get_chunks)(RCore *core, const char *input) {
 			GHT arena = GHT_MAX;
 			arena_t *ar = R_NEW0 (arena_t);
 			extent_node_t *node = R_NEW0 (extent_node_t), *head = R_NEW0 (extent_node_t);
-			input += 1;
+			input++;
 			arena = r_num_math (core->num, input);
 
 			if (arena) {
@@ -145,15 +174,15 @@ static void GH(jemalloc_get_chunks)(RCore *core, const char *input) {
 					PRINT_YA (", end: ");
 					PRINTF_BA ("0x%08"PFMT64x, (ut64)(size_t)((char *)head->en_addr + cnksz));
 					PRINT_YA (", size: ");
-					PRINTF_BA ("0x%08"PFMT64x"\n", (ut64)cnksz); 
+					PRINTF_BA ("0x%08"PFMT64x"\n", (ut64)cnksz);
 					r_io_read_at (core->io, (ut64)(size_t)head->ql_link.qre_next, (ut8 *)node, sizeof (extent_node_t));
 					while (node && node->en_addr != head->en_addr) {
 						PRINT_YA ("   Chunk - start: ");
-						PRINTF_BA ("0x%08"PFMT64x, (ut64)(size_t)node->en_addr); 
+						PRINTF_BA ("0x%08"PFMT64x, (ut64)(size_t)node->en_addr);
 						PRINT_YA (", end: ");
-						PRINTF_BA ("0x%"PFMTx, ((char *)node->en_addr + cnksz));
+						PRINTF_BA ("0x%"PFMT64x, (ut64)(size_t)((char *)node->en_addr + cnksz));
 						PRINT_YA (", size: ");
-						PRINTF_BA ("0x%08"PFMTx"\n", cnksz); 
+						PRINTF_BA ("0x%08"PFMT64x"\n", cnksz);
 						r_io_read_at (core->io, (ut64)(size_t)node->ql_link.qre_next, (ut8 *)node, sizeof (extent_node_t));
 					}
 				}
@@ -171,7 +200,7 @@ static void GH(jemalloc_get_chunks)(RCore *core, const char *input) {
 			arena_t *ar = R_NEW0 (arena_t);
 			extent_node_t *node = R_NEW0 (extent_node_t);
 			extent_node_t *head = R_NEW0 (extent_node_t);
-			
+
 			if (!node || !head) {
 				eprintf ("Error calling calloc\n");
 				free (ar);
@@ -180,8 +209,6 @@ static void GH(jemalloc_get_chunks)(RCore *core, const char *input) {
 				return;
 			}
 
-			input += 1;
-
 			if (GH(r_resolve_jemalloc) (core, "je_arenas", &sym)) {
 				r_io_read_at (core->io, sym, (ut8 *)&arenas, sizeof (GHT));
 				for (;;) {
@@ -196,9 +223,9 @@ static void GH(jemalloc_get_chunks)(RCore *core, const char *input) {
 						PRINT_YA ("   Chunk - start: ");
 						PRINTF_BA ("0x%08"PFMT64x, (ut64)(size_t)head->en_addr);
 						PRINT_YA (", end: ");
-						PRINTF_BA ("0x%"PFMTx, ((char *)head->en_addr + cnksz));
+						PRINTF_BA ("0x%"PFMT64x, (ut64)(size_t)((char *)head->en_addr + cnksz));
 						PRINT_YA (", size: ");
-						PRINTF_BA ("0x%08"PFMT64x"\n", (ut64)cnksz); 
+						PRINTF_BA ("0x%08"PFMT64x"\n", (ut64)cnksz);
 						ut64 addr = (ut64) (size_t)head->ql_link.qre_next;
 						r_io_read_at (core->io, addr, (ut8 *)node, sizeof (extent_node_t));
 						while (node && head && node->en_addr != head->en_addr) {
@@ -243,7 +270,7 @@ static void GH(jemalloc_print_narenas)(RCore *core, const char *input) {
 	case '\0':
 		if (GH(r_resolve_jemalloc)(core, "narenas_total", &symaddr)) {
 			r_io_read_at (core->io, symaddr, (ut8 *)&narenas, sizeof (GHT));
-			PRINTF_GA ("narenas : %d\n", narenas);
+			PRINTF_GA ("narenas : %"PFMT64d"\n", (ut64)narenas);
 		}
 		if (narenas == 0) {
 			eprintf ("No arenas allocated.\n");
@@ -260,7 +287,7 @@ static void GH(jemalloc_print_narenas)(RCore *core, const char *input) {
 
 		if (GH(r_resolve_jemalloc)(core, "je_arenas", &arenas)) {
 			r_io_read_at (core->io, arenas, (ut8 *)&arenas, sizeof (GHT));
-			PRINTF_GA ("arenas[%d] @ 0x%"PFMTx" {\n", narenas, (GHT)arenas);
+			PRINTF_GA ("arenas[%"PFMT64d"] @ 0x%"PFMT64x" {\n", (ut64)narenas, (ut64)arenas);
 			for (i = 0; i < narenas; i++) {
 				ut64 at = arenas + (i * sizeof (GHT));
 				r_io_read_at (core->io, at, (ut8 *)&arena, sizeof (GHT));
@@ -269,7 +296,7 @@ static void GH(jemalloc_print_narenas)(RCore *core, const char *input) {
 					continue;
 				}
 				PRINTF_YA ("  arenas[%d]: ", i);
-				PRINTF_BA ("@ 0x%"PFMTx"\n", at);
+				PRINTF_BA ("@ 0x%"PFMT64x"\n", at);
 			}
 		}
 		PRINT_GA ("}\n");
@@ -279,18 +306,18 @@ static void GH(jemalloc_print_narenas)(RCore *core, const char *input) {
 		r_io_read_at (core->io, (GHT)arena, (ut8 *)ar, sizeof (arena_t));
 
 		PRINT_GA ("struct arena_s {\n");
-#define OO(x) arena + r_offsetof (arena_t, x)
-		PRINTF_BA ("  ind = 0x%"PFMTx"\n", ar->ind);
-		PRINTF_BA ("  nthreads: application allocation = 0x%"PFMTx"\n", (ut32)ar->nthreads[0]);
-		PRINTF_BA ("  nthreads: internal metadata allocation = 0x%"PFMTx"\n", (ut32)ar->nthreads[1]);
+#define OO(x) (ut64)(arena + r_offsetof (arena_t, x))
+		PRINTF_BA ("  ind = 0x%x\n", ar->ind);
+		PRINTF_BA ("  nthreads: application allocation = 0x%"PFMT64x"\n", (ut64)ar->nthreads[0]);
+		PRINTF_BA ("  nthreads: internal metadata allocation = 0x%"PFMT64x"\n", (ut64)ar->nthreads[1]);
 		PRINTF_BA ("  lock = 0x%"PFMT64x"\n", OO(lock));
 		PRINTF_BA ("  stats = 0x%"PFMT64x"\n", OO(stats));
 		PRINTF_BA ("  tcache_ql = 0x%"PFMT64x"\n", OO(tcache_ql));
 		PRINTF_BA ("  prof_accumbytes = 0x%"PFMT64x"x\n", (ut64)ar->prof_accumbytes);
 		PRINTF_BA ("  offset_state = 0x%"PFMT64x"\n", (ut64)ar->offset_state);
-		PRINTF_BA ("  dss_prec_t = 0x%"PFMT64x"\n", OO(dss_prec));
+		PRINTF_BA ("  dss_prec_t = 0x%"PFMT64x"\n",OO(dss_prec));
 		PRINTF_BA ("  achunks = 0x%"PFMT64x"\n", OO(achunks));
-		PRINTF_BA ("  extent_sn_next = 0x%"PFMTx"\n", (ut64)(size_t)ar->extent_sn_next);
+		PRINTF_BA ("  extent_sn_next = 0x%"PFMT64x"\n", (ut64)(size_t)ar->extent_sn_next);
 		PRINTF_BA ("  spare = 0x%"PFMT64x"\n", (ut64)(size_t)ar->spare);
 		PRINTF_BA ("  lg_dirty_mult = 0x%"PFMT64x"\n", (ut64)(ssize_t)ar->lg_dirty_mult);
 		PRINTF_BA ("  purging = %s\n", r_str_bool (ar->purging));
@@ -310,9 +337,9 @@ static void GH(jemalloc_print_narenas)(RCore *core, const char *input) {
 		PRINTF_BA ("  chunks_mtx = 0x%"PFMT64x"\n", OO(chunks_mtx));
 		PRINTF_BA ("  node_cache = 0x%"PFMT64x"\n", OO(node_cache));
 		PRINTF_BA ("  node_cache_mtx = 0x%"PFMT64x"\n", OO(node_cache_mtx));
-		PRINTF_BA ("  chunks_hooks = 0x%"PFMTx"\n", OO(chunk_hooks));
-		PRINTF_BA ("  bins = %d 0x%"PFMTx"\n", JM_NBINS, OO(bins));
-		PRINTF_BA ("  runs_avail = %d 0x%"PFMTx"\n", NPSIZES, OO(runs_avail));
+		PRINTF_BA ("  chunks_hooks = 0x%"PFMT64x"\n", OO(chunk_hooks));
+		PRINTF_BA ("  bins = %d 0x%"PFMT64x"\n", JM_NBINS, OO(bins));
+		PRINTF_BA ("  runs_avail = %d 0x%"PFMT64x"\n", NPSIZES, OO(runs_avail));
 		PRINT_GA ("}\n");
 		break;
 	}
@@ -362,28 +389,32 @@ static void GH(jemalloc_get_bins)(RCore *core, const char *input) {
 						(ut8*)b, sizeof (arena_bin_info_t));
 					PRINT_YA ("    {\n");
 					PRINT_YA ("       regsize : ");
-					PRINTF_BA ("0x%"PFMTx"\n", b->reg_size);
+					PRINTF_BA ("0x%zx\n", b->reg_size);
 					PRINT_YA ("       redzone size ");
-					PRINTF_BA ("0x%"PFMTx"\n", b->redzone_size);
+					PRINTF_BA ("0x%zx\n", b->redzone_size);
 					PRINT_YA ("       reg_interval : ");
-					PRINTF_BA ("0x%"PFMTx"\n", b->reg_interval);
+					PRINTF_BA ("0x%zx\n", b->reg_interval);
 					PRINT_YA ("       run_size : ");
-					PRINTF_BA ("0x%"PFMTx"\n", b->run_size);
+					PRINTF_BA ("0x%zx\n", b->run_size);
 					PRINT_YA ("       nregs : ");
-					PRINTF_BA ("0x%"PFMTx"\n", b->nregs);
-					PRINT_YA ("       bitmap_info : ");
-					PRINTF_BA ("0x%"PFMTx"\n", b->bitmap_info);
+					PRINTF_BA ("0x%x\n", b->nregs);
+					// FIXME: It's a structure of bitmap_info_t
+					//PRINT_YA ("       bitmap_info : ");
+					//PRINTF_BA ("0x%"PFMT64x"\n", b->bitmap_info);
 					PRINT_YA ("       reg0_offset : ");
-					PRINTF_BA ("0x%"PFMTx"\n\n", b->reg0_offset);
-
-					PRINTF_YA ("       bins[%d]->lock ", j);
-					PRINTF_BA ("= 0x%"PFMTx"\n", ar->bins[j].lock);
-					PRINTF_YA ("       bins[%d]->runcur ", j);
-					PRINTF_BA ("@ 0x%"PFMTx"\n", ar->bins[j].runcur);
-					PRINTF_YA ("       bins[%d]->runs ", j);
-					PRINTF_BA ("@ 0x%"PFMTx"\n", ar->bins[j].runs);
-					PRINTF_YA ("       bins[%d]->stats ", j);
-					PRINTF_BA ("= 0x%"PFMTx"\n", ar->bins[j].stats);
+					PRINTF_BA ("0x%"PFMT64x"\n\n", (ut64)b->reg0_offset);
+					// FIXME: It's a structure of malloc_mutex_t
+					//PRINTF_YA ("       bins[%d]->lock ", j);
+					//PRINTF_BA ("= 0x%"PFMT64x"\n", ar->bins[j].lock);
+					// FIXME: It's a structure of arena_run_t*
+					//PRINTF_YA ("       bins[%d]->runcur ", j);
+					//PRINTF_BA ("@ 0x%"PFMT64x"\n", ar->bins[j].runcur);
+					// FIXME: It's a structure of arena_run_heap_t*
+					//PRINTF_YA ("       bins[%d]->runs ", j);
+					//PRINTF_BA ("@ 0x%"PFMTx"\n", ar->bins[j].runs);
+					// FIXME: It's a structure of malloc_bin_stats_t
+					//PRINTF_YA ("       bins[%d]->stats ", j);
+					//PRINTF_BA ("= 0x%"PFMTx"\n", ar->bins[j].stats);
 					PRINT_YA ("    }\n");
 				}
 				PRINT_YA ("  }\n");
@@ -404,7 +435,7 @@ static void GH(jemalloc_get_runs)(RCore *core, const char *input) {
 			int pageind;
 			ut64 npages, chunksize_mask, map_bias, map_misc_offset, chunk, mapbits;;
 			arena_chunk_t *c = R_NEW0 (arena_chunk_t);
-			
+
 			if (!c) {
 				eprintf ("Error calling calloc\n");
 				return;
