@@ -1,7 +1,28 @@
-/* radare - LGPL - Copyright 2014-2018 - pancake, thestr4ng3r */
+/* radare - LGPL - Copyright 2014-2019 - pancake, thestr4ng3r */
 
 #include <r_core.h>
 
+R_API void r_core_task_scheduler_init(RCoreTaskScheduler *tasks, RCore *core) {
+	tasks->task_id_next = 0;
+	tasks->tasks = r_list_newf ((RListFree)r_core_task_decref);
+	tasks->tasks_queue = r_list_new ();
+	tasks->oneshot_queue = r_list_newf (free);
+	tasks->oneshots_enqueued = 0;
+	tasks->lock = r_th_lock_new (true);
+	tasks->tasks_running = 0;
+	tasks->oneshot_running = false;
+	tasks->main_task = r_core_task_new (core, false, NULL, NULL, NULL);
+	r_list_append (tasks->tasks, tasks->main_task);
+	tasks->current_task = NULL;
+}
+
+R_API void r_core_task_scheduler_fini(RCoreTaskScheduler *tasks) {
+	r_list_free (tasks->tasks);
+	r_list_free (tasks->tasks_queue);
+	r_list_free (tasks->oneshot_queue);
+	r_th_lock_free (tasks->lock);
+}
+
 #if HAVE_PTHREAD
 #define TASK_SIGSET_T sigset_t
 static void tasks_lock_block_signals(sigset_t *old_sigset) {
@@ -20,13 +41,13 @@ static void tasks_lock_block_signals(TASK_SIGSET_T *old_sigset) { (void)old_sigs
 static void tasks_lock_block_signals_reset(TASK_SIGSET_T *old_sigset) { (void)old_sigset; }
 #endif
 
-static void tasks_lock_enter(RCore *core, TASK_SIGSET_T *old_sigset) {
+static void tasks_lock_enter(RCoreTaskScheduler *scheduler, TASK_SIGSET_T *old_sigset) {
 	tasks_lock_block_signals (old_sigset);
-	r_th_lock_enter (core->tasks_lock);
+	r_th_lock_enter (scheduler->lock);
 }
 
-static void tasks_lock_leave(RCore *core, TASK_SIGSET_T *old_sigset) {
-	r_th_lock_leave (core->tasks_lock);
+static void tasks_lock_leave(RCoreTaskScheduler *scheduler, TASK_SIGSET_T *old_sigset) {
+	r_th_lock_leave (scheduler->lock);
 	tasks_lock_block_signals_reset (old_sigset);
 }
 
@@ -35,80 +56,84 @@ typedef struct oneshot_t {
 	void *user;
 } OneShot;
 
-R_API void r_core_task_print (RCore *core, RCoreTask *task, int mode) {
+R_API void r_core_task_print (RCore *core, RCoreTask *task, PJ *pj, int mode) {
 	switch (mode) {
-	case 'j':
-		r_cons_printf ("{\"id\":%d,\"state\":\"", task->id);
+	case 'j': {
+		pj_o (pj);
+		pj_ki (pj, "id", task->id);
+		pj_k (pj, "state");
 		switch (task->state) {
 		case R_CORE_TASK_STATE_BEFORE_START:
-			r_cons_print ("before_start");
+			pj_s (pj, "before_start");
 			break;
 		case R_CORE_TASK_STATE_RUNNING:
-			r_cons_print ("running");
+			pj_s (pj, "running");
 			break;
 		case R_CORE_TASK_STATE_SLEEPING:
-			r_cons_print ("sleeping");
+			pj_s (pj, "sleeping");
 			break;
 		case R_CORE_TASK_STATE_DONE:
-			r_cons_print ("done");
+			pj_s (pj, "done");
 			break;
 		}
-		r_cons_printf ("\",\"transient\":%s,\"cmd\":", task->transient ? "true" : "false");
-		if (task->cmd) {
-			r_cons_printf ("\"%s\"}", task->cmd);
-		} else {
-			r_cons_printf ("null}");
-		}
+		pj_kb (pj, "transient", task->transient);
+		pj_ks (pj, "cmd", r_str_get_fail (task->cmd, "null"));
+		pj_end (pj);
 		break;
+	}
 	default: {
 		const char *info = task->cmd;
-		if (task == core->main_task) {
+		if (task == core->tasks.main_task) {
 			info = "-- MAIN TASK --";
 		}
 		r_cons_printf ("%3d %3s %12s  %s\n",
 					   task->id,
 					   task->transient ? "(t)" : "",
 					   r_core_task_status (task),
-					   info ? info : "");
+					   r_str_get (info));
+		}
 		break;
 	}
-	}
 }
 
 R_API void r_core_task_list(RCore *core, int mode) {
 	RListIter *iter;
 	RCoreTask *task;
+	PJ *pj = NULL;
 	if (mode == 'j') {
-		r_cons_printf ("[");
+		pj = r_core_pj_new (core);
+		if (!pj) {
+			return;
+		}
+		pj_a (pj);
 	}
 	TASK_SIGSET_T old_sigset;
-	tasks_lock_enter (core, &old_sigset);
-	r_list_foreach (core->tasks, iter, task) {
-		r_core_task_print (core, task, mode);
-		if (mode == 'j' && iter->n) {
-			r_cons_printf (",");
-		}
+	tasks_lock_enter (&core->tasks, &old_sigset);
+	r_list_foreach (core->tasks.tasks, iter, task) {
+		r_core_task_print (core, task, pj, mode);
 	}
 	if (mode == 'j') {
-		r_cons_printf ("]\n");
+		pj_end (pj);
+		r_cons_println (pj_string (pj));
+		pj_free (pj);
 	} else {
-		r_cons_printf ("--\ntotal running: %d\n", core->tasks_running);
+		r_cons_printf ("--\ntotal running: %d\n", core->tasks.tasks_running);
 	}
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (&core->tasks, &old_sigset);
 }
 
-R_API int r_core_task_running_tasks_count(RCore *core) {
+R_API int r_core_task_running_tasks_count(RCoreTaskScheduler *scheduler) {
 	RListIter *iter;
 	RCoreTask *task;
 	int count = 0;
 	TASK_SIGSET_T old_sigset;
-	tasks_lock_enter (core, &old_sigset);
-	r_list_foreach (core->tasks, iter, task) {
-		if (task != core->main_task && task->state != R_CORE_TASK_STATE_DONE) {
+	tasks_lock_enter (scheduler, &old_sigset);
+	r_list_foreach (scheduler->tasks, iter, task) {
+		if (task != scheduler->main_task && task->state != R_CORE_TASK_STATE_DONE) {
 			count++;
 		}
 	}
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (scheduler, &old_sigset);
 	return count;
 }
 
@@ -122,12 +147,12 @@ static void task_join(RCoreTask *task) {
 	r_th_sem_post (sem);
 }
 
-R_API void r_core_task_join(RCore *core, RCoreTask *current, int id) {
+R_API void r_core_task_join(RCoreTaskScheduler *scheduler, RCoreTask *current, int id) {
 	if (current && id == current->id) {
 		return;
 	}
 	if (id >= 0) {
-		RCoreTask *task = r_core_task_get_incref (core, id);
+		RCoreTask *task = r_core_task_get_incref (scheduler, id);
 		if (!task) {
 			return;
 		}
@@ -141,8 +166,8 @@ R_API void r_core_task_join(RCore *core, RCoreTask *current, int id) {
 		r_core_task_decref (task);
 	} else {
 		TASK_SIGSET_T old_sigset;
-		tasks_lock_enter (core, &old_sigset);
-		RList *tasks = r_list_clone (core->tasks);
+		tasks_lock_enter (scheduler, &old_sigset);
+		RList *tasks = r_list_clone (scheduler->tasks);
 		RListIter *iter;
 		RCoreTask *task;
 		r_list_foreach (tasks, iter, task) {
@@ -151,7 +176,7 @@ R_API void r_core_task_join(RCore *core, RCoreTask *current, int id) {
 			}
 			r_core_task_incref (task);
 		}
-		tasks_lock_leave (core, &old_sigset);
+		tasks_lock_leave (scheduler, &old_sigset);
 
 		r_list_foreach (tasks, iter, task) {
 			if (current == task) {
@@ -195,6 +220,7 @@ R_API RCoreTask *r_core_task_new(RCore *core, bool create_cons, const char *cmd,
 	task->cmd_log = false;
 	task->res = NULL;
 	task->running_sem = NULL;
+	task->dispatched = false;
 	task->dispatch_cond = r_th_cond_new ();
 	task->dispatch_lock = r_th_lock_new (false);
 	if (!task->dispatch_cond || !task->dispatch_lock) {
@@ -202,13 +228,14 @@ R_API RCoreTask *r_core_task_new(RCore *core, bool create_cons, const char *cmd,
 	}
 
 	if (create_cons) {
-		task->cons_context = r_cons_context_new ();
+		task->cons_context = r_cons_context_new (r_cons_singleton ()->context);
 		if (!task->cons_context) {
 			goto hell;
 		}
+		task->cons_context->cmd_depth = core->max_cmd_depth;
 	}
 
-	task->id = core->task_id_next++;
+	task->id = core->tasks.task_id_next++;
 	task->state = R_CORE_TASK_STATE_BEFORE_START;
 	task->refcount = 1;
 	task->transient = false;
@@ -228,9 +255,9 @@ R_API void r_core_task_incref (RCoreTask *task) {
 		return;
 	}
 	TASK_SIGSET_T old_sigset;
-	tasks_lock_enter (task->core, &old_sigset);
+	tasks_lock_enter (&task->core->tasks, &old_sigset);
 	task->refcount++;
-	tasks_lock_leave (task->core, &old_sigset);
+	tasks_lock_leave (&task->core->tasks, &old_sigset);
 }
 
 R_API void r_core_task_decref (RCoreTask *task) {
@@ -238,103 +265,113 @@ R_API void r_core_task_decref (RCoreTask *task) {
 		return;
 	}
 	TASK_SIGSET_T old_sigset;
-	RCore *core = task->core;
-	tasks_lock_enter (core, &old_sigset);
+	RCoreTaskScheduler *scheduler = &task->core->tasks;
+	tasks_lock_enter (scheduler, &old_sigset);
 	task->refcount--;
 	if (task->refcount <= 0) {
 		task_free (task);
 	}
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (scheduler, &old_sigset);
 }
 
 R_API void r_core_task_schedule(RCoreTask *current, RTaskState next_state) {
 	RCore *core = current->core;
+	RCoreTaskScheduler *scheduler = &core->tasks;
 	bool stop = next_state != R_CORE_TASK_STATE_RUNNING;
 
-	if (core->oneshot_running || (!stop && core->tasks_running == 1 && core->oneshots_enqueued == 0)) {
+	if (scheduler->oneshot_running || (!stop && scheduler->tasks_running == 1 && scheduler->oneshots_enqueued == 0)) {
 		return;
 	}
 
-	core->current_task = NULL;
+	scheduler->current_task = NULL;
 
 	TASK_SIGSET_T old_sigset;
-	tasks_lock_enter (core, &old_sigset);
+	tasks_lock_enter (scheduler, &old_sigset);
 
 	current->state = next_state;
 
 	if (stop) {
-		core->tasks_running--;
-		r_th_lock_leave (current->dispatch_lock);
+		scheduler->tasks_running--;
 	}
 
 	// oneshots always have priority.
 	// if there are any queued, run them immediately.
 	OneShot *oneshot;
-	while ((oneshot = r_list_pop_head (core->oneshot_queue))) {
-		core->oneshots_enqueued--;
-		core->oneshot_running = true;
+	while ((oneshot = r_list_pop_head (scheduler->oneshot_queue))) {
+		scheduler->oneshots_enqueued--;
+		scheduler->oneshot_running = true;
 		oneshot->func (oneshot->user);
-		core->oneshot_running = false;
+		scheduler->oneshot_running = false;
 		free (oneshot);
 	}
 
-	RCoreTask *next = r_list_pop_head (core->tasks_queue);
+	RCoreTask *next = r_list_pop_head (scheduler->tasks_queue);
 
 	if (next && !stop) {
-		r_list_append (core->tasks_queue, current);
+		r_list_append (scheduler->tasks_queue, current);
+		r_th_lock_enter (current->dispatch_lock);
 	}
 
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (scheduler, &old_sigset);
 
 	if (next) {
 		r_cons_context_reset ();
 		r_th_lock_enter (next->dispatch_lock);
-		r_th_cond_signal (next->dispatch_cond);
+		next->dispatched = true;
 		r_th_lock_leave (next->dispatch_lock);
+		r_th_cond_signal (next->dispatch_cond);
 		if (!stop) {
-			r_th_cond_wait (current->dispatch_cond, current->dispatch_lock);
+			while (!current->dispatched) {
+				r_th_cond_wait (current->dispatch_cond, current->dispatch_lock);
+			}
+			current->dispatched = false;
+			r_th_lock_leave (current->dispatch_lock);
 		}
 	}
 
 	if (!stop) {
-		core->current_task = current;
+		scheduler->current_task = current;
 		if (current->cons_context) {
 			r_cons_context_load (current->cons_context);
 		} else {
 			r_cons_context_reset ();
 		}
-	} else {
-		r_cons_context_reset ();
 	}
 }
 
 static void task_wakeup(RCoreTask *current) {
 	RCore *core = current->core;
+	RCoreTaskScheduler *scheduler = &core->tasks;
 
 	TASK_SIGSET_T old_sigset;
-	tasks_lock_enter (core, &old_sigset);
+	tasks_lock_enter (scheduler, &old_sigset);
 
-	core->tasks_running++;
+	scheduler->tasks_running++;
 	current->state = R_CORE_TASK_STATE_RUNNING;
 
 	// check if there are other tasks running
-	bool single = core->tasks_running == 1;
+	bool single = scheduler->tasks_running == 1 || scheduler->tasks_running == 0;
 
 	r_th_lock_enter (current->dispatch_lock);
 
 	// if we are not the only task, we must wait until another task signals us.
 
 	if (!single) {
-		r_list_append (current->core->tasks_queue, current);
+		r_list_append (scheduler->tasks_queue, current);
 	}
 
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (scheduler, &old_sigset);
 
-	if(!single) {
-		r_th_cond_wait (current->dispatch_cond, current->dispatch_lock);
+	if (!single) {
+		while (!current->dispatched) {
+			r_th_cond_wait (current->dispatch_cond, current->dispatch_lock);
+		}
+		current->dispatched = false;
 	}
 
-	core->current_task = current;
+	r_th_lock_leave (current->dispatch_lock);
+
+	scheduler->current_task = current;
 
 	if (current->cons_context) {
 		r_cons_context_load (current->cons_context);
@@ -343,8 +380,12 @@ static void task_wakeup(RCoreTask *current) {
 	}
 }
 
-R_API void r_core_task_continue(RCoreTask *t) {
-	r_core_task_schedule (t, R_CORE_TASK_STATE_RUNNING);
+R_API void r_core_task_yield(RCoreTaskScheduler *scheduler) {
+	RCoreTask *task = r_core_task_self (scheduler);
+	if (!task) {
+		return;
+	}
+	r_core_task_schedule (task, R_CORE_TASK_STATE_RUNNING);
 }
 
 static void task_end(RCoreTask *t) {
@@ -353,6 +394,7 @@ static void task_end(RCoreTask *t) {
 
 static RThreadFunctionRet task_run(RCoreTask *task) {
 	RCore *core = task->core;
+	RCoreTaskScheduler *scheduler = &task->core->tasks;
 
 	task_wakeup (task);
 
@@ -362,7 +404,7 @@ static RThreadFunctionRet task_run(RCoreTask *task) {
 	}
 
 	char *res_str;
-	if (task == task->core->main_task) {
+	if (task == scheduler->main_task) {
 		r_core_cmd (core, task->cmd, task->cmd_log);
 		res_str = NULL;
 	} else {
@@ -372,13 +414,13 @@ static RThreadFunctionRet task_run(RCoreTask *task) {
 	free (task->res);
 	task->res = res_str;
 
-	if (task != core->main_task) {
+	if (task != scheduler->main_task && r_cons_default_context_is_interactive ()) {
 		eprintf ("\nTask %d finished\n", task->id);
 	}
 
 	TASK_SIGSET_T old_sigset;
 stillbirth:
-	tasks_lock_enter (core, &old_sigset);
+	tasks_lock_enter (scheduler, &old_sigset);
 
 	task_end (task);
 
@@ -398,16 +440,16 @@ stillbirth:
 	if (task->transient) {
 		RCoreTask *ltask;
 		RListIter *iter;
-		r_list_foreach (core->tasks, iter, ltask) {
+		r_list_foreach (scheduler->tasks, iter, ltask) {
 			if (ltask == task) {
-				r_list_delete (core->tasks, iter);
+				r_list_delete (scheduler->tasks, iter);
 				ret = R_TH_FREED;
 				break;
 			}
 		}
 	}
 
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (scheduler, &old_sigset);
 	return ret;
 }
 
@@ -416,12 +458,12 @@ static RThreadFunctionRet task_run_thread(RThread *th) {
 	return task_run (task);
 }
 
-R_API void r_core_task_enqueue(RCore *core, RCoreTask *task) {
-	if (!core || !task) {
+R_API void r_core_task_enqueue(RCoreTaskScheduler *scheduler, RCoreTask *task) {
+	if (!scheduler || !task) {
 		return;
 	}
 	TASK_SIGSET_T old_sigset;
-	tasks_lock_enter (core, &old_sigset);
+	tasks_lock_enter (scheduler, &old_sigset);
 	if (!task->running_sem) {
 		task->running_sem = r_th_sem_new (1);
 	}
@@ -431,56 +473,56 @@ R_API void r_core_task_enqueue(RCore *core, RCoreTask *task) {
 	if (task->cons_context) {
 		r_cons_context_break_push (task->cons_context, NULL, NULL, false);
 	}
-	r_list_append (core->tasks, task);
+	r_list_append (scheduler->tasks, task);
 	task->thread = r_th_new (task_run_thread, task, 0);
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (scheduler, &old_sigset);
 }
 
-R_API void r_core_task_enqueue_oneshot(RCore *core, RCoreTaskOneShot func, void *user) {
-	if (!core || !func) {
+R_API void r_core_task_enqueue_oneshot(RCoreTaskScheduler *scheduler, RCoreTaskOneShot func, void *user) {
+	if (!scheduler || !func) {
 		return;
 	}
 	TASK_SIGSET_T old_sigset;
-	tasks_lock_enter (core, &old_sigset);
-	if (core->tasks_running == 0) {
+	tasks_lock_enter (scheduler, &old_sigset);
+	if (scheduler->tasks_running == 0) {
 		// nothing is running right now and no other task can be scheduled
 		// while core->tasks_lock is locked => just run it
-		core->oneshot_running = true;
+		scheduler->oneshot_running = true;
 		func (user);
-		core->oneshot_running = false;
+		scheduler->oneshot_running = false;
 	} else {
 		OneShot *oneshot = R_NEW (OneShot);
 		if (oneshot) {
 			oneshot->func = func;
 			oneshot->user = user;
-			r_list_append (core->oneshot_queue, oneshot);
-			core->oneshots_enqueued++;
+			r_list_append (scheduler->oneshot_queue, oneshot);
+			scheduler->oneshots_enqueued++;
 		}
 	}
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (scheduler, &old_sigset);
 }
 
-R_API int r_core_task_run_sync(RCore *core, RCoreTask *task) {
+R_API int r_core_task_run_sync(RCoreTaskScheduler *scheduler, RCoreTask *task) {
 	task->thread = NULL;
 	return task_run (task);
 }
 
 /* begin running stuff synchronously on the main task */
-R_API void r_core_task_sync_begin(RCore *core) {
-	RCoreTask *task = core->main_task;
+R_API void r_core_task_sync_begin(RCoreTaskScheduler *scheduler) {
+	RCoreTask *task = scheduler->main_task;
 	TASK_SIGSET_T old_sigset;
-	tasks_lock_enter (core, &old_sigset);
+	tasks_lock_enter (scheduler, &old_sigset);
 	task->thread = NULL;
 	task->cmd = NULL;
 	task->cmd_log = false;
 	task->state = R_CORE_TASK_STATE_BEFORE_START;
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (scheduler, &old_sigset);
 	task_wakeup (task);
 }
 
 /* end running stuff synchronously, initially started with r_core_task_sync_begin() */
-R_API void r_core_task_sync_end(RCore *core) {
-	task_end (core->main_task);
+R_API void r_core_task_sync_end(RCoreTaskScheduler *scheduler) {
+	task_end (scheduler->main_task);
 }
 
 /* To be called from within a task.
@@ -508,14 +550,14 @@ R_API const char *r_core_task_status (RCoreTask *task) {
 	}
 }
 
-R_API RCoreTask *r_core_task_self (RCore *core) {
-	return core->current_task ? core->current_task : core->main_task;
+R_API RCoreTask *r_core_task_self (RCoreTaskScheduler *scheduler) {
+	return scheduler->current_task ? scheduler->current_task : scheduler->main_task;
 }
 
-static RCoreTask *task_get (RCore *core, int id) {
+static RCoreTask *task_get (RCoreTaskScheduler *scheduler, int id) {
 	RCoreTask *task;
 	RListIter *iter;
-	r_list_foreach (core->tasks, iter, task) {
+	r_list_foreach (scheduler->tasks, iter, task) {
 		if (task->id == id) {
 			return task;
 		}
@@ -523,57 +565,57 @@ static RCoreTask *task_get (RCore *core, int id) {
 	return NULL;
 }
 
-R_API RCoreTask *r_core_task_get_incref(RCore *core, int id) {
+R_API RCoreTask *r_core_task_get_incref(RCoreTaskScheduler *scheduler, int id) {
 	TASK_SIGSET_T old_sigset;
-	tasks_lock_enter (core, &old_sigset);
-	RCoreTask *task = task_get (core, id);
+	tasks_lock_enter (scheduler, &old_sigset);
+	RCoreTask *task = task_get (scheduler, id);
 	if (task) {
 		r_core_task_incref (task);
 	}
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (scheduler, &old_sigset);
 	return task;
 }
 
-R_API void r_core_task_break(RCore *core, int id) {
+R_API void r_core_task_break(RCoreTaskScheduler *scheduler, int id) {
 	TASK_SIGSET_T old_sigset;
-	tasks_lock_enter (core, &old_sigset);
-	RCoreTask *task = task_get (core, id);
+	tasks_lock_enter (scheduler, &old_sigset);
+	RCoreTask *task = task_get (scheduler, id);
 	if (!task || task->state == R_CORE_TASK_STATE_DONE) {
-		tasks_lock_leave (core, &old_sigset);
+		tasks_lock_leave (scheduler, &old_sigset);
 		return;
 	}
 	if (task->cons_context) {
 		r_cons_context_break (task->cons_context);
 	}
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (scheduler, &old_sigset);
 }
 
-R_API void r_core_task_break_all(RCore *core) {
+R_API void r_core_task_break_all(RCoreTaskScheduler *scheduler) {
 	TASK_SIGSET_T old_sigset;
-	tasks_lock_enter (core, &old_sigset);
+	tasks_lock_enter (scheduler, &old_sigset);
 	RCoreTask *task;
 	RListIter *iter;
-	r_list_foreach (core->tasks, iter, task) {
+	r_list_foreach (scheduler->tasks, iter, task) {
 		if (task->state != R_CORE_TASK_STATE_DONE) {
 			r_cons_context_break (task->cons_context);
 		}
 	}
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (scheduler, &old_sigset);
 }
 
-R_API int r_core_task_del (RCore *core, int id) {
+R_API int r_core_task_del (RCoreTaskScheduler *scheduler, int id) {
 	RCoreTask *task;
 	RListIter *iter;
 	bool ret = false;
 	TASK_SIGSET_T old_sigset;
-	tasks_lock_enter (core, &old_sigset);
-	r_list_foreach (core->tasks, iter, task) {
+	tasks_lock_enter (scheduler, &old_sigset);
+	r_list_foreach (scheduler->tasks, iter, task) {
 		if (task->id == id) {
-			if (task == core->main_task) {
+			if (task == scheduler->main_task) {
 				break;
 			}
 			if (task->state == R_CORE_TASK_STATE_DONE) {
-				r_list_delete (core->tasks, iter);
+				r_list_delete (scheduler->tasks, iter);
 			} else {
 				task->transient = true;
 			}
@@ -581,16 +623,16 @@ R_API int r_core_task_del (RCore *core, int id) {
 			break;
 		}
 	}
-	tasks_lock_leave (core, &old_sigset);
+	tasks_lock_leave (scheduler, &old_sigset);
 	return ret;
 }
 
-R_API void r_core_task_del_all_done (RCore *core) {
+R_API void r_core_task_del_all_done (RCoreTaskScheduler *scheduler) {
 	RCoreTask *task;
 	RListIter *iter, *iter2;
-	r_list_foreach_safe (core->tasks, iter, iter2, task) {
-		if (task != core->main_task && task->state == R_CORE_TASK_STATE_DONE) {
-			r_list_delete (core->tasks, iter);
+	r_list_foreach_safe (scheduler->tasks, iter, iter2, task) {
+		if (task != scheduler->main_task && task->state == R_CORE_TASK_STATE_DONE) {
+			r_list_delete (scheduler->tasks, iter);
 		}
 	}
 }
