@@ -1,9 +1,9 @@
-/* radare - LGPL - Copyright 2019 - GustavoLCR */
+/* radare - LGPL - Copyright 2019-2021 - GustavoLCR */
 
 #include <r_core.h>
-#include <TlHelp32.h>
-#include <windows_heap.h>
-#include "..\..\debug\p\native\maps\windows_maps.h"
+#include <tlhelp32.h>
+#include "heap/r_windows.h"
+#include "../debug/p/native/maps/windows_maps.h"
 
 /*
 *	Viewer discretion advised: Spaghetti code ahead
@@ -74,7 +74,7 @@ static size_t RtlpLFHKeyOffset = 0;
 	}\
 	hb->dwFlags |= ((flags) >> SHIFT) << SHIFT;
 
-static bool __is_windows_ten() {
+static bool __is_windows_ten(void) {
 	int major = 0;
 	RSysInfo *info = r_sys_info ();
 	if (info && info->version) {
@@ -120,7 +120,7 @@ static char *get_type(WPARAM flags) {
 	return r_str_newf ("%s %s%s", state, heaptype, type);
 }
 
-static bool init_func() {
+static bool init_func(void) {
 	HANDLE ntdll = LoadLibrary (TEXT ("ntdll.dll"));
 	if (!ntdll) {
 		return false;
@@ -272,7 +272,8 @@ static bool GetHeapGlobalsOffset(RDebug *dbg, HANDLE h_proc) {
 	char *ntdllopen = dbg->corebind.cmdstrf (dbg->corebind.core, "ob~%s", ntdll);
 	if (*ntdllopen) {
 		char *saddr = strtok (ntdllopen, " ");
-		for (int i = 0; i < 3; i++) {
+		size_t i;
+		for (i = 0; i < 3; i++) {
 			saddr = strtok (NULL, " ");
 		}
 		if (doopen) {
@@ -542,7 +543,8 @@ static bool __lfh_segment_loop(HANDLE h_proc, PHeapBlockBasicInfo *blocks, SIZE_
 		ReadProcessMemory (h_proc, (void *)next, &subsegment, sizeof (HEAP_LFH_SUBSEGMENT), NULL);
 		subsegment.BlockOffsets.EncodedData ^= (DWORD)lfhKey ^ ((DWORD)next >> 0xC);
 		WPARAM mask = 1, offset = 0;
-		for (int l = 0; l < subsegment.BlockCount; l++) {
+		int l;
+		for (l = 0; l < subsegment.BlockCount; l++) {
 			if (!mask) {
 				mask = 1;
 				offset++;
@@ -590,7 +592,8 @@ static bool GetSegmentHeapBlocks(RDebug *dbg, HANDLE h_proc, PVOID heapBase, PHe
 
 	// LFH
 	byte numBuckets = _countof (segheapHeader.LfhContext.Buckets);
-	for (int j = 0; j < numBuckets; j++) {
+	int j;
+	for (j = 0; j < numBuckets; j++) {
 		if ((WPARAM)segheapHeader.LfhContext.Buckets[j] & 1) {
 			continue;
 		}
@@ -647,7 +650,8 @@ static bool GetSegmentHeapBlocks(RDebug *dbg, HANDLE h_proc, PVOID heapBase, PHe
 	WPARAM RtlpHpHeapGlobal;
 	ReadProcessMemory (h_proc, (PVOID)RtlpHpHeapGlobalsOffset, &RtlpHpHeapGlobal, sizeof (WPARAM), &bytesRead);
 	// Backend Blocks (And VS)
-	for (int i = 0; i < 2; i++) {
+	int i;
+	for (i = 0; i < 2; i++) {
 		HEAP_SEG_CONTEXT ctx = segheapHeader.SegContexts[i];
 		WPARAM ctxFirstEntry = (WPARAM)heapBase + offsetof (SEGMENT_HEAP, SegContexts) + sizeof (HEAP_SEG_CONTEXT) * i + offsetof (HEAP_SEG_CONTEXT, SegmentListHead);
 		HEAP_PAGE_SEGMENT pageSegment;
@@ -836,7 +840,9 @@ static PDEBUG_BUFFER GetHeapBlocks(DWORD pid, RDebug *dbg) {
 					ReadProcessMemory (h_proc, userdata.BusyBitmap.Buffer, bitmap, bitmapsz, &bytesRead);
 					WPARAM mask = 1;
 					// Walk through the busy bitmap
-					for (int j = 0, offset = 0; j < userdata.BusyBitmap.SizeOfBitMap; j++) {
+					int j;
+					size_t offset;
+					for (j = 0, offset = 0; j < userdata.BusyBitmap.SizeOfBitMap; j++) {
 						if (!mask) {
 							mask = 1;
 							offset++;
@@ -930,7 +936,8 @@ err:
 		CloseHandle (h_proc);
 	}
 	if (db) {
-		for (int i = 0; i < heapInfo->count; i++) {
+		int i;
+		for (i = 0; i < heapInfo->count; i++) {
 			PDEBUG_HEAP_INFORMATION heap = &heapInfo->heaps[i];
 			free_extra_info (heap);
 			R_FREE (heap->Blocks);
@@ -1071,7 +1078,8 @@ static PHeapBlock GetSingleBlock(RDebug *dbg, ut64 offset) {
 	WPARAM NtLFHKey;
 	GetLFHKey (dbg, h_proc, false, &NtLFHKey);
 	PHeapInformation heapInfo = db->HeapInformation;
-	for (int i = 0; i < heapInfo->count; i++) {
+	int i;
+	for (i = 0; i < heapInfo->count; i++) {
 		DEBUG_HEAP_INFORMATION heap = heapInfo->heaps[i];
 		if (is_segment_heap (h_proc, heap.Base)) {
 			free (hb);
@@ -1158,8 +1166,8 @@ err:
 	return NULL;
 }
 
-static RTable *__new_heapblock_tbl() {
-	RTable *tbl = r_table_new ();
+static RTable *__new_heapblock_tbl(void) {
+	RTable *tbl = r_table_new ("heap");
 	r_table_add_column (tbl, r_table_type ("number"), "HeaderAddress", -1);
 	r_table_add_column (tbl, r_table_type ("number"), "UserAddress", -1);
 	r_table_add_column (tbl, r_table_type ("number"), "Size", -1);
@@ -1184,7 +1192,7 @@ static void w32_list_heaps(RCore *core, const char format) {
 	PHeapInformation heapInfo = db->HeapInformation;
 	CHECK_INFO (heapInfo);
 	int i;
-	RTable *tbl = r_table_new ();
+	RTable *tbl = r_table_new ("heaps");
 	r_table_add_column (tbl, r_table_type ("number"), "Address", -1);
 	r_table_add_column (tbl, r_table_type ("number"), "Blocks", -1);
 	r_table_add_column (tbl, r_table_type ("number"), "Allocated", -1);
@@ -1246,7 +1254,7 @@ static void w32_list_heaps_blocks(RCore *core, const char format) {
 		switch (format) {
 		case 'f':
 			if (heapInfo->heaps[i].BlockCount > 50000) {
-				go = r_cons_yesno ('n', "Are you sure you want to add %"PFMT64u" flags? (y/N)", heapInfo->heaps[i].BlockCount);
+				go = r_cons_yesno ('n', "Are you sure you want to add %lu flags? (y/N)", heapInfo->heaps[i].BlockCount);
 			}
 			break;
 		case 'j':
