@@ -38,7 +38,7 @@ static int perform_mapped_file_yank(RCore *core, ut64 offset, ut64 len, const ch
 	// grab the current file descriptor, so we can reset core and io state
 	// after our io op is done
 	RIODesc *yankdesc = NULL;
-	ut64 fd = core->file? core->file->fd: -1, yank_file_sz = 0,
+	ut64 fd = core->io->desc ? core->io->desc->fd: -1, yank_file_sz = 0,
 	     loadaddr = 0, addr = offset;
 	int res = false;
 
@@ -49,9 +49,10 @@ static int perform_mapped_file_yank(RCore *core, ut64 offset, ut64 len, const ch
 		// map the file in for IO operations.
 		if (yankdesc && load_align) {
 			yank_file_sz = r_io_size (core->io);
-			ut64 addr = r_io_map_next_available (core->io, 0, yank_file_sz, load_align);
-        		map = r_io_map_new (core->io, yankdesc->fd, R_PERM_R, 0, addr, yank_file_sz);
-			loadaddr = map? map->itv.addr: -1;
+			ut64 addr = 0;
+			r_io_map_locate (core->io, &addr, yank_file_sz, load_align);
+	 		map = r_io_map_add (core->io, yankdesc->fd, R_PERM_R, 0, addr, yank_file_sz);
+			loadaddr = map? r_io_map_begin (map): -1;
 			if (yankdesc && map && loadaddr != -1) {
 				// ***NOTE*** this is important, we need to
 				// address the file at its physical address!
@@ -145,12 +146,12 @@ R_API int r_core_yank(struct r_core_t *core, ut64 addr, int len) {
 		return false;
 	}
 	if (addr != core->offset) {
-		r_core_seek (core, addr, 1);
+		r_core_seek (core, addr, true);
 	}
 	r_io_read_at (core->io, addr, buf, len);
 	r_core_yank_set (core, addr, buf, len);
 	if (curseek != addr) {
-		r_core_seek (core, curseek, 1);
+		r_core_seek (core, curseek, true);
 	}
 	free (buf);
 	return true;
@@ -165,7 +166,7 @@ R_API int r_core_yank_string(RCore *core, ut64 addr, int maxlen) {
 		return false;
 	}
 	if (addr != core->offset) {
-		r_core_seek (core, addr, 1);
+		r_core_seek (core, addr, true);
 	}
 	/* Ensure space and safe termination for largest possible string allowed */
 	buf = calloc (1, core->blocksize + 1);
@@ -182,7 +183,7 @@ R_API int r_core_yank_string(RCore *core, ut64 addr, int maxlen) {
 	}
 	r_core_yank_set (core, addr, buf, maxlen);
 	if (curseek != addr) {
-		r_core_seek (core, curseek, 1);
+		r_core_seek (core, curseek, true);
 	}
 	free (buf);
 	return true;
@@ -223,10 +224,6 @@ R_API int r_core_yank_to(RCore *core, const char *_arg) {
 		pos = r_num_math (core->num, str + 1);
 		str[0] = ' ';
 	}
-	if (len < 1) {
-		free (arg);
-		return res;
-	}
 	if (!str || pos == -1 || len == 0) {
 		eprintf ("Usage: yt [len] [dst-addr]\n");
 		free (arg);
@@ -252,15 +249,24 @@ R_API bool r_core_yank_dump(RCore *core, ut64 pos, int format) {
 				}
 				r_cons_newline ();
 				break;
-			case 'j':
-				{
-					r_cons_printf ("{\"addr\":%"PFMT64u",\"bytes\":\"", core->yank_addr);
-					for (i = pos; i < r_buf_size (core->yank_buf); i++) {
-						r_cons_printf ("%02x", r_buf_read8_at (core->yank_buf, i));
-					}
-					r_cons_printf ("\"}\n");
+			case 'j': {
+				PJ *pj = r_core_pj_new (core);
+				if (!pj) {
+					break;
 				}
+				pj_o (pj);
+				pj_kn (pj, "addr", core->yank_addr);
+				RStrBuf *buf = r_strbuf_new ("");
+				for (i = pos; i < r_buf_size (core->yank_buf); i++) {
+					r_strbuf_appendf (buf, "%02x", r_buf_read8_at (core->yank_buf, i));
+				}
+				pj_ks (pj, "bytes", r_strbuf_get (buf));
+				r_strbuf_free (buf);
+				pj_end (pj);
+				r_cons_println (pj_string (pj));
+				pj_free (pj);
 				break;
+			}
 			case '*':
 				//r_cons_printf ("yfx ");
 				r_cons_printf ("wx ");
@@ -271,7 +277,7 @@ R_API bool r_core_yank_dump(RCore *core, ut64 pos, int format) {
 				r_cons_newline ();
 				break;
 			default:
-				r_cons_printf ("0x%08" PFMT64x " %d ",
+				r_cons_printf ("0x%08" PFMT64x " %" PFMT64d " ",
 						core->yank_addr + pos,
 						r_buf_size (core->yank_buf) - pos);
 				for (i = pos; i < r_buf_size (core->yank_buf); i++) {
@@ -325,7 +331,7 @@ R_API int r_core_yank_cat(RCore *core, ut64 pos) {
 				return false;
 			}
 			r_buf_read_at (core->yank_buf, pos, (ut8 *)buf, sz);
-			r_cons_memcat (buf, sz);
+			r_cons_write (buf, sz);
 			r_cons_newline ();
 			return true;
 		}
@@ -347,7 +353,7 @@ R_API int r_core_yank_cat_string(RCore *core, ut64 pos) {
 			}
 			r_buf_read_at (core->yank_buf, pos, (ut8 *)buf, sz);
 			int len = r_str_nlen (buf, sz);
-			r_cons_memcat (buf, len);
+			r_cons_write (buf, len);
 			r_cons_newline ();
 			return true;
 		}
