@@ -7,27 +7,39 @@ R_LIB_VERSION (r_crypto);
 
 static const struct {
 	const char *name;
-	ut64 bit;
+	RCryptoSelector bit;
 } crypto_name_bytes[] = {
-	{"all", UT64_MAX},
-	{"rc2", R_CRYPTO_RC2},
-	{"rc4", R_CRYPTO_RC4},
-	{"rc6", R_CRYPTO_RC6},
-	{"aes-ecb", R_CRYPTO_AES_ECB},
-	{"aes-cbc", R_CRYPTO_AES_CBC},
-	{"ror", R_CRYPTO_ROR},
-	{"rol", R_CRYPTO_ROL},
-	{"rot", R_CRYPTO_ROT},
-	{"blowfish", R_CRYPTO_BLOWFISH},
-	{"cps2", R_CRYPTO_CPS2},
-	{"des-ecb", R_CRYPTO_DES_ECB},
-	{"xor", R_CRYPTO_XOR},
-	{"serpent-ecb", R_CRYPTO_SERPENT},
-	{NULL, 0}
+	{ "all", UT64_MAX },
+	{ "rc2", R_CRYPTO_RC2 },
+	{ "rc4", R_CRYPTO_RC4 },
+	{ "rc6", R_CRYPTO_RC6 },
+	{ "aes-ecb", R_CRYPTO_AES_ECB },
+	{ "aes-cbc", R_CRYPTO_AES_CBC },
+	{ "aes-wrap", R_CRYPTO_AES_WRAP },
+	{ "ror", R_CRYPTO_ROR },
+	{ "rol", R_CRYPTO_ROL },
+	{ "rot", R_CRYPTO_ROT },
+	{ "blowfish", R_CRYPTO_BLOWFISH },
+	{ "cps2", R_CRYPTO_CPS2 },
+	{ "des-ecb", R_CRYPTO_DES_ECB },
+	{ "xor", R_CRYPTO_XOR },
+	{ "serpent-ecb", R_CRYPTO_SERPENT },
+	{ NULL, 0 }
 };
 
-R_API const char *r_crypto_name(ut64 bit) {
-	int i;
+static const struct {
+	const char *name;
+	RCryptoSelector bit;
+} codec_name_bytes[] = {
+	{ "all", UT64_MAX },
+	{ "base64", R_CODEC_B64 },
+	{ "base91", R_CODEC_B91 },
+	{ "punycode", R_CODEC_PUNYCODE },
+	{ NULL, 0 }
+};
+
+R_API const char *r_crypto_name(const RCryptoSelector bit) {
+	size_t i;
 	for (i = 1; crypto_name_bytes[i].bit; i++) {
 		if (bit & crypto_name_bytes[i].bit) {
 			return crypto_name_bytes[i].name;
@@ -36,6 +48,16 @@ R_API const char *r_crypto_name(ut64 bit) {
 	return "";
 }
 
+R_API const char *r_crypto_codec_name(const RCryptoSelector bit) {
+	size_t i;
+	for (i = 1; codec_name_bytes[i].bit; i++) {
+		if (bit & codec_name_bytes[i].bit) {
+			return codec_name_bytes[i].name;
+		}
+	}
+	return "";
+}
+
 static RCryptoPlugin *crypto_static_plugins[] = {
 	R_CRYPTO_STATIC_PLUGINS
 };
@@ -51,7 +73,7 @@ R_API RCrypto *r_crypto_init(RCrypto *cry, int hard) {
 			// first call initializes the output_* variables
 			r_crypto_get_output (cry, NULL);
 			cry->plugins = r_list_newf (NULL);
-			for (i=0; crypto_static_plugins[i]; i++) {
+			for (i = 0; crypto_static_plugins[i]; i++) {
 				RCryptoPlugin *p = R_NEW0 (RCryptoPlugin);
 				if (!p) {
 					free (cry);
@@ -76,7 +98,7 @@ R_API int r_crypto_del(RCrypto *cry, RCryptoPlugin *h) {
 	return true;
 }
 
-R_API struct r_crypto_t *r_crypto_new() {
+R_API struct r_crypto_t *r_crypto_new(void) {
 	RCrypto *cry = R_NEW0 (RCrypto);
 	return r_crypto_init (cry, true);
 }
@@ -106,9 +128,7 @@ R_API bool r_crypto_use(RCrypto *cry, const char *algo) {
 	r_list_foreach (cry->plugins, iter, h) {
 		if (h && h->use && h->use (algo)) {
 			cry->h = h;
-			cry->key_len = h->get_key_size (cry);
-			cry->key = calloc (1, cry->key_len);
-			return cry->key != NULL;
+			return cry->h != NULL;
 		}
 	}
 	return false;
@@ -121,6 +141,8 @@ R_API bool r_crypto_set_key(RCrypto *cry, const ut8* key, int keylen, int mode,
 	if (!cry || !cry->h || !cry->h->set_key) {
 		return false;
 	}
+	cry->key_len = keylen;
+	cry->key = calloc (1, cry->key_len);
 	return cry->h->set_key (cry, key, keylen, mode, direction);
 }
 
