@@ -4,32 +4,28 @@
 #include <r_crypto.h>
 #include "crypto_aes_algo.h"
 
-// TODO: avoid globals
-static struct aes_state st = {{0}};
+#define BLOCK_SIZE 16
 
-static bool aes_set_key (RCrypto *cry, const ut8 *key, int keylen, int mode, int direction) {
+static bool aes_set_key(RCrypto *cry, const ut8 *key, int keylen, int mode, int direction) {
 	if (!(keylen == 128 / 8 || keylen == 192 / 8 || keylen == 256 / 8)) {
 		return false;
 	}
-	st.key_size = keylen;
-	st.rounds = 6 + (int)(keylen / 4);
-	st.columns = (int)(keylen / 4);
-	memcpy (st.key, key, keylen);
+	cry->key_len = keylen;
+	memcpy (cry->key, key, keylen);
 	cry->dir = direction;
 	return true;
 }
 
-static int aes_get_key_size (RCrypto *cry) {
-	return st.key_size;
+static int aes_get_key_size(RCrypto *cry) {
+	return cry->key_len;
 }
 
-static bool aes_use (const char *algo) {
+static bool aes_use(const char *algo) {
 	return !strcmp (algo, "aes-ecb");
 }
 
-#define BLOCK_SIZE 16
-
-static bool update (RCrypto *cry, const ut8 *buf, int len) {
+static bool update(RCrypto *cry, const ut8 *buf, int len) {
+	struct aes_state st;
 	// Pad to the block size, do not append dummy block
 	const int diff = (BLOCK_SIZE - (len % BLOCK_SIZE)) % BLOCK_SIZE;
 	const int size = len + diff;
@@ -53,6 +49,11 @@ static bool update (RCrypto *cry, const ut8 *buf, int len) {
 		ibuf[len] = 8; //0b1000;
 	}
 
+	st.key_size = cry->key_len;
+	st.rounds = 6 + (st.key_size / 4);
+	st.columns = (st.key_size / 4);
+	memcpy (st.key, cry->key, st.key_size);
+
 	if (cry->dir == 0) {
 		for (i = 0; i < blocks; i++) {
 			const int delta = BLOCK_SIZE * i;
@@ -65,8 +66,6 @@ static bool update (RCrypto *cry, const ut8 *buf, int len) {
 		}
 	}
 
-	// printf("%128s\n", obuf);
-
 	r_crypto_append (cry, obuf, size);
 	free (obuf);
 	free (ibuf);
