@@ -9,7 +9,7 @@ static bool serpent_set_key(RCrypto *cry, const ut8 *key, int keylen, int mode,
 	if ((keylen != 128 / 8) && (keylen != 192 / 8) && (keylen != 256 / 8)) {
 		return false;
 	}
-	st.key_size = keylen*8;
+	st.key_size = keylen * 8;
 	eprintf ("key_size: %d\n", st.key_size);
 	memcpy (st.key, key, keylen);
 	cry->dir = direction;
@@ -33,51 +33,54 @@ static bool update(RCrypto *cry, const ut8 *buf, int len) {
 	const int blocks = size / BLOCK_SIZE;
 	int i, j;
 
-	ut8 *const obuf = calloc (4, size/4);
+	ut8 *const obuf = calloc (4, size / 4);
 	if (!obuf) {
 		return false;
 	}
-	ut32 *const ibuf = calloc (4, size/4);
+	ut32 *const ibuf = calloc (4, size / 4);
 	if (!ibuf) {
 		free (obuf);
 		return false;
 	}
-	ut32 *const tmp = calloc (4, size/4);
-	if (!ibuf) {
+	ut32 *const tmp = calloc (4, size / 4);
+	if (!tmp) {
 		free (obuf);
 		free (ibuf);
 		return false;
 	}
 
 	// Construct ut32 blocks from byte stream
-	for (j = 0; j < size/4; j++) {
-		ibuf[j] = r_read_le32(&buf[4*j]);
+	for (j = 0; j < len / 4; j++) {
+		ibuf[j] = r_read_le32 (buf + 4 * j);
+	}
+	if (len & 0x3) {
+		ut8 tail[4] = {0}; // Zero padding
+		memcpy (tail, buf + (len & ~0x3), len & 0x3);
+		ibuf[len / 4] = r_read_le32 (tail);
 	}
-
-	// Zero padding.
 
 	if (cry->dir == 0) {
 		for (i = 0; i < blocks; i++) {
 			// delta in number of ut32
-			const int delta = (BLOCK_SIZE * i)/4;
+			const int delta = (BLOCK_SIZE * i) / 4;
 			serpent_encrypt (&st, ibuf + delta, tmp + delta);
 		}
 	} else if (cry->dir > 0) {
 		for (i = 0; i < blocks; i++) {
 			// delta in number of ut32
-			const int delta = (BLOCK_SIZE * i)/4;
+			const int delta = (BLOCK_SIZE * i) / 4;
 			serpent_decrypt (&st, ibuf + delta, tmp + delta);
 		}
 	}
 
 	// Construct ut32 blocks from byte stream
 	int k;
-	for (j = 0; j < size/4; j++) {
-		k = 4*j;
+	for (j = 0; j < size / 4; j++) {
+		k = 4 * j;
 		obuf[k] = tmp[j] & 0xff;
-		obuf[k+1] = (tmp[j] >> 8) & 0xff;
-		obuf[k+2] = (tmp[j] >> 16) & 0xff;
-		obuf[k+3] = (tmp[j] >> 24) & 0xff;
+		obuf[k + 1] = (tmp[j] >> 8) & 0xff;
+		obuf[k + 2] = (tmp[j] >> 16) & 0xff;
+		obuf[k + 3] = (tmp[j] >> 24) & 0xff;
 	}
 
 	r_crypto_append (cry, obuf, size);
