@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2018 - pancake, jduck, TheLemonMan, saucec0de */
+/* radare - LGPL - Copyright 2009-2021 - pancake, jduck, TheLemonMan, saucec0de */
 
 #include <r_debug.h>
 #include <r_drx.h>
@@ -11,13 +11,11 @@ R_LIB_VERSION(r_debug);
 #define DBG_BUF_SIZE 512
 
 R_API RDebugInfo *r_debug_info(RDebug *dbg, const char *arg) {
-	if (!dbg || !dbg->h || !dbg->h->info) {
-		return NULL;
-	}
+	r_return_val_if_fail (dbg && dbg->h, NULL);
 	if (dbg->pid < 0) {
 		return NULL;
 	}
-	return dbg->h->info (dbg, arg);
+	return dbg->h->info? dbg->h->info (dbg, arg): NULL;
 }
 
 R_API void r_debug_info_free(RDebugInfo *rdi) {
@@ -105,6 +103,7 @@ static int r_debug_bp_hit(RDebug *dbg, RRegItem *pc_ri, ut64 pc, RBreakpointItem
 		if (!b) { /* we don't. nothing left to do */
 			/* Some targets set pc to breakpoint */
 			b = r_bp_get_at (dbg->bp, pc);
+#if __i386__ || __x86_64__
 			if (!b) {
 				/* handle the case of hw breakpoints - notify the user */
 				int drx_reg_idx = r_debug_drx_at (dbg, pc);
@@ -115,10 +114,9 @@ static int r_debug_bp_hit(RDebug *dbg, RRegItem *pc_ri, ut64 pc, RBreakpointItem
 				/* Couldn't find the break point. Nothing more to do... */
 				return true;
 			}
-			else {
-				dbg->pc_at_bp_set = true;
-				dbg->pc_at_bp = true;
-			}
+#endif
+			dbg->pc_at_bp_set = true;
+			dbg->pc_at_bp = true;
 		} else {
 			dbg->pc_at_bp_set = true;
 			dbg->pc_at_bp = false;
@@ -177,7 +175,7 @@ static int r_debug_bp_hit(RDebug *dbg, RRegItem *pc_ri, ut64 pc, RBreakpointItem
 
 	/* inform the user of what happened */
 	if (dbg->hitinfo) {
-		eprintf ("hit %spoint at: %" PFMT64x "\n",
+		eprintf ("hit %spoint at: 0x%" PFMT64x "\n",
 			b->trace ? "trace" : "break", pc);
 	}
 
@@ -214,7 +212,7 @@ static int r_debug_bps_enable(RDebug *dbg) {
  *
  * if the user wants to step, the single step here does the job.
  */
-static int r_debug_recoil(RDebug *dbg, RDebugRecoilMode rc_mode) {
+static bool r_debug_recoil(RDebug *dbg, RDebugRecoilMode rc_mode) {
 	/* if bp_addr is not set, we must not have actually hit a breakpoint */
 	if (!dbg->reason.bp_addr) {
 		return r_debug_bps_enable (dbg);
@@ -255,6 +253,7 @@ static int r_debug_recoil(RDebug *dbg, RDebugRecoilMode rc_mode) {
 	if (!dbg->reason.bp_addr && dbg->recoil_mode == R_DBG_RECOIL_STEP) {
 		return true;
 	}
+	dbg->reason.bp_addr = 0;
 
 	return r_debug_bps_enable (dbg);
 }
@@ -263,7 +262,7 @@ static int r_debug_recoil(RDebug *dbg, RDebugRecoilMode rc_mode) {
 R_API RBreakpointItem *r_debug_bp_add(RDebug *dbg, ut64 addr, int hw, bool watch, int rw, char *module, st64 m_delta) {
 	int bpsz = r_bp_size(dbg->bp);
 	RBreakpointItem *bpi;
-	const char *module_name = module;
+	char *module_name = module? strdup (module): NULL;
 	RListIter *iter;
 	RDebugMap *map;
 	if (!addr && module) {
@@ -274,9 +273,10 @@ R_API RBreakpointItem *r_debug_bp_add(RDebug *dbg, ut64 addr, int hw, bool watch
 			detect_module = false;
 			RList *list = r_debug_modules_list (dbg);
 			r_list_foreach (list, iter, map) {
-				if (strstr (map->file, module)) {
+				if (map->file && strstr (map->file, module)) {
 					addr = map->addr + m_delta;
-					module_name = map->file;
+					free (module_name);
+					module_name = strdup (map->file);
 					break;
 				}
 			}
@@ -293,19 +293,21 @@ R_API RBreakpointItem *r_debug_bp_add(RDebug *dbg, ut64 addr, int hw, bool watch
 		r_list_foreach (dbg->maps, iter, map) {
 			if (addr >= map->addr && addr < map->addr_end) {
 				valid = true;
-				if (detect_module) {
-					module_name = map->file;
+				if (detect_module && map->file) {
+					free (module_name);
+					module_name = strdup (map->file);
 					m_delta = addr - map->addr;
 				}
 				perm = ((map->perm & 1) << 2) | (map->perm & 2) | ((map->perm & 4) >> 2);
 				if (!(perm & R_BP_PROT_EXEC)) {
-					eprintf ("WARNING: setting bp within mapped memory without exec perm\n");
+					eprintf ("Warning: setting bp within mapped memory without exec perm\n");
 				}
 				break;
 			}
 		}
 		if (!valid) {
-			eprintf ("WARNING: module's base addr + delta is not a valid address\n");
+			eprintf ("Warning: module's base addr + delta is not a valid address\n");
+			free (module_name);
 			return NULL;
 		}
 	}
@@ -313,15 +315,16 @@ R_API RBreakpointItem *r_debug_bp_add(RDebug *dbg, ut64 addr, int hw, bool watch
 		//express db breakpoints as dbm due to ASLR when saving into project
 		r_debug_map_sync (dbg);
 		r_list_foreach (dbg->maps, iter, map) {
-			if (addr >= map->addr && addr < map->addr_end) {
-				module_name = map->file;
+			if (map->file && addr >= map->addr && addr < map->addr_end) {
+				free (module_name);
+				module_name = strdup (map->file);
 				m_delta = addr - map->addr;
 				break;
 			}
 		}
 	}
 	if (watch) {
-		hw = 1; //XXX
+		hw = 1; // XXX
 		bpi = r_bp_watch_add (dbg->bp, addr, bpsz, hw, rw);
 	} else {
 		bpi = hw
@@ -332,15 +335,17 @@ R_API RBreakpointItem *r_debug_bp_add(RDebug *dbg, ut64 addr, int hw, bool watch
 		if (module_name) {
 			bpi->module_name = strdup (module_name);
 			bpi->name = r_str_newf ("%s+0x%" PFMT64x, module_name, m_delta);
+			R_FREE (module_name);
 		}
 		bpi->module_delta = m_delta;
 	}
+	free (module_name);
 	return bpi;
 }
 
 static const char *r_debug_str_callback(RNum *userptr, ut64 off, int *ok) {
 	// RDebug *dbg = (RDebug *)userptr;
-	eprintf ("STR CALLBACK WTF WTF WTF\n");
+	// TODO: implement the rnum callback for str or just get rid of it as we dont need it
 	return NULL;
 }
 
@@ -364,8 +369,6 @@ R_API RDebug *r_debug_new(int hard) {
 	R_FREE (dbg->btalgo);
 	dbg->trace_execs = 0;
 	dbg->anal = NULL;
-	dbg->snaps = r_list_newf ((RListFree)r_debug_snap_free);
-	dbg->sessions = r_list_newf ((RListFree)r_debug_session_free);
 	dbg->pid = -1;
 	dbg->bpsize = 1;
 	dbg->tid = -1;
@@ -385,6 +388,8 @@ R_API RDebug *r_debug_new(int hard) {
 	dbg->maps_user = r_debug_map_list_new ();
 	dbg->q_regs = NULL;
 	dbg->call_frames = NULL;
+	dbg->main_arena_resolved = false;
+	dbg->glibc_version = 231; /* default version ubuntu 20 */
 	r_debug_signal_init (dbg);
 	if (hard) {
 		dbg->bp = r_bp_new ();
@@ -397,7 +402,7 @@ R_API RDebug *r_debug_new(int hard) {
 
 static int free_tracenodes_entry(RDebug *dbg, const char *k, const char *v) {
 	ut64 v_num = r_num_get (NULL, v);
-	free((void *)(size_t)v_num);
+	free ((void *)(size_t)v_num);
 	return true;
 }
 
@@ -406,14 +411,12 @@ R_API void r_debug_tracenodes_reset(RDebug *dbg) {
 	sdb_reset (dbg->tracenodes);
 }
 
-R_API RDebug *r_debug_free(RDebug *dbg) {
+R_API void r_debug_free(RDebug *dbg) {
 	if (dbg) {
 		// TODO: free it correctly.. we must ensure this is an instance and not a reference..
 		r_bp_free (dbg->bp);
 		//r_reg_free(&dbg->reg);
 		free (dbg->snap_path);
-		r_list_free (dbg->snaps);
-		r_list_free (dbg->sessions);
 		r_list_free (dbg->maps);
 		r_list_free (dbg->maps_user);
 		r_list_free (dbg->threads);
@@ -426,6 +429,8 @@ R_API RDebug *r_debug_free(RDebug *dbg) {
 		r_list_free (dbg->call_frames);
 		free (dbg->btalgo);
 		r_debug_trace_free (dbg->trace);
+		r_debug_session_free (dbg->session);
+		r_anal_op_free (dbg->cur_op);
 		dbg->trace = NULL;
 		r_egg_free (dbg->egg);
 		free (dbg->arch);
@@ -433,17 +438,24 @@ R_API RDebug *r_debug_free(RDebug *dbg) {
 		free (dbg->glob_unlibs);
 		free (dbg);
 	}
-	return NULL;
 }
 
-R_API int r_debug_attach(RDebug *dbg, int pid) {
-	int ret = false;
-	if (dbg && dbg->h && dbg->h->attach) {
+R_API bool r_debug_attach(RDebug *dbg, int pid) {
+	r_return_val_if_fail (dbg, false);
+	if (pid < 0) {
+		return false;
+	}
+	bool ret = false;
+	if (dbg->h && dbg->h->attach) {
 		ret = dbg->h->attach (dbg, pid);
-		if (ret != -1) {
-			r_debug_select (dbg, pid, ret); //dbg->pid, dbg->tid);
+		if (ret) {
+			dbg->tid = pid;
+			// dbg->pid = pid;
+			// r_debug_select (dbg, pid, ret);
+			r_debug_select (dbg, dbg->pid, dbg->tid);
 		}
 	}
+	dbg->reason.type = R_DEBUG_REASON_STOPPED;
 	return ret;
 }
 
@@ -457,38 +469,40 @@ R_API int r_debug_stop(RDebug *dbg) {
 
 R_API bool r_debug_set_arch(RDebug *dbg, const char *arch, int bits) {
 	if (arch && dbg && dbg->h) {
-		bool rc = r_sys_arch_match (dbg->h->arch, arch);
-		if (rc) {
-			switch (bits) {
-			case 27:
-				if (dbg->h->bits == 27) {
-					dbg->bits = 27;
-				}
-				break;
-			case 32:
-				if (dbg->h->bits & R_SYS_BITS_32) {
-					dbg->bits = R_SYS_BITS_32;
-				}
-				break;
-			case 64:
-				dbg->bits = R_SYS_BITS_64;
-				break;
+		switch (bits) {
+		case 16:
+			if (dbg->h->bits == 16) {
+				dbg->bits = R_SYS_BITS_16;
 			}
-			if (!dbg->h->bits) {
-				dbg->bits = dbg->h->bits;
-			} else if (!(dbg->h->bits & dbg->bits)) {
-				dbg->bits = dbg->h->bits & R_SYS_BITS_64;
-				if (!dbg->bits) {
-					dbg->bits = dbg->h->bits & R_SYS_BITS_32;
-				}
-				if (!dbg->bits) {
-					dbg->bits = R_SYS_BITS_32;
-				}
+			break;
+		case 27:
+			if (dbg->h->bits == 27) {
+				dbg->bits = R_SYS_BITS_27;
 			}
-			free (dbg->arch);
-			dbg->arch = strdup (arch);
-			return true;
+			break;
+		case 32:
+			if (dbg->h->bits & R_SYS_BITS_32) {
+				dbg->bits = R_SYS_BITS_32;
+			}
+			break;
+		case 64:
+			dbg->bits = R_SYS_BITS_64;
+			break;
 		}
+		if (!dbg->h->bits) {
+			dbg->bits = dbg->h->bits;
+		} else if (!(dbg->h->bits & dbg->bits)) {
+			dbg->bits = dbg->h->bits & R_SYS_BITS_64;
+			if (!dbg->bits) {
+				dbg->bits = dbg->h->bits & R_SYS_BITS_32;
+			}
+			if (!dbg->bits) {
+				dbg->bits = R_SYS_BITS_32;
+			}
+		}
+		free (dbg->arch);
+		dbg->arch = strdup (arch);
+		return true;
 	}
 	return false;
 }
@@ -511,7 +525,7 @@ R_API ut64 r_debug_execute(RDebug *dbg, const ut8 *buf, int len, int restore) {
 	risp = r_reg_get (dbg->reg, dbg->reg->name[R_REG_NAME_SP], R_REG_TYPE_GPR);
 	if (ripc) {
 		r_debug_reg_sync (dbg, R_REG_TYPE_GPR, false);
-		orig = r_reg_get_bytes (dbg->reg, -1, &orig_sz);
+		orig = r_reg_get_bytes (dbg->reg, R_REG_TYPE_ALL, &orig_sz);
 		if (!orig) {
 			eprintf ("Cannot get register arena bytes\n");
 			return 0LL;
@@ -584,21 +598,18 @@ R_API int r_debug_detach(RDebug *dbg, int pid) {
 }
 
 R_API bool r_debug_select(RDebug *dbg, int pid, int tid) {
-	ut64 pc = 0;
-	int prev_pid = dbg->pid;
-	int prev_tid = dbg->tid;
-
+	r_return_val_if_fail (dbg, false);
 	if (pid < 0) {
 		return false;
 	}
 	if (tid < 0) {
 		tid = pid;
 	}
-	if (pid != -1 && tid != -1) {
-		if ((pid != dbg->pid || tid != dbg->tid) && dbg->verbose) {
-			eprintf ("= attach %d %d\n", pid, tid);
-		}
-	} else {
+#if 0
+	pid = r_io_desc_get_pid (dbg->iob.io->desc);
+	tid = r_io_desc_get_tid (dbg->iob.io->desc);
+#endif
+	if (pid == -1 && tid == -1) {
 		if (dbg->pid != -1) {
 			eprintf ("Child %d is dead\n", dbg->pid);
 		}
@@ -606,20 +617,22 @@ R_API bool r_debug_select(RDebug *dbg, int pid, int tid) {
 	if (pid < 0 || tid < 0) {
 		return false;
 	}
-
-	if (dbg->h && dbg->h->select && !dbg->h->select (dbg, pid, tid)) {
-		return false;
+	if (dbg->h && dbg->h->select) {
+		if (!dbg->h->select (dbg, pid, tid)) {
+			return false;
+		}
 	}
-
-	// Don't change the pid/tid if the plugin already modified it due to internal constraints
-	if (dbg->pid == prev_pid) {
-		dbg->pid = pid;
+	dbg->pid = pid;
+	dbg->tid = tid;
+	if (dbg->tid != -1) {
+		char *pidcmd = r_str_newf ("pid %d", dbg->tid);
+		if (pidcmd) {
+			free (r_io_system (dbg->iob.io, pidcmd));
+			free (pidcmd);
+		}
+	} else {
+		eprintf ("Cannot find pid for child %d\n", dbg->pid);
 	}
-	if (dbg->tid == prev_tid) {
-		dbg->tid = tid;
-	}
-
-	r_io_system (dbg->iob.io, sdb_fmt ("pid %d", dbg->tid));
 
 	// Synchronize with the current thread's data
 	if (dbg->corebind.core) {
@@ -628,40 +641,43 @@ R_API bool r_debug_select(RDebug *dbg, int pid, int tid) {
 		r_reg_arena_swap (core->dbg->reg, true);
 		r_debug_reg_sync (dbg, R_REG_TYPE_ALL, false);
 
-		pc = r_debug_reg_get (dbg, "PC");
-		core->offset = pc;
+		core->offset = r_debug_reg_get (dbg, "PC");
 	}
 
+	dbg->main_arena_resolved = false;
+
 	return true;
 }
 
+// TODO type should be enum so we can ensure to not miss an item
 R_API const char *r_debug_reason_to_string(int type) {
 	switch (type) {
-	case R_DEBUG_REASON_DEAD: return "dead";
 	case R_DEBUG_REASON_ABORT: return "abort";
-	case R_DEBUG_REASON_SEGFAULT: return "segfault";
-	case R_DEBUG_REASON_NONE: return "none";
-	case R_DEBUG_REASON_SIGNAL: return "signal";
 	case R_DEBUG_REASON_BREAKPOINT: return "breakpoint";
-	case R_DEBUG_REASON_TRACEPOINT: return "tracepoint";
-	case R_DEBUG_REASON_READERR: return "read-error";
-	case R_DEBUG_REASON_WRITERR: return "write-error";
+	case R_DEBUG_REASON_DEAD: return "dead";
 	case R_DEBUG_REASON_DIVBYZERO: return "div-by-zero";
-	case R_DEBUG_REASON_ILLEGAL: return "illegal";
-	case R_DEBUG_REASON_UNKNOWN: return "unknown";
 	case R_DEBUG_REASON_ERROR: return "error";
-	case R_DEBUG_REASON_NEW_PID: return "new-pid";
-	case R_DEBUG_REASON_NEW_TID: return "new-tid";
-	case R_DEBUG_REASON_NEW_LIB: return "new-lib";
+	case R_DEBUG_REASON_EXIT_LIB: return "exit-lib";
 	case R_DEBUG_REASON_EXIT_PID: return "exit-pid";
 	case R_DEBUG_REASON_EXIT_TID: return "exit-tid";
-	case R_DEBUG_REASON_EXIT_LIB: return "exit-lib";
-	case R_DEBUG_REASON_TRAP: return "trap";
-	case R_DEBUG_REASON_SWI: return "software-interrupt";
-	case R_DEBUG_REASON_INT: return "interrupt";
 	case R_DEBUG_REASON_FPU: return "fpu";
+	case R_DEBUG_REASON_ILLEGAL: return "illegal";
+	case R_DEBUG_REASON_INT: return "interrupt";
+	case R_DEBUG_REASON_NEW_LIB: return "new-lib";
+	case R_DEBUG_REASON_NEW_PID: return "new-pid";
+	case R_DEBUG_REASON_NEW_TID: return "new-tid";
+	case R_DEBUG_REASON_NONE: return "none";
+	case R_DEBUG_REASON_READERR: return "read-error";
+	case R_DEBUG_REASON_SEGFAULT: return "segfault";
+	case R_DEBUG_REASON_SIGNAL: return "signal";
 	case R_DEBUG_REASON_STEP: return "step";
+	case R_DEBUG_REASON_STOPPED: return "stopped";
+	case R_DEBUG_REASON_SWI: return "software-interrupt";
+	case R_DEBUG_REASON_TRACEPOINT: return "tracepoint";
+	case R_DEBUG_REASON_TRAP: return "trap";
+	case R_DEBUG_REASON_UNKNOWN: return "unknown";
 	case R_DEBUG_REASON_USERSUSP: return "suspended-by-user";
+	case R_DEBUG_REASON_WRITERR: return "write-error";
 	}
 	return "unhandled";
 }
@@ -686,7 +702,6 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp) {
 	if (!dbg) {
 		return reason;
 	}
-
 	if (bp) {
 		*bp = NULL;
 	}
@@ -709,7 +724,14 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp) {
 			//r_debug_select (dbg, -1, -1);
 			return R_DEBUG_REASON_DEAD;
 		}
-
+#if __linux__
+		// Letting other threads running will cause ptrace commands to fail
+		// when writing to the same process memory to set/unset breakpoints
+		// and is problematic in Linux.
+		if (dbg->continue_all_threads) {
+			r_debug_stop (dbg);
+		}
+#endif
 		/* propagate errors from the plugin */
 		if (reason == R_DEBUG_REASON_ERROR) {
 			return R_DEBUG_REASON_ERROR;
@@ -722,9 +744,9 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp) {
 
 		bool libs_bp = (dbg->glob_libs || dbg->glob_unlibs) ? true : false;
 		/* if the underlying stop reason is a breakpoint, call the handlers */
-		if (reason == R_DEBUG_REASON_BREAKPOINT || reason == R_DEBUG_REASON_STEP ||
-			(libs_bp &&
-			((reason == R_DEBUG_REASON_NEW_LIB) || (reason == R_DEBUG_REASON_EXIT_LIB)))) {
+		if (reason == R_DEBUG_REASON_BREAKPOINT ||
+			reason == R_DEBUG_REASON_STEP ||
+			(libs_bp && ((reason == R_DEBUG_REASON_NEW_LIB) || (reason == R_DEBUG_REASON_EXIT_LIB)))) {
 			RRegItem *pc_ri;
 			RBreakpointItem *b = NULL;
 			ut64 pc;
@@ -732,7 +754,7 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp) {
 			/* get the program coounter */
 			pc_ri = r_reg_get (dbg->reg, dbg->reg->name[R_REG_NAME_PC], -1);
 			if (!pc_ri) { /* couldn't find PC?! */
-				eprintf ("Couldn't find PC!\n");
+				eprintf ("Couldn't find the program counter!\n");
 				return R_DEBUG_REASON_ERROR;
 			}
 
@@ -747,6 +769,9 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp) {
 				*bp = b;
 			}
 
+			if (b && reason == R_DEBUG_REASON_STEP) {
+				reason = R_DEBUG_REASON_BREAKPOINT;
+			}
 			/* if we hit a tracing breakpoint, we need to continue in
 			 * whatever mode the user desired. */
 			if (dbg->corebind.core && b && b->cond) {
@@ -762,9 +787,10 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp) {
 			/* handle signal on continuations here */
 			int what = r_debug_signal_what (dbg, dbg->reason.signum);
 			const char *name = r_signal_to_string (dbg->reason.signum);
+			const char *humn = r_signal_to_human (dbg->reason.signum);
 			if (name && strcmp ("SIGTRAP", name)) {
-				r_cons_printf ("[+] signal %d aka %s received %d\n",
-						dbg->reason.signum, name, what);
+				r_cons_printf ("[+] signal %d aka %s received %d (%s)\n",
+						dbg->reason.signum, name, what, humn);
 			}
 		}
 	}
@@ -776,7 +802,7 @@ R_API int r_debug_step_soft(RDebug *dbg) {
 	ut64 pc, sp, r;
 	ut64 next[2];
 	RAnalOp op;
-	int br, i, ret;
+	int br, i;
 	union {
 		ut64 r64;
 		ut32 r32[2];
@@ -870,7 +896,7 @@ R_API int r_debug_step_soft(RDebug *dbg) {
 		}
 	}
 
-	ret = r_debug_continue (dbg);
+	bool ret = r_debug_continue (dbg);
 
 	for (i = 0; i < br; i++) {
 		r_bp_del (dbg->bp, next[i]);
@@ -879,7 +905,7 @@ R_API int r_debug_step_soft(RDebug *dbg) {
 	return ret;
 }
 
-R_API int r_debug_step_hard(RDebug *dbg) {
+R_API int r_debug_step_hard(RDebug *dbg, RBreakpointItem **pb) {
 	RDebugReasonType reason;
 
 	dbg->reason.type = R_DEBUG_REASON_STEP;
@@ -904,12 +930,24 @@ R_API int r_debug_step_hard(RDebug *dbg) {
 	if (!dbg->h->step (dbg)) {
 		return false;
 	}
-	reason = r_debug_wait (dbg, NULL);
+
+#if __linux__
+	// Turn off continue_all_threads to make sure linux_dbg_wait
+	// only waits for one target for a single-step or breakpoint trap
+	bool prev_continue = dbg->continue_all_threads;
+	dbg->continue_all_threads = false;
+#endif
+	reason = r_debug_wait (dbg, pb);
+#if __linux__
+	dbg->continue_all_threads = prev_continue;
+#endif
+
 	if (reason == R_DEBUG_REASON_DEAD || r_debug_is_dead (dbg)) {
 		return false;
 	}
 	// Unset breakpoints before leaving
-	if (reason != R_DEBUG_REASON_BREAKPOINT && reason != R_DEBUG_REASON_COND &&
+	if (reason != R_DEBUG_REASON_BREAKPOINT &&
+		reason != R_DEBUG_REASON_COND &&
 		reason != R_DEBUG_REASON_TRACEPOINT) {
 		r_bp_restore (dbg->bp, false);
 	}
@@ -921,6 +959,7 @@ R_API int r_debug_step_hard(RDebug *dbg) {
 }
 
 R_API int r_debug_step(RDebug *dbg, int steps) {
+	RBreakpointItem *bp = NULL;
 	int ret, steps_taken = 0;
 
 	/* who calls this without giving a positive number? */
@@ -938,16 +977,40 @@ R_API int r_debug_step(RDebug *dbg, int steps) {
 
 	dbg->reason.type = R_DEBUG_REASON_STEP;
 
+	if (dbg->session) {
+		if (dbg->session->cnum != dbg->session->maxcnum) {
+			steps_taken = r_debug_step_cnum (dbg, steps);
+		}
+	}
+
 	for (; steps_taken < steps; steps_taken++) {
+		if (dbg->session && dbg->recoil_mode == R_DBG_RECOIL_NONE) {
+			dbg->session->cnum++;
+			dbg->session->maxcnum++;
+			dbg->session->bp = 0;
+			if (!r_debug_trace_ins_before (dbg)) {
+				eprintf ("trace_ins_before: failed");
+			}
+		}
+
 		if (dbg->swstep) {
 			ret = r_debug_step_soft (dbg);
 		} else {
-			ret = r_debug_step_hard (dbg);
+			ret = r_debug_step_hard (dbg, &bp);
 		}
 		if (!ret) {
 			eprintf ("Stepping failed!\n");
 			return steps_taken;
 		}
+
+		if (dbg->session && dbg->recoil_mode == R_DBG_RECOIL_NONE) {
+			if (!r_debug_trace_ins_after (dbg)) {
+				eprintf ("trace_ins_after: failed");
+			}
+			dbg->session->reasontype = dbg->reason.type;
+			dbg->session->bp = bp;
+		}
+
 		dbg->steps++;
 		dbg->reason.type = R_DEBUG_REASON_STEP;
 	}
@@ -982,9 +1045,17 @@ R_API int r_debug_step_over(RDebug *dbg, int steps) {
 
 	if (dbg->h && dbg->h->step_over) {
 		for (; steps_taken < steps; steps_taken++) {
+			if (dbg->session && dbg->recoil_mode == R_DBG_RECOIL_NONE) {
+				dbg->session->cnum++;
+				dbg->session->maxcnum++;
+				r_debug_trace_ins_before (dbg);
+			}
 			if (!dbg->h->step_over (dbg)) {
 				return steps_taken;
 			}
+			if (dbg->session && dbg->recoil_mode == R_DBG_RECOIL_NONE) {
+				r_debug_trace_ins_after (dbg);
+			}
 		}
 		return steps_taken;
 	}
@@ -1035,105 +1106,35 @@ R_API int r_debug_step_over(RDebug *dbg, int steps) {
 	return steps_taken;
 }
 
-static ut64 get_prev_instr(RDebug *dbg, ut64 from, ut64 to) {
-	int i, ret, bsize = 256;
-	int inc;
-	ut64 prev = 0, at;
-	RAnalOp aop = {0};
-	const int mininstrsz = r_anal_archinfo (dbg->anal, R_ANAL_ARCHINFO_MIN_OP_SIZE);
-	const int minopcode = R_MAX (1, mininstrsz);
-	ut8 *buf = malloc (bsize);
+R_API bool r_debug_goto_cnum(RDebug *dbg, ut32 cnum) {
+	if (cnum > dbg->session->maxcnum) {
+		eprintf ("Error: out of cnum range\n");
+		return false;
+	}
+	dbg->session->cnum = cnum;
+	r_debug_session_restore_reg_mem (dbg, cnum);
 
-	if (!buf) {
-		eprintf ("Cannot allocate %d byte(s)\n", bsize);
-		free (buf);
-		return 0;
-	}
-	for (i = 0, at = from; at < to; at++, i++) {
-		if (i >= (bsize - 32)) {
-			i = 0;
-		}
-		if (!i) {
-			dbg->iob.read_at (dbg->iob.io, at, buf, bsize);
-		}
-		ret = r_anal_op (dbg->anal, &aop, at, buf + i, bsize - i, R_ANAL_OP_MASK_BASIC);
-		inc = ret - 1;
-		if (inc < 0) {
-			inc = minopcode;
-		}
-		prev = at;
-		i += inc;
-		at += inc;
-		r_anal_op_fini (&aop);
-	}
-	free (buf);
-	return prev;
+	return true;
 }
 
-// TODO: add <int steps> parameter for repetition like step() and step_over() do and change return type to int
-R_API bool r_debug_step_back(RDebug *dbg) {
-	ut64 pc, prev = 0, end;
-	RDebugSession *before;
+R_API int r_debug_step_back(RDebug *dbg, int steps) {
+	if (steps > dbg->session->cnum) {
+		steps = dbg->session->cnum;
+	}
+	if (!r_debug_goto_cnum (dbg, dbg->session->cnum - steps)) {
+		return -1;
+	}
+	return steps;
+}
 
-	if (r_debug_is_dead (dbg)) {
-		return false;
+R_API int r_debug_step_cnum(RDebug *dbg, int steps) {
+	if (steps > dbg->session->maxcnum - dbg->session->cnum) {
+		steps = dbg->session->maxcnum - dbg->session->cnum;
 	}
-	if (!dbg->anal || !dbg->reg) {
-		return false;
-	}
-	if (r_list_empty (dbg->sessions)) {
-		return false;
-	}
-	end = r_debug_reg_get (dbg, dbg->reg->name[R_REG_NAME_PC]);
 
-	/* Get previous state */
-	before = r_debug_session_get (dbg, dbg->sessions->tail);
-	if (!before) {
-		return false;
-	}
-#if 0
-	ut64 cnt = 0;
-	//eprintf ("before session (%d) 0x%08"PFMT64x"\n", before->key.id, before->key.addr);
+	r_debug_goto_cnum (dbg, dbg->session->cnum + steps);
 
-	/* Rollback to previous state */
-	r_debug_session_set (dbg, before);
-
-	pc = r_debug_reg_get (dbg, dbg->reg->name[R_REG_NAME_PC]);
-	//eprintf ("execute from 0x%08"PFMT64x" to 0x%08"PFMT64x"\n", pc, end);
-
-	/* Get the previous operation address.
-	 * XXX: too slow... */
-	for (;;) {
-		if (r_debug_is_dead (dbg)) {
-			return false;
-		}
-		pc = r_debug_reg_get (dbg, dbg->reg->name[R_REG_NAME_PC]);
-		if (pc == end) {
-			/* Reached the target address */
-			break;
-		}
-		prev = pc;
-		//eprintf ("executing 0x%08"PFMT64x"\n", pc);
-		if (cnt > CHECK_POINT_LIMIT) {
-			//eprintf ("Hit count limit %lld\n", cnt);
-			r_debug_session_add (dbg, NULL);
-			cnt = 0;
-		}
-		if (!r_debug_step (dbg, 1)) {
-			return false;
-		}
-		cnt++;
-	}
-#endif
-	/* Rollback to previous state and then run to the desired point */
-	r_debug_session_set (dbg, before);
-	pc = r_debug_reg_get (dbg, dbg->reg->name[R_REG_NAME_PC]);
-	prev = get_prev_instr (dbg, pc, end);
-	if (prev) {
-		eprintf ("continue until 0x%08"PFMT64x"\n", prev);
-		r_debug_continue_until_nonblock (dbg, prev);
-	}
-	return true;
+	return steps;
 }
 
 R_API int r_debug_continue_kill(RDebug *dbg, int sig) {
@@ -1144,11 +1145,46 @@ R_API int r_debug_continue_kill(RDebug *dbg, int sig) {
 	if (!dbg) {
 		return 0;
 	}
+
+	// If the debugger is not at the end of the changes
+	// Go to the end or the next breakpoint in the changes
+	if (dbg->session && dbg->session->cnum != dbg->session->maxcnum) {
+		bool has_bp = false;
+		RRegItem *ripc = r_reg_get (dbg->reg, dbg->reg->name[R_REG_NAME_PC], R_REG_TYPE_GPR);
+		RVector *vreg = ht_up_find (dbg->session->registers, ripc->offset | (ripc->arena << 16), NULL);
+		RDebugChangeReg *reg;
+		r_vector_foreach_prev (vreg, reg) {
+			if (reg->cnum <= dbg->session->cnum) {
+				continue;
+			}
+			has_bp = r_bp_get_in (dbg->bp, reg->data, R_BP_PROT_EXEC) != NULL;
+			if (has_bp) {
+				eprintf ("hit breakpoint at: 0x%" PFMT64x " cnum: %d\n", reg->data, reg->cnum);
+				r_debug_goto_cnum (dbg, reg->cnum);
+				return dbg->tid;
+			}
+		}
+
+		r_debug_goto_cnum (dbg, dbg->session->maxcnum);
+		return dbg->tid;
+	}
+
 repeat:
 	if (r_debug_is_dead (dbg)) {
 		return 0;
 	}
-	if (dbg->h && dbg->h->cont) {
+	if (dbg->session && dbg->trace_continue) {
+		while (!r_cons_is_breaked ()) {
+			if (r_debug_step (dbg, 1) != 1) {
+				break;
+			}
+			if (dbg->session->reasontype != R_DEBUG_REASON_STEP) {
+				break;
+			}
+		}
+		reason = dbg->session->reasontype;
+		bp = dbg->session->bp;
+	} else if (dbg->h && dbg->h->cont) {
 		/* handle the stage-2 of breakpoints */
 		if (!r_debug_recoil (dbg, R_DBG_RECOIL_CONTINUE)) {
 			return 0;
@@ -1156,138 +1192,147 @@ repeat:
 		/* tell the inferior to go! */
 		ret = dbg->h->cont (dbg, dbg->pid, dbg->tid, sig);
 		//XXX(jjd): why? //dbg->reason.signum = 0;
-
 		reason = r_debug_wait (dbg, &bp);
-		if (dbg->corebind.core) {
-			RCore *core = (RCore *)dbg->corebind.core;
-			RNum *num = core->num;
-			if (reason == R_DEBUG_REASON_COND) {
-				if (bp && bp->cond && dbg->corebind.cmd) {
-					dbg->corebind.cmd (dbg->corebind.core, bp->cond);
-				}
-				if (num->value) {
-					goto repeat;
-				}
+	} else {
+		return 0;
+	}
+
+	if (dbg->corebind.core) {
+		RCore *core = (RCore *)dbg->corebind.core;
+		RNum *num = core->num;
+		if (reason == R_DEBUG_REASON_COND) {
+			if (bp && bp->cond && dbg->corebind.cmd) {
+				dbg->corebind.cmd (dbg->corebind.core, bp->cond);
+			}
+			if (num->value) {
+				goto repeat;
 			}
 		}
-		if (reason == R_DEBUG_REASON_BREAKPOINT &&
-		   ((bp && !bp->enabled) || (!bp && !r_cons_is_breaked () && dbg->corebind.core &&
-			   		    dbg->corebind.cfggeti (dbg->corebind.core, "dbg.bpsysign")))) {
-			goto repeat;
-		}
+	}
+	if (reason == R_DEBUG_REASON_BREAKPOINT &&
+	   ((bp && !bp->enabled) || (!bp && !r_cons_is_breaked () && dbg->corebind.core &&
+					dbg->corebind.cfggeti (dbg->corebind.core, "dbg.bpsysign")))) {
+		goto repeat;
+	}
 
 #if __linux__
-		if (reason == R_DEBUG_REASON_NEW_PID && dbg->follow_child) {
+	if (reason == R_DEBUG_REASON_NEW_PID && dbg->follow_child) {
 #if DEBUGGER
-			/// if the plugin is not compiled link fails, so better do runtime linking
-			/// until this code gets fixed
-			static bool (*linux_attach_new_process) (RDebug *dbg, int pid) = NULL;
-			if (!linux_attach_new_process) {
-				linux_attach_new_process = r_lib_dl_sym (NULL, "linux_attach_new_process");
-			}
-			if (linux_attach_new_process) {
-				linux_attach_new_process (dbg, dbg->forked_pid);
-			}
-#endif
-			goto repeat;
+		/// if the plugin is not compiled link fails, so better do runtime linking
+		/// until this code gets fixed
+		static bool (*linux_attach_new_process) (RDebug *dbg, int pid) = NULL;
+		if (!linux_attach_new_process) {
+			linux_attach_new_process = r_lib_dl_sym (NULL, "linux_attach_new_process");
 		}
+		if (linux_attach_new_process) {
+			linux_attach_new_process (dbg, dbg->forked_pid);
+		}
+#endif
+		goto repeat;
+	}
 
-		if (reason == R_DEBUG_REASON_NEW_TID) {
-			ret = dbg->tid;
-			if (!dbg->trace_clone) {
-				goto repeat;
-			}
+	if (reason == R_DEBUG_REASON_NEW_TID) {
+		ret = dbg->tid;
+		if (!dbg->trace_clone) {
+			goto repeat;
 		}
+	}
 
-		if (reason == R_DEBUG_REASON_EXIT_TID) {
-			goto repeat;
-		}
+	if (reason == R_DEBUG_REASON_EXIT_TID) {
+		goto repeat;
+	}
 #endif
-		if (reason != R_DEBUG_REASON_DEAD) {
-			ret = dbg->tid;
-		}
+	if (reason != R_DEBUG_REASON_DEAD) {
+		ret = dbg->tid;
+	}
 #if __WINDOWS__
-		if (reason == R_DEBUG_REASON_NEW_LIB ||
-			reason == R_DEBUG_REASON_EXIT_LIB ||
-			reason == R_DEBUG_REASON_NEW_TID ||
-			reason == R_DEBUG_REASON_NONE ||
-			reason == R_DEBUG_REASON_EXIT_TID ) {
-			goto repeat;
-		}
+	if (reason == R_DEBUG_REASON_NEW_LIB ||
+		reason == R_DEBUG_REASON_EXIT_LIB ||
+		reason == R_DEBUG_REASON_NEW_TID ||
+		reason == R_DEBUG_REASON_NONE ||
+		reason == R_DEBUG_REASON_EXIT_TID ) {
+		goto repeat;
+	}
 #endif
-		if (reason == R_DEBUG_REASON_EXIT_PID) {
+	if (reason == R_DEBUG_REASON_EXIT_PID) {
 #if __WINDOWS__
-			dbg->pid = -1;
+		dbg->pid = -1;
 #elif __linux__
-			r_debug_bp_update (dbg);
-			r_bp_restore (dbg->bp, false); // (vdf) there has got to be a better way
+		r_debug_bp_update (dbg);
+		r_bp_restore (dbg->bp, false); // (vdf) there has got to be a better way
 #endif
-		}
+	}
 
-		/* if continuing killed the inferior, we won't be able to get
-		 * the registers.. */
-		if (reason == R_DEBUG_REASON_DEAD || r_debug_is_dead (dbg)) {
-			return 0;
-		}
+	/* if continuing killed the inferior, we won't be able to get
+	 * the registers.. */
+	if (reason == R_DEBUG_REASON_DEAD || r_debug_is_dead (dbg)) {
+		return 0;
+	}
 
-		/* if we hit a tracing breakpoint, we need to continue in
-		 * whatever mode the user desired. */
-		if (reason == R_DEBUG_REASON_TRACEPOINT) {
-			r_debug_step (dbg, 1);
+	/* if we hit a tracing breakpoint, we need to continue in
+	 * whatever mode the user desired. */
+	if (reason == R_DEBUG_REASON_TRACEPOINT) {
+		r_debug_step (dbg, 1);
+		goto repeat;
+	}
+
+	/* choose the thread that was returned from the continue function */
+	// XXX(jjd): there must be a cleaner way to do this...
+	if (ret != dbg->tid) {
+		r_debug_select (dbg, dbg->pid, ret);
+	}
+	sig = 0; // clear continuation after signal if needed
+
+	/* handle general signals here based on the return from the wait
+	 * function */
+	if (dbg->reason.signum != -1) {
+		int what = r_debug_signal_what (dbg, dbg->reason.signum);
+		if (what & R_DBG_SIGNAL_CONT) {
+			sig = dbg->reason.signum;
+			eprintf ("Continue into the signal %d handler\n", sig);
 			goto repeat;
-		}
-
-		/* choose the thread that was returned from the continue function */
-		// XXX(jjd): there must be a cleaner way to do this...
-		if (ret != dbg->tid) {
-			r_debug_select (dbg, dbg->pid, ret);
-		}
-		sig = 0; // clear continuation after signal if needed
-
-		/* handle general signals here based on the return from the wait
-		 * function */
-		if (dbg->reason.signum != -1) {
-			int what = r_debug_signal_what (dbg, dbg->reason.signum);
-			if (what & R_DBG_SIGNAL_CONT) {
-				sig = dbg->reason.signum;
-				eprintf ("Continue into the signal %d handler\n", sig);
+		} else if (what & R_DBG_SIGNAL_SKIP) {
+			// skip signal. requires skipping one instruction
+			ut8 buf[64];
+			RAnalOp op = {0};
+			ut64 pc = r_debug_reg_get (dbg, "PC");
+			dbg->iob.read_at (dbg->iob.io, pc, buf, sizeof (buf));
+			r_anal_op (dbg->anal, &op, pc, buf, sizeof (buf), R_ANAL_OP_MASK_BASIC);
+			if (op.size > 0) {
+				const char *signame = r_signal_to_string (dbg->reason.signum);
+				r_debug_reg_set (dbg, "PC", pc+op.size);
+				eprintf ("Skip signal %d handler %s\n",
+					dbg->reason.signum, signame);
 				goto repeat;
-			} else if (what & R_DBG_SIGNAL_SKIP) {
-				// skip signal. requires skipping one instruction
-				ut8 buf[64];
-				RAnalOp op = {0};
+			} else {
 				ut64 pc = r_debug_reg_get (dbg, "PC");
-				dbg->iob.read_at (dbg->iob.io, pc, buf, sizeof (buf));
-				r_anal_op (dbg->anal, &op, pc, buf, sizeof (buf), R_ANAL_OP_MASK_BASIC);
-				if (op.size > 0) {
-					const char *signame = r_signal_to_string (dbg->reason.signum);
-					r_debug_reg_set (dbg, "PC", pc+op.size);
-					eprintf ("Skip signal %d handler %s\n",
-						dbg->reason.signum, signame);
-					goto repeat;
-				} else {
-					ut64 pc = r_debug_reg_get (dbg, "PC");
-					eprintf ("Stalled with an exception at 0x%08"PFMT64x"\n", pc);
-				}
+				eprintf ("Stalled with an exception at 0x%08"PFMT64x"\n", pc);
 			}
 		}
 	}
 #if __WINDOWS__
 	r_cons_break_pop ();
-#elif __linux__
-	// Letting threads continue after the debugger breaks is currently problematic in linux
-	if (dbg->continue_all_threads) {
-		r_debug_stop (dbg);
-	}
 #endif
+
 	// Unset breakpoints before leaving
 	if (reason != R_DEBUG_REASON_BREAKPOINT) {
 		r_bp_restore (dbg->bp, false);
 	}
+
+	// Add a checkpoint at stops
+	if (dbg->session && !dbg->trace_continue) {
+		dbg->session->cnum++;
+		dbg->session->maxcnum++;
+		r_debug_add_checkpoint (dbg);
+	}
+
 	return ret;
 }
 
 R_API int r_debug_continue(RDebug *dbg) {
+	if (dbg->pid < 0) {
+		return -1;
+	}
 	return r_debug_continue_kill (dbg, 0); //dbg->reason.signum);
 }
 
@@ -1400,78 +1445,36 @@ R_API int r_debug_continue_until_nonblock(RDebug *dbg, ut64 addr) {
 }
 
 R_API bool r_debug_continue_back(RDebug *dbg) {
-	RBreakpointItem *prev = NULL;
-	ut64 pc;
-	if (!dbg) {
-		return false;
-	}
-	if (!dbg->anal || !dbg->reg) {
-		return false;
-	}
-	if (r_debug_is_dead (dbg)) {
-		return false;
-	}
-	if (r_list_empty (dbg->sessions)) {
-		return false;
-	}
+	int cnum;
+	bool has_bp = false;
 
-	if (!dbg->sessions) {
+	RRegItem *ripc = r_reg_get (dbg->reg, dbg->reg->name[R_REG_NAME_PC], R_REG_TYPE_GPR);
+	RVector *vreg = ht_up_find (dbg->session->registers, ripc->offset | (ripc->arena << 16), NULL);
+	if (!vreg) {
+		eprintf ("Error: cannot find PC change vector");
 		return false;
 	}
-	/* Get previous state */
-	RListIter *iter = dbg->sessions->head;
-	if (!iter || !iter->data) {
-		return false;
-	}
-	RDebugSession *before = iter->data; //XXX: currently use first session.
-	ut64 end_addr = r_debug_reg_get (dbg, dbg->reg->name[R_REG_NAME_PC]);
-	//eprintf ("before session (%d) 0x%08"PFMT64x"=> to 0x%08"PFMT64x"\n", before->key.id, before->key.addr, end_addr);
-
-	/* Rollback to previous state */
-	r_debug_session_set (dbg, before);
-
-	/* ### Get previous breakpoint ### */
-	// Firstly set the breakpoint at end address
-	bool has_bp = r_bp_get_in (dbg->bp, end_addr, R_BP_PROT_EXEC) != NULL;
-	if (!has_bp) {
-		r_bp_add_sw (dbg->bp, end_addr, dbg->bpsize, R_BP_PROT_EXEC);
-	}
-
-	// Continue until end_addr
-	for (;;) {
-		if (r_debug_is_dead (dbg)) {
+	RDebugChangeReg *reg;
+	r_vector_foreach_prev (vreg, reg) {
+		if (reg->cnum >= dbg->session->cnum) {
+			continue;
+		}
+		has_bp = r_bp_get_in (dbg->bp, reg->data, R_BP_PROT_EXEC) != NULL;
+		if (has_bp) {
+			cnum = reg->cnum;
+			eprintf ("hit breakpoint at: 0x%" PFMT64x " cnum: %d\n", reg->data, reg->cnum);
 			break;
 		}
-		pc = r_debug_reg_get (dbg, dbg->reg->name[R_REG_NAME_PC]);
-		if (pc == end_addr) {
-			break;
-		}
-		prev = r_bp_get_at (dbg->bp, pc);
-		r_debug_continue (dbg);
 	}
-	// Clean up if needed
-	if (!has_bp) {
-		r_bp_del (dbg->bp, end_addr);
-	}
-	if (!prev) {
-		return false;
-	}
-	//eprintf ("prev->addr = 0x%08"PFMT64x"\n", prev->addr);
-	/* Now we got previous breakpoint.
-	 * ### Continue until prev breakpoint ### */
 
-	/* Rollback to previous state again */
-	r_debug_session_set (dbg, before);
-	for (;;) {
-		if (r_debug_is_dead (dbg)) {
-			break;
+	if (has_bp) {
+		r_debug_goto_cnum (dbg, cnum);
+	} else {
+		if (dbg->session->maxcnum > 0) {
+			r_debug_goto_cnum (dbg, 0);
 		}
-		pc = r_debug_reg_get (dbg, dbg->reg->name[R_REG_NAME_PC]);
-		if (prev == r_bp_get_at (dbg->bp, pc)) {
-			break;
-		}
-		r_debug_continue (dbg);
 	}
+
 	return true;
 }
 
@@ -1483,7 +1486,7 @@ static int show_syscall(RDebug *dbg, const char *sysreg) {
 	reg = (int)r_debug_reg_get (dbg, sysreg);
 	si = r_syscall_get (dbg->anal->syscall, reg, -1);
 	if (si) {
-		sysname = si->name? si->name: "unknown";
+		sysname = r_str_get_fail (si->name, "unknown");
 		args = si->args;
 	} else {
 		sysname = "unknown";
@@ -1634,7 +1637,7 @@ R_API bool r_debug_is_dead(RDebug *dbg) {
 	if (!strcmp (dbg->h->name, "io")) {
 		return false;
 	}
-	bool is_dead = (dbg->pid == -1 && strncmp (dbg->h->name, "gdb", 3)) || (dbg->reason.type == R_DEBUG_REASON_DEAD);
+	bool is_dead = (dbg->pid < 0 && strncmp (dbg->h->name, "gdb", 3)) || (dbg->reason.type == R_DEBUG_REASON_DEAD);
 	if (dbg->pid > 0 && dbg->h && dbg->h->kill) {
 		is_dead = !dbg->h->kill (dbg, dbg->pid, false, 0);
 	}
@@ -1689,7 +1692,7 @@ R_API ut64 r_debug_get_baddr(RDebug *dbg, const char *file) {
 	if (pid < 0 || tid < 0) {
 		return 0LL;
 	}
-	if (r_debug_attach (dbg, pid) == -1) {
+	if (!r_debug_attach (dbg, pid)) {
 		return 0LL;
 	}
 #if __WINDOWS__
@@ -1704,13 +1707,15 @@ R_API ut64 r_debug_get_baddr(RDebug *dbg, const char *file) {
 	r_debug_select (dbg, pid, tid);
 	r_debug_map_sync (dbg);
 	char *abspath = r_sys_pid_to_path (pid);
+	if (file) {
 #if !__WINDOWS__
-	if (!abspath) {
-		abspath = r_file_abspath (file);
-	}
+		if (!abspath) {
+			abspath = r_file_abspath (file);
+		}
 #endif
-	if (!abspath) {
-		abspath = strdup (file);
+		if (!abspath) {
+			abspath = strdup (file);
+		}
 	}
 	if (abspath) {
 		r_list_foreach (dbg->maps, iter, map) {
