@@ -1,11 +1,11 @@
-/* radare - LGPL - Copyright 2013-2017 pancake */
+/* radare - LGPL - Copyright 2013-2021 pancake */
 // r2 -Desil ls
 
 #include <r_asm.h>
 #include <r_debug.h>
 
 #if 0
-static int is_io_esil(RDebug *dbg) {
+static bool is_io_esil(RDebug *dbg) {
 	RIODesc *d = dbg->iob.io->desc;
 	if (d && d->plugin && d->plugin->name)
 		if (!strcmp ("esil", d->plugin->name))
@@ -14,12 +14,12 @@ static int is_io_esil(RDebug *dbg) {
 }
 #endif
 
-static int __esil_step_over(RDebug *dbg) {
+static bool __esil_step_over(RDebug *dbg) {
 	eprintf ("TODO: ESIL STEP OVER\n");
 	return true;
 }
 
-static int __esil_step(RDebug *dbg) {
+static bool __esil_step(RDebug *dbg) {
 	int oplen;
 	ut8 buf[64];
 	ut64 pc = 0LL; // getreg("pc")
@@ -45,29 +45,29 @@ static int __esil_step(RDebug *dbg) {
 	return true;
 }
 
-static int __esil_init(RDebug *dbg) {
+static bool __esil_init(RDebug *dbg) {
 	dbg->tid = dbg->pid = 1;
 	// aeim
 	// aei
 	return true;
 }
 
-static int __esil_continue(RDebug *dbg, int pid, int tid, int sig) {
+static bool __esil_continue(RDebug *dbg, int pid, int tid, int sig) {
 	eprintf ("TODO continue\n");
 	return true;
 }
 
-static int __esil_continue_syscall(RDebug *dbg, int pid, int num) {
+static bool __esil_continue_syscall(RDebug *dbg, int pid, int num) {
 	eprintf ("TODO: esil continue until syscall\n");
 	return true;
 }
 
-static int __esil_wait(RDebug *dbg, int pid) {
+static RDebugReasonType __esil_wait(RDebug *dbg, int pid) {
 	/* do nothing */
-	return true;
+	return R_DEBUG_REASON_NONE;
 }
 
-static int __esil_attach(RDebug *dbg, int pid) {
+static bool __esil_attach(RDebug *dbg, int pid) {
 	eprintf ("OK attach\n");
 	return true;
 #if 0
@@ -84,7 +84,7 @@ eprintf ("input = %llx\n", o->bfvm->input);
 	return true;
 }
 
-static int __esil_detach(RDebug *dbg, int pid) {
+static bool __esil_detach(RDebug *dbg, int pid) {
 	// reset vm?
 	return true;
 }
@@ -95,6 +95,7 @@ static char *__esil_reg_profile(RDebug *dbg) {
 			"=PC	pc\n"
 			"=SP	esp\n"
 			"=BP	ptr\n"
+			"=A0	rax\n"
 			"gpr	rax	.32	0	0\n"
 			"gpr	pc	.32	0	0\n"
 			"gpr	ptr	.32	4	0\n"
