@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include <r_userconf.h>
 #include <r_debug.h>
@@ -16,9 +16,10 @@
 #include "native/drx.c" // x86 specific
 #include "r_cons.h"
 
-static int r_debug_native_continue (RDebug *dbg, int pid, int tid, int sig);
-static int r_debug_native_reg_read (RDebug *dbg, int type, ut8 *buf, int size);
-static int r_debug_native_reg_write (RDebug *dbg, int type, const ut8* buf, int size);
+static bool r_debug_native_continue(RDebug *dbg, int pid, int tid, int sig);
+static int r_debug_native_reg_read(RDebug *dbg, int type, ut8 *buf, int size);
+static int r_debug_native_reg_write(RDebug *dbg, int type, const ut8* buf, int size);
+struct r_debug_desc_plugin_t r_debug_desc_plugin_native;
 
 #include "native/bt.c"
 
@@ -91,20 +92,20 @@ R_API RList *r_w32_dbg_maps(RDebug *);
 /* begin of debugger code */
 #if DEBUGGER
 
-#if !__WINDOWS__ || (!__APPLE__ && defined(WAIT_ON_ALL_CHILDREN))
+#if !__WINDOWS__ && !(__linux__ && !defined(WAIT_ON_ALL_CHILDREN)) && !__APPLE__
 static int r_debug_handle_signals(RDebug *dbg) {
 #if __KFBSD__
 	return bsd_handle_signals (dbg);
 #else
-	return -1;
+	eprintf ("Warning: signal handling is not supported on this platform\n");
+	return 0;
 #endif
 }
 #endif
 
-//this is temporal
 #if __APPLE__ || __linux__
 
-static char *r_debug_native_reg_profile (RDebug *dbg) {
+static char *r_debug_native_reg_profile(RDebug *dbg) {
 #if __APPLE__
 	return xnu_reg_profile (dbg);
 #elif __linux__
@@ -116,7 +117,7 @@ static char *r_debug_native_reg_profile (RDebug *dbg) {
 #include "native/reg.c" // x86 specific
 
 #endif
-static int r_debug_native_step (RDebug *dbg) {
+static bool r_debug_native_step(RDebug *dbg) {
 #if __APPLE__
 	return xnu_step (dbg);
 #elif __WINDOWS__
@@ -133,12 +134,10 @@ static int r_debug_native_step (RDebug *dbg) {
 #endif
 }
 
-// return thread id
-static int r_debug_native_attach (RDebug *dbg, int pid) {
-#if 0
-	if (!dbg || pid == dbg->pid)
-		return dbg->tid;
-#endif
+static bool r_debug_native_attach(RDebug *dbg, int pid) {
+	if (pid < 0) {
+		return false;
+	}
 #if __APPLE__
 	return xnu_attach (dbg, pid);
 #elif __WINDOWS__
@@ -149,18 +148,18 @@ static int r_debug_native_attach (RDebug *dbg, int pid) {
 	if (ptrace (PT_ATTACH, pid, 0, 0) != -1) {
 		perror ("ptrace (PT_ATTACH)");
 	}
-	return pid;
+	return true;
 #else
 	int ret = ptrace (PTRACE_ATTACH, pid, 0, 0);
 	if (ret != -1) {
 		eprintf ("Trying to attach to %d\n", pid);
 		perror ("ptrace (PT_ATTACH)");
 	}
-	return pid;
+	return true;
 #endif
 }
 
-static int r_debug_native_detach (RDebug *dbg, int pid) {
+static bool r_debug_native_detach(RDebug *dbg, int pid) {
 #if __APPLE__
 	return xnu_detach (dbg, pid);
 #elif __WINDOWS__
@@ -172,17 +171,19 @@ static int r_debug_native_detach (RDebug *dbg, int pid) {
 #endif
 }
 
-static int r_debug_native_select(RDebug *dbg, int pid, int tid) {
+#if __WINDOWS__ || __linux__
+static bool r_debug_native_select(RDebug *dbg, int pid, int tid) {
 #if __WINDOWS__
 	return w32_select (dbg, pid, tid);
 #elif __linux__
 	return linux_select (dbg, pid, tid);
 #else
-	return -1;
+	return false;
 #endif
 }
+#endif
 
-static int r_debug_native_continue_syscall (RDebug *dbg, int pid, int num) {
+static bool r_debug_native_continue_syscall(RDebug *dbg, int pid, int num) {
 // XXX: num is ignored
 #if __linux__
 	linux_set_options (dbg, pid);
@@ -193,7 +194,7 @@ static int r_debug_native_continue_syscall (RDebug *dbg, int pid, int num) {
 	return ptrace (PTRACE_SYSCALL, pid, (void*)(size_t)pc, 0) == 0;
 #else
 	eprintf ("TODO: continue syscall not implemented yet\n");
-	return -1;
+	return false;
 #endif
 }
 
@@ -207,21 +208,16 @@ static void interrupt_process(RDebug *dbg) {
 
 static int r_debug_native_stop(RDebug *dbg) {
 #if __linux__
+	// Stop all running threads except the thread reported by waitpid
 	return linux_stop_threads (dbg, dbg->reason.tid);
 #else
 	return 0;
 #endif
 }
 
-/* TODO: specify thread? */
-/* TODO: must return true/false */
-static int r_debug_native_continue(RDebug *dbg, int pid, int tid, int sig) {
+static bool r_debug_native_continue(RDebug *dbg, int pid, int tid, int sig) {
 #if __APPLE__
-	bool ret = xnu_continue (dbg, pid, tid, sig);
-	if (!ret) {
-		return -1;
-	}
-	return tid;
+	return xnu_continue (dbg, pid, tid, sig);
 #elif __WINDOWS__
 	return w32_continue (dbg, pid, tid, sig);
 #elif __BSD__
@@ -229,45 +225,34 @@ static int r_debug_native_continue(RDebug *dbg, int pid, int tid, int sig) {
 	ut64 pc = r_debug_reg_get (dbg, "PC");
 	return ptrace (PTRACE_CONT, pid, (void*)(size_t)pc, (int)(size_t)data) == 0;
 #else
-	int contsig = dbg->reason.signum;
 	int ret = -1;
-
-	if (sig != -1) {
-		contsig = sig;
+	if (sig == -1) {
+		   sig = dbg->reason.signum;
 	}
 	/* SIGINT handler for attached processes: dbg.consbreak (disabled by default) */
 	if (dbg->consbreak) {
 		r_cons_break_push ((RConsBreak)interrupt_process, dbg);
 	}
-
-	if (dbg->continue_all_threads && dbg->n_threads) {
-		RList *list = dbg->threads;
+	if (dbg->continue_all_threads && dbg->n_threads && dbg->threads) {
 		RDebugPid *th;
 		RListIter *it;
-
-		if (list) {
-			r_list_foreach (list, it, th) {
-				if (th->pid) {
-					ret = r_debug_ptrace (dbg, PTRACE_CONT, th->pid, NULL, (r_ptrace_data_t)(size_t)contsig);
-					if (ret) {
-						perror ("PTRACE_CONT");
-					}
-				}
+		r_list_foreach (dbg->threads, it, th) {
+			ret = r_debug_ptrace (dbg, PTRACE_CONT, th->pid, 0, 0);
+			if (ret) {
+				eprintf ("Error: (%d) is running or dead.\n", th->pid);
 			}
 		}
-	}
-	else {
-		ret = r_debug_ptrace (dbg, PTRACE_CONT, tid, NULL, (r_ptrace_data_t)(size_t)contsig);
+	} else {
+		ret = r_debug_ptrace (dbg, PTRACE_CONT, tid, NULL, (r_ptrace_data_t)(size_t) sig);
 		if (ret) {
-			perror ("PTRACE_CONT");
+			r_sys_perror ("PTRACE_CONT");
 		}
 	}
-	//return ret >= 0 ? tid : false;
-	return tid;
+	return ret >= 0;
 #endif
 }
 
-static RDebugInfo* r_debug_native_info (RDebug *dbg, const char *arg) {
+static RDebugInfo* r_debug_native_info(RDebug *dbg, const char *arg) {
 #if __APPLE__
 	return xnu_info (dbg, arg);
 #elif __WINDOWS__
@@ -291,9 +276,7 @@ static bool tracelib(RDebug *dbg, const char *mode, PLIB_ITEM item) {
 		case 'u': needle = dbg->glob_unlibs; break;
 		}
 	}
-	//eprintf ("(%d) %sing library at %p (%s) %s\n", item->pid, mode,
-		//item->BaseOfDll, item->Path, item->Name);
-	r_cons_printf ("(%d) %sing library at %p (%s) %s\n", item->pid, mode,
+	r_cons_printf ("(%d) %sing library at 0x%p (%s) %s\n", item->pid, mode,
 		item->BaseOfDll, item->Path, item->Name);
 	r_cons_flush ();
 	if (needle && strlen (needle)) {
@@ -315,7 +298,7 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 	// require switching to the event's thread that shouldn't bother the user
 	int orig_tid = dbg->tid;
 	bool restore_thread = false;
-	RIOW32Dbg *rio = dbg->user;
+	RW32Dw *wrap = dbg->user;
 
 	if (pid == -1) {
 		eprintf ("ERROR: r_debug_native_wait called with pid -1\n");
@@ -372,7 +355,7 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 		RDebugInfo *r = r_debug_native_info (dbg, "");
 		if (r && r->thread) {
 			PTHREAD_ITEM item = r->thread;
-			r_cons_printf ("(%d) Created thread %d (start @ %p)\n", item->pid, item->tid, item->lpStartAddress);
+			r_cons_printf ("(%d) Created thread %d (start @ %p) (teb @ %p)\n", item->pid, item->tid, item->lpStartAddress, item->lpThreadLocalBase);
 			r_cons_flush ();
 
 			r_debug_info_free (r);
@@ -382,7 +365,7 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 		RDebugInfo *r = r_debug_native_info (dbg, "");
 		if (r && r->thread) {
 			PTHREAD_ITEM item = r->thread;
-			r_cons_printf ("(%d) Finished thread %d Exit code %d\n", (ut32)item->pid, (ut32)item->tid, (ut32)item->dwExitCode);
+			r_cons_printf ("(%d) Finished thread %d Exit code %lu\n", (ut32)item->pid, (ut32)item->tid, item->dwExitCode);
 			r_cons_flush ();
 
 			r_debug_info_free (r);
@@ -394,7 +377,7 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 		RDebugInfo *r = r_debug_native_info (dbg, "");
 		if (r && r->thread) {
 			PTHREAD_ITEM item = r->thread;
-			r_cons_printf ("(%d) Finished process with exit code %d\n", dbg->main_pid, item->dwExitCode);
+			r_cons_printf ("(%d) Finished process with exit code %lu\n", dbg->main_pid, item->dwExitCode);
 			r_cons_flush ();
 			r_debug_info_free (r);
 		}
@@ -411,17 +394,18 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 		}
 		// DebugProcessBreak creates a new thread that will trigger a breakpoint. We record the
 		// tid here to ignore it once the breakpoint is hit.
-		rio->break_tid = dbg->tid;
+		wrap->break_tid = dbg->tid;
 		restore_thread = true;
-	} else if (reason == R_DEBUG_REASON_BREAKPOINT && dbg->tid == rio->break_tid) {
-		rio->break_tid = -2;
+	} else if (reason == R_DEBUG_REASON_BREAKPOINT && dbg->tid == wrap->break_tid) {
+		wrap->break_tid = -2;
 		reason = R_DEBUG_REASON_NONE;
 		restore_thread = true;
 	}
 
 	if (restore_thread) {
 		// Attempt to return to the original thread after handling the event
-		dbg->tid = w32_select(dbg, dbg->pid, orig_tid);
+		w32_select (dbg, dbg->pid, orig_tid);
+		// dbg->tid = w32_select (dbg, dbg->pid, orig_tid);
 		if (dbg->tid == -1) {
 			dbg->pid = -1;
 			reason = R_DEBUG_REASON_DEAD;
@@ -448,14 +432,6 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 	}
 
 	reason = linux_dbg_wait (dbg, dbg->tid);
-	if (reason == R_DEBUG_REASON_EXIT_TID) {
-		RDebugInfo *r = r_debug_native_info (dbg, "");
-		if (r) {
-			eprintf ("(%d) Finished thread %d Exit code\n", r->pid, r->tid);
-			r_debug_info_free (r);
-		}
-	}
-
 	dbg->reason.type = reason;
 	return reason;
 }
@@ -469,6 +445,9 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 	}
 
 #if __APPLE__
+	if (pid < 0) {
+		return R_DEBUG_REASON_ERROR;
+	}
 	r_cons_break_push (NULL, NULL);
 	do {
 		reason = xnu_wait (dbg, pid);
@@ -502,20 +481,16 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 		r_sys_perror ("waitpid");
 		return R_DEBUG_REASON_ERROR;
 	}
-
-	//eprintf ("r_debug_native_wait: status=%d (0x%x) (return=%d)\n", status, status, ret);
-
 #ifdef WAIT_ON_ALL_CHILDREN
 	if (ret != pid) {
 		reason = R_DEBUG_REASON_NEW_PID;
-		eprintf ("switching to pid %d\n", ret);
-		r_debug_select (dbg, ret, ret);
+		r_debug_select (dbg, ret, -1);
 	}
 #endif // WAIT_ON_ALL_CHILDREN
 	// TODO: switch status and handle reasons here
 	// FIXME: Remove linux handling from this function?
 #if __linux__ && defined(PT_GETEVENTMSG)
-	reason = linux_ptrace_event (dbg, pid, status);
+	reason = linux_ptrace_event (dbg, pid, status, true);
 #endif // __linux__
 
 	/* propagate errors */
@@ -549,7 +524,7 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 #if __OpenBSD__ || __NetBSD__
 			reason = R_DEBUG_REASON_BREAKPOINT;
 #else
-			if (!r_debug_handle_signals (dbg)) {
+			if (r_debug_handle_signals (dbg) != 0) {
 				return R_DEBUG_REASON_ERROR;
 			}
 			reason = dbg->reason.type;
@@ -623,7 +598,7 @@ static RList *r_debug_native_pids(RDebug *dbg, int pid) {
 #elif __linux__
 	return linux_pid_list (pid, list);
 #else /* rest is BSD */
-	return bsd_pid_list (dbg, list);
+	return bsd_pid_list (dbg, pid, list);
 #endif
 	return list;
 }
@@ -648,7 +623,7 @@ static RList *r_debug_native_threads (RDebug *dbg, int pid) {
 #if __sun || __NetBSD__ || __KFBSD__ || __OpenBSD__ || __DragonFly__
 
 //Function to read register from Linux, BSD, Android systems
-static int bsd_reg_read (RDebug *dbg, int type, ut8* buf, int size) {
+static int bsd_reg_read(RDebug *dbg, int type, ut8* buf, int size) {
 	int showfpu = false;
 	int pid = dbg->pid;
 	int ret;
@@ -680,7 +655,7 @@ static int bsd_reg_read (RDebug *dbg, int type, ut8* buf, int size) {
 	case R_REG_TYPE_GPR:
 		{
 		R_DEBUG_REG_T regs;
-		memset (&regs, 0, sizeof(regs));
+		memset (&regs, 0, sizeof (regs));
 		memset (buf, 0, size);
 		#if __NetBSD__ || __OpenBSD__
 			ret = ptrace (PTRACE_GETREGS, pid, (caddr_t)&regs, sizeof (regs));
@@ -694,8 +669,12 @@ static int bsd_reg_read (RDebug *dbg, int type, ut8* buf, int size) {
 		// process exists still.. is because there's a
 		// missing call to 'wait'. and the process is not
 		// yet available to accept more ptrace queries.
-		if (ret != 0) return false;
-		if (sizeof(regs) < size) size = sizeof(regs);
+		if (ret != 0) {
+			return false;
+		}
+		if (sizeof (regs) < size) {
+			size = sizeof (regs);
+		}
 		memcpy (buf, &regs, size);
 		return sizeof(regs);
 		}
@@ -789,8 +768,24 @@ static int io_perms_to_prot (int io_perms) {
 	return prot_perms;
 }
 
+#if __linux__
+static int thp_mode(void) {
+	const char *thp = "/sys/kernel/mm/transparent_hugepage/enabled";
+	int ret = 0;
+	char *val = r_file_slurp (thp, NULL);
+	if (val) {
+		if (strstr (val, "[madvise]")) {
+			ret = 1;
+		} else if (strstr (val, "[always]")) {
+			ret = 2;
+		}
+		free (val);
+	}
+	return ret;
+}
+#endif
 
-static int linux_map_thp (RDebug *dbg, ut64 addr, int size) {
+static int linux_map_thp(RDebug *dbg, ut64 addr, int size) {
 #if !defined(__ANDROID__) && defined(MADV_HUGEPAGE)
 	RBuffer *buf = NULL;
 	char code[1024];
@@ -803,19 +798,19 @@ static int linux_map_thp (RDebug *dbg, ut64 addr, int size) {
 	// In architectures where radare is supported, arm and x86, it is 2MB
 	const size_t thpsize = 1<<21;
 
-	if ((size%thpsize)) {
+	if (size % thpsize) {
 		eprintf ("size not a power of huge pages size\n");
 		return false;
 	}
-
+#if __linux__
 	// In always mode, is more into mmap syscall level
 	// even though the address might not have the 'hg'
 	// vmflags
-	if (r_sys_thp_mode() != 1) {
+	if (thp_mode () != 1) {
 		eprintf ("transparent huge page mode is not in madvise mode\n");
 		return false;
 	}
-
+#endif
 	int num = r_syscall_get_num (dbg->anal->syscall, "madvise");
 
 	snprintf (code, sizeof (code),
@@ -1140,7 +1135,7 @@ static RList *r_debug_native_map_get (RDebug *dbg) {
 	return list;
 }
 
-static RList *r_debug_native_modules_get (RDebug *dbg) {
+static RList *r_debug_native_modules_get(RDebug *dbg) {
 	char *lastname = NULL;
 	RDebugMap *map;
 	RListIter *iter, *iter2;
@@ -1167,6 +1162,9 @@ static RList *r_debug_native_modules_get (RDebug *dbg) {
 	r_list_foreach_safe (list, iter, iter2, map) {
 		const char *file = map->file;
 		if (!map->file) {
+			if (!map->name) {
+				map->name = strdup ("");
+			}
 			file = map->file = strdup (map->name);
 		}
 		must_delete = true;
@@ -1223,18 +1221,19 @@ static bool r_debug_native_kill(RDebug *dbg, int pid, int tid, int sig) {
 	return ret;
 }
 
-struct r_debug_desc_plugin_t r_debug_desc_plugin_native;
-static int r_debug_native_init (RDebug *dbg) {
+static bool r_debug_native_init(RDebug *dbg) {
 	dbg->h->desc = r_debug_desc_plugin_native;
 #if __WINDOWS__
-	return w32_init (dbg);
-#else
-	return true;
+	r_w32_init ();
+	if (!dbg->user && dbg->iob.io->dbgwrap) {
+		dbg->user = (RW32Dw *)dbg->iob.io->dbgwrap;
+	}
 #endif
+	return true;
 }
 
-static void sync_drx_regs (RDebug *dbg, drxt *regs, size_t num_regs) {
 #if __i386__ || __x86_64__
+static void sync_drx_regs (RDebug *dbg, drxt *regs, size_t num_regs) {
 	/* sanity check, we rely on this assumption */
 	if (num_regs != NUM_DRX_REGISTERS) {
 		eprintf ("drx: Unsupported number of registers for get_debug_regs\n");
@@ -1254,13 +1253,11 @@ static void sync_drx_regs (RDebug *dbg, drxt *regs, size_t num_regs) {
 */
 	regs[6] = r_reg_getv (R, "dr6");
 	regs[7] = r_reg_getv (R, "dr7");
-#else
-	eprintf ("drx sync_drx_regs: Unsupported platform\n");
-#endif
 }
+#endif
 
-static void set_drx_regs (RDebug *dbg, drxt *regs, size_t num_regs) {
 #if __i386__ || __x86_64__
+static void set_drx_regs (RDebug *dbg, drxt *regs, size_t num_regs) {
 	/* sanity check, we rely on this assumption */
 	if (num_regs != NUM_DRX_REGISTERS){
 		eprintf ("drx: Unsupported number of registers for get_debug_regs\n");
@@ -1274,12 +1271,10 @@ static void set_drx_regs (RDebug *dbg, drxt *regs, size_t num_regs) {
 	r_reg_setv (R, "dr3", regs[3]);
 	r_reg_setv (R, "dr6", regs[6]);
 	r_reg_setv (R, "dr7", regs[7]);
-#else
-	eprintf ("drx set_drx_regs: Unsupported platform\n");
-#endif
 }
+#endif
 
-static int r_debug_native_drx (RDebug *dbg, int n, ut64 addr, int sz, int rwx, int g, int api_type) {
+static int r_debug_native_drx(RDebug *dbg, int n, ut64 addr, int sz, int rwx, int g, int api_type) {
 #if __i386__ || __x86_64__
 	int retval = false;
 	drxt regs[NUM_DRX_REGISTERS] = {0};
@@ -1315,7 +1310,7 @@ static int r_debug_native_drx (RDebug *dbg, int n, ut64 addr, int sz, int rwx, i
 
 	return retval;
 #else
-	eprintf ("drx: Unsupported platform\n");
+	eprintf ("drx: registers only available on x86. Use dbH for native hardware breakpoints and watchpoints.\n");
 #endif
 	return false;
 }
@@ -1349,29 +1344,17 @@ static bool ll_arm32_hwbp_set(pid_t pid, ut64 addr, int size, int wp, int type)
 	return ptrace (PTRACE_SETHBPREGS, pid, -2, &control) != -1;
 }
 
-static bool arm32_hwbp_add (RDebug *dbg, RBreakpoint* bp, RBreakpointItem *b) {
+static bool arm32_hwbp_add(RDebug *dbg, RBreakpoint* bp, RBreakpointItem *b) {
 	return ll_arm32_hwbp_set (dbg->pid, b->addr, b->size, 0, 1 | 2 | 4);
 }
 
-static bool arm32_hwbp_del (RDebug *dbg, RBreakpoint *bp, RBreakpointItem *b) {
+static bool arm32_hwbp_del(RDebug *dbg, RBreakpoint *bp, RBreakpointItem *b) {
 	return false; // TODO: hwbp.del not yetimplemented
 }
-#else
-static bool ll_arm32_hwbp_set(pid_t pid, ut64 addr, int size, int wp, int type) {
-	return false;
-}
-
-static bool arm32_hwbp_add (RDebug *dbg, RBreakpoint* bp, RBreakpointItem *b) {
-	return false;
-}
-
-static bool arm32_hwbp_del (RDebug *dbg, RBreakpoint *bp, RBreakpointItem *b) {
-	return false;
-}
 #endif // PTRACE_GETHWBPREGS
 #endif // __arm
 
-#if __arm64__ || __aarch64__
+#if (__arm64__ || __aarch64__) && defined(PTRACE_GETREGSET)
 // type = 2 = write
 //static volatile uint8_t var[96] __attribute__((__aligned__(32)));
 
@@ -1449,22 +1432,24 @@ static bool arm64_hwbp_del (RDebug *dbg, RBreakpoint *bp, RBreakpointItem *b) {
  * we let the caller handle the work.
  */
 static int r_debug_native_bp(RBreakpoint *bp, RBreakpointItem *b, bool set) {
-	RDebug *dbg = bp->user;
 	if (b && b->hw) {
 #if __i386__ || __x86_64__
+		RDebug *dbg = bp->user;
 		return set
 			? drx_add (dbg, bp, b)
 			: drx_del (dbg, bp, b);
-#elif __arm64__ || __aarch64__
-#if __linux__
+#elif (__arm64__ || __aarch64__) && __linux__
+		RDebug *dbg = bp->user;
 		return set
 			? arm64_hwbp_add (dbg, bp, b)
 			: arm64_hwbp_del (dbg, bp, b);
-#endif
-#elif __arm__
+#elif __arm__ && __linux__
+		RDebug *dbg = bp->user;
 		return set
 			? arm32_hwbp_add (dbg, bp, b)
 			: arm32_hwbp_del (dbg, bp, b);
+#else
+#warning r_debug_native_bp not implemented for this platform
 #endif
 	}
 	return false;
@@ -1472,13 +1457,17 @@ static int r_debug_native_bp(RBreakpoint *bp, RBreakpointItem *b, bool set) {
 
 #if __APPLE__
 
-static int getMaxFiles() {
+#if TARGET_OS_IPHONE || __POWERPC__
+	// nothing to do
+#else
+static int getMaxFiles(void) {
 	struct rlimit limit;
 	if (getrlimit (RLIMIT_NOFILE, &limit) != 0) {
 		return 1024;
 	}
 	return limit.rlim_cur;
 }
+#endif
 
 static RList *xnu_desc_list (int pid) {
 #if TARGET_OS_IPHONE || __POWERPC__
@@ -1543,7 +1532,7 @@ static int r_debug_native_map_protect (RDebug *dbg, ut64 addr, int size, int per
 		"sc@syscall(%d);\n"
 		"main@global(0) { sc(%p,%d,%d);\n"
 		":int3\n"
-		"}\n", num, (void*)addr, size, io_perms_to_prot (perms));
+		"}\n", num, (void*)(size_t)addr, size, io_perms_to_prot (perms));
 
 	r_egg_reset (dbg->egg);
 	r_egg_setup(dbg->egg, dbg->arch, 8 * dbg->bits, 0, 0);
@@ -1616,23 +1605,23 @@ RDebugPlugin r_debug_plugin_native = {
 #if __i386__
 	.bits = R_SYS_BITS_32,
 	.arch = "x86",
-	.canstep = 1,
+	.canstep = true,
 #elif __x86_64__
 	.bits = R_SYS_BITS_32 | R_SYS_BITS_64,
 	.arch = "x86",
-	.canstep = 1, // XXX it's 1 on some platforms...
+	.canstep = true, // XXX it's 1 on some platforms...
 #elif __aarch64__ || __arm64__
-	.bits = R_SYS_BITS_16 | R_SYS_BITS_32 | R_SYS_BITS_64,
+	.bits = R_SYS_BITS_64,
 	.arch = "arm",
-	.canstep = 1,
+	.canstep = false,
 #elif __arm__
 	.bits = R_SYS_BITS_16 | R_SYS_BITS_32 | R_SYS_BITS_64,
 	.arch = "arm",
-	.canstep = 0,
+	.canstep = false,
 #elif __mips__
 	.bits = R_SYS_BITS_32 | R_SYS_BITS_64,
 	.arch = "mips",
-	.canstep = 0,
+	.canstep = false,
 #elif __powerpc__
 # if __powerpc64__
 	.bits = R_SYS_BITS_32 | R_SYS_BITS_64,
@@ -1640,11 +1629,11 @@ RDebugPlugin r_debug_plugin_native = {
 	.bits = R_SYS_BITS_32,
 #endif
 	.arch = "ppc",
-	.canstep = 1,
+	.canstep = true,
 #else
 	.bits = 0,
 	.arch = 0,
-	.canstep = 0,
+	.canstep = false,
 #ifdef _MSC_VER
 #pragma message("Unsupported architecture")
 #else
