@@ -9,7 +9,7 @@
 int bsd_handle_signals(RDebug *dbg);
 int bsd_reg_write(RDebug *dbg, int type, const ut8 *buf, int size);
 RDebugInfo *bsd_info(RDebug *dbg, const char *arg);
-RList *bsd_pid_list(RDebug *dbg, RList *list);
+RList *bsd_pid_list(RDebug *dbg, int pid, RList *list);
 RList *bsd_native_sysctl_map(RDebug *dbg);
 RList *bsd_desc_list(int pid);
 RList *bsd_thread_list(RDebug *dbg, int pid, RList *list);
