@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include <r_userconf.h>
 
@@ -18,6 +18,8 @@
 #include <unistd.h>
 #include <elf.h>
 
+#include "linux_ptrace.h"
+
 #ifdef __GLIBC__
 #define HAVE_YMM 1
 #else
@@ -46,9 +48,6 @@ char *linux_reg_profile (RDebug *dbg) {
 #endif
 	} else {
 #		include "reg/linux-x64.h"
-#if HAVE_YMM
-#		include <bits/sigcontext.h>
-#endif
 	}
 #elif __powerpc__
 	if (dbg->bits & R_SYS_BITS_32) {
@@ -64,19 +63,20 @@ char *linux_reg_profile (RDebug *dbg) {
 #endif
 }
 
-static void linux_detach_all (RDebug *dbg);
-static char *read_link (int pid, const char *file);
-static bool linux_attach_single_pid (RDebug *dbg, int ptid);
-static void linux_attach_all (RDebug *dbg);
-static void linux_remove_thread (RDebug *dbg, int pid);
-static void linux_add_and_attach_new_thread (RDebug *dbg, int tid);
-static bool linux_stop_thread (int tid);
-static bool linux_kill_thread (int tid, int signo);
-static void linux_dbg_wait_break (RDebug *dbg);
+static void linux_detach_all(RDebug *dbg);
+static char *read_link(int pid, const char *file);
+static bool linux_attach_single_pid(RDebug *dbg, int ptid);
+static void linux_remove_thread(RDebug *dbg, int pid);
+static void linux_add_new_thread(RDebug *dbg, int tid);
+static bool linux_stop_thread(RDebug *dbg, int tid);
+static bool linux_kill_thread(int tid, int signo);
+static void linux_dbg_wait_break_main(RDebug *dbg);
+static void linux_dbg_wait_break(RDebug *dbg);
+static RDebugReasonType linux_handle_new_task(RDebug *dbg, int tid);
 
-int linux_handle_signals (RDebug *dbg, int tid) {
+int linux_handle_signals(RDebug *dbg, int tid) {
 	siginfo_t siginfo = { 0 };
-	int ret = r_debug_ptrace (dbg, PTRACE_GETSIGINFO, tid, 0, (r_ptrace_data_t)(size_t)&siginfo);
+	int ret = r_debug_ptrace (dbg, PTRACE_GETSIGINFO, tid, 0, (r_ptrace_data_t)&siginfo);
 	if (ret == -1) {
 		/* ESRCH means the process already went away :-/ */
 		if (errno == ESRCH) {
@@ -92,11 +92,23 @@ int linux_handle_signals (RDebug *dbg, int tid) {
 		//ptrace (PTRACE_SETSIGINFO, dbg->pid, 0, &siginfo);
 		dbg->reason.type = R_DEBUG_REASON_SIGNAL;
 		dbg->reason.signum = siginfo.si_signo;
-		dbg->stopaddr = (ut64)siginfo.si_addr;
+		dbg->stopaddr = (ut64)(size_t)siginfo.si_addr;
 		//dbg->errno = siginfo.si_errno;
 		// siginfo.si_code -> HWBKPT, USER, KERNEL or WHAT
-// TODO: DO MORE RDEBUGREASON HERE
+		// TODO: DO MORE RDEBUGREASON HERE
 		switch (dbg->reason.signum) {
+		case SIGINT:
+			dbg->reason.type = R_DEBUG_REASON_USERSUSP;
+			break;
+		case SIGSEGV:
+			dbg->reason.type = R_DEBUG_REASON_SEGFAULT;
+			break;
+		case SIGSTOP:
+			dbg->reason.type = R_DEBUG_REASON_STOPPED;
+			break;
+		case SIGTERM:
+			dbg->reason.type = R_DEBUG_REASON_TERMINATED;
+			break;
 		case SIGTRAP:
 		{
 			if (dbg->glob_libs || dbg->glob_unlibs) {
@@ -106,47 +118,40 @@ int linux_handle_signals (RDebug *dbg, int tid) {
 					char *p = strstr (b->data, "dbg.");
 					if (p) {
 						if (r_str_startswith (p, "dbg.libs")) {
-							const char *name;
-							if (strstr (b->data, "sym.imp.dlopen")) {
-								name = r_reg_get_name (dbg->reg, R_REG_NAME_A0);
-							} else {
-								name = r_reg_get_name (dbg->reg, R_REG_NAME_A1);
-							}
+							const char *name = strstr (b->data, "sym.imp.dlopen")
+								? r_reg_get_name (dbg->reg, R_REG_NAME_A0)
+								: r_reg_get_name (dbg->reg, R_REG_NAME_A1);
 							b->data = r_str_appendf (b->data, ";ps@r:%s", name);
 							dbg->reason.type = R_DEBUG_REASON_NEW_LIB;
+							break;
 						} else if (r_str_startswith (p, "dbg.unlibs")) {
 							dbg->reason.type = R_DEBUG_REASON_EXIT_LIB;
+							break;
 						}
 					}
 				}
 			}
 			if (dbg->reason.type != R_DEBUG_REASON_NEW_LIB &&
 				dbg->reason.type != R_DEBUG_REASON_EXIT_LIB) {
-				dbg->reason.bp_addr = (ut64)(size_t)siginfo.si_addr;
-				dbg->reason.type = R_DEBUG_REASON_BREAKPOINT;
-				// Switch to the thread that hit the breakpoint
-				r_debug_select (dbg, dbg->pid, tid);
-				dbg->tid = tid;
+				if (siginfo.si_code == TRAP_TRACE) {
+					dbg->reason.type = R_DEBUG_REASON_STEP;
+				} else {
+					dbg->reason.bp_addr = (ut64)(size_t)siginfo.si_addr;
+					dbg->reason.type = R_DEBUG_REASON_BREAKPOINT;
+					// Switch to the thread that hit the breakpoint
+					r_debug_select (dbg, dbg->pid, tid);
+					dbg->tid = tid;
+				}
 			}
-		}
-			break;
-		case SIGABRT: // 6 / SIGIOT // SIGABRT
-			dbg->reason.type = R_DEBUG_REASON_ABORT;
-			break;
-		case SIGSEGV:
-			dbg->reason.type = R_DEBUG_REASON_SEGFAULT;
-			break;
-		case SIGCHLD:
-			dbg->reason.type = R_DEBUG_REASON_SIGNAL;
-			break;
+		} break;
 		default:
 			break;
 		}
-		if (dbg->reason.signum != SIGTRAP) {
+		if (dbg->reason.signum != SIGTRAP && (dbg->reason.signum != SIGINT || !r_cons_is_breaked ())) {
 			eprintf ("[+] SIGNAL %d errno=%d addr=0x%08"PFMT64x
-				" code=%d ret=%d\n",
+				" code=%d si_pid=%d ret=%d\n",
 				siginfo.si_signo, siginfo.si_errno,
-				(ut64)(size_t)siginfo.si_addr, siginfo.si_code, ret);
+				(ut64) (size_t)siginfo.si_addr, siginfo.si_code, siginfo.si_pid, ret);
 		}
 		return true;
 	}
@@ -193,18 +198,16 @@ static void linux_remove_fork_bps(RDebug *dbg) {
 
 #ifdef PT_GETEVENTMSG
 /*
- * Handle PTRACE_EVENT_*
+ * @brief Handle PTRACE_EVENT_* when receiving SIGTRAP
  *
- * Returns R_DEBUG_REASON_*
- *
- * If it's not something we handled, we return ..._UNKNOWN to
- * tell the caller to keep trying to figure out what to do.
- *
- * If something went horribly wrong, we return ..._ERROR;
+ * @param dowait Do waitpid to consume any signal in the newly created task
+ * @return RDebugReasonType,
+ * - R_DEBUG_REASON_UNKNOWN if the ptrace_event cannot be handled,
+ * - R_DEBUG_REASON_ERROR if a ptrace command failed.
  *
  * NOTE: This API was added in Linux 2.5.46
  */
-RDebugReasonType linux_ptrace_event (RDebug *dbg, int pid, int status) {
+RDebugReasonType linux_ptrace_event (RDebug *dbg, int ptid, int status, bool dowait) {
 	ut32 pt_evt;
 #if __powerpc64__ || __arm64__ || __aarch64__ || __x86_64__
 	ut64 data;
@@ -212,7 +215,7 @@ RDebugReasonType linux_ptrace_event (RDebug *dbg, int pid, int status) {
 	ut32 data;
 #endif
 	/* we only handle stops with SIGTRAP here */
-	if (!WIFSTOPPED(status) || WSTOPSIG(status) != SIGTRAP) {
+	if (!WIFSTOPPED (status) || WSTOPSIG (status) != SIGTRAP) {
 		return R_DEBUG_REASON_UNKNOWN;
 	}
 
@@ -222,30 +225,42 @@ RDebugReasonType linux_ptrace_event (RDebug *dbg, int pid, int status) {
 		/* NOTE: this case is handled by linux_handle_signals */
 		break;
 	case PTRACE_EVENT_CLONE:
-		if (r_debug_ptrace (dbg, PTRACE_GETEVENTMSG, pid, 0, (r_ptrace_data_t)(size_t)&data) == -1) {
+		// Get the tid of the new thread
+		if (r_debug_ptrace (dbg, PTRACE_GETEVENTMSG, ptid, 0, (r_ptrace_data_t)(size_t)&data) == -1) {
 			r_sys_perror ("ptrace GETEVENTMSG");
 			return R_DEBUG_REASON_ERROR;
 		}
-		linux_add_and_attach_new_thread (dbg, (int)data);
+		if (dowait) {
+			// The new child has a pending SIGSTOP.  We can't affect it until it
+			// hits the SIGSTOP, but we're already attached.  */
+			if (waitpid ((int)data, &status, 0) == -1) {
+				perror ("waitpid");
+			}
+		}
+
+		linux_add_new_thread (dbg, (int)data);
 		if (dbg->trace_clone) {
 			r_debug_select (dbg, dbg->pid, (int)data);
 		}
-		eprintf ("(%d) Created thread %d\n", dbg->pid, (int)data);
+		eprintf ("(%d) Created thread %d\n", ptid, (int)data);
 		return R_DEBUG_REASON_NEW_TID;
 	case PTRACE_EVENT_VFORK:
 	case PTRACE_EVENT_FORK:
-		if (r_debug_ptrace (dbg, PTRACE_GETEVENTMSG, pid, 0, (r_ptrace_data_t)(size_t)&data) == -1) {
+		// Get the pid of the new process
+		if (r_debug_ptrace (dbg, PTRACE_GETEVENTMSG, ptid, 0, (r_ptrace_data_t)(size_t)&data) == -1) {
 			r_sys_perror ("ptrace GETEVENTMSG");
 			return R_DEBUG_REASON_ERROR;
 		}
-
 		dbg->forked_pid = data;
-		if (!dbg->trace_forks) {
-			// The new child has a pending SIGSTOP.  We can't affect it until it
+		if (dowait) {
+			// The new child has a pending SIGSTOP. We can't affect it until it
 			// hits the SIGSTOP, but we're already attached.  */
-			if (waitpid (dbg->forked_pid, &status, __WALL) == -1) {
+			if (waitpid (dbg->forked_pid, &status, 0) == -1) {
 				perror ("waitpid");
 			}
+		}
+		eprintf ("(%d) Created process %d\n", ptid, (int)data);
+		if (!dbg->trace_forks) {
 			// We need to do this even if the new process will be detached since the
 			// breakpoints are inherited from the parent
 			linux_remove_fork_bps (dbg);
@@ -255,12 +270,19 @@ RDebugReasonType linux_ptrace_event (RDebug *dbg, int pid, int status) {
 		}
 		return R_DEBUG_REASON_NEW_PID;
 	case PTRACE_EVENT_EXIT:
-		if (r_debug_ptrace (dbg, PTRACE_GETEVENTMSG, pid, 0, (r_ptrace_data_t)(size_t)&data) == -1) {
+		// Get the exit status of the exiting task
+		if (r_debug_ptrace (dbg, PTRACE_GETEVENTMSG, ptid, 0, (r_ptrace_data_t)(size_t)&data) == -1) {
 			r_sys_perror ("ptrace GETEVENTMSG");
 			return R_DEBUG_REASON_ERROR;
 		}
-		//eprintf ("PTRACE_EVENT_EXIT pid=%d, status=0x%"PFMT64x"\n", pid, (ut64)data);
-		return pid != dbg->pid ? R_DEBUG_REASON_EXIT_TID : R_DEBUG_REASON_EXIT_PID;
+		//TODO: Check other processes exit if dbg->trace_forks is on
+		if (ptid != dbg->pid) {
+			eprintf ("(%d) Thread exited with status=0x%"PFMT64x"\n", ptid, (ut64)data);
+			return R_DEBUG_REASON_EXIT_TID;
+		} else {
+			eprintf ("(%d) Process exited with status=0x%"PFMT64x"\n", ptid, (ut64)data);
+			return R_DEBUG_REASON_EXIT_PID;
+		}
 	default:
 		eprintf ("Unknown PTRACE_EVENT encountered: %d\n", pt_evt);
 		break;
@@ -269,11 +291,56 @@ RDebugReasonType linux_ptrace_event (RDebug *dbg, int pid, int status) {
 }
 #endif
 
+/*
+ * @brief Search for the parent of the newly created task and
+ * handle the pending SIGTRAP with PTRACE_EVENT_*
+ *
+ * @param tid, TID of the new task
+ * @return RDebugReasonType, Debug reason
+ */
+static RDebugReasonType linux_handle_new_task(RDebug *dbg, int tid) {
+	int ret, status;
+	if (dbg->threads) {
+		RDebugPid *th;
+		RListIter *it;
+		// Search for SIGTRAP with PTRACE_EVENT_* in other threads.
+		r_list_foreach (dbg->threads, it, th) {
+			if (th->pid == tid) {
+				continue;
+			}
+			// Retrieve the signal without consuming it with PTRACE_GETSIGINFO
+			siginfo_t siginfo = { 0 };
+			ret = r_debug_ptrace (dbg, PTRACE_GETSIGINFO, th->pid, 0, (r_ptrace_data_t)(size_t)&siginfo);
+			// Skip if PTRACE_GETSIGINFO fails when the thread is running.
+			if (ret == -1) {
+				continue;
+			}
+#ifdef PT_GETEVENTMSG
+			// NOTE: This API was added in Linux 2.5.46
+			if (siginfo.si_signo == SIGTRAP) {
+				// si_code = (SIGTRAP | PTRACE_EVENT_* << 8)
+				int pt_evt = siginfo.si_code >> 8;
+				// Handle PTRACE_EVENT_* that creates a new task (fork/clone)
+				switch (pt_evt) {
+				case PTRACE_EVENT_CLONE:
+				case PTRACE_EVENT_VFORK:
+				case PTRACE_EVENT_FORK:
+					ret = waitpid (th->pid, &status, 0);
+					return linux_ptrace_event (dbg, ret, status, false);
+				default:
+					break;
+				}
+			}
+#endif
+		}
+	}
+	return R_DEBUG_REASON_UNKNOWN;
+}
+
 int linux_step(RDebug *dbg) {
 	int ret = false;
 	int pid = dbg->tid;
-	ut64 addr = r_debug_reg_get (dbg, "PC");
-	ret = r_debug_ptrace (dbg, PTRACE_SINGLESTEP, pid, (void*)(size_t)addr, 0);
+	ret = r_debug_ptrace (dbg, PTRACE_SINGLESTEP, pid, 0, 0);
 	//XXX(jjd): why?? //linux_handle_signals (dbg);
 	if (ret == -1) {
 		perror ("native-singlestep");
@@ -300,9 +367,13 @@ bool linux_set_options(RDebug *dbg, int pid) {
 	}
 	/* SIGTRAP | 0x80 on signal handler .. not supported on all archs */
 	traceflags |= PTRACE_O_TRACESYSGOOD;
-	if (r_debug_ptrace (dbg, PTRACE_SETOPTIONS, pid, 0, (r_ptrace_data_t)(size_t)traceflags) == -1) {
-		perror ("ptrace (PT_SETOPTIONS)");
-		return false;
+
+	// PTRACE_SETOPTIONS can fail because of the asynchronous nature of ptrace
+	// If the target is traced, the loop will always end with success
+	while (r_debug_ptrace (dbg, PTRACE_SETOPTIONS, pid, 0, (r_ptrace_data_t)(size_t)traceflags) == -1) {
+		void *bed = r_cons_sleep_begin ();
+		usleep (1000);
+		r_cons_sleep_end (bed);
 	}
 	return true;
 }
@@ -327,16 +398,15 @@ static void linux_detach_all(RDebug *dbg) {
 	}
 }
 
-static void linux_remove_thread (RDebug *dbg, int pid) {
-	RList *th_list = dbg->threads;
-
-	if (th_list) {
+static void linux_remove_thread(RDebug *dbg, int tid) {
+	if (dbg->threads) {
 		RDebugPid *th;
 		RListIter *it;
-		r_list_foreach (th_list, it, th) {
-			if (th->pid == pid) {
-				r_list_delete_data (th_list, th);
+		r_list_foreach (dbg->threads, it, th) {
+			if (th->pid == tid) {
+				r_list_delete (dbg->threads, it);
 				dbg->n_threads--;
+				break;
 			}
 		}
 	}
@@ -368,17 +438,41 @@ bool linux_attach_new_process(RDebug *dbg, int pid) {
 	return true;
 }
 
-static void linux_dbg_wait_break(RDebug *dbg) {
-	// Stop the currently debugged thread
-	if (!linux_kill_thread (dbg->tid, SIGSTOP)) {
-		eprintf ("Could not stop pid (%d)\n", dbg->pid);
+static void linux_dbg_wait_break_main(RDebug *dbg) {
+	// Get the debugger and debuggee process group ID
+	pid_t dpgid = getpgid (0);
+	if (dpgid == -1) {
+		r_sys_perror ("getpgid");
 		return;
 	}
+	pid_t tpgid = getpgid (dbg->pid);
+	if (tpgid == -1) {
+		r_sys_perror ("getpgid");
+		return;
+	}
+
+	// If the debuggee process is created by the debugger, do nothing because
+	// SIGINT is already sent to both the debugger and debuggee in the same
+	// process group.
+
+	// If the debuggee is attached by the debugger, send SIGINT to the debuggee
+	// in another process group.
+	if (dpgid != tpgid) {
+		if (!linux_kill_thread (dbg->pid, SIGINT)) {
+			eprintf ("Could not interrupt pid (%d)\n", dbg->pid);
+		}
+	}
 }
 
-RDebugReasonType linux_dbg_wait(RDebug *dbg, int my_pid) {
+static void linux_dbg_wait_break(RDebug *dbg) {
+	if (!linux_kill_thread (dbg->pid, SIGINT)) {
+		eprintf ("Could not interrupt pid (%d)\n", dbg->pid);
+	}
+}
+
+RDebugReasonType linux_dbg_wait(RDebug *dbg, int pid) {
 	RDebugReasonType reason = R_DEBUG_REASON_UNKNOWN;
-	int pid = dbg->tid;
+	int tid = pid;
 	int status, flags = __WALL;
 	int ret = -1;
 
@@ -386,72 +480,75 @@ RDebugReasonType linux_dbg_wait(RDebug *dbg, int my_pid) {
 		flags |= WNOHANG;
 	}
 
-	// Ignore keyboard interrupt while waiting to avoid signaling the child process twice on
-	// break(SIGSTOP is sent by wait_break) which forced the user to continue an extra time
-	// to handle SIGINT
-	r_sys_signal (SIGINT, SIG_IGN);
-	r_cons_break_push ((RConsBreak)linux_dbg_wait_break, dbg);
-repeat:
 	for (;;) {
-		if (r_cons_is_breaked ()) {
-			reason = R_DEBUG_REASON_USERSUSP;
-			break;
+		// In the main context, SIGINT is propagated to the debuggee if it is
+		// in the same process group. Otherwise, the task is running in
+		// background and SIGINT will not be propagated to the debuggee.
+		if (r_cons_context_is_main ()) {
+			r_cons_break_push ((RConsBreak)linux_dbg_wait_break_main, dbg);
+		} else {
+			r_cons_break_push ((RConsBreak)linux_dbg_wait_break, dbg);
 		}
-
 		void *bed = r_cons_sleep_begin ();
-		ret = waitpid (-1, &status, flags);
+		if (dbg->continue_all_threads) {
+			ret = waitpid (-1, &status, flags);
+		} else {
+			ret = waitpid (pid, &status, flags);
+		}
 		r_cons_sleep_end (bed);
+		r_cons_break_pop ();
 
 		if (ret < 0) {
+			// Continue when interrupted by user;
+			if (errno == EINTR) {
+				continue;
+			}
 			perror ("waitpid");
 			break;
-		} else if (!ret) {
+		} else if (ret == 0) {
+			// Unset WNOHANG to call next waitpid in blocking mode.
 			flags &= ~WNOHANG;
 		} else {
-			pid = ret;
-			reason = linux_ptrace_event (dbg, pid, status);
-
-			if (reason == R_DEBUG_REASON_NEW_PID) {
-				break;
-			}
-
-			if (reason == R_DEBUG_REASON_EXIT_TID) {
-				r_debug_ptrace (dbg, PTRACE_CONT, pid, NULL, 0);
-				goto repeat;
-			}
+			tid = ret;
 
+			// Handle SIGTRAP with PTRACE_EVENT_*
+			reason = linux_ptrace_event (dbg, tid, status, true);
 			if (reason != R_DEBUG_REASON_UNKNOWN) {
 				break;
 			}
 
 			if (WIFEXITED (status)) {
-				eprintf ("child exited with status %d\n", WEXITSTATUS (status));
-				if (pid == dbg->main_pid) {
+				if (tid == dbg->main_pid) {
 					r_list_free (dbg->threads);
 					dbg->threads = NULL;
 					reason = R_DEBUG_REASON_DEAD;
+					eprintf ("(%d) Process terminated with status %d\n", tid, WEXITSTATUS (status));
+					break;
 				} else {
-					reason = R_DEBUG_REASON_EXIT_TID;
-					linux_remove_thread (dbg, pid);
+					eprintf ("(%d) Child terminated with status %d\n", tid, WEXITSTATUS (status));
+					linux_remove_thread (dbg, tid);
+					continue;
 				}
 			} else if (WIFSIGNALED (status)) {
 				eprintf ("child received signal %d\n", WTERMSIG (status));
 				reason = R_DEBUG_REASON_SIGNAL;
 			} else if (WIFSTOPPED (status)) {
-				if (WSTOPSIG (status) != SIGTRAP &&
-					WSTOPSIG (status) != SIGSTOP) {
-					if (WSTOPSIG (status) == SIGINT) {
-						reason = R_DEBUG_REASON_USERSUSP;
+				// If tid is not in the thread list and stopped by SIGSTOP,
+				// handle it as a new task.
+				if (!r_list_find (dbg->threads, &tid, &match_pid) &&
+					WSTOPSIG (status) == SIGSTOP) {
+					reason = linux_handle_new_task (dbg, tid);
+					if (reason != R_DEBUG_REASON_UNKNOWN) {
 						break;
 					}
-					eprintf ("child stopped with signal %d\n", WSTOPSIG (status));
-					reason = R_DEBUG_REASON_DEAD;
 				}
-				if (!linux_handle_signals (dbg, pid)) {
+
+				if (linux_handle_signals (dbg, tid)) {
+					reason = dbg->reason.type;
+				} else {
 					eprintf ("can't handle signals\n");
 					return R_DEBUG_REASON_ERROR;
 				}
-				reason = dbg->reason.type;
 #ifdef WIFCONTINUED
 			} else if (WIFCONTINUED (status)) {
 				eprintf ("child continued...\n");
@@ -464,7 +561,7 @@ repeat:
 				eprintf ("STATUS=0?!?!?!?\n");
 				reason = R_DEBUG_REASON_DEAD;
 			} else {
-				if (ret != pid) {
+				if (ret != tid) {
 					reason = R_DEBUG_REASON_NEW_PID;
 				} else {
 					eprintf ("CRAP. returning from wait without knowing why...\n");
@@ -475,19 +572,17 @@ repeat:
 			}
 		}
 	}
-	r_cons_break_pop ();
-	r_sys_signal (SIGINT, SIG_DFL);
-	dbg->reason.tid = pid;
+	dbg->reason.tid = tid;
 	return reason;
 }
 
 int match_pid(const void *pid_o, const void *th_o) {
 	int pid = *(int *)pid_o;
-	RDebug *th = (RDebug *)th_o;
-	return pid == th->pid;
+	RDebugPid *th = (RDebugPid *)th_o;
+	return (pid == th->pid)? 0 : 1;
 }
 
-static void linux_add_and_attach_new_thread(RDebug *dbg, int tid) {
+static void linux_add_new_thread(RDebug *dbg, int tid) {
 	int uid = getuid(); // XXX
 	char info[1024] = {0};
 	RDebugPid *tid_info;
@@ -497,7 +592,7 @@ static void linux_add_and_attach_new_thread(RDebug *dbg, int tid) {
 	} else {
 		tid_info = r_debug_pid_new ("new_path", tid, uid, 's', 0);
 	}
-	(void) linux_attach_single_pid (dbg, tid);
+	linux_set_options (dbg, tid);
 	r_list_append (dbg->threads, tid_info);
 	dbg->n_threads++;
 }
@@ -513,30 +608,34 @@ static bool linux_kill_thread(int tid, int signo) {
 	return true;
 }
 
-static bool linux_stop_thread(int tid) {
-	int status;
-	int ret = -1;
+static bool linux_stop_thread(RDebug *dbg, int tid) {
+	int status, ret;
+	siginfo_t siginfo = { 0 };
 
-	if (linux_kill_thread (tid, SIGSTOP)) {
-		if ((ret = waitpid (tid, &status, WUNTRACED)) == -1) {
-			perror("waitpid");
-		}
+	// Return if the thread is already stopped
+	ret = r_debug_ptrace (dbg, PTRACE_GETSIGINFO, tid, 0,
+		(r_ptrace_data_t) (intptr_t)&siginfo);
+	if (ret == 0) {
+		return true;
 	}
 
-	return ret == tid;
+	if (linux_kill_thread (tid, SIGSTOP)) {
+		if ((ret = waitpid (tid, &status, 0)) == -1) {
+			perror ("waitpid");
+		}
+		return ret == tid;
+	}
+	return false;
 }
 
 bool linux_stop_threads(RDebug *dbg, int except) {
-	// Stop all currently debugged threads
-	RList *list = dbg->threads;
 	bool ret = true;
-	if (list) {
+	if (dbg->threads) {
 		RDebugPid *th;
 		RListIter *it;
-		r_list_foreach (list, it, th) {
+		r_list_foreach (dbg->threads, it, th) {
 			if (th->pid && th->pid != except) {
-				if (!linux_stop_thread (th->pid)) {
-					eprintf ("Could not stop pid (%d)\n", th->pid);
+				if (!linux_stop_thread (dbg, th->pid)) {
 					ret = false;
 				}
 			}
@@ -545,10 +644,10 @@ bool linux_stop_threads(RDebug *dbg, int except) {
 	return ret;
 }
 
-static bool linux_attach_single_pid(RDebug *dbg, int ptid) {
+static bool linux_attach_single_pid(RDebug *dbg, int pid) {
 	siginfo_t sig = { 0 };
 
-	if (ptid < 0) {
+	if (pid < 0) {
 		return false;
 	}
 
@@ -556,25 +655,26 @@ static bool linux_attach_single_pid(RDebug *dbg, int ptid) {
 	// Attaching to a process that has already been started with PTRACE_TRACEME.
 	// sets errno to "Operation not permitted" which may be misleading.
 	// GETSIGINFO can be called multiple times and would fail without attachment.
-	if (r_debug_ptrace (dbg, PTRACE_GETSIGINFO, ptid, NULL,
+	if (r_debug_ptrace (dbg, PTRACE_GETSIGINFO, pid, NULL,
 		(r_ptrace_data_t)&sig) == -1) {
-		if (r_debug_ptrace (dbg, PTRACE_ATTACH, ptid, NULL,
-			(r_ptrace_data_t)NULL) == -1) {
+		if (r_debug_ptrace (dbg, PTRACE_ATTACH, pid, NULL, NULL) == -1) {
 			perror ("ptrace (PT_ATTACH)");
 			return false;
 		}
 
 		// Make sure SIGSTOP is delivered and wait for it since we can't affect the pid
 		// until it hits SIGSTOP.
-		if (!linux_stop_thread (ptid)) {
-			eprintf ("Could not stop pid (%d)\n", ptid);
+		if (!linux_stop_thread (dbg, pid)) {
+			eprintf ("Could not stop pid (%d)\n", pid);
+			return false;
 		}
 	}
 
-	if (!linux_set_options (dbg, ptid)) {
-		eprintf("failed set_options on %d\n", ptid);
+	if (!linux_set_options (dbg, pid)) {
+		eprintf("failed set_options on %d\n", pid);
 		return false;
 	}
+	dbg->pid = pid;
 	return true;
 }
 
@@ -587,38 +687,18 @@ static RList *get_pid_thread_list(RDebug *dbg, int main_pid) {
 	return list;
 }
 
-static void linux_attach_all(RDebug *dbg) {
-	linux_attach_single_pid (dbg, dbg->main_pid);
-
-	RList *list = dbg->threads;
-	if (list) {
-		RDebugPid *th;
-		RListIter *it;
-		r_list_foreach (list, it, th) {
-			if (th->pid && th->pid != dbg->main_pid) {
-				if (!linux_attach_single_pid (dbg, th->pid)) {
-					eprintf ("PID %d\n", th->pid);
-				}
-			}
-		}
-	}
-}
-
-int linux_attach(RDebug *dbg, int pid) {
-	// First time we run: We try to attach to all "possible" threads and to the main pid
+bool linux_attach(RDebug *dbg, int pid) {
 	if (!dbg->threads) {
 		dbg->threads = get_pid_thread_list (dbg, pid);
-		linux_attach_all (dbg);
+		if (dbg->threads) {
+			dbg->pid = pid;
+		}
 	} else {
-		// This means we did a first run, so we probably attached to all possible threads already.
-		// So check if the requested thread is being traced already. If yes: skip
-		if (dbg->threads && !r_list_find (dbg->threads, &pid, &match_pid)) {
-			goto out;
+		if (!r_list_find (dbg->threads, &pid, &match_pid)) {
+			linux_attach_single_pid (dbg, pid);
 		}
-		linux_attach_single_pid (dbg, pid);
 	}
-out:
-	return pid;
+	return true;
 }
 
 static char *read_link(int pid, const char *file) {
@@ -823,10 +903,12 @@ RList *linux_thread_list(RDebug *dbg, int pid, RList *list) {
 				pid_info = r_debug_pid_new (NULL, tid, uid, 's', pc);
 			}
 			r_list_append (list, pid_info);
+			dbg->n_threads++;
 		}
 		closedir (dh);
 		// Return to the original thread
 		linux_attach_single_pid (dbg, prev_tid);
+		dbg->pid = pid;
 		dbg->tid = prev_tid;
 		r_debug_reg_sync (dbg, R_REG_TYPE_GPR, false);
 	} else {
@@ -927,7 +1009,7 @@ static void print_fpu(void *f){
 	}
 #endif // __ANDROID__
 #elif __i386__
-	int i,j;
+	int i;
 #if __ANDROID__
 	struct user_fpxregs_struct fpxregs = *(struct user_fpxregs_struct*)f;
 	r_cons_printf ("---- x86-32 ----\n");
@@ -978,7 +1060,14 @@ static void print_fpu(void *f){
 int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 	bool showfpu = false;
 	int pid = dbg->tid;
-	int ret;
+	if (pid == -1) {
+		if (dbg->pid == -1) {
+			eprintf ("linux_reg_read: Invalid pid %d\n", pid);
+			return 0;
+		}
+		pid = dbg->pid;
+	}
+	int ret = 0;
 	if (type < -1) {
 		showfpu = true;
 		type = -type;
@@ -1020,46 +1109,56 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 		return false;
 #elif __x86_64__ || __i386__
 		{
-		int ret1 = 0;
 		struct user_fpregs_struct fpregs;
 		if (type == R_REG_TYPE_FPU) {
 #if __x86_64__
-			ret1 = r_debug_ptrace (dbg, PTRACE_GETFPREGS, pid, NULL, &fpregs);
+			ret = r_debug_ptrace (dbg, PTRACE_GETFPREGS, pid, NULL, &fpregs);
+			if (ret != 0) {
+				r_sys_perror ("PTRACE_GETFPREGS");
+				return false;
+			}
 			if (showfpu) {
 				print_fpu ((void *)&fpregs);
 			}
-			if (ret1 != 0) {
-				return false;
-			}
-			if (sizeof (fpregs) < size) {
-				size = sizeof (fpregs);
-			}
+			size = R_MIN (sizeof (fpregs), size);
 			memcpy (buf, &fpregs, size);
-			return sizeof(fpregs);
+			return size;
 #elif __i386__
 #if !__ANDROID__
 			struct user_fpxregs_struct fpxregs;
-			ret1 = r_debug_ptrace (dbg, PTRACE_GETFPXREGS, pid, NULL, &fpxregs);
-			if (ret1 == 0) {
-				if (showfpu) print_fpu ((void *)&fpxregs);
-				if (sizeof(fpxregs) < size) size = sizeof(fpxregs);
+			ret = r_debug_ptrace (dbg, PTRACE_GETFPXREGS, pid, NULL, &fpxregs);
+			if (ret == 0) {
+				if (showfpu) {
+					print_fpu ((void *)&fpxregs);
+				}
+				size = R_MIN (sizeof (fpxregs), size);
 				memcpy (buf, &fpxregs, size);
-				return sizeof(fpxregs);
+				return size;
 			} else {
-				ret1 = r_debug_ptrace (dbg, PTRACE_GETFPREGS, pid, NULL, &fpregs);
-				if (showfpu) print_fpu ((void *)&fpregs);
-				if (ret1 != 0) return false;
-				if (sizeof(fpregs) < size) size = sizeof(fpregs);
+				ret = r_debug_ptrace (dbg, PTRACE_GETFPREGS, pid, NULL, &fpregs);
+				if (showfpu) {
+					print_fpu ((void *)&fpregs);
+				}
+				if (ret != 0) {
+					r_sys_perror ("PTRACE_GETFPREGS");
+					return false;
+				}
+				size = R_MIN (sizeof (fpregs), size);
 				memcpy (buf, &fpregs, size);
-				return sizeof(fpregs);
+				return size;
 			}
 #else
-			ret1 = r_debug_ptrace (dbg, PTRACE_GETFPREGS, pid, NULL, &fpregs);
-			if (showfpu) print_fpu ((void *)&fpregs);
-			if (ret1 != 0) return false;
-			if (sizeof (fpregs) < size) size = sizeof(fpregs);
+			ret = r_debug_ptrace (dbg, PTRACE_GETFPREGS, pid, NULL, &fpregs);
+			if (showfpu) {
+				print_fpu ((void *)&fpregs);
+			}
+			if (ret != 0) {
+				r_sys_perror ("PTRACE_GETFPREGS");
+				return false;
+			}
+			size = R_MIN (sizeof (fpregs), size);
 			memcpy (buf, &fpregs, size);
-			return sizeof(fpregs);
+			return size;
 #endif // !__ANDROID__
 #endif // __i386__
 		}
@@ -1075,16 +1174,13 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 			R_DEBUG_REG_T regs;
 			memset (&regs, 0, sizeof (regs));
 			memset (buf, 0, size);
-#if __arm64__ || __aarch64__ || __s390x__
+#if (__arm64__ || __aarch64__ || __s390x__) && defined(PTRACE_GETREGSET)
 			struct iovec io = {
 				.iov_base = &regs,
 				.iov_len = sizeof (regs)
 			};
-			ret = r_debug_ptrace (dbg, PTRACE_GETREGSET, pid, 1, &io);
+			ret = r_debug_ptrace (dbg, PTRACE_GETREGSET, pid, (size_t)1, &io);
 			// ret = ptrace (PTRACE_GETREGSET, pid, (void*)(size_t)(NT_PRSTATUS), NULL); // &io);
-			if (ret != 0) {
-				perror("ptrace");
-			}
 #elif __BSD__ && (__POWERPC__ || __sparc__)
 			ret = r_debug_ptrace (dbg, PTRACE_GETREGS, pid, &regs, NULL);
 #else
@@ -1098,36 +1194,50 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 			 * more ptrace queries.
 			 */
 			if (ret != 0) {
+				r_sys_perror ("PTRACE_GETREGS");
 				return false;
 			}
-			memcpy (buf, &regs, R_MIN (sizeof (regs), size));
-			return sizeof (regs);
+			size = R_MIN (sizeof (regs), size);
+			memcpy (buf, &regs, size);
+			return size;
 		}
 		break;
 	case R_REG_TYPE_YMM:
 		{
-#if HAVE_YMM && __x86_64__
+#if HAVE_YMM && __x86_64__ && defined(PTRACE_GETREGSET)
 		ut32 ymm_space[128];	// full ymm registers
 		struct _xstate xstate;
-		struct iovec iov;
+		struct iovec iov = {};
 		iov.iov_base = &xstate;
 		iov.iov_len = sizeof(struct _xstate);
 		ret = r_debug_ptrace (dbg, PTRACE_GETREGSET, pid, (void*)NT_X86_XSTATE, &iov);
-		if (ret != 0) {
+		if (errno == ENODEV) {
+			// ignore ENODEV, it just means this CPU or kernel doesn't support XSTATE
+			ret = 0;
+		} else if (ret != 0) {
+			if (dbg->verbose) {
+				// WSL1 doesnt support retrieving YMM registers, so this call just fails, no need to be noisy
+				r_sys_perror ("PTRACE_GETREGSET");
+			}
 			return false;
 		}
 		// stitch together xstate.fpstate._xmm and xstate.ymmh assuming LE
 		int ri,rj;
 		for (ri = 0; ri < 16; ri++)	{
 			for (rj=0; rj < 4; rj++)	{
+#ifdef __ANDROID__
+				ymm_space[ri*8+rj] = ((struct _libc_fpstate*) &xstate.fpstate)->_xmm[ri].element[rj];
+#else
 				ymm_space[ri*8+rj] = xstate.fpstate._xmm[ri].element[rj];
+#endif
 			}
-			for (rj=0; rj < 4; rj++)	{
-				ymm_space[ri*8+(rj+4)] = xstate.ymmh.ymmh_space[ri*4+rj];
+			for (rj = 0; rj < 4; rj++)	{
+				ymm_space[ri * 8 + (rj + 4)] = xstate.ymmh.ymmh_space[ri * 4 + rj];
 			}
 		}
-		memcpy (buf, &ymm_space, sizeof (ymm_space));
-		return sizeof (ymm_space);
+		size = R_MIN (sizeof (ymm_space), size);
+		memcpy (buf, &ymm_space, size);
+		return size;
 #endif
 		return false;
 		}
@@ -1136,7 +1246,7 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 	return false;
 }
 
-int linux_reg_write (RDebug *dbg, int type, const ut8 *buf, int size) {
+int linux_reg_write(RDebug *dbg, int type, const ut8 *buf, int size) {
 	int pid = dbg->tid;
 
 	if (type == R_REG_TYPE_DRX) {
@@ -1175,7 +1285,11 @@ int linux_reg_write (RDebug *dbg, int type, const ut8 *buf, int size) {
 			size = sizeof (R_DEBUG_REG_T);
 		}
 #endif
-		return (ret != 0) ? false : true;
+		if (ret == -1) {
+			r_sys_perror ("reg_write");
+			return false;
+		}
+		return true;
 	}
 	if (type == R_REG_TYPE_FPU) {
 #if __i386__ || __x86_64__
@@ -1221,9 +1335,11 @@ RList *linux_desc_list (int pid) {
 		}
 		memcpy (file, path, len);
 		memcpy (file + len, de->d_name, len2 + 1);
-		memset (buf, 0, sizeof(buf));
-		(void)readlink (file, buf, sizeof (buf) - 1);
-		buf[sizeof (buf)-1] = 0;
+		buf[0] = 0;
+		if (readlink (file, buf, sizeof (buf) - 1) == -1) {
+			return NULL;
+		}
+		buf[sizeof (buf) - 1] = 0;
 		type = perm = 0;
 		if (stat (file, &st) != -1) {
 			type  = st.st_mode & S_IFIFO  ? 'P':
