@@ -9,7 +9,7 @@
 #define _XNU_DEBUG_H
 
 #define MACH_ERROR_STRING(ret) \
-	(mach_error_string (ret) ? r_str_get (mach_error_string (ret)) : "(unknown)")
+	(mach_error_string (ret) ? r_str_getf (mach_error_string (ret)) : "(unknown)")
 
 #if TARGET_OS_IPHONE
 // no ptrace
@@ -37,6 +37,7 @@ int ptrace(int _request, pid_t _pid, caddr_t _addr, int _data);
 #if !__POWERPC__
 #include <sys/proc_info.h>
 #include <libproc.h>
+#define HAS_LIBPROC
 #endif
 #endif
 #include <sys/types.h>
@@ -263,22 +264,22 @@ task_t pid_to_task (int pid);
 int xnu_get_vmmap_entries_for_pid (pid_t pid);
 char *xnu_corefile_default_location(void);
 bool xnu_generate_corefile(RDebug *dbg, RBuffer *dest);
-int xnu_reg_read (RDebug *dbg, int type, ut8 *buf, int size);
-int xnu_reg_write (RDebug *dgb, int type, const ut8 *buf, int size);
+int xnu_reg_read(RDebug *dbg, int type, ut8 *buf, int size);
+int xnu_reg_write(RDebug *dgb, int type, const ut8 *buf, int size);
 char *xnu_reg_profile (RDebug *dbg);
-int xnu_attach (RDebug *dbg, int pid);
-bool xnu_step (RDebug *dbg);
-int xnu_detach (RDebug *dbg, int pid);
-int xnu_stop (RDebug *dbg, int pid);
-int xnu_continue (RDebug *dbg, int pid, int tid, int sig);
-RDebugMap *xnu_map_alloc (RDebug *dbg, ut64 addr, int size);
-int xnu_map_dealloc (RDebug *dbg, ut64 addr, int size);
-int xnu_map_protect (RDebug *dbg, ut64 addr, int size, int perms);
-int xnu_init (void);
-int xnu_wait (RDebug *dbg, int pid);
-RDebugPid *xnu_get_pid (int pid);
-RList *xnu_dbg_maps (RDebug *dbg, int only_modules);
-RList *xnu_thread_list (RDebug *dbg, int pid, RList *list);
-RDebugInfo *xnu_info (RDebug *dbg, const char *arg);
+bool xnu_attach(RDebug *dbg, int pid);
+bool xnu_step(RDebug *dbg);
+bool xnu_detach(RDebug *dbg, int pid);
+bool xnu_stop(RDebug *dbg, int pid);
+bool xnu_continue(RDebug *dbg, int pid, int tid, int sig);
+RDebugMap *xnu_map_alloc(RDebug *dbg, ut64 addr, int size);
+int xnu_map_dealloc(RDebug *dbg, ut64 addr, int size);
+int xnu_map_protect(RDebug *dbg, ut64 addr, int size, int perms);
+int xnu_init(void);
+int xnu_wait(RDebug *dbg, int pid);
+RDebugPid *xnu_get_pid(int pid);
+RList *xnu_dbg_maps(RDebug *dbg, int only_modules);
+RList *xnu_thread_list(RDebug *dbg, int pid, RList *list);
+RDebugInfo *xnu_info(RDebug *dbg, const char *arg);
 
 #endif
