@@ -76,8 +76,8 @@ REggPlugin r_egg_plugin_bind = {
 	.build = (void *)build
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_EGG,
 	.data = &r_egg_plugin_bind,
 	.version = R2_VERSION
