@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2019 - pancake, ret2libc */
+/* radare - LGPL - Copyright 2007-2021 - pancake, ret2libc */
 
 #include <r_flag.h>
 #include <r_util.h>
@@ -98,6 +98,9 @@ static void remove_offsetmap(RFlag *f, RFlagItem *item) {
 }
 
 static RFlagsAtOffset *flags_at_offset(RFlag *f, ut64 off) {
+	if (f->mask) {
+		off &= f->mask;
+	}
 	RFlagsAtOffset *res = r_flag_get_nearest_list (f, off, 0);
 	if (res) {
 		return res;
@@ -215,7 +218,7 @@ static void new_spaces(RFlag *f) {
 	r_event_hook (f->spaces.event, R_SPACE_EVENT_UNSET, unset_flagspace, NULL);
 }
 
-R_API RFlag *r_flag_new() {
+R_API RFlag *r_flag_new(void) {
 	RFlag *f = R_NEW0 (RFlag);
 	if (!f) {
 		return NULL;
@@ -227,19 +230,10 @@ R_API RFlag *r_flag_new() {
 	}
 	f->base = 0;
 	f->cb_printf = (PrintfCallback)printf;
-#if R_FLAG_ZONE_USE_SDB
-	f->zones = sdb_new0 ();
-#else
-	f->zones = NULL;
-#endif
+	f->zones = r_list_newf (r_flag_zone_item_free);
 	f->tags = sdb_new0 ();
 	f->ht_name = ht_pp_new (NULL, ht_free_flag, NULL);
 	f->by_off = r_skiplist_new (flag_skiplist_free, flag_skiplist_cmp);
-#if R_FLAG_ZONE_USE_SDB
-	sdb_free (f->zones);
-#else
-	r_list_free (f->zones);
-#endif
 	new_spaces(f);
 	return f;
 }
@@ -350,13 +344,13 @@ static bool print_flag_rad(RFlagItem *flag, void *user) {
 		u->f->cb_printf ("fa %s %s\n", flag->name, flag->alias);
 		if (comment_b64) {
 			u->f->cb_printf ("\"fC %s %s\"\n",
-				flag->name, comment_b64? comment_b64: "");
+				flag->name, r_str_get (comment_b64));
 		}
 	} else {
 		u->f->cb_printf ("f %s %" PFMT64d " 0x%08" PFMT64x "%s%s %s\n",
 			flag->name, flag->size, flag->offset,
-			u->pfx? "+": "", u->pfx? u->pfx: "",
-			comment_b64? comment_b64: "");
+			u->pfx? "+": "", r_str_get (u->pfx),
+			r_str_get (comment_b64));
 	}
 
 	free (comment_b64);
@@ -483,9 +477,12 @@ static RFlagItem *evalFlag(RFlag *f, RFlagItem *item) {
 /* return true if flag.* exist at offset. Otherwise, false is returned.
  * For example (f, "sym", 3, 0x1000)*/
 R_API bool r_flag_exist_at(RFlag *f, const char *flag_prefix, ut16 fp_size, ut64 off) {
-	r_return_val_if_fail (f && flag_prefix, NULL);
+	r_return_val_if_fail (f && flag_prefix, false);
 	RListIter *iter = NULL;
 	RFlagItem *item = NULL;
+	if (f->mask) {
+		off &= f->mask;
+	}
 	const RList *list = r_flag_get_list (f, off);
 	if (list) {
 		r_list_foreach (list, iter, item) {
@@ -508,6 +505,9 @@ R_API RFlagItem *r_flag_get(RFlag *f, const char *name) {
 /* return the first flag item that can be found at offset "off", or NULL otherwise */
 R_API RFlagItem *r_flag_get_i(RFlag *f, ut64 off) {
 	r_return_val_if_fail (f, NULL);
+	if (f->mask) {
+		off &= f->mask;
+	}
 	const RList *list = r_flag_get_list (f, off);
 	return list? evalFlag (f, r_list_get_top (list)): NULL;
 }
@@ -517,6 +517,9 @@ R_API RFlagItem *r_flag_get_i(RFlag *f, ut64 off) {
  * Pass in the name of each space, in order, followed by a NULL */
 R_API RFlagItem *r_flag_get_by_spaces(RFlag *f, ut64 off, ...) {
 	r_return_val_if_fail (f, NULL);
+	if (f->mask) {
+		off &= f->mask;
+	}
 
 	const RList *list = r_flag_get_list (f, off);
 	RFlagItem *ret = NULL;
@@ -599,6 +602,9 @@ static bool isFunctionFlag(const char *n) {
  * NULL is returned if such a item is not found. */
 R_API RFlagItem *r_flag_get_at(RFlag *f, ut64 off, bool closest) {
 	r_return_val_if_fail (f, NULL);
+	if (f->mask) {
+		off &= f->mask;
+	}
 
 	RFlagItem *nice = NULL;
 	RListIter *iter;
@@ -669,6 +675,9 @@ R_API RList *r_flag_all_list(RFlag *f, bool by_space) {
 
 /* return the list of flag items that are associated with a given offset */
 R_API const RList* /*<RFlagItem*>*/ r_flag_get_list(RFlag *f, ut64 off) {
+	if (f->mask) {
+		off &= f->mask;
+	}
 	const RFlagsAtOffset *item = r_flag_get_nearest_list (f, off, 0);
 	return item ? item->flags : NULL;
 }
@@ -676,11 +685,14 @@ R_API const RList* /*<RFlagItem*>*/ r_flag_get_list(RFlag *f, ut64 off) {
 R_API char *r_flag_get_liststr(RFlag *f, ut64 off) {
 	RFlagItem *fi;
 	RListIter *iter;
+	if (f->mask) {
+		off &= f->mask;
+	}
 	const RList *list = r_flag_get_list (f, off);
 	char *p = NULL;
 	r_list_foreach (list, iter, fi) {
 		p = r_str_appendf (p, "%s%s",
-			fi->realname, iter->n? ",": ":");
+			fi->realname, iter->n? ",": "");
 	}
 	return p;
 }
@@ -689,6 +701,9 @@ R_API char *r_flag_get_liststr(RFlag *f, ut64 off) {
 // the same name, slightly change the name by appending ".%d" as suffix
 R_API RFlagItem *r_flag_set_next(RFlag *f, const char *name, ut64 off, ut32 size) {
 	r_return_val_if_fail (f && name, NULL);
+	if (f->mask) {
+		off &= f->mask;
+	}
 	if (!r_flag_get (f, name)) {
 		return r_flag_set (f, name, off, size);
 	}
@@ -711,17 +726,36 @@ R_API RFlagItem *r_flag_set_next(RFlag *f, const char *name, ut64 off, ut32 size
 	return NULL;
 }
 
+R_API RFlagItem *r_flag_set_inspace(RFlag *f, const char *space, const char *name, ut64 off, ut32 size) {
+	if (space) {
+		r_flag_space_push (f, space);
+	}
+	RFlagItem *fi = r_flag_set (f, name, off, size);
+	if (space) {
+		r_flag_space_pop (f);
+	}
+	return fi;
+}
+
 /* create or modify an existing flag item with the given name and parameters.
  * The realname of the item will be the same as the name.
  * NULL is returned in case of any errors during the process. */
 R_API RFlagItem *r_flag_set(RFlag *f, const char *name, ut64 off, ut32 size) {
 	r_return_val_if_fail (f && name && *name, NULL);
+	if (f->mask) {
+		off &= f->mask;
+	}
 
 	bool is_new = false;
 	char *itemname = filter_item_name (name);
 	if (!itemname) {
 		return NULL;
 	}
+	// this should never happen because the name is filtered before..
+	if (!r_name_check (itemname)) {
+		eprintf ("Invalid flag name '%s'\n", name);
+		return NULL;
+	}
 
 	RFlagItem *item = r_flag_get (f, itemname);
 	free (itemname);
@@ -785,6 +819,12 @@ R_API int r_flag_rename(RFlag *f, RFlagItem *item, const char *name) {
 	return update_flag_item_name (f, item, name, false);
 }
 
+R_API void r_flag_item_set_type(RFlagItem *fi, const char *type) {
+	r_return_if_fail (fi && type);
+	free (fi->type);
+	fi->type = strdup (type);
+}
+
 /* unset the given flag item.
  * returns true if the item is successfully unset, false otherwise.
  *
@@ -905,6 +945,7 @@ R_API void r_flag_bind(RFlag *f, RFlagBind *fb) {
 	fb->exist_at = r_flag_exist_at;
 	fb->get = r_flag_get;
 	fb->get_at = r_flag_get_at;
+	fb->get_list = r_flag_get_list;
 	fb->set = r_flag_set;
 	fb->unset = r_flag_unset;
 	fb->unset_name = r_flag_unset_name;
