@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2011-2017 - pancake */
+/* radare - LGPL - Copyright 2011-2021 - pancake */
 
 #include <r_fs.h>
 #include <r_lib.h>
@@ -28,84 +28,38 @@ static RFSFile* fs_posix_open(RFSRoot *root, const char *path, bool create) {
 	return file;
 }
 
-static bool fs_posix_read(RFSFile *file, ut64 addr, int len) {
-	free (file->data);
-	file->data = (void*)r_file_slurp_range (file->name, 0, len, NULL);
-	return false;
+static int fs_posix_read(RFSFile *file, ut64 addr, int len) {
+	R_FREE (file->data);
+	char *abspath = r_str_newf ("%s/%s", file->path, file->name);
+	if (abspath) {
+		file->data = (void*)r_file_slurp_range (abspath, 0, len, NULL);
+		free (abspath);
+		return len;
+	}
+	return 0;
 }
 
 static void fs_posix_close(RFSFile *file) {
-	//fclose (file->ptr);
+	// fclose (file->ptr);
 }
 
 static RList *fs_posix_dir(RFSRoot *root, const char *path, int view /*ignored*/) {
-	RList *list;
-	char fullpath[4096];
+	RListIter *iter;
 	struct stat st;
-#if __WINDOWS__
-	WIN32_FIND_DATAW entry;
-	HANDLE fh;
-	wchar_t *wcpath;
-	char *wctocbuff;
-	wchar_t directory[MAX_PATH];
-#else
-	struct dirent *de;
-	DIR *dir;
-#endif
-	list = r_list_new ();
-	if (!list) {
-		return NULL;
-	}
-#if __WINDOWS__
-	wcpath = r_utf8_to_utf16 (path);
-	if (!wcpath) {
-		return NULL;
-	}
-	swprintf (directory, _countof (directory), L"%ls\\*.*", wcpath);
-	fh = FindFirstFileW (directory, &entry);
-	if (fh == INVALID_HANDLE_VALUE) {
-		free (wcpath);
-		return NULL;
-	}
-	do {
-		if ((wctocbuff = r_utf16_to_utf8 (entry.cFileName))) {
-			RFSFile *fsf = r_fs_file_new (NULL, wctocbuff);
-			if (!fsf) {
-				r_list_free (list);
-				FindClose (fh);
-				return NULL;
-			}
-			fsf->type = 'f';
-			snprintf (fullpath, sizeof (fullpath)-1, "%s/%s", path, wctocbuff);
-			if (!stat (fullpath, &st)) {
-				fsf->type = S_ISDIR (st.st_mode)?'d':'f';
-				fsf->time = st.st_atime;
-			} else {
-				fsf->type = 'f';
-				fsf->time = 0;
-			}
-			r_list_append (list, fsf);
-			free (wctocbuff);
-		}
-
-	} while (FindNextFileW (fh, &entry));
-	FindClose (fh);
-#else
-	dir = opendir (path);
-	if (!dir) {
-		r_list_free (list);
-		return NULL;
-	}
-	while ((de = readdir (dir))) {
-		RFSFile *fsf = r_fs_file_new (NULL, de->d_name);
+	char *file;
+	RList *list = r_list_newf ((RListFree)r_fs_file_free);
+	RList *files = r_sys_dir (path);
+	r_list_foreach (files, iter, file) {
+		RFSFile *fsf = r_fs_file_new (NULL, file);
 		if (!fsf) {
 			r_list_free (list);
-			closedir (dir);
+			r_list_free (files);
 			return NULL;
 		}
+		char *fp = r_str_newf ("%s/%s", path, file);
+		fsf->path = fp;
 		fsf->type = 'f';
-		snprintf (fullpath, sizeof (fullpath)-1, "%s/%s", path, de->d_name);
-		if (!stat (fullpath, &st)) {
+		if (!stat (fp, &st)) {
 			fsf->type = S_ISDIR (st.st_mode)?'d':'f';
 			fsf->time = st.st_atime;
 		} else {
@@ -114,13 +68,12 @@ static RList *fs_posix_dir(RFSRoot *root, const char *path, int view /*ignored*/
 		}
 		r_list_append (list, fsf);
 	}
-	closedir (dir);
-#endif
+	r_list_free (files);
 	return list;
 }
 
 static int fs_posix_mount(RFSRoot *root) {
-	root->ptr = NULL; // XXX: TODO
+	root->ptr = NULL;
 	return true;
 }
 
