@@ -21,16 +21,18 @@ extern "C" {
 #include <r_util/r_file.h>
 #include <r_vector.h>
 #include <sdb.h>
-#include <sdb/ht_up.h>
+#include <ht_up.h>
 
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #if __UNIX__
+#ifndef __wasi__
 #include <termios.h>
-#include <sys/ioctl.h>
 #include <sys/wait.h>
+#endif
+#include <sys/ioctl.h>
 #include <sys/socket.h>
 #endif
 #if __WINDOWS__
@@ -40,6 +42,9 @@ extern "C" {
 # ifndef ENABLE_VIRTUAL_TERMINAL_PROCESSING
 # define ENABLE_VIRTUAL_TERMINAL_PROCESSING 0x0004
 # endif
+# ifndef ENABLE_VIRTUAL_TERMINAL_INPUT
+# define ENABLE_VIRTUAL_TERMINAL_INPUT 0x0200
+# endif
 #else
 #include <unistd.h>
 #endif
@@ -56,6 +61,7 @@ extern "C" {
 #define R_CONS_GREP_WORDS 10
 #define R_CONS_GREP_WORD_SIZE 64
 #define R_CONS_GREP_TOKENS 64
+#define R_CONS_GREP_COUNT 10
 
 R_LIB_VERSION_HEADER(r_cons);
 
@@ -102,75 +108,10 @@ typedef struct r_cons_grep_t {
 	int neg;
 	int begin;
 	int end;
-	int icase;
+	bool icase;
+	bool ascart;
 } RConsGrep;
 
-#if 0
-// TODO Might be better than using r_cons_pal_get_i
-// And have smaller RConsPrintablePalette and RConsPalette
-enum {
-	R_CONS_PAL_0x00 = 0,
-	R_CONS_PAL_0x7f,
-	R_CONS_PAL_0xff,
-	R_CONS_PAL_ARGS,
-	R_CONS_PAL_BIN,
-	R_CONS_PAL_BTEXT,
-	R_CONS_PAL_CALL,
-	R_CONS_PAL_CJMP,
-	R_CONS_PAL_CMP,
-	R_CONS_PAL_COMMENT,
-	R_CONS_PAL_CREG,
-	R_CONS_PAL_FLAG,
-	R_CONS_PAL_FLINE,
-	R_CONS_PAL_FLOC,
-	R_CONS_PAL_FLOW,
-	R_CONS_PAL_FLOW2,
-	R_CONS_PAL_FNAME,
-	R_CONS_PAL_HELP,
-	R_CONS_PAL_INPUT,
-	R_CONS_PAL_INVALID,
-	R_CONS_PAL_JMP,
-	R_CONS_PAL_LABEL,
-	R_CONS_PAL_MATH,
-	R_CONS_PAL_MOV,
-	R_CONS_PAL_NOP,
-	R_CONS_PAL_NUM,
-	R_CONS_PAL_OFFSET,
-	R_CONS_PAL_OTHER,
-	R_CONS_PAL_POP,
-	R_CONS_PAL_PROMPT,
-	R_CONS_PAL_PUSH,
-	R_CONS_PAL_CRYPTO,
-	R_CONS_PAL_REG,
-	R_CONS_PAL_RESET,
-	R_CONS_PAL_RET,
-	R_CONS_PAL_SWI,
-	R_CONS_PAL_TRAP,
-	R_CONS_PAL_AI_READ,
-	R_CONS_PAL_AI_WRITE,
-	R_CONS_PAL_AI_EXEC,
-	R_CONS_PAL_AI_SEQ,
-	R_CONS_PAL_AI_ASCII,
-	R_CONS_PAL_AI_UNMAP,
-	R_CONS_PAL_GUI_CFLOW,
-	R_CONS_PAL_GUI_DATAOFFSET,
-	R_CONS_PAL_GUI_BACKGROUND,
-	R_CONS_PAL_GUI_ALT_BACKGROUND,
-	R_CONS_PAL_GUI_BORDER,
-	R_CONS_PAL_LINEHL,
-	R_CONS_PAL_GRAPH_BOX,
-	R_CONS_PAL_GRAPH_BOX2,
-	R_CONS_PAL_GRAPH_BOX3,
-	R_CONS_PAL_GRAPH_BOX4,
-	R_CONS_PAL_GRAPH_TRUE,
-	R_CONS_PAL_GRAPH_FALSE,
-	R_CONS_PAL_GRAPH_TRUFAE,
-	R_CONS_PAL_GRAPH_TRACED,
-	R_CONS_PAL_GRAPH_CURRENT,
-	R_CONS_PAL_LAST
-};
-#endif
-
 enum { ALPHA_RESET = 0x00, ALPHA_FG = 0x01, ALPHA_BG = 0x02, ALPHA_FGBG = 0x03 };
 enum { R_CONS_ATTR_BOLD = 1u << 1,
        R_CONS_ATTR_DIM = 1u << 2,
@@ -415,8 +356,20 @@ typedef struct r_cons_canvas_t {
 #define RUNE_CURVE_CORNER_BR "╯"
 #define RUNE_CURVE_CORNER_BL "╰"
 #define RUNE_LONG_LINE_HORIZ "―"
-#define UTF_CIRCLE "\u25EF"
-#define UTF_BLOCK "\u2588"
+#define R_UTF8_CIRCLE "\u25EF"
+#define R_UTF8_BLOCK "\u2588"
+
+// Emoji
+#define R_UTF8_POLICE_CARS_REVOLVING_LIGHT "🚨"
+#define R_UTF8_WHITE_HEAVY_CHECK_MARK "✅"
+#define R_UTF8_SEE_NO_EVIL_MONKEY "🙈"
+#define R_UTF8_SKULL_AND_CROSSBONES "☠"
+#define R_UTF8_KEYBOARD "⌨"
+#define R_UTF8_LEFT_POINTING_MAGNIFYING_GLASS "🔍"
+#define R_UTF8_DOOR "🚪"
+
+// Variation Selectors
+#define R_UTF8_VS16 "\xef\xb8\x8f"
 
 typedef char *(*RConsEditorCallback)(void *core, const char *file, const char *str);
 typedef int (*RConsClickCallback)(void *core, int x, int y);
@@ -424,21 +377,26 @@ typedef void (*RConsBreakCallback)(void *core);
 typedef void *(*RConsSleepBeginCallback)(void *core);
 typedef void (*RConsSleepEndCallback)(void *core, void *user);
 typedef void (*RConsQueueTaskOneshot)(void *core, void *task, void *user);
+typedef void (*RConsFunctionKey)(void *core, int fkey);
 
 typedef enum { COLOR_MODE_DISABLED = 0, COLOR_MODE_16, COLOR_MODE_256, COLOR_MODE_16M } RConsColorMode;
 
 typedef struct r_cons_context_t {
 	RConsGrep grep;
 	RStack *cons_stack;
-	char *buffer;
-	int buffer_len;
-	int buffer_sz;
-
+	char *buffer; // TODO: replace with RStrBuf
+	size_t buffer_len;
+	size_t buffer_sz;
+	RStrBuf *error; // r_cons_eprintf / r_cons_errstr / r_cons_errmode
+	int errmode;
 	bool breaked;
+	bool was_breaked;
 	RStack *break_stack;
 	RConsEvent event_interrupt;
 	void *event_interrupt_data;
 	int cmd_depth;
+	int cmd_str_depth;
+	bool noflush;
 
 	// Used for per-task logging redirection
 	RLogCallback log_callback; // TODO: RList of callbacks
@@ -453,14 +411,30 @@ typedef struct r_cons_context_t {
 	int color_mode;
 	RConsPalette cpal;
 	RConsPrintablePalette pal;
+
+	RList *sorted_lines;
+	RList *unsorted_lines;
+	int sorted_column; // -1
+	bool demo;
+	bool is_html;
+	bool was_html;
+	bool grep_color;
+	bool grep_highlight;
+	bool filter;
+	bool use_tts;
+	bool flush;
 } RConsContext;
 
 #define HUD_BUF_SIZE 512
 
+typedef struct {
+	int x;
+	int y;
+} RConsCursorPos;
+
 typedef struct r_cons_t {
 	RConsContext *context;
 	char *lastline;
-	int is_html;
 	int lines;
 	int rows;
 	int echo; // dump to stdout in realtime
@@ -471,7 +445,8 @@ typedef struct r_cons_t {
 	int fix_rows;
 	int fix_columns;
 	bool break_lines;
-	int noflush;
+	int optimize;
+	// move into Completion
 	bool show_autocomplete_widget;
 	FILE *fdin; // FILE? and then int ??
 	int fdout; // only used in pipe.c :?? remove?
@@ -487,12 +462,16 @@ typedef struct r_cons_t {
 	RConsSleepEndCallback cb_sleep_end;
 	RConsClickCallback cb_click;
 	RConsQueueTaskOneshot cb_task_oneshot;
+	RConsFunctionKey cb_fkey;
 
 	void *user; // Used by <RCore*>
-#if __UNIX__
+#if __UNIX__ && !__wasi__
 	struct termios term_raw, term_buf;
 #elif __WINDOWS__
-	DWORD term_raw, term_buf;
+	DWORD term_raw, term_buf, term_xterm;
+	UINT old_cp;
+	bool bCtrl;
+	bool is_arrow;
 #endif
 	RNum *num;
 	/* Pager (like more or less) to use if the output doesn't fit on the
@@ -503,15 +482,12 @@ typedef struct r_cons_t {
 	bool enable_highlight;
 	int null; // if set, does not show anything
 	int mouse;
-	int is_wine;
+	int is_wine; // -1, 0, 1
 	struct r_line_t *line;
 	const char **vline;
 	int refcnt;
 	R_DEPRECATE bool newline;
-#if __WINDOWS__
-	int ansicon;
-#endif
-	bool flush;
+	int vtmode;
 	bool use_utf8; // use utf8 features
 	bool use_utf8_curvy; // use utf8 curved corners
 	bool dotted_lines;
@@ -519,31 +495,16 @@ typedef struct r_cons_t {
 	int pagesize;
 	char *break_word;
 	int break_word_len;
-	ut64 timeout;
-	bool grep_color;
-	bool grep_highlight;
-	bool use_tts;
-	bool filter;
+	ut64 timeout; // must come from r_time_now_mono()
 	char* (*rgbstr)(char *str, size_t sz, ut64 addr);
 	bool click_set;
 	int click_x;
 	int click_y;
-	bool onestream;
 	bool show_vals;		// show which section in Vv
 	// TODO: move into instance? + avoid unnecessary copies
+	RConsCursorPos cpos;
 } RCons;
 
-// XXX THIS MUST BE A SINGLETON AND WRAPPED INTO RCons */
-/* XXX : global variables? or a struct with a singleton? */
-//extern FILE *stdin_fd;
-//extern FILE *r_cons_stdin_fd;
-//extern int r_cons_stdout_fd;
-//extern int r_cons_stdout_file;
-//extern char *r_cons_filterline;
-//extern char *r_cons_teefile;
-// not needed anymoar
-//extern int (*r_cons_user_fgets)(char *buf, int len);
-
 #define R_CONS_KEY_F1 0xf1
 #define R_CONS_KEY_F2 0xf2
 #define R_CONS_KEY_F3 0xf3
@@ -561,12 +522,15 @@ typedef struct r_cons_t {
 
 #define R_CONS_CLEAR_LINE "\x1b[2K\r"
 #define R_CONS_CLEAR_SCREEN "\x1b[2J\r"
-#define R_CONS_ADD_NEWLINES "\x1b[2J"
 #define R_CONS_CLEAR_FROM_CURSOR_TO_END "\x1b[0J\r"
 
 #define R_CONS_CURSOR_SAVE "\x1b[s"
 #define R_CONS_CURSOR_RESTORE "\x1b[u"
 #define R_CONS_GET_CURSOR_POSITION "\x1b[6n"
+#define R_CONS_CURSOR_UP "\x1b[A"
+#define R_CONS_CURSOR_DOWN "\x1b[B"
+#define R_CONS_CURSOR_RIGHT "\x1b[C"
+#define R_CONS_CURSOR_LEFT "\x1b[D"
 
 #define Color_BLINK        "\x1b[5m"
 #define Color_INVERT       "\x1b[7m"
@@ -749,6 +713,7 @@ typedef struct r_cons_canvas_line_style_t {
 	int color;
 	int symbol;
 	int dot_style;
+	const char *ansicolor;
 } RCanvasLineStyle;
 
 // UTF-8 symbols indexes
@@ -801,6 +766,7 @@ R_API void r_cons_canvas_line_square_defined (RConsCanvas *c, int x, int y, int
 R_API void r_cons_canvas_line_back_edge (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int ybendpoint1, int xbendpoint, int ybendpoint2, int isvert);
 R_API RCons *r_cons_new(void);
 R_API RCons *r_cons_singleton(void);
+R_API RConsContext *r_cons_context(void);
 R_API RCons *r_cons_free(void);
 R_API char *r_cons_lastline(int *size);
 R_API char *r_cons_lastline_utf8_ansi_len(int *len);
@@ -808,29 +774,54 @@ R_API void r_cons_set_click(int x, int y);
 R_API bool r_cons_get_click(int *x, int *y);
 
 typedef void (*RConsBreak)(void *);
-R_API void r_cons_break_end(void);
 R_API bool r_cons_is_breaked(void);
+R_API bool r_cons_was_breaked(void);
 R_API bool r_cons_is_interactive(void);
 R_API bool r_cons_default_context_is_interactive(void);
-R_API void r_cons_break_timeout(int timeout);
-R_API void r_cons_breakword(const char *s);
 R_API void *r_cons_sleep_begin(void);
 R_API void r_cons_sleep_end(void *user);
 
+/* ^C */
+R_API void r_cons_break_push(RConsBreak cb, void *user);
+R_API void r_cons_break_pop(void);
+R_API void r_cons_break_clear(void);
+R_API void r_cons_breakword(const char *s);
+R_API void r_cons_break_end(void);
+R_API void r_cons_break_timeout(int timeout);
+
 /* pipe */
 R_API int r_cons_pipe_open(const char *file, int fdn, int append);
 R_API void r_cons_pipe_close(int fd);
 
 #if __WINDOWS__
-R_API bool r_cons_is_ansicon(void);
+R_API int r_cons_is_vtcompat(void);
 R_API void r_cons_w32_clear(void);
 R_API void r_cons_w32_gotoxy(int fd, int x, int y);
 R_API int r_cons_w32_print(const char *ptr, int len, bool vmode);
-R_API int r_cons_win_printf(bool vmode, const char *fmt, ...);
-R_API int r_cons_win_eprintf(bool vmode, const char *fmt, ...);
+R_API int r_cons_win_printf(bool vmode, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
+R_API int r_cons_win_eprintf(bool vmode, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API int r_cons_win_vhprintf(DWORD hdl, bool vmode, const char *fmt, va_list ap);
 #endif
 
+#if 0
+
+Flush Print Buffer
+  0     0     0     null
+  0     0     1     quiet
+  0     1     0     echo
+  0     1     1     buffer
+  1     0     1     flush
+
+#endif
+
+enum {
+	R_CONS_ERRMODE_NULL,   // no buffer no print = null
+	R_CONS_ERRMODE_QUIET,  // buffer no print = quiet
+	R_CONS_ERRMODE_ECHO,   // no buffer, print = like eprintf()
+	R_CONS_ERRMODE_BUFFER, // no buffer, print = like eprintf()
+	R_CONS_ERRMODE_FLUSH,  // no buffer, print = like eprintf + log
+};
+
 R_API void r_cons_push(void);
 R_API void r_cons_pop(void);
 R_API RConsContext *r_cons_context_new(R_NULLABLE RConsContext *parent);
@@ -841,14 +832,16 @@ R_API bool r_cons_context_is_main(void);
 R_API void r_cons_context_break(RConsContext *context);
 R_API void r_cons_context_break_push(RConsContext *context, RConsBreak cb, void *user, bool sig);
 R_API void r_cons_context_break_pop(RConsContext *context, bool sig);
-R_API void r_cons_break_push(RConsBreak cb, void *user);
-R_API void r_cons_break_pop(void);
-R_API void r_cons_break_clear(void);
 
 /* control */
 R_API char *r_cons_editor(const char *file, const char *str);
 R_API void r_cons_reset(void);
 R_API void r_cons_reset_colors(void);
+R_API char *r_cons_errstr(void);
+R_API void r_cons_errmode(int mode);
+R_API void r_cons_errmodes(const char *mode);
+R_API int r_cons_eprintf(const char *format, ...);
+R_API void r_cons_eflush(void);
 R_API void r_cons_print_clear(void);
 R_API void r_cons_echo(const char *msg);
 R_API void r_cons_zero(void);
@@ -862,6 +855,7 @@ R_API void r_cons_stdout_open(const char *file, int append);
 R_API int  r_cons_stdout_set_fd(int fd);
 R_API void r_cons_gotoxy(int x, int y);
 R_API int r_cons_get_cur_line(void);
+R_API void r_cons_line(int x, int y, int x2, int y2, int ch);
 R_API void r_cons_show_cursor(int cursor);
 R_API char *r_cons_swap_ground(const char *col);
 R_API bool r_cons_drop(int n);
@@ -873,7 +867,7 @@ R_API void r_cons_set_utf8(bool b);
 R_API void r_cons_grep(const char *grep);
 
 /* output */
-R_API int r_cons_printf(const char *format, ...);
+R_API int r_cons_printf(const char *format, ...) R_PRINTF_CHECK(1, 2);
 R_API void r_cons_printf_list(const char *format, va_list ap);
 R_API void r_cons_strcat(const char *str);
 R_API void r_cons_strcat_at(const char *str, int x, char y, int w, int h);
@@ -881,7 +875,8 @@ R_API void r_cons_strcat_at(const char *str, int x, char y, int w, int h);
 R_API void r_cons_println(const char* str);
 
 R_API void r_cons_strcat_justify(const char *str, int j, char c);
-R_API int r_cons_memcat(const char *str, int len);
+R_API void r_cons_printat(const char *str, int x, char y);
+R_API int r_cons_write(const char *str, int len);
 R_API void r_cons_newline(void);
 R_API void r_cons_filter(void);
 R_API void r_cons_flush(void);
@@ -894,9 +889,10 @@ R_API void r_cons_memset(char ch, int len);
 R_API void r_cons_visual_flush(void);
 R_API void r_cons_visual_write(char *buffer);
 R_API bool r_cons_is_utf8(void);
+R_API bool r_cons_is_windows(void);
 R_API void r_cons_cmd_help(const char * help[], bool use_color);
 R_API void r_cons_log_stub(const char *output, const char *funcname, const char *filename,
- unsigned int lineno, unsigned int level, const char *tag, const char *fmtstr, ...);
+ unsigned int lineno, unsigned int level, const char *tag, const char *fmtstr, ...) R_PRINTF_CHECK(7, 8);
 
 
 /* input */
@@ -926,7 +922,7 @@ R_API char *r_cons_rgb_tostring(ut8 r, ut8 g, ut8 b);
 R_API void r_cons_pal_list(int rad, const char *arg);
 R_API void r_cons_pal_show(void);
 R_API int r_cons_get_size(int *rows);
-R_API bool r_cons_isatty(void);
+R_API bool r_cons_is_tty(void);
 R_API int r_cons_get_cursor(int *rows);
 R_API int r_cons_arrow_to_hjkl(int ch);
 R_API char *r_cons_html_filter(const char *ptr, int *newlen);
@@ -936,6 +932,8 @@ R_API void r_cons_rainbow_new(RConsContext *ctx, int sz);
 
 R_API int r_cons_fgets(char *buf, int len, int argc, const char **argv);
 R_API char *r_cons_hud(RList *list, const char *prompt);
+R_API char *r_cons_hud_line(RList *list, const char *prompt);
+R_API char *r_cons_hud_line_string(const char *s);
 R_API char *r_cons_hud_path(const char *path, int dir);
 R_API char *r_cons_hud_string(const char *s);
 R_API char *r_cons_hud_file(const char *f);
@@ -959,10 +957,10 @@ R_API void r_cons_color(int fg, int r, int g, int b);
 
 R_API RColor r_cons_color_random(ut8 alpha);
 R_API void r_cons_invert(int set, int color);
-R_API bool r_cons_yesno(int def, const char *fmt, ...);
+R_API bool r_cons_yesno(int def, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API char *r_cons_input(const char *msg);
 R_API char *r_cons_password(const char *msg);
-R_API void r_cons_set_cup(int enable);
+R_API bool r_cons_set_cup(bool enable);
 R_API void r_cons_column(int c);
 R_API int r_cons_get_column(void);
 R_API char *r_cons_message(const char *msg);
@@ -973,6 +971,24 @@ R_API void r_cons_bind(RConsBind *bind);
 R_API const char* r_cons_get_rune(const ut8 ch);
 #endif
 
+/* pixel.c */
+typedef struct {
+	int w;
+	int h;
+	ut8 *buf;
+	size_t buf_size;
+} RConsPixel;
+
+R_API RConsPixel *r_cons_pixel_new(int w, int h);
+R_API void r_cons_pixel_free(RConsPixel *p);
+R_API void r_cons_pixel_flush(RConsPixel *p, int sx, int sy);
+R_API char *r_cons_pixel_drain(RConsPixel *p);
+R_API ut8 r_cons_pixel_get(RConsPixel *p, int x, int y);
+R_API void r_cons_pixel_set(RConsPixel *p, int x, int y, ut8 v);
+R_API void r_cons_pixel_sets(RConsPixel *p, int x, int y, const char *s);
+R_API void r_cons_pixel_fill(RConsPixel *p, int _x, int _y, int w, int h, int v);
+R_API char *r_cons_pixel_tostring(RConsPixel *p);
+
 /* r_line */
 #define R_LINE_BUFSIZE 4096
 #define R_LINE_HISTSIZE 256
@@ -996,10 +1012,12 @@ typedef struct r_selection_widget_t {
 
 typedef struct r_line_hist_t {
 	char **data;
+	char *match;
 	int size;
 	int index;
 	int top;
 	int autosave;
+	bool do_setup_match;
 } RLineHistory;
 
 typedef struct r_line_buffer_t {
@@ -1044,15 +1062,17 @@ struct r_line_t {
 	RLineHistoryUpCb cb_history_up;
 	RLineHistoryDownCb cb_history_down;
 	RLineEditorCb cb_editor;
+	// RLineFunctionKeyCb cb_fkey;
+	RConsFunctionKey cb_fkey;
 	/* state , TODO: use more bool */
-	int echo;
-	int has_echo;
+	bool echo;
 	char *prompt;
 	RList/*<str>*/ *kill_ring;
 	int kill_ring_ptr;
 	char *clipboard;
-	int disable;
+	bool disable;
 	void *user;
+	bool histfilter;
 	int (*hist_up)(void *user);
 	int (*hist_down)(void *user);
 	char *contents;
@@ -1066,9 +1086,7 @@ struct r_line_t {
 	RLineHud *hud;
 	RList *sdbshell_hist;
 	RListIter *sdbshell_hist_iter;
-#if __WINDOWS__
-	int ansicon;
-#endif
+	int vtmode;
 }; /* RLine */
 
 #ifdef R_API
@@ -1088,7 +1106,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user);
 
 R_API int r_line_hist_load(const char *file);
 R_API int r_line_hist_add(const char *line);
-R_API int r_line_hist_save(const char *file);
+R_API bool r_line_hist_save(const char *file);
 R_API int r_line_hist_label(const char *label, void (*cb)(const char*));
 R_API void r_line_label_show(void);
 R_API int r_line_hist_list(void);
@@ -1170,7 +1188,7 @@ typedef struct r_panels_t {
 	RPanelsMenu *panels_menu;
 	Sdb *db;
 	Sdb *rotate_db;
-	Sdb *almighty_db;
+	Sdb *modal_db;
 	HtPP *mht;
 	RPanelsMode mode;
 	RPanelsFun fun;
@@ -1196,15 +1214,18 @@ typedef struct r_panels_root_t {
 	RPanelsRootState root_state;
 } RPanelsRoot;
 
-#ifdef __cplusplus
+
+#ifdef __sun
+static inline void cfmakeraw(struct termios *tm) {
+	tm->c_cflag &= ~(CSIZE | PARENB);
+	tm->c_cflag |= CS8;
+	tm->c_iflag &= ~(IMAXBEL | IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL | IXON);
+	tm->c_oflag &= ~OPOST;
+	tm->c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
 }
 #endif
 
-#if ONE_STREAM_HACK && !R_UTIL_SRC && !R_SEARCH_SRC && !R_FLAG_SRC && !R_BIN_SRC \
-	&& !R_JAVA_SRC && !R_ASM_SRC && !R_EGG_SRC
-R_API int r_cons_onestream_printf(const char *format, ...);
-#undef eprintf
-#define eprintf(...) r_cons_onestream_printf(__VA_ARGS__)
+#ifdef __cplusplus
+}
 #endif
-
 #endif
