@@ -1,6 +1,10 @@
 #ifndef R2_DRX_H
 #define R2_DRX_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 enum {
   DRX_API_LIST = 0,
   DRX_API_GET_BP = 1,
@@ -12,4 +16,8 @@ enum {
 #define NUM_DRX_REGISTERS 8
 #endif
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
