@@ -27,15 +27,16 @@ typedef struct r_lang_plugin_t {
 	const char *name;
 	const char *alias;
 	const char *desc;
+	const char *example;
 	const char *license;
 	const char **help;
 	const char *ext;
-	int (*init)(RLang *user);
+	bool (*init)(RLang *user);
 	bool (*setup)(RLang *user);
-	int (*fini)(RLang *user);
-	int (*prompt)(RLang *user);
-	int (*run)(RLang *user, const char *code, int len);
-	int (*run_file)(RLang *user, const char *file);
+	bool (*fini)(RLang *user);
+	bool (*prompt)(RLang *user);
+	bool (*run)(RLang *user, const char *code, int len);
+	bool (*run_file)(RLang *user, const char *file);
 	int (*set_argv)(RLang *user, int argc, char **argv);
 } RLangPlugin;
 
@@ -52,14 +53,13 @@ R_API bool r_lang_setup(RLang *lang);
 R_API bool r_lang_add(RLang *lang, RLangPlugin *foo);
 R_API bool r_lang_list(RLang *lang);
 R_API bool r_lang_use(RLang *lang, const char *name);
-R_API int r_lang_run(RLang *lang, const char *code, int len);
-R_API int r_lang_run_string(RLang *lang, const char *code);
+R_API bool r_lang_run(RLang *lang, const char *code, int len);
+R_API bool r_lang_run_string(RLang *lang, const char *code);
 /* TODO: user_ptr must be deprecated */
 R_API void r_lang_set_user_ptr(RLang *lang, void *user);
 R_API bool r_lang_set_argv(RLang *lang, int argc, char **argv);
-R_API int r_lang_run(RLang *lang, const char *code, int len);
-R_API int r_lang_run_file(RLang *lang, const char *file);
-R_API int r_lang_prompt(RLang *lang);
+R_API bool r_lang_run_file(RLang *lang, const char *file);
+R_API bool r_lang_prompt(RLang *lang);
 R_API void r_lang_plugin_free(RLangPlugin *p); // XXX
 R_API RLangPlugin *r_lang_get_by_name(RLang *lang, const char *name);
 R_API RLangPlugin *r_lang_get_by_extension(RLang *lang, const char *ext);
