@@ -43,7 +43,7 @@ struct r_buf_t {
 	void *priv;
 	ut8 *whole_buf;
 	bool readonly;
-	int Oxff_priv;
+	ut8 Oxff_priv;
 	int refctr;
 };
 
@@ -77,6 +77,7 @@ R_API st64 r_buf_append_string(RBuffer *b, const char *str);
 R_API bool r_buf_append_buf(RBuffer *b, RBuffer *a);
 R_API bool r_buf_append_bytes(RBuffer *b, const ut8 *buf, ut64 length);
 R_API bool r_buf_append_nbytes(RBuffer *b, ut64 length);
+R_API bool r_buf_append_ut8(RBuffer *b, ut8 n);
 R_API bool r_buf_append_ut16(RBuffer *b, ut16 n);
 R_API bool r_buf_append_buf_slice(RBuffer *b, RBuffer *a, ut64 offset, ut64 size);
 R_API bool r_buf_append_ut32(RBuffer *b, ut32 n);
@@ -86,6 +87,7 @@ R_API st64 r_buf_insert_bytes(RBuffer *b, ut64 addr, const ut8 *buf, ut64 length
 R_API char *r_buf_to_string(RBuffer *b);
 R_API char *r_buf_get_string(RBuffer *b, ut64 addr);
 R_API st64 r_buf_read(RBuffer *b, ut8 *buf, ut64 len);
+R_API ut8 *r_buf_read_all(RBuffer *b, int *blen);
 R_API ut8 r_buf_read8(RBuffer *b);
 R_API st64 r_buf_fread(RBuffer *b, ut8 *buf, const char *fmt, int n);
 R_API st64 r_buf_read_at(RBuffer *b, ut64 addr, ut8 *buf, ut64 len);
@@ -100,12 +102,12 @@ R_API st64 r_buf_fwrite_at(RBuffer *b, ut64 addr, const ut8 *buf, const char *fm
 // WARNING: this function should be used with care because it may allocate the
 // entire buffer in memory. Consider using the r_buf_read* APIs instead and read
 // only the chunks you need.
-R_API const ut8 *r_buf_data(RBuffer *b, ut64 *size);
+R_DEPRECATE R_API const ut8 *r_buf_data(RBuffer *b, ut64 *size);
 R_API ut64 r_buf_size(RBuffer *b);
 R_API bool r_buf_resize(RBuffer *b, ut64 newsize);
 R_API RBuffer *r_buf_ref(RBuffer *b);
 R_API void r_buf_free(RBuffer *b);
-R_API bool r_buf_fini(RBuffer *b);
+R_API void r_buf_fini(RBuffer *b);
 R_API RList *r_buf_nonempty_list(RBuffer *b);
 
 static inline ut16 r_buf_read_be16(RBuffer *b) {
