@@ -12,13 +12,25 @@ typedef struct {
 	ut64 n;
 } RNumCalcValue;
 
+typedef union {
+	ut16   u16;
+	ut32   u32;
+	ut64   u64;
+	st16   s16;
+	st32   s32;
+	st64   s64;
+	float  f32;
+	double f64;
+	/* long double f80; */
+} RNumFloat;
+
 typedef enum {
 	RNCNAME, RNCNUMBER, RNCEND, RNCINC, RNCDEC,
+	RNCLT, RNCGT, // comparison operators
 	RNCPLUS='+', RNCMINUS='-', RNCMUL='*', RNCDIV='/', RNCMOD='%',
-	//RNCXOR='^', RNCOR='|', RNCAND='&',
-	RNCNEG='~', RNCAND='&', RNCORR='|', RNCXOR='^',
+	RNCNEG='~', RNCAND='&', RNCOR='|', RNCXOR='^',
 	RNCPRINT=';', RNCASSIGN='=', RNCLEFTP='(', RNCRIGHTP=')',
-	RNCSHL='<', RNCSHR = '>', RNCROL = '#', RNCROR = '$'
+	RNCSHL='<', RNCSHR = '>', RNCROL = '#', RNCROR = '$',
 } RNumCalcToken;
 
 typedef struct r_num_calc_t {
@@ -61,17 +73,17 @@ R_API const char *r_num_get_name(RNum *num, ut64 n);
 R_API char* r_num_as_string(RNum *___, ut64 n, bool printable_only);
 R_API ut64 r_num_tail(RNum *num, ut64 addr, const char *hex);
 R_API ut64 r_num_tail_base(RNum *num, ut64 addr, ut64 off);
+R_API bool r_num_segaddr(ut64 addr, ut64 sb, int sg, ut32 *a, ut32 *b);
 R_API void r_num_minmax_swap(ut64 *a, ut64 *b);
 R_API void r_num_minmax_swap_i(int *a, int *b); // XXX this can be a cpp macro :??
 R_API ut64 r_num_math(RNum *num, const char *str);
 R_API ut64 r_num_get(RNum *num, const char *str);
 R_API int r_num_to_bits(char *out, ut64 num);
-R_API int r_num_to_trits(char *out, ut64 num);	//Rename this please
+R_API int r_num_to_ternary(char *out, ut64 num);
 R_API int r_num_rand(int max);
 R_API void r_num_irand(void);
-R_API ut16 r_num_ntohs(ut16 foo);
 R_API ut64 r_get_input_num_value(RNum *num, const char *input_value);
-R_API int r_is_valid_input_num_value(RNum *num, const char *input_value);
+R_API bool r_is_valid_input_num_value(RNum *num, const char *input_value);
 R_API int r_num_between(RNum *num, const char *input_value);
 R_API bool r_num_is_op(const char c);
 R_API int r_num_str_len(const char *str);
@@ -80,6 +92,7 @@ R_API RList *r_num_str_split_list(char *str);
 R_API void *r_num_dup(ut64 n);
 R_API double r_num_cos(double a);
 R_API double r_num_sin(double a);
+R_API size_t r_num_bit_count(ut32 val);
 R_API double r_num_get_float(RNum *num, const char *str);
 
 static inline st64 r_num_abs(st64 num) {
