@@ -34,13 +34,46 @@ extern "C" {
 #define R_PRINT_FLAGS_BGFILL   0x00100000
 #define R_PRINT_FLAGS_SECTION  0x00200000
 
+/*
+
+ 1 2   1  8   x  xx  xx  x   xx   x  xx  xx  xx
+ 3 4   2 16   x   x   x  xx  x   x    x  xx  xx
+ 5 6   4 32   x  x    x   x   x  xx  x   xx   x
+ 7 8  1< 2<   x  xx  xx   x  x   xx  x   xx  x
+
+*/
+
+#define $00 1
+#define $01 8
+#define $10 2
+#define $11 16
+#define $20 4
+#define $21 32
+#define $30 (1 << 8)
+#define $31 (2 << 8)
+#define BRAILE_ONE $00+$01+$11+$21+$31
+#define BRAILE_TWO $00+$01+$11+$20+$30+$31
+#define BRAILE_TRI $00+$01+$11+$21+$30+$31
+#define BRAILE_FUR $00+$10+$11+$21+$31
+#define BRAILE_FIV $00+$01+$10+$21+$30
+#define BRAILE_SIX $01+$10+$20+$21+$30+$31
+#define BRAILE_SEV $00+$01+$11+$20+$30
+#define BRAILE_EIG $00+$01+$10+$11+$20+$21+$30+$31
+#define BRAILE_NIN $00+$01+$10+$11+$21+$30
+
+typedef struct {
+	char str[4];
+} RBraile;
+
+R_API RBraile r_print_braile(int u);
+
 typedef int (*RPrintZoomCallback)(void *user, int mode, ut64 addr, ut8 *bufz, ut64 size);
 typedef const char *(*RPrintNameCallback)(void *user, ut64 addr);
 typedef int (*RPrintSizeCallback)(void *user, ut64 addr);
 typedef char *(*RPrintCommentCallback)(void *user, ut64 addr);
 typedef const char *(*RPrintSectionGet)(void *user, ut64 addr);
 typedef const char *(*RPrintColorFor)(void *user, ut64 addr, bool verbose);
-typedef char *(*RPrintHasRefs)(void *user, ut64 addr, bool verbose);
+typedef char *(*RPrintHasRefs)(void *user, ut64 addr, int mode);
 
 typedef struct r_print_zoom_t {
 	ut8 *buf;
@@ -65,7 +98,6 @@ typedef struct r_print_t {
 	bool scr_prompt;
 	int (*disasm)(void *p, ut64 addr);
 	PrintfCallback oprintf;
-	int interrupt;
 	int big_endian;
 	int width;
 	int limit;
@@ -80,7 +112,6 @@ typedef struct r_print_t {
 	int ocur;
 	int cols;
 	int flags;
-	int seggrn;
 	bool use_comments;
 	int addrmod;
 	int col;
@@ -112,6 +143,7 @@ typedef struct r_print_t {
 	const char *strconv_mode;
 	RList *vars;
 	char io_unalloc_ch;
+	bool show_offset;
 
 	// when true it uses row_offsets
 	bool calc_row_offsets;
@@ -125,6 +157,14 @@ typedef struct r_print_t {
 	// represents the first not-visible offset on the screen
 	// (only when in visual disasm mode)
 	ut64 screen_bounds;
+	// HACK: Used to temporarily disable the progress bar when it doesn't make sense to have it,
+	// eg. when setting the default flag tags on startup. Does not override scr.progressbar.
+	bool enable_progressbar;
+	RCharset *charset;
+
+	// segmented memory addressing
+	int seggrn;
+	int segbas;
 } RPrint;
 
 #ifdef R_API
@@ -137,7 +177,7 @@ R_API void r_print_set_is_interrupted_cb(RPrintIsInterruptedCallback cb);
 
 /* ... */
 R_API char *r_print_hexpair(RPrint *p, const char *str, int idx);
-R_API void r_print_hex_from_bin (RPrint *p, char *bin_str);
+R_API void r_print_hex_from_bin(RPrint *p, char *bin_str);
 R_API RPrint *r_print_new(void);
 R_API RPrint *r_print_free(RPrint *p);
 R_API bool r_print_mute(RPrint *p, int x);
@@ -145,9 +185,9 @@ R_API void r_print_set_flags(RPrint *p, int _flags);
 R_API void r_print_unset_flags(RPrint *p, int flags);
 R_API void r_print_addr(RPrint *p, ut64 addr);
 R_API void r_print_section(RPrint *p, ut64 at);
-R_API void r_print_columns (RPrint *p, const ut8 *buf, int len, int height);
+R_API void r_print_columns(RPrint *p, const ut8 *buf, int len, int height);
 R_API void r_print_hexii(RPrint *p, ut64 addr, const ut8 *buf, int len, int step);
-R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int base, int step, int zoomsz);
+R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int base, int step, size_t zoomsz);
 R_API void r_print_hexdump_simple(const ut8 *buf, int len);
 R_API int r_print_jsondump(RPrint *p, const ut8 *buf, int len, int wordsize);
 R_API void r_print_hexpairs(RPrint *p, ut64 addr, const ut8 *buf, int len);
@@ -182,8 +222,7 @@ R_API void r_print_code(RPrint *p, ut64 addr, const ut8 *buf, int len, char lang
 R_API int r_print_format_struct_size(RPrint *p, const char *format, int mode, int n);
 R_API int r_print_format(RPrint *p, ut64 seek, const ut8* buf, const int len, const char *fmt, int elem, const char *setval, char *field);
 R_API const char *r_print_format_byname(RPrint *p, const char *name);
-R_API int r_print_format_length(const char *fmt);
-R_API void r_print_offset(RPrint *p, ut64 off, int invert, int opt, int dec, int delta, const char *label);
+R_API void r_print_offset(RPrint *p, ut64 off, int invert, int delta, const char *label);
 R_API void r_print_offset_sg(RPrint *p, ut64 off, int invert, int offseg, int seggrn, int offdec, int delta, const char *label);
 #define R_PRINT_STRING_WIDE 1
 #define R_PRINT_STRING_ZEROEND 2
@@ -200,11 +239,12 @@ R_API int r_print_date_get_now(RPrint *p, char *str);
 R_API void r_print_zoom(RPrint *p, void *user, RPrintZoomCallback cb, ut64 from, ut64 to, int len, int maxlen);
 R_API void r_print_zoom_buf(RPrint *p, void *user, RPrintZoomCallback cb, ut64 from, ut64 to, int len, int maxlen);
 R_API void r_print_progressbar(RPrint *pr, int pc, int _cols);
+R_API void r_print_progressbar_with_count(RPrint *pr, unsigned int pc, unsigned int total, int _cols, bool reset_line);
 R_API void r_print_portionbar(RPrint *p, const ut64 *portions, int n_portions);
 R_API void r_print_rangebar(RPrint *p, ut64 startA, ut64 endA, ut64 min, ut64 max, int cols);
 R_API char * r_print_randomart(const ut8 *dgst_raw, ut32 dgst_raw_len, ut64 addr);
-R_API void r_print_2bpp_row(RPrint *p, ut8 *buf);
-R_API void r_print_2bpp_tiles(RPrint *p, ut8 *buf, ut32 tiles);
+R_API void r_print_2bpp_row(RPrint *p, ut8 *buf, const char **colors);
+R_API void r_print_2bpp_tiles(RPrint *p, ut8 *buf, size_t buflen, ut32 tiles, const char **colors);
 R_API char * r_print_colorize_opcode(RPrint *print, char *p, const char *reg, const char *num, bool partial_reset, ut64 func_addr);
 R_API const char * r_print_color_op_type(RPrint *p, ut32 anal_type);
 R_API void r_print_set_interrupted(int i);
@@ -216,6 +256,7 @@ R_API int r_print_pie(RPrint *p, ut64 *values, int nvalues, int size);
 
 R_API const char* r_print_rowlog(RPrint *print, const char *str);
 R_API void r_print_rowlog_done(RPrint *print, const char *str);
+R_API void r_print_graphline(RPrint *print, const ut8 *buf, size_t len);
 
 // WIP
 R_API int r_print_unpack7bit(const char *src, char *dest);
