@@ -1,5 +1,9 @@
 #ifndef R_STR_UTIL_H
 #define R_STR_UTIL_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
  
 #define IS_NULLSTR(x) (!(x) || !*(x))
 #define IS_WHITECHAR(x) ((x) == ' ' || (x)=='\t' || (x) == '\n' || (x) == '\r')
@@ -14,4 +18,8 @@
 #define IS_UPPER(c) ((c) >= 'A' && (c) <= 'Z')
 #define IS_LOWER(c) ((c) >= 'a' && (c) <= 'z')
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif //  R_STR_UTIL_H
