@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2008-2019 - pancake */
+/* radare - LGPL - Copyright 2008-2021 - pancake */
 
 #include <r_userconf.h>
 #include <r_io.h>
@@ -6,16 +6,14 @@
 #include <stdio.h>
 
 typedef struct r_io_mmo_t {
-	char * filename;
+	char *filename;
 	int mode;
 	int perm;
 	int fd;
-	int opened;
+	bool rawio;
 	bool nocache;
-	ut8 modified;
 	RBuffer *buf;
 	RIO * io_backref;
-	int rawio;
 } RIOMMapFileObj;
 
 static int __io_posix_open(const char *file, int perm, int mode) {
@@ -38,7 +36,7 @@ static int __io_posix_open(const char *file, int perm, int mode) {
 		fd = r_sandbox_open (file, O_RDONLY | O_BINARY, 0);
 	}
 #else
-	const int posixFlags = (perm & R_PERM_W) ? (perm & R_PERM_CREAT)
+	const size_t posixFlags = (perm & R_PERM_W) ? (perm & R_PERM_CREAT)
 			? (O_RDWR | O_CREAT) : O_RDWR : O_RDONLY;
 	fd = r_sandbox_open (file, posixFlags, mode);
 #endif
@@ -73,7 +71,7 @@ static int r_io_def_mmap_refresh_def_mmap_buf(RIOMMapFileObj *mmo) {
 	st64 sz = r_file_size (mmo->filename);
 	if (sz > ST32_MAX) {
 		// Do not use mmap if the file is huge
-		mmo->rawio = 1;
+		mmo->rawio = true;
 	}
 	if (mmo->rawio) {
 		mmo->fd = __io_posix_open (mmo->filename, mmo->perm, mmo->mode);
@@ -82,21 +80,21 @@ static int r_io_def_mmap_refresh_def_mmap_buf(RIOMMapFileObj *mmo) {
 			fcntl (mmo->fd, F_NOCACHE, 1);
 #endif
 		}
-		return (mmo->fd != -1);
+		return mmo->fd != -1;
 	}
 	mmo->buf = r_buf_new_mmap (mmo->filename, mmo->perm);
 	if (mmo->buf) {
 		r_io_def_mmap_seek (io, mmo, cur, SEEK_SET);
 		return true;
 	} else {
-		mmo->rawio = 1;
+		mmo->rawio = true;
 		mmo->fd = __io_posix_open (mmo->filename, mmo->perm, mmo->mode);
 		if (mmo->nocache) {
 #ifdef F_NOCACHE
 			fcntl (mmo->fd, F_NOCACHE, 1);
 #endif
 		}
-		return (mmo->fd != -1);
+		return mmo->fd != -1;
 	}
 	return false;
 }
@@ -121,7 +119,7 @@ RIOMMapFileObj *r_io_def_mmap_create_new_file(RIO  *io, const char *filename, in
 	}
 	mmo->nocache = r_str_startswith (filename, "nocache://");
 	if (mmo->nocache) {
-		filename += strlen ("nocache://");;
+		filename += strlen ("nocache://");
 	}
 	mmo->filename = strdup (filename);
 	mmo->perm = perm;
@@ -140,7 +138,7 @@ RIOMMapFileObj *r_io_def_mmap_create_new_file(RIO  *io, const char *filename, in
 		return NULL;
 	}
 	if (!r_io_def_mmap_refresh_def_mmap_buf (mmo)) {
-		mmo->rawio = 1;
+		mmo->rawio = true;
 		if (!r_io_def_mmap_refresh_def_mmap_buf (mmo)) {
 			r_io_def_mmap_free (mmo);
 			mmo = NULL;
@@ -149,14 +147,7 @@ RIOMMapFileObj *r_io_def_mmap_create_new_file(RIO  *io, const char *filename, in
 	return mmo;
 }
 
-static int r_io_def_mmap_close(RIODesc *fd) {
-	r_return_val_if_fail (fd && fd->data, -1);
-	r_io_def_mmap_free ((RIOMMapFileObj *) fd->data);
-	fd->data = NULL;
-	return 0;
-}
-
-static bool r_io_def_mmap_check_default (const char *filename) {
+static bool r_io_def_mmap_check_default(const char *filename) {
 	r_return_val_if_fail (filename && *filename, false);
 	if (r_str_startswith (filename, "file://")) {
 		filename += strlen ("file://");
@@ -178,35 +169,6 @@ static int r_io_def_mmap_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 		return -1;
 	}
 	if (mmo->rawio) {
-		if (fd->obsz) {
-			// only do aligned reads in aligned offsets
-			const int aligned = fd->obsz;
-			ut64 a_off = io->off - (io->off % aligned);
-			int a_delta = io->off - a_off;
-			if (a_delta < 0) {
-				memset (buf, 0xff, count);
-				return -1;
-			}
-			int a_count = count + (aligned - (count % aligned));
-
-			char *a_buf = malloc (a_count + aligned);
-			if (a_buf) {
-				int i;
-				memset (a_buf, 0xff, a_count + aligned);
-				if (lseek (mmo->fd, a_off, SEEK_SET) < 0) {
-					free (a_buf);
-					return -1;
-				}
-				for (i = 0; i < a_count ; i += aligned) {
-					(void)read (mmo->fd, a_buf+i, aligned);
-				}
-				memcpy (buf, a_buf+a_delta, count);
-			} else {
-				memset (buf, 0xff, count);
-			}
-			free (a_buf);
-			return count;
-		}
 		if (lseek (mmo->fd, io->off, SEEK_SET) < 0) {
 			return -1;
 		}
@@ -219,6 +181,7 @@ static int r_io_def_mmap_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	if (r < 0) {
 		return r;
 	}
+	r_buf_seek (mmo->buf, r, R_BUF_CUR);
 	io->off += r;
 	return r;
 }
@@ -230,39 +193,6 @@ static int r_io_def_mmap_write(RIO *io, RIODesc *fd, const ut8 *buf, int count)
 	ut64 addr = io->off;
 	RIOMMapFileObj *mmo = fd->data;
 	if (mmo->rawio) {
-		if (fd->obsz) {
-			char *a_buf;
-			int a_count;
-			// only do aligned reads in aligned offsets
-			const int aligned = fd->obsz; //512; // XXX obey fd->obsz? or it may be too slow? 128K..
-			//ut64 a_off = (io->off >> 9 ) << 9; //- (io->off & aligned);
-			ut64 a_off = io->off - (io->off % aligned); //(io->off >> 9 ) << 9; //- (io->off & aligned);
-			int a_delta = io->off - a_off;
-			if (a_delta < 0) {
-				return -1;
-			}
-			a_count = count + (aligned - (count % aligned));
-			a_buf = malloc (a_count + aligned);
-			if (a_buf) {
-				int i;
-				memset (a_buf, 0xff, a_count+aligned);
-				for (i = 0; i < a_count; i += aligned) {
-					(void)lseek (mmo->fd, a_off + i, SEEK_SET);
-					(void)read (mmo->fd, a_buf + i, aligned);
-				}
-				memcpy (a_buf+a_delta, buf, count);
-				for (i = 0; i < a_count; i += aligned) {
-					(void)lseek (mmo->fd, a_off + i, SEEK_SET);
-					len = write (mmo->fd, a_buf + i, aligned);
-					if (len != aligned) {
-						free (a_buf);
-						return len;
-					}
-				}
-			}
-			free (a_buf);
-			return count;
-		}
 		if (lseek (mmo->fd, addr, 0) < 0) {
 			return -1;
 		}
@@ -297,6 +227,16 @@ static int r_io_def_mmap_write(RIO *io, RIODesc *fd, const ut8 *buf, int count)
 
 static RIODesc *r_io_def_mmap_open(RIO *io, const char *file, int perm, int mode) {
 	r_return_val_if_fail (io && file, NULL);
+#if __wasi__
+	RIOPlugin *_plugin = r_io_plugin_resolve (io, (const char *)"slurp://", false);
+	if (!_plugin || !_plugin->open) {
+		return NULL;
+	}
+	char *uri = r_str_newf ("slurp://%s", file);
+	RIODesc *d = _plugin->open (io, uri, perm, mode);
+	free (uri);
+	return d;
+#else
 	RIOMMapFileObj *mmo = r_io_def_mmap_create_new_file (io, file, perm, mode);
 	if (!mmo) {
 		return NULL;
@@ -311,11 +251,7 @@ static RIODesc *r_io_def_mmap_open(RIO *io, const char *file, int perm, int mode
 		free (oldname);
 	}
 	return d;
-}
-
-static ut64 r_io_def_mmap_lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
-	r_return_val_if_fail (fd && fd->data, UT64_MAX);
-	return r_io_def_mmap_seek (io, (RIOMMapFileObj *)fd->data, offset, whence);
+#endif
 }
 
 static int r_io_def_mmap_truncate(RIOMMapFileObj *mmo, ut64 size) {
@@ -350,11 +286,17 @@ static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int len) {
 }
 
 static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
-	return r_io_def_mmap_lseek (io, fd, offset, whence);
+	r_return_val_if_fail (fd && fd->data, UT64_MAX);
+	return r_io_def_mmap_seek (io, (RIOMMapFileObj *)fd->data, offset, whence);
 }
 
-static int __close(RIODesc *fd) {
-	return r_io_def_mmap_close (fd);
+static bool __close(RIODesc *fd) {
+	r_return_val_if_fail (fd, false);
+	if (fd->data) {
+		r_io_def_mmap_free ((RIOMMapFileObj *) fd->data);
+		fd->data = NULL;
+	}
+	return true;
 }
 
 static bool __resize(RIO *io, RIODesc *fd, ut64 size) {
@@ -387,7 +329,7 @@ RIOPlugin r_io_plugin_default = {
 	.close = __close,
 	.read = __read,
 	.check = __plugin_open_default,
-	.lseek = __lseek,
+	.seek = __lseek,
 	.write = __write,
 	.resize = __resize,
 #if __UNIX__
