@@ -5,11 +5,6 @@
 #include <r_util.h>
 #include <r_util/r_print.h>
 
-#if __UNIX__
-#include <errno.h>
-#include <fcntl.h>
-#endif
-
 #define USE_OWNTIMER 1
 #if USE_OWNTIMER
 #include "io_gprobe.h"
@@ -26,7 +21,12 @@
 #include <windows.h>
 #else
 
-#if __linux__ ||  __APPLE__ || __OpenBSD__ || __FreeBSD__ || __NetBSD__ || __DragonFly__
+#if __UNIX__
+#include <errno.h>
+#include <fcntl.h>
+#endif
+
+#if HAVE_PTY
 #include <sys/ioctl.h>
 #include <termios.h>
 #else
@@ -37,6 +37,9 @@
 
 #define GPROBE_SIZE (1LL << 32)
 #define GPROBE_I2C_ADDR 0x6e
+#ifndef B115200
+#define B115200 0010002
+#endif
 
 #define I2C_SLAVE 0x0703
 
@@ -1077,21 +1080,17 @@ static int __read (RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	return has_read;
 }
 
-static int __close (RIODesc *fd) {
-	RIOGprobe *gprobe;
-
+static bool __close(RIODesc *fd) {
 	if (!fd || !fd->data) {
-		return -1;
+		return false;
 	}
-	gprobe = (RIOGprobe *)fd->data;
-
+	RIOGprobe *gprobe = (RIOGprobe *)fd->data;
 	sp_close (&gprobe->gport);
 	R_FREE (fd->data);
-
-	return 0;
+	return true;
 }
 
-static ut64 __lseek (RIO *io, RIODesc *fd, ut64 offset, int whence) {
+static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
 	RIOGprobe *gprobe;
 	if (!fd || !fd->data) {
 		return offset;
@@ -1114,7 +1113,7 @@ static ut64 __lseek (RIO *io, RIODesc *fd, ut64 offset, int whence) {
 	return offset;
 }
 
-static bool __plugin_open (RIO *io, const char *pathname, bool many) {
+static bool __plugin_open(RIO *io, const char *pathname, bool many) {
 	return pathname && r_str_startswith (pathname, "gprobe://") && strlen (pathname + strlen ("gprobe://"));
 }
 
@@ -1160,13 +1159,11 @@ static RIODesc *__open (RIO *io, const char *pathname, int rw, int mode) {
 	return NULL;
 }
 
-static char *__system (RIO *io, RIODesc *fd, const char *cmd) {
-	RIOGprobe *gprobe;
-
+static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
 	if (!fd || !fd->data) {
 		return NULL;
 	}
-	gprobe = (RIOGprobe *)fd->data;
+	RIOGprobe *gprobe = (RIOGprobe *)fd->data;
 
 	if (!cmd[0] || cmd[0] == '?' || !strcmp (cmd, "help")) {
 		printf ("Usage: =!cmd args\n"
@@ -1236,7 +1233,7 @@ RIOPlugin r_io_plugin_gprobe = {
 	.close = __close,
 	.read = __read,
 	.check = __plugin_open,
-	.lseek = __lseek,
+	.seek = __lseek,
 	.write = __write,
 	.system = __system,
 };
