@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2008-2017 - pancake */
+/* radare - LGPL - Copyright 2008-2021 - pancake */
 
 #include "r_io.h"
 #include "r_lib.h"
@@ -10,6 +10,7 @@ typedef struct {
 	ut8 *buf;
 	ut32 size;
 	ut64 offset;
+	bool has_changed;
 } RIOGzip;
 
 static inline ut32 _io_malloc_sz(RIODesc *desc) {
@@ -74,6 +75,8 @@ static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
 		count -= (_io_malloc_off (fd) + count -_io_malloc_sz (fd));
 	}
 	if (count > 0) {
+		RIOGzip *riom = fd->data;
+		riom->has_changed = true;
 		memcpy (_io_malloc_buf (fd) + _io_malloc_off (fd), buf, count);
 		_io_malloc_set_off (fd, _io_malloc_off (fd) + count);
 		return count;
@@ -92,7 +95,7 @@ static bool __resize(RIO *io, RIODesc *fd, ut64 count) {
 	}
 	new_buf = malloc (count);
 	if (!new_buf) {
-		return -1;
+		return false;
 	}
 	memcpy (new_buf, _io_malloc_buf (fd), R_MIN (count, mallocsz));
 	if (count > mallocsz) {
@@ -120,16 +123,17 @@ static int __read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	return count;
 }
 
-static int __close(RIODesc *fd) {
-	RIOGzip *riom;
+static bool __close(RIODesc *fd) {
 	if (!fd || !fd->data) {
-		return -1;
+		return false;
+	}
+	RIOGzip *riom = fd->data;
+	if (riom->has_changed) {
+		eprintf ("TODO: Writing changes into gzipped files is not yet supported\n");
 	}
-	riom = fd->data;
 	R_FREE (riom->buf);
 	R_FREE (fd->data);
-	eprintf ("TODO: Writing changes into gzipped files is not yet supported\n");
-	return 0;
+	return true;
 }
 
 static ut64 __lseek(RIO* io, RIODesc *fd, ut64 offset, int whence) {
@@ -163,10 +167,10 @@ static RIODesc *__open(RIO *io, const char *pathname, int rw, int mode) {
 		if (!mal) {
 			return NULL;
 		}
-		int len;
-		ut8 *data = (ut8*)r_file_slurp (pathname+7, &len);	//memleak here?
+		size_t len;
+		ut8 *data = (ut8 *)r_file_slurp (pathname+7, &len);	//memleak here?
 		int *size = (int*)&mal->size;
-		mal->buf = r_inflate (data, len, NULL, size);
+		mal->buf = r_inflate (data, (int)len, NULL, size);
 		if (mal->buf) {
 			return r_io_desc_new (io, &r_io_plugin_gzip, pathname, rw, mode, mal);
 		}
@@ -186,7 +190,7 @@ RIOPlugin r_io_plugin_gzip = {
 	.close = __close,
 	.read = __read,
 	.check = __plugin_open,
-	.lseek = __lseek,
+	.seek = __lseek,
 	.write = __write,
 	.resize = __resize,
 };
