@@ -65,10 +65,12 @@ static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
 		fclose (out);
 		return -1;
 	}
+	r_buf_seek (rih->rbuf, count, R_BUF_CUR);
 
 	/* disk write : process each sparse chunk */
 	//TODO : sort addresses + check overlap?
-	r_list_foreach (rih->rbuf->sparse, iter, rbs) {
+	RList *nonempty = r_buf_nonempty_list (rih->rbuf);
+	r_list_foreach (nonempty, iter, rbs) {
 		ut16 addl0 = rbs->from & 0xffff;
 		ut16 addh0 = rbs->from >> 16;
 		ut16 addh1 = rbs->to >> 16;
@@ -82,6 +84,7 @@ static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
 			//04 record (ext address)
 			if (fw04b (out, addh0) < 0) {
 				eprintf("ihex:write: file error\n");
+				r_list_free (nonempty);
 				fclose (out);
 				return -1;
 			}
@@ -90,6 +93,7 @@ static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
 			addl0 = 0;
 			if (fwblock (out, rbs->data, rbs->from, tsiz)) {
 				eprintf ("ihex:fwblock error\n");
+				r_list_free (nonempty);
 				fclose (out);
 				return -1;
 			}
@@ -97,17 +101,20 @@ static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
 		//04 record (ext address)
 		if (fw04b (out, addh1) < 0) {
 			eprintf ("ihex:write: file error\n");
+			r_list_free (nonempty);
 			fclose (out);
 			return -1;
 		}
 		//00 records (remaining data)
 		if (fwblock (out, rbs->data + tsiz, (addh1 << 16) | addl0, rbs->size - tsiz)) {
 			eprintf ("ihex:fwblock error\n");
+			r_list_free (nonempty);
 			fclose (out);
 			return -1;
 		}
 	}	//list_foreach
 
+	r_list_free (nonempty);
 	fprintf (out, ":00000001FF\n");
 	fclose (out);
 	out = NULL;
@@ -183,27 +190,31 @@ static int __read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	}
 	Rihex *rih = fd->data;
 	memset (buf, io->Oxff, count);
-	return r_buf_read_at (rih->rbuf, io->off, buf, count);
+	int r = r_buf_read_at (rih->rbuf, io->off, buf, count);
+	if (r >= 0) {
+		r_buf_seek (rih->rbuf, r, R_BUF_CUR);
+	}
+	return r;
 }
 
-static int __close(RIODesc *fd) {
+static bool __close(RIODesc *fd) {
 	if (!fd || !fd->data) {
-		return -1;
+		return false;
 	}
 	Rihex *rih = fd->data;
 	r_buf_free (rih->rbuf);
 	free (rih);
 	fd->data = NULL;
-	return 0;
+	return true;
 }
 
 static ut64 __lseek(struct r_io_t *io, RIODesc *fd, ut64 offset, int whence) {
-	Rihex *rih;
 	if (!fd || !fd->data) {
-		return -1;
+		return UT64_MAX;
 	}
-	rih = fd->data;
-	return r_buf_seek (rih->rbuf, offset, whence);
+	Rihex *rih = fd->data;
+	io->off = r_buf_seek (rih->rbuf, offset, whence);
+	return io->off;
 }
 
 static bool __plugin_open(RIO *io, const char *pathname, bool many) {
@@ -364,7 +375,7 @@ static bool ihex_parse(RBuffer *rbuf, char *str) {
 			str = eol;
 			break;
 		case 3:	//undefined rec. Just skip.
-		case 5:	//non-standard, sometimes "start linear adddress"
+		case 5:	//non-standard, sometimes "start linear address"
 			str = strchr (str + 1, ':');
 			break;
 		}
@@ -422,21 +433,22 @@ static bool __resize(RIO *io, RIODesc *fd, ut64 size) {
 
 RIOPlugin r_io_plugin_ihex = {
 	.name = "ihex",
-        .desc = "Intel HEX file (ihex://eeproms.hex)",
+	.desc = "Open intel HEX file",
+	.uris = "ihex://",
 	.license = "LGPL",
-        .open = __open,
-        .close = __close,
+	.open = __open,
+	.close = __close,
 	.read = __read,
-        .check = __plugin_open,
-	.lseek = __lseek,
+	.check = __plugin_open,
+	.seek = __lseek,
 	.write = __write,
 	.resize = __resize
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
-	.data = &r_io_plugin_hex,
+	.data = &r_io_plugin_ihex,
 	.version = R2_VERSION
 };
 #endif
