@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2017 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include <r_userconf.h>
 
@@ -8,7 +8,7 @@
 
 #if __APPLE__ && DEBUGGER
 
-static int __get_pid (RIODesc *desc);
+static int __get_pid(RIODesc *desc);
 #define EXCEPTION_PORT 0
 
 // NOTE: mach/mach_vm is not available for iOS
@@ -39,6 +39,13 @@ static int __get_pid (RIODesc *desc);
 
 #define R_MACH_MAGIC r_str_hash ("mach")
 
+typedef struct r_io_mach_data_t {
+	ut32 magic;
+	int pid;
+	int tid;
+	void *data;
+} RIOMachData;
+
 typedef struct {
 	task_t task;
 } RIOMach;
@@ -47,7 +54,7 @@ typedef struct {
 #define RIOMACH_TASK(x) (x ? ((RIOMach*)(x))->task : -1)
 */
 
-int RIOMACH_TASK(RIODescData *x) {
+int RIOMACH_TASK(RIOMachData *x) {
 	// TODO
 	return -1;
 }
@@ -110,7 +117,7 @@ static task_t pid_to_task(RIODesc *fd, int pid) {
 	static int old_pid = -1;
 	kern_return_t kr;
 
-	RIODescData *iodd = fd? (RIODescData *)fd->data: NULL;
+	RIOMachData *iodd = fd? (RIOMachData *)fd->data: NULL;
 	RIOMach *riom = NULL;
 	if (iodd) {
 		riom = iodd->data;
@@ -213,7 +220,7 @@ static int __read(RIO *io, RIODesc *desc, ut8 *buf, int len) {
 	vm_size_t size = 0;
 	int blen, err, copied = 0;
 	int blocksize = 32;
-	RIODescData *dd = (RIODescData *)desc->data;
+	RIOMachData *dd = (RIOMachData *)desc->data;
 	if (!io || !desc || !buf || !dd) {
 		return -1;
 	}
@@ -414,7 +421,7 @@ static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 		}
 		return NULL;
 	}
-	RIODescData *iodd = R_NEW0 (RIODescData);
+	RIOMachData *iodd = R_NEW0 (RIOMachData);
 	if (iodd) {
 		iodd->pid = pid;
 		iodd->tid = pid;
@@ -457,12 +464,11 @@ static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
 	return io->off;
 }
 
-static int __close(RIODesc *fd) {
+static bool __close(RIODesc *fd) {
 	if (!fd) {
 		return false;
 	}
-	RIODescData *iodd = fd->data;
-	kern_return_t kr;
+	RIOMachData *iodd = fd->data;
 	if (!iodd) {
 		return false;
 	}
@@ -470,7 +476,7 @@ static int __close(RIODesc *fd) {
 		return false;
 	}
 	task_t task = pid_to_task (fd, iodd->pid);
-	kr = mach_port_deallocate (mach_task_self (), task);
+	kern_return_t kr = mach_port_deallocate (mach_task_self (), task);
 	if (kr != KERN_SUCCESS) {
 		perror ("__close io_mach");
 	}
@@ -482,13 +488,10 @@ static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
 	if (!io || !fd || !cmd || !fd->data) {
 		return NULL;
 	}
-	RIODescData *iodd = fd->data;
+	RIOMachData *iodd = fd->data;
 	if (iodd->magic != R_MACH_MAGIC) {
 		return NULL;
 	}
-
-	task_t task = pid_to_task (fd, iodd->tid);
-	/* XXX ugly hack for testing purposes */
 	if (!strcmp (cmd, "")) {
 		return NULL;
 	}
@@ -496,6 +499,7 @@ static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
 		int perm = r_str_rwx (cmd + 4);
 		if (perm) {
 			int pagesize = tsk_pagesize (fd);
+			task_t task = pid_to_task (fd, iodd->tid);
 			tsk_setperm (io, task, io->off, pagesize, perm);
 		} else {
 			eprintf ("Usage: =!perm [rwx]\n");
@@ -503,26 +507,23 @@ static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
 		return NULL;
 	}
 	if (!strncmp (cmd, "pid", 3)) {
-		RIODescData *iodd = fd->data;
+		RIOMachData *iodd = fd->data;
 		RIOMach *riom = iodd->data;
-		const char *pidstr = cmd + 3;
-		int pid = -1;
-		if (*pidstr) {
-			pid = __get_pid (fd);
-			//return NULL;
-		} else {
-			eprintf ("%d\n", iodd->pid);
+		const char *pidstr = r_str_trim_head_ro (cmd + 3);
+		if (R_STR_ISEMPTY (pidstr)) {
+			io->cb_printf ("%d\n", iodd->pid);
 			return NULL;
 		}
+		int pid = __get_pid (fd);
 		if (!strcmp (pidstr, "0")) {
 			pid = 0;
 		} else {
 			pid = atoi (pidstr);
-			if (!pid) {
+			if (pid < 1) {
 				pid = -1;
 			}
 		}
-		if (pid != -1) {
+		if (pid >= 0) {
 			task_t task = pid_to_task (fd, pid);
 			if (task != -1) {
 				riom->task = task;
@@ -533,22 +534,21 @@ static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
 		}
 		eprintf ("io_mach_system: Invalid pid %d\n", pid);
 	} else {
-		eprintf ("Try: '=!pid' or '=!perm'\n");
+		eprintf ("Try: ':pid' or ':perm'\n");
 	}
 	return NULL;
 }
 
-static int __get_pid (RIODesc *desc) {
+static int __get_pid(RIODesc *desc) {
 	// dupe for ? r_io_desc_get_pid (desc);
-	if (!desc || !desc->data) {
-		return -1;
-	}
-	RIODescData *iodd = desc->data;
-	if (iodd) {
-		if (iodd->magic != R_MACH_MAGIC) {
-			return -1;
+	if (desc) {
+		RIOMachData *iodd = desc->data;
+		if (iodd) {
+			if (iodd->magic != R_MACH_MAGIC) {
+				return -1;
+			}
+			return iodd->pid;
 		}
-		return iodd->pid;
 	}
 	return -1;
 }
@@ -565,7 +565,7 @@ RIOPlugin r_io_plugin_mach = {
 	.getpid = __get_pid,
 	.gettid = __get_pid,
 	.check = __plugin_open,
-	.lseek = __lseek,
+	.seek = __lseek,
 	.system = __system,
 	.write = __write,
 	.isdbg = true
