@@ -76,15 +76,6 @@ RIOMMapFileObj *r_io_mmap_create_new_file(RIO  *io, const char *filename, int mo
 	return mmo;
 }
 
-static int r_io_mmap_close(RIODesc *fd) {
-	if (!fd || !fd->data) {
-		return -1;
-	}
-	r_io_mmap_free ((RIOMMapFileObj *) fd->data);
-	fd->data = NULL;
-	return 0;
-}
-
 static int r_io_mmap_check (const char *filename) {
 	return (filename && !strncmp (filename, "mmap://", 7) && *(filename + 7));
 }
@@ -98,7 +89,11 @@ static int r_io_mmap_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	if (r_buf_size (mmo->buf) < io->off) {
 		io->off = r_buf_size (mmo->buf);
 	}
-	return r_buf_read_at (mmo->buf, io->off, buf, count);
+	int r = r_buf_read_at (mmo->buf, io->off, buf, count);
+	if (r >= 0) {
+		r_buf_seek (mmo->buf, r, R_BUF_CUR);
+	}
+	return r;
 }
 
 static int r_io_mmap_write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
@@ -178,18 +173,23 @@ static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
 	return r_io_mmap_lseek (io, fd, offset, whence);
 }
 
-static int __close(RIODesc *fd) {
-	return r_io_mmap_close (fd);
+static bool __close(RIODesc *fd) {
+	if (!fd || !fd->data) {
+		return false;
+	}
+	r_io_mmap_free ((RIOMMapFileObj *) fd->data);
+	fd->data = NULL;
+	return true;
 }
 
 static bool __resize(RIO *io, RIODesc *fd, ut64 size) {
 	if (!fd || !fd->data) {
-		return -1;
+		return false;
 	}
 	return r_io_mmap_truncate ((RIOMMapFileObj*)fd->data, size);
 }
 
-struct r_io_plugin_t r_io_plugin_mmap = {
+RIOPlugin r_io_plugin_mmap = {
 	.name = "mmap",
 	.desc = "Open files using mmap",
 	.uris = "mmap://",
@@ -198,7 +198,7 @@ struct r_io_plugin_t r_io_plugin_mmap = {
 	.close = __close,
 	.read = __read,
 	.check = __plugin_open,
-	.lseek = __lseek,
+	.seek = __lseek,
 	.write = __write,
 	.resize = __resize,
 };
