@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2015-2019 - pancake */
+/* radare - LGPL - Copyright 2015-2021 - pancake */
 
 #include "r_io.h"
 #include "r_lib.h"
@@ -27,6 +27,7 @@ static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
 		snprintf (bufn, bufn_sz, "%s%d", i ? "," : "", buf[i]);
 		bufn += strlen (bufn);
 	}
+	//TODO PJ (?)
 	int len = snprintf (fmt, sizeof (fmt),
 		"{\"op\":\"write\",\"address\":%" PFMT64d ",\"data\":[%s]}",
 		io->off, bufnum);
@@ -42,7 +43,7 @@ static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
 	res = r2pipe_read (R2P (fd));
 	/* TODO: parse json back */
 	r = strstr (res, "result");
-	if (r) {
+	if (r && r[6]) {
 		count = atoi (r + 6 + 1);
 	}
 	free (res);
@@ -60,6 +61,7 @@ static int __read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	if (count > 1024) {
 		count = 1024;
 	}
+	//TODO PJ (?)
 	snprintf (fmt, sizeof (fmt),
 		"{\"op\":\"read\",\"address\":%"PFMT64d",\"count\":%d}",
 		io->off, count);
@@ -120,13 +122,13 @@ beach:
 	return rescount;
 }
 
-static int __close(RIODesc *fd) {
+static bool __close(RIODesc *fd) {
 	if (!fd || !fd->data) {
-		return -1;
+		return false;
 	}
 	r2pipe_close (fd->data);
 	fd->data = NULL;
-	return 0;
+	return true;
 }
 
 static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
@@ -186,7 +188,7 @@ RIOPlugin r_io_plugin_r2pipe = {
 	.close = __close,
 	.read = __read,
 	.check = __check,
-	.lseek = __lseek,
+	.seek = __lseek,
 	.write = __write,
 	.system = __system
 };
