@@ -23,14 +23,13 @@ static int w32__read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	return ReadFile (RIOW32_HANDLE (fd), buf, count, &ret, NULL)?ret:-1;
 }
 
-static int w32__close(RIODesc *fd) {
+static bool w32__close(RIODesc *fd) {
 	if (fd->data) {
-		// TODO: handle return value
 		CloseHandle (RIOW32_HANDLE (fd));
 		R_FREE (fd->data);
-		return 0;
+		return true;
 	}
-	return -1;
+	return false;
 }
 
 // TODO: handle filesize and so on
@@ -46,11 +45,14 @@ static bool w32__plugin_open(RIO *io, const char *pathname, bool many) {
 static RIODesc *w32__open(RIO *io, const char *pathname, int rw, int mode) {
 	if (!strncmp (pathname, "w32://", 6)) {
 		RIOW32 *w32 = R_NEW0 (RIOW32);
-		const char *filename = pathname+6;
+		if (!w32) {
+			return NULL;
+		}
+		const char *filename = pathname + 6;
 		LPTSTR filename_ = r_sys_conv_utf8_to_win (filename);
 		w32->hnd = CreateFile (filename_,
-			GENERIC_READ | rw?GENERIC_WRITE:0,
-			FILE_SHARE_READ | rw? FILE_SHARE_WRITE:0,
+			GENERIC_READ | (rw ? GENERIC_WRITE : 0),
+			FILE_SHARE_READ | FILE_SHARE_WRITE,
 			NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
 		free (filename_);
 		if (w32->hnd != INVALID_HANDLE_VALUE)
@@ -78,7 +80,7 @@ RIOPlugin r_io_plugin_w32 = {
 	.close = w32__close,
 	.read = w32__read,
 	.check = w32__plugin_open,
-	.lseek = w32__lseek,
+	.seek = w32__lseek,
 	.system = w32__system,
 	.write = w32__write,
 };
