@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2008-2016 - pancake */
+/* radare - LGPL - Copyright 2008-2021 - pancake */
 
 #include <r_userconf.h>
 
@@ -11,9 +11,9 @@
 
 #include <windows.h>
 #include <tlhelp32.h>
-#include <w32dbg_wrap.h>
+#include <r_util/r_w32dw.h>
 
-#define RIOW32DBG_PID(x) (((RIOW32Dbg*)x->data)->pi.dwProcessId)
+#define RW32Dw_PID(x) (((RW32Dw*)x->data)->pi.dwProcessId)
 
 #undef R_IO_NFDS
 #define R_IO_NFDS 2
@@ -29,9 +29,8 @@ static ut64 __find_next_valid_addr(HANDLE h, ut64 from, ut64 to) {
 	return from < to ? from : UT64_MAX;
 }
 
-static int debug_os_read_at(RIOW32Dbg *dbg, ut8 *buf, int len, ut64 addr) {
+static int debug_os_read_at(RW32Dw *dbg, ut8 *buf, int len, ut64 addr) {
 	SIZE_T ret = 0;
-	const int page_size = 0x1000;
 	if (!ReadProcessMemory (dbg->pi.hProcess, (void*)(size_t)addr, buf, len, &ret)
 		&& GetLastError () == ERROR_PARTIAL_COPY) {
 		int skipped = 0;
@@ -80,7 +79,7 @@ static int __read(RIO *io, RIODesc *fd, ut8 *buf, int len) {
 	return debug_os_read_at (fd->data, buf, len, io->off);
 }
 
-static int w32dbg_write_at(RIOW32Dbg *dbg, const ut8 *buf, int len, ut64 addr) {
+static int w32dbg_write_at(RW32Dw *dbg, const ut8 *buf, int len, ut64 addr) {
 	SIZE_T ret;
 	return 0 != WriteProcessMemory (dbg->pi.hProcess, (void *)(size_t)addr, buf, len, &ret)? len: 0;
 }
@@ -96,13 +95,6 @@ static bool __plugin_open(RIO *io, const char *file, bool many) {
 	return !strncmp (file, "w32dbg://", 9);
 }
 
-// mingw32 toolchain doesnt have this symbol
-static HANDLE (WINAPI *r2_OpenThread)(
-	DWORD dwDesiredAccess,
-	BOOL  bInheritHandle,
-	DWORD dwThreadId
-) = NULL;
-
 static int __w32_first_thread(int pid) {
 	HANDLE th;
 	HANDLE thid;
@@ -120,9 +112,7 @@ static int __w32_first_thread(int pid) {
 	do {
 		/* get all threads of process */
 		if (te32.th32OwnerProcessID == pid) {
-			r2_OpenThread = OpenThread;
-			thid = r2_OpenThread
-			? r2_OpenThread (THREAD_ALL_ACCESS, 0, te32.th32ThreadID) : NULL;
+			thid = OpenThread (THREAD_ALL_ACCESS, 0, te32.th32ThreadID);
 			if (!thid) {
 				r_sys_perror ("__w32_first_thread/OpenThread");
 				goto err_first_th;
@@ -137,36 +127,37 @@ err_first_th:
 	return pid;
 }
 
-static int __open_proc(RIO *io, RIOW32Dbg *dbg, bool attach) {
+static int __open_proc(RIO *io, int pid, bool attach) {
 	DEBUG_EVENT de;
 	int ret = -1;
-	HANDLE h_proc = OpenProcess (PROCESS_ALL_ACCESS, FALSE, dbg->pi.dwProcessId);
+	if (!io->dbgwrap) {
+		io->dbgwrap = r_w32dw_new ();
+	}
+
+	HANDLE h_proc = OpenProcess (PROCESS_ALL_ACCESS, FALSE, pid);
 
 	if (!h_proc) {
 		r_sys_perror ("__open_proc/OpenProcess");
 		goto att_exit;
 	}
-	if (!io->w32dbg_wrap) {
-		io->w32dbg_wrap = w32dbg_wrap_new ();
-	}
+	RW32Dw *wrap = (RW32Dw *)io->dbgwrap;
+	wrap->pi.dwProcessId = pid;
 	if (attach) {
-		/* Attach to the process */
-		w32dbg_wrap_instance *inst = io->w32dbg_wrap;
-		inst->params->type = W32_ATTACH;
-		inst->params->pid = dbg->pi.dwProcessId;
-		w32dbg_wrap_wait_ret (inst);
-		if (!w32dbgw_ret (inst)) {
-			w32dbgw_err (inst);
+		/* Attach to the process */	
+		wrap->params.type = W32_ATTACH;
+		r_w32dw_waitret (wrap);
+		if (!r_w32dw_ret (wrap)) {
+			r_w32dw_err (wrap);
 			r_sys_perror ("__open_proc/DebugActiveProcess");
 			goto att_exit;
 		}
 		/* catch create process event */
-		inst->params->type = W32_WAIT;
-		inst->params->wait.wait_time = 10000;
-		inst->params->wait.de = &de;
-		w32dbg_wrap_wait_ret (inst);
-		if (!w32dbgw_ret (inst)) {
-			w32dbgw_err (inst);
+		wrap->params.type = W32_WAIT;
+		wrap->params.wait.wait_time = 10000;
+		wrap->params.wait.de = &de;
+		r_w32dw_waitret (wrap);
+		if (!r_w32dw_ret (wrap)) {
+			r_w32dw_err (wrap);
 			r_sys_perror ("__open_proc/WaitForDebugEvent");
 			goto att_exit;
 		}
@@ -174,12 +165,12 @@ static int __open_proc(RIO *io, RIOW32Dbg *dbg, bool attach) {
 			eprintf ("exception code 0x%04x\n", (ut32)de.dwDebugEventCode);
 			goto att_exit;
 		}
-		dbg->winbase = (ut64)de.u.CreateProcessInfo.lpBaseOfImage;
+		wrap->winbase = (ut64)de.u.CreateProcessInfo.lpBaseOfImage;
+		wrap->pi.dwThreadId = de.dwThreadId;
+		wrap->pi.hThread = de.u.CreateProcessInfo.hThread;
 	}
-	dbg->inst = io->w32dbg_wrap;
-	dbg->pi.hProcess = h_proc;
-	dbg->pi.dwProcessId = dbg->pi.dwProcessId;
-	ret = dbg->pi.dwProcessId;
+	wrap->pi.hProcess = h_proc;
+	ret = wrap->pi.dwProcessId;
 att_exit:
 	if (ret == -1 && h_proc) {
 		CloseHandle (h_proc);
@@ -190,20 +181,19 @@ att_exit:
 static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 	if (__plugin_open (io, file, 0)) {
 		RIODesc *ret;
-		RIOW32Dbg *dbg = R_NEW0 (RIOW32Dbg);
-		if (!dbg) {
+		if (__open_proc (io, atoi (file + 9), !strncmp (file, "attach://", 9)) == -1) {
 			return NULL;
 		}
-		dbg->pi.dwProcessId = atoi (file + 9);
-		if (__open_proc (io, dbg, !strncmp (file, "attach://", 9)) == -1) {
-			free (dbg);
-			return NULL;
+		RW32Dw *wrap = io->dbgwrap;
+		if (!wrap->pi.dwThreadId) {
+			wrap->pi.dwThreadId = __w32_first_thread (wrap->pi.dwProcessId);
+		}
+		if (!wrap->pi.hThread) {
+			wrap->pi.hThread = OpenThread (THREAD_ALL_ACCESS, FALSE, wrap->pi.dwThreadId);
 		}
-		dbg->pi.dwThreadId = __w32_first_thread (dbg->pi.dwProcessId);
-		dbg->pi.hThread = OpenThread (THREAD_ALL_ACCESS, FALSE, dbg->pi.dwThreadId);
 		ret = r_io_desc_new (io, &r_io_plugin_w32dbg,
-				file, rw | R_PERM_X, mode, dbg);
-		ret->name = r_sys_pid_to_path (dbg->pi.dwProcessId);
+				file, rw | R_PERM_X, mode, wrap);
+		ret->name = r_sys_pid_to_path (wrap->pi.dwProcessId);
 		return ret;
 	}
 	return NULL;
@@ -224,15 +214,18 @@ static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
 	return io->off;
 }
 
-static int __close(RIODesc *fd) {
-	RIOW32Dbg *iop = fd->data;
-	iop->inst->params->type = W32_DETTACH;
-	w32dbg_wrap_wait_ret (iop->inst);
+static bool __close(RIODesc *fd) {
+	if (r_str_startswith (fd->uri, "attach://")) {
+		RW32Dw *wrap = fd->data;
+		wrap->params.type = W32_DETACH;
+		r_w32dw_waitret (wrap);
+		return true;
+	}
 	return false;
 }
 
 static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
-	RIOW32Dbg *iop = fd->data;
+	RW32Dw *wrap = fd->data;
 	//printf("w32dbg io command (%s)\n", cmd);
 	/* XXX ugly hack for testing purposes */
 	if (!strcmp (cmd, "")) {
@@ -240,16 +233,16 @@ static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
 	} else if (!strncmp (cmd, "pid", 3)) {
 		if (cmd[3] == ' ') {
 			int pid = atoi (cmd + 3);
-			if (pid > 0 && pid != iop->pi.dwThreadId && pid != iop->pi.dwProcessId) {
-				iop->pi.hThread = OpenThread (PROCESS_ALL_ACCESS, FALSE, pid);
-				if (iop->pi.hThread) {
-					iop->pi.dwThreadId = pid;
+			if (pid > 0 && pid != wrap->pi.dwThreadId && pid != wrap->pi.dwProcessId) {
+				HANDLE ht = OpenThread (PROCESS_ALL_ACCESS, FALSE, pid);
+				if (ht) {
+					wrap->pi.hThread = ht;
 				} else {
-					eprintf ("Cannot attach to %d\n", pid);
+					eprintf ("Cannot attach to %d (%s)\n", pid, cmd);
 				}
 			}
 		}
-		return r_str_newf ("%d", iop->pi.dwProcessId);
+		return r_str_newf ("%lu", wrap->pi.dwProcessId);
 	} else {
 		eprintf ("Try: '=!pid'\n");
 	}
@@ -257,22 +250,22 @@ static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
 }
 
 static int __getpid(RIODesc *fd) {
-	RIOW32Dbg *iow = (RIOW32Dbg *)(fd ? fd->data : NULL);
-	if (!iow) {
+	RW32Dw *wrap = (RW32Dw *)(fd ? fd->data : NULL);
+	if (!wrap) {
 		return -1;
 	}
-	return iow->pi.dwProcessId;
+	return wrap->pi.dwProcessId;
 }
 
 static int __gettid(RIODesc *fd) {
-	RIOW32Dbg *iow = (RIOW32Dbg *)(fd ? fd->data : NULL);
-	return iow? iow->pi.dwThreadId: -1;
+	RW32Dw *wrap = (RW32Dw *)(fd ? fd->data : NULL);
+	return wrap? wrap->pi.dwThreadId: -1;
 }
 
 static bool __getbase(RIODesc *fd, ut64 *base) {
-	RIOW32Dbg *iow = (RIOW32Dbg *)(fd ? fd->data : NULL);
-	if (base && iow) {
-		*base = iow->winbase;
+	RW32Dw *wrap = (RW32Dw *)(fd ? fd->data : NULL);
+	if (base && wrap) {
+		*base = wrap->winbase;
 		return true;
 	}
 	return false;
@@ -287,7 +280,7 @@ RIOPlugin r_io_plugin_w32dbg = {
 	.close = __close,
 	.read = __read,
 	.check = __plugin_open,
-	.lseek = __lseek,
+	.seek = __lseek,
 	.system = __system,
 	.write = __write,
 	.getpid = __getpid,
