@@ -29,7 +29,7 @@ R_PACKED (struct winedbg_x86_32 {
 });
 
 // TODO: make it vargarg...
-static char *runcmd (const char *cmd) {
+static char *runcmd(const char *cmd) {
 	char buf[4096] = {0};
 	if (cmd) {
 		r_socket_printf (gs, "%s\n", cmd);
@@ -39,7 +39,11 @@ static char *runcmd (const char *cmd) {
 	r_socket_block_time (gs, 1, timeout, 0);
 	while (true) {
 		memset (buf, 0, sizeof (buf));
-		r_socket_read (gs, (ut8*)buf, sizeof (buf) - 1); // NULL-terminate the string always
+		int rc = r_socket_read (gs, (ut8*)buf, sizeof (buf) - 1); // NULL-terminate the string always
+		if (rc == -1) {
+			break;
+		}
+		buf[sizeof (buf) - 1] = 0;
 		char *promptFound = strstr (buf, "Wine-dbg>");
 		if (promptFound) {
 			*promptFound = 0;
@@ -130,13 +134,14 @@ Wine-dbg>
 	return count;
 }
 
-static int __close(RIODesc *fd) {
+static bool __close(RIODesc *fd) {
 	if (!fd || !fd->data) {
-		return -1;
+		return false;
 	}
-	// XXX
-	r_sys_cmdf ("pkill rarun2");
-	return 0;
+#if __UNIX__
+	r_sys_cmdf ("pkill rarun2 2>/dev/null");
+#endif
+	return true;
 }
 
 static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
@@ -188,7 +193,7 @@ static void printcmd (RIO *io, const char *cmd) {
 	free (res);
 }
 
-static struct winedbg_x86_32 regState() {
+static struct winedbg_x86_32 regState(void) {
 	struct winedbg_x86_32 r = {0};
 	char *res = runcmd ("info reg");
 	if (res) {
@@ -291,7 +296,7 @@ const char *msg =
 "flg	rf	.1	.202	0\n"\
 "flg	vm	.1	.203	0\n";
 		return strdup (msg);
-	} else if (!strncmp (cmd, "dr*", 2)) {
+	} else if (!strncmp (cmd, "dr*", 3)) {
 		struct winedbg_x86_32 r = regState ();
 		io->cb_printf ("f eip = 0x%08x\n", r.eip);
 		io->cb_printf ("f esp = 0x%08x\n", r.esp);
@@ -368,7 +373,7 @@ RIOPlugin r_io_plugin_winedbg = {
 	.close = __close,
 	.read = __read,
 	.check = __plugin_open,
-	.lseek = __lseek,
+	.seek = __lseek,
 	.write = __write,
 	.system = __system,
 	.isdbg = true
