@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2015-2019 pancake */
+/* radare2 - LGPL - Copyright 2015-2020 pancake */
 
 #include "r_lib.h"
 #include "r_core.h"
@@ -10,35 +10,17 @@
 #include <process.h>
 #endif
 
-static int lang_pipe_run(RLang *lang, const char *code, int len);
-static int lang_pipe_file(RLang *lang, const char *file) {
-	return lang_pipe_run (lang, file, -1);
-}
-
 #if __WINDOWS__
-static HANDLE pipe = 0;
 static HANDLE myCreateChildProcess(const char * szCmdline) {
 	PROCESS_INFORMATION piProcInfo = {0};
 	STARTUPINFO siStartInfo = {0};
 	BOOL bSuccess = FALSE;
 	siStartInfo.cb = sizeof (STARTUPINFO);
 	siStartInfo.dwFlags |= STARTF_USESTDHANDLES;
+	siStartInfo.hStdInput = GetStdHandle (STD_INPUT_HANDLE);
+	siStartInfo.hStdOutput = GetStdHandle (STD_OUTPUT_HANDLE);
+	siStartInfo.hStdError = GetStdHandle (STD_ERROR_HANDLE);
 
-	SECURITY_ATTRIBUTES saAttr;
-	saAttr.nLength = sizeof (SECURITY_ATTRIBUTES);
-	saAttr.bInheritHandle = TRUE;
-	saAttr.lpSecurityDescriptor = NULL;
-	LPTSTR pipeName = calloc (128, sizeof (TCHAR)); 
-	GetEnvironmentVariable (TEXT ("R2PIPE_PATH"), pipeName, 128);
-
-	pipe = CreateFile (pipeName,
-				GENERIC_READ | GENERIC_WRITE,
-				FILE_SHARE_DELETE | FILE_SHARE_READ | FILE_SHARE_WRITE,
-				&saAttr, OPEN_EXISTING, 0, NULL);
-
-	siStartInfo.hStdOutput = pipe;
-	siStartInfo.hStdInput = pipe;
-	siStartInfo.hStdError = pipe;
 	LPTSTR cmdline_ = r_sys_conv_utf8_to_win (szCmdline);
 	bSuccess = CreateProcess (NULL, cmdline_, NULL, NULL,
 		TRUE, 0, NULL, NULL, &siStartInfo, &piProcInfo);
@@ -46,43 +28,53 @@ static HANDLE myCreateChildProcess(const char * szCmdline) {
 	return bSuccess ? piProcInfo.hProcess : NULL;
 }
 
-static volatile BOOL bStopPipeLoop = FALSE;
 static HANDLE hPipeInOut = NULL;
 static HANDLE hproc = NULL;
 #define PIPE_BUF_SIZE 8192
 
-static DWORD WINAPI WaitForProcThread(LPVOID lParam) {
-	WaitForSingleObject (hproc, INFINITE);
-	bStopPipeLoop = TRUE;
-	return 0;
-}
 static void lang_pipe_run_win(RLang *lang) {
 	CHAR buf[PIPE_BUF_SIZE];
 	BOOL bSuccess = TRUE;
 	int i, res = 0;
-	DWORD dwRead = 0, dwWritten = 0, dwLeft = 1;
+	DWORD dwRead = 0, dwWritten = 0, dwEvent;
+	HANDLE hRead = CreateEvent (NULL, TRUE, FALSE, NULL);
+	if (!hRead) {
+		r_sys_perror ("lang_pipe_run_win/CreateEvent hRead");
+		return;
+	}
+	HANDLE hWritten = CreateEvent (NULL, TRUE, FALSE, NULL);
+	if (!hWritten) {
+		r_sys_perror ("lang_pipe_run_win/CreateEvent hWritten");
+		CloseHandle (hRead);
+		return;
+	}
 	r_cons_break_push (NULL, NULL);
 	do {
 		if (r_cons_is_breaked ()) {
 			TerminateProcess (hproc, 0);
 			break;
 		}
+		OVERLAPPED oRead = { 0 };
+		oRead.hEvent = hRead;
 		memset (buf, 0, PIPE_BUF_SIZE);
-		while (bSuccess && (!dwLeft || !dwRead)) {
-			bSuccess = PeekNamedPipe (hPipeInOut, buf, PIPE_BUF_SIZE, &dwRead, NULL, &dwLeft);
-			if (bStopPipeLoop && (!dwLeft || !dwRead)) {
-				break;
-			}
+		ReadFile (hPipeInOut, buf, PIPE_BUF_SIZE, NULL, &oRead);
+		HANDLE hReadEvents[] = { hRead, hproc };
+		dwEvent = WaitForMultipleObjects (R_ARRAY_SIZE (hReadEvents), hReadEvents,
+		                                  FALSE, INFINITE);
+		if (dwEvent == WAIT_OBJECT_0 + 1) { // hproc
+			break;
+		} else if (dwEvent == WAIT_FAILED) {
+			r_sys_perror ("lang_pipe_run_win/WaitForMultipleObjects read");
+			break;
 		}
-		if (bSuccess && (dwRead || dwLeft)) {
-			bSuccess = ReadFile (hPipeInOut, buf, PIPE_BUF_SIZE, &dwRead, NULL);
+		bSuccess = GetOverlappedResult (hPipeInOut, &oRead, &dwRead, TRUE);
+		if (!bSuccess) {
+			break;
 		}
 		if (bSuccess && dwRead > 0) {
 			buf[sizeof (buf) - 1] = 0;
-			r_cons_print (buf);
-			if (bStopPipeLoop) {
-				break;
-			}
+			OVERLAPPED oWrite = { 0 };
+			oWrite.hEvent = hWritten;
 			char *res = lang->cmd_str ((RCore*)lang->user, buf);
 			if (res) {
 				int res_len = strlen (res) + 1;
@@ -90,29 +82,42 @@ static void lang_pipe_run_win(RLang *lang) {
 					memset (buf, 0, PIPE_BUF_SIZE);
 					dwWritten = 0;
 					int writelen = res_len - i;
-					int rc = WriteFile (hPipeInOut, res + i, writelen > PIPE_BUF_SIZE ? PIPE_BUF_SIZE : writelen, &dwWritten, 0);
-					if (bStopPipeLoop) {
+					WriteFile (hPipeInOut, res + i,
+					           writelen > PIPE_BUF_SIZE ? PIPE_BUF_SIZE : writelen,
+					           NULL, &oWrite);
+					HANDLE hWriteEvents[] = { hWritten, hproc };
+					dwEvent = WaitForMultipleObjects (R_ARRAY_SIZE (hWriteEvents), hWriteEvents,
+					                                  FALSE, INFINITE);
+					if (dwEvent == WAIT_OBJECT_0 + 1) { // hproc
 						break;
+					} else if (dwEvent == WAIT_FAILED) {
+						r_sys_perror ("lang_pipe_run_win/WaitForMultipleObjects write");
 					}
+					BOOL rc = GetOverlappedResult (hPipeInOut, &oWrite, &dwWritten, TRUE);
 					if (!rc) {
-						r_sys_perror ("lang_pipe_run_win/WriteFile");
+						r_sys_perror ("lang_pipe_run_win/WriteFile res");
 					}
 					if (dwWritten > 0) {
 						i += dwWritten - 1;
 					} else {
-						/* send null termination // chop */
-						eprintf ("w32-lang-pipe: 0x%x\n", (ut32)GetLastError ());
+						// send null termination // chop
+						r_sys_perror ("lang_pipe_run_win/dwWritten");
 						//WriteFile (hPipeInOut, "", 1, &dwWritten, NULL);
 						//break;
 					}
 				}
 				free (res);
 			} else {
-				WriteFile (hPipeInOut, "", 1, &dwWritten, NULL);
+				WriteFile (hPipeInOut, "", 1, NULL, &oWrite);
+				if (!GetOverlappedResult (hPipeInOut, &oWrite, &dwWritten, TRUE)) {
+					r_sys_perror ("lang_pipe_run_win/WriteFile nul");
+				}
 			}
 		}
-	} while (!bStopPipeLoop);
+	} while (true);
 	r_cons_break_pop ();
+	CloseHandle (hWritten);
+	CloseHandle (hRead);
 }
 #else
 static void env(const char *s, int f) {
@@ -123,7 +128,7 @@ static void env(const char *s, int f) {
 }
 #endif
 
-static int lang_pipe_run(RLang *lang, const char *code, int len) {
+static bool lang_pipe_run(RLang *lang, const char *code, int len) {
 #if __UNIX__
 	int safe_in = dup (0);
 	int child, ret;
@@ -187,13 +192,13 @@ static int lang_pipe_run(RLang *lang, const char *code, int len) {
 			}
 			buf[sizeof (buf) - 1] = 0;
 			res = lang->cmd_str ((RCore*)lang->user, buf);
-			//eprintf ("%d %s\n", ret, buf);
 			if (res) {
-				write (input[1], res, strlen (res) + 1);
+				// r_cons_print (res);
+				(void) write (input[1], res, strlen (res) + 1);
 				free (res);
 			} else {
 				eprintf ("r_lang_pipe: NULL reply for (%s)\n", buf);
-				write (input[1], "", 1); // NULL byte
+				(void) write (input[1], "", 1); // NULL byte
 			}
 		}
 		r_cons_break_pop ();
@@ -201,11 +206,15 @@ static int lang_pipe_run(RLang *lang, const char *code, int len) {
 		if (safe_in != -1) {
 			close (safe_in);
 		}
-		safe_in = open (ttyname(0), O_RDONLY);
-		if (safe_in != -1) {
-			dup2 (safe_in, 0);
-		} else {
-			eprintf ("Cannot open ttyname(0) %s\n", ttyname(0));
+		safe_in = -1;
+		char *term_in = ttyname (0);
+		if (term_in) {
+			safe_in = open (term_in, O_RDONLY);
+			if (safe_in != -1) {
+				dup2 (safe_in, 0);
+			} else {
+				eprintf ("Cannot open ttyname(0) %s\n", term_in);
+			}
 		}
 	}
 
@@ -216,36 +225,67 @@ static int lang_pipe_run(RLang *lang, const char *code, int len) {
 	if (safe_in != -1) {
 		close (safe_in);
 	}
+#ifndef __wasi__
 	waitpid (child, NULL, WNOHANG);
+#endif
 	return true;
 #else
 #if __WINDOWS__
 	char *r2pipe_var = r_str_newf ("R2PIPE_IN%x", _getpid ());
 	char *r2pipe_paz = r_str_newf ("\\\\.\\pipe\\%s", r2pipe_var);
 	LPTSTR r2pipe_paz_ = r_sys_conv_utf8_to_win (r2pipe_paz);
-	SECURITY_ATTRIBUTES saAttr;
-	saAttr.nLength = sizeof (SECURITY_ATTRIBUTES);
-	saAttr.bInheritHandle = TRUE;
-	saAttr.lpSecurityDescriptor = NULL;
 
 	SetEnvironmentVariable (TEXT ("R2PIPE_PATH"), r2pipe_paz_);
 	hPipeInOut = CreateNamedPipe (r2pipe_paz_,
-			PIPE_ACCESS_DUPLEX,
+			PIPE_ACCESS_DUPLEX | FILE_FLAG_OVERLAPPED,
 			PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE | PIPE_WAIT, PIPE_UNLIMITED_INSTANCES,
 			PIPE_BUF_SIZE,
 			PIPE_BUF_SIZE,
-			0, &saAttr);
-	hproc = myCreateChildProcess (code);
-	if (hproc) {
-		/* a separate thread is created that sets bStopPipeLoop once hproc terminates. */
-		bStopPipeLoop = FALSE;
-		CloseHandle (CreateThread (NULL, 0, WaitForProcThread, NULL, 0, NULL));
-		/* lang_pipe_run_win has to run in the command thread to prevent deadlock. */
-		lang_pipe_run_win (lang);
-		CloseHandle (pipe);
+			0, NULL);
+	if (hPipeInOut == INVALID_HANDLE_VALUE) {
+		r_sys_perror ("lang_pipe_run/CreateNamedPipe");
+		goto beach;
 	}
+	HANDLE hConnected = CreateEvent (NULL, TRUE, FALSE, NULL);
+	if (!hConnected) {
+		r_sys_perror ("lang_pipe_run/CreateEvent hConnected");
+		goto pipe_cleanup;
+	}
+	OVERLAPPED oConnect = { 0 };
+	oConnect.hEvent = hConnected;
+	hproc = myCreateChildProcess (code);
+	BOOL connected = FALSE;
+	if (hproc) {
+		connected = ConnectNamedPipe (hPipeInOut, &oConnect);
+		DWORD err = GetLastError ();
+		if (!connected && err != ERROR_PIPE_CONNECTED) {
+			if (err == ERROR_IO_PENDING) {
+				HANDLE hEvents[] = { hConnected, hproc };
+				DWORD dwEvent = WaitForMultipleObjects (R_ARRAY_SIZE (hEvents), hEvents,
+				                                        FALSE, INFINITE);
+				if (dwEvent == WAIT_OBJECT_0 + 1) { // hproc
+					goto cleanup;
+				} else if (dwEvent == WAIT_FAILED) {
+					r_sys_perror ("lang_pipe_run/WaitForMultipleObjects connect");
+					goto cleanup;
+				}
+				DWORD dummy;
+				connected = GetOverlappedResult (hPipeInOut, &oConnect, &dummy, TRUE);
+				err = GetLastError ();
+			}
+			if (!connected && err != ERROR_PIPE_CONNECTED) {
+				r_sys_perror ("lang_pipe_run/ConnectNamedPipe");
+				goto cleanup;
+			}
+		}
+		lang_pipe_run_win (lang);
+	}
+cleanup:
+	CloseHandle (hConnected);
+pipe_cleanup:
 	DeleteFile (r2pipe_paz_);
 	CloseHandle (hPipeInOut);
+beach:
 	free (r2pipe_var);
 	free (r2pipe_paz);
 	free (r2pipe_paz_);
@@ -254,6 +294,10 @@ static int lang_pipe_run(RLang *lang, const char *code, int len) {
 #endif
 }
 
+static bool lang_pipe_file(RLang *lang, const char *file) {
+	return lang_pipe_run (lang, file, -1);
+}
+
 static RLangPlugin r_lang_plugin_pipe = {
 	.name = "pipe",
 	.ext = "pipe",
