@@ -38,7 +38,7 @@
 #include "file.h"
 #include "patchlevel.h"
 
-#if __UNIX__ && !defined(_MSC_VER)
+#if __UNIX__ && !defined(_MSC_VER) && !(defined(__wasi__))
 # define QUICK 1
 # include <sys/mman.h>
 # include <sys/param.h>
@@ -46,6 +46,10 @@
 # define QUICK 0
 #endif
 
+#ifdef __wasi__
+# define MAXPATHLEN 255
+#endif
+
 #ifdef _MSC_VER
 # include <sys\stat.h>
 # define S_ISREG(m) (((m) & S_IFMT) == S_IFREG)
@@ -268,7 +272,7 @@ void file_delmagic(struct r_magic *p, int type, size_t entries) {
 }
 
 /* const char *fn: list of magic files and directories */
-struct mlist * file_apprentice(RMagic *ms, const char *fn, int action) {
+struct mlist * file_apprentice(RMagic *ms, const char *fn, size_t fn_size, int action) {
 	char *p, *mfn;
 	int file_err, errs = -1;
 	struct mlist *mlist;
@@ -278,8 +282,8 @@ struct mlist * file_apprentice(RMagic *ms, const char *fn, int action) {
 		return NULL;
 	}
 
-	if (!(mfn = strdup (fn))) {
-		file_oomem (ms, strlen (fn));
+	if (!(mfn = r_str_ndup (fn, fn_size))) {
+		file_oomem (ms, fn_size);
 		return NULL;
 	}
 	fn = mfn;
@@ -1985,6 +1989,9 @@ static char *mkdbname(const char *fn, int strip) {
 	}
 	fnlen = strlen (fn);
 	extlen = strlen (ext);
+#ifndef MAXPATHLEN
+#define MAXPATHLEN 256
+#endif
 	if (fnlen + extlen + 1 > MAXPATHLEN) {
 		return NULL;
 	}
@@ -2080,4 +2087,4 @@ static void bs1(struct r_magic *m) {
 }
 #endif
 
-#include "print.c"
+#include "mdump.c"
