@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2020 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include <r_core.h>
 #include <r_types.h>
@@ -75,18 +75,20 @@ static int rabin_show_help(int v) {
 	}
 	if (v) {
 		printf ("Environment:\n"
-		" RABIN2_LANG:      e bin.lang         # assume lang for demangling\n"
-		" RABIN2_NOPLUGINS: # do not load shared plugins (speedup loading)\n"
-		" RABIN2_DEMANGLE=0:e bin.demangle     # do not demangle symbols\n"
-		" RABIN2_MAXSTRBUF: e bin.maxstrbuf    # specify maximum buffer size\n"
-		" RABIN2_STRFILTER: e bin.str.filter   #  r2 -qc 'e bin.str.filter=?" "?' -\n"
-		" RABIN2_STRPURGE:  e bin.str.purge    # try to purge false positives\n"
+		" RABIN2_CHARSET:   e cfg.charset      # set default value charset for -z strings\n"
 		" RABIN2_DEBASE64:  e bin.debase64     # try to debase64 all strings\n"
+		" RABIN2_DEMANGLE=0:e bin.demangle     # do not demangle symbols\n"
 		" RABIN2_DMNGLRCMD: e bin.demanglercmd # try to purge false positives\n"
+		" RABIN2_LANG:      e bin.lang         # assume lang for demangling\n"
+		" RABIN2_MAXSTRBUF: e bin.maxstrbuf    # specify maximum buffer size\n"
+		" RABIN2_NOPLUGINS: 1|0|               # do not load shared plugins (speedup loading)\n"
 		" RABIN2_PDBSERVER: e pdb.server       # use alternative PDB server\n"
-		" RABIN2_SYMSTORE:  e pdb.symstore     # path to downstream symbol store\n"
 		" RABIN2_PREFIX:    e bin.prefix       # prefix symbols/sections/relocs with a specific string\n"
-		" R2_CONFIG:        # sdb config file\n");
+		" RABIN2_STRFILTER: e bin.str.filter   # r2 -qc 'e bin.str.filter=?" "?' -\n"
+		" RABIN2_STRPURGE:  e bin.str.purge    # try to purge false positives\n"
+		" RABIN2_SYMSTORE:  e pdb.symstore     # path to downstream symbol store\n"
+		" RABIN2_SWIFTLIB:  1|0|               # load Swift libsto demangle (default: true)\n"
+		);
 	}
 	return 1;
 }
@@ -510,14 +512,27 @@ static int __lib_bin_ldr_dt(RLibPlugin *pl, void *p, void *u) {
 	return true;
 }
 
+static void setup_trylib_from_environment(RBin *bin, int type) {
+	bool trylib = false;
+	if (type == R_BIN_NM_SWIFT) {
+		trylib = true;
+		char *swiftlib = r_sys_getenv ("RABIN2_TRYLIB");
+		if (swiftlib) {
+			trylib = r_str_is_true (swiftlib);
+			free (swiftlib);
+		}
+	}
+	bin->demangle_trylib = trylib;
+}
+
 static char *__demangleAs(RBin *bin, int type, const char *file) {
-	bool syscmd = bin? bin->demanglercmd: false;
+	bool syscmd = bin->demangle_usecmd;
 	char *res = NULL;
 	switch (type) {
 	case R_BIN_NM_CXX: res = r_bin_demangle_cxx (NULL, file, 0); break;
 	case R_BIN_NM_JAVA: res = r_bin_demangle_java (file); break;
 	case R_BIN_NM_OBJC: res = r_bin_demangle_objc (NULL, file); break;
-	case R_BIN_NM_SWIFT: res = r_bin_demangle_swift (file, syscmd); break;
+	case R_BIN_NM_SWIFT: res = r_bin_demangle_swift (file, syscmd, bin->demangle_trylib); break;
 	case R_BIN_NM_MSVC: res = r_bin_demangle_msvc (file); break;
 	case R_BIN_NM_RUST: res = r_bin_demangle_rust (NULL, file, 0); break;
 	default:
@@ -527,41 +542,40 @@ static char *__demangleAs(RBin *bin, int type, const char *file) {
 	return res;
 }
 
-static void __listPlugins(RBin *bin, const char* plugin_name, int rad) {
+static void __listPlugins(RBin *bin, const char* plugin_name, PJ *pj, int rad) {
 	int format = (rad == R_MODE_JSON) ? 'j': rad? 'q': 0;
 	bin->cb_printf = (PrintfCallback)printf;
 	if (plugin_name) {
-		r_bin_list_plugin (bin, plugin_name, format);
+		r_bin_list_plugin (bin, plugin_name, pj, format);
 	} else {
-		r_bin_list (bin, format);
+		r_bin_list (bin, pj, format);
 	}
 }
 
-R_API int r_main_rabin2(int argc, char **argv) {
-	char *name = NULL;
-	char *file = NULL;
+R_API int r_main_rabin2(int argc, const char **argv) {
 	RBin *bin = NULL;
-	char *output = NULL;
+	const char *name = NULL;
+	const char *file = NULL;
+	const char *output = NULL;
 	int rad = 0;
 	ut64 laddr = UT64_MAX;
 	ut64 baddr = UT64_MAX;
 	const char *do_demangle = NULL;
 	const char *query = NULL;
-	int c, bits = 0, actions_done = 0, actions = 0;
+	int c, bits = 0, actions = 0;
 	char* create = NULL;
 	bool va = true;
 	ut64 action = R_BIN_REQ_UNK;
-	char *tmp, *ptr, *arch = NULL, *arch_name = NULL;
+	char *tmp, *ptr, *arch_name = NULL;
+	const char *arch = NULL;
 	const char *forcebin = NULL;
 	const char *chksum = NULL;
 	const char *op = NULL;
-	RCoreFile *fh = NULL;
 	RCoreBinFilter filter;
 	int xtr_idx = 0; // load all files if extraction is necessary.
 	int rawstr = 0;
 	int fd = -1;
 	RCore core = {0};
-	RLib *l = NULL;
 	ut64 at = UT64_MAX;
 
 	r_core_init (&core);
@@ -572,7 +586,7 @@ R_API int r_main_rabin2(int argc, char **argv) {
 		char *plugindir = r_str_r2_prefix (R2_PLUGINS);
 		char *extrasdir = r_str_r2_prefix (R2_EXTRAS);
 		char *bindingsdir = r_str_r2_prefix (R2_BINDINGS);
-		l = r_lib_new (NULL, NULL);
+		RLib *l = r_lib_new (NULL, NULL);
 		r_lib_add_handler (l, R_LIB_TYPE_BIN, "bin plugins",
 			&__lib_bin_cb, &__lib_bin_dt, bin);
 		r_lib_add_handler (l, R_LIB_TYPE_BIN_XTR, "bin xtr plugins",
@@ -593,9 +607,10 @@ R_API int r_main_rabin2(int argc, char **argv) {
 		free (extrasdir);
 		free (bindingsdir);
 		free (path);
+		r_lib_free (l);
 	}
 	free (tmp);
-
+#if 0
 	if ((tmp = r_sys_getenv ("R2_CONFIG"))) {
 		Sdb *config_sdb = sdb_new (NULL, tmp, 0);
 		if (config_sdb) {
@@ -606,8 +621,13 @@ R_API int r_main_rabin2(int argc, char **argv) {
 		}
 		free (tmp);
 	}
+#endif
 	if ((tmp = r_sys_getenv ("RABIN2_DMNGLRCMD"))) {
-		r_config_set (core.config, "bin.demanglecmd", tmp);
+		r_config_set (core.config, "cmd.demangle", tmp);
+		free (tmp);
+	}
+	if ((tmp = r_sys_getenv ("RABIN2_CHARSET"))) {
+		r_config_set (core.config, "cfg.charset", tmp);
 		free (tmp);
 	}
 	if ((tmp = r_sys_getenv ("RABIN2_LANG"))) {
@@ -642,7 +662,9 @@ R_API int r_main_rabin2(int argc, char **argv) {
 #define is_active(x) (action & (x))
 #define set_action(x) { actions++; action |= (x); }
 #define unset_action(x) action &= ~x
-	while ((c = r_getopt (argc, argv, "DjgAf:F:a:B:G:b:cC:k:K:dD:Mm:n:N:@:isSVIHeEUlRwO:o:pPqQrTtvLhuxXzZ")) != -1) {
+	RGetopt opt;
+	r_getopt_init (&opt, argc, argv, "DjgAf:F:a:B:G:b:cC:k:K:dD:Mm:n:N:@:isSVIHeEUlRwO:o:pPqQrTtvLhuxXzZ");
+	while ((c = r_getopt_next (&opt)) != -1) {
 		switch (c) {
 		case 'g':
 			set_action (R_BIN_REQ_CLASSES);
@@ -673,14 +695,14 @@ R_API int r_main_rabin2(int argc, char **argv) {
 			break;
 		case 'j': rad = R_MODE_JSON; break;
 		case 'A': set_action (R_BIN_REQ_LISTARCHS); break;
-		case 'a': arch = r_optarg; break;
+		case 'a': arch = opt.arg; break;
 		case 'C':
 			set_action (R_BIN_REQ_CREATE);
-			create = strdup (r_optarg);
+			create = strdup (opt.arg);
 			break;
 		case 'u': bin->filter = 0; break;
-		case 'k': query = r_optarg; break;
-		case 'K': chksum = r_optarg; break;
+		case 'k': query = opt.arg; break;
+		case 'K': chksum = opt.arg; break;
 		case 'c':
 			if (is_active (R_BIN_REQ_CLASSES)) {
 				rad = R_MODE_CLASSDUMP;
@@ -688,11 +710,11 @@ R_API int r_main_rabin2(int argc, char **argv) {
 			  	set_action (R_BIN_REQ_CLASSES);
 			}
 			break;
-		case 'f': arch_name = strdup (r_optarg); break;
-		case 'F': forcebin = r_optarg; break;
-		case 'b': bits = r_num_math (NULL, r_optarg); break;
+		case 'f': arch_name = strdup (opt.arg); break;
+		case 'F': forcebin = opt.arg; break;
+		case 'b': bits = r_num_math (NULL, opt.arg); break;
 		case 'm':
-			at = r_num_math (NULL, r_optarg);
+			at = r_num_math (NULL, opt.arg);
 			set_action (R_BIN_REQ_SRCLINE);
 			break;
 		case 'i':
@@ -739,13 +761,13 @@ R_API int r_main_rabin2(int argc, char **argv) {
 			}
 			break;
 		case 'D':
-			if (argv[r_optind] && argv[r_optind+1] && \
-				(!argv[r_optind+1][0] || !strcmp (argv[r_optind+1], "all"))) {
-				r_config_set (core.config, "bin.lang", argv[r_optind]);
+			if (argv[opt.ind] && argv[opt.ind+1] && \
+				(!argv[opt.ind+1][0] || !strcmp (argv[opt.ind+1], "all"))) {
+				r_config_set (core.config, "bin.lang", argv[opt.ind]);
 				r_config_set (core.config, "bin.demangle", "true");
-				r_optind += 2;
+				opt.ind += 2;
 			} else {
-				do_demangle = argv[r_optind];
+				do_demangle = argv[opt.ind];
 			}
 			break;
 		case 'e':
@@ -765,7 +787,7 @@ R_API int r_main_rabin2(int argc, char **argv) {
 		case 'x': set_action (R_BIN_REQ_EXTRACT); break;
 		case 'X': set_action (R_BIN_REQ_PACKAGE); break;
 		case 'O':
-			op = r_optarg;
+			op = opt.arg;
 			set_action (R_BIN_REQ_OPERATION);
 			if (*op == 'c') {
 				r_sys_setenv ("RABIN2_CODESIGN_VERBOSE", "1");
@@ -784,13 +806,13 @@ R_API int r_main_rabin2(int argc, char **argv) {
 				r_core_fini (&core);
 				return 0;
 			}
-			if (r_optind == argc) {
+			if (opt.ind == argc) {
 				eprintf ("Missing filename\n");
 				r_core_fini (&core);
 				return 1;
 			}
 			break;
-		case 'o': output = r_optarg; break;
+		case 'o': output = opt.arg; break;
 		case 'p': va = false; break;
 		case 'r': rad = true; break;
 		case 'v':
@@ -800,26 +822,26 @@ R_API int r_main_rabin2(int argc, char **argv) {
 			set_action (R_BIN_REQ_LISTPLUGINS);
 			break;
 		case 'G':
-			laddr = r_num_math (NULL, r_optarg);
+			laddr = r_num_math (NULL, opt.arg);
 			if (laddr == UT64_MAX) {
 				va = false;
 			}
 			break;
 		case 'B':
-			baddr = r_num_math (NULL, r_optarg);
+			baddr = r_num_math (NULL, opt.arg);
 			break;
 		case '@':
-			at = r_num_math (NULL, r_optarg);
-			if (at == 0LL && *r_optarg != '0') {
+			at = r_num_math (NULL, opt.arg);
+			if (at == 0LL && *opt.arg != '0') {
 				at = UT64_MAX;
 			}
 			break;
 		case 'n':
-			name = r_optarg;
+			name = opt.arg;
 			break;
 		case 'N':
-			tmp = strchr (r_optarg, ':');
-			r_config_set (core.config, "bin.minstr", r_optarg);
+			tmp = strchr (opt.arg, ':');
+			r_config_set (core.config, "bin.minstr", opt.arg);
 			if (tmp) {
 				r_config_set (core.config, "bin.maxstr", tmp + 1);
 			}
@@ -833,25 +855,38 @@ R_API int r_main_rabin2(int argc, char **argv) {
 		}
 	}
 
+	PJ *pj = NULL;
+	if (rad == R_MODE_JSON) {
+		pj = r_core_pj_new (&core);
+		if (!pj) {
+			return 1;
+		}
+	}
+
 	if (is_active (R_BIN_REQ_LISTPLUGINS)) {
 		const char* plugin_name = NULL;
-		if (r_optind < argc) {
-			plugin_name = argv[r_optind];
+		if (opt.ind < argc) {
+			plugin_name = argv[opt.ind];
+		}
+		__listPlugins (bin, plugin_name, pj, rad);
+		if (rad == R_MODE_JSON) {
+			r_cons_println (pj_string (pj));
+			r_cons_flush ();
+			pj_free (pj);
 		}
-		__listPlugins (bin, plugin_name, rad);
 		r_core_fini (&core);
 		return 0;
 	}
 
 	if (do_demangle) {
 		char *res = NULL;
-		int type;
-		if ((argc - r_optind) < 2) {
+		if ((argc - opt.ind) < 2) {
 			r_core_fini (&core);
 			return rabin_show_help (0);
 		}
-		type = r_bin_demangle_type (do_demangle);
-		file = argv[r_optind + 1];
+		int type = r_bin_demangle_type (do_demangle);
+		setup_trylib_from_environment (bin, type);
+		file = argv[opt.ind + 1];
 		if (!strcmp (file, "-")) {
 			for (;;) {
 				file = stdin_gets (false);
@@ -872,7 +907,8 @@ R_API int r_main_rabin2(int argc, char **argv) {
 				R_FREE (res);
 				R_FREE (file);
 			}
-			stdin_gets (true);
+			char *r = stdin_gets (true);
+			free (r);
 		} else {
 			res = __demangleAs (bin, type, file);
 			if (res && *res) {
@@ -889,7 +925,14 @@ R_API int r_main_rabin2(int argc, char **argv) {
 		r_core_fini (&core);
 		return 1;
 	}
-	file = argv[r_optind];
+	file = argv[opt.ind];
+
+	if (file && !*file) {
+		eprintf ("Cannot open empty path\n");
+		r_core_fini (&core);
+		return 1;
+	}
+
 	if (!query) {
 		if (action & R_BIN_REQ_HELP || action == R_BIN_REQ_UNK || !file) {
 			r_core_fini (&core);
@@ -943,7 +986,7 @@ R_API int r_main_rabin2(int argc, char **argv) {
 			if (r_file_dump (file, tmp, tmpsz, 0)) {
 				eprintf ("Dumped %" PFMT64d " bytes in '%s'\n",
 					tmpsz, file);
-				r_file_chmod (file, "+x", 0);
+				(void)r_file_chmod (file, "+x", 0);
 			} else {
 				eprintf ("Error dumping into a.out\n");
 			}
@@ -966,7 +1009,7 @@ R_API int r_main_rabin2(int argc, char **argv) {
 	}
 
 	if (file && *file && action & R_BIN_REQ_DLOPEN) {
-#if __UNIX__
+#if __UNIX__ && HAVE_FORK
 		int child = r_sys_fork ();
 		if (child == -1) {
 			r_core_fini (&core);
@@ -989,20 +1032,21 @@ R_API int r_main_rabin2(int argc, char **argv) {
 	}
 	if (action & R_BIN_REQ_PACKAGE) {
 		RList *files = r_list_newf (NULL);
-		const char *format = argv[r_optind];
-		const char *file = argv[r_optind + 1];
+		const char *format = argv[opt.ind];
+		const char *file = argv[opt.ind + 1];
 		int i, rc = 0;
 
-		if (r_optind + 3 > argc) {
+		if (opt.ind + 3 > argc) {
 			eprintf ("Usage: rabin2 -X [fat|zip] foo.zip a b c\n");
 			r_core_fini (&core);
 			return 1;
 		}
+
 		eprintf ("FMT %s\n", format);
 		eprintf ("PKG %s\n", file);
-		for (i = r_optind + 2; i < argc; i++) {
+		for (i = opt.ind + 2; i < argc; i++) {
 			eprintf ("ADD %s\n", argv[i]);
-			r_list_append (files, argv[i]);
+			r_list_append (files, (void*)argv[i]);
 		}
 		RBuffer *buf = r_bin_package (core.bin, format, file, files);
 		/* TODO: return bool or something to catch errors\n") */
@@ -1019,11 +1063,10 @@ R_API int r_main_rabin2(int argc, char **argv) {
 	}
 
 	if (file && *file) {
-		if ((fh = r_core_file_open (&core, file, R_PERM_R, 0))) {
+		if (r_core_file_open (&core, file, R_PERM_R, 0)) {
 			fd = r_io_fd_get_current (core.io);
 			if (fd == -1) {
 				eprintf ("r_core: Cannot open file '%s'\n", file);
-				r_core_file_free (fh);
 				r_core_fini (&core);
 				return 1;
 			}
@@ -1039,16 +1082,15 @@ R_API int r_main_rabin2(int argc, char **argv) {
 	r_bin_force_plugin (bin, forcebin);
 	r_bin_load_filter (bin, action);
 
-	RBinOptions opt;
-	r_bin_options_init (&opt, fd, baddr, laddr, rawstr);
-	opt.xtr_idx = xtr_idx;
+	RBinFileOptions bo;
+	r_bin_file_options_init (&bo, fd, baddr, laddr, rawstr);
+	bo.xtr_idx = xtr_idx;
 
-	if (!r_bin_open (bin, file, &opt)) {
+	if (!r_bin_open (bin, file, &bo)) {
 		//if this return false means that we did not return a valid bin object
 		//but we have yet the chance that this file is a fat binary
 		if (!bin->cur || !bin->cur->xtr_data) {
 			eprintf ("r_bin: Cannot open file\n");
-			r_core_file_free (fh);
 			r_core_fini (&core);
 			return 1;
 		}
@@ -1068,7 +1110,7 @@ R_API int r_main_rabin2(int argc, char **argv) {
 	}
 	if (query) {
 		if (rad) {
-			r_core_bin_export_info_rad (&core);
+			r_core_bin_export_info (&core, R_MODE_RADARE);
 			r_cons_flush ();
 		} else {
 			if (!strcmp (query, "-")) {
@@ -1077,18 +1119,16 @@ R_API int r_main_rabin2(int argc, char **argv) {
 				sdb_query (bin->cur->sdb, query);
 			}
 		}
-		r_core_file_free (fh);
 		r_core_fini (&core);
 		return 0;
 	}
 #define isradjson (rad==R_MODE_JSON&&actions>0)
 #define run_action(n,x,y) {\
 	if (action&(x)) {\
-		if (isradjson) r_cons_printf ("%s\"%s\":",actions_done?",":"",n);\
-		if (!r_core_bin_info (&core, y, rad, va, &filter, chksum)) {\
-			if (isradjson) r_cons_print ("false");\
+		if (isradjson) pj_k (pj, n);\
+		if (!r_core_bin_info (&core, y, pj, rad, va, &filter, chksum)) {\
+			if (isradjson) pj_b (pj, false);\
 		};\
-		actions_done++;\
 	}\
 }
 	core.bin = bin;
@@ -1098,17 +1138,16 @@ R_API int r_main_rabin2(int argc, char **argv) {
 	r_cons_new ()->context->is_interactive = false;
 
 	if (isradjson) {
-		r_cons_print ("{");
+		pj_o (pj);
 	}
 	// List fatmach0 sub-binaries, etc
 	if (action & R_BIN_REQ_LISTARCHS || ((arch || bits || arch_name) &&
 		!r_bin_select (bin, arch, bits, arch_name))) {
 		if (rad == R_MODE_SIMPLEST || rad == R_MODE_SIMPLE) {
-			r_bin_list_archs (bin, 'q');
+			r_bin_list_archs (bin, pj, 'q');
 		} else {
-			r_bin_list_archs (bin, (rad == R_MODE_JSON)? 'j': 1);
+			r_bin_list_archs (bin, pj, (rad == R_MODE_JSON)? 'j': 1);
 		}
-		actions_done++;
 		free (arch_name);
 	}
 	if (action & R_BIN_REQ_PDB_DWNLD) {
@@ -1122,10 +1161,7 @@ R_API int r_main_rabin2(int argc, char **argv) {
 			R_FREE (tmp);
 		}
 		pdbopts.symbol_store_path = (char *)r_config_get (core.config, "pdb.symstore");
-		int r = r_bin_pdb_download (&core, isradjson, &actions_done, &pdbopts);
-		r_core_file_free (fh);
-		r_core_fini (&core);
-		return r;
+		r_bin_pdb_download (&core, pj, isradjson, &pdbopts);
 	}
 
 	if ((tmp = r_sys_getenv ("RABIN2_PREFIX"))) {
@@ -1173,12 +1209,12 @@ R_API int r_main_rabin2(int argc, char **argv) {
 		rabin_do_operation (bin, op, rad, output, file);
 	}
 	if (isradjson) {
-		r_cons_print ("}");
+		pj_end (pj);
+		r_cons_println (pj_string (pj));
 	}
+	pj_free (pj);
 	r_cons_flush ();
-	r_core_file_free (fh);
 	r_core_fini (&core);
-	r_lib_free (l);
 
 	return 0;
 }
