@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2020 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #define USE_THREADS 1
 #define ALLOW_THREADED 0
@@ -11,8 +11,7 @@
 
 #include <r_core.h>
 
-static bool is_valid_gdb_file(RCoreFile *fh) {
-	RIODesc *d = fh && fh->core ? r_io_desc_get (fh->core->io, fh->fd) : NULL;
+static bool is_valid_gdb_file(RIODesc *d) {
 	return d && strncmp (d->name, "gdb://", 6);
 }
 
@@ -74,11 +73,7 @@ static int r_main_version_verify(int show) {
 		}
 	}
 	if (ret) {
-		if (show) {
-			eprintf ("WARNING: r2 library versions mismatch!\n");
-		} else {
-			eprintf ("WARNING: r2 library versions mismatch! See r2 -V\n");
-		}
+		eprintf ("Warning: r2 library versions mismatch! Check r2 -V");
 	}
 	return ret;
 }
@@ -111,6 +106,7 @@ static int main_help(int line) {
 		" -H ([var])   display variable\n"
 		" -i [file]    run script file\n"
 		" -I [file]    run script file before the file is opened\n"
+		" -j           use json for -v, -L and maybe others\n"
 		" -k [OS/kern] set asm.os (linux, macos, w32, netbsd, ...)\n"
 		" -l [lib]     load plugin file\n"
 		" -L           list supported IO plugins\n"
@@ -118,6 +114,7 @@ static int main_help(int line) {
 		" -M           do not demangle symbol names\n"
 		" -n, -nn      do not load RBin info (-nn only load bin structures)\n"
 		" -N           do not load user settings and scripts\n"
+		" -NN          do not load any script or plugin\n"
 		" -q           quiet mode (no prompt) and quit after -i\n"
 		" -qq          quit after running all -c and -i\n"
 		" -Q           quiet mode (no prompt) and quit faster (quickLeak=true)\n"
@@ -152,13 +149,15 @@ static int main_help(int line) {
 		" R2_LIBR_PLUGINS " R_JOIN_2_PATHS ("%s", R2_PLUGINS) "\n"
 		" R2_USER_ZIGNS " R_JOIN_2_PATHS ("~", R2_HOME_ZIGNS) "\n"
 		"Environment:\n"
-		" R2_RDATAHOME %s\n" // TODO: rename to RHOME R2HOME?
-		" RCFILE       ~/.radare2rc (user preferences, batch script)\n" // TOO GENERIC
+		" R2_CFG_NEWSHELL sets cfg.newshell=true\n"
+		" R2_DEBUG      if defined, show error messages and crash signal.\n"
+		" R2_DEBUG_ASSERT=1 set a breakpoint when hitting an assert.\n"
+		" R2_IGNVER=1   load plugins ignoring the specified version. (be careful)\n"
 		" R2_MAGICPATH " R_JOIN_2_PATHS ("%s", R2_SDB_MAGIC) "\n"
-		" R_DEBUG      if defined, show error messages and crash signal\n"
-		" R_DEBUG_ASSERT=1 set a breakpoint when hitting an assert\n"
-		" VAPIDIR      path to extra vapi directory\n"
 		" R2_NOPLUGINS do not load r2 shared plugins\n"
+		" R2_RCFILE    ~/.radare2rc (user preferences, batch script)\n" // TOO GENERIC
+		" R2_RDATAHOME %s\n" // TODO: rename to RHOME R2HOME?
+		" R2_VERSION   contains the current version of r2\n"
 		"Paths:\n"
 		" R2_PREFIX    "R2_PREFIX"\n"
 		" R2_INCDIR    "R2_INCDIR"\n"
@@ -190,9 +189,9 @@ static int main_print_var(const char *var_name) {
 		const char *name;
 		const char *value;
 	} r2_vars[] = {
+		{ "R2_VERSION", R2_VERSION },
 		{ "R2_PREFIX", R2_PREFIX },
 		{ "R2_MAGICPATH", magicpath },
-		{ "R2_PREFIX", R2_PREFIX },
 		{ "R2_INCDIR", incdir },
 		{ "R2_LIBDIR", libdir },
 		{ "R2_LIBEXT", R_LIB_EXT },
@@ -240,28 +239,29 @@ static bool run_commands(RCore *r, RList *cmds, RList *files, bool quiet, int do
 	const char *file;
 	int ret;
 	/* -i */
+	bool has_failed = false;
 	r_list_foreach (files, iter, file) {
 		if (!r_file_exists (file)) {
 			eprintf ("Script '%s' not found.\n", file);
 			goto beach;
 		}
 		ret = r_core_run_script (r, file);
+		r_cons_flush ();
 		if (ret == -2) {
 			eprintf ("[c] Cannot open '%s'\n", file);
 		}
-		if (ret < 0 || (ret == 0 && quiet)) {
-			r_cons_flush ();
-			return false;
+		if (ret < 0) {
+			has_failed = true;
+			break;
 		}
 	}
 	/* -c */
 	r_list_foreach (cmds, iter, cmdn) {
-		//r_core_cmd0 (r, cmdn);
-		r_core_cmd (r, cmdn, false);
+		r_core_cmd_lines (r, cmdn);
 		r_cons_flush ();
 	}
 beach:
-	if (quiet) {
+	if (quiet && !has_failed) {
 		if (do_analysis) {
 			return true;
 		}
@@ -276,7 +276,7 @@ beach:
 }
 
 static bool mustSaveHistory(RConfig *c) {
-	if (!r_config_get_i (c, "scr.histsave")) {
+	if (!r_config_get_i (c, "scr.hist.save")) {
 		return false;
 	}
 	if (!r_cons_is_interactive ()) {
@@ -321,22 +321,20 @@ static void set_color_default(RCore *r) {
 	free (tmp);
 }
 
-R_API int r_main_radare2(int argc, char **argv) {
+R_API int r_main_radare2(int argc, const char **argv) {
 	RCore *r;
 	bool forcequit = false;
 	bool haveRarunProfile = false;
 	RListIter *iter;
 	int do_analysis = 0;
 	char *cmdn, *tmp;
-	RCoreFile *fh = NULL;
+	RIODesc *fh = NULL;
 	RIODesc *iod = NULL;
 	const char *patchfile = NULL;
-	const char *prj = NULL;
 	int debug = 0;
 	int zflag = 0;
 	bool do_connect = false;
 	bool fullfile = false;
-	int has_project;
 	bool zerosep = false;
 	int help = 0;
 	enum { LOAD_BIN_ALL, LOAD_BIN_NOTHING, LOAD_BIN_STRUCTURES_ONLY } load_bin = LOAD_BIN_ALL;
@@ -346,8 +344,8 @@ R_API int r_main_radare2(int argc, char **argv) {
 	ut64 baddr = UT64_MAX;
 	ut64 seek = UT64_MAX;
 	bool do_list_io_plugins = false;
-	char *pfile = NULL, *file = NULL;
-	const char *debugbackend = "native";
+	char *file = NULL;
+	char *pfile = NULL;
 	const char *asmarch = NULL;
 	const char *asmos = NULL;
 	const char *forcebin = NULL;
@@ -381,21 +379,36 @@ R_API int r_main_radare2(int argc, char **argv) {
 	r_signal_sigmask (SIG_BLOCK, &sigBlockMask, NULL);
 #endif
 
-	char **envp = r_sys_get_environ ();
-	if (envp) {
-		r_sys_set_environ (envp);
-	}
+	r_sys_env_init ();
+	// Create rarun2 profile with startup environ
+	char **env = r_sys_get_environ ();
+	char *envprofile = r_run_get_environ_profile (env);
 
-	if ((tmp = r_sys_getenv ("R_DEBUG"))) {
-		r_sys_crash_handler ("gdb --pid %d");
-		free (tmp);
+	if (r_sys_getenv_asbool ("R2_DEBUG")) {
+		char *sysdbg = r_sys_getenv ("R2_DEBUG_TOOL");
+		char *fmt = (sysdbg && *sysdbg)
+			? strdup (sysdbg)
+#if __APPLE__
+			: strdup ("lldb -p");
+#else
+			: strdup ("gdb --pid");
+#endif
+		r_sys_crash_handler (fmt);
+		free (fmt);
+		free (sysdbg);
 	}
 	if (argc < 2) {
 		LISTS_FREE ();
+		free (envprofile);
 		return main_help (1);
 	}
-	// r_core_init (r); // TODO: use r_core_new() for simplicity
 	r = r_core_new ();
+	if (!r) {
+		eprintf ("Cannot initialize RCore\n");
+		free (envprofile);
+		LISTS_FREE ();
+		return 1;
+	}
 	r->r_main_radare2 = r_main_radare2;
 	r->r_main_radiff2 = r_main_radiff2;
 	r->r_main_rafind2 = r_main_rafind2;
@@ -404,10 +417,13 @@ R_API int r_main_radare2(int argc, char **argv) {
 	r->r_main_rasm2 = r_main_rasm2;
 	r->r_main_rax2 = r_main_rax2;
 
+	r->io->envprofile = envprofile;
+
 	r_core_task_sync_begin (&r->tasks);
 	if (argc == 2 && !strcmp (argv[1], "-p")) {
 		r_core_project_list (r, 0);
 		r_cons_flush ();
+		free (envprofile);
 		LISTS_FREE ();
 		return 0;
 	}
@@ -421,20 +437,28 @@ R_API int r_main_radare2(int argc, char **argv) {
 	// -H option without argument
 	if (argc == 2 && !strcmp (argv[1], "-H")) {
 		main_print_var (NULL);
+		free (envprofile);
 		LISTS_FREE ();
 		return 0;
 	}
 
 	set_color_default (r);
+	bool show_version = false;
+	bool json = false;
+	bool load_l = true;
+	char *debugbackend = strdup ("native");
+	const char *project_name = NULL;
 
-	while ((c = r_getopt (argc, argv, "=02AMCwxfF:H:hm:e:nk:NdqQs:p:b:B:a:Lui:I:l:P:R:r:c:D:vVSTzuX"
-#if USE_THREADS
-"t"
-#endif
-	)) != -1) {
+	RGetopt opt;
+	r_getopt_init (&opt, argc, argv, "=02AjMCwxfF:H:hm:e:nk:NdqQs:p:b:B:a:Lui:I:l:P:R:r:c:D:vVSTzuXt");
+	while ((c = r_getopt_next (&opt)) != -1) {
 		switch (c) {
+		case 'j':
+			json = true;
+			break;
 		case '=':
-			r->cmdremote = 1;
+			R_FREE (r->cmdremote);
+			r->cmdremote = strdup ("");
 			break;
 		case '2':
 			noStderr = true;
@@ -451,7 +475,7 @@ R_API int r_main_radare2(int argc, char **argv) {
 			r_config_set (r->config, "bin.filter", "false");
 			break;
 		case 'a':
-			asmarch = r_optarg;
+			asmarch = opt.arg;
 			break;
 		case 'z':
 			zflag++;
@@ -460,16 +484,17 @@ R_API int r_main_radare2(int argc, char **argv) {
 			do_analysis += do_analysis ? 1: 2;
 			break;
 		case 'b':
-			asmbits = r_optarg;
+			asmbits = opt.arg;
+			r_config_set (r->config, "asm.bits", opt.arg);
 			break;
 		case 'B':
-			baddr = r_num_math (r->num, r_optarg);
+			baddr = r_num_math (r->num, opt.arg);
 			break;
 		case 'X':
 			r_config_set (r->config, "bin.usextr", "false");
 			break;
 		case 'c':
-			r_list_append (cmds, r_optarg);
+			r_list_append (cmds, (void*)opt.arg);
 			break;
 		case 'C':
 			do_connect = true;
@@ -481,53 +506,67 @@ R_API int r_main_radare2(int argc, char **argv) {
 #endif
 		case 'D':
 			debug = 2;
-			debugbackend = r_optarg;
-			if (!strcmp (r_optarg, "?")) {
+			free (debugbackend);
+			debugbackend = strdup (opt.arg);
+			if (!strcmp (opt.arg, "?")) {
 				r_debug_plugin_list (r->dbg, 'q');
-				r_cons_flush();
+				r_cons_flush ();
 				LISTS_FREE ();
+				free (envprofile);
+				free (debugbackend);
 				return 0;
 			}
 			break;
 		case 'e':
-			if (!strcmp (r_optarg, "q")) {
+			if (!strcmp (opt.arg, "q")) {
 				r_core_cmd0 (r, "eq");
 			} else {
-				r_config_eval (r->config, r_optarg, false);
-				r_list_append (evals, r_optarg);
+				r_config_eval (r->config, opt.arg, false);
+				r_list_append (evals, (void*)opt.arg);
 			}
 			break;
 		case 'f':
 			fullfile = true;
 			break;
 		case 'F':
-			forcebin = r_optarg;
+			forcebin = opt.arg;
 			break;
 		case 'h':
 			help++;
 			break;
 		case 'H':
-			main_print_var (r_optarg);
+			main_print_var (opt.arg);
 			LISTS_FREE ();
+            free (debugbackend);
 			return 0;
 		case 'i':
-			r_list_append (files, r_optarg);
+			if (R_STR_ISEMPTY (opt.arg)) {
+				eprintf ("Cannot open empty script path\n");
+				ret = 1;
+				goto beach;
+			}
+			r_list_append (files, (void*)opt.arg);
 			break;
 		case 'I':
-			r_list_append (prefiles, r_optarg);
+			if (R_STR_ISEMPTY (opt.arg)) {
+				eprintf ("Cannot open empty script path\n");
+				ret = 1;
+				goto beach;
+			}
+			r_list_append (prefiles, (void*)opt.arg);
 			break;
 		case 'k':
-			asmos = r_optarg;
+			asmos = opt.arg;
 			break;
 		case 'l':
-			r_lib_open (r->lib, r_optarg);
+			r_lib_open (r->lib, opt.arg);
 			break;
 		case 'L':
 			do_list_io_plugins = true;
 			break;
 		case 'm':
-			mapaddr = r_num_math (r->num, r_optarg);
-			s_seek = r_optarg;
+			mapaddr = r_num_math (r->num, opt.arg);
+			s_seek = opt.arg;
 			r_config_set_i (r->config, "file.offset", mapaddr);
 			break;
 		case 'M':
@@ -543,19 +582,30 @@ R_API int r_main_radare2(int argc, char **argv) {
 			r_config_set (r->config, "file.info", "false");
 			break;
 		case 'N':
-			run_rc = false;
+			if (run_rc) {
+				run_rc = false;
+			} else {
+				load_l = false;
+			}
 			break;
 		case 'p':
-			if (!strcmp (r_optarg, "?")) {
+			if (!strcmp (opt.arg, "?")) {
 				r_core_project_list (r, 0);
 				r_cons_flush ();
+				free (envprofile);
+				free (debugbackend);
 				LISTS_FREE ();
 				return 0;
 			}
-			r_config_set (r->config, "prj.name", r_optarg);
+			project_name = opt.arg;
 			break;
 		case 'P':
-			patchfile = r_optarg;
+			if (R_STR_ISEMPTY (opt.arg)) {
+				eprintf ("Cannot open empty rapatch path\n");
+				ret = 1;
+				goto beach;
+			}
+			patchfile = opt.arg;
 			break;
 		case 'Q':
 			quiet = true;
@@ -571,14 +621,19 @@ R_API int r_main_radare2(int argc, char **argv) {
 			quiet = true;
 			break;
 		case 'r':
+			if (R_STR_ISEMPTY (opt.arg)) {
+				eprintf ("Cannot open empty rarun2 profile path\n");
+				ret = 1;
+				goto beach;
+			}
 			haveRarunProfile = true;
-			r_config_set (r->config, "dbg.profile", r_optarg);
+			r_config_set (r->config, "dbg.profile", opt.arg);
 			break;
 		case 'R':
-			customRarunProfile = r_str_appendf (customRarunProfile, "%s\n", r_optarg);
+			customRarunProfile = r_str_appendf (customRarunProfile, "%s\n", opt.arg);
 			break;
 		case 's':
-			s_seek = r_optarg;
+			s_seek = opt.arg;
 			break;
 		case 'S':
 			sandbox = true;
@@ -588,7 +643,7 @@ R_API int r_main_radare2(int argc, char **argv) {
 #if ALLOW_THREADED
 			threaded = true;
 #else
-			eprintf ("WARNING: -t is temporarily disabled!\n");
+			eprintf ("Warning: -t is temporarily disabled!\n");
 #endif
 			break;
 #endif
@@ -596,17 +651,8 @@ R_API int r_main_radare2(int argc, char **argv) {
 			compute_hashes = false;
 			break;
 		case 'v':
-			if (quiet) {
-				printf ("%s\n", R2_VERSION);
-				LISTS_FREE ();
-				free (customRarunProfile);
-				return 0;
-			} else {
-				r_main_version_verify (0);
-				LISTS_FREE ();
-				free (customRarunProfile);
-				return r_main_version_print ("radare2");
-			}
+			show_version = true;
+			break;
 		case 'V':
 			return r_main_version_verify (1);
 		case 'w':
@@ -620,10 +666,45 @@ R_API int r_main_radare2(int argc, char **argv) {
 			help++;
 		}
 	}
+	if (show_version) {
+		if (json) {
+			PJ *pj = pj_new ();
+			pj_o (pj);
+			pj_ks (pj, "name", "radare2");
+			pj_ks (pj, "version", R2_VERSION);
+			pj_ks (pj, "birth", R2_BIRTH);
+			pj_ks (pj, "commit", R2_GITTIP);
+			pj_ki (pj, "commits", R2_VERSION_COMMIT);
+			pj_ks (pj, "license", "LGPLv3");
+			pj_ks (pj, "tap", R2_GITTAP);
+			pj_ko (pj, "semver");
+			pj_ki (pj, "major", R2_VERSION_MAJOR);
+			pj_ki (pj, "minor", R2_VERSION_MINOR);
+			pj_ki (pj, "patch", R2_VERSION_MINOR);
+			pj_end (pj);
+			pj_end (pj);
+			char *s = pj_drain (pj);
+			printf ("%s\n", s);
+			free (s);
+		} else if (quiet) {
+			printf ("%s\n", R2_VERSION);
+			LISTS_FREE ();
+			free (debugbackend);
+			free (customRarunProfile);
+		} else {
+			r_main_version_verify (0);
+			LISTS_FREE ();
+			free (customRarunProfile);
+			free (debugbackend);
+			return r_main_version_print ("radare2");
+		}
+		return 0;
+	}
 	if (noStderr) {
 		if (-1 == close (2)) {
 			eprintf ("Failed to close stderr");
 			LISTS_FREE ();
+			R_FREE (debugbackend);
 			return 1;
 		}
 		const char nul[] = R_SYS_DEVNULL;
@@ -631,17 +712,24 @@ R_API int r_main_radare2(int argc, char **argv) {
 		if (-1 == new_stderr) {
 			eprintf ("Failed to open %s", nul);
 			LISTS_FREE ();
+			R_FREE (debugbackend);
 			return 1;
 		}
 		if (2 != new_stderr) {
+#if !__wasi__
 			if (-1 == dup2 (new_stderr, 2)) {
 				eprintf ("Failed to dup2 stderr");
+				free (envprofile);
 				LISTS_FREE ();
+				R_FREE (debugbackend);
 				return 1;
 			}
+#endif
 			if (-1 == close (new_stderr)) {
 				eprintf ("Failed to close %s", nul);
 				LISTS_FREE ();
+				free (envprofile);
+				R_FREE (debugbackend);
 				return 1;
 			}
 		}
@@ -670,9 +758,16 @@ R_API int r_main_radare2(int argc, char **argv) {
 				pfile = NULL; //strdup ("");
 			}
 		} else {
-			pfile = argv[r_optind] ? strdup (argv[r_optind]) : NULL;
+			pfile = argv[opt.ind] ? strdup (argv[opt.ind]) : NULL;
 		}
 	}
+
+	if (pfile && !*pfile) {
+		eprintf ("Cannot open empty path\n");
+		ret = 1;
+		goto beach;
+	}
+
 	if (do_list_io_plugins) {
 		if (r_config_get_i (r->config, "cfg.plugins")) {
 			r_core_loadlibs (r, R_CORE_LOADLIBS_ALL, NULL);
@@ -682,16 +777,24 @@ R_API int r_main_radare2(int argc, char **argv) {
 		if (quietLeak) {
 			exit (0);
 		}
-		r_io_plugin_list (r->io);
+		if (json) {
+			r_io_plugin_list_json (r->io);
+		} else {
+			r_io_plugin_list (r->io);
+		}
 		r_cons_flush ();
 		LISTS_FREE ();
 		free (pfile);
+		R_FREE (debugbackend);
+		free (envprofile);
 		return 0;
 	}
 
 	if (help > 0) {
 		LISTS_FREE ();
 		free (pfile);
+		R_FREE (debugbackend);
+		free (envprofile);
 		return main_help (help > 1? 2: 0);
 	}
 #if __WINDOWS__
@@ -706,14 +809,17 @@ R_API int r_main_radare2(int argc, char **argv) {
 			r_config_set (r->config, "dbg.profile", tfn);
 		}
 		free (tfn);
+		R_FREE (customRarunProfile);
 	}
 	if (debug == 1) {
-		if (r_optind >= argc && !haveRarunProfile) {
+		if (opt.ind >= argc && !haveRarunProfile) {
 			eprintf ("Missing argument for -d\n");
 			LISTS_FREE ();
+			R_FREE (debugbackend);
+			free (envprofile);
 			return 1;
 		}
-		const char *src = haveRarunProfile? pfile: argv[r_optind];
+		const char *src = haveRarunProfile? pfile: argv[opt.ind];
 		if (src && *src) {
 			char *uri = strdup (src);
 			if (uri) {
@@ -722,19 +828,20 @@ R_API int r_main_radare2(int argc, char **argv) {
 					*p = 0;
 					// TODO: this must be specified by the io plugin, not hardcoded here
 					if (!strcmp (uri, "winedbg")) {
-						debugbackend = "io";
+						debugbackend = strdup ("io");
 					} else {
 						debugbackend = uri;
+						uri = NULL;
 					}
 					debug = 2;
-				} else {
-					free (uri);
 				}
+				free (uri);
 			}
 		}
 	}
 
-	if ((tmp = r_sys_getenv ("R2_NOPLUGINS"))) {
+	tmp = NULL;
+	if (!load_l || (tmp = r_sys_getenv ("R2_NOPLUGINS"))) {
 		r_config_set_i (r->config, "cfg.plugins", 0);
 		free (tmp);
 	}
@@ -747,27 +854,30 @@ R_API int r_main_radare2(int argc, char **argv) {
 
 	r_bin_force_plugin (r->bin, forcebin);
 
-	prj = r_config_get (r->config, "prj.name");
-	if (prj && *prj) {
-		r_core_project_open (r, prj, false);
-		r_config_set (r->config, "bin.strings", "false");
+	if (project_name) {
+		if (!r_core_project_open (r, project_name)) {
+			eprintf ("Cannot find project.\n");
+			return 1;
+		}
 	}
 
 	if (do_connect) {
-		const char *uri = argv[r_optind];
-		if (r_optind >= argc) {
+		const char *uri = argv[opt.ind];
+		if (opt.ind >= argc) {
 			eprintf ("Missing URI for -C\n");
 			LISTS_FREE ();
+			R_FREE (debugbackend);
+			free (envprofile);
 			return 1;
 		}
 		if (strstr (uri, "://")) {
-			r_core_cmdf (r, "=+%s", uri);
+			r_core_cmdf (r, "=+ %s", uri);
 		} else {
-			r_core_cmdf (r, "=+http://%s/cmd/", argv[r_optind]);
+			argv[opt.ind] = r_str_newf ("http://%s/cmd/", argv[opt.ind]);
+			r_core_cmdf (r, "=+ %s", argv[opt.ind]);
 		}
-		r_core_cmd0 (r, "=!=");
-		//LISTS_FREE ();
-	//	return 0;
+		r_core_cmd0 (r, "=!=0");
+		argv[opt.ind] = "-";
 	}
 
 	switch (zflag) {
@@ -799,9 +909,9 @@ R_API int r_main_radare2(int argc, char **argv) {
 			if (file && *file && *file != '.') {
 				complete_path = r_str_newf ("%s"R_SYS_DIR"%s", path, file);
 				if (r_str_endswith (complete_path, "gz")) {
-					r_sign_load_gz (r->anal, complete_path);
+					r_sign_load_gz (r->anal, complete_path, false);
 				} else {
-					r_sign_load (r->anal, complete_path);
+					r_sign_load (r->anal, complete_path, false);
 				}
 				free (complete_path);
 			}
@@ -815,27 +925,34 @@ R_API int r_main_radare2(int argc, char **argv) {
 			eprintf ("Error: Cannot debug directories, yet.\n");
 			LISTS_FREE ();
 			free (pfile);
+			R_FREE (debugbackend);
+			free (envprofile);
 			return 1;
 		}
-		if (r_sys_chdir (argv[r_optind])) {
+		if (r_sys_chdir (argv[opt.ind])) {
 			eprintf ("[d] Cannot open directory\n");
 			LISTS_FREE ();
 			free (pfile);
+			R_FREE (debugbackend);
+			free (envprofile);
 			return 1;
 		}
-	} else if (argv[r_optind] && !strcmp (argv[r_optind], "=")) {
+	} else if (argv[opt.ind] && !strcmp (argv[opt.ind], "=")) {
 		int sz;
 		/* stdin/batch mode */
-		ut8 *buf = (ut8 *)r_stdin_slurp (&sz);
+		char *buf = r_stdin_slurp (&sz);
 		eprintf ("^D\n");
 		r_cons_set_raw (false);
 #if __UNIX__
 		// TODO: keep flags :?
 		(void)freopen ("/dev/tty", "rb", stdin);
-		(void)freopen ("/dev/tty","w",stdout);
-		(void)freopen ("/dev/tty","w",stderr);
+		(void)freopen ("/dev/tty", "w", stdout);
+		(void)freopen ("/dev/tty", "w", stderr);
 #else
 		eprintf ("Cannot reopen stdin without UNIX\n");
+		free (buf);
+		R_FREE (debugbackend);
+		free (envprofile);
 		return 1;
 #endif
 		if (buf && sz > 0) {
@@ -847,21 +964,25 @@ R_API int r_main_radare2(int argc, char **argv) {
 				eprintf ("[=] Cannot open '%s'\n", path);
 				LISTS_FREE ();
 				free (path);
+				free (envprofile);
+				R_FREE (debugbackend);
 				return 1;
 			}
-			r_io_map_new (r->io, fh->fd, 7, 0LL, mapaddr,
+			r_io_map_add (r->io, fh->fd, 7, 0LL, mapaddr,
 					r_io_fd_size (r->io, fh->fd));
-			r_io_write_at (r->io, mapaddr, buf, sz);
+			r_io_write_at (r->io, mapaddr, (const ut8 *)buf, sz);
 			r_core_block_read (r);
 			free (buf);
 			free (path);
 			// TODO: load rbin thing
 		} else {
 			eprintf ("Cannot slurp from stdin\n");
+			free (buf);
 			LISTS_FREE ();
+			free (envprofile);
 			return 1;
 		}
-	} else if (strcmp (argv[r_optind - 1], "--") && !(r_config_get (r->config, "prj.name") && r_config_get (r->config, "prj.name")[0]) ) {
+	} else if (strcmp (argv[opt.ind - 1], "--") && !project_name) {
 		if (debug) {
 			if (asmbits) {
 				r_config_set (r->config, "asm.bits", asmbits);
@@ -869,9 +990,11 @@ R_API int r_main_radare2(int argc, char **argv) {
 			r_config_set (r->config, "search.in", "dbg.map"); // implicit?
 			r_config_set (r->config, "cfg.debug", "true");
 			perms = R_PERM_RWX;
-			if (r_optind >= argc) {
+			if (opt.ind >= argc) {
 				eprintf ("No program given to -d\n");
 				LISTS_FREE ();
+				R_FREE (debugbackend);
+				free (envprofile);
 				return 1;
 			}
 			if (debug == 2) {
@@ -879,12 +1002,12 @@ R_API int r_main_radare2(int argc, char **argv) {
 				r_config_set (r->config, "dbg.backend", debugbackend);
 				if (strcmp (debugbackend, "native")) {
 					if (!haveRarunProfile) {
-						pfile = strdup (argv[r_optind++]);
+						pfile = strdup (argv[opt.ind++]);
 					}
 					perms = R_PERM_RX; // XXX. should work with rw too
 					debug = 2;
 					if (!strstr (pfile, "://")) {
-						r_optind--; // take filename
+						opt.ind--; // take filename
 					}
 #if __WINDOWS__
 					pfile = r_acp_to_utf8 (pfile);
@@ -931,9 +1054,10 @@ R_API int r_main_radare2(int argc, char **argv) {
 					}
 				}
 			} else {
-				const char *f = (haveRarunProfile && pfile)? pfile: argv[r_optind];
+				char *f = (haveRarunProfile && pfile)? strdup (pfile): strdup (argv[opt.ind]);
 				is_gdb = (!memcmp (f, "gdb://", R_MIN (f? strlen (f):0, 6)));
 				if (!is_gdb) {
+					free (pfile);
 					pfile = strdup ("dbg://");
 				}
 #if __UNIX__
@@ -959,7 +1083,9 @@ R_API int r_main_radare2(int argc, char **argv) {
 				}
 #else
 #	if __WINDOWS__
-				f = r_acp_to_utf8 (f);
+				char *f2 = r_acp_to_utf8 (f);
+				free (f);
+				f = f2;
 #	endif // __WINDOWS__
 				if (f) {
 					char *escaped_path = r_str_arg_escape (f);
@@ -968,13 +1094,14 @@ R_API int r_main_radare2(int argc, char **argv) {
 					file = pfile; // r_str_append (file, escaped_path);
 				}
 #endif
-				r_optind++;
-				while (r_optind < argc) {
-					char *escaped_arg = r_str_arg_escape (argv[r_optind]);
+				opt.ind++;
+				free (f);
+				while (opt.ind < argc) {
+					char *escaped_arg = r_str_arg_escape (argv[opt.ind]);
 					file = r_str_append (file, " ");
 					file = r_str_append (file, escaped_arg);
 					free (escaped_arg);
-					r_optind++;
+					opt.ind++;
 				}
 				pfile = file;
 			}
@@ -991,19 +1118,25 @@ R_API int r_main_radare2(int argc, char **argv) {
 
 		if (!debug || debug == 2) {
 			const char *dbg_profile = r_config_get (r->config, "dbg.profile");
-			if (r_optind == argc && dbg_profile && *dbg_profile) {
+			if (opt.ind == argc && dbg_profile && *dbg_profile) {
+				if (R_STR_ISEMPTY (pfile)) {
+					eprintf ("Missing file to open\n");
+					ret = 1;
+					R_FREE (debugbackend);
+					goto beach;
+				}
 				fh = r_core_file_open (r, pfile, perms, mapaddr);
 				if (fh) {
 					r_core_bin_load (r, pfile, baddr);
 				}
 			}
-			if (r_optind < argc) {
+			if (opt.ind < argc) {
 				R_FREE (pfile);
-				while (r_optind < argc) {
-					pfile = argv[r_optind++];
+				while (opt.ind < argc) {
+					pfile = strdup (argv[opt.ind++]);
 #if __WINDOWS__
 					pfile = r_acp_to_utf8 (pfile);
-#endif // __WINDOWS__
+#endif
 					fh = r_core_file_open (r, pfile, perms, mapaddr);
 					if (!fh && perms & R_PERM_W) {
 						perms |= R_PERM_CREAT;
@@ -1016,6 +1149,12 @@ R_API int r_main_radare2(int argc, char **argv) {
 							 eprintf ("r_io_create: Permission denied.\n");
 						}
 					}
+					if (baddr == UT64_MAX) {
+						const ut64 io_plug_baddr = r_config_get_i (r->config, "bin.baddr");
+						if (io_plug_baddr != UT64_MAX) {
+							baddr = io_plug_baddr;
+						}
+					}
 					if (fh) {
 						iod = r->io ? r_io_desc_get (r->io, fh->fd) : NULL;
 						if (iod && perms & R_PERM_X) {
@@ -1028,20 +1167,22 @@ R_API int r_main_radare2(int argc, char **argv) {
 								filepath = file? strstr (file, "://"): NULL;
 								filepath = filepath ? filepath + 3 : pfile;
 							}
-							if (r->file && iod && (iod->fd == r->file->fd) && iod->name) {
+							if (r->io->desc && iod && (iod->fd == r->io->desc->fd) && iod->name) {
 								filepath = iod->name;
 							}
 							/* Load rbin info from r2 dbg:// or r2 /bin/ls */
 							/* the baddr should be set manually here */
-							(void)r_core_bin_load (r, filepath, baddr);
-							// check if bin info is loaded and complain if -B was used
-							RBinFile *bi = r_bin_cur (r->bin);
-							bool haveBinInfo = bi && bi->o && bi->o->info && bi->o->info->type;
-							if (!haveBinInfo && baddr != UT64_MAX) {
-								eprintf ("Warning: Don't use -B on unknown files. Consider using -m.\n");
+							if (filepath) {
+								(void)r_core_bin_load (r, filepath, baddr);
+								// check if bin info is loaded and complain if -B was used
+								RBinFile *bi = r_bin_cur (r->bin);
+								bool haveBinInfo = bi && bi->o && bi->o->info && bi->o->info->type;
+								if (!haveBinInfo && baddr != UT64_MAX) {
+									eprintf ("Warning: Don't use -B on unknown files. Consider using -m.\n");
+								}
 							}
 						} else {
-							r_io_map_new (r->io, iod->fd, perms, 0LL, mapaddr, r_io_desc_size (iod));
+							r_io_map_add (r->io, iod->fd, perms, 0LL, mapaddr, r_io_desc_size (iod));
 							if (load_bin == LOAD_BIN_STRUCTURES_ONLY) {
 								r_core_bin_load_structs (r, iod->name);
 							}
@@ -1049,9 +1190,8 @@ R_API int r_main_radare2(int argc, char **argv) {
 					}
 				}
 			} else {
-				const char *prj = r_config_get (r->config, "prj.name");
-				if (prj && *prj) {
-					pfile = r_core_project_info (r, prj);
+				if (project_name) {
+					pfile = r_core_project_name (r, project_name);
 					if (pfile) {
 						if (!fh) {
 							fh = r_core_file_open (r, pfile, perms, mapaddr);
@@ -1066,22 +1206,21 @@ R_API int r_main_radare2(int argc, char **argv) {
 						iod = r->io ? r_io_desc_get (r->io, fh->fd) : NULL;
 						if (iod) {
 							perms = iod->perm;
-							r_io_map_new (r->io, iod->fd, perms, 0LL, 0LL, r_io_desc_size (iod));
+							r_io_map_add (r->io, iod->fd, perms, 0LL, 0LL, r_io_desc_size (iod));
 						}
 					}
 				}
 			}
 			if (mapaddr) {
-				eprintf ("WARNING: using oba to load the syminfo from different mapaddress.\n");
-				eprintf ("TODO: Must use the API instead of running commands to speedup loading times.\n");
 				if (r_config_get_i (r->config, "file.info")) {
+					eprintf ("Warning: using oba to load the syminfo from different mapaddress.\n");
 					// load symbols when using r2 -m 0x1000 /bin/ls
 					r_core_cmdf (r, "oba 0 0x%"PFMT64x, mapaddr);
 					r_core_cmd0 (r, ".ies*");
 				}
 			}
 		} else {
-			RCoreFile *f = r_core_file_open (r, pfile, perms, mapaddr);
+			RIODesc *f = r_core_file_open (r, pfile, perms, mapaddr);
 			if (f) {
 				fh = f;
 			}
@@ -1092,19 +1231,10 @@ R_API int r_main_radare2(int argc, char **argv) {
 			// NOTE: the baddr is redefined to support PIE/ASLR
 			baddr = r_debug_get_baddr (r->dbg, pfile);
 
-			if (baddr != UT64_MAX && baddr != 0 && r->dbg->verbose) {
-				eprintf ("bin.baddr 0x%08" PFMT64x "\n", baddr);
-			}
 			if (load_bin == LOAD_BIN_ALL) {
-				if (baddr && baddr != UT64_MAX && r->dbg->verbose) {
-					eprintf ("Using 0x%" PFMT64x "\n", baddr);
-				}
 				if (r_core_bin_load (r, pfile, baddr)) {
 					RBinObject *obj = r_bin_cur_object (r->bin);
 					if (obj && obj->info) {
-						if (r->dbg->verbose) {
-							eprintf ("asm.bits %d\n", obj->info->bits);
-						}
 #if __linux__ && __GNU_LIBRARY__ && __GLIBC__ && __GLIBC_MINOR__ && __x86_64__
 						ut64 bitness = r_config_get_i (r->config, "asm.bits");
 						if (bitness == 32) {
@@ -1116,8 +1246,10 @@ R_API int r_main_radare2(int argc, char **argv) {
 				}
 			}
 			r_core_cmd0 (r, ".dm*");
-			// Set Thumb Mode if necessary
-			r_core_cmd0 (r, "dr? thumb;?? e asm.bits=16");
+			if (asmarch && r_str_startswith (asmarch, "arm") && r_config_get_i (r->config, "asm.bits") < 64) {
+				// Set Thumb Mode if necessary
+				r_core_cmd0 (r, "dr? thumb;?? e asm.bits=16");
+			}
 			r_cons_reset ();
 		}
 		if (!pfile) {
@@ -1137,7 +1269,7 @@ R_API int r_main_radare2(int argc, char **argv) {
 			ret = 1;
 			goto beach;
 		}
-		if (!r->file) { // no given file
+		if (!r->io->desc) { // no given file
 			ret = 1;
 			goto beach;
 		}
@@ -1148,39 +1280,33 @@ R_API int r_main_radare2(int argc, char **argv) {
 		r_core_cmd0 (r, "=!"); // initalize io subsystem
 		iod = r->io ? r_io_desc_get (r->io, fh->fd) : NULL;
 		if (mapaddr) {
-			r_core_seek (r, mapaddr, 1);
+			r_core_seek (r, mapaddr, true);
 		}
 		r_list_foreach (evals, iter, cmdn) {
 			r_config_eval (r->config, cmdn, false);
 			r_cons_flush ();
 		}
-#if 0
-// Do not autodetect utf8 terminals to avoid problems on initial
-// stdin buffer and some terminals that just hang (android/ios)
-		if (!quiet && r_cons_is_utf8 ()) {
-			r_config_set_i (r->config, "scr.utf8", true);
-		}
-#endif
-		if (asmarch) {
-			r_config_set (r->config, "asm.arch", asmarch);
-		}
 		if (asmbits) {
 			r_config_set (r->config, "asm.bits", asmbits);
 		}
+		if (asmarch) {
+			r_config_set (r->config, "asm.arch", asmarch);
+		}
 		if (asmos) {
 			r_config_set (r->config, "asm.os", asmos);
 		}
 
-		debug = r->file && iod && (r->file->fd == iod->fd) && iod->plugin && \
+		debug = r->io->desc && iod && (r->io->desc->fd == iod->fd) && iod->plugin && \
 			iod->plugin->isdbg;
 		if (debug) {
 			r_core_setup_debugger (r, debugbackend, baddr == UT64_MAX);
 		}
+		R_FREE (debugbackend);
 		RBinObject *o = r_bin_cur_object (r->bin);
 		if (!debug && o && !o->regstate) {
 			RFlagItem *fi = r_flag_get (r->flags, "entry0");
 			if (fi) {
-				r_core_seek (r, fi->offset, 1);
+				r_core_seek (r, fi->offset, true);
 			} else {
 				if (o) {
 					RList *sections = r_bin_get_sections (r->bin);
@@ -1189,14 +1315,14 @@ R_API int r_main_radare2(int argc, char **argv) {
 					r_list_foreach (sections, iter, s) {
 						if (s->perm & R_PERM_X) {
 							ut64 addr = s->vaddr? s->vaddr: s->paddr;
-							r_core_seek (r, addr, 1);
+							r_core_seek (r, addr, true);
 							break;
 						}
 					}
 				}
 			}
 		}
-		if (o && compute_hashes) {
+		if (o && o->info && compute_hashes) {
 			// TODO: recall with limit=0 ?
 			ut64 limit = r_config_get_i (r->config, "bin.hashlimit");
 			r_bin_file_set_hashes (r->bin, r_bin_file_compute_hashes (r->bin, limit));
@@ -1204,7 +1330,7 @@ R_API int r_main_radare2(int argc, char **argv) {
 		if (s_seek) {
 			seek = r_num_math (r->num, s_seek);
 			if (seek != UT64_MAX) {
-				r_core_seek (r, seek, 1);
+				r_core_seek (r, seek, true);
 			}
 		}
 
@@ -1212,23 +1338,7 @@ R_API int r_main_radare2(int argc, char **argv) {
 			r_core_block_size (r, r_io_desc_size (iod));
 		}
 
-		r_core_seek (r, r->offset, 1); // read current block
-
-		/* check if file.path has changed */
-		if (iod && !strstr (iod->uri, "://")) {
-			const char *npath;
-			char *path = strdup (r_config_get (r->config, "file.path"));
-			has_project = r_core_project_open (r, r_config_get (r->config, "prj.name"), false);
-			iod = r->io ? r_io_desc_get (r->io, fh->fd) : NULL;
-			if (has_project) {
-				r_config_set (r->config, "bin.strings", "false");
-			}
-			npath = r_config_get (r->config, "file.path");
-			if (!quiet && path && *path && npath && strcmp (path, npath)) {
-				eprintf ("WARNING: file.path change: %s => %s\n", path, npath);
-			}
-			free (path);
-		}
+		r_core_seek (r, r->offset, true); // read current block
 
 		r_list_foreach (evals, iter, cmdn) {
 			r_config_eval (r->config, cmdn, false);
@@ -1261,6 +1371,7 @@ R_API int r_main_radare2(int argc, char **argv) {
 		}
 		free (global_rc);
 	}
+
 	// only analyze if file contains entrypoint
 	{
 		char *s = r_core_cmd_str (r, "ieq");
@@ -1283,7 +1394,7 @@ R_API int r_main_radare2(int argc, char **argv) {
 	}
 #if UNCOLORIZE_NONTTY
 #if __UNIX__
-	if (!r_cons_isatty ()) {
+	if (!r_cons_is_tty ()) {
 		r_config_set_i (r->config, "scr.color", COLOR_MODE_DISABLED);
 	}
 #endif
@@ -1325,7 +1436,7 @@ R_API int r_main_radare2(int argc, char **argv) {
 		char *data = r_file_slurp (patchfile, NULL);
 		if (data) {
 			ret = r_core_patch (r, data);
-			r_core_seek (r, 0, 1);
+			r_core_seek (r, 0, true);
 			free (data);
 		} else {
 			eprintf ("[p] Cannot open '%s'\n", patchfile);
@@ -1336,7 +1447,7 @@ R_API int r_main_radare2(int argc, char **argv) {
 			r_cons_zero ();
 		}
 		if (seek != UT64_MAX) {
-			r_core_seek (r, seek, 1);
+			r_core_seek (r, seek, true);
 		}
 
 		// no flagspace selected by default the beginning
@@ -1347,7 +1458,9 @@ R_API int r_main_radare2(int argc, char **argv) {
 			}
 		}
 		for (;;) {
-			r_core_prompt_loop (r);
+			if (!r_core_prompt_loop (r)) {
+				quietLeak = true;
+			}
 			ret = r->num->value;
 			debug = r_config_get_i (r->config, "cfg.debug");
 			if (ret != -1 && r_cons_is_interactive ()) {
@@ -1385,27 +1498,31 @@ R_API int r_main_radare2(int argc, char **argv) {
 					}
 				}
 
-				prj = r_config_get (r->config, "prj.name");
-				if (no_question_save) {
-					if (prj && *prj && y_save_project){
-						r_core_project_save (r, prj);
+				const char *prj = r_config_get (r->config, "prj.name");
+				if (R_STR_ISNOTEMPTY (prj)) {
+					if (r_core_project_is_saved (r)) {
+						break;
 					}
-				} else {
-					question = r_str_newf ("Do you want to save the '%s' project? (Y/n)", prj);
-					if (prj && *prj && r_cons_yesno ('y', "%s", question)) {
-						r_core_project_save (r, prj);
+					if (no_question_save) {
+						if (y_save_project) {
+							r_core_project_save (r, prj);
+						}
+					} else {
+						question = r_str_newf ("Do you want to save the '%s' project? (Y/n)", prj);
+						if (r_cons_yesno ('y', "%s", question)) {
+							r_core_project_save (r, prj);
+						}
+						free (question);
 					}
-					free (question);
 				}
-
-				if (r_config_get_i (r->config, "scr.confirmquit")) {
+				if (r_config_get_b (r->config, "scr.confirmquit")) {
 					if (!r_cons_yesno ('n', "Do you want to quit? (Y/n)")) {
 						continue;
 					}
 				}
 			} else {
 				// r_core_project_save (r, prj);
-				if (debug && r_config_get_i (r->config, "dbg.exitkills")) {
+				if (debug && r_config_get_b (r->config, "dbg.exitkills")) {
 					r_debug_kill (r->dbg, 0, false, 9); // KILL
 				}
 
@@ -1432,10 +1549,10 @@ beach:
 	// and this fh may be come stale during the command
 	// execution.
 	//r_core_file_close (r, fh);
+	free (envprofile);
+	free (debugbackend);
 	r_core_free (r);
-	r_cons_set_raw (0);
-	free (file);
-	r_cons_free ();
 	LISTS_FREE ();
+	R_FREE (pfile);
 	return ret;
 }
