@@ -1,23 +1,14 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2020 - pancake */
 
-#include <r_diff.h>
 #include <r_core.h>
-#include <r_hash.h>
-#include <limits.h>
 #include <r_main.h>
-#ifdef _MSC_VER
-#ifndef WIN32_LEAN_AND_MEAN
-#define WIN32_LEAN_AND_MEAN
-#endif
-#endif
 
 enum {
 	MODE_DIFF,
 	MODE_DIFF_STRS,
 	MODE_DIFF_IMPORTS,
-	MODE_DIST,
 	MODE_DIST_MYERS,
-	MODE_DIST_LEVENSTEIN,
+	MODE_DIST_LEVENSHTEIN,
 	MODE_CODE,
 	MODE_GRAPH,
 	MODE_COLS,
@@ -25,39 +16,47 @@ enum {
 };
 
 enum {
-        GRAPH_DEFAULT_MODE,
-        GRAPH_SDB_MODE,
-        GRAPH_JSON_MODE,
-        GRAPH_JSON_DIS_MODE,
-        GRAPH_TINY_MODE,
-        GRAPH_INTERACTIVE_MODE,
-        GRAPH_DOT_MODE,
-        GRAPH_STAR_MODE,
-        GRAPH_GML_MODE
+	GRAPH_DEFAULT_MODE,
+	GRAPH_SDB_MODE,
+	GRAPH_JSON_MODE,
+	GRAPH_JSON_DIS_MODE,
+	GRAPH_TINY_MODE,
+	GRAPH_INTERACTIVE_MODE,
+	GRAPH_DOT_MODE,
+	GRAPH_STAR_MODE,
+	GRAPH_GML_MODE
 };
 
-static bool zignatures = false;
-static char *file = NULL;
-static char *file2 = NULL;
-static ut32 count = 0;
-static int showcount = 0;
-static int useva = true;
-static int delta = 0;
-static int showbare = false;
-static int json_started = 0;
-static int diffmode = 0;
-static bool disasm = false;
-static bool pdc = false;
-static bool quiet = false;
-static RCore *core = NULL;
-static const char *arch = NULL;
-const char *runcmd = NULL;
-static int bits = 0;
-static int anal_all = 0;
-static bool verbose = false;
-static RList *evals = NULL;
+typedef struct {
+	ut64 gdiff_start;
+	bool zignatures;
+	const char *file;
+	const char *file2;
+	ut32 count;
+	int showcount;
+	int useva;
+	int delta;
+	int showbare;
+	bool json_started;
+	int diffmode;
+	int diffops;
+	int mode;
+	int gmode;
+	bool disasm;
+	bool pdc;
+	bool quiet;
+	RCore *core;
+	const char *arch;
+	const char *runcmd;
+	int bits;
+	int anal_all;
+	int threshold;
+	bool verbose;
+	RList *evals;
+	PJ *pj;
+} RadiffOptions;
 
-static RCore *opencore(const char *f) {
+static RCore *opencore(RadiffOptions *ro, const char *f) {
 	RListIter *iter;
 	const ut64 baddr = UT64_MAX;
 	const char *e;
@@ -66,16 +65,22 @@ static RCore *opencore(const char *f) {
 		return NULL;
 	}
 	r_core_loadlibs (c, R_CORE_LOADLIBS_ALL, NULL);
-	r_config_set_i (c->config, "io.va", useva);
+	r_config_set_i (c->config, "io.va", ro->useva);
 	r_config_set_i (c->config, "scr.interactive", false);
-	r_list_foreach (evals, iter, e) {
+	r_list_foreach (ro->evals, iter, e) {
 		r_config_eval (c->config, e, false);
 	}
 	if (f) {
+		RIODesc * rfile = NULL;
 #if __WINDOWS__
-		f = r_acp_to_utf8 (f);
+		char *winf = r_acp_to_utf8 (f);
+		rfile = r_core_file_open (c, winf, 0, 0);
+		free (winf);
+#else
+		rfile = r_core_file_open (c, f, 0, 0);
 #endif
-		if (!r_core_file_open (c, f, 0, 0)) {
+
+		if (!rfile) {
 			r_core_free (c);
 			return NULL;
 		}
@@ -86,20 +91,19 @@ static RCore *opencore(const char *f) {
 		if (r_list_empty (r_bin_get_sections (c->bin))) {
 			r_config_set_i (c->config, "io.va", false);
 		}
-
-		if (anal_all) {
+		if (ro->anal_all) {
 			const char *cmd = "aac";
-			switch (anal_all) {
+			switch (ro->anal_all) {
 			case 1: cmd = "aaa"; break;
 			case 2: cmd = "aaaa"; break;
 			}
 			r_core_cmd0 (c, cmd);
 		}
-		if (runcmd) {
-			r_core_cmd0 (c, runcmd);
+		if (ro->runcmd) {
+			r_core_cmd0 (c, ro->runcmd);
 		}
 		// generate zignaturez?
-		if (zignatures) {
+		if (ro->zignatures) {
 			r_core_cmd0 (c, "zg");
 		}
 		r_cons_flush ();
@@ -122,57 +126,58 @@ static void readstr(char *s, int sz, const ut8 *buf, int len) {
 }
 
 static int cb(RDiff *d, void *user, RDiffOp *op) {
-	int i; // , diffmode = (int)(size_t)user;
+	int i;
+	RadiffOptions *ro = user;
 	char s[256] = {0};
-	if (showcount) {
-		count++;
+	if (ro->showcount) {
+		ro->count++;
 		return 1;
 	}
-	switch (diffmode) {
+	switch (ro->diffmode) {
 	case 'U': // 'U' in theory never handled here
 	case 'u':
 		if (op->a_len > 0) {
 			readstr (s, sizeof (s), op->a_buf, op->a_len);
 			if (*s) {
-				if (!quiet) {
+				if (!ro->quiet) {
 					printf (Color_RED);
 				}
 				printf ("-0x%08"PFMT64x":", op->a_off);
 				int len = op->a_len; // R_MIN (op->a_len, strlen (op->a_buf));
 				for (i = 0; i < len; i++) {
-					printf ("%02x", op->a_buf[i]);
+					printf ("%02x ", op->a_buf[i]);
 				}
-				char *p = r_str_escape ((const char*)op->a_buf);
-				printf (" \"%s\"\n", p);
-				free (p);
-				if (!quiet) {
+				if (!ro->quiet) {
+					char *p = r_str_escape ((const char*)op->a_buf);
+					printf (" \"%s\"", p);
+					free (p);
 					printf (Color_RESET);
 				}
+				printf ("\n");
 			}
 		}
 		if (op->b_len > 0) {
 			readstr (s, sizeof (s), op->b_buf, op->b_len);
 			if (*s) {
-				if (!quiet) {
+				if (!ro->quiet) {
 					printf (Color_GREEN);
 				}
 				printf ("+0x%08"PFMT64x":", op->b_off);
 				for (i = 0; i < op->b_len; i++) {
-					printf ("%02x", op->b_buf[i]);
+					printf ("%02x ", op->b_buf[i]);
 				}
-				if (!quiet) {
-					char *p = r_str_escape((const char*)op->b_buf);
-					printf (" \"%s\"\n", p);
+				if (!ro->quiet) {
+					char *p = r_str_escape ((const char*)op->b_buf);
+					printf (" \"%s\"", p);
 					free (p);
 					printf (Color_RESET);
-				} else {
-					printf ("\n");
 				}
+				printf ("\n");
 			}
 		}
 		break;
 	case 'r':
-		if (disasm) {
+		if (ro->disasm) {
 			eprintf ("r2cmds (-r) + disasm (-D) not yet implemented\n");
 		}
 		if (op->a_len == op->b_len) {
@@ -184,73 +189,69 @@ static int cb(RDiff *d, void *user, RDiffOp *op) {
 		} else {
 			if (op->a_len > 0) {
 				printf ("r-%d @ 0x%08"PFMT64x "\n",
-					op->a_len, op->a_off + delta);
+					op->a_len, op->a_off + ro->delta);
 			}
 			if (op->b_len > 0) {
 				printf ("r+%d @ 0x%08"PFMT64x "\n",
-					op->b_len, op->b_off + delta);
+					op->b_len, op->b_off + ro->delta);
 				printf ("wx ");
 				for (i = 0; i < op->b_len; i++) {
 					printf ("%02x", op->b_buf[i]);
 				}
-				printf (" @ 0x%08"PFMT64x "\n", op->b_off + delta);
+				printf (" @ 0x%08"PFMT64x "\n", op->b_off + ro->delta);
 			}
-			delta += (op->b_off - op->a_off);
+			ro->delta += (op->b_off - op->a_off);
 		}
 		return 1;
 	case 'j':
-		if (disasm) {
+		// TODO PJ
+		if (ro->disasm) {
 			eprintf ("JSON (-j) + disasm (-D) not yet implemented\n");
 		}
-		if (json_started) {
-			printf (",\n");
+		{
+			PJ *pj = ro->pj;
+			pj_o (pj);
+			pj_kn (pj, "addr", op->a_off);
+			char *hex_from = r_hex_bin2strdup (op->a_buf, op->a_len);
+			pj_ks (pj, "from", hex_from);
+			char *hex_to = r_hex_bin2strdup (op->b_buf, op->b_len);
+			pj_ks (pj, "to", hex_to);
+			pj_end (pj);
 		}
-		json_started = 1;
-		printf ("{\"offset\":%"PFMT64d ",", op->a_off);
-		printf ("\"from\":\"");
-		for (i = 0; i < op->a_len; i++) {
-			printf ("%02x", op->a_buf[i]);
-		}
-		printf ("\", \"to\":\"");
-		for (i = 0; i < op->b_len; i++) {
-			printf ("%02x", op->b_buf[i]);
-		}
-		printf ("\"}"); // ,\n");
 		return 1;
 	case 0:
 	default:
-		if (disasm) {
+		if (ro->disasm) {
 			int i;
 			printf ("--- 0x%08"PFMT64x "  ", op->a_off);
-			if (!core) {
-				core = opencore (file);
-				if (arch) {
-					r_config_set (core->config, "asm.arch", arch);
+			if (!ro->core) {
+				ro->core = opencore (ro, ro->file);
+				if (ro->arch) {
+					r_config_set (ro->core->config, "asm.arch", ro->arch);
 				}
-				if (bits) {
-					r_config_set_i (core->config, "asm.bits", bits);
+				if (ro->bits) {
+					r_config_set_i (ro->core->config, "asm.bits", ro->bits);
 				}
 			}
 			for (i = 0; i < op->a_len; i++) {
 				printf ("%02x", op->a_buf[i]);
 			}
 			printf ("\n");
-			if (core) {
+			if (ro->core) {
 				int len = R_MAX (4, op->a_len);
-				RAsmCode *ac = r_asm_mdisassemble (core->assembler, op->a_buf, len);
+				RAsmCode *ac = r_asm_mdisassemble (ro->core->rasm, op->a_buf, len);
 				char *acbufasm = strdup (ac->assembly);
-				if (quiet) {
+				if (ro->quiet) {
 					char *bufasm = r_str_prefix_all (acbufasm, "- ");
 					printf ("%s\n", bufasm);
 					free (bufasm);
-					free (acbufasm);
 				} else {
 					char *bufasm = r_str_prefix_all (acbufasm, Color_RED"- ");
 					printf ("%s"Color_RESET, bufasm);
 					free (bufasm);
-					free (acbufasm);
 				}
-				// r_asm_code_free (ac);
+				free (acbufasm);
+				r_asm_code_free (ac);
 			}
 		} else {
 			printf ("0x%08"PFMT64x " ", op->a_off);
@@ -258,21 +259,21 @@ static int cb(RDiff *d, void *user, RDiffOp *op) {
 				printf ("%02x", op->a_buf[i]);
 			}
 		}
-		if (disasm) {
+		if (ro->disasm) {
 			int i;
 			printf ("+++ 0x%08"PFMT64x "  ", op->b_off);
-			if (!core) {
-				core = opencore (NULL);
+			if (!ro->core) {
+				ro->core = opencore (ro, NULL);
 			}
 			for (i = 0; i < op->b_len; i++) {
 				printf ("%02x", op->b_buf[i]);
 			}
 			printf ("\n");
-			if (core) {
+			if (ro->core) {
 				int len = R_MAX (4, op->b_len);
-				RAsmCode *ac = r_asm_mdisassemble (core->assembler, op->b_buf, len);
+				RAsmCode *ac = r_asm_mdisassemble (ro->core->rasm, op->b_buf, len);
 				char *acbufasm = strdup (ac->assembly);
-				if (quiet) {
+				if (ro->quiet) {
 					char *bufasm = r_str_prefix_all (acbufasm, "+ ");
 					printf ("%s\n", bufasm);
 					free (bufasm);
@@ -297,18 +298,19 @@ static int cb(RDiff *d, void *user, RDiffOp *op) {
 	return 0;
 }
 
-static ut64 gdiff_start = 0;
-
 void print_bytes(const void *p, size_t len, bool big_endian) {
 	size_t i;
-	for (i = 0; i < len; ++i) {
+	for (i = 0; i < len; i++) {
 		ut8 ch = ((ut8*) p)[big_endian ? (len - i - 1) : i];
-		write (1, &ch, 1);
+		if (write (1, &ch, 1) != 1) {
+			break;
+		}
 	}
 }
 
 static int bcb(RDiff *d, void *user, RDiffOp *op) {
-	ut64 offset_diff = op->a_off - gdiff_start;
+	RadiffOptions *ro = user;
+	ut64 offset_diff = op->a_off - ro->gdiff_start;
 	unsigned char opcode;
 	unsigned short USAddr = 0;
 	int IAddr = 0;
@@ -320,12 +322,12 @@ static int bcb(RDiff *d, void *user, RDiffOp *op) {
 	if (offset_diff > 0) {
 
 		// size for the position
-		if (gdiff_start <= USHRT_MAX) {
+		if (ro->gdiff_start <= USHRT_MAX) {
 			opcode = 249;
-			USAddr = (unsigned short) gdiff_start;
-		} else if (gdiff_start <= INT_MAX) {
+			USAddr = (unsigned short) ro->gdiff_start;
+		} else if (ro->gdiff_start <= INT_MAX) {
 			opcode = 252;
-			IAddr = (int) gdiff_start;
+			IAddr = (int) ro->gdiff_start;
 		} else {
 			opcode = 255;
 		}
@@ -360,7 +362,7 @@ static int bcb(RDiff *d, void *user, RDiffOp *op) {
 		} else if (opcode < 255) {
 			print_bytes (&IAddr, sizeof (IAddr), true);
 		} else {
-			print_bytes (&gdiff_start, sizeof (gdiff_start), true);
+			print_bytes (&ro->gdiff_start, sizeof (ro->gdiff_start), true);
 		}
 
 		// print length for COPY
@@ -384,15 +386,15 @@ static int bcb(RDiff *d, void *user, RDiffOp *op) {
 	// we append data
 	if (op->b_len <= 246) {
 		ut8 data = op->b_len;
-		write (1, &data, 1);
+		(void) write (1, &data, 1);
 	} else if (op->b_len <= USHRT_MAX) {
 		USLen = (ut16) op->b_len;
 		ut8 data = 247;
-		write (1, &data, 1);
+		(void) write (1, &data, 1);
 		print_bytes (&USLen, sizeof (USLen), true);
 	} else if (op->b_len <= INT_MAX) {
 		ut8 data = 248;
-		write (1, &data, 1);
+		(void) write (1, &data, 1);
 		ILen = (int) op->b_len;
 		print_bytes (&ILen, sizeof (ILen), true);
 	} else {
@@ -400,9 +402,11 @@ static int bcb(RDiff *d, void *user, RDiffOp *op) {
 		int times = op->b_len / INT_MAX;
 		int max = INT_MAX;
 		size_t i;
-		for(i = 0;i < times; i++) {
+		for (i = 0; i < times; i++) {
 			ut8 data = 248;
-			write (1, &data, 1);
+			if (write (1, &data, 1) != 1) {
+				break;
+			}
 			print_bytes (&max, sizeof (max), true);
 			print_bytes (op->b_buf, max, false);
 			op->b_buf += max;
@@ -411,15 +415,15 @@ static int bcb(RDiff *d, void *user, RDiffOp *op) {
 
 		// print the remaining size
 		int remain_size = op->b_len;
-		print_bytes(&remain_size, sizeof(remain_size), true);
+		print_bytes (&remain_size, sizeof (remain_size), true);
 	}
-	print_bytes(op->b_buf, op->b_len, false);
-	gdiff_start = op->b_off + op->b_len;
+	print_bytes (op->b_buf, op->b_len, false);
+	ro->gdiff_start = op->b_off + op->b_len;
 	return 0;
 }
 
 static int show_help(int v) {
-	printf ("Usage: radiff2 [-abBcCdjrspOxuUvV] [-A[A]] [-g sym] [-m graph_mode][-t %%] [file] [file]\n");
+	printf ("Usage: radiff2 [-abBcCdeGhijnrOpqsSxuUvVzZ] [-A[A]] [-g sym] [-m graph_mode][-t %%] [file] [file]\n");
 	if (v) {
 		printf (
 			"  -a [arch]  specify architecture plugin to use (x86, arm, ..)\n"
@@ -679,42 +683,46 @@ static void dump_cols_hexii(ut8 *a, int as, ut8 *b, int bs, int w) {
 	}
 }
 
-static void handle_sha256(const ut8 *block, int len) {
+static char *handle_sha256(const ut8 *block, int len) {
 	int i = 0;
+	char *p = malloc (128);
 	RHash *ctx = r_hash_new (true, R_HASH_SHA256);
 	const ut8 *c = r_hash_do_sha256 (ctx, block, len);
 	if (!c) {
 		r_hash_free (ctx);
-		return;
+		free (p);
+		return NULL;
 	}
+	char *r = p;
 	for (i = 0; i < R_HASH_SIZE_SHA256; i++) {
-		printf ("%02x", c[i]);
+		snprintf (r + (i * 2), 3, "%02x", c[i]);
 	}
 	r_hash_free (ctx);
+	return p;
 }
 
-static ut8 *slurp(RCore **c, const char *file, int *sz) {
-	RIODesc *d;
+static ut8 *slurp(RadiffOptions *ro, RCore **c, const char *file, size_t *sz) {
+	int fd;
 	RIO *io;
 	if (c && file && strstr (file, "://")) {
 		ut8 *data = NULL;
 		ut64 size;
 		if (!*c) {
-			*c = opencore (NULL);
+			*c = opencore (ro, NULL);
 		}
 		if (!*c) {
 			eprintf ("opencore failed\n");
 			return NULL;
 		}
 		io = (*c)->io;
-		d = r_io_open (io, file, 0, 0);
-		if (!d) {
+		fd = r_io_fd_open (io, file, R_PERM_R, 0);
+		if (fd < 1) {
 			return NULL;
 		}
-		size = r_io_size (io);
+		size = r_io_fd_size (io, fd);
 		if (size > 0 && size < ST32_MAX) {
 			data = calloc (1, size);
-			if (r_io_read_at (io, 0, data, size)) {
+			if (r_io_fd_read_at (io, fd, 0, data, size)) {
 				if (sz) {
 					*sz = size;
 				}
@@ -725,7 +733,7 @@ static ut8 *slurp(RCore **c, const char *file, int *sz) {
 		} else {
 			eprintf ("slurp: Invalid file size\n");
 		}
-		r_io_desc_close (d);
+		r_io_fd_close (io, fd);
 		return data;
 	}
 	return (ut8 *) r_file_slurp (file, sz);
@@ -744,8 +752,9 @@ static ut8 *get_imports(RCore *c, int *len) {
 		return NULL;
 	}
 
-	RList *list = r_bin_get_imports (c->bin);
-	r_list_sort (list, (RListComparator) import_cmp);
+	const RList *list = r_bin_get_imports (c->bin);
+	// XXX we probably dont want to sort an unowned list
+	r_list_sort ((RList *)list, (RListComparator) import_cmp);
 
 	*len = 0;
 
@@ -823,19 +832,19 @@ static ut8 *get_strings(RCore *c, int *len) {
 }
 
 static char *get_graph_commands(RCore *c, ut64 off) {
-        bool tmp_html = r_cons_singleton ()->is_html;
-        r_cons_singleton ()->is_html = false;
+        bool tmp_html = r_cons_context ()->is_html;
+        r_cons_context ()->is_html = false;
         r_cons_push ();
         r_core_anal_graph (c, off, R_CORE_ANAL_GRAPHBODY | R_CORE_ANAL_GRAPHDIFF |  R_CORE_ANAL_STAR);
         const char *static_str = r_cons_get_buffer ();
-        char *retstr = strdup (static_str? static_str: "");
+        char *retstr = strdup (r_str_get (static_str));
         r_cons_pop ();
         r_cons_echo (NULL);
-        r_cons_singleton ()->is_html = tmp_html;
+        r_cons_context ()->is_html = tmp_html;
         return retstr;
 }
 
-static void __generate_graph (RCore *c, ut64 off) {
+static void __generate_graph(RCore *c, ut64 off) {
         r_return_if_fail (c);
         char *ptr = get_graph_commands (c, off);
 	char *str = ptr;
@@ -907,188 +916,221 @@ static void __print_diff_graph(RCore *c, ut64 off, int gmode) {
                 __generate_graph (c, off);
                 r_core_agraph_print (c, use_utf8, "");
                 r_cons_reset_colors ();
-        break;
+        	break;
         }
 }
 
-R_API int r_main_radiff2(int argc, char **argv) {
+static void radiff_options_init(RadiffOptions *ro) {
+	memset (ro, 0, sizeof (RadiffOptions));
+	ro->threshold = -1;
+	ro->useva = true;
+	ro->evals = r_list_newf (NULL);
+	ro->mode = MODE_DIFF;
+	ro->gmode = GRAPH_DEFAULT_MODE;
+}
+
+static void radiff_options_fini(RadiffOptions *ro) {
+	r_list_free (ro->evals);
+	r_core_free (ro->core);
+	r_cons_free ();
+}
+
+static void fileobj(RadiffOptions *ro, const char *ro_file, const ut8 *buf, size_t sz) {
+	PJ *pj = ro->pj;
+	pj_o (pj);
+	pj_ks (pj, "filename", ro_file);
+	pj_kn (pj, "size", sz);
+	char *hasha = handle_sha256 (buf, (int)sz);
+	pj_ks (pj, "sha256", hasha);
+	free (hasha);
+	pj_end (pj);
+}
+
+R_API int r_main_radiff2(int argc, const char **argv) {
+	RadiffOptions ro;
 	const char *columnSort = NULL;
 	const char *addr = NULL;
 	RCore *c = NULL, *c2 = NULL;
-	RDiff *d;
 	ut8 *bufa = NULL, *bufb = NULL;
-	int o, sza, szb, /*diffmode = 0,*/ delta = 0;
-	int mode = MODE_DIFF;
-	int gmode = GRAPH_DEFAULT_MODE;
-	int diffops = 0;
-	int threshold = -1;
+	int o, /*diffmode = 0,*/ delta = 0;
+	ut64 sza = 0, szb = 0;
 	double sim = 0.0;
-	evals = r_list_newf (NULL);
+	RDiff *d;
+	RGetopt opt;
 
-	while ((o = r_getopt (argc, argv, "Aa:b:BCDe:npg:m:G:OijrhcdsS:uUvVxXt:zqZ")) != -1) {
+	radiff_options_init (&ro);
+
+	r_getopt_init (&opt, argc, argv, "Aa:b:BCDe:npg:m:G:OijrhcdsS:uUvVxXt:zqZ");
+	while ((o = r_getopt_next (&opt)) != -1) {
 		switch (o) {
 		case 'a':
-			arch = r_optarg;
+			ro.arch = opt.arg;
 			break;
 		case 'A':
-			anal_all++;
+			ro.anal_all++;
 			break;
 		case 'b':
-			bits = atoi (r_optarg);
+			ro.bits = atoi (opt.arg);
 			break;
 		case 'B':
-			diffmode = 'B';
+			ro.diffmode = 'B';
 			break;
 		case 'e':
-			r_list_append (evals, r_optarg);
+			r_list_append (ro.evals, (void*)opt.arg);
 			break;
 		case 'p':
-			useva = false;
+			ro.useva = false;
 			break;
 		case 'r':
-			diffmode = 'r';
+			ro.diffmode = 'r';
 			break;
 		case 'g':
-			mode = MODE_GRAPH;
-			addr = r_optarg;
+			ro.mode = MODE_GRAPH;
+			addr = opt.arg;
 			break;
 		case 'm':{
-		        char *tmp = r_optarg;
-		        switch(tmp[0]) {
-	                case 'i': gmode = GRAPH_INTERACTIVE_MODE; break;
-	                case 'k': gmode = GRAPH_SDB_MODE; break;
-	                case 'j': gmode = GRAPH_JSON_MODE; break;
-	                case 'J': gmode = GRAPH_JSON_DIS_MODE; break;
-	                case 't': gmode = GRAPH_TINY_MODE; break;
-	                case 'd': gmode = GRAPH_DOT_MODE; break;
-	                case 's': gmode = GRAPH_STAR_MODE; break;
-	                case 'g': gmode = GRAPH_GML_MODE; break;
+		        const char *tmp = opt.arg;
+		        switch (tmp[0]) {
+	                case 'i': ro.gmode = GRAPH_INTERACTIVE_MODE; break;
+	                case 'k': ro.gmode = GRAPH_SDB_MODE; break;
+	                case 'j': ro.gmode = GRAPH_JSON_MODE; break;
+	                case 'J': ro.gmode = GRAPH_JSON_DIS_MODE; break;
+	                case 't': ro.gmode = GRAPH_TINY_MODE; break;
+	                case 'd': ro.gmode = GRAPH_DOT_MODE; break;
+	                case 's': ro.gmode = GRAPH_STAR_MODE; break;
+	                case 'g': ro.gmode = GRAPH_GML_MODE; break;
 	                case 'a':
-                        default: gmode = GRAPH_DEFAULT_MODE; break;
+                        default: ro.gmode = GRAPH_DEFAULT_MODE; break;
 		        }
 		}       break;
 		case 'G':
-			runcmd = r_optarg;
+			ro.runcmd = opt.arg;
 			break;
 		case 'c':
-			showcount = 1;
+			ro.showcount = 1;
 			break;
 		case 'C':
-			mode = MODE_CODE;
+			ro.mode = MODE_CODE;
 			break;
 		case 'i':
-			mode = MODE_DIFF_IMPORTS;
+			ro.mode = MODE_DIFF_IMPORTS;
 			break;
 		case 'n':
-			showbare = true;
+			ro.showbare = true;
 			break;
 		case 'O':
-			diffops = 1;
+			ro.diffops = 1;
 			break;
 		case 't':
-			threshold = atoi (r_optarg);
-			printf ("%s\n", r_optarg);
+			ro.threshold = atoi (opt.arg);
+			printf ("%s\n", opt.arg);
 			break;
 		case 'd':
 			delta = 1;
 			break;
 		case 'D':
-			if (disasm) {
-				pdc = true;
-				disasm = false;
-				mode = MODE_CODE;
+			if (ro.disasm) {
+				ro.pdc = true;
+				ro.disasm = false;
+				ro.mode = MODE_CODE;
 			} else {
-				disasm = true;
+				ro.disasm = true;
 			}
 			break;
 		case 'h':
 			return show_help (1);
 		case 's':
-			if (mode == MODE_DIST) {
-				mode = MODE_DIST_LEVENSTEIN;
-			} else if (mode == MODE_DIST_LEVENSTEIN) {
-				mode = MODE_DIST_MYERS;
+			if (ro.mode == MODE_DIST_MYERS) {
+				ro.mode = MODE_DIST_LEVENSHTEIN;
 			} else {
-				mode = MODE_DIST;
+				ro.mode = MODE_DIST_MYERS;
 			}
 			break;
 		case 'S':
-			columnSort = r_optarg;
+			columnSort = opt.arg;
 			break;
 		case 'x':
-			mode = MODE_COLS;
+			ro.mode = MODE_COLS;
 			break;
 		case 'X':
-			mode = MODE_COLSII;
+			ro.mode = MODE_COLSII;
 			break;
 		case 'u':
-			diffmode = 'u';
+			ro.diffmode = 'u';
 			break;
 		case 'U':
-			diffmode = 'U';
+			ro.diffmode = 'U';
 			break;
 		case 'v':
 			return r_main_version_print ("radiff2");
 		case 'q':
-			quiet = true;
+			ro.quiet = true;
 			break;
 		case 'V':
-			verbose = true;
+			ro.verbose = true;
 			break;
 		case 'j':
-			diffmode = 'j';
+			ro.diffmode = 'j';
+			ro.pj = pj_new ();
 			break;
 		case 'z':
-			mode = MODE_DIFF_STRS;
+			ro.mode = MODE_DIFF_STRS;
 			break;
 		case 'Z':
-			zignatures = true;
+			ro.zignatures = true;
 			break;
 		default:
 			return show_help (0);
 		}
 	}
 
-	if (argc < 3 || r_optind + 2 > argc) {
+	if (argc < 3 || opt.ind + 2 > argc) {
 		return show_help (0);
 	}
-	file = (r_optind < argc)? argv[r_optind]: NULL;
-	file2 = (r_optind + 1 < argc)? argv[r_optind + 1]: NULL;
+	ro.file = (opt.ind < argc)? argv[opt.ind]: NULL;
+	ro.file2 = (opt.ind + 1 < argc)? argv[opt.ind + 1]: NULL;
 
-	switch (mode) {
+	if (R_STR_ISEMPTY (ro.file) || R_STR_ISEMPTY (ro.file2)) {
+		eprintf ("Cannot open empty path\n");
+		return 1;
+	}
+
+	switch (ro.mode) {
 	case MODE_GRAPH:
 	case MODE_CODE:
 	case MODE_DIFF_STRS:
 	case MODE_DIFF_IMPORTS:
-		c = opencore (file);
+		c = opencore (&ro, ro.file);
 		if (!c) {
-			eprintf ("Cannot open '%s'\n", r_str_get (file));
+			eprintf ("Cannot open '%s'\n", r_str_getf (ro.file));
+		}
+		c2 = opencore (&ro, ro.file2);
+		if (!c2) {
+			eprintf ("Cannot open '%s'\n", r_str_getf (ro.file2));
 		}
-		c2 = opencore (file2);
 		if (!c || !c2) {
-			eprintf ("Cannot open '%s'\n", r_str_get (file2));
 			return 1;
 		}
 		c->c2 = c2;
 		c2->c2 = c;
 		r_core_parse_radare2rc (c);
-		if (arch) {
-			r_config_set (c->config, "asm.arch", arch);
-			r_config_set (c2->config, "asm.arch", arch);
+		if (ro.arch) {
+			r_config_set (c->config, "asm.arch", ro.arch);
+			r_config_set (c2->config, "asm.arch", ro.arch);
 		}
-		if (bits) {
-			r_config_set_i (c->config, "asm.bits", bits);
-			r_config_set_i (c2->config, "asm.bits", bits);
+		if (ro.bits) {
+			r_config_set_i (c->config, "asm.bits", ro.bits);
+			r_config_set_i (c2->config, "asm.bits", ro.bits);
 		}
 		if (columnSort) {
 			r_config_set (c->config, "diff.sort", columnSort);
 			r_config_set (c2->config, "diff.sort", columnSort);
 		}
-		r_config_set_i (c->config, "diff.bare", showbare);
-		r_config_set_i (c2->config, "diff.bare", showbare);
-		r_anal_diff_setup_i (c->anal, diffops, threshold, threshold);
-		r_anal_diff_setup_i (c2->anal, diffops, threshold, threshold);
-		if (pdc) {
+		r_config_set_i (c->config, "diff.bare", ro.showbare);
+		r_config_set_i (c2->config, "diff.bare", ro.showbare);
+		r_anal_diff_setup_i (c->anal, ro.diffops, ro.threshold, ro.threshold);
+		r_anal_diff_setup_i (c2->anal, ro.diffops, ro.threshold, ro.threshold);
+		if (ro.pdc) {
 			if (!addr) {
 				//addr = "entry0";
 				addr = "main";
@@ -1099,18 +1141,18 @@ R_API int r_main_radiff2(int argc, char **argv) {
 
 			ut64 addra = r_num_math (c->num, addr);
 			bufa = (ut8 *) r_core_cmd_strf (c, "af;pdc @ 0x%08"PFMT64x, addra);
-			sza = strlen ((const char *) bufa);
+			sza = (ut64)strlen ((const char *) bufa);
 
 			ut64 addrb = r_num_math (c2->num, addr);
 			bufb = (ut8 *) r_core_cmd_strf (c2, "af;pdc @ 0x%08"PFMT64x, addrb);
-			szb = strlen ((const char *) bufb);
-			mode = MODE_DIFF;
-		} else if (mode == MODE_GRAPH) {
+			szb = (ut64)strlen ((const char *) bufb);
+			ro.mode = MODE_DIFF;
+		} else if (ro.mode == MODE_GRAPH) {
 			int depth = r_config_get_i (c->config, "anal.depth");
 			if (depth < 1) {
 				depth = 64;
 			}
-			char *words = strdup (addr? addr: "0");
+			char *words = strdup (r_str_get_fail (addr, "0"));
 			char *second = strchr (words, ',');
 			if (second) {
 				*second++ = 0;
@@ -1120,18 +1162,18 @@ R_API int r_main_radiff2(int argc, char **argv) {
 				r_core_anal_fcn (c2, r_num_math (c2->num, second),
 					UT64_MAX, R_ANAL_REF_TYPE_NULL, depth);
 				r_core_gdiff (c, c2);
-				__print_diff_graph (c, off, gmode);
+				__print_diff_graph (c, off, ro.gmode);
 			} else {
 				r_core_anal_fcn (c, r_num_math (c->num, words),
 					UT64_MAX, R_ANAL_REF_TYPE_NULL, depth);
 				r_core_anal_fcn (c2, r_num_math (c2->num, words),
 					UT64_MAX, R_ANAL_REF_TYPE_NULL, depth);
 				r_core_gdiff (c, c2);
-				__print_diff_graph (c, r_num_math (c->num, addr), gmode);
+				__print_diff_graph (c, r_num_math (c->num, addr), ro.gmode);
 			}
 			free (words);
-		} else if (mode == MODE_CODE) {
-			if (zignatures) {
+		} else if (ro.mode == MODE_CODE) {
+			if (ro.zignatures) {
 				r_core_cmd0 (c, "z~?");
 				r_core_cmd0 (c2, "z~?");
 				r_core_zdiff (c, c2);
@@ -1139,124 +1181,137 @@ R_API int r_main_radiff2(int argc, char **argv) {
 				r_core_gdiff (c, c2);
 				r_core_diff_show (c, c2);
 			}
-		} else if (mode == MODE_DIFF_IMPORTS) {
-			bufa = get_imports (c, &sza);
-			bufb = get_imports (c2, &szb);
-		} else if (mode == MODE_DIFF_STRS) {
-			bufa = get_strings (c, &sza);
-			bufb = get_strings (c2, &szb);
+		} else if (ro.mode == MODE_DIFF_IMPORTS) {
+			int sz;
+			bufa = get_imports (c, &sz);
+			sza = sz;
+			bufb = get_imports (c2, &sz);
+			szb = sz;
+		} else if (ro.mode == MODE_DIFF_STRS) {
+			int sz;
+			bufa = get_strings (c, &sz);
+			sza = sz;
+			bufb = get_strings (c2, &sz);
+			szb = sz;
 		}
-		if (mode == MODE_CODE || mode == MODE_GRAPH) {
+		if (ro.mode == MODE_CODE || ro.mode == MODE_GRAPH) {
 			r_cons_flush ();
 		}
 		r_core_free (c);
 		r_core_free (c2);
 
-		if (mode == MODE_CODE || mode == MODE_GRAPH) {
+		if (ro.mode == MODE_CODE || ro.mode == MODE_GRAPH) {
 			return 0;
 		}
 		break;
-	default:
-		bufa = slurp (&c, file, &sza);
+	default: {
+		size_t fsz = 0;
+		bufa = slurp (&ro, &c, ro.file, &fsz);
+		sza = fsz;
 		if (!bufa) {
-			eprintf ("radiff2: Cannot open %s\n", r_str_get (file));
+			eprintf ("radiff2: Cannot open %s\n", r_str_getf (ro.file));
 			return 1;
 		}
-		bufb = slurp (&c, file2, &szb);
+		bufb = slurp (&ro, &c, ro.file2, &fsz);
+		szb = fsz;
 		if (!bufb) {
-			eprintf ("radiff2: Cannot open: %s\n", r_str_get (file2));
+			eprintf ("radiff2: Cannot open: %s\n", r_str_getf (ro.file2));
 			free (bufa);
 			return 1;
 		}
 		if (sza != szb) {
-			eprintf ("File size differs %d vs %d\n", sza, szb);
+			eprintf ("File size differs %"PFMT64u" vs %"PFMT64u"\n", (ut64)sza, (ut64)szb);
 		}
 		break;
 	}
+	}
 
 	// initialize RCons
 	(void)r_cons_new ();
 
-	switch (mode) {
+	switch (ro.mode) {
 	case MODE_COLSII:
-		if (!c && !r_list_empty (evals)) {
-			c = opencore (NULL);
+		if (!c && !r_list_empty (ro.evals)) {
+			c = opencore (&ro, NULL);
 		}
-		dump_cols_hexii (bufa, sza, bufb, szb, (r_cons_get_size (NULL) > 112)? 16: 8);
+		dump_cols_hexii (bufa, (int)sza, bufb, (int)szb, (r_cons_get_size (NULL) > 112)? 16: 8);
 		break;
 	case MODE_COLS:
-		if (!c && !r_list_empty (evals)) {
-			c = opencore (NULL);
+		if (!c && !r_list_empty (ro.evals)) {
+			c = opencore (&ro, NULL);
 		}
-		dump_cols (bufa, sza, bufb, szb, (r_cons_get_size (NULL) > 112)? 16: 8);
+		dump_cols (bufa, (int)sza, bufb, (int)szb, (r_cons_get_size (NULL) > 112)? 16: 8);
 		break;
 	case MODE_DIFF:
 	case MODE_DIFF_STRS:
 	case MODE_DIFF_IMPORTS:
 		d = r_diff_new ();
 		r_diff_set_delta (d, delta);
-		if (diffmode == 'j') {
-			printf ("{\"files\":[{\"filename\":\"%s\", \"size\":%d, \"sha256\":\"", file, sza);
-			handle_sha256 (bufa, sza);
-			printf ("\"},\n{\"filename\":\"%s\", \"size\":%d, \"sha256\":\"", file2, szb);
-			handle_sha256 (bufb, szb);
-			printf ("\"}],\n");
-			printf ("\"changes\":[");
+		if (ro.diffmode == 'j') {
+			pj_o (ro.pj);
+			pj_ka (ro.pj, "files");
+			fileobj (&ro, ro.file, bufa, sza);
+			fileobj (&ro, ro.file2, bufb, szb);
+			pj_end (ro.pj);
+			pj_ka (ro.pj, "changes");
 		}
-		if (diffmode == 'B') {
-			write (1, "\xd1\xff\xd1\xff", 4);
-			write (1, "\x04", 1);
+		if (ro.diffmode == 'B') {
+			(void) write (1, "\xd1\xff\xd1\xff\x04", 5);
 		}
-		if (diffmode == 'U') {
-			char * res = r_diff_buffers_unified (d, bufa, sza, bufb, szb);
-			printf ("%s", res);
-			free (res);
-		} else if (diffmode == 'B') {
-			r_diff_set_callback (d, &bcb, 0);
-			r_diff_buffers (d, bufa, sza, bufb, szb);
-			write (1, "\x00", 1);
+		if (ro.diffmode == 'U') {
+			char *res = r_diff_buffers_unified (d, bufa, (int)sza, bufb, (int)szb);
+			if (res) {
+				printf ("%s", res);
+				free (res);
+			}
+		} else if (ro.diffmode == 'B') {
+			r_diff_set_callback (d, &bcb, &ro);
+			r_diff_buffers (d, bufa, (ut32)sza, bufb, (ut32)szb);
+			(void) write (1, "\x00", 1);
 		} else {
-			r_diff_set_callback (d, &cb, 0); // (void *)(size_t)diffmode);
-			r_diff_buffers (d, bufa, sza, bufb, szb);
+			r_diff_set_callback (d, &cb, &ro);
+			r_diff_buffers (d, bufa, (ut32)sza, bufb, (ut32)szb);
 		}
-		if (diffmode == 'j') {
-			printf ("]\n");
+		if (ro.diffmode == 'j') {
+			pj_end (ro.pj);
 		}
 		r_diff_free (d);
 		break;
-	case MODE_DIST:
 	case MODE_DIST_MYERS:
-	case MODE_DIST_LEVENSTEIN:
+	case MODE_DIST_LEVENSHTEIN:
 		{
 			RDiff *d = r_diff_new ();
 			if (d) {
-				d->verbose = verbose;
-				if (mode == MODE_DIST_LEVENSTEIN) {
-					d->type = 'l';
-				} else if (mode == MODE_DIST_MYERS) {
+				d->verbose = ro.verbose;
+				if (ro.mode == MODE_DIST_MYERS) {
 					d->type = 'm';
 				} else {
-					d->type = 0;
+					d->type = 'l';
 				}
-				r_diff_buffers_distance (d, bufa, sza, bufb, szb, &count, &sim);
+				r_diff_buffers_distance (d, bufa, (ut32)sza, bufb, (ut32)szb, &ro.count, &sim);
 				r_diff_free (d);
 			}
 		}
 		printf ("similarity: %.3f\n", sim);
-		printf ("distance: %d\n", count);
+		printf ("distance: %d\n", ro.count);
 		break;
 	}
-	r_cons_free ();
 
-	if (diffmode == 'j' && showcount) {
-		printf (",\"count\":%d}\n", count);
-	} else if (showcount && diffmode != 'j') {
-		printf ("%d\n", count);
-	} else if (!showcount && diffmode == 'j') {
-		printf ("}\n");
+	if (ro.diffmode == 'j' && ro.showcount) {
+		pj_kd (ro.pj, "count", ro.count);
+	} else if (ro.showcount && ro.diffmode != 'j') {
+		printf ("%d\n", ro.count);
+	}
+	if (ro.pj) {
+		pj_end (ro.pj);
+		char *s = pj_drain (ro.pj);
+		printf ("%s\n", s);
+		free (s);
+		ro.pj = NULL;
 	}
 	free (bufa);
 	free (bufb);
+	radiff_options_fini (&ro);
 
 	return 0;
 }
