@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2011-2019 - pancake */
+/* radare - LGPL - Copyright 2011-2021 - pancake */
 
 #include <r_egg.h>
 #include <r_bin.h>
@@ -6,10 +6,77 @@
 #include <r_util/r_print.h>
 #include <r_util.h>
 
+typedef struct {
+	RLib *l;
+	REgg *e;
+	// TODO flags
+	// bool oneliner;
+	// bool coutput;
+	// bool json;
+	// bool quiet;
+} REggState;
+
+static void __load_plugins(REggState *es);
+
+static REggState *__es_new(void) {
+	REggState *es = R_NEW0 (REggState);
+	if (es) {
+		es->l = r_lib_new (NULL, NULL);
+		es->e = r_egg_new ();
+		__load_plugins (es);
+	}
+	return es;
+}
+
+static void __es_free(REggState *es) {
+	r_egg_free (es->e);
+	r_lib_free (es->l);
+	free (es);
+}
+
+/* egg callback */
+static int __lib_egg_cb(RLibPlugin *pl, void *user, void *data) {
+	REggPlugin *hand = (REggPlugin *)data;
+	REggState *es = (REggState *)user;
+	r_egg_add (es->e, hand);
+	return true;
+}
+
+static void __load_plugins(REggState *es) {
+	if (r_sys_getenv_asbool ("RAGG2_NOPLUGINS")) {
+		return;
+	}
+
+	r_lib_add_handler (es->l, R_LIB_TYPE_EGG, "egg plugins", &__lib_egg_cb, NULL, es);
+
+	char *path = r_sys_getenv (R_LIB_ENV);
+	if (!R_STR_ISEMPTY (path)) {
+		r_lib_opendir (es->l, path);
+	}
+
+	// load plugins from the home directory
+	char *homeplugindir = r_str_home (R2_HOME_PLUGINS);
+	r_lib_opendir (es->l, homeplugindir);
+	free (homeplugindir);
+
+	// load plugins from the system directory
+	char *plugindir = r_str_r2_prefix (R2_PLUGINS);
+	char *extrasdir = r_str_r2_prefix (R2_EXTRAS);
+	char *bindingsdir = r_str_r2_prefix (R2_BINDINGS);
+	r_lib_opendir (es->l, plugindir);
+	r_lib_opendir (es->l, extrasdir);
+	r_lib_opendir (es->l, bindingsdir);
+	free (plugindir);
+	free (extrasdir);
+	free (bindingsdir);
+
+	free (path);
+}
+
 static int usage(int v) {
 	printf ("Usage: ragg2 [-FOLsrxhvz] [-a arch] [-b bits] [-k os] [-o file] [-I path]\n"
-		"             [-i sc] [-e enc] [-B hex] [-c k=v] [-C file] [-p pad] [-q off]\n"
-		"             [-S string] [-f fmt] [-nN dword] [-dDw off:hex] file|f.asm|-\n");
+		"             [-i sc] [-E enc] [-B hex] [-c k=v] [-C file] [-p pad] [-q off]\n"
+		"             [-S string] [-f fmt] [-nN dword] [-dDw off:hex] [-e expr] file|f.asm|-\n");
 	if (v) {
 		printf (
 			" -a [arch]       select architecture (x86, mips, arm)\n"
@@ -19,7 +86,8 @@ static int usage(int v) {
 			" -C [file]       append contents of file\n"
 			" -d [off:dword]  patch dword (4 bytes) at given offset\n"
 			" -D [off:qword]  patch qword (8 bytes) at given offset\n"
-			" -e [encoder]    use specific encoder. see -L\n"
+			" -e [egg-expr]   take egg program from string instead of file\n"
+			" -E [encoder]    use specific encoder. see -L\n"
 			" -f [format]     output format (raw, c, pe, elf, mach0, python, javascript)\n"
 			" -F              output native format (osx=mach0, linux=elf, ..)\n"
 			" -h              show this help\n"
@@ -94,7 +162,7 @@ static int openfile(const char *f, int x) {
 			return -1;
 		}
 	}
-#if __UNIX__
+#if __UNIX__ && !__wasi__
 	if (x) {
 		fchmod (fd, 0755);
 	}
@@ -108,12 +176,14 @@ static int openfile(const char *f, int x) {
 		eprintf ("Could not resize\n");
 	}
 	close (1);
+#if !__wasi__
 	dup2 (fd, 1);
+#endif
 	return fd;
 }
 #define ISEXEC (fmt!='r')
 
-R_API int r_main_ragg2(int argc, char **argv) {
+R_API int r_main_ragg2(int argc, const char **argv) {
 	const char *file = NULL;
 	const char *padding = NULL;
 	const char *pattern = NULL;
@@ -122,60 +192,77 @@ R_API int r_main_ragg2(int argc, char **argv) {
 	const char *contents = NULL;
 	const char *arch = R_SYS_ARCH;
 	const char *os = R_EGG_OS_NAME;
-	char *format = "raw";
+	const char *format = "raw";
 	bool show_execute = false;
 	bool show_execute_rop = false;
-	int show_hex = 1;
-	int show_asm = 0;
-	int show_raw = 0;
+	bool show_hex = true;
+	bool show_asm = false;
+	bool show_raw = false;
 	int append = 0;
 	int show_str = 0;
 	ut64 get_offset  = 0;
-	char *shellcode = NULL;
-	char *encoder = NULL;
+	const char *shellcode = NULL;
+	const char *encoder = NULL;
+	const char *eggprg = NULL;
 	char *sequence = NULL;
 	int bits = (R_SYS_BITS & R_SYS_BITS_64)? 64: 32;
 	int fmt = 0;
 	const char *ofile = NULL;
 	int ofileauto = 0;
 	RBuffer *b;
-	int c, i;
-	REgg *egg = r_egg_new ();
+	int c, i, fd = -1;
 
-	while ((c = r_getopt (argc, argv, "n:N:he:a:b:f:o:sxXrk:FOI:Li:c:p:P:B:C:vd:D:w:zq:S:")) != -1) {
+	if (argc < 2) {
+		return usage (1);
+	}
+
+	REggState *es = __es_new ();
+
+	RGetopt opt;
+	r_getopt_init (&opt, argc, argv, "a:b:B:c:C:d:D:e:E:f:Fhi:I:k:Ln:N:o:Op:P:q:rsS:vw:xX:z");
+	while ((c = r_getopt_next (&opt)) != -1) {
 		switch (c) {
 		case 'a':
-			arch = r_optarg;
+			arch = opt.arg;
 			if (!strcmp (arch, "trace")) {
-				show_asm = 1;
-				show_hex = 0;
+				show_asm = true;
+				show_hex = false;
 			}
 			break;
 		case 'e':
-			encoder = r_optarg;
+			eggprg = opt.arg;
+			break;
+		case 'E':
+			encoder = opt.arg;
 			break;
 		case 'b':
-			bits = atoi (r_optarg);
+			bits = atoi (opt.arg);
 			break;
 		case 'B':
-			bytes = r_str_append (bytes, r_optarg);
+			bytes = r_str_append (bytes, opt.arg);
 			break;
 		case 'C':
-			contents = r_optarg;
+			if (R_STR_ISEMPTY (opt.arg)) {
+				eprintf ("Cannot open empty contents path\n");
+				free (sequence);
+				__es_free (es);
+				return 1;
+			}
+			contents = opt.arg;
 			break;
 		case 'w':
 			{
-			char *arg = strdup (r_optarg);
+			char *arg = strdup (opt.arg);
 			char *p = strchr (arg, ':');
 			if (p) {
 				int len, off;
 				ut8 *b;
 				*p++ = 0;
 				off = r_num_math (NULL, arg);
-				b = malloc (strlen (r_optarg) + 1);
+				b = malloc (strlen (opt.arg) + 1);
 				len = r_hex_str2bin (p, b);
 				if (len > 0) {
-					r_egg_patch (egg, off, (const ut8*)b, len);
+					r_egg_patch (es->e, off, (const ut8 *)b, len);
 				} else {
 					eprintf ("Invalid hexstr for -w\n");
 				}
@@ -188,29 +275,29 @@ R_API int r_main_ragg2(int argc, char **argv) {
 			break;
 		case 'n':
 			{
-			ut32 n = r_num_math (NULL, r_optarg);
+			ut32 n = r_num_math (NULL, opt.arg);
 			append = 1;
-			r_egg_patch (egg, -1, (const ut8*)&n, 4);
+			r_egg_patch (es->e, -1, (const ut8 *)&n, 4);
 			}
 			break;
 		case 'N':
 			{
-			ut64 n = r_num_math (NULL, r_optarg);
-			r_egg_patch (egg, -1, (const ut8*)&n, 8);
+			ut64 n = r_num_math (NULL, opt.arg);
+			r_egg_patch (es->e, -1, (const ut8 *)&n, 8);
 			append = 1;
 			}
 			break;
 		case 'd':
 			{
 			ut32 off, n;
-			char *p = strchr (r_optarg, ':');
+			char *p = strchr (opt.arg, ':');
 			if (p) {
 				*p = 0;
-				off = r_num_math (NULL, r_optarg);
+				off = r_num_math (NULL, opt.arg);
 				n = r_num_math (NULL, p + 1);
 				*p = ':';
 				// TODO: honor endianness here
-				r_egg_patch (egg, off, (const ut8*)&n, 4);
+				r_egg_patch (es->e, off, (const ut8 *)&n, 4);
 			} else {
 				eprintf ("Missing colon in -d\n");
 			}
@@ -218,46 +305,52 @@ R_API int r_main_ragg2(int argc, char **argv) {
 			break;
 		case 'D':
 			{
-			char *p = strchr (r_optarg, ':');
+			char *p = strchr (opt.arg, ':');
 			if (p) {
-				ut64 n, off = r_num_math (NULL, r_optarg);
+				ut64 n, off = r_num_math (NULL, opt.arg);
 				n = r_num_math (NULL, p + 1);
 				// TODO: honor endianness here
-				r_egg_patch (egg, off, (const ut8*)&n, 8);
+				r_egg_patch (es->e, off, (const ut8 *)&n, 8);
 			} else {
 				eprintf ("Missing colon in -d\n");
 			}
 			}
 			break;
 		case 'S':
-			str = r_optarg;
+			str = opt.arg;
 			break;
 		case 'o':
-			ofile = r_optarg;
+			ofile = opt.arg;
 			break;
 		case 'O':
 			ofileauto = 1;
 			break;
 		case 'I':
-			r_egg_lang_include_path (egg, r_optarg);
+			if (R_STR_ISEMPTY (opt.arg)) {
+				eprintf ("Cannot open empty include path\n");
+				free (sequence);
+				__es_free (es);
+				return 1;
+			}
+			r_egg_lang_include_path (es->e, opt.arg);
 			break;
 		case 'i':
-			 shellcode = r_optarg;
-			 break;
+			shellcode = opt.arg;
+			break;
 		case 'p':
-			padding = r_optarg;
+			padding = opt.arg;
 			break;
 		case 'P':
-			pattern = r_optarg;
+			pattern = opt.arg;
 			break;
 		case 'c':
 			{
-			char *p = strchr (r_optarg, '=');
+			char *p = strchr (opt.arg, '=');
 			if (p) {
 				*p++ = 0;
-				r_egg_option_set (egg, r_optarg, p);
+				r_egg_option_set (es->e, opt.arg, p);
 			} else {
-				r_egg_option_set (egg, r_optarg, "true");
+				r_egg_option_set (es->e, opt.arg, "true");
 			}
 			}
 			break;
@@ -269,21 +362,21 @@ R_API int r_main_ragg2(int argc, char **argv) {
 #else
 			format = "elf";
 #endif
-			show_asm = 0;
+			show_asm = false;
 			break;
 		case 'f':
-			format = r_optarg;
-			show_asm = 0;
+			format = opt.arg;
+			show_asm = false;
 			break;
 		case 's':
-			show_asm = 1;
-			show_hex = 0;
+			show_asm = true;
+			show_hex = false;
 			break;
 		case 'k':
-			os = r_optarg;
+			os = opt.arg;
 			break;
 		case 'r':
-			show_raw = 1;
+			show_raw = true;
 			break;
 		case 'x':
 			// execute
@@ -295,38 +388,39 @@ R_API int r_main_ragg2(int argc, char **argv) {
 			show_execute_rop = 1;
 			break;
 		case 'L':
-			list (egg);
-			r_egg_free (egg);
+			list (es->e);
+			__es_free (es);
 			free (sequence);
 			return 0;
 		case 'h':
-			r_egg_free (egg);
+			__es_free (es);
 			free (sequence);
 			return usage (1);
 		case 'v':
 			free (sequence);
-			r_egg_free (egg);
+			__es_free (es);
 			return r_main_version_print ("ragg2");
 		case 'z':
 			show_str = 1;
 			break;
 		case 'q':
 			get_offset = 1;
-			sequence = strdup (r_optarg);
+			sequence = strdup (opt.arg);
 			break;
 		default:
 			free (sequence);
-			r_egg_free (egg);
+			__es_free (es);
 			return 1;
 		}
 	}
 
-	if (r_optind == argc && !shellcode && !bytes && !contents && !encoder && !padding && !pattern && !append && !get_offset && !str) {
+	if (opt.ind == argc && !eggprg && !shellcode && !bytes && !contents && !encoder && !padding && !pattern && !append && !get_offset && !str) {
 		free (sequence);
-		r_egg_free (egg);
+		__es_free (es);
 		return usage (0);
-	} else {
-		file = argv[r_optind];
+	}
+	if (opt.ind != argc) {
+		file = argv[opt.ind];
 	}
 
 	if (bits == 64) {
@@ -342,7 +436,7 @@ R_API int r_main_ragg2(int argc, char **argv) {
 		if (strncmp (sequence, "0x", 2)) {
 			eprintf ("Need hex value with `0x' prefix e.g. 0x41414142\n");
 			free (sequence);
-			r_egg_free (egg);
+			__es_free (es);
 			return 1;
 		}
 
@@ -350,13 +444,17 @@ R_API int r_main_ragg2(int argc, char **argv) {
 		printf ("Little endian: %d\n", r_debruijn_offset (get_offset, false));
 		printf ("Big endian: %d\n", r_debruijn_offset (get_offset, true));
 		free (sequence);
-		r_egg_free (egg);
+		__es_free (es);
 		return 0;
 	}
 
 	// initialize egg
-	r_egg_setup (egg, arch, bits, 0, os);
+	r_egg_setup (es->e, arch, bits, 0, os);
 	if (file) {
+		if (R_STR_ISEMPTY (file)) {
+			eprintf ("Cannot open empty path\n");
+			goto fail;
+		}
 		if (!strcmp (file, "-")) {
 			char buf[1024];
 			for (;;) {
@@ -366,7 +464,7 @@ R_API int r_main_ragg2(int argc, char **argv) {
 				if (feof (stdin)) {
 					break;
 				}
-				r_egg_load (egg, buf, 0);
+				r_egg_load (es->e, buf, 0);
 			}
 		} else if (strstr (file, ".c")) {
 			char *fileSanitized = strdup (file);
@@ -378,10 +476,10 @@ R_API int r_main_ragg2(int argc, char **argv) {
 				goto fail;
 			}
 
-			int l;
+			size_t l;
 			char *buf = r_file_slurp (textFile, &l);
 			if (buf && l > 0) {
-				r_egg_raw (egg, (const ut8*)buf, l);
+				r_egg_raw (es->e, (const ut8 *)buf, (int)l);
 			} else {
 				eprintf ("Error loading '%s'\n", textFile);
 			}
@@ -393,22 +491,29 @@ R_API int r_main_ragg2(int argc, char **argv) {
 		} else {
 			if (strstr (file, ".s") || strstr (file, ".asm")) {
 				fmt = 'a';
+			} else if (strstr (file, ".bin") || strstr (file, ".raw")) {
+				fmt = 'r';
 			} else {
 				fmt = 0;
 			}
-			if (!r_egg_include (egg, file, fmt)) {
+			if (!r_egg_include (es->e, file, fmt)) {
 				eprintf ("Cannot open '%s'\n", file);
 				goto fail;
 			}
 		}
+	} else {
+		if (eggprg && !r_egg_include_str (es->e, eggprg)) {
+			eprintf ("Cannot parse egg program.\n");
+			goto fail;
+		}
 	}
 
 	// compile source code to assembly
-	if (!r_egg_compile (egg)) {
+	if (!r_egg_compile (es->e)) {
 		if (!fmt) {
 			eprintf ("r_egg_compile: fail\n");
 			free (sequence);
-			r_egg_free (egg);
+			__es_free (es);
 			return 1;
 		}
 	}
@@ -417,16 +522,16 @@ R_API int r_main_ragg2(int argc, char **argv) {
 	if (str) {
 		int l = strlen (str);
 		if (l > 0) {
-			r_egg_raw (egg, (const ut8*)str, l);
+			r_egg_raw (es->e, (const ut8 *)str, l);
 		}
 	}
 
 	// add raw file
 	if (contents) {
-		int l;
+		size_t l;
 		char *buf = r_file_slurp (contents, &l);
 		if (buf && l > 0) {
-			r_egg_raw (egg, (const ut8*)buf, l);
+			r_egg_raw (es->e, (const ut8 *)buf, (int)l);
 		} else {
 			eprintf ("Error loading '%s'\n", contents);
 		}
@@ -435,7 +540,7 @@ R_API int r_main_ragg2(int argc, char **argv) {
 
 	// add shellcode
 	if (shellcode) {
-		if (!r_egg_shellcode (egg, shellcode)) {
+		if (!r_egg_shellcode (es->e, shellcode)) {
 			eprintf ("Unknown shellcode '%s'\n", shellcode);
 			goto fail;
 		}
@@ -446,8 +551,9 @@ R_API int r_main_ragg2(int argc, char **argv) {
 		ut8 *b = malloc (strlen (bytes) + 1);
 		int len = r_hex_str2bin (bytes, b);
 		if (len > 0) {
-			if (!r_egg_raw (egg, b, len)) {
+			if (!r_egg_raw (es->e, b, len)) {
 				eprintf ("Unknown '%s'\n", shellcode);
+				free (b);
 				goto fail;
 			}
 		} else {
@@ -460,10 +566,9 @@ R_API int r_main_ragg2(int argc, char **argv) {
 
 	/* set output (create output file if needed) */
 	if (ofileauto) {
-		int fd;
 		if (file) {
 			char *o, *q, *p = strdup (file);
-			if ( (o = strchr (p, '.')) ) {
+			if ((o = strchr (p, '.'))) {
 				while ( (q = strchr (o + 1, '.')) ) {
 					o = q;
 				}
@@ -477,24 +582,28 @@ R_API int r_main_ragg2(int argc, char **argv) {
 			fd = openfile ("a.out", ISEXEC);
 		}
 		if (fd == -1) {
-			eprintf ("cannot open file '%s'\n", r_optarg);
+			eprintf ("cannot open file '%s'\n", opt.arg);
 			goto fail;
 		}
+		close (fd);
 	}
 	if (ofile) {
-		if (openfile (ofile, ISEXEC) == -1) {
+		fd = openfile (ofile, ISEXEC);
+		if (fd == -1) {
 			eprintf ("cannot open file '%s'\n", ofile);
 			goto fail;
 		}
 	}
 
 	// assemble to binary
-	if (!r_egg_assemble (egg)) {
-		eprintf ("r_egg_assemble: invalid assembly\n");
-		goto fail;
+	if (!show_asm) {
+		if (!r_egg_assemble (es->e)) {
+			eprintf ("r_egg_assemble: invalid assembly\n");
+			goto fail;
+		}
 	}
 	if (encoder) {
-		if (!r_egg_encode (egg, encoder)) {
+		if (!r_egg_encode (es->e, encoder)) {
 			eprintf ("Invalid encoder '%s'\n", encoder);
 			goto fail;
 		}
@@ -502,40 +611,44 @@ R_API int r_main_ragg2(int argc, char **argv) {
 
 	// add padding
 	if (padding) {
-		r_egg_padding (egg, padding);
+		r_egg_padding (es->e, padding);
 	}
 
 	// add pattern
 	if (pattern) {
-		r_egg_pattern (egg, r_num_math (NULL, pattern));
+		r_egg_pattern (es->e, r_num_math (NULL, pattern));
 	}
 
 	// apply patches
-	if (!egg->bin) {
-		egg->bin = r_buf_new ();
+	if (!es->e->bin) {
+		es->e->bin = r_buf_new ();
 	}
-	if (!(b = r_egg_get_bin (egg))) {
+	if (!(b = r_egg_get_bin (es->e))) {
 		eprintf ("r_egg_get_bin: invalid egg :(\n");
 		goto fail;
 	}
-	r_egg_finalize (egg);
+	r_egg_finalize (es->e);
 
 	if (show_asm) {
-		printf ("%s\n", r_egg_get_assembly (egg));
+		printf ("%s\n", r_egg_get_assembly (es->e));
 	}
 
 	if (show_raw || show_hex || show_execute) {
 		if (show_execute) {
 			int r;
 			if (show_execute_rop) {
-				r = r_egg_run_rop (egg);
+				r = r_egg_run_rop (es->e);
 			} else {
-				r = r_egg_run (egg);
+				r = r_egg_run (es->e);
 			}
-			r_egg_free (egg);
+			r_egg_free (es->e);
+			if (fd != -1) {
+				close (fd);
+			}
+			free (sequence);
 			return r;
 		}
-		b = r_egg_get_bin (egg);
+		b = r_egg_get_bin (es->e);
 		if (show_raw) {
 			ut64 blen;
 			const ut8 *tmp = r_buf_data (b, &blen);
@@ -574,7 +687,7 @@ R_API int r_main_ragg2(int argc, char **argv) {
 				} // else show_raw is_above()
 				break;
 			case 'p': // PE
-				if (strlen(format) >= 2 && format[1] == 'y') { // Python
+				if (strlen (format) >= 2 && format[1] == 'y') { // Python
 					r_print_code (p, 0, tmp, tmpsz, 'p');
 				}
 				break;
@@ -589,11 +702,17 @@ R_API int r_main_ragg2(int argc, char **argv) {
 			r_print_free (p);
 		}
 	}
+	if (fd != -1) {
+		close (fd);
+	}
 	free (sequence);
-	r_egg_free (egg);
+	__es_free (es);
 	return 0;
 fail:
+	if (fd != -1) {
+		close (fd);
+	}
 	free (sequence);
-	r_egg_free (egg);
+	__es_free (es);
 	return 1;
 }
