@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include <stdio.h>
 #include <string.h>
@@ -42,7 +42,9 @@ static void do_hash_seed(const char *seed) {
 		return;
 	}
 	if (seed[0] == '@') {
-		s.buf = (ut8 *)r_file_slurp (seed + 1, &s.len);
+		size_t len;
+		s.buf = (ut8 *)r_file_slurp (seed + 1, &len);
+		s.len = (size_t)len;
 		return;
 	}
 	s.buf = (ut8 *) malloc (strlen (seed) + 128);
@@ -69,23 +71,30 @@ static void do_hash_seed(const char *seed) {
 	}
 }
 
-static void do_hash_hexprint(const ut8 *c, int len, int ule, int rad) {
+static void do_hash_hexprint(const ut8 *c, int len, int ule, PJ *pj, int rad) {
 	int i;
+	char *buf = malloc (len * 2 + 1);
+	if (!buf) {
+		return;
+	}
 	if (ule) {
-		for (i = len - 1; i >= 0; i--) {
-			printf ("%02x", c[i]);
+		for (i = 0; i < len; i++) {
+			snprintf (buf + i * 2, (len - i) * 2 + 1, "%02x", c[len - i - 1]);
 		}
 	} else {
 		for (i = 0; i < len; i++) {
-			printf ("%02x", c[i]);
+			snprintf (buf + i * 2, (len - i) * 2 + 1, "%02x", c[i]);
 		}
 	}
-	if (rad != 'j') {
-		printf ("\n");
+	if (rad == 'j') {
+		pj_ks (pj, "hash", buf);
+	} else {
+		printf ("%s%s", buf, rad == 'n' ? "" : "\n");
 	}
+	free (buf);
 }
 
-static void do_hash_print(RHash *ctx, ut64 hash, int dlen, int rad, int ule) {
+static void do_hash_print(RHash *ctx, ut64 hash, int dlen, PJ *pj, int rad, int ule) {
 	char *o;
 	const ut8 *c = ctx->digest;
 	const char *hname = r_hash_name (hash);
@@ -95,23 +104,30 @@ static void do_hash_print(RHash *ctx, ut64 hash, int dlen, int rad, int ule) {
 			printf ("0x%08"PFMT64x "-0x%08"PFMT64x " %s: ",
 				from, to > 0? to - 1: 0, hname);
 		}
-		if (dlen == R_HASH_SIZE_ENTROPY) {
+		if (hash & R_HASH_SSDEEP) {
+			printf ("%s\n", ctx->digest);
+		} else if (dlen == R_HASH_SIZE_ENTROPY) {
 			printf("%.8f\n", ctx->entropy);
 		} else {
-			do_hash_hexprint (c, dlen, ule, rad);
+			do_hash_hexprint (c, dlen, ule, pj, rad);
 		}
 		break;
 	case 1:
-		printf ("e file.%s=", hname);
-		do_hash_hexprint (c, dlen, ule, rad);
+		printf ("CC file %s:", hname);
+		do_hash_hexprint (c, dlen, ule, pj, rad);
 		break;
 	case 'n':
-		do_hash_hexprint (c, dlen, ule, 'j');
+		if (hash & R_HASH_SSDEEP) {
+			printf ("%s", ctx->digest);
+		} else {
+			do_hash_hexprint (c, dlen, ule, pj, rad);
+		}
 		break;
 	case 'j':
-		printf ("{\"name\":\"%s\",\"hash\":\"", hname);
-		do_hash_hexprint (c, dlen, ule, rad);
-		printf ("\"}");
+		pj_o (pj);
+		pj_ks (pj, "name", hname);
+		do_hash_hexprint (c, dlen, ule, pj, rad);
+		pj_end (pj);
 		break;
 	default:
 		o = r_print_randomart (c, dlen, from);
@@ -121,7 +137,7 @@ static void do_hash_print(RHash *ctx, ut64 hash, int dlen, int rad, int ule) {
 	}
 }
 
-static int do_hash_internal(RHash *ctx, ut64 hash, const ut8 *buf, int len, int rad, int print, int le) {
+static int do_hash_internal(RHash *ctx, ut64 hash, const ut8 *buf, int len, PJ *pj, int rad, int print, int le) {
 	if (len < 0) {
 		return 0;
 	}
@@ -132,7 +148,7 @@ static int do_hash_internal(RHash *ctx, ut64 hash, const ut8 *buf, int len, int
 	if (iterations > 0) {
 		r_hash_do_spice (ctx, hash, iterations, _s);
 	}
-	do_hash_print (ctx, hash, dlen, rad, le);
+	do_hash_print (ctx, hash, dlen, pj, rad, le);
 	return 1;
 }
 
@@ -142,7 +158,6 @@ static int do_hash(const char *file, const char *algo, RIO *io, int bsize, int r
 	ut8 *buf;
 	int ret = 0;
 	ut64 i;
-	bool first = true;
 	if (algobit == R_HASH_NONE) {
 		eprintf ("rahash2: Invalid hashing algorithm specified\n");
 		return 1;
@@ -173,11 +188,17 @@ static int do_hash(const char *file, const char *algo, RIO *io, int bsize, int r
 	if (!buf) {
 		return 1;
 	}
+	PJ *pj = NULL;
+	if (rad == 'j') {
+		pj = pj_new ();
+		if (!pj) {
+			free (buf);
+			return 1;
+		}
+		pj_a (pj);
+	}
 	ctx = r_hash_new (true, algobit);
 
-	if (rad == 'j') {
-		printf ("[");
-	}
 	if (incremental) {
 		for (i = 1; i < R_HASH_ALL; i <<= 1) {
 			if (algobit & i) {
@@ -185,15 +206,15 @@ static int do_hash(const char *file, const char *algo, RIO *io, int bsize, int r
 				int dlen = r_hash_size (hashbit);
 				r_hash_do_begin (ctx, i);
 				if (s.buf && s.prefix) {
-					do_hash_internal (ctx, hashbit, s.buf, s.len, rad, 0, ule);
+					do_hash_internal (ctx, hashbit, s.buf, s.len, pj, rad, 0, ule);
 				}
 				for (j = from; j < to; j += bsize) {
 					int len = ((j + bsize) > to)? (to - j): bsize;
 					r_io_pread_at (io, j, buf, len);
-					do_hash_internal (ctx, hashbit, buf, len, rad, 0, ule);
+					do_hash_internal (ctx, hashbit, buf, len, pj, rad, 0, ule);
 				}
 				if (s.buf && !s.prefix) {
-					do_hash_internal (ctx, hashbit, s.buf, s.len, rad, 0, ule);
+					do_hash_internal (ctx, hashbit, s.buf, s.len, pj, rad, 0, ule);
 				}
 				r_hash_do_end (ctx, i);
 				if (iterations > 0) {
@@ -202,17 +223,10 @@ static int do_hash(const char *file, const char *algo, RIO *io, int bsize, int r
 				if (!*r_hash_name (i)) {
 					continue;
 				}
-				if (rad == 'j') {
-					if (first) {
-						first = false;
-					} else {
-						printf (",");
-					}
-				}
 				if (!quiet && rad != 'j') {
 					printf ("%s: ", file);
 				}
-				do_hash_print (ctx, i, dlen, quiet? 'n': rad, ule);
+				do_hash_print (ctx, i, dlen, pj, quiet? 'n': rad, ule);
 				if (quiet == 1) {
 					printf (" %s\n", file);
 				} else {
@@ -246,16 +260,18 @@ static int do_hash(const char *file, const char *algo, RIO *io, int bsize, int r
 					if (to > fsize) {
 						to = fsize;
 					}
-					do_hash_internal (ctx, hashbit, buf, nsize, rad, 1, ule);
+					do_hash_internal (ctx, hashbit, buf, nsize, pj, rad, 1, ule);
 				}
-				do_hash_internal (ctx, hashbit, NULL, 0, rad, 1, ule);
+				do_hash_internal (ctx, hashbit, NULL, 0, pj, rad, 1, ule);
 				from = ofrom;
 				to = oto;
 			}
 		}
 	}
 	if (rad == 'j') {
-		printf ("]\n");
+		pj_end (pj);
+		printf ("%s\n", pj_string (pj));
+		pj_free (pj);
 	}
 
 	compare_hashes (ctx, compare, r_hash_size (algobit), &ret);
@@ -265,7 +281,7 @@ static int do_hash(const char *file, const char *algo, RIO *io, int bsize, int r
 }
 
 static int do_help(int line) {
-	printf ("Usage: rahash2 [-rBhLkv] [-b S] [-a A] [-c H] [-E A] [-s S] [-f O] [-t O] [file] ...\n");
+	printf ("Usage: rahash2 [-BhjkLqrv] [-b S] [-a A] [-c H] [-E A] [-s S] [-f O] [-t O] [file] ...\n");
 	if (line) {
 		return 0;
 	}
@@ -280,6 +296,7 @@ static int do_help(int line) {
 		" -f from     start hashing at given address\n"
 		" -i num      repeat hash N iterations\n"
 		" -I iv       use give initialization vector (IV) (hexa or s:string)\n"
+		" -j          output in json\n"
 		" -S seed     use given seed (hexa or s:string) use ^ to prefix (key for -E)\n"
 		"             (- will slurp the key from stdin, the @ prefix points to a file\n"
 		" -k          show hash using the openssh's randomkey algorithm\n"
@@ -293,7 +310,7 @@ static int do_help(int line) {
 	return 0;
 }
 
-static void algolist() {
+static void algolist(void) {
 	ut64 bits;
 	ut64 i;
 	for (i = 0; i < R_HASH_NBITS; i++) {
@@ -318,12 +335,12 @@ static void algolist() {
 }
 
 #define setHashString(x, y) {\
-		if (hashstr) {\
-			eprintf ("Hashstring already defined\n");\
-			return 1;\
-		}\
-		hashstr_hex = y;\
-		hashstr = x;\
+	if (hashstr) {\
+		eprintf ("Hashstring already defined\n");\
+		return 1;\
+	}\
+	hashstr_hex = y;\
+	hashstr = (char*)x;\
 }
 
 static bool is_power_of_two(const ut64 x) {
@@ -346,12 +363,13 @@ static int encrypt_or_decrypt(const char *algo, int direction, const char *hashs
 				}
 
 				r_crypto_update (cry, (const ut8 *) buf, buflen);
-				r_crypto_final (cry, NULL, 0);
 
 				int result_size = 0;
 				ut8 *result = r_crypto_get_output (cry, &result_size);
 				if (result) {
-					write (1, result, result_size);
+					if (write (1, result, result_size) != result_size) {
+						eprintf ("Warning: cannot write result\n");
+					}
 					free (result);
 				}
 			} else {
@@ -368,16 +386,21 @@ static int encrypt_or_decrypt(const char *algo, int direction, const char *hashs
 	return 1;
 }
 
-static int encrypt_or_decrypt_file(const char *algo, int direction, char *filename, const ut8 *iv, int ivlen, int mode) {
+static int encrypt_or_decrypt_file(const char *algo, int direction, const char *filename, const ut8 *iv, int ivlen, int mode) {
 	bool no_key_mode = !strcmp ("base64", algo) || !strcmp ("base91", algo) || !strcmp ("punycode", algo); // TODO: generalise this for all non key encoding/decoding.
 	if (no_key_mode || s.len > 0) {
 		RCrypto *cry = r_crypto_new ();
 		if (r_crypto_use (cry, algo)) {
 			if (r_crypto_set_key (cry, s.buf, s.len, 0, direction)) {
-				int file_size;
-				ut8 *buf = strcmp (filename, "-")
-				           ? (ut8 *) r_file_slurp (filename, &file_size)
-					   : (ut8 *) r_stdin_slurp (&file_size);
+				size_t file_size;
+				ut8 *buf;
+				if (strcmp (filename, "-") == 0) {
+					int sz;
+					buf = (ut8 *)r_stdin_slurp (&sz);
+					file_size = (size_t)sz;
+				} else {
+					buf = (ut8 *)r_file_slurp (filename, &file_size);
+				}
 				if (!buf) {
 					eprintf ("rahash2: Cannot open '%s'\n", filename);
 					return -1;
@@ -390,12 +413,11 @@ static int encrypt_or_decrypt_file(const char *algo, int direction, char *filena
 				}
 
 				r_crypto_update (cry, buf, file_size);
-				r_crypto_final (cry, NULL, 0);
 
 				int result_size = 0;
 				ut8 *result = r_crypto_get_output (cry, &result_size);
 				if (result) {
-					write (1, result, result_size);
+					(void)write (1, result, result_size);
 					free (result);
 				}
 				free (buf);
@@ -413,9 +435,10 @@ static int encrypt_or_decrypt_file(const char *algo, int direction, char *filena
 	return 1;
 }
 
-int r_main_rahash2(int argc, char **argv) {
+R_API int r_main_rahash2(int argc, const char **argv) {
 	ut64 i;
 	int ret, c, rad = 0, bsize = 0, numblocks = 0, ule = 0;
+	const char *file = NULL;
 	const char *algo = "sha256"; /* default hashing algorithm */
 	const char *seed = NULL;
 	const char *decrypt = NULL;
@@ -423,7 +446,7 @@ int r_main_rahash2(int argc, char **argv) {
 	char *hashstr = NULL;
 	ut8 *iv = NULL;
 	int ivlen = -1;
-	char *ivseed = NULL;
+	const char *ivseed = NULL;
 	const char *compareStr = NULL;
 	const char *ptype = NULL;
 	ut8 *compareBin = NULL;
@@ -434,37 +457,39 @@ int r_main_rahash2(int argc, char **argv) {
 	RHash *ctx;
 	RIO *io;
 
-	while ((c = r_getopt (argc, argv, "p:jD:rveE:a:i:I:S:s:x:b:nBhf:t:kLqc:")) != -1) {
+	RGetopt opt;
+	r_getopt_init (&opt, argc, argv, "p:jD:rveE:a:i:I:S:s:x:b:nBhf:t:kLqc:");
+	while ((c = r_getopt_next (&opt)) != -1) {
 		switch (c) {
 		case 'q': quiet++; break;
 		case 'i':
-			iterations = atoi (r_optarg);
+			iterations = atoi (opt.arg);
 			if (iterations < 0) {
 				eprintf ("error: -i argument must be positive\n");
 				return 1;
 			}
 			break;
 		case 'j': rad = 'j'; break;
-		case 'S': seed = r_optarg; break;
-		case 'I': ivseed = r_optarg; break;
+		case 'S': seed = opt.arg; break;
+		case 'I': ivseed = opt.arg; break;
 		case 'n': numblocks = 1; break;
-		case 'D': decrypt = r_optarg; break;
-		case 'E': encrypt = r_optarg; break;
+		case 'D': decrypt = opt.arg; break;
+		case 'E': encrypt = opt.arg; break;
 		case 'L': algolist (); return 0;
 		case 'e': ule = 1; break;
 		case 'r': rad = 1; break;
 		case 'k': rad = 2; break;
-		case 'p': ptype = r_optarg; break;
-		case 'a': algo = r_optarg; break;
+		case 'p': ptype = opt.arg; break;
+		case 'a': algo = opt.arg; break;
 		case 'B': incremental = false; break;
-		case 'b': bsize = (int) r_num_math (NULL, r_optarg); break;
-		case 'f': from = r_num_math (NULL, r_optarg); break;
-		case 't': to = 1 + r_num_math (NULL, r_optarg); break;
+		case 'b': bsize = (int) r_num_math (NULL, opt.arg); break;
+		case 'f': from = r_num_math (NULL, opt.arg); break;
+		case 't': to = 1 + r_num_math (NULL, opt.arg); break;
 		case 'v': return r_main_version_print ("rahash2");
 		case 'h': return do_help (0);
-		case 's': setHashString (r_optarg, 0); break;
-		case 'x': setHashString (r_optarg, 1); break;
-		case 'c': compareStr = r_optarg; break;
+		case 's': setHashString (opt.arg, 0); break;
+		case 'x': setHashString (opt.arg, 1); break;
+		case 'c': compareStr = opt.arg; break;
 		default: return do_help (0);
 		}
 	}
@@ -525,7 +550,7 @@ int r_main_rahash2(int argc, char **argv) {
 		// TODO: support p=%s (horizontal bars)
 		// TODO: list supported statistical metrics
 		// TODO: support -f and -t
-		for (i = r_optind; i < argc; i++) {
+		for (i = opt.ind; i < argc; i++) {
 			printf ("%s:\n", argv[i]);
 			r_sys_cmdf ("r2 -qfnc \"p==%s 100\" \"%s\"", ptype, argv[i]);
 		}
@@ -628,17 +653,34 @@ int r_main_rahash2(int argc, char **argv) {
 				free (iv);
 				return 1;
 			}
+			PJ *pj = NULL;
+			if (rad == 'j') {
+				pj = pj_new ();
+				if (!pj) {
+					if (str != hashstr) {
+						free (str);
+					}
+                    free (iv);
+					return 1;
+				}
+				pj_a (pj);
+			}
 			for (i = 1; i < R_HASH_ALL; i <<= 1) {
 				if (algobit & i) {
 					ut64 hashbit = i & algobit;
 					ctx = r_hash_new (true, hashbit);
 					from = 0;
 					to = strsz;
-					do_hash_internal (ctx, hashbit, (const ut8 *) str, strsz, rad, 1, ule);
+					do_hash_internal (ctx, hashbit, (const ut8 *) str, strsz, pj, rad, 1, ule);
 					compare_hashes (ctx, compareBin, r_hash_size (algobit), &ret);
 					r_hash_free (ctx);
 				}
 			}
+			if (rad == 'j') {
+				pj_end (pj);
+				printf ("%s\n", pj_string (pj));
+				pj_free (pj);
+			}
 			if (_s) {
 				if (str != hashstr) {
 					free (str);
@@ -648,7 +690,7 @@ int r_main_rahash2(int argc, char **argv) {
 			return ret;
 		}
 	}
-	if (r_optind >= argc) {
+	if (opt.ind >= argc) {
 		free (iv);
 		return do_help (1);
 	}
@@ -661,7 +703,14 @@ int r_main_rahash2(int argc, char **argv) {
 	}
 
 	io = r_io_new ();
-	for (ret = 0, i = r_optind; i < argc; i++) {
+	for (ret = 0, i = opt.ind; i < argc; i++) {
+		file = argv[i];
+
+		if (file && !*file) {
+			eprintf ("Cannot open empty path\n");
+			return 1;
+		}
+
 		if (encrypt) {// for encrytion when files are provided
 			int rt = encrypt_or_decrypt_file (encrypt, 0, argv[i], iv, ivlen, 0);
 			if (rt == -1) {
@@ -692,6 +741,7 @@ int r_main_rahash2(int argc, char **argv) {
 					r_io_pwrite_at (io, 0, buf, sz);
 				}
 				free (uri);
+				free (buf);
 			} else {
 				if (r_file_is_directory (argv[i])) {
 					eprintf ("rahash2: Cannot hash directories\n");
