@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake, nibble, maijin */
+/* radare - LGPL - Copyright 2009-2021 - pancake, nibble, maijin */
 
 #include <r_anal.h>
 #include <r_asm.h>
@@ -24,26 +24,30 @@ static void __load_plugins(RAsmState *as);
 static void __as_set_archbits(RAsmState *as) {
 	r_asm_use (as->a, R_SYS_ARCH);
 	r_anal_use (as->anal, R_SYS_ARCH);
-
 	int sysbits = (R_SYS_BITS & R_SYS_BITS_64)? 64: 32;
 	r_asm_set_bits (as->a, sysbits);
 	r_anal_set_bits (as->anal, sysbits);
 }
 
-static RAsmState *__as_new() {
+static RAsmState *__as_new(void) {
 	RAsmState *as = R_NEW0 (RAsmState);
-	if (!as) {
-		return NULL;
+	if (as) {
+		as->l = r_lib_new (NULL, NULL);
+		as->a = r_asm_new ();
+		if (as->a) {
+			as->a->num = r_num_new (NULL, NULL, NULL);
+		}
+		as->anal = r_anal_new ();
+		__load_plugins (as);
+		__as_set_archbits (as);
 	}
-	as->l = r_lib_new (NULL, NULL);
-	as->a = r_asm_new ();
-	as->anal = r_anal_new ();
-	__load_plugins (as);
-	__as_set_archbits (as);
 	return as;
 }
 
 static void __as_free(RAsmState *as) {
+	if (as->a) {
+		r_num_free (as->a->num);
+	}
 	r_asm_free (as->a);
 	r_anal_free (as->anal);
 	r_lib_free (as->l);
@@ -61,7 +65,6 @@ static char *stackop2str(int type) {
 	return strdup ("unknown");
 }
 
-// TODO: use pj
 static int showanal(RAsmState *as, RAnalOp *op, ut64 offset, ut8 *buf, int len, PJ *pj) {
 	int ret = r_anal_op (as->anal, op, offset, buf, len, R_ANAL_OP_MASK_ESIL);
 	if (ret < 1) {
@@ -173,6 +176,74 @@ static const char *has_esil(RAsmState *as, const char *name) {
 	return "__";
 }
 
+static void ranal2_list(RAsmState *as, const char *arch) {
+	char bits[32];
+	RAnalPlugin *h;
+	RListIter *iter;
+	PJ *pj = NULL;
+	if (as->json) {
+		pj = pj_new ();
+		pj_a (pj);
+	}
+	r_list_foreach (as->anal->plugins, iter, h) {
+		bits[0] = 0;
+		if (h->bits == 27) {
+			strcat (bits, "27");
+		} else if (h->bits == 0) {
+			strcat (bits, "any");
+		} else {
+			if (h->bits & 4) {
+				strcat (bits, "4 ");
+			}
+			if (h->bits & 8) {
+				strcat (bits, "8 ");
+			}
+			if (h->bits & 16) {
+				strcat (bits, "16 ");
+			}
+			if (h->bits & 32) {
+				strcat (bits, "32 ");
+			}
+			if (h->bits & 64) {
+				strcat (bits, "64 ");
+			}
+		}
+		const char *feat = "__";
+		const char *feat2 = has_esil (as, h->name);
+		if (as->quiet) {
+			printf ("%s\n", h->name);
+		} else if (as->json) {
+			pj_o (pj);
+			pj_ks (pj, "name", h->name);
+			pj_k (pj, "bits");
+			pj_a (pj);
+			pj_i (pj, 32);
+			pj_i (pj, 64);
+			pj_end (pj);
+			pj_ks (pj, "license", r_str_get_fail (h->license, "unknown"));
+			pj_ks (pj, "description", h->desc);
+			pj_ks (pj, "features", feat);
+			pj_end (pj);
+		} else {
+			printf ("%s%s  %-9s  %-11s %-7s %s",
+					feat, feat2, bits, h->name,
+					r_str_get_fail (h->license, "unknown"), h->desc);
+			if (h->author) {
+				printf (" (by %s)", h->author);
+			}
+			if (h->version) {
+				printf (" v%s", h->version);
+			}
+			printf ("\n");
+		}
+	}
+	if (as->json) {
+		pj_end (pj);
+		printf ("%s\n", pj_string (pj));
+	}
+	pj_free (pj);
+}
+
 static void rasm2_list(RAsmState *as, const char *arch) {
 	int i;
 	char bits[32];
@@ -184,7 +255,7 @@ static void rasm2_list(RAsmState *as, const char *arch) {
 		return;
 	}
 	if (as->json) {
-		pj_o (pj);
+		pj_a (pj);
 	}
 	r_list_foreach (as->a->plugins, iter, h) {
 		if (arch) {
@@ -201,7 +272,12 @@ static void rasm2_list(RAsmState *as, const char *arch) {
 			bits[0] = 0;
 			if (h->bits == 27) {
 				strcat (bits, "27");
+			} else if (h->bits == 0) {
+				strcat (bits, "any");
 			} else {
+				if (h->bits & 4) {
+					strcat (bits, "4 ");
+				}
 				if (h->bits & 8) {
 					strcat (bits, "8 ");
 				}
@@ -229,21 +305,21 @@ static void rasm2_list(RAsmState *as, const char *arch) {
 			if (as->quiet) {
 				printf ("%s\n", h->name);
 			} else if (as->json) {
-				pj_k (pj, h->name);
 				pj_o (pj);
+				pj_ks (pj, "name", h->name);
 				pj_k (pj, "bits");
 				pj_a (pj);
 				pj_i (pj, 32);
 				pj_i (pj, 64);
 				pj_end (pj);
-				pj_ks (pj, "license", h->license? h->license: "unknown");
+				pj_ks (pj, "license", r_str_get_fail (h->license, "unknown"));
 				pj_ks (pj, "description", h->desc);
 				pj_ks (pj, "features", feat);
 				pj_end (pj);
 			} else {
 				printf ("%s%s  %-9s  %-11s %-7s %s",
 					feat, feat2, bits, h->name,
-					h->license? h->license: "unknown", h->desc);
+					r_str_get_fail (h->license, "unknown"), h->desc);
 				if (h->author) {
 					printf (" (by %s)", h->author);
 				}
@@ -283,7 +359,8 @@ static int rasm_show_help(int v) {
 			" -j           output in json format\n"
 			" -k [kernel]  Select operating system (linux, windows, darwin, ..)\n"
 			" -l [len]     Input/Output length\n"
-			" -L           List Asm plugins: (a=asm, d=disasm, A=analyze, e=ESIL)\n"
+			" -L           List RAsm plugins: (a=asm, d=disasm, A=analyze, e=ESIL)\n"
+			" -LL          List RAnal plugins\n"
 			" -o,-@ [addr] Set start address for code (default 0)\n"
 			" -O [file]    Output file name (rasm2 -Bf a.asm -O a)\n"
 			" -p           Run SPP over input for assembly\n"
@@ -296,20 +373,53 @@ static int rasm_show_help(int v) {
 			" If '-l' value is greater than output length, output is padded with nops\n"
 			" If the last argument is '-' reads from stdin\n");
 		printf ("Environment:\n"
-			" RASM2_NOPLUGINS  do not load shared plugins (speedup loading)\n"
-			" RASM2_ARCH       same as rasm2 -a\n"
-			" RASM2_BITS       same as rasm2 -b\n"
-			" R_DEBUG          if defined, show error messages and crash signal\n"
+			" RASM2_NOPLUGINS   do not load shared plugins (speedup loading)\n"
+			" RASM2_ARCH        same as rasm2 -a\n"
+			" RASM2_BITS        same as rasm2 -b\n"
+			" R2_DEBUG          if defined, show error messages and crash signal\n"
+			" R2_DEBUG_ASSERT=1 lldb -- r2 to get proper backtrace of the runtime assert\n"
 			"");
 	}
 	if (v == 2) {
-		printf ("Supported Assembler directives:\n");
+		printf ("Preprocessor directives:\n");
 		r_asm_list_directives ();
+		printf ("Assembler directives:\n");
+		printf (".intel_syntax\n"
+			".att_syntax     sets e asm.syntax=att to use AT&T syntax parser\n"
+			".endian [0,1]   default endian is system endian, 0=little, 1=big\n"
+			".big_endian     call e cfg.bigendian=true, same as .endian 1\n"
+			".lil_endian     call e cfg.bigendian=false, same as .endian 0\n"
+			".asciz \"msg\" zero byte terminated string\n"
+			".string         non-null terminated string\n"
+			".ascii          same as .string\n"
+			".align          force a specific alignment when writing code\n"
+			".arm            set asm.bits=32 when asm.arch=arm\n"
+			".thumb          set asm.bits=16 when asm.arch=arm\n"
+			".arch [mips]    specify asm.arch\n"
+			".bits [32|64]   specify 8,16,32,64 e asm.bits\n"
+			".fill [count]   fill N bytes with zeroes\n"
+			".kernel [ios]   set asm.os=linux,windows,macos,...\n"
+			".cpu [name]     set asm.cpu=?\n"
+			".os [os]        same as .kernel\n"
+			".hex 102030     set bytes in linear hexpair string, no endian applied\n"
+			".int16 [num]    write int16 number honoring endian\n"
+			".int32 [num]    same for 32bit\n"
+			".int64 [num]    same for 64bit\n"
+			".size           N/A\n"
+			".section        N/A\n"
+			".byte 0x10,0x20 space or comma separated list of byte values\n"
+			".glob           N/A\n"
+			".equ K=V        define K to be replaced with V in the lines below\n"
+			".org            change the PC=$$ to make relative instructions work\n"
+			".text           tell the linker where the code starts\n"
+			".data           tell the linker where the data starts\n"
+			".incbin foo.bin include binary file\n"
+			);
 	}
 	return 0;
 }
 
-static int rasm_disasm(RAsmState *as, char *buf, ut64 offset, int len, int bits, int ascii, int bin, int hex) {
+static int rasm_disasm(RAsmState *as, ut64 addr, const char *buf, int len, int bits, int bin, int hex) {
 	RAsmCode *acode;
 	ut8 *data = NULL;
 	int ret = 0;
@@ -323,9 +433,6 @@ static int rasm_disasm(RAsmState *as, char *buf, ut64 offset, int len, int bits,
 		}
 		clen = len; // XXX
 		data = (ut8 *)buf;
-	} else if (ascii) {
-		clen = strlen (buf);
-		data = (ut8 *)buf;
 	} else {
 		clen = r_hex_str2bin (buf, NULL);
 		if ((int)clen < 1 || !(data = malloc (clen))) {
@@ -333,6 +440,7 @@ static int rasm_disasm(RAsmState *as, char *buf, ut64 offset, int len, int bits,
 			goto beach;
 		}
 		r_hex_str2bin (buf, data);
+		len = clen;
 	}
 
 	if (!len || clen <= len) {
@@ -343,7 +451,7 @@ static int rasm_disasm(RAsmState *as, char *buf, ut64 offset, int len, int bits,
 		RAnalOp aop = { 0 };
 		while (ret < len) {
 			aop.size = 0;
-			if (r_anal_op (as->anal, &aop, offset, data + ret, len - ret, R_ANAL_OP_MASK_ESIL) > 0) {
+			if (r_anal_op (as->anal, &aop, addr, data + ret, len - ret, R_ANAL_OP_MASK_ESIL) > 0) {
 				printf ("%s\n", R_STRBUF_SAFEGET (&aop.esil));
 			}
 			if (aop.size < 1) {
@@ -355,7 +463,7 @@ static int rasm_disasm(RAsmState *as, char *buf, ut64 offset, int len, int bits,
 		}
 	} else if (hex) {
 		RAsmOp op;
-		r_asm_set_pc (as->a, offset);
+		r_asm_set_pc (as->a, addr);
 		while ((len - ret) > 0) {
 			int dr = r_asm_disassemble (as->a, &op, data + ret, len - ret);
 			if (dr == -1 || op.size < 1) {
@@ -368,10 +476,10 @@ static int rasm_disasm(RAsmState *as, char *buf, ut64 offset, int len, int bits,
 				r_asm_op_get_asm (&op));
 			free (op_hex);
 			ret += op.size;
-			r_asm_set_pc (as->a, offset + ret);
+			r_asm_set_pc (as->a, addr+ ret);
 		}
 	} else {
-		r_asm_set_pc (as->a, offset);
+		r_asm_set_pc (as->a, addr);
 		if (!(acode = r_asm_mdisassemble (as->a, data, len))) {
 			goto beach;
 		}
@@ -480,17 +588,16 @@ static int __lib_anal_cb(RLibPlugin *pl, void *user, void *data) {
 }
 
 static int print_assembly_output(RAsmState *as, const char *buf, ut64 offset, ut64 len, int bits,
-                                 int bin, bool use_spp, bool rad, bool hexwords, char *arch) {
-	int ret = 0;
+                                 int bin, bool use_spp, bool rad, bool hexwords, const char *arch) {
 	if (rad) {
 		printf ("e asm.arch=%s\n", arch? arch: R_SYS_ARCH);
-		printf ("e asm.bits=%d\n", bits);
+		printf ("e asm.bits=%d\n", bits? bits: R_SYS_BITS);
 		if (offset) {
 			printf ("s 0x%"PFMT64x"\n", offset);
 		}
 		printf ("wx ");
 	}
-	ret = rasm_asm (as, (char *)buf, offset, len, as->a->bits, bin, use_spp, hexwords);
+	int ret = rasm_asm (as, (char *)buf, offset, len, as->a->bits, bin, use_spp, hexwords);
 	if (rad) {
 		printf ("f entry = $$\n");
 		printf ("f label.main = $$ + 1\n");
@@ -533,16 +640,22 @@ static void __load_plugins(RAsmState *as) {
 	free (path);
 }
 
-R_API int r_main_rasm2(int argc, char *argv[]) {
+R_API int r_main_rasm2(int argc, const char *argv[]) {
 	const char *env_arch = r_sys_getenv ("RASM2_ARCH");
 	const char *env_bits = r_sys_getenv ("RASM2_BITS");
-	char *arch = NULL, *file = NULL, *filters = NULL, *kernel = NULL, *cpu = NULL;
+	const char *arch = R_SYS_ARCH;
+	const char *cpu = NULL;
+	const char *kernel = NULL;
+	const char *filters = NULL;
+	const char *file = NULL;
+	bool list_plugins = false;
+	bool list_anal_plugins = false;
 	bool isbig = false;
 	bool rad = false;
 	bool use_spp = false;
 	bool hexwords = false;
 	ut64 offset = 0;
-	int fd = -1, dis = 0, ascii = 0, bin = 0, ret = 0, bits = 32, c, whatsop = 0;
+	int fd = -1, dis = 0, bin = 0, ret = 0, bits = 32, c, whatsop = 0;
 	int help = 0;
 	ut64 len = 0, idx = 0, skip = 0;
 	bool analinfo = false;
@@ -565,22 +678,24 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 		free (r2bits);
 	}
 
-	while ((c = r_getopt (argc, argv, "a:Ab:Bc:CdDeEf:F:hi:jk:l:L@:o:O:pqrs:vwx")) != -1) {
+	RGetopt opt;
+	r_getopt_init (&opt, argc, argv, "a:Ab:Bc:CdDeEf:F:hi:jk:l:L@:o:O:pqrs:vwx");
+	while ((c = r_getopt_next (&opt)) != -1) {
 		switch (c) {
 		case 'a':
-			arch = r_optarg;
+			arch = opt.arg;
 			break;
 		case 'A':
 			analinfo = true;
 			break;
 		case 'b':
-			bits = r_num_math (NULL, r_optarg);
+			bits = r_num_math (NULL, opt.arg);
 			break;
 		case 'B':
 			bin = 1;
 			break;
 		case 'c':
-			cpu = r_optarg;
+			cpu = opt.arg;
 			break;
 		case 'C':
 			as->coutput = true;
@@ -598,38 +713,43 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 			dis = 3;
 			break;
 		case 'f':
-			file = r_optarg;
+			file = opt.arg;
 			break;
 		case 'F':
-			filters = r_optarg;
+			filters = opt.arg;
 			break;
 		case 'h':
 			help++;
 		case 'i':
-			skip = r_num_math (NULL, r_optarg);
+			skip = r_num_math (NULL, opt.arg);
 			break;
 		case 'j':
 			as->json = true;
 			break;
 		case 'k':
-			kernel = r_optarg;
+			kernel = opt.arg;
 			break;
 		case 'l':
-			len = r_num_math (NULL, r_optarg);
+			len = r_num_math (NULL, opt.arg);
 			break;
 		case 'L':
-			rasm2_list (as, argv[r_optind]);
-			ret = 1;
-			goto beach;
+			if (list_plugins) {
+				list_anal_plugins = true;
+			} else {
+				list_plugins = true;
+			}
+			break;
 		case '@':
 		case 'o':
-			offset = r_num_math (NULL, r_optarg);
+			offset = r_num_math (NULL, opt.arg);
 			break;
 		case 'O':
-			fd = open (r_optarg, O_TRUNC | O_RDWR | O_CREAT, 0644);
+			fd = open (opt.arg, O_TRUNC | O_RDWR | O_CREAT, 0644);
+#ifndef __wasi__
 			if (fd != -1) {
 				dup2 (fd, 1);
 			}
+#endif
 			break;
 		case 'p':
 			use_spp = true;
@@ -641,12 +761,12 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 			rad = true;
 			break;
 		case 's':
-			if (*r_optarg == '?') {
+			if (*opt.arg == '?') {
 				printf ("att\nintel\nmasm\njz\nregnum\n");
 				__as_free (as);
 				return 0;
 			} else {
-				int syntax = r_asm_syntax_from_string (r_optarg);
+				int syntax = r_asm_syntax_from_string (opt.arg);
 				if (syntax == -1) {
 					__as_free (as);
 					return 1;
@@ -677,6 +797,16 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 		ret = rasm_show_help (help > 1? 2: 0);
 		goto beach;
 	}
+	if (list_anal_plugins) {
+		ranal2_list (as, opt.argv[opt.ind]);
+		ret = 1;
+		goto beach;
+	}
+	if (list_plugins) {
+		rasm2_list (as, opt.argv[opt.ind]);
+		ret = 1;
+		goto beach;
+	}
 
 	if (arch) {
 		if (!r_asm_use (as->a, arch)) {
@@ -685,21 +815,20 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 			goto beach;
 		}
 		r_anal_use (as->anal, arch);
-		if (!strcmp (arch, "bf")) {
-			ascii = 1;
-		}
 	} else if (env_arch) {
 		if (!r_asm_use (as->a, env_arch)) {
 			eprintf ("rasm2: Unknown asm plugin '%s'\n", env_arch);
 			ret = 0;
 			goto beach;
 		}
-	} else if (!r_asm_use (as->a, "x86")) {
+	} else if (!r_asm_use (as->a, R_SYS_ARCH)) {
 		eprintf ("rasm2: Cannot find asm.x86 plugin\n");
 		ret = 0;
 		goto beach;
 	}
-	r_asm_set_cpu (as->a, cpu);
+	if (cpu) {
+		r_asm_set_cpu (as->a, cpu);
+	}
 	r_asm_set_bits (as->a, (env_bits && *env_bits)? atoi (env_bits): bits);
 	r_anal_set_bits (as->anal, (env_bits && *env_bits)? atoi (env_bits): bits);
 	as->a->syscall = r_syscall_new ();
@@ -712,7 +841,7 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 		r_anal_set_big_endian (as->anal, canbebig);
 	}
 	if (whatsop) {
-		const char *s = r_asm_describe (as->a, argv[r_optind]);
+		const char *s = r_asm_describe (as->a, opt.argv[opt.ind]);
 		ret = 1;
 		if (s) {
 			printf ("%s\n", s);
@@ -725,24 +854,24 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 		if (p) {
 			*p = 0;
 			if (*filters) {
-				r_asm_filter_input (as->a, filters);
+				r_asm_sub_names_input (as->a, filters);
 			}
 			if (p[1]) {
-				r_asm_filter_output (as->a, p + 1);
+				r_asm_sub_names_output (as->a, p + 1);
 			}
 			*p = ':';
 		} else {
 			if (dis) {
-				r_asm_filter_output (as->a, filters);
+				r_asm_sub_names_output (as->a, filters);
 			} else {
-				r_asm_filter_input (as->a, filters);
+				r_asm_sub_names_input (as->a, filters);
 			}
 		}
 	}
 
 	if (file) {
 		char *content;
-		int length = 0;
+		size_t length = 0;
 		if (!strcmp (file, "-")) {
 			int sz = 0;
 			ut8 *buf = (ut8 *)r_stdin_slurp (&sz);
@@ -758,7 +887,7 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 						length -= skip;
 					}
 				}
-				ret = rasm_disasm (as, (char *)buf, offset, len, as->a->bits, ascii, bin, dis - 1);
+				ret = rasm_disasm (as, offset, (char *)buf, len, as->a->bits, bin, dis - 1);
 			} else if (analinfo) {
 				ret = show_analinfo (as, (const char *)buf, offset);
 			} else {
@@ -770,7 +899,7 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 		} else {
 			content = r_file_slurp (file, &length);
 			if (content) {
-				if (length < 0) {
+				if (length > ST32_MAX) {
 					eprintf ("rasm2: File %s is too big\n", file);
 					ret = 1;
 				} else {
@@ -785,8 +914,8 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 						}
 					}
 					if (dis) {
-						ret = rasm_disasm (as, content, offset,
-								length, as->a->bits, ascii, bin, dis - 1);
+						ret = rasm_disasm (as, offset, content,
+								length, as->a->bits, bin, dis - 1);
 					} else if (analinfo) {
 						ret = show_analinfo (as, (const char *)content, offset);
 					} else {
@@ -801,8 +930,8 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 				ret = 1;
 			}
 		}
-	} else if (argv[r_optind]) {
-		if (!strcmp (argv[r_optind], "-")) {
+	} else if (opt.argv[opt.ind]) {
+		if (!strcmp (opt.argv[opt.ind], "-")) {
 			int length;
 			do {
 				char buf[1024]; // TODO: use(implement) r_stdin_line() or so
@@ -830,7 +959,7 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 					}
 				}
 				if (dis) {
-					ret = rasm_disasm (as, (char *)buf, offset, length, as->a->bits, ascii, bin, dis - 1);
+					ret = rasm_disasm (as, offset, (char *)buf, length, as->a->bits, bin, dis - 1);
 				} else if (analinfo) {
 					ret = show_analinfo (as, (const char *)buf, offset);
 				} else {
@@ -846,7 +975,7 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 			goto beach;
 		}
 		if (dis) {
-			char *usrstr = argv[r_optind];
+			char *usrstr = strdup (opt.argv[opt.ind]);
 			len = strlen (usrstr);
 			if (skip && len > skip) {
 				skip *= 2;
@@ -855,8 +984,9 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 				len -= skip;
 				usrstr[len] = 0;
 			}
+			// XXX this is a wrong usage of endianness
 			if (!strncmp (usrstr, "0x", 2)) {
-				usrstr += 2;
+				memmove (usrstr, usrstr + 2, strlen (usrstr + 2) + 1);
 			}
 			if (rad) {
 				as->oneliner = true;
@@ -864,12 +994,13 @@ R_API int r_main_rasm2(int argc, char *argv[]) {
 				printf ("e asm.bits=%d\n", bits);
 				printf ("\"wa ");
 			}
-			ret = rasm_disasm (as, (char *)usrstr, offset, len,
-					as->a->bits, ascii, bin, dis - 1);
+			ret = rasm_disasm (as, offset, (char *)usrstr, len,
+					as->a->bits, bin, dis - 1);
+			free (usrstr);
 		} else if (analinfo) {
-			ret = show_analinfo (as, (const char *)argv[r_optind], offset);
+			ret = show_analinfo (as, (const char *)opt.argv[opt.ind], offset);
 		} else {
-			ret = print_assembly_output (as, argv[r_optind], offset, len, as->a->bits,
+			ret = print_assembly_output (as, opt.argv[opt.ind], offset, len, as->a->bits,
 							bin, use_spp, rad, hexwords, arch);
 		}
 		if (!ret) {
