@@ -13,30 +13,30 @@
 static int replace(int argc, const char *argv[], char *newstr) {
 	int i,j,k;
 	struct {
-		char *op;
-		char *str;
+		const char *op;
+		const char *str;
 	} ops[] = {
 		{ "add",     "B += A"},
 		{ "addc",    "B += A + t"},
 		{ "addv",    "B += A; t = int_overflow (B)"},
 		{ "and",     "B &= A"},
 		{ "and.b",   "B &= A"},
-		{ "bf",      "if(!t) goto A"},
-		{ "bf.s",    "if(!t) goto A"},
+		{ "bf",      "if (!t) goto A"},
+		{ "bf.s",    "if (!t) goto A"},
 		{ "bra",     "goto A"},
 		{ "brk",     "_break_exception ()"},
-		{ "bsr",     "call A"},
-		{ "bsrf",    "call A"},
-		{ "bt",      "if(t) goto A"},
-		{ "bt.s",    "if(t) goto A"},
+		{ "bsr",     "A ()"},
+		{ "bsrf",    "A ()"},
+		{ "bt",      "if (t) goto A"},
+		{ "bt.s",    "if (t) goto A"},
 		{ "clrmac",  "_clrmac ()"},
 		{ "clrs",    "_clrs ()"},
 		{ "clrt",    "_clrt ()"},
-		{ "cmp/eq",  "t = A == B ? 1 : 0"},
-		{ "cmp/ge",  "t = A >= B ? 1 : 0"},
-		{ "cmp/gt",  "t = A > B ? 1 : 0"},
-		{ "cmp/hi",  "t = (unsigned) A > (unsigned) B ? 1 : 0"},
-		{ "cmp/hs",  "t = (unsigned) A >= (unsigned) B ? 1 : 0"},
+		{ "cmp/eq",  "t = B == A ? 1 : 0"},
+		{ "cmp/ge",  "t = B >= A ? 1 : 0"},
+		{ "cmp/gt",  "t = B > A ? 1 : 0"},
+		{ "cmp/hi",  "t = (unsigned) B > (unsigned) A ? 1 : 0"},
+		{ "cmp/hs",  "t = (unsigned) B >= (unsigned) A ? 1 : 0"},
 		{ "cmp/pl",  "t = A > 0 ? 1 : 0"},
 		{ "cmp/pz",  "t = A >= 0 ? 1 : 0"},
 		{ "cmp/str", "t = A ^ B ? 1 : 0"},
@@ -44,12 +44,14 @@ static int replace(int argc, const char *argv[], char *newstr) {
 		{ "dmuls.l", "mac = B * A"},
 		{ "dmulu.l", "mac = (unsigned) B * (unsigned) A"},
 		{ "dt",      "A--; t = !A ? 1 : 0"},
+		{ "exts.b",  "B = (int) A"},
+		{ "extu.b",  "B = (unsigned int) A"},
 		{ "exts.w",  "B = (int) A"},
 		{ "extu.w",  "B = (unsigned int) A"},
 		{ "fabs",    "A = abs (A)"},
 		{ "fadd",    "B += A"},
-		{ "fcmp/eq", "t = A == B ? 1 : 0"},
-		{ "fcmp/gt", "t = A > B ? 1 : 0"},
+		{ "fcmp/eq", "t = B == A ? 1 : 0"},
+		{ "fcmp/gt", "t = B > A ? 1 : 0"},
 		{ "fcnvds",  "B = A"},
 		{ "fdiv",    "B /= A"},
 		{ "flds",    "B = A"},
@@ -64,10 +66,10 @@ static int replace(int argc, const char *argv[], char *newstr) {
 		{ "fsqrt",   "A = sqrt (A)"},
 		{ "fsts",    "B = A"},
 		{ "fsub",    "B -= A"},
-		{ "ftrc",    "B = A"},
+		{ "ftrc",    "B = trunc (A)"},
 		{ "ftrv",    "B *= A"},
 		{ "jmp",     "goto A"},
-		{ "jsr",     "goto A"},
+		{ "jsr",     "A ()"},
 		{ "ldr",     "B = A"},
 		{ "ldr.l",   "B = A"},
 		{ "lds",     "B = A"},
@@ -116,7 +118,7 @@ static int replace(int argc, const char *argv[], char *newstr) {
 		{ "swap.w",  "swap_word (B, A)"},
 		{ "tas.b",   "test_and_set (A)"},
 		{ "trapa",   "trap (A)"},
-		{ "tst",     "t = B & A ? 1 : 0"},
+		{ "tst",     "t = B & A ? 0 : 1"},
 		{ "xor",     "B ^= A"},
 		{ "xor.b",   "B ^= A"},
 		{ NULL }
@@ -268,7 +270,7 @@ RParsePlugin r_parse_plugin_sh_pseudo = {
 	.parse = parse
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_PARSE,
 	.data = &r_parse_plugin_sh_pseudo,
