@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2020 - nibble, pancake */
+/* radare - LGPL - Copyright 2009-2021 - nibble, pancake */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -12,13 +12,16 @@
 //    0x0001f3a4      9a67620eca       call word 0xca0e:0x6267
 //    0x0001f41c      eabe76de12       jmp word 0x12de:0x76be [2]
 //    0x0001f56a      ea7ed73cd3       jmp word 0xd33c:0xd77e [6]
-static int replace (int argc, char *argv[], char *newstr) {
+
+// XXX seems like '(#)' doesnt works.. so it needs to be '( # )'
+// this is a bug somewhere else
+static int replace(int argc, char *argv[], char *newstr) {
 #define MAXPSEUDOOPS 10
 	int i, j, k, d;
 	char ch;
 	struct {
-		char *op;
-		char *str;
+		const char *op;
+		const char *str;
 		int args[MAXPSEUDOOPS];  // XXX can't use flex arrays, all unused will be 0
 	} ops[] = {
 		{ "adc",  "# += #", {1, 2}},
@@ -52,19 +55,22 @@ static int replace (int argc, char *argv[], char *newstr) {
 		{ "fxch",  "#,# = #,#", {1, 2, 2, 1}},
 		{ "idiv",  "# /= #", {1, 2}},
 		{ "imul",  "# = # * #", {1, 2, 3}},
-		{ "in",   "# = io[#]", {1, 2}},
+		{ "in",   "# = io[ # ]", {1, 2}},
 		{ "inc",  "#++", {1}},
 		{ "ja", "if (((unsigned) var) > 0) goto #", {1}},
 		{ "jb", "if (((unsigned) var) < 0) goto #", {1}},
 		{ "jbe", "if (((unsigned) var) <= 0) goto #", {1}},
-		{ "je", "if (!var) goto #", {1}},
-		{ "jg", "if (var > 0) goto #", {1}},
-		{ "jge", "if (var >= 0) goto #", {1}},
-		{ "jle", "if (var <= 0) goto #", {1}},
-		{ "jmp",  "goto #", {1}},
-		{ "jne", "if (var) goto #", {1}},
+		{ "je", "if (!var) goto loc_#", {1}},
+		{ "jg", "if (var > 0) goto loc_#", {1}},
+		{ "jge", "if (var >= 0) goto loc_#", {1}},
+		{ "jle", "if (var <= 0) goto loc_#", {1}},
+		{ "jmp",  "goto loc_#", {1}},
+		{ "jne", "if (var) goto loc_#", {1}},
 		{ "lea",  "# = #", {1, 2}},
 		{ "mov",  "# = #", {1, 2}},
+		{ "movabs", "# = #", {1, 2}},
+		{ "cmovne", "if (!zf) # = #", {1, 2}},
+		{ "cmove", "if (zf) # = #", {1, 2}},
 		{ "movq",  "# = #", {1, 2}},
 		{ "movaps",  "# = #", {1, 2}},
 		{ "movups",  "# = #", {1, 2}},
@@ -76,10 +82,9 @@ static int replace (int argc, char *argv[], char *newstr) {
 		{ "movnti", "# = #", {1, 2}},
 		{ "movntpd", "# = #", {1, 2}},
 		{ "pcmpeqb", "# == #", {1, 2}},
-
 		{ "movdqu", "# = #", {1, 2}},
 		{ "movdqa", "# = #", {1, 2}},
-		{ "pextrb", "# = (byte) # [#]", {1, 2, 3}},
+		{ "pextrb", "# = (byte) # [ # ]", {1, 2, 3}},
 		{ "palignr", "# = # align #", {1, 2, 3}},
 		{ "pxor", "# ^= #", {1, 2}},
 		{ "xorps", "# ^= #", {1, 2}},
@@ -89,9 +94,10 @@ static int replace (int argc, char *argv[], char *newstr) {
 		{ "nop",  "", {0}},
 		{ "not",  "# = !#", {1, 1}},
 		{ "or",   "# |= #", {1, 2}},
-		{ "out",  "io[#] = #", {1, 2}},
-		{ "pop",  "pop #", {1}},
-		{ "push", "push #", {1}},
+		{ "out",  "io[ # ] = #", {1, 2}},
+		{ "pop",  "# = pop ()", {1}},
+		{ "pushf", "push (cpuflags)", {0}},
+		{ "push", "push (#)", {1}},
 		{ "ret",  "return", {0}},
 		{ "sal",  "# <<= #", {1, 2}},
 		{ "sar",  "# >>= #", {1, 2}},
@@ -107,7 +113,7 @@ static int replace (int argc, char *argv[], char *newstr) {
 		{ "swap", "var = #; # = #; # = var", {1, 1, 2, 2}},
 		{ "test", "var = # & #", {1, 2}},
 		{ "xchg",  "#,# = #,#", {1, 2, 2, 1}},
-		{ "xadd",  "#,# = #,#+#", {1, 2, 2, 1, 2}},
+		{ "xadd",  "#,# = #,# + #", {1, 2, 2, 1, 2}},
 		{ "xor",  "# ^= #", {1, 2}},
 		{ NULL }
 	};
@@ -118,9 +124,9 @@ static int replace (int argc, char *argv[], char *newstr) {
 			argv[2] = "0";
 		}
 	}
-	for (i = 0; ops[i].op != NULL; i++) {
+	for (i = 0; ops[i].op; i++) {
 		if (!strcmp (ops[i].op, argv[0])) {
-			if (newstr != NULL) {
+			if (newstr) {
 				d = 0;
 				j = 0;
 				ch = ops[i].str[j];
@@ -138,7 +144,7 @@ static int replace (int argc, char *argv[], char *newstr) {
 							continue;
 						}
 						const char *w = argv[idx];
-						if (w != NULL) {
+						if (w) {
 							strcpy (newstr + k, w);
 							k += strlen (w) - 1;
 						}
@@ -161,10 +167,9 @@ static int replace (int argc, char *argv[], char *newstr) {
 		}
 	}
 	return false;
-#undef MAXPSEUDOOPS
 }
 
-static int parse (RParse *p, const char *data, char *str) {
+static int parse(RParse *p, const char *data, char *str) {
 	char w0[256], w1[256], w2[256], w3[256];
 	int i;
 	size_t len = strlen (data);
@@ -179,28 +184,40 @@ static int parse (RParse *p, const char *data, char *str) {
 	}
 	*w0 = *w1 = *w2 = *w3 = '\0';
 	if (*buf) {
-		end = strchr (buf, '\0');
-		ptr = strchr (buf, ' ');
+		end = buf + strlen (buf);
+		
+		ptr = strchr (buf, '(');
 		if (!ptr) {
-			ptr = strchr (buf, '\t');
-		}
-		if (!ptr) {
-			ptr = end;
-		}
-		*ptr = '\0';
-		if (ptr != end) {
-			for (++ptr; *ptr == ' '; ptr++) {
-				;
+			ptr = strchr (buf, ' ');
+			if (!ptr) {
+				ptr = strchr (buf, '\t');
+				if (!ptr) {
+					ptr = end;
+				}
 			}
 		}
-		r_str_ncpy (w0, buf, sizeof (w0));
-		r_str_ncpy (w1, ptr, sizeof (w1));
+		bool par = (ptr != buf && *ptr == '(');
+		for (; ptr < end; ptr++) {
+			if (*ptr != ' ') {
+				if (*ptr == ')') {
+					ptr--;
+				}
+				break;
+			}
+		}
+		if (par) ptr++;
+		r_str_ncpy (w0, buf, R_MIN (ptr - buf, sizeof (w0)));
+		if (par) ptr--;
+		r_str_ncpy (w1, ptr, R_MIN (end-ptr+1, sizeof (w1)));
 		optr = ptr;
 		ptr = strchr (ptr, ',');
 		if (ptr) {
-			*ptr = '\0';
-			for (++ptr; *ptr == ' '; ptr++) {
-				;
+			*ptr++ = '\0';
+			for (++ptr; ptr < end ; ptr++) {
+				if (*ptr != ')' && *ptr != ' ') {
+		//			ptr++;
+					break;
+				}
 			}
 			r_str_ncpy (w1, optr, sizeof (w1));
 			r_str_ncpy (w2, ptr, sizeof (w2));
@@ -226,8 +243,7 @@ static int parse (RParse *p, const char *data, char *str) {
 	/* TODO: interpretation of memory location fails*/
 	//ensure imul & mul interpretations works
 	if (strstr (w0, "mul")) {
-		if (nw == 2)
-		{
+		if (nw == 2) {
 			r_str_ncpy (wa[3], wa[1], sizeof (w3));
 
 			switch (wa[3][0]) {
@@ -253,25 +269,27 @@ static int parse (RParse *p, const char *data, char *str) {
 					r_str_ncpy (wa[2], "al", sizeof (w2));
 				}
 			}
-		}
-		else if (nw == 3)
-		{
+		} else if (nw == 3) {
 			r_str_ncpy (wa[3], wa[2], sizeof (w3));
 			r_str_ncpy (wa[2], wa[1], sizeof (w2));
 		}
-
 		replace (nw, wa, str);
-
+	} else if (strstr (w0, "lea")) {
+		r_str_replace_char (w2, '[', 0);
+		r_str_replace_char (w2, ']', 0);
+		replace (nw, wa, str);
 	} else if ((strstr (w1, "ax") || strstr (w1, "ah") || strstr (w1, "al")) && !p->retleave_asm) {
 		if (!(p->retleave_asm = (char *) malloc (sz))) {
 			return false;
 		}
 		r_snprintf (p->retleave_asm, sz, "return %s", w2);
 		replace (nw, wa, str);
+#if 0
 	} else if ((strstr (w0, "leave") && p->retleave_asm) || (strstr (w0, "pop") && strstr (w1, "bp"))) {
-		r_str_ncpy (wa[0], " \0", 2);
-		r_str_ncpy (wa[1], " \0", 2);
+		r_str_ncpy (wa[0], " ", 2);
+		r_str_ncpy (wa[1], " ", 2);
 		replace (nw, wa, str);
+#endif
 	} else if (strstr (w0, "ret") && p->retleave_asm) {
 		r_str_ncpy (str, p->retleave_asm, sz);
 		R_FREE (p->retleave_asm);
@@ -285,29 +303,7 @@ static int parse (RParse *p, const char *data, char *str) {
 	return true;
 }
 
-#if 0
-static inline int ishexch (char c) {
-	if (c>=0 && c<=9) return 1;
-	if (c>='a' && c<='f') return 1;
-	if (c>='A' && c<='F') return 1;
-	return 0;
-}
-
-static inline int issegoff (const char *w) {
-	if (!ishexch (w[0])) return 0;
-	if (!ishexch (w[1])) return 0;
-	if (!ishexch (w[2])) return 0;
-	if (!ishexch (w[3])) return 0;
-	// :
-	if (!ishexch (w[5])) return 0;
-	if (!ishexch (w[6])) return 0;
-	if (!ishexch (w[7])) return 0;
-	if (!ishexch (w[8])) return 0;
-	return 1;
-}
-#endif
-
-static void parse_localvar (RParse *p, char *newstr, size_t newstr_len, const char *var, const char *reg, char sign, char *ireg, bool att) {
+static void parse_localvar(RParse *p, char *newstr, size_t newstr_len, const char *var, const char *reg, char sign, char *ireg, bool att) {
 	RStrBuf *sb = r_strbuf_new ("");
 	if (att) {
 		if (p->localvar_only) {
@@ -326,21 +322,23 @@ static void parse_localvar (RParse *p, char *newstr, size_t newstr_len, const ch
 			r_strbuf_setf (sb, " + %s", ireg);
 		}
 		if (p->localvar_only) {
-			snprintf (newstr, newstr_len - 1, "[%s%s]", var, r_strbuf_get (sb));
+			snprintf (newstr, newstr_len - 1, "%s%s", var, r_strbuf_get (sb));
 		} else {
-			snprintf (newstr, newstr_len - 1, "[%s%s %c %s]", reg, r_strbuf_get (sb), sign, var);
+			snprintf (newstr, newstr_len - 1, "%s%s %c %s", reg, r_strbuf_get (sb), sign, var);
 		}
 	}
 	r_strbuf_free (sb);
 }
 
-static inline void mk_reg_str(const char *regname, int delta, bool sign, bool att, char *ireg, char *dest, int len) {
+static void mk_reg_str(const char *regname, int delta, bool sign, bool att, char *ireg, char *dest, int len) {
 	RStrBuf *sb = r_strbuf_new ("");
 	if (att) {
 		if (ireg) {
 			r_strbuf_setf (sb, ", %%%s", ireg);
 		}
-		if (delta < 10) {
+		if (delta == 0) {
+			snprintf (dest, len - 1, "(%%%s%s)", regname, r_strbuf_get (sb));
+		} else if (delta < 10) {
 			snprintf (dest, len - 1, "%s%d(%%%s%s)", sign ? "" : "-", delta, regname, r_strbuf_get (sb));
 		} else {
 			snprintf (dest, len - 1, "%s0x%x(%%%s%s)", sign ? "" : "-", delta, regname, r_strbuf_get (sb));
@@ -349,19 +347,20 @@ static inline void mk_reg_str(const char *regname, int delta, bool sign, bool at
 		if (ireg) {
 			r_strbuf_setf (sb, " + %s", ireg);
 		}
-		if (delta < 10) {
-			snprintf (dest, len - 1, "[%s%s %c %d]", regname, r_strbuf_get (sb), sign ? '+':'-', delta);
+		if (delta == 0) {
+			snprintf (dest, len - 1, "%s%s", regname, r_strbuf_get (sb));
+		} else if (delta < 10) {
+			snprintf (dest, len - 1, "%s%s %c %d", regname, r_strbuf_get (sb), sign ? '+':'-', delta);
 		} else {
-			snprintf (dest, len - 1, "[%s%s %c 0x%x]", regname, r_strbuf_get (sb), sign ? '+':'-', delta);
+			snprintf (dest, len - 1, "%s%s %c 0x%x", regname, r_strbuf_get (sb), sign ? '+':'-', delta);
 		}
 	}
 	r_strbuf_free (sb);
 }
 
-static bool varsub (RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *data, char *str, int len) {
+static bool subvar(RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *data, char *str, int len) {
 	RList *bpargs, *spargs;
 	RAnal *anal = p->analb.anal;
-	RAnalVar *bparg, *sparg;
 	RListIter *bpargiter, *spiter;
 	char oldstr[64], newstr[64];
 	char *tstr = strdup (data);
@@ -371,7 +370,7 @@ static bool varsub (RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *dat
 
 	bool att = strchr (data, '%');
 
-	if (p->relsub) {
+	if (p->subrel) {
 		if (att) {
 			char *rip = (char *) r_str_casestr (tstr, "(%rip)");
 			if (rip) {
@@ -425,8 +424,8 @@ static bool varsub (RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *dat
 		free (tstr);
 		return false;
 	}
-	bpargs = p->varlist (anal, f, 'b');
-	spargs = p->varlist (anal, f, 's');
+	bpargs = p->varlist (f, 'b');
+	spargs = p->varlist (f, 's');
 	/* Iterate over stack pointer arguments/variables */
 	bool ucase = *tstr >= 'A' && *tstr <= 'Z';
 	if (ucase && tstr[1]) {
@@ -436,24 +435,47 @@ static bool varsub (RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *dat
 	if (p->get_op_ireg) {
 		ireg = p->get_op_ireg(p->user, addr);
 	}
+	RAnalVarField *bparg, *sparg;
 	r_list_foreach (spargs, spiter, sparg) {
-		// assuming delta always positive?
-		if (p->get_ptr_at) {
-			sparg->delta = p->get_ptr_at (p->user, sparg, addr);
+		char sign = '+';
+		st64 delta = p->get_ptr_at
+			? p->get_ptr_at (f, sparg->delta, addr)
+			: ST64_MAX;
+		if (delta == ST64_MAX && sparg->field) {
+			delta = sparg->delta;
+		} else if (delta == ST64_MAX) {
+			continue;
 		}
-		mk_reg_str (anal->reg->name[R_REG_NAME_SP], sparg->delta, true, att, ireg, oldstr, sizeof (oldstr));
+		if (delta < 0) {
+			sign = '-';
+			delta = -delta;
+		}
+		const char *reg = NULL;
+		if (p->get_reg_at) {
+			reg = p->get_reg_at (f, sparg->delta, addr);
+		}
+		if (!reg) {
+			reg = anal->reg->name[R_REG_NAME_SP];
+		}
+		mk_reg_str (reg, delta, sign == '+', att, ireg, oldstr, sizeof (oldstr));
 
 		if (ucase) {
 			r_str_case (oldstr, true);
 		}
-		parse_localvar (p, newstr, sizeof (newstr), sparg->name, anal->reg->name[R_REG_NAME_SP], '+', ireg, att);
-		char *ptr = strstr(tstr, oldstr);
+		parse_localvar (p, newstr, sizeof (newstr), sparg->name, reg, sign, ireg, att);
+		char *ptr = strstr (tstr, oldstr);
 		if (ptr && (!att || *(ptr - 1) == ' ')) {
+			if (delta == 0) {
+				char *end = ptr + strlen (oldstr);
+				if (*end != ']' && *end != '\0') {
+					continue;
+				}
+			}
 			tstr = r_str_replace (tstr, oldstr, newstr, 1);
 			break;
 		} else {
 			r_str_case (oldstr, false);
-			ptr = strstr(tstr, oldstr);
+			ptr = strstr (tstr, oldstr);
 			if (ptr && (!att || *(ptr - 1) == ' ')) {
 				tstr = r_str_replace (tstr, oldstr, newstr, 1);
 				break;
@@ -463,30 +485,51 @@ static bool varsub (RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *dat
 	/* iterate over base pointer args/vars */
 	r_list_foreach (bpargs, bpargiter, bparg) {
 		char sign = '+';
-		if (bparg->delta < 0) {
-			sign = '-';
-			bparg->delta = -bparg->delta;
+		st64 delta = p->get_ptr_at
+			? p->get_ptr_at (f, bparg->delta, addr)
+			: ST64_MAX;
+		if (delta == ST64_MAX && bparg->field) {
+			delta = bparg->delta + f->bp_off;
+		} else if (delta == ST64_MAX) {
+			continue;
 		}
-		mk_reg_str (anal->reg->name[R_REG_NAME_BP], bparg->delta, sign=='+', att, ireg, oldstr, sizeof (oldstr));
+		if (delta < 0) {
+			sign = '-';
+			delta = -delta;
+		}
+		const char *reg = NULL;
+		if (p->get_reg_at) {
+			reg = p->get_reg_at (f, bparg->delta, addr);
+		}
+		if (!reg) {
+			reg = anal->reg->name[R_REG_NAME_BP];
+		}
+		mk_reg_str (reg, delta, sign == '+', att, ireg, oldstr, sizeof (oldstr));
 		if (ucase) {
 			r_str_case (oldstr, true);
 		}
-		parse_localvar (p, newstr, sizeof (newstr), bparg->name, anal->reg->name[R_REG_NAME_BP], sign, ireg, att);
+		parse_localvar (p, newstr, sizeof (newstr), bparg->name, reg, sign, ireg, att);
 		char *ptr = strstr (tstr, oldstr);
 		if (ptr && (!att || *(ptr - 1) == ' ')) {
+			if (delta == 0) {
+				char *end = ptr + strlen (oldstr);
+				if (*end != ']' && *end != '\0') {
+					continue;
+				}
+			}
 			tstr = r_str_replace (tstr, oldstr, newstr, 1);
 			break;
 		} else {
 			r_str_case (oldstr, false);
-			ptr = strstr(tstr, oldstr);
+			ptr = strstr (tstr, oldstr);
 			if (ptr && (!att || *(ptr - 1) == ' ')) {
 				tstr = r_str_replace (tstr, oldstr, newstr, 1);
 				break;
 			}
 		}
 		// Try with no spaces
-		snprintf (oldstr, sizeof (oldstr)-1, "[%s%c0x%x]", anal->reg->name[R_REG_NAME_BP], sign, bparg->delta);
-		if (strstr (tstr, oldstr) != NULL) {
+		snprintf (oldstr, sizeof (oldstr) - 1, "[%s%c0x%x]", reg, sign, (int)delta);
+		if (strstr (tstr, oldstr)) {
 			tstr = r_str_replace (tstr, oldstr, newstr, 1);
 			break;
 		}
@@ -521,7 +564,7 @@ RParsePlugin r_parse_plugin_x86_pseudo = {
 	.name = "x86.pseudo",
 	.desc = "X86 pseudo syntax",
 	.parse = &parse,
-	.varsub = &varsub,
+	.subvar = &subvar,
 };
 
 #ifndef R2_PLUGIN_INCORE
