@@ -1,7 +1,6 @@
-/* radare2 - LGPL - Copyright 2009-2020 - nibble, pancake, maijin */
+/* radare2 - LGPL - Copyright 2009-2021 - nibble, pancake, maijin */
 
 #include <stdio.h>
-
 #include <r_types.h>
 #include <r_parse.h>
 #include <config.h>
@@ -12,7 +11,6 @@ static RParsePlugin *parse_static_plugins[] =
 	{ R_PARSE_STATIC_PLUGINS };
 
 R_API RParse *r_parse_new(void) {
-	int i;
 	RParse *p = R_NEW0 (RParse);
 	if (!p) {
 		return NULL;
@@ -25,10 +23,11 @@ R_API RParse *r_parse_new(void) {
 	p->notin_flagspace = NULL;
 	p->flagspace = NULL;
 	p->pseudo = false;
-	p->relsub = false;
-	p->tailsub = false;
+	p->subrel = false;
+	p->subtail = false;
 	p->minval = 0x100;
 	p->localvar_only = false;
+	size_t i;
 	for (i = 0; parse_static_plugins[i]; i++) {
 		r_parse_add (p, parse_static_plugins[i]);
 	}
@@ -51,6 +50,15 @@ R_API bool r_parse_add(RParse *p, RParsePlugin *foo) {
 	return true;
 }
 
+static char *predotname(const char *name) {
+	char *sname = strdup (name);
+	char *dot = strchr (sname, '.');
+	if (dot) {
+		*dot = 0;
+	}
+	return sname;
+}
+
 R_API bool r_parse_use(RParse *p, const char *name) {
 	RListIter *iter;
 	RParsePlugin *h;
@@ -61,6 +69,18 @@ R_API bool r_parse_use(RParse *p, const char *name) {
 			return true;
 		}
 	}
+	char *sname = predotname (name);
+	bool found = false;
+	r_list_foreach (p->parsers, iter, h) {
+		char *shname = predotname (h->name);
+		found = !strcmp (shname, sname);
+		free (shname);
+		if (found) {
+			p->cur = h;
+			break;
+		}
+	}
+	free (sname);
 	return false;
 }
 
@@ -71,7 +91,7 @@ R_API bool r_parse_assemble(RParse *p, char *data, char *str) {
 	bool ret = false;
 	char *s, *o;
 
-	data[0]='\0';
+	data[0] = '\0';
 	if (p->cur && p->cur->assemble) {
 		o = data + strlen (data);
 		do {
@@ -105,7 +125,7 @@ R_API bool r_parse_parse(RParse *p, const char *data, char *str) {
 }
 
 R_API char *r_parse_immtrim(char *opstr) {
-	if (!opstr || !*opstr) {
+	if (R_STR_ISEMPTY (opstr)) {
 		return NULL;
 	}
 	char *n = strstr (opstr, "0x");
@@ -134,9 +154,9 @@ R_API char *r_parse_immtrim(char *opstr) {
 	return opstr;
 }
 
-R_API bool r_parse_varsub(RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *data, char *str, int len) {
-	if (p->cur && p->cur->varsub) {
-		return p->cur->varsub (p, f, addr, oplen, data, str, len);
+R_API bool r_parse_subvar(RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *data, char *str, int len) {
+	if (p->cur && p->cur->subvar) {
+		return p->cur->subvar (p, f, addr, oplen, data, str, len);
 	}
 	return false;
 }
