@@ -1,6 +1,7 @@
-/* radare - LGPL - Copyright 2009-2018 - pancake */
+/* radare - LGPL - Copyright 2009-2020 - pancake */
 
 #include <r_reg.h>
+#include <r_util/r_str.h>
 
 /* non-endian safe - used for raw mapping with system registers */
 R_API ut8 *r_reg_get_bytes(RReg *reg, int type, int *size) {
@@ -41,6 +42,9 @@ R_API ut8 *r_reg_get_bytes(RReg *reg, int type, int *size) {
 	if (size) {
 		*size = sz;
 	}
+	if (!sz) {
+		return NULL;
+	}
 	buf = malloc (sz);
 	if (buf) {
 		memcpy (buf, reg->regset[type].arena->bytes, sz);
@@ -135,6 +139,9 @@ R_API int r_reg_fit_arena(RReg *reg) {
 
 	for (i = 0; i < R_REG_TYPE_LAST; i++) {
 		arena = reg->regset[i].arena;
+		if (!arena) {
+			continue;
+		}
 		newsize = 0;
 		r_list_foreach (reg->regset[i].regs, iter, r) {
 			// XXX: bits2bytes doesnt seems to work fine
@@ -256,7 +263,7 @@ R_API void r_reg_arena_zero(RReg *reg) {
 	}
 }
 
-R_API ut8 *r_reg_arena_peek(RReg *reg) {
+R_API ut8 *r_reg_arena_peek(RReg *reg, int *size) {
 	RRegSet *regset = r_reg_regset_get (reg, R_REG_TYPE_GPR);
 	if (!reg || !regset || !regset->arena || (regset->arena->size < 1)) {
 		return NULL;
@@ -266,14 +273,22 @@ R_API ut8 *r_reg_arena_peek(RReg *reg) {
 		return NULL;
 	}
 	memcpy (ret, regset->arena->bytes, regset->arena->size);
+	if (size) {
+		*size = regset->arena->size;
+	}
 	return ret;
 }
 
-R_API void r_reg_arena_poke(RReg *reg, const ut8 *ret) {
+R_API void r_reg_arena_poke(RReg *reg, const ut8 *ret, int len) {
 	RRegSet *regset = r_reg_regset_get (reg, R_REG_TYPE_GPR);
 	if (!ret || !regset || !regset->arena || !regset->arena->bytes) {
 		return;
 	}
+	if (len > 0 && len != regset->arena->size) {
+		eprintf ("Invalid size of the arena bytes to poke (%d vs %d).\n",
+			len, regset->arena->size);
+		return;
+	}
 	memcpy (regset->arena->bytes, ret, regset->arena->size);
 }
 
@@ -291,9 +306,7 @@ R_API ut8 *r_reg_arena_dup(RReg *reg, const ut8 *source) {
 }
 
 R_API int r_reg_arena_set_bytes(RReg *reg, const char *str) {
-	while (IS_WHITESPACE (*str)) {
-		str++;
-	}
+	str = r_str_trim_head_ro (str);
 	int len = r_hex_str_is_valid (str);
 	if (len == -1) {
 		eprintf ("Invalid input\n");
@@ -308,7 +321,7 @@ R_API int r_reg_arena_set_bytes(RReg *reg, const char *str) {
 	r_hex_str2bin (str, bin_str);
 
 	int i, n = 0; //n - cumulative sum of arena's sizes
-	for (i = 0; i < R_REG_TYPE_LAST; ++i) {
+	for (i = 0; i < R_REG_TYPE_LAST; i++) {
 		int sz = reg->regset[i].arena->size;
 		int bl = bin_str_len - n; //bytes left
 		int bln = bl - n;
