@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2018 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include <r_reg.h>
 #include <r_util.h>
@@ -31,6 +31,7 @@ static ut64 parse_size(char *s, char **end) {
 	return strtoul (s, end, 0) << 3;
 }
 
+//TODO: implement bool r_reg_set_def_string()
 static const char *parse_def(RReg *reg, char **tok, const int n) {
 	char *end;
 	int type, type2;
@@ -56,11 +57,13 @@ static const char *parse_def(RReg *reg, char **tok, const int n) {
 	if (type < 0 || type2 < 0) {
 		return "Invalid register type";
 	}
+#if 1
 	if (r_reg_get (reg, tok[1], R_REG_TYPE_ALL)) {
 		eprintf ("Ignoring duplicated register definition '%s'\n", tok[1]);
 		return NULL;
 		//return "Duplicate register definition";
 	}
+#endif
 
 	RRegItem *item = R_NEW0 (RRegItem);
 	if (!item) {
@@ -95,7 +98,12 @@ static const char *parse_def(RReg *reg, char **tok, const int n) {
 
 	// This is optional
 	if (n == 6) {
-		item->flags = strdup (tok[5]);
+		if (*tok[5] == '#') {
+			// Remove # from the comment
+			item->comment = strdup (tok[5] + 1);
+		} else {
+			item->flags = strdup (tok[5]);
+		}
 	}
 
 	item->arena = type2;
@@ -103,6 +111,10 @@ static const char *parse_def(RReg *reg, char **tok, const int n) {
 		reg->regset[type2].regs = r_list_newf ((RListFree)r_reg_item_free);
 	}
 	r_list_append (reg->regset[type2].regs, item);
+	if (!reg->regset[type2].ht_regs) {
+		reg->regset[type2].ht_regs = ht_pp_new0 ();
+	}
+	ht_pp_insert (reg->regset[type2].ht_regs, item->name, item);
 
 	// Update the overall profile size
 	if (item->offset + item->size > reg->size) {
@@ -114,18 +126,21 @@ static const char *parse_def(RReg *reg, char **tok, const int n) {
 }
 
 #define PARSER_MAX_TOKENS 8
-R_API int r_reg_set_profile_string(RReg *reg, const char *str) {
+R_API bool r_reg_set_profile_string(RReg *reg, const char *str) {
 	char *tok[PARSER_MAX_TOKENS];
 	char tmp[128];
 	int i, j, l;
 	const char *p = str;
 
-	if (!reg || !str) {
-		return false;
+	r_return_val_if_fail (reg && str, false);
+	if (R_STR_ISEMPTY (str)) {
+		return true;
 	}
 
 	// Same profile, no need to change
 	if (reg->reg_profile_str && !strcmp (reg->reg_profile_str, str)) {
+	//	r_reg_free_internal (reg, false);
+	//	r_reg_init (reg);
 		return true;
 	}
 
@@ -140,6 +155,7 @@ R_API int r_reg_set_profile_string(RReg *reg, const char *str) {
 
 	// Line number
 	l = 0;
+	bool have_a0 = false;
 	// For every line
 	do {
 		// Increment line number
@@ -162,20 +178,23 @@ R_API int r_reg_set_profile_string(RReg *reg, const char *str) {
 			while (*p == ' ' || *p == '\t') {
 				p++;
 			}
-			// Skip the rest of the line is a comment is encountered
-			if (*p == '#') {
-				while (*p != '\n') {
-					p++;
-				}
-			}
 			// EOL ?
 			if (*p == '\n') {
 				break;
 			}
-			// Gather a handful of chars
-			// Use isgraph instead of isprint because the latter considers ' ' printable
-			for (i = 0; isgraph ((const unsigned char)*p) && i < sizeof (tmp) - 1;) {
-				tmp[i++] = *p++;
+			if (*p == '#') {
+				// Place the rest of the line in the token if a comment is encountered
+				for (i = 0; *p != '\n'; p++) {
+					if (i < sizeof (tmp) - 1) {
+						tmp[i++] = *p;
+					}
+				}
+			} else {
+				// Save all characters up to a space/tab
+				// Use isgraph instead of isprint because the latter considers ' ' printable
+				for (i = 0; isgraph ((const unsigned char)*p) && i < sizeof (tmp) - 1;) {
+					tmp[i++] = *p++;
+				}
 			}
 			tmp[i] = '\0';
 			// Limit the number of tokens
@@ -190,29 +209,48 @@ R_API int r_reg_set_profile_string(RReg *reg, const char *str) {
 			// Do the actual parsing
 			char *first = tok[0];
 			// Check whether it's defining an alias or a register
-			const char *r = (*first == '=')
-				? parse_alias (reg, tok, j)
-				: parse_def (reg, tok, j);
-			// Clean up
-			for (i = 0; i < j; i++) {
-				free (tok[i]);
-			}
-			// Warn the user if something went wrong
-			if (r) {
-				eprintf ("%s: Parse error @ line %d (%s)\n",
-					__FUNCTION__, l, r);
-				//eprintf ("(%s)\n", str);
+			if (!strncmp (first, "=RS", 3)) {
+				reg->bits_default = atoi (tok[1]);
 				// Clean up
-				r_reg_free_internal (reg, false);
-				return false;
+				for (i = 0; i < j; i++) {
+					free (tok[i]);
+				}
+			} else {
+				const char *r = (*first == '=')
+					? parse_alias (reg, tok, j)
+					: parse_def (reg, tok, j);
+				if (!strncmp (first, "=A0", 3)) {
+					have_a0 = true;
+				}
+				// Clean up
+				for (i = 0; i < j; i++) {
+					free (tok[i]);
+				}
+				// Warn the user if something went wrong
+				if (r) {
+					eprintf ("%s: Parse error @ line %d (%s)\n",
+							__FUNCTION__, l, r);
+					//eprintf ("(%s)\n", str);
+					// Clean up
+					r_reg_free_internal (reg, false);
+					r_reg_init (reg);
+					return false;
+				}
 			}
 		}
 	} while (*p++);
+	if (!have_a0) {
+		eprintf ("Warning: =A0 not defined\n");
+		//r_reg_free_internal (reg, false);
+		///return false;
+	}
 	reg->size = 0;
 	for (i = 0; i < R_REG_TYPE_LAST; i++) {
 		RRegSet *rs = &reg->regset[i];
 		//eprintf ("* arena %s size %d\n", r_reg_get_type (i), rs->arena->size);
-		reg->size += rs->arena->size;
+		if (rs && rs->arena) {
+			reg->size += rs->arena->size;
+		}
 	}
 	// Align to byte boundary if needed
 	//if (reg->size & 7) {
@@ -227,14 +265,14 @@ R_API int r_reg_set_profile_string(RReg *reg, const char *str) {
 	return true;
 }
 
-R_API int r_reg_set_profile(RReg *reg, const char *profile) {
-	int ret;
-	char *base, *file;
+// read profile from file
+R_API bool r_reg_set_profile(RReg *reg, const char *profile) {
+	r_return_val_if_fail (reg && profile, false);
 	char *str = r_file_slurp (profile, NULL);
 	if (!str) {
-		base = r_sys_getenv (R_LIB_ENV);
+		char *base = r_sys_getenv (R_LIB_ENV);
 		if (base) {
-			file = r_str_append (base, profile);
+			char *file = r_str_append (base, profile);
 			str = r_file_slurp (file, NULL);
 			free (file);
 		}
@@ -243,28 +281,30 @@ R_API int r_reg_set_profile(RReg *reg, const char *profile) {
 		eprintf ("r_reg_set_profile: Cannot find '%s'\n", profile);
 		return false;
 	}
-	ret = r_reg_set_profile_string (reg, str);
+	bool ret = r_reg_set_profile_string (reg, str);
 	free (str);
 	return ret;
 }
 
-static int gdb_to_r2_profile(char *gdb) {
-	char *ptr = gdb, *ptr1, *gptr, *gptr1;
+static char *gdb_to_r2_profile(const char *gdb) {
+	r_return_val_if_fail (gdb, NULL);
+	RStrBuf *sb = r_strbuf_new ("");
+	if (!sb) {
+		return NULL;
+	}
+	char *ptr1, *gptr, *gptr1;
 	char name[16], groups[128], type[16];
 	const int all = 1, gpr = 2, save = 4, restore = 8, float_ = 16,
 		  sse = 32, vector = 64, system = 128, mmx = 256;
 	int number, rel, offset, size, type_bits, ret;
 	// Every line is -
 	// Name Number Rel Offset Size Type Groups
+	const char *ptr = r_str_trim_head_ro (gdb);
 
-	// Skip whitespace at beginning of line and empty lines
-	while (isspace ((ut8)*ptr)) {
-		ptr++;
-	}
 	// It's possible someone includes the heading line too. Skip it
 	if (r_str_startswith (ptr, "Name")) {
 		if (!(ptr = strchr (ptr, '\n'))) {
-			return false;
+			return NULL;
 		}
 		ptr++;
 	}
@@ -278,14 +318,19 @@ static int gdb_to_r2_profile(char *gdb) {
 		}
 		if ((ptr1 = strchr (ptr, '\n'))) {
 			*ptr1 = '\0';
+		} else {
+			eprintf ("Could not parse line: %s (missing \\n)\n", ptr);
+			r_strbuf_free (sb);
+			return false;
 		}
 		ret = sscanf (ptr, " %s %d %d %d %d %s %s", name, &number, &rel,
 			&offset, &size, type, groups);
 		// Groups is optional, others not
 		if (ret < 6) {
-			eprintf ("Could not parse line: %s\n", ptr);
-			if (!ptr1) {
-				return true;
+			if (*ptr != '*') {
+				eprintf ("Could not parse line: %s\n", ptr);
+				r_strbuf_free (sb);
+				return false;
 			}
 			ptr = ptr1 + 1;
 			continue;
@@ -293,7 +338,7 @@ static int gdb_to_r2_profile(char *gdb) {
 		// If name is '', then skip
 		if (r_str_startswith (name, "''")) {
 			if (!ptr1) {
-				return true;
+				break;
 			}
 			ptr = ptr1 + 1;
 			continue;
@@ -301,7 +346,7 @@ static int gdb_to_r2_profile(char *gdb) {
 		// If size is 0, skip
 		if (size == 0) {
 			if (!ptr1) {
-				return true;
+				break;
 			}
 			ptr = ptr1 + 1;
 			continue;
@@ -340,7 +385,7 @@ static int gdb_to_r2_profile(char *gdb) {
 		// If type is not defined, skip
 		if (!*type) {
 			if (!ptr1) {
-				return true;
+				break;
 			}
 			ptr = ptr1 + 1;
 			continue;
@@ -350,36 +395,65 @@ static int gdb_to_r2_profile(char *gdb) {
 			type_bits |= gpr;
 		}
 		// Print line
-		eprintf ("%s\t%s\t.%d\t%d\t0\n",
+		r_strbuf_appendf (sb, "%s\t%s\t.%d\t%d\t0\n",
 			// Ref: Comment above about more register type mappings
 			((type_bits & mmx) || (type_bits & float_) || (type_bits & sse)) ? "fpu" : "gpr",
 			name, size * 8, offset);
 		// Go to next line
 		if (!ptr1) {
-			return true;
+			break;
 		}
 		ptr = ptr1 + 1;
 		continue;
 	}
-	return true;
+	return r_strbuf_drain (sb);
 }
 
-R_API int r_reg_parse_gdb_profile(const char *profile_file) {
-	char *base, *str = NULL;
+R_API char *r_reg_parse_gdb_profile(const char *profile_file) {
+	char *str = NULL;
 	if (!(str = r_file_slurp (profile_file, NULL))) {
-		if ((base = r_sys_getenv (R_LIB_ENV))) {
-			char *file = r_str_append (base, profile_file);
+		char *base = r_sys_getenv (R_LIB_ENV);
+		if (base) {
+			char *file = r_str_appendf (base, R_SYS_DIR "%s", profile_file);
 			if (file) {
 				str = r_file_slurp (file, NULL);
 				free (file);
 			}
 		}
 	}
-	if (!str) {
-		eprintf ("r_reg_parse_gdb_profile: Cannot find '%s'\n", profile_file);
-		return false;
+	if (str) {
+		char *ret = gdb_to_r2_profile (str);
+		free (str);
+		return ret;
 	}
-	int ret = gdb_to_r2_profile (str);
-	free (str);
-	return ret;
+	eprintf ("r_reg_parse_gdb_profile: Cannot find '%s'\n", profile_file);
+	return NULL;
 }
+
+R_API char *r_reg_profile_to_cc(RReg *reg) {
+	const char *r0 = r_reg_get_name_by_type (reg, "R0");
+	const char *a0 = r_reg_get_name_by_type (reg, "A0");
+	const char *a1 = r_reg_get_name_by_type (reg, "A1");
+	const char *a2 = r_reg_get_name_by_type (reg, "A2");
+	const char *a3 = r_reg_get_name_by_type (reg, "A3");
+
+	if (!r0) {
+		r0 = a0;
+	}
+	if (!r0) {
+		// it is mandatory to have at least =A0 defined in the reg profile
+		// this will be enforced in reg/profile at parsing time
+		return NULL;
+	}
+	if (a3 && a2 && a1) {
+		return r_str_newf ("%s reg(%s, %s, %s, %s)", r0, a0, a1, a2, a3);
+	}
+	if (a2 && a1) {
+		return r_str_newf ("%s reg(%s, %s, %s)", r0, a0, a1, a2);
+	}
+	if (a1) {
+		return r_str_newf ("%s reg(%s, %s)", r0, a0, a1);
+	}
+	return r_str_newf ("%s reg(%s)", r0, a0);
+}
+
