@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include <r_reg.h>
 #include <r_util.h>
@@ -16,7 +16,7 @@ R_API const char *r_reg_32_to_64(RReg *reg, const char *rreg32) {
 	int i, j = -1;
 	RListIter *iter;
 	RRegItem *item;
-	for (i = 0; i < R_REG_TYPE_LAST; ++i) {
+	for (i = 0; i < R_REG_TYPE_LAST; i++) {
 		r_list_foreach (reg->regset[i].regs, iter, item) {
 			if (item->size == 32 && !r_str_casecmp (rreg32, item->name)) {
 				j = item->offset;
@@ -25,7 +25,7 @@ R_API const char *r_reg_32_to_64(RReg *reg, const char *rreg32) {
 		}
 	}
 	if (j != -1) {
-		for (i = 0; i < R_REG_TYPE_LAST; ++i) {
+		for (i = 0; i < R_REG_TYPE_LAST; i++) {
 			r_list_foreach (reg->regset[i].regs, iter, item) {
 				if (item->offset == j && item->size == 64) {
 					return item->name;
@@ -43,7 +43,7 @@ R_API const char *r_reg_64_to_32(RReg *reg, const char *rreg64) {
 	int i, j = -1;
 	RListIter *iter;
 	RRegItem *item;
-	for (i = 0; i < R_REG_TYPE_LAST; ++i) {
+	for (i = 0; i < R_REG_TYPE_LAST; i++) {
 		r_list_foreach (reg->regset[i].regs, iter, item) {
 			if (item->size == 64 && !r_str_casecmp (rreg64, item->name)) {
 				j = item->offset;
@@ -52,7 +52,7 @@ R_API const char *r_reg_64_to_32(RReg *reg, const char *rreg64) {
 		}
 	}
 	if (j != -1) {
-		for (i = 0; i < R_REG_TYPE_LAST; ++i) {
+		for (i = 0; i < R_REG_TYPE_LAST; i++) {
 			r_list_foreach (reg->regset[i].regs, iter, item) {
 				if (item->offset == j && item->size == 32) {
 					return item->name;
@@ -67,6 +67,15 @@ R_API const char *r_reg_get_type(int idx) {
 	return (idx >= 0 && idx < R_REG_TYPE_LAST) ? types[idx] : NULL;
 }
 
+R_API const char *r_reg_get_name_by_type(RReg *reg, const char *alias_name) {
+	const int n = r_reg_get_name_idx (alias_name);
+	return (n != -1)? r_reg_get_name (reg, n): NULL;
+}
+
+R_API int r_reg_default_bits(RReg *reg) {
+	return reg->bits_default;
+}
+
 R_API int r_reg_type_by_name(const char *str) {
 	r_return_val_if_fail (str, -1);
 	int i;
@@ -88,7 +97,8 @@ R_API void r_reg_item_free(RRegItem *item) {
 }
 
 R_API int r_reg_get_name_idx(const char *type) {
-	if (type)
+	r_return_val_if_fail (type, -1);
+	if (type[0] && type[1] && !type[2])
 	switch (*type | (type[1] << 8)) {
 	/* flags */
 	case 'Z' + ('F' << 8): return R_REG_NAME_ZF;
@@ -139,7 +149,7 @@ R_API const char *r_reg_get_name(RReg *reg, int role) {
 }
 
 static const char *roles[R_REG_NAME_LAST + 1] = {
-	"PC", "SP", "SR", "BP", "LR",
+	"PC", "SP", "SR", "BP", "LR", "RS",
 	"A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7", "A8", "A9",
 	"R0", "R1", "R2", "R3",
 	"ZF", "SF", "CF", "OF",
@@ -199,7 +209,7 @@ R_API void r_reg_free_internal(RReg *reg, bool init) {
 static int regcmp(RRegItem *a, RRegItem *b) {
 	int offa = (a->offset * 16) + a->size;
 	int offb = (b->offset * 16) + b->size;
-	return offa > offb;
+	return (offa > offb) - (offa < offb);
 }
 
 R_API void r_reg_reindex(RReg *reg) {
@@ -245,13 +255,10 @@ R_API void r_reg_free(RReg *reg) {
 	}
 }
 
-R_API RReg *r_reg_new(void) {
+R_API RReg *r_reg_init(RReg *reg) {
+	r_return_val_if_fail (reg, NULL);
 	RRegArena *arena;
-	RReg *reg = R_NEW0 (RReg);
-	int i;
-	if (!reg) {
-		return NULL;
-	}
+	size_t i;
 	for (i = 0; i < R_REG_TYPE_LAST; i++) {
 		arena = r_reg_arena_new (0);
 		if (!arena) {
@@ -270,6 +277,10 @@ R_API RReg *r_reg_new(void) {
 	return reg;
 }
 
+R_API RReg *r_reg_new(void) {
+	return r_reg_init (R_NEW0 (RReg));
+}
+
 R_API bool r_reg_is_readonly(RReg *reg, RRegItem *item) {
 	const char *name;
 	RListIter *iter;
@@ -286,16 +297,21 @@ R_API bool r_reg_is_readonly(RReg *reg, RRegItem *item) {
 }
 
 R_API ut64 r_reg_setv(RReg *reg, const char *name, ut64 val) {
-	return r_reg_set_value (reg, r_reg_get (reg, name, -1), val);
+	r_return_val_if_fail (reg && name, UT64_MAX);
+	RRegItem *ri = r_reg_get (reg, name, -1);
+	return ri? r_reg_set_value (reg, ri, val): UT64_MAX;
 }
 
 R_API ut64 r_reg_getv(RReg *reg, const char *name) {
-	return r_reg_get_value (reg, r_reg_get (reg, name, -1));
+	r_return_val_if_fail (reg && name, UT64_MAX);
+	RRegItem *ri = r_reg_get (reg, name, -1);
+	return ri? r_reg_get_value (reg, ri): UT64_MAX;
 }
 
 R_API RRegItem *r_reg_get(RReg *reg, const char *name, int type) {
 	int i, e;
 	r_return_val_if_fail (reg && name, NULL);
+	//TODO: define flag register as R_REG_TYPE_FLG
 	if (type == R_REG_TYPE_FLG) {
 		type = R_REG_TYPE_GPR;
 	}
@@ -327,10 +343,27 @@ R_API RRegItem *r_reg_get(RReg *reg, const char *name, int type) {
 }
 
 R_API RList *r_reg_get_list(RReg *reg, int type) {
+	if (type == R_REG_TYPE_ALL) {
+		return reg->allregs;
+	}
+
+	RList *regs;
+	int i, mask;
 	if (type < 0 || type > (R_REG_TYPE_LAST - 1)) {
 		return NULL;
 	}
-	return reg->regset[type].regs;
+
+	regs = reg->regset[type].regs;
+	if (regs && r_list_length (regs) == 0) {
+		mask = ((int)1 << type);
+		for (i = 0; i < R_REG_TYPE_LAST; i++) {
+			if (reg->regset[i].maskregstype & mask) {
+				regs = reg->regset[i].regs;
+			}
+		}
+	}
+
+	return regs;
 }
 
 // TODO regsize is in bits, delta in bytes, maybe we should standarize this..
