@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2008-2014 - pancake, TheLemonMan */
+/* radare - LGPL - Copyright 2008-2020 - pancake, TheLemonMan */
 
 #include "r_search.h"
 #include <r_regex.h>
@@ -7,7 +7,7 @@ R_API int r_search_regexp_update(RSearch *s, ut64 from, const ut8 *buf, int len)
 	RSearchKeyword *kw;
 	RListIter *iter;
 	RRegexMatch match;
-	RRegex compiled = {0};
+	RRegex rx = {0};
 	const int old_nhits = s->nhits;
 	int ret = 0;
 
@@ -18,7 +18,7 @@ R_API int r_search_regexp_update(RSearch *s, ut64 from, const ut8 *buf, int len)
 			reflags |= R_REGEX_ICASE;
 		}
 
-		if (r_regex_comp (&compiled, (char *)kw->bin_keyword, reflags)) {
+		if (r_regex_init (&rx, (char *)kw->bin_keyword, reflags)) {
 			eprintf ("Cannot compile '%s' regexp\n", kw->bin_keyword);
 			return -1;
 		}
@@ -26,7 +26,7 @@ R_API int r_search_regexp_update(RSearch *s, ut64 from, const ut8 *buf, int len)
 		match.rm_so = 0;
 		match.rm_eo = len;
 
-		while (!r_regex_exec (&compiled, (char *)buf, 1, &match, R_REGEX_STARTEND)) {
+		while (!r_regex_exec (&rx, (char *)buf, 1, &match, R_REGEX_STARTEND)) {
 			int t = r_search_hit_new (s, kw, from + match.rm_so);
 			if (!t) {
 				ret = -1;
@@ -42,7 +42,7 @@ R_API int r_search_regexp_update(RSearch *s, ut64 from, const ut8 *buf, int len)
 	}
 
 beach:
-	r_regex_fini (&compiled);
+	r_regex_fini (&rx);
 	if (!ret) {
 		ret = s->nhits - old_nhits;
 	}
