@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2014-2017 - pancake */
+/* radare - LGPL - Copyright 2014-2021 - pancake */
 
 /* this helper api is here because it depends on r_util and r_socket */
 /* we should find a better place for it. r_io? */
@@ -9,6 +9,7 @@
 #include <r_socket.h>
 #include <r_util.h>
 #include <r_lib.h>
+#include <r_cons.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 
@@ -33,8 +34,9 @@
 #if __UNIX__
 #include <sys/ioctl.h>
 #include <sys/resource.h>
-#include <termios.h>
+#ifndef __wasi__
 #include <grp.h>
+#endif
 #include <errno.h>
 #if defined(__sun)
 #include <sys/filio.h>
@@ -54,64 +56,105 @@
 #ifdef _MSC_VER
 #include <direct.h>   // to compile chdir in msvc windows
 #include <process.h>  // to compile execv in msvc windows
+#define pid_t int
 #endif
 
-#define HAVE_PTY __UNIX__ && !__ANDROID__ && LIBC_HAVE_FORK && !__sun
 
-#if EMSCRIPTEN
-#undef HAVE_PTY
-#define HAVE_PTY 0
+#if HAVE_PTY
+static int (*dyn_openpty)(int *amaster, int *aslave, char *name, struct termios *termp, struct winsize *winp) = NULL;
+static int (*dyn_login_tty)(int fd) = NULL;
+static id_t (*dyn_forkpty)(int *amaster, char *name, struct termios *termp, struct winsize *winp) = NULL;
+static void dyn_init(void) {
+	if (!dyn_openpty) {
+		dyn_openpty = r_lib_dl_sym (NULL, "openpty");
+	}
+	if (!dyn_login_tty) {
+		dyn_login_tty = r_lib_dl_sym (NULL, "login_tty");
+	}
+	if (!dyn_forkpty) {
+		dyn_forkpty = r_lib_dl_sym (NULL, "forkpty");
+	}
+#if __UNIX__
+	// attempt to fall back on libutil if we failed to load anything
+	if (!(dyn_openpty && dyn_login_tty && dyn_forkpty)) {
+		void *libutil;
+		if (!(libutil = r_lib_dl_open ("libutil." R_LIB_EXT))) {
+			eprintf ("[ERROR] rarun2: Could not find PTY utils, failed to load %s\n", "libutil." R_LIB_EXT);
+			return;
+		}
+		if (!dyn_openpty) {
+			dyn_openpty = r_lib_dl_sym (libutil, "openpty");
+		}
+		if (!dyn_login_tty) {
+			dyn_login_tty = r_lib_dl_sym (libutil, "login_tty");
+		}
+		if (!dyn_forkpty) {
+			dyn_forkpty = r_lib_dl_sym (libutil, "forkpty");
+		}
+		r_lib_dl_close (libutil);
+	}
+#endif
+}
+
 #endif
 
 R_API RRunProfile *r_run_new(const char *str) {
 	RRunProfile *p = R_NEW0 (RRunProfile);
 	if (p) {
 		r_run_reset (p);
-		r_run_parsefile (p, str);
+		if (str) {
+			r_run_parsefile (p, str);
+		}
 	}
 	return p;
 }
 
 R_API void r_run_reset(RRunProfile *p) {
+	r_return_if_fail (p);
 	memset (p, 0, sizeof (RRunProfile));
 	p->_aslr = -1;
 }
 
 R_API bool r_run_parse(RRunProfile *pf, const char *profile) {
-	if (!pf || !profile) {
-		return false;
-	}
+	r_return_val_if_fail (pf && profile, false);
 	char *p, *o, *str = strdup (profile);
 	if (!str) {
 		return false;
 	}
 	r_str_replace_char (str, '\r',0);
-	for (p = str; (o = strchr (p, '\n')); p = o) {
-		*o++ = 0;
+	p = str;
+	while (p) {
+		if ((o = strchr (p, '\n'))) {
+			*o++ = 0;
+		}
 		r_run_parseline (pf, p);
+		p = o;
 	}
 	free (str);
 	return true;
 }
 
-R_API void r_run_free (RRunProfile *r) {
-	free (r->_system);
-	free (r->_program);
-	free (r->_runlib);
-	free (r->_runlib_fcn);
-	free (r->_stdio);
-	free (r->_stdin);
-	free (r->_stdout);
-	free (r->_stderr);
-	free (r->_chgdir);
-	free (r->_chroot);
-	free (r->_libpath);
-	free (r->_preload);
-	free (r);
+R_API void r_run_free(RRunProfile *r) {
+	if (r) {
+		free (r->_system);
+		free (r->_program);
+		free (r->_runlib);
+		free (r->_runlib_fcn);
+		free (r->_stdio);
+		free (r->_stdin);
+		free (r->_stdout);
+		free (r->_stderr);
+		free (r->_chgdir);
+		free (r->_chroot);
+		free (r->_libpath);
+		free (r->_preload);
+		free (r);
+	}
 }
 
 #if __UNIX__
 static void set_limit(int n, int a, ut64 b) {
+#ifndef __wasi__
 	if (n) {
 		struct rlimit cl = {b, b};
 		setrlimit (RLIMIT_CORE, &cl);
@@ -119,6 +162,7 @@ static void set_limit(int n, int a, ut64 b) {
 		struct rlimit cl = {0, 0};
 		setrlimit (a, &cl);
 	}
+#endif
 }
 #endif
 
@@ -162,15 +206,16 @@ static char *getstr(const char *src) {
 		{
 			char *pat = strchr (src + 1, '@');
 			if (pat) {
-				int i, len, rep;
+				size_t len;
+				long i, rep;
 				*pat++ = 0;
-				rep = atoi (src + 1);
+				rep = strtol (src + 1, NULL, 10);
 				len = strlen (pat);
 				if (rep > 0) {
 					char *buf = malloc (rep);
 					if (buf) {
 						for (i = 0; i < rep; i++) {
-							buf[i] = pat[i%len];
+							buf[i] = pat[i % len];
 						}
 					}
 					return buf;
@@ -185,7 +230,8 @@ static char *getstr(const char *src) {
 		int msg_len = strlen (msg);
 		if (msg_len > 0) {
 			msg [msg_len - 1] = 0;
-			char *ret = r_str_trim_tail (r_sys_cmd_str (msg, NULL, NULL));
+			char *ret = r_sys_cmd_str (msg, NULL, NULL);
+			r_str_trim_tail (ret);
 			free (msg);
 			return ret;
 		}
@@ -193,10 +239,15 @@ static char *getstr(const char *src) {
 		return strdup ("");
 		}
 	case '!':
-		return r_str_trim_tail (r_sys_cmd_str (src + 1, NULL, NULL));
+		{
+		char *a = r_sys_cmd_str (src + 1, NULL, NULL);
+		r_str_trim_tail (a);
+		return a;
+		}
 	case ':':
 		if (src[1] == '!') {
-			ret = r_str_trim_tail (r_sys_cmd_str (src + 1, NULL, NULL));
+			ret = r_sys_cmd_str (src + 1, NULL, NULL);
+			r_str_trim_tail (ret); // why no head :?
 		} else {
 			ret = strdup (src);
 		}
@@ -208,11 +259,6 @@ static char *getstr(const char *src) {
 		eprintf ("Invalid hexpair string\n");
 		free (ret);
 		return NULL;
-#if 0
-	// what is this for??
-	case '%':
-		return (char *) strtoul (src + 1, NULL, 0);
-#endif
 	}
 	r_str_unescape ((ret = strdup (src)));
 	return ret;
@@ -250,13 +296,15 @@ static void setASLR(RRunProfile *r, int enabled) {
 	// for osxver>=10.7
 	// "unset the MH_PIE bit in an already linked executable" with --no-pie flag of the script
 	// the right way is to disable the aslr bit in the spawn call
-#elif __FreeBSD__
+#elif __FreeBSD__ || __NetBSD__ || __DragonFly__
 	r_sys_aslr (enabled);
 #else
 	// not supported for this platform
 #endif
 }
 
+#if __APPLE__ && !__POWERPC__
+#else
 #if HAVE_PTY
 static void restore_saved_fd(int saved, bool restore, int fd) {
 	if (saved == -1) {
@@ -271,6 +319,10 @@ static void restore_saved_fd(int saved, bool restore, int fd) {
 
 static int handle_redirection_proc(const char *cmd, bool in, bool out, bool err) {
 #if HAVE_PTY
+	if (!dyn_forkpty) {
+		// No forkpty api found, maybe we should fallback to just fork without any pty allocated
+		return -1;
+	}
 	// use PTY to redirect I/O because pipes can be problematic in
 	// case of interactive programs.
 	int saved_stdin = dup (STDIN_FILENO);
@@ -278,11 +330,12 @@ static int handle_redirection_proc(const char *cmd, bool in, bool out, bool err)
 		return -1;
 	}
 	int saved_stdout = dup (STDOUT_FILENO);
-	if (saved_stdout== -1) {
+	if (saved_stdout == -1) {
 		close (saved_stdin);
 		return -1;
 	}
-	int fdm, pid = forkpty (&fdm, NULL, NULL, NULL);
+	
+	int fdm, pid = dyn_forkpty (&fdm, NULL, NULL, NULL);
 	if (pid == -1) {
 		close (saved_stdin);
 		close (saved_stdout);
@@ -348,19 +401,21 @@ static int handle_redirection_proc(const char *cmd, bool in, bool out, bool err)
 	return -1;
 #endif
 }
+#endif
 
 static int handle_redirection(const char *cmd, bool in, bool out, bool err) {
-	if (!cmd || cmd[0] == '\0') {
-		return 0;
-	}
-
 #if __APPLE__ && !__POWERPC__
 	//XXX handle this in other layer since things changes a little bit
 	//this seems like a really good place to refactor stuff
 	return 0;
-#endif
+#else
+	if (!cmd || !*cmd) {
+		return 0;
+	}
 	if (cmd[0] == '"') {
-#if __UNIX__
+#ifdef __wasi__
+		eprintf ("[ERROR] rarun2: Cannot create pipe\n");
+#elif __UNIX__
 		if (in) {
 			int pipes[2];
 			if (pipe (pipes) != -1) {
@@ -388,7 +443,6 @@ static int handle_redirection(const char *cmd, bool in, bool out, bool err) {
 #warning quoted string redirection handle not yet done
 #endif
 #endif
-		return 0;
 	} else if (cmd[0] == '!') {
 		// redirection to a process
 		return handle_redirection_proc (cmd + 1, in, out, err);
@@ -408,6 +462,7 @@ static int handle_redirection(const char *cmd, bool in, bool out, bool err) {
 			eprintf ("[ERROR] rarun2: Cannot open: %s\n", cmd);
 			return 1;
 		}
+#ifndef __wasi__
 #define DUP(x) { close(x); dup2(f,x); }
 		if (in) {
 			DUP(0);
@@ -418,29 +473,32 @@ static int handle_redirection(const char *cmd, bool in, bool out, bool err) {
 		if (err) {
 			DUP(2);
 		}
+#endif
 		close (f);
-		return 0;
 	}
+	return 0;
+#endif
 }
 
-R_API int r_run_parsefile(RRunProfile *p, const char *b) {
+R_API bool r_run_parsefile(RRunProfile *p, const char *b) {
+	r_return_val_if_fail (p && b, false);
 	char *s = r_file_slurp (b, NULL);
 	if (s) {
-		int ret = r_run_parse (p, s);
+		bool ret = r_run_parse (p, s);
 		free (s);
 		return ret;
 	}
 	return 0;
 }
 
-R_API bool r_run_parseline(RRunProfile *p, char *b) {
+R_API bool r_run_parseline(RRunProfile *p, const char *b) {
 	int must_free = false;
 	char *e = strchr (b, '=');
 	if (!e || *b == '#') {
 		return 0;
 	}
 	*e++ = 0;
-	if (*e=='$') {
+	if (*e == '$') {
 		must_free = true;
 		e = r_sys_getenv (e);
 	}
@@ -449,6 +507,8 @@ R_API bool r_run_parseline(RRunProfile *p, char *b) {
 	}
 	if (!strcmp (b, "program")) {
 		p->_args[0] = p->_program = strdup (e);
+	} else if (!strcmp (b, "daemon")) {
+		p->_daemon = true;
 	} else if (!strcmp (b, "system")) {
 		p->_system = strdup (e);
 	} else if (!strcmp (b, "runlib")) {
@@ -543,7 +603,7 @@ R_API bool r_run_parseline(RRunProfile *p, char *b) {
 			return false;
 		}
 		for (;;) {
-			if (!fgets (buf, sizeof (buf) - 1, fd)) {
+			if (!fgets (buf, sizeof (buf), fd)) {
 				break;
 			}
 			if (feof (fd)) {
@@ -582,7 +642,7 @@ R_API bool r_run_parseline(RRunProfile *p, char *b) {
 	return true;
 }
 
-R_API const char *r_run_help() {
+R_API const char *r_run_help(void) {
 	return
 	"program=/bin/ls\n"
 	"arg1=/bin\n"
@@ -593,6 +653,7 @@ R_API const char *r_run_help() {
 	"# arg6=@arg.txt\n"
 	"# arg7=@300@ABCD # 300 chars filled with ABCD pattern\n"
 	"# system=r2 -\n"
+	"# daemon=false\n"
 	"# aslr=no\n"
 	"setenv=FOO=BAR\n"
 	"# unsetenv=FOO\n"
@@ -665,21 +726,28 @@ static int redirect_socket_to_stdio(RSocket *sock) {
 	close (0);
 	close (1);
 	close (2);
-
+#ifndef __wasi__
 	dup2 (sock->fd, 0);
 	dup2 (sock->fd, 1);
 	dup2 (sock->fd, 2);
-
+#endif
 	return 0;
 }
 
+#if __WINDOWS__
+static RThreadFunctionRet exit_process(RThread *th) {
+	// eprintf ("\nrarun2: Interrupted by timeout\n");
+	exit (0);
+}
+#endif
+
 static int redirect_socket_to_pty(RSocket *sock) {
 #if HAVE_PTY
 	// directly duplicating the fds using dup2() creates problems
 	// in case of interactive applications
-	int fdm, fds;
+	int fdm = -1, fds = -1;
 
-	if (openpty (&fdm, &fds, NULL, NULL, NULL) == -1) {
+	if (dyn_openpty && dyn_openpty (&fdm, &fds, NULL, NULL, NULL) == -1) {
 		perror ("opening pty");
 		return -1;
 	}
@@ -688,8 +756,12 @@ static int redirect_socket_to_pty(RSocket *sock) {
 
 	if (child_pid == -1) {
 		eprintf ("cannot fork\n");
-		close(fdm);
-		close(fds);
+		if (fdm != -1) {
+			close (fdm);
+		}
+		if (fds != -1) {
+			close (fds);
+		}
 		return -1;
 	}
 
@@ -726,15 +798,22 @@ static int redirect_socket_to_pty(RSocket *sock) {
 		}
 
 		free (buff);
-		close (fdm);
+		if (fdm != -1) {
+			close (fdm);
+			fdm = -1;
+		}
 		r_socket_free (sock);
 		exit (0);
 	}
 
 	// parent
 	r_socket_close_fd (sock);
-	login_tty (fds);
-	close (fdm);
+	if (dyn_login_tty) {
+		dyn_login_tty (fds);
+	}
+	if (fdm != -1) {
+		close (fdm);
+	}
 
 	// disable the echo on slave stdin
 	struct termios t;
@@ -752,8 +831,12 @@ static int redirect_socket_to_pty(RSocket *sock) {
 R_API int r_run_config_env(RRunProfile *p) {
 	int ret;
 
+#if HAVE_PTY
+	dyn_init ();
+#endif
+
 	if (!p->_program && !p->_system && !p->_runlib) {
-		printf ("No program, system or runlib rule defined\n");
+		eprintf ("No program, system or runlib rule defined\n");
 		return 1;
 	}
 	// when IO is redirected to a process, handle them together
@@ -772,7 +855,7 @@ R_API int r_run_config_env(RRunProfile *p) {
 	if (p->_aslr != -1) {
 		setASLR (p, p->_aslr);
 	}
-#if __UNIX__
+#if __UNIX__ && !__wasi__ && !defined(serenity)
 	set_limit (p->_docore, RLIMIT_CORE, RLIM_INFINITY);
 	if (p->_maxfd) {
 		set_limit (p->_maxfd, RLIMIT_NOFILE, p->_maxfd);
@@ -796,6 +879,7 @@ R_API int r_run_config_env(RRunProfile *p) {
 			*q = 0;
 			if (!r_socket_connect_tcp (fd, p->_connect, q+1, 30)) {
 				eprintf ("Cannot connect\n");
+				r_socket_free (fd);
 				return 1;
 			}
 			if (p->_pty) {
@@ -826,11 +910,7 @@ R_API int r_run_config_env(RRunProfile *p) {
 				is_child = true;
 
 				if (p->_dofork && !p->_dodebug) {
-#ifdef _MSC_VER
-					int child_pid = r_sys_fork ();
-#else
 					pid_t child_pid = r_sys_fork ();
-#endif
 					if (child_pid == -1) {
 						eprintf("rarun2: cannot fork\n");
 						r_socket_free (child);
@@ -869,7 +949,7 @@ R_API int r_run_config_env(RRunProfile *p) {
 	if (p->_r2sleep != 0) {
 		r_sys_sleep (p->_r2sleep);
 	}
-#if __UNIX__
+#if __UNIX__ && !__wasi__
 	if (p->_chroot) {
 		if (chdir (p->_chroot) == -1) {
 			eprintf ("Cannot chdir to chroot in %s\n", p->_chroot);
@@ -912,7 +992,7 @@ R_API int r_run_config_env(RRunProfile *p) {
 		}
 	}
 #endif
-#if __UNIX__
+#if __UNIX__ && !__wasi__
 	if (p->_setuid) {
 		ret = setgroups (0, NULL);
 		if (ret < 0) {
@@ -930,7 +1010,11 @@ R_API int r_run_config_env(RRunProfile *p) {
 		}
 	}
 	if (p->_setgid) {
+#if __wasi__
+		ret = 0;
+#else
 		ret = setgid (atoi (p->_setgid));
+#endif
 		if (ret < 0) {
 			return 1;
 		}
@@ -940,7 +1024,9 @@ R_API int r_run_config_env(RRunProfile *p) {
 		int f2[2];
 		if (pipe (f2) != -1) {
 			close (0);
+#if !__wasi__
 			dup2 (f2[0], 0);
+#endif
 		} else {
 			eprintf ("[ERROR] rarun2: Cannot create pipe\n");
 			return 1;
@@ -960,7 +1046,7 @@ R_API int r_run_config_env(RRunProfile *p) {
 #endif
 	if (p->_r2preload) {
 		if (p->_preload) {
-			eprintf ("WARNING: Only one library can be opened at a time\n");
+			eprintf ("Warning: Only one library can be opened at a time\n");
 		}
 #ifdef __WINDOWS__
 		p->_preload = r_str_r2_prefix (R_JOIN_2_PATHS (R2_LIBDIR, "libr2."R_LIB_EXT));
@@ -972,7 +1058,11 @@ R_API int r_run_config_env(RRunProfile *p) {
 #if __WINDOWS__
 		eprintf ("rarun2: libpath unsupported for this platform\n");
 #elif __HAIKU__
-		r_sys_setenv ("LIBRARY_PATH", p->_libpath);
+		char *orig = r_sys_getenv ("LIBRARY_PATH");
+		char *newlib = r_str_newf ("%s:%s", p->_libpath, orig);
+		r_sys_setenv ("LIBRARY_PATH", newlib);
+		free (newlib);
+		free (orig);
 #elif __APPLE__
 		r_sys_setenv ("DYLD_LIBRARY_PATH", p->_libpath);
 #else
@@ -993,8 +1083,10 @@ R_API int r_run_config_env(RRunProfile *p) {
 #endif
 	}
 	if (p->_timeout) {
-#if __UNIX__
-		int mypid = getpid ();
+#if __wasi__
+		// do nothing
+#elif __UNIX__
+		int mypid = r_sys_getpid ();
 		if (!r_sys_fork ()) {
 			int use_signal = p->_timeout_sig;
 			if (use_signal < 1) {
@@ -1002,18 +1094,23 @@ R_API int r_run_config_env(RRunProfile *p) {
 			}
 			sleep (p->_timeout);
 			if (!kill (mypid, 0)) {
-				eprintf ("\nrarun2: Interrupted by timeout\n");
+				// eprintf ("\nrarun2: Interrupted by timeout\n");
 			}
 			kill (mypid, use_signal);
 			exit (0);
 		}
 #else
-		eprintf ("timeout not supported for this platform\n");
+		if (p->_timeout_sig < 1 || p->_timeout_sig == 9) {
+			r_th_new (exit_process, NULL, p->_timeout);
+		} else {
+			eprintf ("timeout with signal not supported for this platform\n");
+		}
 #endif
 	}
 	return 0;
 }
 
+// NOTE: return value is like in unix return code (0 = ok, 1 = not ok)
 R_API int r_run_start(RRunProfile *p) {
 #if LIBC_HAVE_FORK
 	if (p->_execve) {
@@ -1071,7 +1168,69 @@ R_API int r_run_start(RRunProfile *p) {
 		if (p->_pid) {
 			eprintf ("PID: Cannot determine pid with 'system' directive. Use 'program'.\n");
 		}
-		exit (r_sys_cmd (p->_system));
+		if (p->_daemon) {
+#if __WINDOWS__
+	//		eprintf ("PID: Cannot determine pid with 'system' directive. Use 'program'.\n");
+#else
+			pid_t child = r_sys_fork ();
+			if (child == -1) {
+				perror ("fork");
+				exit (1);
+			}
+			if (child) {
+				if (p->_pidfile) {
+					char pidstr[32];
+					snprintf (pidstr, sizeof (pidstr), "%d\n", child);
+					r_file_dump (p->_pidfile,
+							(const ut8*)pidstr,
+							strlen (pidstr), 0);
+				}
+				exit (0);
+			}
+#if !__wasi__
+			setsid ();
+#endif
+			if (p->_timeout) {
+#if __UNIX__
+				int mypid = r_sys_getpid ();
+				if (!r_sys_fork ()) {
+					int use_signal = p->_timeout_sig;
+					if (use_signal < 1) {
+						use_signal = SIGKILL;
+					}
+					sleep (p->_timeout);
+#if !__wasi__
+					if (!kill (mypid, 0)) {
+						// eprintf ("\nrarun2: Interrupted by timeout\n");
+					}
+					kill (mypid, use_signal);
+#endif
+					exit (0);
+				}
+#else
+				eprintf ("timeout not supported for this platform\n");
+#endif
+			}
+#endif
+#if __UNIX__ && !__wasi__
+			close (0);
+			close (1);
+			char *bin_sh = r_file_binsh ();
+			if (bin_sh) {
+				exit (execl (bin_sh, bin_sh, "-c", p->_system, NULL));
+			} else {
+				exit (r_sys_cmd (p->_system));
+			}
+			free (bin_sh);
+#else
+			exit (r_sys_cmd (p->_system));
+#endif
+		} else {
+			if (p->_pidfile) {
+				eprintf ("Warning: pidfile doesnt work with 'system'.\n");
+			}
+			exit (r_sys_cmd (p->_system));
+		}
 	}
 	if (p->_program) {
 		if (!r_file_exists (p->_program)) {
@@ -1088,23 +1247,27 @@ R_API int r_run_start(RRunProfile *p) {
 #if __UNIX__
 		// XXX HACK close all non-tty fds
 		{ int i;
-			for (i = 3; i < 10; i++) {
+			for (i = 3; i < 1024; i++) {
 				close (i);
 			}
 		}
 		// TODO: use posix_spawn
 		if (p->_setgid) {
+#if __wasi__
+			int ret = -1;
+#else
 			int ret = setgid (atoi (p->_setgid));
+#endif
 			if (ret < 0) {
 				return 1;
 			}
 		}
 		if (p->_pid) {
-			eprintf ("PID: %d\n", getpid ());
+			eprintf ("PID: %d\n", r_sys_getpid ());
 		}
 		if (p->_pidfile) {
 			char pidstr[32];
-			snprintf (pidstr, sizeof (pidstr), "%d\n", getpid ());
+			snprintf (pidstr, sizeof (pidstr), "%d\n", r_sys_getpid ());
 			r_file_dump (p->_pidfile,
 				(const ut8*)pidstr,
 				strlen (pidstr), 0);
@@ -1112,12 +1275,39 @@ R_API int r_run_start(RRunProfile *p) {
 #endif
 
 		if (p->_nice) {
-#if __UNIX__ && !defined(__HAIKU__)
+#if __UNIX__ && !defined(__HAIKU__) && !defined(__serenity__) && !__wasi__
 			if (nice (p->_nice) == -1) {
 				return 1;
 			}
 #else
 			eprintf ("nice not supported for this platform\n");
+#endif
+		}
+		if (p->_daemon) {
+#if __WINDOWS__
+			eprintf ("PID: Cannot determine pid with 'system' directive. Use 'program'.\n");
+#else
+			pid_t child = r_sys_fork ();
+			if (child == -1) {
+				perror ("fork");
+				exit (1);
+			}
+			if (child) {
+				if (p->_pidfile) {
+					char pidstr[32];
+					snprintf (pidstr, sizeof (pidstr), "%d\n", child);
+					r_file_dump (p->_pidfile,
+							(const ut8*)pidstr,
+							strlen (pidstr), 0);
+					exit (0);
+				}
+			}
+#if !__wasi__
+			setsid ();
+#if !LIBC_HAVE_FORK
+			exit (execv (p->_program, (char* const*)p->_args));
+#endif
+#endif
 #endif
 		}
 // TODO: must be HAVE_EXECVE
@@ -1188,3 +1378,25 @@ R_API int r_run_start(RRunProfile *p) {
 	}
 	return 0;
 }
+
+R_API char *r_run_get_environ_profile(char **env) {
+	if (!env) {
+		return NULL;
+	}
+	RStrBuf *sb = r_strbuf_new (NULL);
+	while (*env) {
+		char *k = strdup (*env);
+		char *v = strchr (k, '=');
+		if (v) {
+			*v++ = 0;
+			v = r_str_escape_latin1 (v, false, true, true);
+			if (v) {
+				r_strbuf_appendf (sb, "setenv=%s=\"%s\"\n", k, v);
+				free (v);
+			}
+		}
+		free (k);
+		env++;
+	}
+	return r_strbuf_drain (sb);
+}
