@@ -1,5 +1,5 @@
 #!/bin/sh
-echo "_=0x80"
+#echo "_=0x80"
 awk -F '(=|,)' '{
 	# 0x80.1=exit
 	if ($1 == "_") {
