@@ -1,11 +1,10 @@
-/* radare - Copyright 2008-2018 - LGPL -- pancake */
+/* radare - Copyright 2008-2021 - LGPL -- pancake */
 
 #include <r_types.h>
 #include <r_util.h>
 #include <r_syscall.h>
 #include <stdio.h>
 #include <string.h>
-#include "fastcall.h"
 
 R_LIB_VERSION (r_syscall);
 
@@ -18,11 +17,10 @@ R_API RSyscall* r_syscall_ref(RSyscall *sc) {
 	return sc;
 }
 
-R_API RSyscall* r_syscall_new() {
+R_API RSyscall* r_syscall_new(void) {
 	RSyscall *rs = R_NEW0 (RSyscall);
 	if (rs) {
 		rs->sysport = sysport_x86;
-		rs->regs = fastcall_x86_32;
 		rs->srdb = sdb_new0 (); // sysregs database
 		rs->db = sdb_new0 ();
 	}
@@ -44,26 +42,28 @@ R_API void r_syscall_free(RSyscall *s) {
 	}
 }
 
-/* return fastcall register argument 'idx' for a syscall with 'num' args */
-R_API const char *r_syscall_reg(RSyscall *s, int idx, int num) {
-	if (num < 0 || num >= R_SYSCALL_ARGS || idx < 0 || idx >= R_SYSCALL_ARGS) {
-		return NULL;
-	}
-	return s->regs[num].arg[idx];
-}
+R_API SdbGperf *r_syscall_get_gperf(const char *k);
 
 static Sdb *openDatabase(Sdb *db, const char *name) {
-	char *file = r_str_newf ( R_JOIN_3_PATHS ("%s", R2_SDB, "%s.sdb"),
-		r_sys_prefix (NULL), name);
-	if (r_file_exists (file)) {
-		if (db) {
-			sdb_reset (db);
-			sdb_open (db, file);
+	SdbGperf *sg = r_syscall_get_gperf (name);
+	if (sg) {
+		sdb_reset (db);
+		sdb_open_gperf (db, sg);
+	} else {
+		char *file = r_str_newf (R_JOIN_3_PATHS ("%s", R2_SDB, "%s.sdb"), r_sys_prefix (NULL), name);
+		if (r_file_exists (file)) {
+			if (db) {
+				sdb_reset (db);
+				sdb_open (db, file);
+			} else {
+				db = sdb_new (0, file, 0);
+			}
 		} else {
-			db = sdb_new (0, file, 0);
+			sdb_free (db);
+			db = sdb_new0 ();
 		}
+		free (file);
 	}
-	free (file);
 	return db;
 }
 
@@ -89,6 +89,7 @@ static inline bool sysregs_reload_needed(RSyscall *s, const char *arch, int bits
 
 // TODO: should be renamed to r_syscall_use();
 R_API bool r_syscall_setup(RSyscall *s, const char *arch, int bits, const char *cpu, const char *os) {
+	r_return_val_if_fail (s, false);
 	bool syscall_changed, sysregs_changed;
 
 	if (!os || !*os) {
@@ -117,38 +118,12 @@ R_API bool r_syscall_setup(RSyscall *s, const char *arch, int bits, const char *
 	if (!strcmp (os, "any")) { // ignored
 		return true;
 	}
-	if (!strcmp (arch, "mips")) {
-		s->regs = fastcall_mips;
-	} else if (!strcmp (arch, "avr")) {
+	if (!strcmp (arch, "avr")) {
 		s->sysport = sysport_avr;
 	} else if (!strcmp (os, "darwin") || !strcmp (os, "osx") || !strcmp (os, "macos")) {
 		os = "darwin";
-		s->regs = fastcall_x86_64;
-	} else if (!strcmp (arch,"sh")) {
-		s->regs = fastcall_sh;
-	} else if (!strcmp (arch, "arm")) {
-		switch (bits) {
-		case 16:
-		case 32:
-			s->regs = fastcall_arm;
-			break;
-		case 64:
-			s->regs = fastcall_arm64;
-			break;
-		}
 	} else if (!strcmp (arch, "x86")) {
 		s->sysport = sysport_x86;
-		switch (bits) {
-		case 8:
-			s->regs = fastcall_x86_8;
-			break;
-		case 32:
-			s->regs = fastcall_x86_32;
-			break;
-		case 64:
-			s->regs = fastcall_x86_64;
-			break;
-		}
 	}
 
 	if (syscall_changed) {
@@ -233,26 +208,26 @@ R_API int r_syscall_get_swi(RSyscall *s) {
 
 R_API RSyscallItem *r_syscall_get(RSyscall *s, int num, int swi) {
 	r_return_val_if_fail (s && s->db, NULL);
-	const char *ret, *ret2, *key;
+	char key[128];
 	swi = getswi (s, swi);
 	if (swi < 16) {
-		key = sdb_fmt ("%d.%d", swi, num);
+		snprintf (key, sizeof (key), "%d.%d", swi, num);
 	} else {
-		key = sdb_fmt ("0x%02x.%d", swi, num);
+		snprintf (key, sizeof (key), "0x%02x.%d", swi, num);
 	}
-	ret = sdb_const_get (s->db, key, 0);
+	const char *ret = sdb_const_get (s->db, key, 0);
 	if (!ret) {
-		key = sdb_fmt ("0x%02x.0x%02x", swi, num); // Workaround until Syscall SDB is fixed
+		snprintf (key, sizeof (key), "0x%02x.0x%02x", swi, num); // Workaround until Syscall SDB is fixed
 		ret = sdb_const_get (s->db, key, 0);
 		if (!ret) {
-			key = sdb_fmt ("0x%02x.%d", num, swi); // Workaround until Syscall SDB is fixed
+			snprintf (key, sizeof (key), "0x%02x.%d", num, swi); // Workaround until Syscall SDB is fixed
 			ret = sdb_const_get (s->db, key, 0);
 			if (!ret) {
 				return NULL;
 			}
 		}
 	}
-	ret2 = sdb_const_get (s->db, ret, 0);
+	const char *ret2 = sdb_const_get (s->db, ret, 0);
 	if (!ret2) {
 		return NULL;
 	}
@@ -276,12 +251,12 @@ R_API const char *r_syscall_get_i(RSyscall *s, int num, int swi) {
 	return sdb_const_get (s->db, foo, 0);
 }
 
-static int callback_list(void *u, const char *k, const char *v) {
+static bool callback_list(void *u, const char *k, const char *v) {
 	RList *list = (RList*)u;
 	if (!strchr (k, '.')) {
 		RSyscallItem *si = r_syscall_item_new_from_string (k, v);
 		if (!si) {
-			return 1;
+			return true;
 		}
 		if (!strchr (si->name, '.')) {
 			r_list_append (list, si);
@@ -289,7 +264,7 @@ static int callback_list(void *u, const char *k, const char *v) {
 			r_syscall_item_free (si);
 		}
 	}
-	return 1; // continue loop
+	return true; // continue loop
 }
 
 R_API RList *r_syscall_list(RSyscall *s) {
