@@ -25,7 +25,6 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <getopt.h>
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
@@ -118,7 +117,7 @@ R_API bool r_base85_decode(FILE *fp, int delims, int ignore_garbage) {
 			}
 			ungetc (c, fp);
 		} else if (c == EOF) {
-			eprint ("ascii85: missing <~");
+			eprintf ("ascii85: missing <~");
 			return false;
 		}
 	}
@@ -135,7 +134,7 @@ R_API bool r_base85_decode(FILE *fp, int delims, int ignore_garbage) {
 		if (c == '~' && delims) {
 			c = getc_nospace (fp);
 			if (c != '>') {
-				eprint ("ascii85: ~ without >\n");
+				eprintf ("ascii85: ~ without >\n");
 				return false;
 			}
 			c = EOF;
@@ -143,7 +142,7 @@ R_API bool r_base85_decode(FILE *fp, int delims, int ignore_garbage) {
 		}
 		if (c == EOF) {
 			if (delims && !end) {
-				eprint ("ascii85: missing ~>");
+				eprintf ("ascii85: missing ~>");
 				return false;
 			}
 			if (count > 0) {
