@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - ret2libc, pancake */
 
 #include <r_types.h>
 #include <r_util.h>
@@ -25,6 +25,12 @@ static bool buf_init(RBuffer *b, const void *user) {
 	return b->methods->init? b->methods->init (b, user): true;
 }
 
+static void buf_wholefree(RBuffer *b) {
+	if (!b->methods->get_whole_buf) {
+		R_FREE (b->whole_buf);
+	}
+}
+
 static bool buf_fini(RBuffer *b) {
 	r_return_val_if_fail (b && b->methods, false);
 	return b->methods->fini? b->methods->fini (b): true;
@@ -42,6 +48,7 @@ static st64 buf_read(RBuffer *b, ut8 *buf, size_t len) {
 
 static st64 buf_write(RBuffer *b, const ut8 *buf, size_t len) {
 	r_return_val_if_fail (b && b->methods, -1);
+	buf_wholefree (b);
 	return b->methods->write? b->methods->write (b, buf, len): -1;
 }
 
@@ -190,7 +197,7 @@ R_API RBuffer *r_buf_new(void) {
 	return new_buffer (R_BUFFER_BYTES, &u);
 }
 
-R_API const ut8 *r_buf_data(RBuffer *b, ut64 *size) {
+R_DEPRECATE R_API const ut8 *r_buf_data(RBuffer *b, ut64 *size) {
 	r_return_val_if_fail (b, NULL);
 	b->whole_buf = get_whole_buf (b, size);
 	return b->whole_buf;
@@ -220,7 +227,7 @@ R_API RBuffer *r_buf_new_file(const char *file, int perm, int mode) {
 
 // TODO: rename to new_from_file ?
 R_API RBuffer *r_buf_new_slurp(const char *file) {
-	int len;
+	size_t len;
 	char *tmp = r_file_slurp (file, &len);
 	if (!tmp) {
 		return NULL;
@@ -228,7 +235,7 @@ R_API RBuffer *r_buf_new_slurp(const char *file) {
 
 	struct buf_bytes_user u = { 0 };
 	u.data_steal = (ut8 *)tmp;
-	u.length = len;
+	u.length = (ut64)len;
 	u.steal = true;
 	return new_buffer (R_BUFFER_BYTES, &u);
 }
@@ -348,6 +355,11 @@ restore_pos:
 	return r;
 }
 
+R_API bool r_buf_append_ut8(RBuffer *b, ut8 n) {
+	r_return_val_if_fail (b && !b->readonly, false);
+	return r_buf_append_bytes (b, (const ut8 *)&n, sizeof (n));
+}
+
 R_API bool r_buf_append_ut16(RBuffer *b, ut16 n) {
 	r_return_val_if_fail (b && !b->readonly, false);
 	return r_buf_append_bytes (b, (const ut8 *)&n, sizeof (n));
@@ -400,7 +412,6 @@ R_API char *r_buf_get_string(RBuffer *b, ut64 addr) {
 	while (r > 0) {
 		const ut8 *needle = r_mem_mem (res + sz, r, (ut8 *)"\x00", 1);
 		if (needle) {
-			sz += (needle - (res + sz));
 			null_found = true;
 			break;
 		}
@@ -422,9 +433,24 @@ R_API char *r_buf_get_string(RBuffer *b, ut64 addr) {
 	return (char *)res;
 }
 
+R_API ut8 *r_buf_read_all(RBuffer *b, int *blen) {
+	r_return_val_if_fail (b, NULL);
+	int buflen = r_buf_size (b);
+	if (buflen < 0) {
+		return NULL;
+	}
+	ut8 *buf = malloc (buflen + 1);
+	buf_seek (b, 0, R_BUF_SET);
+	buf_read (b, buf, buflen);
+	if (blen) {
+		*blen = buflen;
+	}
+	return buf;
+}
+
 R_API st64 r_buf_read(RBuffer *b, ut8 *buf, ut64 len) {
 	r_return_val_if_fail (b && buf, -1);
-	ut64 r = buf_read (b, buf, len);
+	st64 r = buf_read (b, buf, len);
 	if (r >= 0 && r < len) {
 		memset (buf + r, b->Oxff_priv, len - r);
 	}
@@ -518,9 +544,7 @@ static st64 buf_format(RBuffer *dst, RBuffer *src, const char *fmt, int n) {
 				}
 				res += r;
 			}
-
 			m = 1;
-			bigendian = true;
 		}
 	}
 	return res;
@@ -537,11 +561,14 @@ R_API st64 r_buf_fread(RBuffer *b, ut8 *buf, const char *fmt, int n) {
 
 R_API st64 r_buf_fread_at(RBuffer *b, ut64 addr, ut8 *buf, const char *fmt, int n) {
 	r_return_val_if_fail (b && buf && fmt, -1);
-	int r = r_buf_seek (b, addr, R_BUF_SET);
+	st64 o_addr = r_buf_seek (b, 0, R_BUF_CUR);
+	st64 r = r_buf_seek (b, addr, R_BUF_SET);
 	if (r < 0) {
 		return r;
 	}
-	return r_buf_fread (b, buf, fmt, n);
+	r = r_buf_fread (b, buf, fmt, n);
+	r_buf_seek (b, o_addr, R_BUF_SET);
+	return r;
 }
 
 R_API st64 r_buf_fwrite(RBuffer *b, const ut8 *buf, const char *fmt, int n) {
@@ -555,40 +582,49 @@ R_API st64 r_buf_fwrite(RBuffer *b, const ut8 *buf, const char *fmt, int n) {
 
 R_API st64 r_buf_fwrite_at(RBuffer *b, ut64 addr, const ut8 *buf, const char *fmt, int n) {
 	r_return_val_if_fail (b && buf && fmt && !b->readonly, -1);
+	st64 o_addr = r_buf_seek (b, 0, R_BUF_CUR);
 	st64 r = r_buf_seek (b, addr, R_BUF_SET);
 	if (r < 0) {
 		return r;
 	}
-	return r_buf_fwrite (b, buf, fmt, n);
+	r = r_buf_fwrite (b, buf, fmt, n);
+	r_buf_seek (b, o_addr, R_BUF_SET);
+	return r;
 }
 
 R_API st64 r_buf_read_at(RBuffer *b, ut64 addr, ut8 *buf, ut64 len) {
 	r_return_val_if_fail (b && buf, -1);
+	st64 o_addr = r_buf_seek (b, 0, R_BUF_CUR);
 	st64 r = r_buf_seek (b, addr, R_BUF_SET);
 	if (r < 0) {
 		return r;
 	}
 
-	return r_buf_read (b, buf, len);
+	r = r_buf_read (b, buf, len);
+	r_buf_seek (b, o_addr, R_BUF_SET);
+	return r;
 }
 
 R_API st64 r_buf_write_at(RBuffer *b, ut64 addr, const ut8 *buf, ut64 len) {
 	r_return_val_if_fail (b && buf && !b->readonly, -1);
+	st64 o_addr = r_buf_seek (b, 0, R_BUF_CUR);
 	st64 r = r_buf_seek (b, addr, R_BUF_SET);
 	if (r < 0) {
 		return r;
 	}
 
-	return r_buf_write (b, buf, len);
+	r = r_buf_write (b, buf, len);
+	r_buf_seek (b, o_addr, R_BUF_SET);
+	return r;
 }
 
-R_API bool r_buf_fini(RBuffer *b) {
+R_API void r_buf_fini(RBuffer *b) {
 	if (!b) {
-		return false;
+		return;
 	}
 	if (b->refctr > 0) {
 		b->refctr--;
-		return false;
+		return;
 	}
 
 	// free the whole_buf only if it was initially allocated by the buf types
@@ -597,13 +633,15 @@ R_API bool r_buf_fini(RBuffer *b) {
 			b->methods->free_whole_buf (b);
 		}
 	} else {
-		R_FREE (b->whole_buf);
+		buf_wholefree (b);
 	}
-	return buf_fini (b);
+	buf_fini (b);
 }
 
 R_API void r_buf_free(RBuffer *b) {
-	if (r_buf_fini (b)) {
+	bool unreferenced = b && b->refctr == 0;
+	r_buf_fini (b);
+	if (unreferenced) {
 		free (b);
 	}
 }
@@ -635,12 +673,16 @@ R_API st64 r_buf_uleb128(RBuffer *b, ut64 *v) {
 	do {
 		ut8 data;
 		st64 r = r_buf_read (b, &data, sizeof (data));
-		if (r <= 0) {
+		if (r < 1) {
 			return -1;
 		}
 		c = data & 0xff;
-		sum |= ((ut64) (c & 0x7f) << s);
-		s += 7;
+		if (s < 64) {
+			sum |= ((ut64) (c & 0x7f) << s);
+			s += 7;
+		} else {
+			sum = 0;
+		}
 		l++;
 	} while (c & 0x80);
 	if (v) {
@@ -659,12 +701,18 @@ R_API st64 r_buf_sleb128(RBuffer *b, st64 *v) {
 			return -1;
 		}
 		chunk = value & 0x7f;
-		result |= (chunk << offset);
-		offset += 7;
+		if (offset < 64) {
+			result |= (chunk << offset);
+			offset += 7;
+		} else {
+			result = 0;
+		}
 	} while (value & 0x80);
 
 	if ((value & 0x40) != 0) {
-		result |= ~0ULL << offset;
+		if (offset < 64) {
+			result |= ~0ULL << offset;
+		}
 	}
 	if (v) {
 		*v = result;
