@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2019 - pancake */
+/* radare - LGPL - Copyright 2007-2021 - pancake */
 
 #include "r_types.h"
 #include "r_util.h"
@@ -25,11 +25,16 @@
 #include <process.h>
 #endif
 
+#if __UNIX__ && !defined(__serenity__)
+#define RDWR_FLAGS O_RDWR | O_SYNC
+#else
+#define RDWR_FLAGS O_RDWR
+#endif
 
-static int file_stat (const char *file, struct stat* const pStat) {
-	if (!file || !pStat) {
-		return -1;
-	}
+#define BS 1024
+
+static int file_stat(const char *file, struct stat* const pStat) {
+	r_return_val_if_fail (file && pStat, -1);
 #if __WINDOWS__
 	wchar_t *wfile = r_utf8_to_utf16 (file);
 	if (!wfile) {
@@ -43,7 +48,41 @@ static int file_stat (const char *file, struct stat* const pStat) {
 #endif // __WINDOWS__
 }
 
-R_API bool r_file_truncate (const char *filename, ut64 newsize) {
+// r_file_new("", "bin", NULL) -> /bin
+// r_file_new(".", "bin", NULL) -> ./bin
+// r_file_new("/", "bin", NULL) -> //bin # shall we be stricts?
+R_API char *r_file_new(const char *root, ...) {
+	va_list ap;
+	va_start (ap, root);
+	RStrBuf *sb = r_strbuf_new ("");
+	if (!strcmp (root, "~")) {
+		char *h = r_str_home (NULL);
+		if (!h) {
+			va_end (ap);
+			r_strbuf_free (sb);
+			return NULL;
+		}
+		r_strbuf_append (sb, h);
+		free (h);
+	} else {
+		r_strbuf_append (sb, root);
+	}
+	r_strbuf_append (sb, R_SYS_DIR);
+	const char *arg = va_arg (ap, char *);
+	while (arg) {
+		r_strbuf_append (sb, R_SYS_DIR);
+		r_strbuf_append (sb, arg);
+		arg = va_arg (ap, char *);
+	}
+	va_end (ap);
+	char *path = r_strbuf_drain (sb);
+	char *abs = r_file_abspath (path);
+	free (path);
+	return abs;
+}
+
+R_API bool r_file_truncate(const char *filename, ut64 newsize) {
+	r_return_val_if_fail (filename, false);
 	int fd;
 	if (r_file_is_directory (filename)) {
 		return false;
@@ -51,15 +90,11 @@ R_API bool r_file_truncate (const char *filename, ut64 newsize) {
 	if (!r_file_exists (filename) || !r_file_is_regular (filename)) {
 		return false;
 	}
-#if __WINDOWS__
-	fd = r_sandbox_open (filename, O_RDWR, 0644);
-#else
-	fd = r_sandbox_open (filename, O_RDWR | O_SYNC, 0644);
-#endif
+	fd = r_sandbox_open (filename, RDWR_FLAGS, 0644);
 	if (fd == -1) {
 		return false;
 	}
-#ifdef _MSC_VER
+#if defined(_MSC_VER) || __WINDOWS__
 	int r = _chsize (fd, newsize);
 #else
 	int r = ftruncate (fd, newsize);
@@ -78,7 +113,8 @@ Example:
 	str = r_file_basename ("home/inisider/Downloads/user32.dll");
 	// str == user32.dll
 */
-R_API const char *r_file_basename (const char *path) {
+R_API const char *r_file_basename(const char *path) {
+	r_return_val_if_fail (path, NULL);
 	const char *ptr = r_str_rchr (path, NULL, '/');
 	if (ptr) {
 		path = ptr + 1;
@@ -96,10 +132,8 @@ Example:
 	// str == "home/inisider/Downloads"
 	free (str);
 */
-R_API char *r_file_dirname (const char *path) {
-	if (!path) {
-		return NULL;
-	}
+R_API char *r_file_dirname(const char *path) {
+	r_return_val_if_fail (path, NULL);
 	char *newpath = strdup (path);
 	char *ptr = (char*)r_str_rchr (newpath, NULL, '/');
 	if (ptr) {
@@ -114,6 +148,7 @@ R_API char *r_file_dirname (const char *path) {
 }
 
 R_API bool r_file_is_c(const char *file) {
+	r_return_val_if_fail (file, false);
 	const char *ext = r_str_lchr (file, '.'); // TODO: add api in r_file_extension or r_str_ext for this
 	if (ext) {
 		ext++;
@@ -136,9 +171,7 @@ R_API bool r_file_is_regular(const char *str) {
 
 R_API bool r_file_is_directory(const char *str) {
 	struct stat buf = {0};
-	if (!str || !*str) {
-		return false;
-	}
+	r_return_val_if_fail (!R_STR_ISEMPTY (str), false);
 	if (file_stat (str, &buf) == -1) {
 		return false;
 	}
@@ -152,7 +185,7 @@ R_API bool r_file_is_directory(const char *str) {
 
 R_API bool r_file_fexists(const char *fmt, ...) {
 	int ret;
-	char string[1024];
+	char string[BS];
 	va_list ap;
 	va_start (ap, fmt);
 	vsnprintf (string, sizeof (string), fmt, ap);
@@ -162,34 +195,19 @@ R_API bool r_file_fexists(const char *fmt, ...) {
 }
 
 R_API bool r_file_exists(const char *str) {
+	char *absfile = r_file_abspath (str);
 	struct stat buf = {0};
-	if (!str || !*str) {
-		return false;
-	}
-#if 0
-	// TODO: file_exists doesnt uses the sandbox or many things may fail
-	if (strncmp (str, "/usr/bin", 8)) {
-		if (str && !r_sandbox_check_path (str)) {
-			return false;
-		}
-	}
-#endif
-
-	if (file_stat (str, &buf) == -1) {
+	r_return_val_if_fail (!R_STR_ISEMPTY (str), false);
+	if (file_stat (absfile, &buf) == -1) {
+		free (absfile);
 		return false;
 	}
+	free (absfile);
 	return S_IFREG == (S_IFREG & buf.st_mode);
 }
 
-R_API long r_file_proc_size(FILE *fd) {
-	long size = 0;
-	while (fgetc (fd) != EOF) {
-		size++;
-	}
-	return size;
-}
-
 R_API ut64 r_file_size(const char *str) {
+	r_return_val_if_fail (!R_STR_ISEMPTY (str), 0);
 	struct stat buf = {0};
 	if (file_stat (str, &buf) == -1) {
 		return 0;
@@ -197,19 +215,19 @@ R_API ut64 r_file_size(const char *str) {
 	return (ut64)buf.st_size;
 }
 
-R_API int r_file_is_abspath(const char *file) {
+R_API bool r_file_is_abspath(const char *file) {
+	r_return_val_if_fail (!R_STR_ISEMPTY (file), 0);
 	return ((*file && file[1]==':') || *file == '/');
 }
 
-R_API char *r_file_abspath(const char *file) {
-	char *cwd, *ret = NULL;
-	if (!file || !strcmp (file, ".") || !strcmp (file, "./")) {
+R_API char *r_file_abspath_rel(const char *cwd, const char *file) {
+	char *ret = NULL;
+	if (!file || !*file || !strcmp (file, ".") || !strcmp (file, "./")) {
 		return r_sys_getdir ();
 	}
 	if (strstr (file, "://")) {
 		return strdup (file);
 	}
-	cwd = r_sys_getdir ();
 	if (!strncmp (file, "~/", 2) || !strncmp (file, "~\\", 2)) {
 		ret = r_str_home (file + 2);
 	} else {
@@ -228,7 +246,8 @@ R_API char *r_file_abspath(const char *file) {
 				PTCHAR f = r_sys_conv_utf8_to_win (file);
 				int s = GetFullPathName (f, MAX_PATH, abspath, NULL);
 				if (s > MAX_PATH) {
-					R_LOG_ERROR ("r_file_abspath/GetFullPathName: Path to file too long.\n");
+					// R_LOG_ERROR ("r_file_abspath/GetFullPathName: Path to file too long.\n");
+					eprintf ("r_file_abspath/GetFullPathName: Path to file too long.\n");
 				} else if (!s) {
 					r_sys_perror ("r_file_abspath/GetFullPathName");
 				} else {
@@ -240,11 +259,10 @@ R_API char *r_file_abspath(const char *file) {
 		}
 #endif
 	}
-	free (cwd);
 	if (!ret) {
 		ret = strdup (file);
 	}
-#if __UNIX__
+#if __UNIX__ && !__wasi__
 	char *abspath = realpath (ret, NULL);
 	if (abspath) {
 		free (ret);
@@ -254,22 +272,41 @@ R_API char *r_file_abspath(const char *file) {
 	return ret;
 }
 
+R_API char *r_file_abspath(const char *file) {
+	r_return_val_if_fail (file, NULL);
+	char *cwd = r_sys_getdir ();
+	if (cwd) {
+		char *ret = r_file_abspath_rel (cwd, file);
+		free (cwd);
+		return ret;
+	}
+	return NULL;
+}
+
+R_API char *r_file_binsh(void) {
+	char *bin_sh = r_sys_getenv ("SHELL");
+	if (R_STR_ISEMPTY (bin_sh)) {
+		free (bin_sh);
+		bin_sh = r_file_path ("sh");
+		if (R_STR_ISEMPTY (bin_sh)) {
+			free (bin_sh);
+			bin_sh = strdup (SHELL_PATH);
+		}
+	}
+	return bin_sh;
+}
+
 R_API char *r_file_path(const char *bin) {
-	char *path_env;
+	r_return_val_if_fail (bin, NULL);
 	char *file = NULL;
 	char *path = NULL;
 	char *str, *ptr;
 	const char *extension = "";
-	if (!bin) {
-		return NULL;
-	}
 	if (!strncmp (bin, "./", 2)) {
-		if (r_file_exists (bin)) {
-			return r_file_abspath (bin);
-		}
-		return NULL;
+		return r_file_exists (bin)
+			? r_file_abspath (bin): NULL;
 	}
-	path_env = (char *)r_sys_getenv ("PATH");
+	char *path_env = (char *)r_sys_getenv ("PATH");
 #if __WINDOWS__
 	if (!r_str_endswith (bin, ".exe")) {
 		extension = ".exe";
@@ -297,9 +334,11 @@ R_API char *r_file_path(const char *bin) {
 	return strdup (bin);
 }
 
-R_API char *r_stdin_slurp (int *sz) {
-#define BS 1024
-#if __UNIX__ || __WINDOWS__
+R_API char *r_stdin_slurp(int *sz) {
+#if __wasi__
+#warning r_stdin_slurp not available for wasi
+	return NULL;
+#elif __UNIX__ || __WINDOWS__
 	int i, ret, newfd;
 	if ((newfd = dup (0)) < 0) {
 		return NULL;
@@ -313,7 +352,8 @@ R_API char *r_stdin_slurp (int *sz) {
 		char *new = realloc (buf, i + BS);
 		if (!new) {
 			eprintf ("Cannot realloc to %d\n", i+BS);
-			break;
+			free (buf);
+			return NULL;
 		}
 		buf = new;
 		ret = read (0, buf + i, BS);
@@ -342,74 +382,102 @@ R_API char *r_stdin_slurp (int *sz) {
 #endif
 }
 
-R_API char *r_file_slurp(const char *str, int *usz) {
-	size_t rsz;
-	char *ret;
-	FILE *fd;
-	long sz;
+R_API char *r_file_slurp(const char *str, R_NULLABLE size_t *usz) {
+	r_return_val_if_fail (str, NULL);
 	if (usz) {
 		*usz = 0;
 	}
 	if (!r_file_exists (str)) {
 		return NULL;
 	}
-	fd = r_sandbox_fopen (str, "rb");
+	FILE *fd = r_sandbox_fopen (str, "rb");
 	if (!fd) {
 		return NULL;
 	}
-
-	(void)fseek (fd, 0, SEEK_END);
-	sz = ftell (fd);
-	if (!sz) {
-		if (r_file_is_regular (str)) {
-			/* proc file */
-			fseek (fd, 0, SEEK_SET);
-			sz = r_file_proc_size (fd);
-			if (!sz) {
-				sz = -1;
-			}
-		} else {
-			sz = 65536;
-		}
+	if (fseek (fd, 0, SEEK_END) == -1) {
+		// cannot determine the size of the file
 	}
+	long sz = ftell (fd);
 	if (sz < 0) {
 		fclose (fd);
 		return NULL;
 	}
-	(void)fseek (fd, 0, SEEK_SET);
-	ret = (char *)calloc (sz + 1, 1);
+#if __wasi__
+	fclose (fd);
+	fd = r_sandbox_fopen (str, "rb");
+	if (!fd) {
+		return NULL;
+	}
+#endif
+	if (!sz) {
+		if (r_file_is_regular (str)) {
+			char *buf = NULL;
+			long size = 0;
+			(void)fseek (fd, 0, SEEK_SET);
+			do {
+				char *nbuf = realloc (buf, size + BS);
+				if (!nbuf) {
+					break;
+				}
+				buf = nbuf;
+				size_t r = fread (buf + size, 1, BS, fd);
+				if (ferror (fd)) {
+					R_FREE (buf);
+					goto regular_err;
+				}
+				size += r;
+			} while (!feof (fd));
+			char *nbuf = realloc (buf, size + 1);
+			if (!nbuf) {
+				free (buf);
+				return NULL;
+			}
+			buf = nbuf;
+			buf[size] = '\0';
+			if (usz) {
+				*usz = size;
+			}
+		regular_err:
+			fclose (fd);
+			return buf;
+		}
+		// try to read 64K
+		sz = UT16_MAX;
+	}
+	rewind (fd);
+	char *ret = (char *)calloc (sz + 1, 1);
 	if (!ret) {
 		fclose (fd);
 		return NULL;
 	}
-	rsz = fread (ret, 1, sz, fd);
+	size_t rsz = fread (ret, 1, sz, fd);
 	if (rsz != sz) {
-		// eprintf ("r_file_slurp: fread: error\n");
+		eprintf ("Warning: r_file_slurp: fread: truncated read (%d / %d)\n", (int)rsz, (int)sz);
 		sz = rsz;
 	}
 	fclose (fd);
 	ret[sz] = '\0';
 	if (usz) {
-		*usz = (int)sz;
+		*usz = sz;
 	}
 	return ret;
 }
 
 R_API ut8 *r_file_gzslurp(const char *str, int *outlen, int origonfail) {
-	int sz;
-	ut8 *in, *out;
+	r_return_val_if_fail (str, NULL);
 	if (outlen) {
 		*outlen = 0;
 	}
-	in = (ut8*)r_file_slurp (str, &sz);
+	size_t sz;
+	ut8 *in = (ut8*)r_file_slurp (str, &sz);
 	if (!in) {
 		return NULL;
 	}
-	out = r_inflate (in, sz, NULL, outlen);
+	ut8 *out = r_inflate (in, (int)sz, NULL, outlen);
 	if (!out && origonfail) {
 		// if uncompression fails, return orig buffer ?
 		if (outlen) {
-			*outlen = sz;
+			*outlen = (int)sz;
 		}
 		in[sz] = 0;
 		return in;
@@ -419,6 +487,10 @@ R_API ut8 *r_file_gzslurp(const char *str, int *outlen, int origonfail) {
 }
 
 R_API ut8 *r_file_slurp_hexpairs(const char *str, int *usz) {
+	r_return_val_if_fail (str, NULL);
+	if (usz) {
+		*usz = 0;
+	}
 	ut8 *ret;
 	long sz;
 	int c, bytes = 0;
@@ -487,16 +559,18 @@ R_API char *r_file_slurp_range(const char *str, ut64 off, int sz, int *osz) {
 }
 
 R_API char *r_file_slurp_random_line(const char *file) {
+	r_return_val_if_fail (file, NULL);
 	int i = 0;
 	return r_file_slurp_random_line_count (file, &i);
 }
 
 R_API char *r_file_slurp_random_line_count(const char *file, int *line) {
+	r_return_val_if_fail (file && line, NULL);
 	/* Reservoir Sampling */
 	char *ptr = NULL, *str;
-	int sz, i, lines, selection = -1;
+	size_t i, lines, selection = -1;
 	int start = *line;
-	if ((str = r_file_slurp (file, &sz))) {
+	if ((str = r_file_slurp (file, NULL))) {
 		r_num_irand ();
 		for (i = 0; str[i]; i++) {
 			if (str[i] == '\n') {
@@ -534,8 +608,9 @@ R_API char *r_file_slurp_random_line_count(const char *file, int *line) {
 }
 
 R_API char *r_file_slurp_line(const char *file, int line, int context) {
+	r_return_val_if_fail (file, NULL);
 	int i, lines = 0;
-	int sz;
+	size_t sz;
 	char *ptr = NULL, *str = r_file_slurp (file, &sz);
 	// TODO: Implement context
 	if (str) {
@@ -568,8 +643,9 @@ R_API char *r_file_slurp_line(const char *file, int line, int context) {
 }
 
 R_API char *r_file_slurp_lines_from_bottom(const char *file, int line) {
+	r_return_val_if_fail (file, NULL);
 	int i, lines = 0;
-	int sz;
+	size_t sz;
 	char *ptr = NULL, *str = r_file_slurp (file, &sz);
 	// TODO: Implement context
 	if (str) {
@@ -595,8 +671,9 @@ R_API char *r_file_slurp_lines_from_bottom(const char *file, int line) {
 }
 
 R_API char *r_file_slurp_lines(const char *file, int line, int count) {
+	r_return_val_if_fail (file, NULL);
 	int i, lines = 0;
-	int sz;
+	size_t sz;
 	char *ptr = NULL, *str = r_file_slurp (file, &sz);
 	// TODO: Implement context
 	if (str) {
@@ -633,6 +710,7 @@ R_API char *r_file_slurp_lines(const char *file, int line, int count) {
 }
 
 R_API char *r_file_root(const char *root, const char *path) {
+	r_return_val_if_fail (root && path, NULL);
 	char *ret, *s = r_str_replace (strdup (path), "..", "", 1);
 	// XXX ugly hack
 	while (strstr (s, "..")) {
@@ -690,15 +768,13 @@ R_API bool r_file_hexdump(const char *file, const ut8 *buf, int len, int append)
 }
 
 R_API bool r_file_touch(const char *file) {
-	return r_file_dump(file, NULL, 0, true);
+	r_return_val_if_fail (file, false);
+	return r_file_dump (file, NULL, 0, true);
 }
 
 R_API bool r_file_dump(const char *file, const ut8 *buf, int len, bool append) {
+	r_return_val_if_fail (!R_STR_ISEMPTY (file), false);
 	FILE *fd;
-	if (!file || !*file) {
-		eprintf ("r_file_dump file: %s buf: %p\n", file, buf);
-		return false;
-	}
 	if (append) {
 		fd = r_sandbox_fopen (file, "ab");
 	} else {
@@ -723,7 +799,31 @@ R_API bool r_file_dump(const char *file, const ut8 *buf, int len, bool append) {
 	return true;
 }
 
+R_API bool r_file_move(const char *src, const char *dst) {
+	r_return_val_if_fail (!R_STR_ISEMPTY (src) && !R_STR_ISEMPTY (dst), false);
+	if (r_sandbox_enable (0)) {
+		return false;
+	}
+	// rename fails when files are in different mountpoints
+	// in this situation it needs to be copied and removed
+	if (rename (src, dst) != 0) {
+		char *a = r_str_escape (src);
+		char *b = r_str_escape (dst);
+		char *input = r_str_newf ("\"%s\" \"%s\"", a, b);
+#if __WINDOWS__
+		int rc = r_sys_cmdf ("move %s", input);
+#else
+		int rc = r_sys_cmdf ("mv %s", input);
+#endif
+		free (a);
+		free (b);
+		return rc == 0;
+	}
+	return true;
+}
+
 R_API bool r_file_rm(const char *file) {
+	r_return_val_if_fail (!R_STR_ISEMPTY (file), false);
 	if (r_sandbox_enable (0)) {
 		return false;
 	}
@@ -751,6 +851,7 @@ R_API bool r_file_rm(const char *file) {
 }
 
 R_API char *r_file_readlink(const char *path) {
+	r_return_val_if_fail (!R_STR_ISEMPTY (path), false);
 	if (!r_sandbox_enable (0)) {
 #if __UNIX__
 		int ret;
@@ -799,8 +900,10 @@ err_r_file_mmap_write:
 		CloseHandle (fh);
 	}
 	return ret;
+#elif __wasi__
+	return -1;
 #elif __UNIX__
-	int fd = r_sandbox_open (file, O_RDWR|O_SYNC, 0644);
+	int fd = r_sandbox_open (file, RDWR_FLAGS, 0644);
 	const int pagesize = getpagesize ();
 	int mmlen = len + pagesize;
 	int rest = addr % pagesize;
@@ -811,12 +914,15 @@ err_r_file_mmap_write:
 	if ((st64)addr < 0) {
 		return -1;
 	}
-	mmap_buf = mmap (NULL, mmlen*2, PROT_READ|PROT_WRITE, MAP_SHARED, fd, (off_t)addr - rest);
+	mmap_buf = mmap (NULL, mmlen * 2, PROT_READ|PROT_WRITE, MAP_SHARED, fd, (off_t)addr - rest);
 	if (((int)(size_t)mmap_buf) == -1) {
 		return -1;
 	}
-	memcpy (mmap_buf+rest, buf, len);
-	munmap (mmap_buf, mmlen*2);
+	memcpy (mmap_buf + rest, buf, len);
+#if !defined(__serenity__)
+	msync (mmap_buf + rest, len, MS_INVALIDATE);
+#endif
+	munmap (mmap_buf, mmlen * 2);
 	close (fd);
 	return len;
 #else
@@ -859,6 +965,8 @@ err_r_file_mmap_read:
 	}
 	free (file_);
 	return ret;
+#elif __wasi__
+	return 0;
 #elif __UNIX__
 	int fd = r_sandbox_open (file, O_RDONLY, 0644);
 	const int pagesize = 4096;
@@ -876,28 +984,33 @@ err_r_file_mmap_read:
 	munmap (mmap_buf, mmlen*2);
 	close (fd);
 	return len;
-#endif
+#else
 	return 0;
+#endif
 }
 
-#if __UNIX__
-static RMmap *r_file_mmap_unix (RMmap *m, int fd) {
+#if __wasi__
+static RMmap *r_file_mmap_unix(RMmap *m, int fd) {
+	return NULL;
+}
+#elif __UNIX__
+static RMmap *r_file_mmap_unix(RMmap *m, int fd) {
 	ut8 empty = m->len == 0;
-	m->buf = mmap (NULL, (empty?1024:m->len) ,
+	m->buf = mmap (NULL, (empty?BS:m->len) ,
 		m->rw?PROT_READ|PROT_WRITE:PROT_READ,
 		MAP_SHARED, fd, (off_t)m->base);
 	if (m->buf == MAP_FAILED) {
-		R_FREE (m);
+		m->buf = NULL;
 	}
 	return m;
 }
 #elif __WINDOWS__
-static RMmap *r_file_mmap_windows (RMmap *m, const char *file) {
+static RMmap *r_file_mmap_windows(RMmap *m, const char *file) {
 	LPTSTR file_ = r_sys_conv_utf8_to_win (file);
 	bool success = false;
 
-	m->fh = CreateFile (file_, GENERIC_READ | (m->rw?GENERIC_WRITE:0),
-		FILE_SHARE_READ|(m->rw?FILE_SHARE_WRITE:0), NULL,
+	m->fh = CreateFile (file_, GENERIC_READ | (m->rw ? GENERIC_WRITE : 0),
+		FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
 		OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
 	if (m->fh == INVALID_HANDLE_VALUE) {
 		r_sys_perror ("CreateFile");
@@ -926,9 +1039,9 @@ err_r_file_mmap_windows:
 	return m;
 }
 #else
-static RMmap *r_file_mmap_other (RMmap *m) {
+static RMmap *file_mmap_other (RMmap *m) {
 	ut8 empty = m->len == 0;
-	m->buf = malloc ((empty?1024:m->len));
+	m->buf = malloc ((empty? BS: m->len));
 	if (!empty && m->buf) {
 		lseek (m->fd, (off_t)0, SEEK_SET);
 		read (m->fd, m->buf, m->len);
@@ -949,7 +1062,7 @@ R_API RMmap *r_file_mmap_arch(RMmap *mmap, const char *filename, int fd) {
 #else
 	(void)filename;
 	(void)fd;
-	return r_file_mmap_other (mmap);
+	return file_mmap_other (mmap);
 #endif
 }
 
@@ -960,7 +1073,7 @@ R_API RMmap *r_file_mmap(const char *file, bool rw, ut64 base) {
 	if (!rw && !r_file_exists (file)) {
 		return m;
 	}
-	fd = r_sandbox_open (file, rw? O_RDWR: O_RDONLY, 0644);
+	fd = r_sandbox_open (file, rw? RDWR_FLAGS: O_RDONLY, 0644);
 	if (fd == -1 && !rw) {
 		eprintf ("r_file_mmap: file does not exis.\n");
 		//m->buf = malloc (m->len);
@@ -995,11 +1108,11 @@ R_API RMmap *r_file_mmap(const char *file, bool rw, ut64 base) {
 	m->fd = -1;
 	return r_file_mmap_windows (m, file);
 #else
-	return r_file_mmap_other (m);
+	return file_mmap_other (m);
 #endif
 }
 
-R_API void r_file_mmap_free (RMmap *m) {
+R_API void r_file_mmap_free(RMmap *m) {
 	if (!m) {
 		return;
 	}
@@ -1019,28 +1132,24 @@ R_API void r_file_mmap_free (RMmap *m) {
 		return;
 	}
 	free (m->filename);
-#if __UNIX__
+#if __UNIX__ && !__wasi__
 	munmap (m->buf, m->len);
 #endif
 	close (m->fd);
 	free (m);
 }
 
-R_API char *r_file_temp (const char *prefix) {
-	int namesz;
-	char *name;
-	char *path = r_file_tmpdir ();
+R_API char *r_file_temp(const char *prefix) {
 	if (!prefix) {
 		prefix = "";
 	}
-	namesz = strlen (prefix) + strlen (path) + 32;
-	name = malloc (namesz);
-	snprintf (name, namesz, "%s/%s.%"PFMT64x, path, prefix, r_sys_now ());
+	char *path = r_file_tmpdir ();
+	char *res = r_str_newf ("%s/%s.%"PFMT64x, path, prefix, r_time_now ());
 	free (path);
-	return name;
+	return res;
 }
 
-R_API int r_file_mkstemp(const char *prefix, char **oname) {
+R_API int r_file_mkstemp(R_NULLABLE const char *prefix, char **oname) {
 	int h = -1;
 	char *path = r_file_tmpdir ();
 	if (!prefix) {
@@ -1057,7 +1166,7 @@ R_API int r_file_mkstemp(const char *prefix, char **oname) {
 	}
 	if (GetTempFileName (path_, prefix_, 0, name)) {
 		char *name_ = r_sys_conv_win_to_utf8 (name);
-		h = r_sandbox_open (name_, O_RDWR|O_EXCL|O_BINARY, 0644);
+		h = r_sandbox_open (name_, RDWR_FLAGS | O_EXCL | O_BINARY, 0644);
 		if (oname) {
 			if (h != -1) {
 				*oname = name_;
@@ -1086,13 +1195,29 @@ err_r_file_mkstemp:
 	}
 
 	char *name = r_str_newf ("%s/r2.%s.XXXXXX%s", path, prefix, suffix);
+#if __wasi__
+	// nothing to do
+#else
 	mode_t mask = umask (S_IWGRP | S_IWOTH);
 	if (suffix && *suffix) {
+#if defined(__GLIBC__) && defined(__GLIBC_MINOR__) && 2 <= __GLIBC__ && 19 <= __GLIBC__MINOR__
 		h = mkstemps (name, strlen (suffix));
+#else
+		char *const xpos = strrchr (name, 'X');
+		const char c = (char)(NULL != xpos ? *(xpos + 1) : 0);
+		if (0 != c) {
+			xpos[1] = 0;
+			h = mkstemp (name);
+			xpos[1] = c;
+		} else {
+			h = -1;
+		}
+#endif
 	} else {
 		h = mkstemp (name);
 	}
 	umask (mask);
+#endif
 	if (oname) {
 		*oname = (h!=-1)? strdup (name): NULL;
 	}
@@ -1102,7 +1227,7 @@ err_r_file_mkstemp:
 	return h;
 }
 
-R_API char *r_file_tmpdir() {
+R_API char *r_file_tmpdir(void) {
 #if __WINDOWS__
 	LPTSTR tmpdir;
 	char *path = NULL;
@@ -1129,7 +1254,7 @@ R_API char *r_file_tmpdir() {
 	free (tmpdir);
 	// Windows 7, stat() function fail if tmpdir ends with '\\'
 	if (path) {
-		int path_len = strlen (path);
+		size_t path_len = strlen (path);
 		if (path_len > 0 && path[path_len - 1] == '\\') {
 			path[path_len - 1] = '\0';
 		}
@@ -1141,19 +1266,29 @@ R_API char *r_file_tmpdir() {
 	}
 	if (!path) {
 #if __ANDROID__
-		path = strdup ("/data/data/org.radare.radare2installer/radare2/tmp");
+		if (r_file_is_directory (TERMUX_PREFIX "/tmp")) {
+			path = strdup (TERMUX_PREFIX "/tmp");
+		} else {
+			path = strdup ("/data/local/tmp");
+		}
 #else
 		path = strdup ("/tmp");
 #endif
 	}
 #endif
 	if (!r_file_is_directory (path)) {
-		eprintf ("Cannot find temporary directory '%s'\n", path);
+		free (path);
+		return NULL;
+		//eprintf ("Cannot find dir.tmp '%s'\n", path);
 	}
 	return path;
 }
 
-R_API bool r_file_copy (const char *src, const char *dst) {
+R_API bool r_file_copy(const char *src, const char *dst) {
+	r_return_val_if_fail (R_STR_ISNOTEMPTY (src) && R_STR_ISNOTEMPTY (dst), false);
+	if (!strcmp (src, dst)) {
+		return false;
+	}
 	/* TODO: implement in C */
 	/* TODO: Use NO_CACHE for iOS dyldcache copying */
 #if HAVE_COPYFILE_H
@@ -1162,7 +1297,6 @@ R_API bool r_file_copy (const char *src, const char *dst) {
 	PTCHAR s = r_sys_conv_utf8_to_win (src);
 	PTCHAR d = r_sys_conv_utf8_to_win (dst);
 	if (!s || !d) {
-		R_LOG_ERROR ("r_file_copy: Failed to allocate memory\n");
 		free (s);
 		free (d);
 		return false;
@@ -1184,23 +1318,84 @@ R_API bool r_file_copy (const char *src, const char *dst) {
 #endif
 }
 
-static void recursive_search_glob (const char *path, const char *glob, RList* list, int depth) {
+static bool dir_recursive(RList *dst, const char *dir) {
+	char *name;
+	RListIter *iter;
+	bool ret = true;
+	RList *files = r_sys_dir (dir);
+	if (!files) {
+		return false;
+	}
+	r_list_foreach (files, iter, name) {
+		char *path;
+		if (!strcmp (name, "..") || !strcmp (name, ".")) {
+			continue;
+		}
+		path = r_str_newf ("%s" R_SYS_DIR "%s", dir, name);
+		if (!path) {
+			ret = false;
+			break;
+		}
+		if (!r_list_append (dst, path)) {
+			ret = false;
+			free (path);
+			break;
+		}
+		if (r_file_is_directory (path)) {
+			if (!dir_recursive (dst, path)) {
+				ret = false;
+				break;
+			}
+		}
+	}
+	r_list_free (files);
+	return ret;
+}
+
+R_API RList *r_file_lsrf(const char *dir) {
+	RList *ret = r_list_new ();
+	if (!ret) {
+		return NULL;
+	}
+	if (!dir_recursive (ret, dir)) {
+		r_list_free (ret);
+		return NULL;
+	}
+	return ret;
+}
+
+R_API bool r_file_rm_rf(const char *dir) {
+	RList *files = r_file_lsrf (dir);
+	if (r_file_exists (dir)) {
+		return r_file_rm (dir);
+	}
+	if (!files) {
+		return false;
+	}
+	r_list_sort (files, (RListComparator) strcmp);
+	RListIter *iter;
+	char *f;
+	r_list_foreach_prev (files, iter, f)  {
+		r_file_rm (f);
+	}
+	return r_file_rm (dir);
+}
+
+
+static void recursive_glob(const char *path, const char *glob, RList* list, int depth) {
 	if (depth < 1) {
 		return;
 	}
 	char* file;
 	RListIter *iter;
-	RList *dir = r_sys_dir (path);
-	r_list_foreach (dir, iter, file) {
+	RList *files = r_sys_dir (path);
+	r_list_foreach (files, iter, file) {
 		if (!strcmp (file, ".") || !strcmp (file, "..")) {
 			continue;
 		}
-		char *filename = malloc (strlen (path) + strlen (file) + 2);
-		strcpy (filename, path);
-		strcat (filename, file);
+		char *filename = r_str_newf ("%s%s", path, file);
 		if (r_file_is_directory (filename)) {
-			strcat (filename, R_SYS_DIR);
-			recursive_search_glob (filename, glob, list, depth - 1);
+			recursive_glob (filename, glob, list, depth - 1);
 			free (filename);
 		} else if (r_str_glob (file, glob)) {
 			r_list_append (list, filename);
@@ -1208,10 +1403,10 @@ static void recursive_search_glob (const char *path, const char *glob, RList* li
 			free (filename);
 		}
 	}
-	r_list_free (dir);
+	r_list_free (files);
 }
 
-R_API RList* r_file_globsearch (const char *_globbed_path, int maxdepth) {
+R_API RList* r_file_glob(const char *_globbed_path, int maxdepth) {
 	char *globbed_path = strdup (_globbed_path);
 	RList *files = r_list_newf (free);
 	char *glob = strchr (globbed_path, '*');
@@ -1226,7 +1421,7 @@ R_API RList* r_file_globsearch (const char *_globbed_path, int maxdepth) {
 			glob_ptr = last_slash + 1;
 			if (globbed_path[0] == '~') {
 				char *rpath = r_str_newlen (globbed_path + 2, last_slash - globbed_path - 1);
-				path = r_str_home (rpath ? rpath : "");
+				path = r_str_home (r_str_get (rpath));
 				free (rpath);
 			} else {
 				path = r_str_newlen (globbed_path, last_slash - globbed_path + 1);
@@ -1243,9 +1438,9 @@ R_API RList* r_file_globsearch (const char *_globbed_path, int maxdepth) {
 		}
 
 		if (*(glob + 1) == '*') {  // "**"
-			recursive_search_glob (path, glob_ptr, files, maxdepth);
-		} else {                   // "*"
-			recursive_search_glob (path, glob_ptr, files, 1);
+			recursive_glob (path, glob_ptr, files, maxdepth);
+		} else {            // "*"
+			recursive_glob (path, glob_ptr, files, 1);
 		}
 		free (path);
 	}
