@@ -1,12 +1,9 @@
-/* radare - LGPL - Copyright 2007-2019 - pancake & Skia */
+/* radare - LGPL - Copyright 2007-2021 - pancake & Skia */
 
 #include "r_cons.h"
 #include "r_util.h"
 #include "r_util/r_print.h"
 #include "r_reg.h"
-#ifdef _MSC_VER
-#include <time.h>
-#endif
 #define NOPTR 0
 #define PTRSEEK 1
 #define PTRBACK 2
@@ -24,12 +21,26 @@
 #define MUSTSEEJSON (mode & R_PRINT_JSON && mode & R_PRINT_ISFIELD)
 #define MUSTSEESTRUCT (mode & R_PRINT_STRUCT)
 
+// FOR MINGW only
+#if __MINGW32__
+// gmtime_r can be defined by mingw
+#ifndef gmtime_r
+static struct tm* gmtime_r(const time_t* t, struct tm* r) {
+	// gmtime is threadsafe in windows because it uses TLS
+	struct tm *theTm = gmtime(t);
+	if (theTm) {
+		*r = *theTm;
+		return r;
+	}
+	return 0;
+}
+#endif // gmtime_r
+#endif
+
 //this define is used as a way to acknowledge when updateAddr should take len
 //as real len of the buffer
 #define THRESHOLD (-4444)
 
-//TODO REWRITE THIS IS BECOMING A NIGHTMARE
-
 static float updateAddr(const ut8 *buf, int len, int endian, ut64 *addr, ut64 *addr64) {
 	float f = 0.0;
 	// assert sizeof (float) == sizeof (ut32))
@@ -39,6 +50,9 @@ static float updateAddr(const ut8 *buf, int len, int endian, ut64 *addr, ut64 *a
 	} else {
 		len = 999;
 	}
+	if (len < 1) {
+		return 0;
+	}
 	if (len >= sizeof (float)) {
 		r_mem_swaporcopy ((ut8*)&f, buf, sizeof (float), endian);
 	}
@@ -53,18 +67,37 @@ static float updateAddr(const ut8 *buf, int len, int endian, ut64 *addr, ut64 *a
 }
 
 static int r_get_size(RNum *num, ut8 *buf, int endian, const char *s) {
-	int size = 0, len = strlen (s);
-	ut64 addr;
-
+	size_t len = strlen (s);
 	if (s[0] == '*' && len >= 4) { // value pointed by the address
+		ut64 addr;
 		int offset = (int)r_num_math (num, s + 1);
 		(void)updateAddr (buf + offset, 999, endian, &addr, NULL);
 		return addr;
-	} else {
-		// flag handling doesnt seems to work here
-		size = r_num_math (num, s);
 	}
-	return size;
+	// flag handling doesnt seems to work here
+	return r_num_math (num, s);
+}
+
+static void r_print_format_u128(const RPrint* p, int endian, int mode,
+		const char *setval, ut64 seeki, ut8* buf, int i, int size) {
+	ut64 low = r_read_ble64 (buf, endian);
+	ut64 hig = r_read_ble64 (buf + 8, endian);
+	if (MUSTSEEJSON) {
+		p->cb_printf ("\"");
+	} else if (!SEEVALUE && !ISQUIET) {
+		p->cb_printf ("0x%08"PFMT64x" = (uint128_t)", seeki);
+	}
+	if (endian) {
+		p->cb_printf ("0x%016"PFMT64x"", low);
+		p->cb_printf ("%016"PFMT64x, hig);
+	} else {
+		p->cb_printf ("0x%016"PFMT64x"", hig);
+		p->cb_printf ("%016"PFMT64x, low);
+	}
+	if (MUSTSEEJSON) {
+		const char *end = endian? "big": "little";
+		p->cb_printf ("\",\"endian\":\"%s\",\"ctype\":\"uint128_t\"}", end);
+	}
 }
 
 static void r_print_format_quadword(const RPrint* p, int endian, int mode,
@@ -360,8 +393,7 @@ static void r_print_format_char(const RPrint* p, int endian, int mode,
 	}
 }
 
-static void r_print_format_decchar(const RPrint* p, int endian, int mode,
-		const char *setval, ut64 seeki, ut8* buf, int i, int size) {
+static void r_print_format_decchar(const RPrint* p, int endian, int mode, const char *setval, ut64 seeki, ut8* buf, int i, int size) {
 	int elem = -1;
 	if (size >= ARRAYINDEX_COEF) {
 		elem = size/ARRAYINDEX_COEF-1;
@@ -438,7 +470,7 @@ static int r_print_format_string(const RPrint* p, ut64 seeki, ut64 addr64, ut64
 	if (MUSTSEEJSON) {
 		char *encstr = r_str_utf16_encode ((const char *)buffer, -1);
 		if (encstr) {
-			p->cb_printf ("%d,\"string\":\"%s\"}", seeki, encstr);
+			p->cb_printf ("%"PFMT64d",\"string\":\"%s\"}", seeki, encstr);
 			free (encstr);
 		}
 	} else if (MUSTSEESTRUCT) {
@@ -484,7 +516,11 @@ static void r_print_format_time(const RPrint* p, int endian, int mode,
 	if (MUSTSET) {
 		p->cb_printf ("wv4 %s @ 0x%08"PFMT64x"\n", setval, seeki+((elem>=0)?elem*4:0));
 	} else if (MUSTSEE) {
-		char *timestr = strdup(asctime (gmtime_r ((time_t*)&addr, &timestruct)));
+		char *timestr = malloc (ASCTIME_BUF_MINLEN);
+		if (!timestr) {
+			return;
+		}
+		r_asctime_r (gmtime_r ((time_t*)&addr, &timestruct), timestr);
 		*(timestr+24) = '\0';
 		if (!SEEVALUE && !ISQUIET) {
 			p->cb_printf ("0x%08" PFMT64x " = ", seeki + ((elem >= 0) ? elem * 4 : 0));
@@ -497,8 +533,7 @@ static void r_print_format_time(const RPrint* p, int endian, int mode,
 			}
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
-				free (timestr);
-				timestr = strdup (asctime (gmtime_r ((time_t*)&addr, &timestruct)));
+				r_asctime_r (gmtime_r ((time_t*)&addr, &timestruct), timestr);
 				*(timestr+24) = '\0';
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("%s", timestr);
@@ -520,7 +555,11 @@ static void r_print_format_time(const RPrint* p, int endian, int mode,
 		}
 		free (timestr);
 	} else if (MUSTSEEJSON || MUSTSEESTRUCT) {
-		char *timestr = strdup (asctime (gmtime_r ((time_t*)&addr, &timestruct)));
+		char *timestr = malloc (ASCTIME_BUF_MINLEN);
+		if (!timestr) {
+			return;
+		}
+		r_asctime_r (gmtime_r ((time_t*)&addr, &timestruct), timestr);
 		*(timestr+24) = '\0';
 		if (size==-1) {
 			p->cb_printf ("\"%s\"", timestr);
@@ -528,8 +567,7 @@ static void r_print_format_time(const RPrint* p, int endian, int mode,
 			p->cb_printf ("[ ");
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
-				free (timestr);
-				timestr = strdup (asctime (gmtime_r ((time_t*)&addr, &timestruct)));
+				r_asctime_r (gmtime_r ((time_t*)&addr, &timestruct), timestr);
 				*(timestr+24) = '\0';
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("\"%s\"", timestr);
@@ -612,13 +650,13 @@ static void r_print_format_hex(const RPrint* p, int endian, int mode,
 		}
 	} else if (MUSTSEEJSON) {
 		if (size == -1) {
-			p->cb_printf ("%d", addr);
+			p->cb_printf ("%"PFMT64d, addr);
 		} else {
 			p->cb_printf ("[ ");
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
 				if (elem == -1 || elem == 0) {
-					p->cb_printf ("%d", addr);
+					p->cb_printf ("%"PFMT64d, addr);
 					if (elem == 0) {
 						elem = -2;
 					}
@@ -682,13 +720,13 @@ static void r_print_format_int(const RPrint* p, int endian, int mode,
 		}
 	} else if (MUSTSEEJSON) {
 		if (size == -1) {
-			p->cb_printf ("%d", addr);
+			p->cb_printf ("%"PFMT64d, addr);
 		} else {
 			p->cb_printf ("[ ");
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
 				if (elem == -1 || elem == 0) {
-					p->cb_printf ("%d", addr);
+					p->cb_printf ("%"PFMT64d, addr);
 					if (elem == 0) {
 						elem = -2;
 					}
@@ -737,7 +775,6 @@ static void r_print_format_octal(const RPrint* p, int endian, int mode,
 	} else if ((mode & R_PRINT_DOT) || MUSTSEESTRUCT) {
 		p->cb_printf ("0%"PFMT64o, addr);
 	} else if (MUSTSEE) {
-		ut32 addr32 = (ut32)addr;
 		if (!SEEVALUE && !ISQUIET) {
 			p->cb_printf ("0x%08" PFMT64x " = ", seeki + ((elem >= 0) ? elem * 4 : 0));
 		}
@@ -745,16 +782,15 @@ static void r_print_format_octal(const RPrint* p, int endian, int mode,
 			p->cb_printf ("(octal) ");
 		}
 		if (size == -1) {
-			p->cb_printf (" 0%08"PFMT64o, addr32);
+			p->cb_printf (" 0%08"PFMT64o, addr);
 		} else {
 			if (!SEEVALUE) {
 				p->cb_printf ("[ ");
 			}
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
-				addr32 = (ut32)addr;
 				if (elem == -1 || elem == 0) {
-					p->cb_printf ("0%08"PFMT64o, addr32);
+					p->cb_printf ("0%08"PFMT64o, addr);
 					if (elem == 0) {
 						elem = -2;
 					}
@@ -772,16 +808,14 @@ static void r_print_format_octal(const RPrint* p, int endian, int mode,
 			}
 		}
 	} else if (MUSTSEEJSON) {
-		ut32 addr32 = (ut32)addr;
 		if (size == -1) {
-			p->cb_printf ("%d", addr32);
+			p->cb_printf ("%"PFMT64d, addr);
 		} else {
 			p->cb_printf ("[ ");
 			while (size--) {
 				updateAddr (buf, i, endian, &addr, NULL);
-				addr32 = (ut32)addr;
 				if (elem == -1 || elem == 0) {
-					p->cb_printf ("%d", addr32);
+					p->cb_printf ("%"PFMT64d, addr);
 					if (elem == 0) {
 						elem = -2;
 					}
@@ -818,7 +852,7 @@ static void r_print_format_hexflag(const RPrint* p, int endian, int mode,
 		if (!SEEVALUE && !ISQUIET) {
 			p->cb_printf ("0x%08" PFMT64x " = ", seeki + ((elem >= 0) ? elem * 4 : 0));
 		}
-		if (size==-1) {
+		if (size == -1) {
 			if (ISQUIET && (addr32 == UT32_MAX)) {
 				p->cb_printf ("-1");
 			} else {
@@ -830,9 +864,8 @@ static void r_print_format_hexflag(const RPrint* p, int endian, int mode,
 			}
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
-				addr32 = (ut32)addr;
 				if (elem == -1 || elem == 0) {
-					p->cb_printf ("0x%08"PFMT64x, addr32);
+					p->cb_printf ("0x%08"PFMT64x, addr);
 					if (elem == 0) {
 						elem = -2;
 					}
@@ -850,16 +883,14 @@ static void r_print_format_hexflag(const RPrint* p, int endian, int mode,
 			}
 		}
 	} else if (MUSTSEEJSON) {
-		ut32 addr32 = (ut32)addr;
 		if (size == -1) {
-			p->cb_printf ("%d", addr32);
+			p->cb_printf ("%"PFMT64d, addr);
 		} else {
 			p->cb_printf ("[ ");
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
-				addr32 = (ut32)addr;
 				if (elem == -1 || elem == 0) {
-					p->cb_printf ("%d", addr32);
+					p->cb_printf ("%"PFMT64d, addr);
 					if (elem == 0) {
 						elem = -2;
 					}
@@ -981,15 +1012,14 @@ static int r_print_format_hexpairs(const RPrint* p, int endian, int mode,
 }
 
 static void r_print_format_float(const RPrint* p, int endian, int mode,
-		const char *setval, ut64 seeki, ut8* buf, int i, int size) {
-	float val_f = 0.0f;
+		const char *setval, ut64 seeki, const ut8* buf, int i, int size) {
 	ut64 addr = 0;
 	int elem = -1;
 	if (size >= ARRAYINDEX_COEF) {
-		elem = size/ARRAYINDEX_COEF - 1;
+		elem = size / ARRAYINDEX_COEF - 1;
 		size %= ARRAYINDEX_COEF;
 	}
-	val_f = updateAddr (buf + i, 999, endian, &addr, NULL);
+	float val_f = updateAddr (buf + i, 999, endian, &addr, NULL);
 	if (MUSTSET) {
 		p->cb_printf ("wv4 %s @ 0x%08"PFMT64x"\n", setval,
 			seeki + ((elem >= 0) ? elem * 4 : 0));
@@ -1034,6 +1064,62 @@ static void r_print_format_float(const RPrint* p, int endian, int mode,
 	}
 }
 
+static void r_print_format_long_double(const RPrint* p, int endian, int mode,
+		const char *setval, ut64 seeki, ut8* buf, int i, int size) {
+	long double val_f = 0.0;
+	ut64 addr = 0;
+	int elem = -1;
+	if (size >= ARRAYINDEX_COEF) {
+		elem = size/ARRAYINDEX_COEF - 1;
+		size %= ARRAYINDEX_COEF;
+	}
+	updateAddr (buf + i, 999, endian, &addr, NULL);
+	r_mem_swaporcopy ((ut8*)&val_f, buf + i, sizeof (long double), endian);
+	if (MUSTSET) {
+		p->cb_printf ("wv8 %s @ 0x%08"PFMT64x"\n", setval,
+			seeki + ((elem >= 0) ? elem * 8 : 0));
+	} else if ((mode & R_PRINT_DOT) || MUSTSEESTRUCT) {
+		p->cb_printf ("%.17Lg", val_f);
+	} else {
+		if (MUSTSEE) {
+			if (!SEEVALUE && !ISQUIET) {
+				p->cb_printf ("0x%08"PFMT64x" = ",
+					seeki + ((elem >= 0) ? elem * 8 : 0));
+			}
+		}
+		if (size == -1) {
+			p->cb_printf ("%.17Lg", val_f);
+		} else {
+			if (!SEEVALUE) {
+				p->cb_printf ("[ ");
+			}
+			while (size--) {
+				// XXX this 999 is scary
+				updateAddr (buf + i, 9999, endian, &addr, NULL);
+				r_mem_swaporcopy ((ut8*)&val_f, buf + i, sizeof (double), endian);
+				if (elem == -1 || elem == 0) {
+					p->cb_printf ("%.17Lg", val_f);
+					if (elem == 0) {
+						elem = -2;
+					}
+				}
+				if (size != 0 && elem == -1) {
+					p->cb_printf (", ");
+				}
+				if (elem > -1) {
+					elem--;
+				}
+				i += 8;
+			}
+			if (!SEEVALUE) {
+				p->cb_printf (" ]");
+			}
+		}
+		if (MUSTSEEJSON) {
+			p->cb_printf ("}");
+		}
+	}
+}
 
 static void r_print_format_double(const RPrint* p, int endian, int mode,
 		const char *setval, ut64 seeki, ut8* buf, int i, int size) {
@@ -1050,7 +1136,7 @@ static void r_print_format_double(const RPrint* p, int endian, int mode,
 		p->cb_printf ("wv8 %s @ 0x%08"PFMT64x"\n", setval,
 			seeki + ((elem >= 0) ? elem * 8 : 0));
 	} else if ((mode & R_PRINT_DOT) || MUSTSEESTRUCT) {
-		p->cb_printf ("%.17g", val_f);
+		p->cb_printf ("%.17lg", val_f);
 	} else {
 		if (MUSTSEE) {
 			if (!SEEVALUE && !ISQUIET) {
@@ -1059,7 +1145,7 @@ static void r_print_format_double(const RPrint* p, int endian, int mode,
 			}
 		}
 		if (size == -1) {
-			p->cb_printf ("%.17g", val_f);
+			p->cb_printf ("%.17lg", val_f);
 		} else {
 			if (!SEEVALUE) {
 				p->cb_printf ("[ ");
@@ -1069,7 +1155,7 @@ static void r_print_format_double(const RPrint* p, int endian, int mode,
 				updateAddr (buf + i, 9999, endian, &addr, NULL);
 				r_mem_swaporcopy ((ut8*)&val_f, buf + i, sizeof (double), endian);
 				if (elem == -1 || elem == 0) {
-					p->cb_printf ("%.17g", val_f);
+					p->cb_printf ("%.17lg", val_f);
 					if (elem == 0) {
 						elem = -2;
 					}
@@ -1107,14 +1193,14 @@ static void r_print_format_word(const RPrint* p, int endian, int mode,
 		p->cb_printf ("wv2 %s @ 0x%08"PFMT64x"\n", setval, seeki+((elem>=0)?elem*2:0));
 	} else if ((mode & R_PRINT_DOT) || MUSTSEESTRUCT) {
 		if (size == -1) {
-			p->cb_printf ("0x%04x", addr);
+			p->cb_printf ("0x%04"PFMT64x, addr);
 		}
 		while ((size -= 2) > 0) {
 			addr = endian
 				? (*(buf+i))<<8 | (*(buf+i+1))
 				: (*(buf+i+1))<<8 | (*(buf+i));
 			if (elem == -1 || elem == 0) {
-				p->cb_printf ("%d", addr);
+				p->cb_printf ("%"PFMT64d, addr);
 				if (elem == 0) {
 					elem = -2;
 				}
@@ -1132,7 +1218,7 @@ static void r_print_format_word(const RPrint* p, int endian, int mode,
 			p->cb_printf ("0x%08"PFMT64x" = ", seeki+((elem>=0)?elem*2:0));
 		}
 		if (size==-1) {
-			p->cb_printf ("0x%04x", addr);
+			p->cb_printf ("0x%04"PFMT64x, addr);
 		} else {
 			if (!SEEVALUE) {
 				p->cb_printf ("[ ");
@@ -1142,7 +1228,7 @@ static void r_print_format_word(const RPrint* p, int endian, int mode,
 					? (*(buf+i))<<8 | (*(buf+i+1))
 					: (*(buf+i+1))<<8 | (*(buf+i));
 				if (elem == -1 || elem == 0) {
-					p->cb_printf ("0x%04x", addr);
+					p->cb_printf ("0x%04"PFMT64x, addr);
 					if (elem == 0) {
 						elem = -2;
 					}
@@ -1161,7 +1247,7 @@ static void r_print_format_word(const RPrint* p, int endian, int mode,
 		}
 	} else if (MUSTSEEJSON) {
 		if (size==-1) {
-			p->cb_printf ("%d", addr);
+			p->cb_printf ("%"PFMT64d, addr);
 		} else {
 			p->cb_printf ("[ ");
 			while ( (size -= 2) >0) {
@@ -1169,7 +1255,7 @@ static void r_print_format_word(const RPrint* p, int endian, int mode,
 					? (*(buf+i))<<8 | (*(buf+i+1))
 					: (*(buf+i+1))<<8 | (*(buf+i));
 				if (elem == -1 || elem == 0) {
-					p->cb_printf ("%d", addr);
+					p->cb_printf ("%"PFMT64d, addr);
 					if (elem == 0) {
 						elem = -2;
 					}
@@ -1209,14 +1295,14 @@ static void r_print_format_nulltermstring(const RPrint* p, int len, int endian,
 		RIOMap *map;
 		while (total_map_left < len
 		       && (map = p->iob.io->va
-		           ? p->iob.map_get (p->iob.io, addr)
+		           ? p->iob.map_get_at (p->iob.io, addr)
 		           : p->iob.map_get_paddr (p->iob.io, addr))
 		       && map->perm & R_PERM_R) {
-			if (!map->itv.size) {
+			if (!r_io_map_size(map)) {
 				total_map_left = addr == 0 ? UT64_MAX : UT64_MAX - addr + 1;
 				break;
 			}
-			total_map_left += map->itv.size - (addr - (p->iob.io->va ? map->itv.addr : map->delta));
+			total_map_left += r_io_map_size (map) - (addr - (p->iob.io->va ? r_io_map_begin (map) : map->delta));
 			addr += total_map_left;
 		}
 		if (total_map_left < len) {
@@ -1328,7 +1414,7 @@ static void r_print_format_nulltermwidestring(const RPrint* p, const int len, in
 		}
 	} else if (MUSTSEEJSON) {
 		int j = i;
-		p->cb_printf ("%d,\"string\":\"", seeki);
+		p->cb_printf ("%"PFMT64d",\"string\":\"", seeki);
 		for (; j < len && ((size == -1 || size-- > 0) && buf[j]); j += 2) {
 			if (IS_PRINTABLE (buf[j])) {
 				p->cb_printf ("%c", buf[j]);
@@ -1356,9 +1442,9 @@ static void r_print_format_bitfield(const RPrint* p, ut64 seeki, char *fmtname,
 		}
 	} else {
 		if (MUSTSEEJSON) {
-			p->cb_printf ("\"`tb %s 0x%x`\"}", fmtname, addr);
+			p->cb_printf ("\"`tb %s 0x%"PFMT64x"`\"}", fmtname, addr);
 		} else if (MUSTSEE) {
-			p->cb_printf ("%s (bitfield) = `tb %s 0x%x`\n",
+			p->cb_printf ("%s (bitfield) = `tb %s 0x%"PFMT64x"`\n",
 				fieldname, fmtname, addr);
 		}
 	}
@@ -1368,7 +1454,11 @@ static void r_print_format_bitfield(const RPrint* p, ut64 seeki, char *fmtname,
 static void r_print_format_enum(const RPrint* p, ut64 seeki, char *fmtname,
 		char *fieldname, ut64 addr, int mode, int size) {
 	char *enumvalue = NULL;
-	addr &= (1ULL << (size * 8)) - 1;
+	if (size >= 8) {
+		// avoid shift overflow
+	} else {
+		addr &= (1ULL << (size * 8)) - 1;
+	}
 	if (MUSTSEE && !SEEVALUE) {
 		p->cb_printf ("0x%08"PFMT64x" = ", seeki);
 	}
@@ -1389,7 +1479,7 @@ static void r_print_format_enum(const RPrint* p, ut64 seeki, char *fmtname,
 		if (MUSTSEEJSON) {
 			p->cb_printf ("%"PFMT64d",\"enum\":\"%s\"}", addr, fmtname);
 		} else if (MUSTSEE) {
-			p->cb_printf ("%s (enum %s) = 0x%x\n",//`te %s 0x%x`\n",
+			p->cb_printf ("%s (enum %s) = 0x%"PFMT64x"\n",//`te %s 0x%x`\n",
 				fieldname, fmtname, addr); //enumvalue); //fmtname, addr);
 		}
 	}
@@ -1412,7 +1502,7 @@ static void r_print_format_register(const RPrint* p, int mode,
 				p->cb_printf ("0x%08" PFMT64x "\n", p->get_register_value (p->reg, ri));
 			}
 		} else if (MUSTSEEJSON) {
-			p->cb_printf ("%d}", p->get_register_value (p->reg, ri));
+			p->cb_printf ("%"PFMT64d"}", p->get_register_value (p->reg, ri));
 		}
 	} else {
 		p->cb_printf ("Register %s does not exists\n", name);
@@ -1450,7 +1540,7 @@ static void r_print_format_num(const RPrint *p, int endian, int mode, const char
 	if (MUSTSET) {
 		p->cb_printf ("wv%d %s @ 0x%08"PFMT64x"\n", bytes, setval, seeki+((elem>=0)?elem*(bytes):0));
 	} else if ((mode & R_PRINT_DOT) || MUSTSEESTRUCT) {
-		p->cb_printf ("%"PFMT64u, addr);
+		r_print_format_num_specifier (p, addr, bytes, sign);
 	} else if (MUSTSEE) {
 		if (!SEEVALUE && !ISQUIET) {
 			p->cb_printf ("0x%08"PFMT64x" = ", seeki + ((elem >= 0)? elem * bytes: 0));
@@ -1531,7 +1621,7 @@ R_API int r_print_format_struct_size(RPrint *p, const char *f, int mode, int n)
 	if (n >= 5) {  // This is the nesting level, is this not a bit arbitrary?!
 		return 0;
 	}
-	const char *fmt2 = sdb_get (p->formats, f, NULL);
+	const char *fmt2 = p? sdb_get (p->formats, f, NULL): NULL;
 	if (!fmt2) {
 		fmt2 = f;
 	}
@@ -1564,7 +1654,6 @@ R_API int r_print_format_struct_size(RPrint *p, const char *f, int mode, int n)
 		*end = '\0';
 		times = r_num_math (NULL, fmt + i + 1);
 		fmt = end + 1;
-		i = 0;
 	}
 	if (fmt[0] == '0') {
 		mode |= R_PRINT_UNIONMODE;
@@ -1573,6 +1662,7 @@ R_API int r_print_format_struct_size(RPrint *p, const char *f, int mode, int n)
 		mode &= ~R_PRINT_UNIONMODE;
 	}
 
+	int p_bits = p? p->bits: 32;
 	int words = r_str_word_set0_stack (args);
 	fmt_len = strlen (fmt);
 	for (; i < fmt_len; i++) {
@@ -1620,12 +1710,16 @@ R_API int r_print_format_struct_size(RPrint *p, const char *f, int mode, int n)
 		case 'F':
 			size += tabsize * 8;
 			break;
+		case 'G': // long double (10 byte aligned to 16)
+		case 'Q': // uint128
+			size += tabsize * 16;
+			break;
 		case 'z':
 		case 'Z':
 			size += tabsize;
 			break;
 		case '*':
-			size += tabsize * (p->bits / 8);
+			size += tabsize * (p_bits / 8);
 			i++;
 			idx--;	//no need to go ahead for args
 			break;
@@ -1673,7 +1767,7 @@ R_API int r_print_format_struct_size(RPrint *p, const char *f, int mode, int n)
 					tmp = *format;
 				}
 			} else {
-				format = sdb_get (p->formats, structname + 1, NULL);
+				format = p? sdb_get (p->formats, structname + 1, NULL): NULL;
 				if (format && !strncmp (format, f, strlen (format) - 1)) { // Avoid recursion here
 					free (o);
 					free (structname);
@@ -1696,7 +1790,7 @@ R_API int r_print_format_struct_size(RPrint *p, const char *f, int mode, int n)
 				free (o);
 				return 0;
 			}
-			if (format && newsize > 0) {
+			if (format) {
 				size += tabsize * newsize;
 			}
 			free (structname);
@@ -1730,7 +1824,7 @@ R_API int r_print_format_struct_size(RPrint *p, const char *f, int mode, int n)
 			} else if (fmt[i+1] == '8') {
 				size += tabsize * 8;
 			} else {
-				size += tabsize * (p->bits / 8);
+				size += tabsize * (p_bits / 8);
 				break;
 			}
 			i++;
@@ -1755,9 +1849,9 @@ R_API int r_print_format_struct_size(RPrint *p, const char *f, int mode, int n)
 			}
 			i++;
 			break;
+		case 'u':
 		case 'D':
 		case 'T':
-		case 'u':
 			//TODO complete this.
 		default:
 			//idx--; //Does this makes sense?
@@ -1824,7 +1918,7 @@ static char *get_args_offset(const char *arg) {
 	return args;
 }
 
-static char *get_format_type(const char fmt) {
+static char *get_format_type(const char fmt, const char arg) {
 	char *type = NULL;
 	switch (fmt) {
 	case 'b':
@@ -1849,12 +1943,18 @@ static char *get_format_type(const char fmt) {
 	case 'F':
 		type = strdup ("double");
 		break;
+	case 'G':
+		type = strdup ("long_double");
+		break;
 	case 'q':
 		type = strdup ("uint64_t");
 		break;
 	case 'u':
 		type = strdup ("uleb128_t");
 		break;
+	case 'Q':
+		type = strdup ("uint128_t");
+		break;
 	case 'w':
 		type = strdup ("uint16_t");
 		break;
@@ -1868,16 +1968,35 @@ static char *get_format_type(const char fmt) {
 	case 'z':
 	case 'Z':
 		type = strdup ("char*");
+		break;
+	case 'n':
+	case 'N':
+		switch (arg) {
+		case '1':
+			type = strdup (fmt == 'n' ? "int8_t" : "uint8_t");
+			break;
+		case '2':
+			type = strdup (fmt == 'n' ? "int16_t" : "uint16_t");
+			break;
+		case '4':
+			type = strdup (fmt == 'n' ? "int32_t" : "uint32_t");
+			break;
+		case '8':
+			type = strdup (fmt == 'n' ? "int64_t" : "uint64_t");
+			break;
+		}
+		break;
 	}
 	return type;
 }
 
+//TODO PJ
 #define MINUSONE ((void*)(size_t)-1)
 #define ISSTRUCT (tmp == '?' || (tmp == '*' && *(arg+1) == '?'))
 R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 		const char *formatname, int mode, const char *setval, char *ofield) {
 	int nargs, i, j, invalid, nexti, idx, times, otimes, endian, isptr = 0;
-	const int old_bits = p->bits;
+	const int old_bits = p? p->bits: 32;
 	char *args = NULL, *bracket, tmp, last = 0;
 	ut64 addr = 0, addr64 = 0, seeki = 0;
 	static int slide = 0, oldslide = 0, ident = 4;
@@ -1964,10 +2083,8 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 		}
 		for (i = 0; i < nargs; i++) {
 			const char *tmp = r_str_word_get0 (args, i);
-			const char *nm = NULL;
-			int len;
-			nm = r_str_rchr (tmp, NULL, ')');
-			len = strlen (nm ? nm + 1 : tmp);
+			const char *nm = r_str_rchr (tmp, NULL, ')');
+			int len = strlen (nm ? nm + 1 : tmp);
 			if (len > maxl) {
 				maxl = len;
 			}
@@ -2027,7 +2144,7 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 				if (otimes > times) {
 					p->cb_printf (",");
 				}
-				p->cb_printf ("[{\"index\":%d,\"offset\":%d},", otimes-times, seek+i);
+				p->cb_printf ("[{\"index\":%d,\"offset\":%"PFMT64d"},", otimes-times, seek+i);
 			} else if (mode) {
 				p->cb_printf ("0x%08"PFMT64x" [%d] {\n", seek + i, otimes-times);
 			}
@@ -2303,13 +2420,13 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 				if (isptr) {
 					p->cb_printf ("*");
 				}
-				p->cb_printf ("\",\"offset\":%d,\"value\":",
+				p->cb_printf ("\",\"offset\":%"PFMT64d",\"value\":",
 					isptr? (seek + nexti - (p->bits / 8)) : seek + i);
 			}
 
 			/* c struct */
 			if (MUSTSEESTRUCT) {
-				char *type = get_format_type (tmp);
+				char *type = get_format_type (tmp, (tmp == 'n' || tmp == 'N') ? arg[1] : 0);
 				if (type) {
 					p->cb_printf ("%*c%s %s; // ", ident, ' ', type, fieldname);
 				} else {
@@ -2322,7 +2439,7 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 			int oi = i;
 			if (isptr == NULLPTR) {
 				if (MUSTSEEJSON) {
-					p->cb_printf ("\"NULL\"}", tmp, seek + i);
+					p->cb_printf ("\"NULL\"}");
 				} else if (MUSTSEE) {
 					p->cb_printf (" NULL\n");
 				}
@@ -2337,23 +2454,27 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					break;
 				case 't':
 					r_print_format_time (p, endian, mode, setval, seeki, buf, i, size);
-					i += (size==-1) ? 4 : 4*size;
+					i += (size==-1)? 4: 4 * size;
 					break;
 				case 'q':
 					r_print_format_quadword (p, endian, mode, setval, seeki, buf, i, size);
-					i += (size==-1) ? 8 : 8*size;
+					i += (size == -1)? 8: 8 * size;
+					break;
+				case 'Q':
+					r_print_format_u128 (p, endian, mode, setval, seeki, buf, i, size);
+					i += (size == -1)? 16: 16 * size;
 					break;
 				case 'b':
 					r_print_format_byte (p, endian, mode, setval, seeki, buf, i, size);
-					i += (size==-1) ? 1 : size;
+					i += (size==-1)? 1: size;
 					break;
 				case 'C':
 					r_print_format_decchar (p, endian, mode, setval, seeki, buf, i, size);
-					i += (size==-1) ? 1 : size;
+					i += (size==-1)? 1: size;
 					break;
 				case 'c':
 					r_print_format_char (p, endian, mode, setval, seeki, buf, i, size);
-					i += (size==-1) ? 1 : size;
+					i += (size==-1)? 1: size;
 					break;
 				case 'X':
 					size = r_print_format_hexpairs (p, endian, mode, setval, seeki, buf, i, size);
@@ -2362,26 +2483,33 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 				case 'T':
 					if (r_print_format_10bytes (p, mode,
 						setval, seeki, addr, buf) == 0) {
-						i += (size==-1) ? 4 : 4*size;
+						i += (size == -1)? 4: 4 * size;
 					}
 					break;
 				case 'f':
 					r_print_format_float (p, endian, mode, setval, seeki, buf, i, size);
-					i += (size==-1) ? 4 : 4*size;
+					i += (size == -1)? 4: 4 * size;
 					break;
 				case 'F':
 					r_print_format_double (p, endian, mode, setval, seeki, buf, i, size);
-					i += (size==-1) ? 8 : 8*size;
+					i += (size == -1)? sizeof (double): sizeof (double) * size;
+					break;
+				case 'G':
+					r_print_format_long_double (p, endian, mode, setval, seeki, buf, i, size);
+					i += (size == -1)? sizeof (long double): sizeof (long double) * size;
 					break;
 				case 'i':
 					r_print_format_int (p, endian, mode, setval, seeki, buf, i, size);
-					i+= (size==-1) ? 4 : 4*size;
+					i+= (size == -1)? 4: 4 * size;
 					break;
 				case 'd': //WHY?? help says: 0x%%08x hexadecimal value (4 bytes)
 					r_print_format_hex (p, endian, mode, setval, seeki, buf, i, size);
-					i+= (size==-1) ? 4 : 4*size;
+					i+= (size == -1)? 4: 4 * size;
 					break;
 				case 'D':
+					if (MUSTSET) {
+						eprintf ("Set val not implemented yet for disassembler!\n");
+					}
 					if (isptr) {
 						if (p->bits == 64) {
 							i += r_print_format_disasm (p, addr64, size);
@@ -2394,29 +2522,29 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					break;
 				case 'o':
 					r_print_format_octal (p, endian, mode, setval, seeki, buf, i, size);
-					i += (size==-1) ? 4 : 4 * size;
+					i += (size == -1)? 4: 4 * size;
 					break;
 				case ';':
 					noline = true;
-					i -= (size==-1) ? 4 : 4 * size;
+					i -= (size == -1)? 4: 4 * size;
 					if (i < 0) {
 						i = 0;
 					}
 					break;
 				case ',':
 					noline = true;
-					i -= (size==-1) ? 1 : size;
+					i -= (size == -1)? 1: size;
 					if (i < 0) {
 						i = 0;
 					}
 					break;
 				case 'x':
 					r_print_format_hexflag (p, endian, mode, setval, seeki, buf, i, size);
-					i += (size==-1) ? 4 : 4*size;
+					i += (size == -1)? 4: 4*size;
 					break;
 				case 'w':
 					r_print_format_word (p, endian, mode, setval, seeki, buf, i, size);
-					i += (size==-1) ? 2 : 2*size;
+					i += (size == -1)? 2: 2 * size;
 					break;
 				case 'z': // zero terminated string
 					r_print_format_nulltermstring (p, len, endian, mode, setval, seeki, buf, i, size);
@@ -2439,11 +2567,17 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					}
 					break;
 				case 's':
+					if (MUSTSET) {
+						eprintf ("Set val not implemented yet for strings!\n");
+					}
 					if (r_print_format_string (p, seeki, addr64, addr, 0, mode) == 0) {
 						i += (size==-1) ? 4 : 4*size;
 					}
 					break;
 				case 'S':
+					if (MUSTSET) {
+						eprintf ("Set val not implemented yet for strings!\n");
+					}
 					if (r_print_format_string (p, seeki, addr64, addr, 1, mode) == 0) {
 						i += (size == -1) ? 8 : 8 * size;
 					}
@@ -2452,15 +2586,23 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					if (size >= ARRAYINDEX_COEF) {
 						size %= ARRAYINDEX_COEF;
 					}
+					if (MUSTSET) {
+						p->cb_printf ("wv4 %s @ 0x%08"PFMT64x"\n", setval, seeki+((elem>=0)?elem*4:0));
+						// eprintf ("Set val not implemented yet for bitfields!\n");
+					}
 					r_print_format_bitfield (p, seeki, fmtname, fieldname, addr, mode, size);
-					i+=(size==-1)?1:size;
+					i+=(size == -1)? 1: size;
 					break;
 				case 'E': // resolve enum
+					if (MUSTSET) {
+						// eprintf ("Set val not implemented yet for enums!\n");
+						p->cb_printf ("wv4 %s @ 0x%08"PFMT64x"\n", setval, seeki+((elem>=0)?elem*4:0));
+					}
 					if (size >= ARRAYINDEX_COEF) {
 						size %= ARRAYINDEX_COEF;
 					}
 					r_print_format_enum (p, seeki, fmtname, fieldname, addr, mode, size);
-					i += (size==-1)? 1: size;
+					i += (size == -1)? 1: size;
 					break;
 				case 'r':
 					if (fmtname) {
@@ -2495,14 +2637,14 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					}
 					if (MUSTSEEJSON) {
 						if (isptr) {
-							p->cb_printf ("%d},", seeki);
+							p->cb_printf ("%"PFMT64d"},", seeki);
 						} else {
 							p->cb_printf ("[");
 						}
 					}
 					if (MUSTSEESTRUCT) {
 						if (isptr) {
-							p->cb_printf ("%d", seeki);
+							p->cb_printf ("%"PFMT64d, seeki);
 						} else {
 							ident += 4;
 							p->cb_printf ("\n");
@@ -2550,7 +2692,7 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 							} else {
 								mode &= ~R_PRINT_MUSTSEE;
 							}
-							s = r_print_format_struct (p, seek+i,
+							s = r_print_format_struct (p, seek + i,
 									buf+i, len-i, fmtname, slide, mode, setval, nxtfield, anon);
 							if ((MUSTSEE || MUSTSEEJSON || MUSTSEESTRUCT) && size != 0 && elem == -1) {
 								if (MUSTSEEJSON) {
@@ -2601,7 +2743,7 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 							//or goto beach;???
 						}
 						r_print_format_num (p, endian, mode, setval, seeki, buf, i, bytes, sign, size);
-						i += (size == -1) ? bytes : size * bytes;
+						i += (size == -1)? bytes: size * bytes;
 						arg++;
 						break;
 					}
