@@ -1,18 +1,17 @@
-/* radare - LGPL - Copyright 2008-2019 - pancake */
+/* radare - LGPL - Copyright 2008-2021 - pancake */
 
 #include <r_util.h>
 #include <r_lib.h>
 
-// TODO: boolify
-#define IFDBG if(__has_debug)
-
 R_LIB_VERSION(r_lib);
 
+/* TODO: avoid globals */
+#define IFDBG if(__has_debug)
 static bool __has_debug = false;
 
 /* XXX : this must be registered in runtime */
 static const char *r_lib_types[] = {
-	"io", "dbg", "lang", "asm", "anal", "parse", "bin", "bin_xtr",
+	"io", "dbg", "lang", "asm", "anal", "parse", "bin", "bin_xtr", "bin_ldr",
 	"bp", "syscall", "fastcall", "crypto", "core", "egg", "fs", NULL
 };
 
@@ -35,9 +34,15 @@ R_API int r_lib_types_get_i(const char *str) {
 
 R_API void *r_lib_dl_open(const char *libname) {
 	void *ret = NULL;
+#if WANT_DYLINK
 #if __UNIX__
 	if (libname) {
-		ret = dlopen (libname, RTLD_GLOBAL | RTLD_LAZY);
+#if __linux__
+		ret = dlopen (libname, RTLD_NOW);
+#endif
+		if (!ret) {
+			ret = dlopen (libname, RTLD_GLOBAL | RTLD_LAZY);
+		}
 	} else {
 		ret = dlopen (NULL, RTLD_NOW);
 	}
@@ -51,7 +56,6 @@ R_API void *r_lib_dl_open(const char *libname) {
 	} else {
 		libname_ = calloc (MAX_PATH, sizeof (TCHAR));
 		if (!libname_) {
-			R_LOG_ERROR ("lib/r_lib_dl_open: Failed to allocate memory.\n");
 			return NULL;
 		}
 		if (!GetModuleFileName (NULL, libname_, MAX_PATH)) {
@@ -63,11 +67,13 @@ R_API void *r_lib_dl_open(const char *libname) {
 	if (!ret && __has_debug) {
 		eprintf ("r_lib_dl_open: error: %s\n", libname);
 	}
+#endif
 #endif
 	return ret;
 }
 
 R_API void *r_lib_dl_sym(void *handler, const char *name) {
+#if WANT_DYLINK
 #if __UNIX__
 	return dlsym (handler, name);
 #elif __WINDOWS__
@@ -75,10 +81,13 @@ R_API void *r_lib_dl_sym(void *handler, const char *name) {
 #else
 	return NULL;
 #endif
+#else
+	return NULL;
+#endif
 }
 
 R_API int r_lib_dl_close(void *handler) {
-#if __UNIX__
+#if __UNIX__ && WANT_DYLINK
 	return dlclose (handler);
 #else
 	return handler? 0: -1;
@@ -136,7 +145,7 @@ err:
 		if (next) {
 			*next = 0;
 		}
-		char *libpath = r_str_newf ("%s/%s." R_LIB_EXT, path0, libname);
+		char *libpath = r_str_newf ("%s"R_SYS_DIR"%s." R_LIB_EXT, path0, libname);
 		if (r_file_exists (libpath)) {
 			free (env);
 			return libpath;
@@ -152,13 +161,8 @@ err:
 R_API RLib *r_lib_new(const char *symname, const char *symnamefunc) {
 	RLib *lib = R_NEW (RLib);
 	if (lib) {
-		char *e = r_sys_getenv ("R_DEBUG");
-		if (e) {
-			__has_debug = *e;
-			free (e);
-		} else {
-			__has_debug = false;
-		}
+		__has_debug = r_sys_getenv_asbool ("R2_DEBUG");
+		lib->ignore_version = r_sys_getenv_asbool ("R2_IGNVER");
 		lib->handlers = r_list_newf (free);
 		lib->plugins = r_list_newf (free);
 		lib->symname = strdup (symname? symname: R_LIB_SYMNAME);
@@ -265,7 +269,7 @@ R_API int r_lib_open(RLib *lib, const char *file) {
 	}
 
 	if (__already_loaded (lib, file)) {
-		eprintf("Not loading library because it has already been loaded from somewhere else: '%s'\n", file);
+		eprintf ("Not loading library because it has already been loaded from somewhere else: '%s'\n", file);
 		return -1;
 	}
 
@@ -297,18 +301,35 @@ R_API int r_lib_open(RLib *lib, const char *file) {
 	return res;
 }
 
+static char *major_minor(const char *s) {
+	char *a = strdup (s);
+	char *p = strchr (a, '.');
+	if (p) {
+		p = strchr (p + 1, '.');
+		if (p) {
+			*p = 0;
+		}
+	}
+	return a;
+}
+
 R_API int r_lib_open_ptr(RLib *lib, const char *file, void *handler, RLibStruct *stru) {
 	r_return_val_if_fail (lib && file && stru, -1);
-	if (stru->version) {
-		if (strcmp (stru->version, R2_VERSION)) {
+	if (stru->version && !lib->ignore_version) {
+		char *mm0 = major_minor (stru->version);
+		char *mm1 = major_minor (R2_VERSION);
+		bool mismatch = strcmp (mm0, mm1);
+		free (mm0);
+		free (mm1);
+		if (mismatch) {
 			eprintf ("Module version mismatch %s (%s) vs (%s)\n",
-				file, stru->version, R2_VERSION);
-			if (stru->pkgname) {
-				const char *dot = strchr (stru->version, '.');
-				int major = atoi (stru->version);
-				int minor = dot ? atoi (dot + 1) : 0;
-				// The pkgname member was introduced in 4.2.0
-				if (major > 4 || (major == 4 && minor >= 2)) {
+					file, stru->version, R2_VERSION);
+			const char *dot = strchr (stru->version, '.');
+			int major = atoi (stru->version);
+			int minor = dot ? atoi (dot + 1) : 0;
+			// The pkgname member was introduced in 4.2.0
+			if (major > 4 || (major == 4 && minor >= 2)) {
+				if (stru->pkgname) {
 					printf ("r2pm -ci %s\n", stru->pkgname);
 				}
 			}
@@ -337,6 +358,7 @@ R_API int r_lib_open_ptr(RLib *lib, const char *file, void *handler, RLibStruct
 }
 
 R_API bool r_lib_opendir(RLib *lib, const char *path) {
+#if WANT_DYLINK
 	r_return_val_if_fail (lib && path, false);
 #if __WINDOWS__
 	wchar_t file[1024];
@@ -404,6 +426,7 @@ R_API bool r_lib_opendir(RLib *lib, const char *path) {
 		}
 	}
 	closedir (dh);
+#endif
 #endif
 	return true;
 }
