@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2019 - pancake, alvarofe */
+/* radare - LGPL - Copyright 2007-2021 - pancake, alvarofe */
 // TODO: RRef - reference counting
 
 #include <stdio.h>
@@ -6,7 +6,18 @@
 #define _R_LIST_C_
 #include "r_util.h"
 
-inline RListIter *r_list_iter_new() {
+#define MERGE_LIMIT 24
+
+R_API size_t r_list_iter_length(RListIter *iter) {
+	size_t count = 0;
+	while (iter->n) {
+		count++;
+		iter = iter->n;
+	}
+	return count;
+}
+
+inline RListIter *r_list_iter_new(void) {
 	return calloc (1, sizeof (RListIter));
 }
 
@@ -19,6 +30,11 @@ R_API RListIter *r_list_iter_get_next(RListIter *list) {
 	return list->n;
 }
 
+R_API RListIter *r_list_iter_get_prev(RListIter *list) {
+	r_return_val_if_fail (list, NULL);
+	return list->p;
+}
+
 R_API void *r_list_iter_get_data(RListIter *list) {
 	r_return_val_if_fail (list, NULL);
 	return list->data;
@@ -64,8 +80,9 @@ R_API int r_list_length(const RList *list) {
 
 /* remove all elements of a list */
 R_API void r_list_purge(RList *list) {
-	r_return_if_fail (list);
-
+	if (!list) {
+		return;
+	}
 	RListIter *it = list->head;
 	while (it) {
 		RListIter *next = it->n;
@@ -165,7 +182,7 @@ R_API int r_list_join(RList *list1, RList *list2) {
 	return 1;
 }
 
-R_API RList *r_list_new() {
+R_API RList *r_list_new(void) {
 	RList *list = R_NEW0 (RList);
 	if (!list) {
 		return NULL;
@@ -191,20 +208,16 @@ R_API RListIter *r_list_item_new(void *data) {
 }
 
 R_API RListIter *r_list_append(RList *list, void *data) {
-	RListIter *item = NULL;
+	r_return_val_if_fail (list, NULL);
 
-	r_return_val_if_fail (list && data, NULL);
-
-	item = R_NEW (RListIter);
+	RListIter *item = r_list_item_new (data);
 	if (!item) {
-		return item;
+		return NULL;
 	}
 	if (list->tail) {
 		list->tail->n = item;
 	}
-	item->data = data;
 	item->p = list->tail;
-	item->n = NULL;
 	list->tail = item;
 	if (!list->head) {
 		list->head = item;
@@ -217,41 +230,38 @@ R_API RListIter *r_list_append(RList *list, void *data) {
 R_API RListIter *r_list_prepend(RList *list, void *data) {
 	r_return_val_if_fail (list, NULL);
 
-	RListIter *item = R_NEW0 (RListIter);
+	RListIter *item = r_list_item_new (data);
 	if (!item) {
 		return NULL;
 	}
 	if (list->head) {
 		list->head->p = item;
 	}
-	item->data = data;
 	item->n = list->head;
-	item->p = NULL;
 	list->head = item;
 	if (!list->tail) {
 		list->tail = item;
 	}
 	list->length++;
-	list->sorted = true;
+	list->sorted = false;
 	return item;
 }
 
-R_API RListIter *r_list_insert(RList *list, int n, void *data) {
-	RListIter *it, *item;
-	int i;
-
+R_API RListIter *r_list_insert(RList *list, ut32 n, void *data) {
 	r_return_val_if_fail (list, NULL);
 
 	if (!list->head || !n) {
 		return r_list_prepend (list, data);
 	}
+
+	RListIter *it;
+	ut32 i;
 	for (it = list->head, i = 0; it && it->data; it = it->n, i++) {
 		if (i == n) {
-			item = R_NEW (RListIter);
+			RListIter *item = r_list_item_new (data);
 			if (!item) {
 				return NULL;
 			}
-			item->data = data;
 			item->n = it;
 			item->p = it->p;
 			if (it->p) {
@@ -259,7 +269,7 @@ R_API RListIter *r_list_insert(RList *list, int n, void *data) {
 			}
 			it->p = item;
 			list->length++;
-			list->sorted = true;
+			list->sorted = false;
 			return item;
 		}
 	}
@@ -282,8 +292,8 @@ R_API void *r_list_pop(RList *list) {
 		}
 		data = iter->data;
 		free (iter);
+		list->length--;
 	}
-	list->length--;
 	return data;
 }
 
@@ -302,8 +312,8 @@ R_API void *r_list_pop_head(RList *list) {
 		}
 		data = iter->data;
 		free (iter);
+		list->length--;
 	}
-	list->length--;
 	return data;
 }
 
@@ -347,6 +357,26 @@ R_API void *r_list_get_bottom(const RList *list) {
 	return list->head ? list->head->data : NULL;
 }
 
+// Moves an iter to the top(tail) of the list
+// There is an underlying assumption here, that iter is an RListIter of this RList
+R_API void r_list_iter_to_top(RList *list, RListIter *iter) {
+	r_return_if_fail (list && iter);
+	if (list->tail == iter) {
+		return;
+	}
+	iter->n->p = iter->p;
+	if (list->head == iter) {
+		list->head = iter->n;
+	} else {
+		iter->p->n = iter->n;
+	}
+	iter->p = list->tail;
+	list->tail->n = iter;
+	iter->n = NULL;
+	list->tail = iter;
+	list->sorted = false;
+}
+
 R_API void r_list_reverse(RList *list) {
 	RListIter *it, *tmp;
 
@@ -363,13 +393,12 @@ R_API void r_list_reverse(RList *list) {
 }
 
 R_API RList *r_list_clone(const RList *list) {
-	RList *l = NULL;
 	RListIter *iter;
 	void *data;
 
 	r_return_val_if_fail (list, NULL);
 
-	l = r_list_new ();
+	RList *l = r_list_new ();
 	if (!l) {
 		return NULL;
 	}
@@ -480,7 +509,7 @@ static RListIter *_merge(RListIter *first, RListIter *second, RListComparator cm
 		} else if (!first) {
 			next = second;
 			second = second->n;
-		} else if (cmp (first->data, second->data) < 0) {
+		} else if (cmp (first->data, second->data) <= 0) {
 			next = first;
 			first = first->n;
 		} else {
@@ -511,24 +540,46 @@ static RListIter * _r_list_half_split(RListIter *head) {
 	}
 	slow = head;
 	fast = head;
+	int count = 0;
 	while (fast && fast->n && fast->n->n) {
 		fast = fast->n->n;
 		slow = slow->n;
+		count++;
+	}
+	if (count < MERGE_LIMIT) {
+		return NULL;
 	}
 	tmp = slow->n;
 	slow->n = NULL;
 	return tmp;
 }
 
+static void list_insertion_sort_iter(RListIter *iter, RListComparator cmp) {
+	RListIter *it, *it2;
+	for (it = iter; it && it->data; it = it->n) {
+		for (it2 = it->n; it2 && it2->data; it2 = it2->n) {
+			if (cmp (it->data, it2->data) > 0) {
+				void *t = it->data;
+				it->data = it2->data;
+				it2->data = t;
+			}
+		}
+	}
+}
+
 static RListIter * _merge_sort(RListIter *head, RListComparator cmp) {
 	RListIter *second;
 	if (!head || !head->n) {
 		return head;
 	}
 	second = _r_list_half_split (head);
-	head = _merge_sort (head, cmp);
-	second = _merge_sort (second, cmp);
-	return _merge (head, second, cmp);
+	if (second) {
+		head = _merge_sort (head, cmp);
+		second = _merge_sort (second, cmp);
+		return _merge (head, second, cmp);
+	}
+	list_insertion_sort_iter (head, cmp);
+	return head;
 }
 
 R_API void r_list_merge_sort(RList *list, RListComparator cmp) {
@@ -571,7 +622,7 @@ R_API void r_list_insertion_sort(RList *list, RListComparator cmp) {
 //chose wisely based on length
 R_API void r_list_sort(RList *list, RListComparator cmp) {
 	r_return_if_fail (list);
-	if (list->length > 43) {
+	if (list->length > MERGE_LIMIT) {
 		r_list_merge_sort (list, cmp);
 	} else {
 		r_list_insertion_sort (list, cmp);
