@@ -137,7 +137,7 @@ static void r_pkcs7_free_contentinfo(RPKCS7ContentInfo *ci) {
 }
 
 static bool r_pkcs7_parse_issuerandserialnumber(RPKCS7IssuerAndSerialNumber *iasu, RASN1Object *object) {
-	if (!iasu || !object || object->list.length != 2) {
+	if (!iasu || !object || object->list.length < 2) {
 		return false;
 	}
 	r_x509_parse_name (&iasu->issuer, object->list.objects[0]);
@@ -324,6 +324,11 @@ R_API RCMS *r_pkcs7_parse_cms(const ut8 *buffer, ut32 length) {
 	}
 	if (object->list.objects[0]) {
 		container->contentType = r_asn1_stringify_oid (object->list.objects[0]->sector, object->list.objects[0]->length);
+		if (!container->contentType) {
+			r_asn1_free_object (object);
+			free (container);
+			return NULL;
+		}
 	}
 	if (object->list.objects[1]) {
 		r_pkcs7_parse_signeddata (&container->signedData, object->list.objects[1]->list.objects[0]);
@@ -640,6 +645,9 @@ static bool r_pkcs7_parse_spcdata(SpcAttributeTypeAndOptionalValue *data, RASN1O
 		return false;
 	}
 	data->type = r_asn1_stringify_oid (object->list.objects[0]->sector, object->list.objects[0]->length);
+	if (!data->type) {
+		return false;
+	}
 	RASN1Object *obj1 = object->list.objects[1];
 	if (object->list.length > 1) {
 		if (obj1) {
@@ -654,39 +662,51 @@ static bool r_pkcs7_parse_spcmessagedigest(SpcDigestInfo *messageDigest, RASN1Ob
 		!object->list.objects[0] || !object->list.objects[1]) {
 		return false;
 	}
-	r_x509_parse_algorithmidentifier (&messageDigest->digestAlgorithm, object->list.objects[0]);
+	if (!r_x509_parse_algorithmidentifier (&messageDigest->digestAlgorithm, object->list.objects[0])) {
+		return false;
+	}
 	RASN1Object *obj1 = object->list.objects[1];
 	messageDigest->digest = r_asn1_create_binary (obj1->sector, obj1->length);
 	return true;
 }
 
 R_API SpcIndirectDataContent *r_pkcs7_parse_spcinfo(RCMS *cms) {
-	RASN1Object *object;
-	RASN1Binary *content;
-	SpcIndirectDataContent *spcinfo;
-	if (!cms) {
+	r_return_val_if_fail (cms, NULL);
+
+	RASN1String *type = cms->signedData.contentInfo.contentType;
+	if (type && strcmp (type->string, "spcIndirectDataContext")) {
 		return NULL;
 	}
 
-	spcinfo = R_NEW0 (SpcIndirectDataContent);
+	SpcIndirectDataContent *spcinfo = R_NEW0 (SpcIndirectDataContent);
 	if (!spcinfo) {
 		return NULL;
 	}
 
-	content = cms->signedData.contentInfo.content;
-	object = r_asn1_create_object (content->binary, content->length, content->binary);
-	if (!object || object->list.length < 2 || !object->list.objects ||
-		!object->list.objects[0] || !object->list.objects[1]) {
-		r_asn1_free_object (object);
+	RASN1Binary *content = cms->signedData.contentInfo.content;
+	if (!content) {
 		free (spcinfo);
 		return NULL;
 	}
+	RASN1Object *object = r_asn1_create_object (content->binary, content->length, content->binary);
+	if (!object || object->list.length < 2 || !object->list.objects ||
+		!object->list.objects[0] || !object->list.objects[1]) {
+		R_FREE (spcinfo);
+		goto beach;
+	}
 	if (object->list.objects[0]) {
-		r_pkcs7_parse_spcdata (&spcinfo->data, object->list.objects[0]);
+		if (!r_pkcs7_parse_spcdata (&spcinfo->data, object->list.objects[0])) {
+			R_FREE (spcinfo);
+			goto beach;
+		}
 	}
 	if (object->list.objects[1]) {
-		r_pkcs7_parse_spcmessagedigest (&spcinfo->messageDigest, object->list.objects[1]);
+		if (!r_pkcs7_parse_spcmessagedigest (&spcinfo->messageDigest, object->list.objects[1])) {
+			R_FREE (spcinfo);
+			goto beach;
+		}
 	}
+beach:
 	r_asn1_free_object (object);
 	return spcinfo;
 }
