@@ -41,6 +41,8 @@
 #include <limits.h>
 #include <stdlib.h>
 #include "r_regex.h"
+#include "r_util/r_str.h"
+#include "r_util/r_assert.h"
 
 #include "utils.h"
 #include "regex2.h"
@@ -81,6 +83,7 @@ static char p_b_coll_elem(struct parse *, int);
 static char othercase(int);
 static void bothcases(struct parse *, int);
 static void ordinary(struct parse *, int);
+static void special(struct parse *, int);
 static void nonnewline(struct parse *);
 static void repeat(struct parse *, sopno, int, int);
 static int seterr(struct parse *, int);
@@ -89,7 +92,7 @@ static void freeset(struct parse *, cset *);
 static int freezeset(struct parse *, cset *);
 static int firstch(struct parse *, cset *);
 static int nch(struct parse *, cset *);
-static void mcadd(struct parse *, cset *, char *);
+static void mcadd(struct parse *, cset *, const char *);
 static void mcinvert(struct parse *, cset *);
 static void mccase(struct parse *, cset *);
 static int isinsets(struct re_guts *, int);
@@ -137,30 +140,40 @@ static char nuls[10];		/* place to point scanner in event of error */
 #define	DROP(n)	(p->slen -= (n))
 
 
-R_API int r_regex_match (const char *pattern, const char *flags, const char *text) {
-	int ret;
+R_API bool r_regex_match(const char *pattern, const char *flags, const char *text) {
 	RRegex rx;
 	int re_flags = r_regex_flags (flags);
-	if (r_regex_comp (&rx, pattern, re_flags)) {
-		eprintf ("FAIL TO COMPILE %s\n", pattern);
-		return 0;
+	if (r_regex_init (&rx, pattern, re_flags)) {
+		eprintf ("r_regex_match: /%s/ fails to compile.\n", pattern);
+		return false;
 	}
-	ret = r_regex_exec (&rx, text, 0, 0, re_flags);
+	int rc = r_regex_exec (&rx, text, 0, 0, re_flags);
 	r_regex_fini (&rx);
-	return ret? 0: 1;
-#if 0
-	regex_t preg;
-	regmatch_t pmatch[NUM_MATCHES];
-	if (regcomp(&preg, reg, REG_EXTENDED))
-		return -1;
-	return (regexec (&preg, str, NUM_MATCHES, pmatch, 0))?1:0;
-#endif
+	return rc == 0;
 }
 
-R_API RRegex *r_regex_new (const char *pattern, const char *flags) {
-	RRegex rx, *r;
-	memset(&rx, 0, sizeof(RRegex));
-	if (r_regex_comp (&rx, pattern, r_regex_flags (flags))) {
+R_API RList *r_regex_match_list(RRegex *rx, const char *text) {
+	RList *list = r_list_newf (free);
+	RRegexMatch match;
+
+	/* Initialize the boundaries for R_REGEX_STARTEND */
+	match.rm_so = 0;
+	match.rm_eo = strlen (text);
+	while (!r_regex_exec (rx, text, 1, &match, rx->re_flags | R_REGEX_STARTEND)) {
+		size_t entry_len = match.rm_eo - match.rm_so + 1;
+		char *entry = r_str_ndup (text + match.rm_so, entry_len);
+		r_list_append (list, entry);
+		/* Update the boundaries for R_REGEX_STARTEND */
+		match.rm_so = match.rm_eo;
+		match.rm_eo = strlen (text);
+	}
+	return list;
+}
+
+R_API RRegex *r_regex_new(const char *pattern, const char *flags) {
+	r_return_val_if_fail (pattern, NULL);
+	RRegex *r, rx = {0};
+	if (r_regex_init (&rx, pattern, r_regex_flags (flags))) {
 		return NULL;
 	}
 	r = R_NEW (RRegex);
@@ -232,18 +245,12 @@ R_API void r_regex_free(RRegex *preg) {
  - regcomp - interface for parser and compilation
  - 0 success, otherwise R_REGEX_something
  */
-R_API int r_regex_comp(RRegex *preg, const char *pattern, int cflags) {
+R_API int r_regex_init(RRegex *preg, const char *pattern, int cflags) {
 	struct parse pa;
-	struct re_guts *g;
 	struct parse *p = &pa;
 	int i;
 	size_t len;
-#ifdef REDEBUG
-#	define	GOODFLAGS(f)	(f)
-#else
-#	define	GOODFLAGS(f)	((f)&~R_REGEX_DUMP)
-#endif
-	cflags = GOODFLAGS (cflags);
+	cflags &= ~R_REGEX_DUMP;
 	if (!preg || ((cflags & R_REGEX_EXTENDED) && (cflags & R_REGEX_NOSPEC))) {
 		return R_REGEX_INVARG;
 	}
@@ -256,7 +263,7 @@ R_API int r_regex_comp(RRegex *preg, const char *pattern, int cflags) {
 		len = strlen ((char *)pattern);
 	}
 	/* do the mallocs early so failure handling is easy */
-	g = (struct re_guts *)calloc (sizeof (struct re_guts) + (NC - 1), sizeof (cat_t));
+	struct re_guts *g = (struct re_guts *)calloc (sizeof (struct re_guts) + (NC - 1), sizeof (cat_t));
 	if (!g) {
 		return R_REGEX_ESPACE;
 	}
@@ -340,12 +347,10 @@ R_API int r_regex_comp(RRegex *preg, const char *pattern, int cflags) {
 	preg->re_nsub = g->nsub;
 	preg->re_g = g;
 	preg->re_magic = MAGIC1;
-#ifndef REDEBUG
 	/* not debugging, so can't rely on the asssert() in regexec() */
 	if (g->iflags & BAD) {
 		SETERROR (R_REGEX_ASSERT);
 	}
-#endif
 	if (p->error) {
 		r_regex_fini (preg);
 	}
@@ -463,7 +468,11 @@ static void p_ere_exp(struct parse *p) {
 	case '\\':
 		REQUIRE(MORE(), R_REGEX_EESCAPE);
 		c = GETNEXT();
-		ordinary(p, c);
+		if (!isalpha(c)) {
+			ordinary(p, c);
+		} else {
+			special(p, c);
+		}
 		break;
 	case '{':		/* okay as ordinary except if digit follows */
 		REQUIRE(!MORE() || !isdigit((ut8)PEEK()), R_REGEX_BADRPT);
@@ -898,7 +907,7 @@ static void p_b_cclass(struct parse *p, cset *cs) {
 	char *sp = p->next;
 	struct cclass *cp;
 	size_t len;
-	char *u;
+	const char *u;
 	char c;
 
 	while (MORE () && isalpha ((unsigned char)PEEK ())) {
@@ -1049,6 +1058,58 @@ ordinary(struct parse *p, int ch)
 	}
 }
 
+static void
+special(struct parse *p, int ch) {
+	char *oldnext = p->next;
+	char *oldend = p->end;
+	char bracket[16] = {0};
+	char digits[3] = {0};
+	char c;
+	int num = 0;
+	switch (ch) {
+	case 'x':
+		digits[0] = GETNEXT ();
+		digits[1] = GETNEXT ();
+		c = (char)strtol (digits, NULL, 16);
+		ordinary (p, c);
+		return;
+	case 'n':
+		ordinary (p, '\n');
+		return;
+	case 't':
+		ordinary (p, '\t');
+		return;
+	case 'r':
+		ordinary (p, '\r');
+		return;
+	case 's':
+		num = 5;
+		memcpy (bracket, "\t\r\n ]", num);
+		break;
+	case 'd':
+		num = 4;
+		memcpy (bracket, "0-9]", num);
+		break;
+	case 'w':
+		num = 4;
+		memcpy (bracket, "a-z]", num);
+		break;
+	default:
+		SETERROR (R_REGEX_INVARG);
+		return;
+	}
+
+	p->next = bracket;
+	p->end = bracket + num;
+
+	p_bracket (p);
+
+	if (p->next == bracket + num) {
+		p->next = oldnext;
+		p->end = oldend;
+	}
+}
+
 /*
  - nonnewline - emit R_REGEX_NEWLINE version of OANY
  *
@@ -1316,7 +1377,7 @@ static int nch(struct parse *p, cset *cs) {
 /*
  - mcadd - add a collating element to a cset
  */
-static void mcadd( struct parse *p, cset *cs, char *cp) {
+static void mcadd( struct parse *p, cset *cs, const char *cp) {
 	size_t oldend = cs->smultis;
 	void *np;
 
@@ -1332,7 +1393,7 @@ static void mcadd( struct parse *p, cset *cs, char *cp) {
 	}
 	cs->multis = np;
 
-	STRLCPY(cs->multis + oldend - 1, cp, cs->smultis - oldend + 1);
+	r_str_ncpy (cs->multis + oldend - 1, cp, cs->smultis - oldend + 1);
 }
 
 /*
