@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2012-2017 - pancake */
+/* radare - LGPL - Copyright 2012-2020 - pancake */
 
 #include <r_util.h>
 #include <signal.h>
@@ -11,10 +11,15 @@
 #include <sys/capsicum.h>
 #endif
 
+#if LIBC_HAVE_PRIV_SET
+#include <priv.h>
+#endif
+
 static bool enabled = false;
 static bool disabled = false;
 
 static bool inHomeWww(const char *path) {
+	r_return_val_if_fail (path, false);
 	bool ret = false;
 	char *homeWww = r_str_home (R2_HOME_WWWROOT R_SYS_DIR);
 	if (homeWww) {
@@ -33,13 +38,10 @@ static bool inHomeWww(const char *path) {
  * path are ok.
  */
 R_API bool r_sandbox_check_path (const char *path) {
+	r_return_val_if_fail (path, false);
 	size_t root_len;
 	char *p;
 	/* XXX: the sandbox can be bypassed if a directory is symlink */
-
-	if (!path) {
-		return false;
-	}
 	root_len = strlen (R2_LIBDIR"/radare2");
 	if (!strncmp (path, R2_LIBDIR"/radare2", root_len)) {
 		return true;
@@ -102,6 +104,12 @@ R_API bool r_sandbox_disable (bool e) {
 			eprintf ("sandbox mode couldn't be disabled in capability mode\n");
 			return enabled;
 		}
+#endif
+#if LIBC_HAVE_PRIV_SET
+		if (enabled) {
+			eprintf ("sandbox mode couldn't be disabled in priv mode\n");
+			return enabled;
+		}
 #endif
 		disabled = enabled;
 		enabled = false;
@@ -162,11 +170,42 @@ R_API bool r_sandbox_enable (bool e) {
 			return false;
 		}
 	}
+#endif
+#if LIBC_HAVE_PRIV_SET
+	if (enabled) {
+		priv_set_t *priv = priv_allocset();
+		const char *const privrules[] = {
+			PRIV_PROC_INFO,
+			PRIV_PROC_SESSION,
+			PRIV_PROC_ZONE,
+			PRIV_NET_OBSERVABILITY
+		};
+
+		size_t i, privrulescnt = sizeof (privrules) / sizeof (privrules[0]);
+		
+		if (!priv) {
+			eprintf ("sandbox: priv_allocset failed\n");
+			return false;
+		}
+		priv_basicset(priv);
+		
+		for (i = 0; i < privrulescnt; i ++) {
+			if (priv_delset (priv, privrules[i]) != 0) {
+				priv_emptyset (priv);
+				priv_freeset (priv);
+				eprintf ("sandbox: priv_delset failed\n");
+				return false;
+			}
+		}
+
+		priv_freeset (priv);
+	}
 #endif
 	return enabled;
 }
 
-R_API int r_sandbox_system (const char *x, int n) {
+R_API int r_sandbox_system(const char *x, int n) {
+	r_return_val_if_fail (x, -1);
 	if (enabled) {
 		eprintf ("sandbox: system call disabled\n");
 		return -1;
@@ -175,18 +214,29 @@ R_API int r_sandbox_system (const char *x, int n) {
 #if LIBC_HAVE_SYSTEM
 	if (n) {
 #if APPLE_SDK_IPHONEOS
-#include <dlfcn.h>
-		int (*__system)(const char *cmd)
-			= dlsym (NULL, "system");
-		if (__system) {
-			return __system (x);
+#include <spawn.h>
+		int argc;
+		char *cmd = strdup (x);
+		char **argv = r_str_argv (cmd, &argc);
+		if (argv) {
+			char *argv0 = r_file_path (argv[0]);
+			pid_t pid = 0;
+			int r = posix_spawn (&pid, argv0, NULL, NULL, argv, NULL);
+			if (r != 0) {
+				return -1;
+			}
+			int status;
+			int s = waitpid (pid, &status, 0);
+			return WEXITSTATUS (s);
 		}
-		return -1;
 #else
 		return system (x);
 #endif
 	}
-	return execl ("/bin/sh", "sh", "-c", x, (const char*)NULL);
+	char *bin_sh = r_file_binsh ();
+	int rc = execl (bin_sh, "sh", "-c", x, (const char*)NULL);
+	free (bin_sh);
+	return rc;
 #else
 	#include <spawn.h>
 	if (n && !strchr (x, '|')) {
@@ -219,12 +269,18 @@ R_API int r_sandbox_system (const char *x, int n) {
 		eprintf ("Error parsing command arguments\n");
 		return -1;
 	}
-	int child = fork();
-	if (child == -1) return -1;
+	int child = fork ();
+	if (child == -1) {
+		return -1;
+	}
 	if (child) {
 		return waitpid (child, NULL, 0);
 	}
-	execl ("/bin/sh", "sh", "-c", x, (const char*)NULL);
+	char *bin_sh = r_file_binsh ();
+	if (execl (bin_sh, "sh", "-c", x, (const char*)NULL) == -1) {
+		perror ("execl");
+	}
+	free (bin_sh);
 	exit (1);
 #endif
 #endif
@@ -249,11 +305,8 @@ R_API bool r_sandbox_creat (const char *path, int mode) {
 	return false;
 }
 
-static char *expand_home(const char *p) {
-	if (*p == '~') {
-		return r_str_home (p);
-	}
-	return strdup (p);
+static inline char *expand_home(const char *p) {
+	return (*p == '~')? r_str_home (p): strdup (p);
 }
 
 R_API int r_sandbox_lseek(int fd, ut64 addr, int whence) {
@@ -288,20 +341,16 @@ R_API int r_sandbox_close(int fd) {
 
 /* perm <-> mode */
 R_API int r_sandbox_open(const char *path, int perm, int mode) {
-	if (!path) {
-		return -1;
-	}
+	r_return_val_if_fail (path, -1);
 	char *epath = expand_home (path);
 	int ret = -1;
 #if __WINDOWS__
-	mode |= O_BINARY;
 	if (!strcmp (path, "/dev/null")) {
 		path = "NUL";
 	}
 #endif
 	if (enabled) {
-		if ((mode & O_CREAT)
-			|| (mode & O_RDWR)
+		if ((perm & O_CREAT) || (perm & O_RDWR)
 			|| (!r_sandbox_check_path (epath))) {
 			free (epath);
 			return -1;
@@ -309,12 +358,58 @@ R_API int r_sandbox_open(const char *path, int perm, int mode) {
 	}
 #if __WINDOWS__
 	{
+		DWORD flags = 0;
+		if (perm & O_RANDOM) {
+			flags = FILE_FLAG_RANDOM_ACCESS;
+		} else if (perm & O_SEQUENTIAL) {
+			flags = FILE_FLAG_SEQUENTIAL_SCAN;
+		}
+		if (perm & O_TEMPORARY) {
+			flags |= FILE_FLAG_DELETE_ON_CLOSE | FILE_ATTRIBUTE_TEMPORARY;
+		} else if (perm & _O_SHORT_LIVED) {
+			flags |= FILE_ATTRIBUTE_TEMPORARY;
+		} else {
+			flags |= FILE_ATTRIBUTE_NORMAL;
+		}
+		DWORD creation = 0;
+		bool read_only = false;
+		if (perm & O_CREAT) {
+			if (perm & O_EXCL) {
+				creation = CREATE_NEW;
+			} else {
+				creation = OPEN_ALWAYS;
+			}
+			if (mode & S_IREAD && !(mode & S_IWRITE)) {
+				flags = FILE_ATTRIBUTE_READONLY;
+				read_only = true;
+			}
+		} else if (perm & O_TRUNC) {
+			creation = TRUNCATE_EXISTING;
+		}
+		if (!creation || !strcasecmp ("NUL", path)) {
+			creation = OPEN_EXISTING;
+		}
+		DWORD permission = 0;
+		if (perm & O_WRONLY) {
+			permission = GENERIC_WRITE;
+		} else if (perm & O_RDWR) {
+			permission = GENERIC_WRITE | GENERIC_READ;
+		} else {
+			permission = GENERIC_READ;
+		}
+		if (perm & O_APPEND) {
+			permission |= FILE_APPEND_DATA;
+		}
+
 		wchar_t *wepath = r_utf8_to_utf16 (epath);
 		if (!wepath) {
 			free (epath);
 			return -1;
 		}
-		ret = _wopen (wepath, perm, mode);
+		HANDLE h = CreateFileW (wepath, permission, FILE_SHARE_READ | (read_only ? 0 : FILE_SHARE_WRITE), NULL, creation, flags, NULL);
+		if (h != INVALID_HANDLE_VALUE) {
+			ret = _open_osfhandle ((intptr_t)h, perm);
+		}
 		free (wepath);
 	}
 #else // __WINDOWS__
@@ -325,11 +420,9 @@ R_API int r_sandbox_open(const char *path, int perm, int mode) {
 }
 
 R_API FILE *r_sandbox_fopen (const char *path, const char *mode) {
+	r_return_val_if_fail (path && mode, NULL);
 	FILE *ret = NULL;
 	char *epath = NULL;
-	if (!path) {
-		return NULL;
-	}
 	if (enabled) {
 		if (strchr (mode, 'w') || strchr (mode, 'a') || strchr (mode, '+')) {
 			return NULL;
@@ -343,7 +436,7 @@ R_API FILE *r_sandbox_fopen (const char *path, const char *mode) {
 	if (!epath) {
 		epath = expand_home (path);
 	}
-	if ((strchr (mode, 'w') || r_file_is_regular (epath))) {
+	if ((strchr (mode, 'w') || strchr (mode, 'a') || r_file_is_regular (epath))) {
 #if __WINDOWS__
 		wchar_t *wepath = r_utf8_to_utf16 (epath);
 		if (!wepath) {
@@ -367,7 +460,8 @@ R_API FILE *r_sandbox_fopen (const char *path, const char *mode) {
 	return ret;
 }
 
-R_API int r_sandbox_chdir (const char *path) {
+R_API int r_sandbox_chdir(const char *path) {
+	r_return_val_if_fail (path, -1);
 	if (enabled) {
 		// TODO: check path
 		if (strstr (path, "../")) {
@@ -382,22 +476,21 @@ R_API int r_sandbox_chdir (const char *path) {
 }
 
 R_API int r_sandbox_kill(int pid, int sig) {
+	r_return_val_if_fail (pid != -1, -1);
 	// XXX: fine-tune. maybe we want to enable kill for child?
 	if (enabled) {
 		return -1;
 	}
-#if __UNIX__
+#if HAVE_SYSTEM && __UNIX__
 	return kill (pid, sig);
 #endif
 	return -1;
 }
 #if __WINDOWS__
 R_API HANDLE r_sandbox_opendir (const char *path, WIN32_FIND_DATAW *entry) {
+	r_return_val_if_fail (path, NULL);
 	wchar_t dir[MAX_PATH];
 	wchar_t *wcpath = 0;
-	if (!path) {
-		return NULL;
-	}
 	if (r_sandbox_enable (0)) {
 		if (path && !r_sandbox_check_path (path)) {
 			return NULL;
@@ -412,9 +505,7 @@ R_API HANDLE r_sandbox_opendir (const char *path, WIN32_FIND_DATAW *entry) {
 }
 #else
 R_API DIR* r_sandbox_opendir (const char *path) {
-	if (!path) {
-		return NULL;
-	}
+	r_return_val_if_fail (path, NULL);
 	if (r_sandbox_enable (0)) {
 		if (path && !r_sandbox_check_path (path)) {
 			return NULL;
@@ -423,7 +514,7 @@ R_API DIR* r_sandbox_opendir (const char *path) {
 	return opendir (path);
 }
 #endif
-R_API bool r_sys_stop () {
+R_API bool r_sys_stop (void) {
 	if (enabled) {
 		return false;
 	}
