@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2019 - pancake */
+/* radare - LGPL - Copyright 2007-2021 - pancake */
 
 #include "r_types.h"
 #include "r_util.h"
@@ -9,6 +9,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdarg.h>
+#include <r_util/r_base64.h>
 
 /* stable code */
 static const char *nullstr = "";
@@ -33,6 +34,7 @@ static const char *rwxstr[] = {
 	[15] = "rwx",
 };
 
+
 R_API int r_str_casecmp(const char *s1, const char *s2) {
 #ifdef _MSC_VER
 	return stricmp (s1, s2);
@@ -77,49 +79,22 @@ R_API int r_str_replace_ch(char *s, char a, char b, bool global) {
 	return ret;
 }
 
-// DEPRECATED
 R_API int r_str_replace_char_once(char *s, int a, int b) {
-	int ret = 0;
-	char *o = s;
-	if (a == b) {
-		return 0;
-	}
-	for (; *o; s++, o++) {
-		if (*o == a) {
-			if (b) {
-				*s = b;
-				return ++ret;
-			}
-			o++;
-		}
-		*s = *o;
-	}
-	*s = 0;
-	return ret;
+	return r_str_replace_ch (s, a, b, false);
 }
 
-// DEPRECATED
 R_API int r_str_replace_char(char *s, int a, int b) {
-	int ret = 0;
-	char *o = s;
-	if (!s || a == b) {
-		return 0;
-	}
-	for (; *o; s++, o++) {
-		if (*o == a) {
-			ret++;
-			if (b) {
-				*s = b;
-			} else {
-				/* remove char */
-				s--;
-			}
-		} else {
-			*s = *o;
+	return r_str_replace_ch (s, a, b, true);
+}
+
+R_API void r_str_remove_char(char *str, char c) {
+	while (*str) {
+		if (*str == c) {
+			memmove (str, str + 1, strlen (str + 1) + 1);
+			continue;
 		}
+		str++;
 	}
-	*s = 0;
-	return ret;
 }
 
 R_API void r_str_reverse(char *str) {
@@ -303,16 +278,16 @@ R_API char *r_str_home(const char *str) {
 		}
 	}
 	length = strlen (home) + 1;
-	if (str) {
+	if (R_STR_ISNOTEMPTY (str)) {
 		length += strlen (R_SYS_DIR) + strlen (str);
 	}
-	dst = (char *)malloc (length);
+	dst = (char *)calloc (1, length);
 	if (!dst) {
 		goto fail;
 	}
 	int home_len = strlen (home);
 	memcpy (dst, home, home_len + 1);
-	if (str) {
+	if (R_STR_ISNOTEMPTY (str)) {
 		dst[home_len] = R_SYS_DIR[0];
 		strcpy (dst + home_len + 1, str);
 	}
@@ -567,27 +542,27 @@ R_API int r_str_char_count(const char *string, char ch) {
 			count++;
 		}
 	}
-	return count;
+	return R_MAX (0, count);
 }
 
 // Counts the number of words (separated by separator characters: newlines, tabs,
 // return, space). See r_util.h for more details of the IS_SEPARATOR macro.
 R_API int r_str_word_count(const char *string) {
-	const char *text, *tmp;
+	const char *text;
 	int word;
 
-	for (text = tmp = string; *text && IS_SEPARATOR (*text); text++) {
+	for (text = string; *text && IS_SEPARATOR (*text); text++) {
 		;
 	}
 	for (word = 0; *text; word++) {
 		for (; *text && !IS_SEPARATOR (*text); text++) {
 			;
 		}
-		for (tmp = text; *text && IS_SEPARATOR (*text); text++) {
+		for (; *text && IS_SEPARATOR (*text); text++) {
 			;
 		}
 	}
-	return word;
+	return R_MAX (0, word);
 }
 
 // Returns a pointer to the first instance of a character that isn't chr in a
@@ -676,7 +651,7 @@ R_API const char *r_str_rstr(const char *base, const char *p) {
 R_API const char *r_str_rchr(const char *base, const char *p, int ch) {
 	r_return_val_if_fail (base, NULL);
 	if (!p) {
-		p = base + strlen (base);
+		return strrchr (base, ch);
 	}
 	for (; p >= base; p--) {
 		if (ch == *p) {
@@ -741,6 +716,19 @@ R_API char *r_str_trunc_ellipsis(const char *str, int len) {
 	return buf;
 }
 
+R_API char *r_str_newvf(const char *fmt, va_list ap) {
+	va_list ap2;
+	va_copy (ap2, ap);
+	int ret = vsnprintf (NULL, 0, fmt, ap2);
+	ret++;
+	char *p = calloc (1, ret);
+	if (p) {
+		(void)vsnprintf (p, ret, fmt, ap);
+	}
+	va_end (ap2);
+	return p;
+}
+
 R_API char *r_str_newf(const char *fmt, ...) {
 	va_list ap, ap2;
 
@@ -762,12 +750,12 @@ R_API char *r_str_newf(const char *fmt, ...) {
 }
 
 // Secure string copy with null terminator (like strlcpy or strscpy but ours
-R_API void r_str_ncpy(char *dst, const char *src, size_t n) {
-	int i;
+R_API size_t r_str_ncpy(char *dst, const char *src, size_t n) {
+	size_t i;
 
 	// do not do anything if n is 0
 	if (n == 0) {
-		return;
+		return 0;
 	}
 
 	n--;
@@ -775,6 +763,7 @@ R_API void r_str_ncpy(char *dst, const char *src, size_t n) {
 		dst[i] = src[i];
 	}
 	dst[i] = 0;
+	return i;
 }
 
 /* memccmp("foo.bar", "foo.cow, '.') == 0 */
@@ -845,30 +834,37 @@ R_API char *r_str_word_get_first(const char *text) {
 }
 
 R_API const char *r_str_get(const char *str) {
-	return str? str: nullstr_c;
-}
-
-R_API const char *r_str_get2(const char *str) {
 	return str? str: nullstr;
 }
 
+R_API const char *r_str_get_fail(const char *str, const char *failstr) {
+	return str? str: failstr;
+}
+
+R_API const char *r_str_getf(const char *str) {
+	return str? str: nullstr_c;
+}
+
 R_API char *r_str_ndup(const char *ptr, int len) {
-	if (len < 0) {
+	if (!ptr || len < 0) {
 		return NULL;
 	}
-	char *out = malloc (len + 1);
+	size_t plen = r_str_nlen (ptr, len);
+	size_t olen = R_MIN (len, plen);
+	char *out = malloc (olen + 1);
 	if (!out) {
 		return NULL;
 	}
-	strncpy (out, ptr, len);
-	out[len] = 0;
+	memcpy (out, ptr, olen);
+	out[olen] = 0;
 	return out;
 }
 
 // TODO: deprecate?
 R_API char *r_str_dup(char *ptr, const char *string) {
-	free (ptr);
-	return r_str_new (string);
+	char *str = r_str_new (string);
+	free (ptr); // in case ptr == string
+	return str;
 }
 
 R_API char *r_str_prepend(char *ptr, const char *string) {
@@ -876,8 +872,6 @@ R_API char *r_str_prepend(char *ptr, const char *string) {
 	if (!ptr) {
 		return strdup (string);
 	}
-	//plen = r_str_len_utf8 (ptr);
-	//slen = r_str_len_utf8 (string);
 	plen = strlen (ptr);
 	slen = strlen (string);
 	ptr = realloc (ptr, slen + plen + 1);
@@ -912,7 +906,7 @@ R_API char *r_str_append(char *ptr, const char *string) {
 	if (string && !ptr) {
 		return strdup (string);
 	}
-	if (!string) {
+	if (R_STR_ISEMPTY (string)) {
 		return ptr;
 	}
 	int plen = strlen (ptr);
@@ -1182,6 +1176,15 @@ R_API int r_str_unescape(char *buf) {
 		case 'f':
 			buf[i] = 0x0c;
 			break;
+		case '"':
+			buf[i] = '"';
+			break;
+		case '\'':
+			buf[i] = '\'';
+			break;
+		case '`':
+			buf[i] = '`';
+			break;
 		case 'x':
 			err = ch2 = ch = 0;
 			if (!buf[i + 2] || !buf[i + 3]) {
@@ -1197,6 +1200,9 @@ R_API int r_str_unescape(char *buf) {
 			buf[i] = (ch << 4) + ch2;
 			esc_seq_len = 4;
 			break;
+		case '$':
+			buf[i] = '$';
+			break;
 		default:
 			if (IS_OCTAL (buf[i + 1])) {
 				int num_digits = 1;
@@ -1211,8 +1217,7 @@ R_API int r_str_unescape(char *buf) {
 				}
 				esc_seq_len = 1 + num_digits;
 			} else {
-				eprintf ("Error: Unknown escape sequence.\n");
-				return 0; // -1?
+				esc_seq_len = 1;
 			}
 		}
 		memmove (buf + i + 1, buf + i + esc_seq_len, strlen (buf + i + esc_seq_len) + 1);
@@ -1376,10 +1381,89 @@ out:
 	return new_buf;
 }
 
+/* hex-escape unprintable characters in a raw buffer (null-safe) */
+R_API char *r_str_escape_raw(const ut8 *buf, int sz) {
+	r_return_val_if_fail (buf, NULL);
+
+	/* Worst case scenario, we convert every byte to a \xhh escape */
+	char *new_buf = malloc (1 + sz * 4);
+	if (!new_buf) {
+		return NULL;
+	}
+	char *q = new_buf;
+	int i;
+	for (i = 0; i < sz; i++) {
+		r_str_byte_escape ((char *)&buf[i], &q, false, false, true);
+	}
+	*q = '\0';
+	return new_buf;
+}
+
 R_API char *r_str_escape(const char *buf) {
 	return r_str_escape_ (buf, false, true, true, false, true);
 }
 
+R_API char *r_str_sanitize_r2(const char *buf) {
+	r_return_val_if_fail (buf, NULL);
+	char *new_buf = malloc (1 + strlen (buf) * 2);
+	if (!new_buf) {
+		return NULL;
+	}
+	const char *p = buf;
+	char *q = new_buf;
+	while (*p) {
+		switch (*p) {
+		case ';':
+		case '$':
+		case '`':
+		case '\\':
+		case '"':
+			*q++ = ' ';
+			p++;
+			break;
+		default:
+			*q++ = *p++;
+			break;
+		}
+	}
+	*q = '\0';
+	return new_buf;
+}
+
+R_API char *r_str_escape_sql(const char *buf) {
+	r_return_val_if_fail (buf, NULL);
+	char *res = r_str_replace (strdup (buf), "'", "\\'", true);
+	return res;
+}
+
+// Return MUST BE surrounded by double-quotes
+R_API char *r_str_escape_sh(const char *buf) {
+	r_return_val_if_fail (buf, NULL);
+	char *new_buf = malloc (1 + strlen (buf) * 2);
+	if (!new_buf) {
+		return NULL;
+	}
+	const char *p = buf;
+	char *q = new_buf;
+	while (*p) {
+		switch (*p) {
+#if __UNIX__
+		case '$':
+		case '`':
+#endif
+		case '\\':
+		case '"':
+			*q++ = '\\';
+			/* FALLTHRU */
+		default:
+			*q++ = *p++;
+			break;
+		}
+	}
+	*q = '\0';
+	return new_buf;
+}
+
 R_API char *r_str_escape_dot(const char *buf) {
 	return r_str_escape_ (buf, true, true, true, false, true);
 }
@@ -1388,7 +1472,7 @@ R_API char *r_str_escape_latin1(const char *buf, bool show_asciidot, bool esc_bs
 	return r_str_escape_ (buf, false, colors, !colors, show_asciidot, esc_bslash);
 }
 
-static char *r_str_escape_utf(const char *buf, int buf_size, RStrEnc enc, bool show_asciidot, bool esc_bslash) {
+static char *r_str_escape_utf(const char *buf, int buf_size, RStrEnc enc, bool show_asciidot, bool esc_bslash, bool keep_printable) {
 	char *new_buf, *q;
 	const char *p, *end;
 	RRune ch;
@@ -1451,11 +1535,15 @@ static char *r_str_escape_utf(const char *buf, int buf_size, RStrEnc enc, bool s
 		if (show_asciidot && !IS_PRINTABLE(ch)) {
 			*q++ = '.';
 		} else if (ch_bytes > 1) {
-			*q++ = '\\';
-			*q++ = ch_bytes == 4 ? 'U' : 'u';
-			for (i = ch_bytes == 4 ? 6 : 2; i >= 0; i -= 2) {
-				*q++ = "0123456789abcdef"[ch >> 4 * (i + 1) & 0xf];
-				*q++ = "0123456789abcdef"[ch >> 4 * i & 0xf];
+			if (keep_printable) {
+				q += r_utf8_encode ((ut8 *)q, ch);
+			} else {
+				*q++ = '\\';
+				*q++ = ch_bytes == 4 ? 'U' : 'u';
+				for (i = ch_bytes == 4 ? 6 : 2; i >= 0; i -= 2) {
+					*q++ = "0123456789abcdef"[ch >> 4 * (i + 1) & 0xf];
+					*q++ = "0123456789abcdef"[ch >> 4 * i & 0xf];
+				}
 			}
 		} else {
 			int offset = enc == R_STRING_ENC_UTF16BE ? 1 : enc == R_STRING_ENC_UTF32BE ? 3 : 0;
@@ -1479,27 +1567,174 @@ static char *r_str_escape_utf(const char *buf, int buf_size, RStrEnc enc, bool s
 }
 
 R_API char *r_str_escape_utf8(const char *buf, bool show_asciidot, bool esc_bslash) {
-	return r_str_escape_utf (buf, -1, R_STRING_ENC_UTF8, show_asciidot, esc_bslash);
+	return r_str_escape_utf (buf, -1, R_STRING_ENC_UTF8, show_asciidot, esc_bslash, false);
+}
+
+R_API char *r_str_escape_utf8_keep_printable(const char *buf, bool show_asciidot, bool esc_bslash) {
+	return r_str_escape_utf (buf, -1, R_STRING_ENC_UTF8, show_asciidot, esc_bslash, true);
 }
 
 R_API char *r_str_escape_utf16le(const char *buf, int buf_size, bool show_asciidot, bool esc_bslash) {
-	return r_str_escape_utf (buf, buf_size, R_STRING_ENC_UTF16LE, show_asciidot, esc_bslash);
+	return r_str_escape_utf (buf, buf_size, R_STRING_ENC_UTF16LE, show_asciidot, esc_bslash, false);
 }
 
 R_API char *r_str_escape_utf32le(const char *buf, int buf_size, bool show_asciidot, bool esc_bslash) {
-	return r_str_escape_utf (buf, buf_size, R_STRING_ENC_UTF32LE, show_asciidot, esc_bslash);
+	return r_str_escape_utf (buf, buf_size, R_STRING_ENC_UTF32LE, show_asciidot, esc_bslash, false);
 }
 
 R_API char *r_str_escape_utf16be(const char *buf, int buf_size, bool show_asciidot, bool esc_bslash) {
-	return r_str_escape_utf (buf, buf_size, R_STRING_ENC_UTF16BE, show_asciidot, esc_bslash);
+	return r_str_escape_utf (buf, buf_size, R_STRING_ENC_UTF16BE, show_asciidot, esc_bslash, false);
 }
 
 R_API char *r_str_escape_utf32be(const char *buf, int buf_size, bool show_asciidot, bool esc_bslash) {
-	return r_str_escape_utf (buf, buf_size, R_STRING_ENC_UTF32BE, show_asciidot, esc_bslash);
+	return r_str_escape_utf (buf, buf_size, R_STRING_ENC_UTF32BE, show_asciidot, esc_bslash, false);
 }
 
-// JSON has special escaping requirements
-// TODO: merge with r_str_escape_utf() and r_str_byte_escape() using RStrEsc
+R_API char *r_str_encoded_json(const char *buf, int buf_size, int encoding) {
+	r_return_val_if_fail (buf, NULL);
+	size_t buf_sz = buf_size < 0 ? strlen (buf) : buf_size;
+	char *encoded_str;
+
+	if (encoding == PJ_ENCODING_STR_BASE64) {
+		encoded_str = r_base64_encode_dyn (buf, buf_sz);
+	} else if (encoding == PJ_ENCODING_STR_HEX || encoding == PJ_ENCODING_STR_ARRAY) {
+		size_t loop = 0;
+		size_t i = 0;
+		size_t increment = encoding == PJ_ENCODING_STR_ARRAY ? 4 : 2;
+
+		if (!SZT_MUL_OVFCHK (((buf_sz * increment) + 1), SZT_MAX)) {
+			return NULL;
+		}
+		size_t new_sz = (buf_sz * increment) + 1;
+
+		encoded_str = malloc (new_sz);
+		if (!encoded_str) {
+			return NULL;
+		}
+
+		const char *format = encoding == PJ_ENCODING_STR_ARRAY ? "%03u," : "%02X";
+		while (buf[loop] != '\0' && i < (new_sz - 1)) {
+			snprintf (encoded_str + i, new_sz - i, format, (ut8) buf[loop]);
+			loop++;
+			i += increment;
+		}
+		if (encoding == PJ_ENCODING_STR_ARRAY && i) {
+			// get rid of the trailing comma
+			encoded_str[i - 1] = '\0';
+		} else {
+			encoded_str[i] = '\0';
+		}
+	} else if (encoding == PJ_ENCODING_STR_STRIP) {
+		encoded_str = r_str_escape_utf8_for_json_strip (buf, buf_sz);
+	} else {
+		encoded_str = r_str_escape_utf8_for_json (buf, buf_sz);
+	}
+	return encoded_str;
+}
+
+R_API char *r_str_escape_utf8_for_json_strip(const char *buf, int buf_size) {
+	char *new_buf, *q;
+	const char *p, *end;
+	RRune ch;
+	int i, len, ch_bytes;
+
+	if (!buf) {
+		return NULL;
+	}
+	len = buf_size < 0 ? strlen (buf) : buf_size;
+	end = buf + len;
+	/* Worst case scenario, we convert every byte to \u00hh */
+	new_buf = malloc (1 + (len * 6));
+	if (!new_buf) {
+		return NULL;
+	}
+	p = buf;
+	q = new_buf;
+	while (p < end) {
+		ch_bytes = r_utf8_decode ((ut8 *)p, end - p, &ch);
+		if (ch_bytes == 1) {
+			switch (*p) {
+			case '\n':
+				*q++ = '\\';
+				*q++ = 'n';
+				break;
+			case '\r':
+				*q++ = '\\';
+				*q++ = 'r';
+				break;
+			case '\\':
+				*q++ = '\\';
+				*q++ = '\\';
+				break;
+			case '\t':
+				*q++ = '\\';
+				*q++ = 't';
+				break;
+			case '"' :
+				*q++ = '\\';
+				*q++ = '"';
+				break;
+			case '\f':
+				*q++ = '\\';
+				*q++ = 'f';
+				break;
+			case '\b':
+				*q++ = '\\';
+				*q++ = 'b';
+				break;
+			default:
+				if (IS_PRINTABLE (*p)) {
+					*q++ = *p;
+				}
+			}
+		} else if (ch_bytes == 4) {
+			if (r_isprint (ch)) {
+				// Assumes buf is UTF8-encoded
+				for (i = 0; i < ch_bytes; i++) {
+					*q++ = *(p + i);
+				}
+			} else {
+				RRune high, low;
+				ch -= 0x10000;
+				high = 0xd800 + (ch >> 10 & 0x3ff);
+				low = 0xdc00 + (ch & 0x3ff);
+				*q++ = '\\';
+				*q++ = 'u';
+				for (i = 2; i >= 0; i -= 2) {
+					*q++ = "0123456789abcdef"[high >> 4 * (i + 1) & 0xf];
+					*q++ = "0123456789abcdef"[high >> 4 * i & 0xf];
+				}
+				*q++ = '\\';
+				*q++ = 'u';
+				for (i = 2; i >= 0; i -= 2) {
+					*q++ = "0123456789abcdef"[low >> 4 * (i + 1) & 0xf];
+					*q++ = "0123456789abcdef"[low >> 4 * i & 0xf];
+				}
+			}
+		} else if (ch_bytes > 1) {
+			if (r_isprint (ch)) {
+				// Assumes buf is UTF8-encoded
+				for (i = 0; i < ch_bytes; i++) {
+					*q++ = *(p + i);
+				}
+			} else {
+				*q++ = '\\';
+				*q++ = 'u';
+				for (i = 2; i >= 0; i -= 2) {
+					*q++ = "0123456789abcdef"[ch >> 4 * (i + 1) & 0xf];
+					*q++ = "0123456789abcdef"[ch >> 4 * i & 0xf];
+				}
+			}
+		} else {
+			ch_bytes = 1;
+		}
+		p += ch_bytes;
+	}
+	*q = '\0';
+	return new_buf;
+}
+
+
 R_API char *r_str_escape_utf8_for_json(const char *buf, int buf_size) {
 	char *new_buf, *q;
 	const char *p, *end;
@@ -1534,12 +1769,6 @@ R_API char *r_str_escape_utf8_for_json(const char *buf, int buf_size) {
 				*q++ = '\\';
 				*q++ = '\\';
 				break;
-#if 0
-			case '/': /* has 2-char esc seq in JSON spec, but escaping is optional */
-				*q++ = '\\';
-				*q++ = '/';
-				break;
-#endif
 			case '\t':
 				*q++ = '\\';
 				*q++ = 't';
@@ -1621,8 +1850,68 @@ R_API char *r_str_escape_utf8_for_json(const char *buf, int buf_size) {
 	return new_buf;
 }
 
-static int __str_ansi_length (char const *str) {
-	int i = 1;
+// http://daviddeley.com/autohotkey/parameters/parameters.htm#WINCRULES
+// https://docs.microsoft.com/en-us/cpp/cpp/main-function-command-line-args?redirectedfrom=MSDN&view=vs-2019#parsing-c-command-line-arguments
+R_API char *r_str_format_msvc_argv(size_t argc, const char **argv) {
+	RStrBuf sb;
+	r_strbuf_init (&sb);
+
+	size_t i;
+	for (i = 0; i < argc; i++) {
+		if (i > 0) {
+			r_strbuf_append (&sb, " ");
+		}
+		const char *arg = argv[i];
+		bool must_escape = strchr (arg, '\"') != NULL;
+		bool must_quote = strpbrk (arg, " \t") != NULL || !*arg;
+		if (!must_escape && must_quote && *arg && arg[strlen (arg) - 1] == '\\') {
+			// if the last char is a bs and we would quote it, we must also escape
+			must_escape = true;
+		}
+		if (must_quote) {
+			r_strbuf_append (&sb, "\"");
+		}
+		if (must_escape) {
+			size_t bs_count = 0; // bullshit counter
+			for (; *arg; arg++) {
+				switch (*arg) {
+				case '\"':
+					for (; bs_count; bs_count--) {
+						// backslashes must be escaped iff they precede a "
+						// so just duplicate the number of backslashes already printed
+						r_strbuf_append (&sb, "\\");
+					}
+					r_strbuf_append (&sb, "\\\"");
+					break;
+				case '\\':
+					bs_count++;
+					r_strbuf_append (&sb, "\\");
+					break;
+				default:
+					bs_count = 0;
+					r_strbuf_append_n (&sb, arg, 1);
+					break;
+				}
+			}
+			if (must_quote) {
+				// there will be a quote after this so we have to escape bs here as well
+				for (; bs_count; bs_count--) {
+					r_strbuf_append (&sb, "\\");
+				}
+			}
+		} else {
+			r_strbuf_append (&sb, arg);
+		}
+		if (must_quote) {
+			r_strbuf_append (&sb, "\"");
+		}
+	}
+
+	return r_strbuf_drain_nofree (&sb);
+}
+
+static size_t __str_ansi_length(char const *str) {
+	size_t i = 1;
 	if (str[0] == 0x1b) {
 		if (str[1] == '[') {
 			i++;
@@ -1642,11 +1931,11 @@ static int __str_ansi_length (char const *str) {
 }
 
 /* ansi helpers */
-R_API int r_str_ansi_nlen(const char *str, int slen) {
-	int i = 0, len = 0;
+R_API size_t r_str_ansi_nlen(const char *str, size_t slen) {
+	size_t i = 0, len = 0;
 	if (slen > 0) {
 		while (str[i] && i < slen) {
-			int chlen = __str_ansi_length (str + i);
+			size_t chlen = __str_ansi_length (str + i);
 			if (chlen == 1) {
 				len ++;
 			}
@@ -1655,23 +1944,23 @@ R_API int r_str_ansi_nlen(const char *str, int slen) {
 		return len > 0 ? len: 1;
 	}
 	while (str[i]) {
-		int chlen = __str_ansi_length (str + i);
+		size_t chlen = __str_ansi_length (str + i);
 		if (chlen == 1) {
 			len ++;
 		}
 		i += chlen;
 	}
-	return len > 0 ? len: 1;
+	return len; // len > 0 ? len: 1;
 }
 
-R_API int r_str_ansi_len(const char *str) {
+R_API size_t r_str_ansi_len(const char *str) {
 	return r_str_ansi_nlen (str, 0);
 }
 
-R_API int r_str_nlen(const char *str, int n) {
-	int len = 0;
+R_API size_t r_str_nlen(const char *str, int n) {
+	size_t len = 0;
 	if (str) {
-		while (*str && n > 0) {
+		while (n > 0 && *str) {
 			len++;
 			str++;
 			n--;
@@ -1682,8 +1971,8 @@ R_API int r_str_nlen(const char *str, int n) {
 
 //to handle wide string as well
 //XXX can be error prone
-R_API int r_str_nlen_w(const char *str, int n) {
-	int len = 0;
+R_API size_t r_str_nlen_w(const char *str, int n) {
+	size_t len = 0;
 	if (str) {
 		while (*str && n > 0) {
 			len++;
@@ -1714,7 +2003,7 @@ R_API bool r_str_is_ascii(const char *str) {
 	return true;
 }
 
-R_API int r_str_is_printable(const char *str) {
+R_API bool r_str_is_printable(const char *str) {
 	while (*str) {
 		int ulen = r_utf8_decode ((const ut8*)str, strlen (str), NULL);
 		if (ulen > 1) {
@@ -1722,14 +2011,14 @@ R_API int r_str_is_printable(const char *str) {
 			continue;
 		}
 		if (!IS_PRINTABLE (*str)) {
-			return 0;
+			return false;
 		}
 		str++;
 	}
-	return 1;
+	return true;
 }
 
-R_API int r_str_is_printable_limited(const char *str, int size) {
+R_API bool r_str_is_printable_limited(const char *str, int size) {
 	while (size > 0 && *str) {
 		int ulen = r_utf8_decode ((const ut8*)str, strlen (str), NULL);
 		if (ulen > 1) {
@@ -1737,12 +2026,12 @@ R_API int r_str_is_printable_limited(const char *str, int size) {
 			continue;
 		}
 		if (!IS_PRINTABLE (*str)) {
-			return 0;
+			return false;
 		}
 		str++;
 		size--;
 	}
-	return 1;
+	return true;
 }
 
 R_API bool r_str_is_printable_incl_newlines(const char *str) {
@@ -1763,8 +2052,8 @@ R_API bool r_str_is_printable_incl_newlines(const char *str) {
 }
 
 // Length in chars of a wide string (find better name?)
-R_API int r_wstr_clen(const char *s) {
-	int len = 0;
+R_API size_t r_wstr_clen(const char *s) {
+	size_t len = 0;
 	if (!*s++) {
 		return 0;
 	}
@@ -1774,10 +2063,15 @@ R_API int r_wstr_clen(const char *s) {
 	return len + 1;
 }
 
-R_API const char *r_str_ansi_chrn(const char *str, int n) {
+// TODO: rename to r_str_ansi_at() ? or find better name?
+R_API const char *r_str_ansi_chrn(const char *str, size_t n) {
+#if 0
+	size_t pos = r_str_ansi_nlen (str, at);
+	return str + pos;
+#endif
 	int len, i, li;
 	for (li = i = len = 0; str[i] && (n != len); i++) {
-		int chlen = __str_ansi_length (str + i);
+		size_t chlen = __str_ansi_length (str + i);
 		if (chlen > 1) {
 			i += chlen - 1;
 		} else {
@@ -1822,7 +2116,7 @@ R_API int r_str_ansi_filter(char *str, char **out, int **cposs, int len) {
 
 	for (i = j = 0; i < len; i++) {
 		if (tmp[i] == 0x1b) {
-			int chlen = __str_ansi_length (str + i);
+			size_t chlen = __str_ansi_length (str + i);
 			if (chlen > 1) {
 				i += chlen;
 				i--;
@@ -1878,7 +2172,6 @@ R_API char *r_str_ansi_crop(const char *str, ut32 x, ut32 y, ut32 x2, ut32 y2) {
 			r--;
 			break;
 		}
-
 		if (*str == '\n') {
 			if (ch >= y && ch < y2) {
 				const char *reset = Color_RESET "\n";
@@ -1941,22 +2234,21 @@ R_API char *r_str_ansi_crop(const char *str, ut32 x, ut32 y, ut32 x2, ut32 y2) {
 	return ret;
 }
 
-R_API int r_str_utf8_codepoint (const char* s, int left) {
-	bool safe = left >= 0;
+R_API size_t r_str_utf8_codepoint(const char* s, size_t left) {
 	if ((*s & 0x80) != 0x80) {
 		return 0;
-	} else if ((*s & 0xe0) == 0xc0 && (safe ? left >= 1 : *(s + 1))) {
+	} else if ((*s & 0xe0) == 0xc0 && left >= 1) {
 		return ((*s & 0x1f) << 6) + (*(s + 1) & 0x3f);
-	} else if ((*s & 0xf0) == 0xe0 && (safe ? left >= 2 : (*(s + 1) && *(s + 2)))) {
+	} else if ((*s & 0xf0) == 0xe0 && left >= 2) {
 		return ((*s & 0xf) << 12) + ((*(s + 1) & 0x3f) << 6) + (*(s + 2) & 0x3f);
-	} else if ((*s & 0xf8) == 0xf0 && (safe ? left >= 3 : (*(s + 1) && *(s + 2) && *(s + 3)))) {
+	} else if ((*s & 0xf8) == 0xf0 && left >= 3) {
 		return ((*s & 0x7) << 18) + ((*(s + 1) & 0x3f) << 12) + ((*(s + 2) & 0x3f) << 6) + (*(s + 3) & 0x3f);
 	}
 	return 0;
 }
 
-R_API bool r_str_char_fullwidth (const char* s, int left) {
-	int codepoint = r_str_utf8_codepoint (s, left);
+R_API bool r_str_char_fullwidth(const char* s, size_t left) {
+	size_t codepoint = r_str_utf8_codepoint (s, left);
 	return (codepoint >= 0x1100 &&
 		 (codepoint <= 0x115f ||                  /* Hangul Jamo init. consonants */
 			  codepoint == 0x2329 || codepoint == 0x232a ||
@@ -1970,7 +2262,6 @@ R_API bool r_str_char_fullwidth (const char* s, int left) {
 		 R_BETWEEN (0xffe0, codepoint, 0xffe6) ||
 		 R_BETWEEN (0x20000, codepoint, 0x2fffd) ||
 		 R_BETWEEN (0x30000, codepoint, 0x3fffd)));
-
 }
 
 /**
@@ -1978,10 +2269,10 @@ R_API bool r_str_char_fullwidth (const char* s, int left) {
  * Returns 1 in case of ASCII
  * str - Pointer to buffer
  */
-R_API int r_str_utf8_charsize(const char *str) {
+R_API size_t r_str_utf8_charsize(const char *str) {
 	r_return_val_if_fail (str, 0);
-	int size = 0;
-	int length = strlen (str);
+	size_t size = 0;
+	size_t length = strlen (str);
 	while (size < length && size < 5) {
 		size++;
 		if ((str[size] & 0xc0) != 0x80) {
@@ -1997,10 +2288,11 @@ R_API int r_str_utf8_charsize(const char *str) {
  * str - Pointer to leading utf8 char
  * prev_len - Length in bytes of the buffer until str
  */
-R_API int r_str_utf8_charsize_prev(const char *str, int prev_len) {
+R_API size_t r_str_utf8_charsize_prev(const char *str, int prev_len) {
 	r_return_val_if_fail (str, 0);
-	int size = 0, pos = 0;
-	while (size < prev_len && size < 5) {
+	int pos = 0;
+	size_t size = 0, minsize = R_MIN (5, prev_len);
+	while (size < minsize) {
 		size++;
 		if ((str[--pos] & 0xc0) != 0x80) {
 			break;
@@ -2014,9 +2306,9 @@ R_API int r_str_utf8_charsize_prev(const char *str, int prev_len) {
  * Returns 1 in case of ASCII
  * str - Pointer to buffer
  */
-R_API int r_str_utf8_charsize_last(const char *str) {
+R_API size_t r_str_utf8_charsize_last(const char *str) {
 	r_return_val_if_fail (str, 0);
-	int len = strlen (str);
+	size_t len = strlen (str);
 	return r_str_utf8_charsize_prev (str + len, len);
 }
 
@@ -2034,7 +2326,7 @@ R_API void r_str_filter_zeroline(char *str, int len) {
 }
 
 R_API void r_str_filter(char *str, int len) {
-	int i;
+	size_t i;
 	if (len < 1) {
 		len = strlen (str);
 	}
@@ -2046,55 +2338,45 @@ R_API void r_str_filter(char *str, int len) {
 }
 
 R_API bool r_str_glob(const char* str, const char *glob) {
-        const char* cp = NULL, *mp = NULL;
-        if (!glob || !strcmp (glob, "*")) {
-                return true;
-        }
-        if (!strchr (glob, '*')) {
-                if (*glob == '^') {
-                        glob++;
-                        while (*str) {
-                                if (*glob != *str) {
-                                        return false;
-                                }
-                                if (!*++glob) {
-                                        return true;
-                                }
-                                str++;
-                        }
-                } else {
-                        return strstr (str, glob) != NULL;
-                }
-        }
-        if (*glob == '^') {
-                glob++;
-        }
-        while (*str && (*glob != '*')) {
-                if (*glob != *str) {
-                        return false;
-                }
-                glob++;
-                str++;
-        }
-        while (*str) {
-                if (*glob == '*') {
-                        if (!*++glob) {
-                                return true;
-                        }
-                        mp = glob;
-                        cp = str + 1;
-                } else if (*glob == *str) {
-                        glob++;
-                        str++;
-                } else {
-                        glob = mp;
-                        str = cp++;
-                }
-        }
-        while (*glob == '*') {
-                ++glob;
-        }
-        return (*glob == '\x00');
+	if (!glob) {
+		return true;
+	}
+	char* begin = strchr (glob, '^');
+	if (begin) {
+		glob = ++begin;
+	}
+	while (*str) {
+		if (!*glob) {
+			return true;
+		}
+		switch (*glob) {
+		case '*':
+			if (!*++glob) {
+				return true;
+			}
+			while (*str) {
+				if (*glob == *str) {
+					break;
+				}
+				str++;
+			}
+			break;
+		case '$':
+			return (*++glob == '\x00');
+		case '?':
+			str++;
+			glob++;
+			break;
+		default:
+			if (*glob != *str) {
+				return false;
+			}
+			str++;
+			glob++;
+		}
+	}
+	while (*glob == '*') { ++glob; }
+	return ((*glob == '$' && !*glob++)  || !*glob);
 }
 
 // Escape the string arg so that it is parsed as a single argument by r_str_argv
@@ -2202,7 +2484,7 @@ R_API char **r_str_argv(const char *cmdline, int *_argc) {
 	int args_current = 0; // Current character index in  args
 	int arg_begin = 0; // Index of the first character of the current argument in args
 
-	if (!cmdline || argv_len < 1) {
+	if (!cmdline) {
 		return NULL;
 	}
 
@@ -2355,6 +2637,46 @@ R_API const char *r_str_firstbut(const char *s, char ch, const char *but) {
 	return NULL;
 }
 
+R_API const char *r_str_firstbut_escape(const char *s, char ch, const char *but) {
+	int idx, _b = 0;
+	ut8 *b = (ut8*)&_b;
+	const char *isbut, *p;
+	const int bsz = sizeof (_b) * 8;
+	if (!but) {
+		return strchr (s, ch);
+	}
+	if (strlen (but) >= bsz) {
+		eprintf ("r_str_firstbut: but string too long\n");
+		return NULL;
+	}
+	for (p = s; *p; p++) {
+		if (*p == '\\') {
+			p++;
+			if (*p == ch || strchr(but, *p)) {
+				if (!*p) {
+					break;
+				}
+				continue;
+			} else if (!*p) {
+				break;
+			}
+		}
+		isbut = strchr (but, *p);
+		if (isbut) {
+			idx = (int)(size_t)(isbut - but);
+			_b = R_BIT_TOGGLE (b, idx);
+			if (_b && (_b & (_b - 1))) {
+				_b = R_BIT_TOGGLE (b, idx); // cancel a but char if a but is already toggle
+			}
+			continue;
+		}
+		if (*p == ch && !_b) {
+			return p;
+		}
+	}
+	return NULL;
+}
+
 R_API const char *r_str_lastbut(const char *s, char ch, const char *but) {
 	int idx, _b = 0;
 	ut8 *b = (ut8*)&_b;
@@ -2382,8 +2704,8 @@ R_API const char *r_str_lastbut(const char *s, char ch, const char *but) {
 }
 
 // Must be merged inside strlen
-R_API int r_str_len_utf8char(const char *s, int left) {
-	int i = 1;
+R_API size_t r_str_len_utf8char(const char *s, int left) {
+	size_t i = 1;
 	while (s[i] && (!left || i<left)) {
 		if ((s[i] & 0xc0) != 0x80) {
 			i++;
@@ -2394,8 +2716,8 @@ R_API int r_str_len_utf8char(const char *s, int left) {
 	return i;
 }
 
-R_API int r_str_len_utf8(const char *s) {
-	int i = 0, j = 0, fullwidths = 0;
+R_API size_t r_str_len_utf8(const char *s) {
+	size_t i = 0, j = 0, fullwidths = 0;
 	while (s[i]) {
 		if ((s[i] & 0xc0) != 0x80) {
 			j++;
@@ -2408,11 +2730,11 @@ R_API int r_str_len_utf8(const char *s) {
 	return j + fullwidths;
 }
 
-R_API int r_str_len_utf8_ansi(const char *str) {
+R_API size_t r_str_len_utf8_ansi(const char *str) {
 	int i = 0, len = 0, fullwidths = 0;
 	while (str[i]) {
 		char ch = str[i];
-		int chlen = __str_ansi_length (str + i);
+		size_t chlen = __str_ansi_length (str + i);
 		if (chlen > 1) {
 			i += chlen - 1;
 		} else if ((ch & 0xc0) != 0x80) { // utf8
@@ -2427,7 +2749,7 @@ R_API int r_str_len_utf8_ansi(const char *str) {
 }
 
 // XXX must find across the ansi tags, as well as support utf8
-R_API const char *r_strstr_ansi (const char *a, const char *b) {
+R_API const char *r_strstr_ansi(const char *a, const char *b) {
 	const char *ch, *p = a;
 	do {
 		ch = strchr (p, '\x1b');
@@ -2732,136 +3054,6 @@ R_API char *r_str_utf16_encode(const char *s, int len) {
 	return tmp;
 }
 
-// TODO: merge print inside rutil
-/* hack from print */
-R_API int r_print_format_length(const char *fmt) {
-	int nargs, i, j, idx, times, endian;
-	char *args, *bracket, tmp, last = 0;
-	const char *arg = fmt;
-	const char *argend = arg+strlen (fmt);
-	char namefmt[8];
-	int viewflags = 0;
-	endian = i = j = 0;
-
-	while (*arg && IS_WHITECHAR (*arg)) {
-		arg++;
-	}
-	/* get times */
-	times = atoi (arg);
-	if (times > 0) {
-		while ((*arg >= '0' && *arg <= '9')) {
-			arg++;
-		}
-	}
-	bracket = strchr (arg,'{');
-	if (bracket) {
-		char *end = strchr (arg,'}');
-		if (!end) {
-			eprintf ("No end bracket. Try pm {ecx}b @ esi\n");
-			return 0;
-		}
-		*end='\0';
-		times = r_num_math (NULL, bracket+1);
-		arg = end + 1;
-	}
-
-	if (*arg == '\0') {
-		return 0;
-	}
-	/* get args */
-	args = strchr (arg, ' ');
-	if (args) {
-		int l = 0, maxl = 0;
-		argend = args;
-		args = strdup (args+1);
-		nargs = r_str_word_set0 (args+1);
-		if (!nargs) {
-			R_FREE (args);
-		}
-		for (i = 0; i<nargs; i++) {
-			int len = strlen (r_str_word_get0 (args + 1, i));
-			if (len > maxl) {
-				maxl = len;
-			}
-		}
-		l++;
-		snprintf (namefmt, sizeof (namefmt), "%%%ds : ", maxl);
-	}
-
-	/* go format */
-	i = 0;
-	if (!times) {
-		times = 1;
-	}
-	for (; times; times--) { // repeat N times
-		const char * orig = arg;
-		arg = orig;
-		for (idx = 0; arg < argend && *arg; idx++, arg++) {
-			tmp = *arg;
-		feed_me_again:
-			if (!tmp && last != '*') {
-				break;
-			}
-			/* skip chars */
-			switch (tmp) {
-			case '*':
-				if (i <= 0) {
-					break;
-				}
-				tmp = last;
-				arg--;
-				idx--;
-				goto feed_me_again;
-			case '+':
-				idx--;
-				viewflags = !viewflags;
-				continue;
-			case 'e': // tmp swap endian
-				idx--;
-				endian ^= 1;
-				continue;
-			case '.': // skip char
-				i++;
-				idx--;
-				continue;
-			case 'p':
-				tmp = (sizeof (void*) == 8)? 'q': 'x';
-				break;
-			case '?': // help
-				idx--;
-				free (args);
-				return 0;
-			}
-			switch (tmp) {
-			case 'e': i += 8; break;
-			case 'q': i += 8; break;
-			case 'b': i++; break;
-			case 'c': i++; break;
-			case 'B': i += 4; break;
-			case 'i': i += 4; break;
-			case 'd': i += 4; break;
-			case 'x': i += 4; break;
-			case 'w':
-			case '1': i += 2; break;
-			case 'z': // XXX unsupported
-			case 'Z': // zero terminated wide string
-				break;
-			case 's': i += 4; break; // S for 8?
-			case 'S': i += 8; break; // S for 8?
-			default:
-				/* ignore unknown chars */
-				break;
-			}
-			last = tmp;
-		}
-		arg = orig;
-	}
-	if (args) {
-		free (args);
-	}
-	return i;
-}
-
 R_API char *r_str_prefix_all(const char *s, const char *pfx) {
 	const char *os = s;
 	char *p;
@@ -3024,6 +3216,39 @@ R_API char *r_str_crop(const char *str, unsigned int x, unsigned int y,
 	return ret;
 }
 
+// TODO: improve loop to wrap by words
+R_API char *r_str_wrap(const char *str, int w) {
+	char *r, *ret;
+	if (w < 1 || !str) {
+		return strdup ("");
+	}
+	size_t r_size = 8 * strlen (str);
+	r = ret = malloc (r_size);
+	char *end = r + r_size;
+	int cw = 0;
+	while (*str && r + 1 < end) {
+		if (*str == '\t') {
+			// skip
+		} else if (*str == '\r') {
+			// skip
+		} else if (*str == '\n') {
+			*r++ = *str++;
+			cw = 0;
+		} else {
+			if (cw > w) {
+				*r++ = '\n';
+				*r++ = *str++;
+				cw = 1;
+			} else {
+				*r++ = *str++;
+				cw++;
+			}
+		}
+	}
+	*r = 0;
+	return ret;
+}
+
 R_API const char * r_str_tok(const char *str1, const char b, size_t len) {
 	const char *p = str1;
 	size_t i = 0;
@@ -3150,39 +3375,35 @@ R_API RList *r_str_split_list(char *str, const char *c, int n)  {
 	return lst;
 }
 
-R_API RList *r_str_split_duplist(const char *_str, const char *c)  {
+R_API RList *r_str_split_duplist(const char *_str, const char *c, bool trim) {
 	r_return_val_if_fail (_str && c, NULL);
 	RList *lst = r_list_newf (free);
 	char *str = strdup (_str);
-
-	char *aux;
-	bool first_loop = true;
-
-	for (;;) {
-		if (first_loop) {
-			aux = strtok (str, c);
-			first_loop = false;
-		} else {
-			aux = strtok (NULL, c);
+	char *aux = str;
+	size_t clen = strlen (c);
+	while (aux) {
+		char *next = strstr (aux, c);
+		if (next) {
+			*next = '\0';
+			next += clen;
 		}
-		if (!aux) {
-			break;
+		if (trim) {
+			r_str_trim (aux);
 		}
-		r_str_trim (aux);
 		r_list_append (lst, strdup (aux));
+		aux = next;
 	}
-
 	free (str);
 	return lst;
 }
 
-R_API int *r_str_split_lines(char *str, int *count) {
+R_API size_t *r_str_split_lines(char *str, size_t *count) {
 	int i;
-	int lines = 0;
+	size_t lines = 0;
 	if (!str) {
 		return NULL;
 	}
-	int *indexes = NULL;
+	size_t *indexes = NULL;
 	// count lines
 	for (i = 0; str[i]; i++) {
 		if (str[i] == '\n') {
@@ -3190,11 +3411,11 @@ R_API int *r_str_split_lines(char *str, int *count) {
 		}
 	}
 	// allocate and set indexes
-	indexes = calloc (sizeof (int), lines + 1);
+	indexes = calloc (sizeof (count[0]), lines + 1);
 	if (!indexes) {
 		return NULL;
 	}
-	int line = 0;
+	size_t line = 0;
 	indexes[line++] = 0;
 	for (i = 0; str[i]; i++) {
 		if (str[i] == '\n') {
@@ -3209,16 +3430,17 @@ R_API int *r_str_split_lines(char *str, int *count) {
 }
 
 R_API bool r_str_isnumber(const char *str) {
-	if (!str || !*str) {
+	if (!str || (!IS_DIGIT (*str) && *str != '-')) {
 		return false;
 	}
-	bool isnum = IS_DIGIT (*str) || *str == '-';
-	while (isnum && *++str) {
+
+	while (*++str) {
 		if (!IS_DIGIT (*str)) {
-			isnum = false;
+			return false;
 		}
 	}
-	return isnum;
+
+	return true;
 }
 
 /* TODO: optimize to start searching by the end of the string */
@@ -3408,14 +3630,14 @@ err_r_str_mb_to_wc:
 }
 
 R_API char* r_str_wc_to_mb_l(const wchar_t *buf, int len) {
+	r_return_val_if_fail (buf, NULL);
 	char *res_buf = NULL;
 	bool fail = true;
-	size_t sz;
 
-	if (!buf || len <= 0) {
+	if (len <= 0) {
 		return NULL;
 	}
-	sz = wcstombs (NULL, buf, len);
+	size_t sz = wcstombs (NULL, buf, 0);
 	if (sz == (size_t)-1) {
 		goto err_r_str_wc_to_mb;
 	}
@@ -3456,6 +3678,10 @@ R_API char *r_str_from_ut64(ut64 val) {
 	if (!str) {
 		return NULL;
 	}
+	while (!*v && i < 8) {
+		v++;
+		i++;
+	}
 	while (i < 8 && *v) {
 		str[i++] = *v++;
 	}
@@ -3465,8 +3691,13 @@ R_API char *r_str_from_ut64(ut64 val) {
 R_API int r_snprintf(char *string, int len, const char *fmt, ...) {
 	va_list ap;
 	va_start (ap, fmt);
-	int ret = vsnprintf (string, len, fmt, ap);
-	string[len - 1] = 0;
+	int ret = 0;
+	if (len > 0) {
+		ret = vsnprintf (string, len, fmt, ap);
+		string[len - 1] = 0;
+	} else {
+		*string = 0;
+	}
 	va_end (ap);
 	return ret;
 }
@@ -3519,6 +3750,21 @@ R_API char *r_str_list_join(RList *str, const char *sep) {
 	return r_strbuf_drain (sb);
 }
 
+R_API char *r_str_array_join(const char **a, size_t n, const char *sep) {
+	RStrBuf *sb = r_strbuf_new ("");
+	size_t i;
+
+	if (n > 0) {
+		r_strbuf_append (sb, a[0]);
+	}
+
+	for (i = 1; i < n; i++) {
+		r_strbuf_append (sb, sep);
+		r_strbuf_append (sb, a[i]);
+	}
+	return r_strbuf_drain (sb);
+}
+
 /* return the number of arguments expected as extra arguments */
 R_API int r_str_fmtargs(const char *fmt) {
 	int n = 0;
@@ -3594,11 +3840,11 @@ R_API char *r_str_scale(const char *s, int w, int h) {
 	int curline = -1;
 	char *linetext = (char*)r_str_pad (' ', w);
 	for (i = 0; i < h; i++) {
-		int zoomedline = i * ((float)rows / h);
+		int zoomedline = (int)(i * ((double)rows / h));
 		const char *srcline = r_list_get_n (lines, zoomedline);
 		int cols = strlen (srcline);
 		for (j = 0; j < w; j++) {
-			int zoomedcol = j * ( (float)cols / w);
+			int zoomedcol = (int)(j * ( (double)cols / w));
 			linetext[j] = srcline[zoomedcol];
 		}
 		if (curline != zoomedline) {
@@ -3610,3 +3856,91 @@ R_API char *r_str_scale(const char *s, int w, int h) {
 	free (str);
 	return r_str_list_join (out, "\n");
 }
+
+// returns value between 0 and 100 about how much different the strings are
+R_API int r_str_distance(const char *a, const char *b) {
+	ut32 distance = 0;
+	double similarity = 0;
+	r_diff_buffers_distance_levenshtein (NULL, (const ut8*)a, strlen (a), (const ut8*)b, strlen (b), &distance, &similarity);
+	return (int)(similarity * 100);;
+}
+
+R_API const char *r_str_str_xy(const char *s, const char *word, const char *prev, int *x, int *y) {
+	r_return_val_if_fail (s && word && x && y, NULL);
+	r_return_val_if_fail (word[0] != '\0' && word[0] != '\n', NULL);
+	const char *src = prev ? prev + 1 : s;
+	const char *d = strstr (src, word);
+	if (!d) {
+		return NULL;
+	}
+	const char *q;
+	for (q = prev ? prev : s; q < d; q++) {
+		if (*q == '\n') {
+			(*y)++;
+			*x = 0;
+
+		} else {
+			(*x)++;
+		}
+	}
+	return d;
+}
+
+
+// version.c
+#include <r_userconf.h>
+#include <r_util.h>
+
+#ifndef R2_GITTAP
+#define R2_GITTAP ""
+#endif
+
+#ifndef R2_GITTIP
+#define R2_GITTIP ""
+#endif
+
+#ifndef R2_BIRTH
+#define R2_BIRTH "unknown"
+#endif
+
+R_API char *r_str_version(const char *program) {
+	char *s = r_str_newf ("%s "R2_VERSION" %d @ "
+			R_SYS_OS"-"
+			R_SYS_ARCH"-%d git.%s\n",
+			program, R2_VERSION_COMMIT,
+			(R_SYS_BITS & 8)? 64: 32,
+			*R2_GITTAP ? R2_GITTAP: "");
+	if (*R2_GITTIP) {
+		s = r_str_appendf (s, "commit: "R2_GITTIP" build: "R2_BIRTH);
+	}
+	return s;
+}
+
+R_API int r_str_size(const char *s, int *rows) {
+	RRune ch;
+	int cols = 0;
+	int h = 0;
+	const char *e = s + strlen (s);
+	int ll = 0;
+	while (*s) {
+		if (*s == '\n') {
+			h++;
+			s++;
+			if (ll > cols) {
+				cols = ll;
+			}
+			ll = 0;
+			continue;
+		}
+		int chsz = r_utf8_decode ((const ut8*)s, e - s, &ch);
+		if (chsz < 1) {
+			chsz = 1;
+		}
+		s += chsz;
+		ll++;
+	}
+	if (rows) {
+		*rows = h;
+	}
+	return cols;
+}
