@@ -1,14 +1,11 @@
-/* radare - LGPL - Copyright 2009-2018 - pancake */
-
-#if __linux__
-#include <time.h>
-#endif
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include <r_userconf.h>
 #include <stdlib.h>
 #include <string.h>
 #if defined(__NetBSD__)
 # include <sys/param.h>
+# include <sys/sysctl.h>
 # if __NetBSD_Prereq__(7,0,0)
 #  define NETBSD_WITH_BACKTRACE
 # endif
@@ -20,6 +17,14 @@
 #  define FREEBSD_WITH_BACKTRACE
 # endif
 #endif
+#if defined(__DragonFly__)
+# include <sys/param.h>
+# include <sys/sysctl.h>
+#endif
+#if defined(__HAIKU__)
+# include <kernel/image.h>
+# include <sys/param.h>
+#endif
 #include <sys/types.h>
 #include <r_types.h>
 #include <r_util.h>
@@ -28,7 +33,7 @@
 static char** env = NULL;
 
 #if (__linux__ && __GNU_LIBRARY__) || defined(NETBSD_WITH_BACKTRACE) || \
-  defined(FREEBSD_WITH_BACKTRACE) || __DragonFly__
+  defined(FREEBSD_WITH_BACKTRACE) || __DragonFly__ || __sun
 # include <execinfo.h>
 #endif
 #if __APPLE__
@@ -53,9 +58,12 @@ int proc_pidpath(int pid, void * buffer, ut32 buffersize);
 #endif
 #if __UNIX__
 # include <sys/utsname.h>
-# include <sys/wait.h>
 # include <sys/stat.h>
 # include <errno.h>
+#ifndef __wasi__
+# include <pwd.h>
+# include <sys/wait.h>
+#endif
 # include <signal.h>
 extern char **environ;
 
@@ -131,7 +139,7 @@ static const struct {const char* name; ut64 bit;} arch_bit_array[] = {
     {NULL, 0}
 };
 
-R_API int r_sys_fork() {
+R_API int r_sys_fork(void) {
 #if HAVE_FORK
 #if __WINDOWS__
 	return -1;
@@ -143,7 +151,15 @@ R_API int r_sys_fork() {
 #endif
 }
 
-#if HAVE_SIGACTION
+#if __WINDOWS__
+R_API int r_sys_sigaction(int *sig, void (*handler) (int)) {
+	return -1;
+}
+#elif __wasi__
+R_API int r_sys_sigaction(int *sig, void (*handler)(int)) {
+	return 0;
+}
+#elif HAVE_SIGACTION
 R_API int r_sys_sigaction(int *sig, void (*handler) (int)) {
 	struct sigaction sigact = { };
 	int ret, i;
@@ -166,19 +182,16 @@ R_API int r_sys_sigaction(int *sig, void (*handler) (int)) {
 			return ret;
 		}
 	}
-
 	return 0;
 }
 #else
-R_API int r_sys_sigaction(int *sig, void (*handler) (int)) {
-	int ret, i;
-
+R_API int r_sys_sigaction(int *sig, void (*handler)(int)) {
 	if (!sig) {
 		return -EINVAL;
 	}
-
+	size_t i;
 	for (i = 0; sig[i] != 0; i++) {
-		ret = signal (sig[i], handler);
+		void (*ret)(int) = signal (sig[i], handler);
 		if (ret == SIG_ERR) {
 			eprintf ("Failed to set signal handler for signal '%d': %s\n", sig[i], strerror(errno));
 			return -1;
@@ -201,19 +214,6 @@ R_API void r_sys_exit(int status, bool nocleanup) {
 	}
 }
 
-/* TODO: import stuff from bininfo/p/bininfo_addr2line */
-/* TODO: check endianness issues here */
-R_API ut64 r_sys_now(void) {
-	ut64 ret;
-	struct timeval now;
-	gettimeofday (&now, NULL);
-	ret = now.tv_sec;
-	ret <<= 20;
-	ret |= now.tv_usec;
-	//(sizeof (now.tv_sec) == 4
-	return ret;
-}
-
 R_API int r_sys_truncate(const char *file, int sz) {
 #if __WINDOWS__
 	int fd = r_sandbox_open (file, O_RDWR, 0644);
@@ -287,6 +287,23 @@ R_API char *r_sys_cmd_strf(const char *fmt, ...) {
 	return ret;
 }
 
+R_API ut8 *r_sys_unxz(const ut8 *buf, size_t len, size_t *olen) {
+	char *err = NULL;
+	ut8 *out = NULL;
+	int _olen = 0;
+	int rc = r_sys_cmd_str_full ("xz -d", (const char *)buf, (int)len, (char **)&out, &_olen, &err);
+	if (rc == 0 || rc == 1) {
+		if (olen) {
+			*olen = (size_t)_olen;
+		}
+		free (err);
+		return out;
+	}
+	free (out);
+	free (err);
+	return NULL;
+}
+
 #ifdef __MAC_10_7
 #define APPLE_WITH_BACKTRACE 1
 #endif
@@ -296,7 +313,7 @@ R_API char *r_sys_cmd_strf(const char *fmt, ...) {
 
 #if (__linux__ && __GNU_LIBRARY__) || (__APPLE__ && APPLE_WITH_BACKTRACE) || \
   defined(NETBSD_WITH_BACKTRACE) || defined(FREEBSD_WITH_BACKTRACE) || \
-  __DragonFly__
+  __DragonFly__ || __sun || __HAIKU__
 #define HAVE_BACKTRACE 1
 #endif
 
@@ -324,11 +341,7 @@ R_API void r_sys_backtrace(void) {
 		printf ("[%d] pc == %p fp == %p\n", depth++, saved_pc, saved_fp);
 	}
 #else
-#ifdef _MSC_VER
 #pragma message ("TODO: r_sys_bt : unimplemented")
-#else
-#warning TODO: r_sys_bt : unimplemented
-#endif
 #endif
 }
 
@@ -353,7 +366,13 @@ R_API int r_sys_usleep(int usecs) {
 	rqtp.tv_nsec = (usecs - (rqtp.tv_sec * 1000000)) * 1000;
 	return clock_nanosleep (CLOCK_MONOTONIC, 0, &rqtp, NULL);
 #elif __UNIX__
+#if defined(__GLIBC__) && defined(__GLIBC_MINOR__) && (__GLIBC__ <= 2) && (__GLIBC_MINOR__ <= 2)
+	// Old versions of GNU libc return void for usleep
+	usleep (usecs);
+	return 0;
+#else
 	return usleep (usecs);
+#endif
 #else
 	// w32 api uses milliseconds
 	usecs /= 1000;
@@ -367,14 +386,12 @@ R_API int r_sys_clearenv(void) {
 #if __APPLE__ && !HAVE_ENVIRON
 	/* do nothing */
 	if (!env) {
-		env = r_sys_get_environ ();
+		r_sys_env_init ();
 		return 0;
 	}
-	if (env) {
-		char **e = env;
-		while (*e) {
-			*e++ = NULL;
-		}
+	char **e = env;
+	while (*e) {
+		*e++ = NULL;
 	}
 #else
 	if (!environ) {
@@ -386,11 +403,7 @@ R_API int r_sys_clearenv(void) {
 #endif
 	return 0;
 #else
-#ifdef _MSC_VER
 #pragma message ("r_sys_clearenv : unimplemented for this platform")
-#else
-#warning r_sys_clearenv : unimplemented for this platform
-#endif
 	return 0;
 #endif
 }
@@ -416,7 +429,7 @@ R_API int r_sys_setenv(const char *key, const char *value) {
 	free (value_);
 	return ret ? 0 : -1;
 #else
-#warning r_sys_setenv : unimplemented for this platform
+#pragma message("r_sys_setenv : unimplemented for this platform")
 	return 0;
 #endif
 }
@@ -425,13 +438,17 @@ R_API int r_sys_setenv(const char *key, const char *value) {
 static char *crash_handler_cmd = NULL;
 
 static void signal_handler(int signum) {
-	char cmd[1024];
 	if (!crash_handler_cmd) {
 		return;
 	}
-	snprintf (cmd, sizeof(cmd) - 1, crash_handler_cmd, getpid ());
-	r_sys_backtrace ();
-	exit (r_sys_cmd (cmd));
+	char *cmd = r_str_newf ("%s %d", crash_handler_cmd, r_sys_getpid ());
+	int rc = 1;
+	if (cmd) {
+		r_sys_backtrace ();
+		rc = r_sys_cmd (cmd);
+		free (cmd);
+	}
+	exit (rc);
 }
 
 static int checkcmd(const char *c) {
@@ -480,7 +497,7 @@ R_API char *r_sys_getenv(const char *key) {
 	if (!key) {
 		return NULL;
 	}
-	envbuf = (LPTSTR)malloc (sizeof (TCHAR) * TMP_BUFSIZE);
+	envbuf = (LPTSTR)calloc (sizeof (TCHAR), TMP_BUFSIZE);
 	if (!envbuf) {
 		goto err_r_sys_get_env;
 	}
@@ -517,8 +534,9 @@ err_r_sys_get_env:
 }
 
 R_API bool r_sys_getenv_asbool(const char *key) {
+	r_return_val_if_fail (key, false);
 	char *env = r_sys_getenv (key);
-	const bool res = (env && *env == '1');
+	const bool res = env && r_str_is_true (env);
 	free (env);
 	return res;
 }
@@ -527,12 +545,22 @@ R_API char *r_sys_getdir(void) {
 #if __WINDOWS__
 	return _getcwd (NULL, 0);
 #else
+#ifdef __GLIBC__
 	return getcwd (NULL, 0);
+#else
+	const size_t maxpathlen = 4096;
+	char *res = calloc (maxpathlen, 1);
+	char *cwd = getcwd (res, maxpathlen);
+	if (!cwd) {
+		free (res);
+	}
+	return cwd;
+#endif
 #endif
 }
 
-R_API int r_sys_chdir(const char *s) {
-	return r_sandbox_chdir (s)==0;
+R_API bool r_sys_chdir(const char *s) {
+	return r_sandbox_chdir (s) == 0;
 }
 
 R_API bool r_sys_aslr(int val) {
@@ -562,38 +590,31 @@ R_API bool r_sys_aslr(int val) {
 		ret = false;
 	}
 #endif
-#endif
-	return ret;
-}
-
-R_API int r_sys_thp_mode(void) {
-#if __linux__
-	const char *thp = "/sys/kernel/mm/transparent_hugepage/enabled";
-	int sz;
-	int ret = 0;
-	char *val = r_file_slurp (thp, &sz);
-	if (val) {
-		if (strstr (val, "[madvise]")) {
-			ret = 1;
-		} else if (strstr (val, "[always]")) {
-			ret = 2;
-		}
-		free (val);
+#elif __NetBSD__
+	size_t vlen = sizeof (val);
+	if (sysctlbyname ("security.pax.aslr.enabled", NULL, 0, &val, vlen) == -1) {
+		eprintf ("Failed to set RVA\n");
+		ret = false;
 	}
-
-	return ret;
-#else
-  return 0;
+#elif __DragonFly__
+	size_t vlen = sizeof (val);
+	if (sysctlbyname ("vm.randomize_mmap", NULL, 0, &val, vlen) == -1) {
+		eprintf ("Failed to set RVA\n");
+		ret = false;
+	}
+#elif __DragonFly__
 #endif
+	return ret;
 }
 
-#if __UNIX__
-R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output, int *len, char **sterr) {
+#if __UNIX__ && HAVE_SYSTEM
+R_API int r_sys_cmd_str_full(const char *cmd, const char *input, int ilen, char **output, int *len, char **sterr) {
 	char *mysterr = NULL;
 	if (!sterr) {
 		sterr = &mysterr;
 	}
-	char buffer[1024], *outputptr = NULL;
+	ut8 buffer[1024];
+	char *outputptr = NULL;
 	char *inputptr = (char *)input;
 	int pid, bytes = 0, status;
 	int sh_in[2], sh_out[2], sh_err[2];
@@ -601,6 +622,9 @@ R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output,
 	if (len) {
 		*len = 0;
 	}
+	if (ilen == -1 && inputptr) {
+		ilen = strlen (inputptr);
+	}
 	if (pipe (sh_in)) {
 		return false;
 	}
@@ -661,6 +685,8 @@ R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output,
 		}
 		// we should handle broken pipes somehow better
 		r_sys_signal (SIGPIPE, SIG_IGN);
+		size_t err_len = 0, out_len = 0;
+		size_t written = 0;
 		for (;;) {
 			fd_set rfds, wfds;
 			int nfd;
@@ -678,37 +704,40 @@ R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output,
 			memset (buffer, 0, sizeof (buffer));
 			nfd = select (sh_err[0] + 1, &rfds, &wfds, NULL, NULL);
 			if (nfd < 0) {
-				break;
+				// eprintf ("nfd %d 2%c", nfd, 10);
 			}
 			if (output && FD_ISSET (sh_out[0], &rfds)) {
-				if (!(bytes = read (sh_out[0], buffer, sizeof (buffer)-1))) {
+				if ((bytes = read (sh_out[0], buffer, sizeof (buffer))) < 1) {
 					break;
 				}
-				buffer[sizeof (buffer) - 1] = '\0';
-				if (len) {
-					*len += bytes;
+				char *tmp = realloc (outputptr, out_len + bytes + 1);
+				if (!tmp) {
+					R_FREE (outputptr);
+					break;
 				}
-				outputptr = r_str_append (outputptr, buffer);
+				outputptr = tmp;
+				memcpy (outputptr + out_len, buffer, bytes);
+				out_len += bytes;
 			} else if (FD_ISSET (sh_err[0], &rfds) && sterr) {
-				if (!read (sh_err[0], buffer, sizeof (buffer)-1)) {
+				if ((bytes = read (sh_err[0], buffer, sizeof (buffer))) < 1) {
 					break;
 				}
-				buffer[sizeof (buffer) - 1] = '\0';
-				*sterr = r_str_append (*sterr, buffer);
-			} else if (FD_ISSET (sh_in[1], &wfds) && inputptr && *inputptr) {
-				int inputptr_len = strlen (inputptr);
-				bytes = write (sh_in[1], inputptr, inputptr_len);
-				if (bytes != inputptr_len) {
+				char *tmp = realloc (*sterr, err_len + bytes + 1);
+				if (!tmp) {
+					R_FREE (*sterr);
 					break;
 				}
-				inputptr += bytes;
-				if (!*inputptr) {
+				*sterr = tmp;
+				memcpy (*sterr + err_len, buffer, bytes);
+				err_len += bytes;
+			} else if (FD_ISSET (sh_in[1], &wfds) && written < ilen) {
+				int inputptr_len = ilen >= 0? ilen - written: strlen (inputptr + written);
+				inputptr_len = R_MIN (inputptr_len, sizeof (buffer));
+				bytes = write (sh_in[1], inputptr + written, inputptr_len);
+				written += bytes;
+				if (written >= ilen) {
 					close (sh_in[1]);
-					/* If neither stdout nor stderr should be captured,
-					 * abort now - nothing more to do for select(). */
-					if (!output && !sterr) {
-						break;
-					}
+					// break;
 				}
 			}
 		}
@@ -728,27 +757,37 @@ R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output,
 			ret = false;
 		}
 
+		if (len) {
+			*len = out_len;
+		}
+		if (*sterr) {
+			(*sterr)[err_len] = 0;
+		}
+		if (outputptr) {
+			outputptr[out_len] = 0;
+		}
 		if (output) {
 			*output = outputptr;
 		} else {
 			free (outputptr);
 		}
+		free (mysterr);
 		return ret;
 	}
 	return false;
 }
 #elif __WINDOWS__
-R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output, int *len, char **sterr) {
-	return r_sys_cmd_str_full_w32 (cmd, input, output, len, sterr);
+R_API int r_sys_cmd_str_full(const char *cmd, const char *input, int ilen, char **output, int *len, char **sterr) {
+	return r_sys_cmd_str_full_w32 (cmd, input, ilen, output, len, sterr);
 }
 #else
-R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output, int *len, char **sterr) {
+R_API int r_sys_cmd_str_full(const char *cmd, const char *input, int ilen, char **output, int *len, char **sterr) {
 	eprintf ("r_sys_cmd_str: not yet implemented for this platform\n");
 	return false;
 }
 #endif
 
-R_API int r_sys_cmdf (const char *fmt, ...) {
+R_API int r_sys_cmdf(const char *fmt, ...) {
 	int ret;
 	char cmd[4096];
 	va_list ap;
@@ -773,11 +812,7 @@ R_API int r_sys_cmdbg (const char *str) {
 	exit (0);
 	return -1;
 #else
-#ifdef _MSC_VER
 #pragma message ("r_sys_cmdbg is not implemented for this platform")
-#else
-#warning r_sys_cmdbg is not implemented for this platform
-#endif
 	return -1;
 #endif
 }
@@ -786,35 +821,12 @@ R_API int r_sys_cmd(const char *str) {
 	if (r_sandbox_enable (0)) {
 		return false;
 	}
-#if __FreeBSD__
-	/* freebsd system() is broken */
-	int st, pid, fds[2];
-	if (pipe (fds)) {
-		return -1;
-	}
-	pid = vfork ();
-	if (pid == -1) {
-		return -1;
-	}
-	if (!pid) {
-		dup2 (1, fds[1]);
-		// char *argv[] = { "/bin/sh", "-c", str, NULL};
-		// execv (argv[0], argv);
-		r_sandbox_system (str, 0);
-		_exit (127); /* error */
-	} else {
-		dup2 (1, fds[0]);
-		waitpid (pid, &st, 0);
-	}
-	return WEXITSTATUS (st);
-#else
 	return r_sandbox_system (str, 1);
-#endif
 }
 
 R_API char *r_sys_cmd_str(const char *cmd, const char *input, int *len) {
 	char *output = NULL;
-	if (r_sys_cmd_str_full (cmd, input, &output, len, NULL)) {
+	if (r_sys_cmd_str_full (cmd, input, -1, &output, len, NULL)) {
 		return output;
 	}
 	free (output);
@@ -853,7 +865,7 @@ R_API bool r_sys_mkdirp(const char *dir) {
 	{
 		char *p = strstr (ptr, ":\\");
 		if (p) {
-			ptr = p + 2;
+			ptr = p + 3;
 		}
 	}
 #endif
@@ -904,7 +916,8 @@ R_API void r_sys_perror_str(const char *fun) {
 			0, NULL )) {
 		char *err = r_sys_conv_win_to_utf8 (lpMsgBuf);
 		if (err) {
-			eprintf ("%s: %s\n", fun, err);
+			eprintf ("%s: (%#lx) %s%s", fun, dw, err,
+			         r_str_endswith (err, "\n") ? "" : "\n");
 			free (err);
 		}
 		LocalFree (lpMsgBuf);
@@ -1057,36 +1070,16 @@ R_API int r_sys_run_rop(const ut8 *buf, int len) {
 	return 0;
 }
 
-R_API bool r_is_heap (void *p) {
-	void *q = malloc (8);
-	ut64 mask = UT64_MAX;
-	ut64 addr = (ut64)(size_t)q;
-	addr >>= 16;
-	addr <<= 16;
-	mask >>= 16;
-	mask <<= 16;
-	free (q);
-	return (((ut64)(size_t)p) == mask);
-}
-
-R_API char *r_sys_pid_to_path(int pid) {
 #if __WINDOWS__
-	// TODO: add maximum path length support
-	HANDLE processHandle;
+// w32 specific API
+R_API char *r_w32_handle_to_path(HANDLE processHandle) {
 	const DWORD maxlength = MAX_PATH;
-	TCHAR filename[MAX_PATH];
+	char *filename = calloc ((MAX_PATH * 2) + 2, 1);
 	char *result = NULL;
-
-	processHandle = OpenProcess (PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, pid);
-	if (!processHandle) {
-		eprintf ("r_sys_pid_to_path: Cannot open process.\n");
-		return NULL;
-	}
-	DWORD length = GetModuleFileNameEx (processHandle, NULL, filename, maxlength);
+	DWORD length = r_w32_GetModuleFileNameEx (processHandle, NULL, (LPSTR)filename, maxlength);
 	if (length == 0) {
 		// Upon failure fallback to GetProcessImageFileName
-		length = GetProcessImageFileName (processHandle, filename, maxlength);
-		CloseHandle (processHandle);
+		length = r_w32_GetProcessImageFileName (processHandle, filename, maxlength);
 		if (length == 0) {
 			eprintf ("r_sys_pid_to_path: Error calling GetProcessImageFileName\n");
 			return NULL;
@@ -1116,10 +1109,10 @@ R_API char *r_sys_pid_to_path(int pid) {
 			eprintf ("r_sys_pid_to_path: Error allocating memory\n");
 			return NULL;
 		}
-		strncpy (tmp, name, length);
-		tmp[length] = '\0';
+		r_str_ncpy (tmp, name, length);
 		TCHAR device[MAX_PATH];
-		for (TCHAR drv[] = TEXT("A:"); drv[0] <= TEXT('Z'); drv[0]++) {
+		TCHAR drv[3] = {'A',':', 0};
+		for (; drv[0] <= 'Z'; drv[0]++) {
 			if (QueryDosDevice (drv, device, maxlength) > 0) {
 				char *dvc = r_sys_conv_win_to_utf8 (device);
 				if (!dvc) {
@@ -1153,13 +1146,26 @@ R_API char *r_sys_pid_to_path(int pid) {
 		free (name);
 		free (tmp);
 	} else {
-		CloseHandle (processHandle);
 		result = r_sys_conv_win_to_utf8 (filename);
 	}
+	free (filename);
 	return result;
+}
+#endif
+
+R_API char *r_sys_pid_to_path(int pid) {
+#if __WINDOWS__
+	HANDLE processHandle = OpenProcess (PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, pid);
+	if (!processHandle) {
+		// eprintf ("r_sys_pid_to_path: Cannot open process.\n");
+		return NULL;
+	}
+	char *filename = r_w32_handle_to_path (processHandle);
+	CloseHandle (processHandle);
+	return filename;
 #elif __APPLE__
 #if __POWERPC__
-#warning TODO getpidproc
+#pragma message("TODO getpidproc")
 	return NULL;
 #else
 	char pathbuf[PROC_PIDPATHINFO_MAXSIZE];
@@ -1172,7 +1178,7 @@ R_API char *r_sys_pid_to_path(int pid) {
 #endif
 #else
 	int ret;
-#if __FreeBSD__
+#if __FreeBSD__ || __DragonFly__
 	char pathbuf[PATH_MAX];
 	size_t pathbufl = sizeof (pathbuf);
 	int mib[4] = {CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, pid};
@@ -1180,6 +1186,22 @@ R_API char *r_sys_pid_to_path(int pid) {
 	if (ret != 0) {
 		return NULL;
 	}
+#elif __HAIKU__
+	char pathbuf[MAXPATHLEN];
+	int32_t group = 0;
+	image_info ii;
+
+	while (get_next_image_info ((team_id)pid, &group, &ii) == B_OK) {
+		if (ii.type == B_APP_IMAGE) {
+			break;
+		}
+	}
+
+	if (ii.type == B_APP_IMAGE) {
+		r_str_ncpy (pathbuf, ii.name, MAXPATHLEN);
+	} else {
+		pathbuf[0] = '\0';
+	}
 #else
 	char buf[128], pathbuf[1024];
 	snprintf (buf, sizeof (buf), "/proc/%d/exe", pid);
@@ -1193,8 +1215,14 @@ R_API char *r_sys_pid_to_path(int pid) {
 #endif
 }
 
-// TODO: rename to r_sys_env_init()
-R_API char **r_sys_get_environ () {
+R_API void r_sys_env_init(void) {
+	char **envp = r_sys_get_environ ();
+	if (envp) {
+		r_sys_set_environ (envp);
+	}
+}
+
+R_API char **r_sys_get_environ(void) {
 #if __APPLE__ && !HAVE_ENVIRON
 	env = *_NSGetEnviron();
 #else
@@ -1207,28 +1235,56 @@ R_API char **r_sys_get_environ () {
 	return env;
 }
 
-R_API void r_sys_set_environ (char **e) {
+R_API void r_sys_set_environ(char **e) {
 	env = e;
 }
 
-R_API char *r_sys_whoami (char *buf) {
-	char _buf[32];
-	int pid = getpid ();
-	int hasbuf = (buf)? 1: 0;
-	if (!hasbuf) {
-		buf = _buf;
+R_API char *r_sys_whoami(void) {
+	char buf[32];
+#if __WINDOWS__
+	DWORD buf_sz = sizeof (buf);
+	if (!GetUserName (buf, (LPDWORD)&buf_sz) ) {
+		return strdup ("?");
 	}
-	sprintf (buf, "pid%d", pid);
-	return hasbuf? buf: strdup (buf);
+#elif __wasi__
+	strcpy (buf, "user");
+#else
+	struct passwd *pw = getpwuid (getuid ());
+	if (pw) {
+		return strdup (pw->pw_name);
+	}
+	int uid = getuid ();
+	snprintf (buf, sizeof (buf), "uid%d", uid);
+#endif
+	return strdup (buf);
 }
 
-R_API int r_sys_getpid() {
-#if __UNIX__
+R_API int r_sys_uid(void) {
+#if __WINDOWS__
+#pragma message ("r_sys_uid not implemented for windows")
+	char buf[32];
+	DWORD buf_sz = sizeof (buf);
+	// TODO
+	if (!GetUserName (buf, (LPDWORD)&buf_sz) ) {
+		return 1; // 
+	}
+	return 0;
+#elif __wasi__
+	return 0;
+#else
+	return getuid ();
+#endif
+}
+
+R_API int r_sys_getpid(void) {
+#if __wasi__
+	return 0;
+#elif __UNIX__
 	return getpid ();
 #elif __WINDOWS__
 	return GetCurrentProcessId();
 #else
-#warning r_sys_getpid not implemented for this platform
+#pragma message ("r_sys_getpid not implemented for this platform")
 	return -1;
 #endif
 }
@@ -1296,7 +1352,7 @@ R_API RSysInfo *r_sys_info(void) {
 	if (!si) {
 		return NULL;
 	}
-	
+
 	if (RegOpenKeyExA (HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", 0,
 		KEY_QUERY_VALUE, &key) != ERROR_SUCCESS) {
 		r_sys_perror ("r_sys_info/RegOpenKeyExA");
@@ -1331,7 +1387,7 @@ R_API RSysInfo *r_sys_info(void) {
 		|| type != REG_SZ) {
 		goto beach;
 	}
-	si->version = r_str_newf ("%d.%d.%s", major, minor, tmp);
+	si->version = r_str_newf ("%lu.%lu.%s", major, minor, tmp);
 
 	size = sizeof (tmp);
 	if (RegQueryValueExA (key, "ReleaseId", NULL, &type,
