@@ -1,15 +1,15 @@
-/* radare - LGPL - Copyright 2013-2018 - pancake */
+/* radare - LGPL - Copyright 2013-2021 - pancake */
 
 #include <r_core.h>
 #include <errno.h>
 
 #define FMT_NONE 0
 #define FMT_RAW  1
-#define FMT_JSON 2
+#define FMT_JSON 'j'
+#define FMT_QUIET 'q'
+#define FMT_EMOJI 'e'
 
-static int needs_newline = 0;
-
-static char *showfile(char *res, const int nth, const char *fpath, const char *name, int printfmt) {
+static char *showfile(char *res, const int nth, const char *fpath, const char *name, int printfmt, bool needs_newline) {
 #if __UNIX__
 	struct stat sb;
 #endif
@@ -33,8 +33,7 @@ static char *showfile(char *res, const int nth, const char *fpath, const char *n
 	}
 	perm = isdir? 0755: 0644;
 	if (!printfmt) {
-		needs_newline = ((nth + 1) % 4)? 1: 0;
-		res = r_str_appendf (res, "%18s%s", nn, needs_newline? "  ": "\n");
+		res = r_str_appendf (res, "%18s%s", nn, needs_newline? "\n": "  ");
 		free (nn);
 		return res;
 	}
@@ -62,8 +61,10 @@ static char *showfile(char *res, const int nth, const char *fpath, const char *n
 			case S_IFBLK: fch = 'b'; break;
 			case S_IFLNK: fch = 'l'; break;
 			case S_IFIFO: fch = 'p'; break;
+#ifndef __wasi__
 #ifdef S_IFSOCK
 			case S_IFSOCK: fch = 's'; break;
+#endif
 #endif
 			}
 		}
@@ -72,13 +73,11 @@ static char *showfile(char *res, const int nth, const char *fpath, const char *n
 	u_rwx = strdup ("-");
 	fch = isdir? 'd': '-';
 #endif
-	if (printfmt == 'q') {
+	if (printfmt == FMT_QUIET) {
 		res = r_str_appendf (res, "%s\n", nn);
-	} else if (printfmt == 'e') {
+	} else if (printfmt == FMT_EMOJI) {
 		const char *eDIR = "📁";
-		const char *eLNK = "📎";
 		const char *eIMG = "🌅";
-		const char *eUID = "🔼";
 		const char *eHID = "👀";
 		const char *eANY = "  ";
 		// --
@@ -87,8 +86,10 @@ static char *showfile(char *res, const int nth, const char *fpath, const char *n
 			icon = eDIR;
 #if __UNIX__
 		} else if ((sb.st_mode & S_IFMT) == S_IFLNK) {
+			const char *eLNK = "📎";
 			icon = eLNK;
 		} else if (sb.st_mode & S_ISUID) {
+			const char *eUID = "🔼";
 			icon = eUID;
 #endif
 		} else if (r_str_casestr (nn, ".jpg") || r_str_casestr (nn, ".png") || r_str_casestr (nn, ".gif")) {
@@ -100,7 +101,7 @@ static char *showfile(char *res, const int nth, const char *fpath, const char *n
 	} else if (printfmt == FMT_RAW) {
 		res = r_str_appendf (res, "%c%s%s%s  1 %4d:%-4d  %-10d  %s\n",
 			isdir?'d': fch,
-			u_rwx? u_rwx: "-",
+			r_str_get_fail (u_rwx, "-"),
 			r_str_rwx_i ((perm >> 3) & 7),
 			r_str_rwx_i (perm & 7),
 			uid, gid, sz, nn);
@@ -108,9 +109,23 @@ static char *showfile(char *res, const int nth, const char *fpath, const char *n
 		if (nth > 0) {
 			res = r_str_append (res, ",");
 		}
-		res = r_str_appendf (res, "{\"name\":\"%s\",\"size\":%d,\"uid\":%d,"
-			"\"gid\":%d,\"perm\":%d,\"isdir\":%s}",
-			name, sz, uid, gid, perm, isdir? "true": "false");
+		PJ *pj = pj_new ();
+		pj_o (pj);
+		pj_ks (pj, "name", name);
+		pj_kn (pj, "size", sz);
+		pj_kn (pj, "uid", uid);
+		pj_kn (pj, "gid", gid);
+		pj_kn (pj, "perm", perm);
+		pj_ks (pj, "perm_root", r_str_rwx_i ((perm >> 6)&7));
+		pj_ks (pj, "perm_group", r_str_rwx_i ((perm >> 3)&7));
+		pj_ks (pj, "perm_other", r_str_rwx_i (perm & 7));
+		pj_kb (pj, "isdir", isdir);
+		pj_end (pj);
+		char *js = pj_drain (pj);
+		res = r_str_append (res, js);
+		free (js);
+	} else {
+		eprintf ("unknown format\n");
 	}
 	free (nn);
 	free (u_rwx);
@@ -118,7 +133,7 @@ static char *showfile(char *res, const int nth, const char *fpath, const char *n
 }
 
 // TODO: Move into r_util .. r_print maybe? r_cons dep is annoying
-R_API char *r_syscmd_ls(const char *input) {
+R_API char *r_syscmd_ls(const char *input, int cons_width) {
 	char *res = NULL;
 	const char *path = ".";
 	char *d = NULL;
@@ -127,7 +142,6 @@ R_API char *r_syscmd_ls(const char *input) {
 	char *pattern = NULL;
 	int printfmt = 0;
 	RListIter *iter;
-	RList *files;
 	char *name;
 	char *dir;
 	int off;
@@ -135,8 +149,16 @@ R_API char *r_syscmd_ls(const char *input) {
 		input = "";
 		path = ".";
 	}
-	if (*input == 'q') {
-		printfmt = 'q';
+	if (*input == '?') {
+		input = "-h";
+	} else if (*input == 'e') {
+		printfmt = FMT_EMOJI;
+		input++;
+	} else if (*input == 'j') {
+		printfmt = FMT_JSON;
+		input++;
+	} else if (*input == 'q') {
+		printfmt = FMT_QUIET;
 		input++;
 	}
 	if (r_sandbox_enable (0)) {
@@ -147,13 +169,15 @@ R_API char *r_syscmd_ls(const char *input) {
 		input++;
 	}
 	if (*input) {
-		if ((!strncmp (input, "-h", 2))) {
+		if (!strncmp (input, "-h", 2) || *input == '?') {
 			eprintf ("Usage: ls ([-e,-l,-j,-q]) ([path]) # long, json, quiet\n");
-		} else if ((!strncmp (input, "-e", 2))) {
-			printfmt = 'e';
+			return NULL;
+		}
+		if ((!strncmp (input, "-e", 2))) {
+			printfmt = FMT_EMOJI;
 			path = r_str_trim_head_ro (path + 1);
 		} else if ((!strncmp (input, "-q", 2))) {
-			printfmt = 'q';
+			printfmt = FMT_QUIET;
 			path = r_str_trim_head_ro (path + 1);
 		} else if ((!strncmp (input, "-l", 2)) || (!strncmp (input, "-j", 2))) {
 			// mode = 'l';
@@ -205,13 +229,14 @@ R_API char *r_syscmd_ls(const char *input) {
 		pattern = strdup ("*");
 	}
 	if (r_file_is_regular (path)) {
-		res = showfile (res, 0, path, path, printfmt);
+		res = showfile (res, 0, path, path, printfmt, false);
 		free (homepath);
 		free (pattern);
 		free (d);
 		return res;
 	}
-	files = r_sys_dir (path);
+	RList *files = r_sys_dir (path);
+	r_list_sort (files, (RListComparator)strcmp);
 
 	if (path[strlen (path) - 1] == '/') {
 		dir = strdup (path);
@@ -222,7 +247,8 @@ R_API char *r_syscmd_ls(const char *input) {
 	if (printfmt == FMT_JSON) {
 		res = strdup ("[");
 	}
-	needs_newline = 0;
+	bool needs_newline = false;
+	int linelen = 0;
 	r_list_foreach (files, iter, name) {
 		char *n = r_str_append (strdup (dir), name);
 		if (!n) {
@@ -230,7 +256,16 @@ R_API char *r_syscmd_ls(const char *input) {
 		}
 		if (r_str_glob (name, pattern)) {
 			if (*n) {
-				res = showfile (res, nth, n, name, printfmt);
+				int namelen = strlen (name);
+				linelen += (namelen > 20)? namelen*2: 32;
+				if (linelen > cons_width) {
+					needs_newline = true;
+				}
+				res = showfile (res, nth, n, name, printfmt, needs_newline);
+				if (needs_newline) {
+					needs_newline = false;
+					linelen = 0;
+				}
 			}
 			nth++;
 		}
@@ -238,9 +273,13 @@ R_API char *r_syscmd_ls(const char *input) {
 	}
 	if (printfmt == FMT_JSON) {
 		res = r_str_append (res, "]");
-	}
-	if (needs_newline) {
-		res = r_str_append (res, "\n");
+	} else {
+		if (res) {
+			char * last = res + strlen (res) - 1;
+			if (*last != '\n') {
+				res = r_str_append (res, "\n");
+			}
+		}
 	}
 	free (dir);
 	free (d);
@@ -256,7 +295,6 @@ static int cmpstr (const void *_a, const void *_b) {
 }
 
 R_API char *r_syscmd_sort(const char *file) {
-	int sz;
 	const char *p = NULL;
 	RList *list = NULL;
 	if (file) {
@@ -269,7 +307,7 @@ R_API char *r_syscmd_sort(const char *file) {
 	if (p && *p) {
 		char *filename = strdup (p);
 		r_str_trim (filename);
-		char *data = r_file_slurp (filename, &sz);
+		char *data = r_file_slurp (filename, NULL);
 		if (!data) {
 			eprintf ("No such file or directory\n");
 		} else {
@@ -335,7 +373,6 @@ R_API char *r_syscmd_tail(const char *file, int count) {
 }
 
 R_API char *r_syscmd_uniq(const char *file) {
-	int sz;
 	const char *p = NULL;
 	RList *list = NULL;
 	if (file) {
@@ -348,7 +385,7 @@ R_API char *r_syscmd_uniq(const char *file) {
 	if (p && *p) {
 		char *filename = strdup (p);
 		r_str_trim (filename);
-		char *data = r_file_slurp (filename, &sz);
+		char *data = r_file_slurp (filename, NULL);
 		if (!data) {
 			eprintf ("No such file or directory\n");
 		} else {
@@ -367,7 +404,6 @@ R_API char *r_syscmd_uniq(const char *file) {
 }
 
 R_API char *r_syscmd_join(const char *file1, const char *file2) {
-	int sz1, sz2;
 	const char *p1 = NULL, *p2 = NULL;
 	RList *list1, *list2, *list = r_list_newf (NULL);
 	if (!list) {
@@ -392,8 +428,8 @@ R_API char *r_syscmd_join(const char *file1, const char *file2) {
 		char *filename2 = strdup (p2);
 		r_str_trim (filename1);
 		r_str_trim (filename2);
-		char *data1 = r_file_slurp (filename1, &sz1);
-		char *data2 = r_file_slurp (filename2, &sz2);
+		char *data1 = r_file_slurp (filename1, NULL);
+		char *data2 = r_file_slurp (filename2, NULL);
 		char *data = NULL;
 		RListIter *iter1, *iter2;
 		if (!data1 && !data2) {
@@ -417,8 +453,8 @@ R_API char *r_syscmd_join(const char *file1, const char *file2) {
 						char *out = r_str_new (field);
 						char *first = strchr (str1, ' ');
 						char *second = strchr (str2, ' ');
-						r_str_append (out, first ? first : " ");
-						r_str_append (out, second ? second : " ");
+						r_str_append (out, r_str_get_fail (first, " "));
+						r_str_append (out, r_str_get_fail (second, " "));
 						r_list_append (list, out);
 					}
 				}
@@ -439,7 +475,6 @@ R_API char *r_syscmd_join(const char *file1, const char *file2) {
 }
 
 R_API char *r_syscmd_cat(const char *file) {
-	int sz;
 	const char *p = NULL;
 	if (file) {
 		if ((p = strchr (file, ' '))) {
@@ -451,7 +486,7 @@ R_API char *r_syscmd_cat(const char *file) {
 	if (p && *p) {
 		char *filename = strdup (p);
 		r_str_trim (filename);
-		char *data = r_file_slurp (filename, &sz);
+		char *data = r_file_slurp (filename, NULL);
 		if (!data) {
 			eprintf ("No such file or directory\n");
 		}
@@ -489,13 +524,17 @@ R_API bool r_syscmd_mv(const char *input) {
 		eprintf ("Usage: mv src dst\n");
 		return false;
 	}
-	input = input + 2;
-	if (!r_sandbox_enable (0)) {
-#if __WINDOWS__
-		r_sys_cmdf ("move %s", input);
-#else
-		r_sys_cmdf ("mv %s", input);
-#endif
+	char *inp = strdup (input + 2);
+	char *arg = strchr (inp, ' ');
+	bool rc = false;
+	if (arg) {
+		*arg++ = 0;
+		if (!(rc = r_file_move (inp, arg))) {
+			eprintf ("Cannot move file\n");
+		}
+	} else {
+		eprintf ("Usage: mv src dst\n");
 	}
-	return false;
+	free (inp);
+	return rc;
 }
