@@ -1,6 +1,7 @@
 /* radare - LGPL - Copyright 2009-2018 - pancake */
 
 #include <r_th.h>
+#include <r_util.h>
 
 #if __APPLE__
 // Here to avoid polluting mach types macro redefinitions...
@@ -8,6 +9,14 @@
 #include <mach/thread_policy.h>
 #endif
 
+#if __sun
+#include <sys/pset.h>
+#endif
+
+#if __HAIKU__
+#include <kernel/scheduler.h>
+#endif
+
 #if __WINDOWS__
 static DWORD WINAPI _r_th_launcher(void *_th) {
 #else
@@ -16,17 +25,11 @@ static void *_r_th_launcher(void *_th) {
 	int ret;
 	RThread *th = _th;
 	th->ready = true;
-#if __UNIX__
 	if (th->delay > 0) {
-		sleep (th->delay);
+		r_sys_sleep (th->delay);
 	} else if (th->delay < 0) {
 		r_th_lock_wait (th->lock);
 	}
-#else
-	if (th->delay < 0) {
-		r_th_lock_wait (th->lock);
-	}
-#endif
 	r_th_lock_enter (th->lock);
 	do {
 		r_th_lock_leave (th->lock);
@@ -39,6 +42,7 @@ static void *_r_th_launcher(void *_th) {
 		th->running = false;
 		r_th_lock_enter (th->lock);
 	} while (ret);
+	r_th_lock_leave (th->lock);
 #if HAVE_PTHREAD
 	pthread_exit (&ret);
 #endif
@@ -65,23 +69,28 @@ R_API R_TH_TID r_th_self(void) {
 
 R_API bool r_th_setname(RThread *th, const char *name) {
 #if defined(HAVE_PTHREAD_NP) && HAVE_PTHREAD_NP
-#if __linux__
+#if __linux__ || __sun
 	if (pthread_setname_np (th->tid, name) != 0) {
 		eprintf ("Failed to set thread name\n");
 		return false;
-	}	
+	}
 #elif __APPLE__
 	if (pthread_setname_np (name) != 0) {
 		eprintf ("Failed to set thread name\n");
 		return false;
 	}
-#elif __FreeBSD__ || __OpenBSD__ || __DragonFly__
+#elif __FreeBSD__ || __OpenBSD__ || __DragonFly__ || __sun
 	pthread_set_name_np (th->tid, name);
 #elif __NetBSD__
 	if (pthread_setname_np (th->tid, "%s", (void *)name) != 0) {
 		eprintf ("Failed to set thread name\n");
 		return false;
-	}	
+	}
+#elif __HAIKU__
+	if (rename_thread ((thread_id)th->tid, name) != B_OK) {
+		eprintf ("Failed to set thread name\n");
+		return false;
+	}
 #else
 #pragma message("warning r_th_setname not implemented")
 #endif
@@ -91,13 +100,23 @@ R_API bool r_th_setname(RThread *th, const char *name) {
 
 R_API bool r_th_getname(RThread *th, char *name, size_t len) {
 #if defined(HAVE_PTHREAD_NP) && HAVE_PTHREAD_NP
-#if __linux__ || __NetBSD__ || __APPLE__
+#if __linux__ || __NetBSD__ || __APPLE__ || __sun
 	if (pthread_getname_np (th->tid, name, len) != 0) {
 		eprintf ("Failed to get thread name\n");
 		return false;
 	}
 #elif (__FreeBSD__ &&  __FreeBSD_version >= 1200000) || __DragonFly__  || (__OpenBSD__ && OpenBSD >= 201905)
 	pthread_get_name_np (th->tid, name, len);
+#elif defined(__HAIKU__)
+	thread_info ti;
+	size_t flen = len < B_OS_NAME_LENGTH ? len : B_OS_NAME_LENGTH;
+
+	if (get_thread_info ((thread_id)th->tid, &ti) != B_OK) {
+		eprintf ("Failed to get thread name\n");
+		return false;
+	}
+
+	r_str_ncpy (name, ti.name, flen);
 #else
 #pragma message("warning r_th_getname not implemented")
 #endif
@@ -106,7 +125,13 @@ R_API bool r_th_getname(RThread *th, char *name, size_t len) {
 }
 
 R_API bool r_th_setaffinity(RThread *th, int cpuid) {
-#if __linux__
+#if defined(__wasi__) || defined(_WASI_EMULATED_SIGNAL)
+	return true;
+#elif __linux__
+#if defined(__GLIBC__) && defined (__GLIBC_MINOR__) && (__GLIBC__ <= 2) && (__GLIBC_MINOR__ <= 2)
+	// Old versions of GNU libc don't have this feature
+#pragma message("warning r_th_setaffinity not implemented")
+#else
 	cpu_set_t c;
 	CPU_ZERO(&c);
 	CPU_SET(cpuid, &c);
@@ -115,6 +140,7 @@ R_API bool r_th_setaffinity(RThread *th, int cpuid) {
 		eprintf ("Failed to set cpu affinity\n");
 		return false;
 	}
+#endif
 #elif __FreeBSD__ || __DragonFly__
 	cpuset_t c;
 	CPU_ZERO(&c);
@@ -147,6 +173,19 @@ R_API bool r_th_setaffinity(RThread *th, int cpuid) {
 		eprintf ("Failed to set cpu affinity\n");
 		return false;
 	}
+#elif __sun
+	psetid_t c;
+
+	pset_create (&c);
+	pset_assign (c, cpuid, NULL);
+
+	if (pset_bind (c, P_PID, r_sys_getpid (), NULL)) {
+		pset_destroy (c);
+		eprintf ("Failed to set cpu affinity\n");
+		return false;
+	}
+
+	pset_destroy (c);
 #else
 #pragma message("warning r_th_setaffinity not implemented")
 #endif
@@ -218,9 +257,9 @@ R_API bool r_th_start(RThread *th, int enable) {
 
 R_API int r_th_wait(struct r_th_t *th) {
 	int ret = false;
-	void *thret;
 	if (th) {
 #if HAVE_PTHREAD
+		void *thret;
 		ret = pthread_join (th->tid, &thret);
 #elif __WINDOWS__
 		ret = WaitForSingleObject (th->tid, INFINITE);
