@@ -1,8 +1,10 @@
-/* radare - LGPL - Copyright 2009-2018 - thestr4ng3r */
+/* radare - LGPL - Copyright 2009-2021 - thestr4ng3r */
 
 #include <r_th.h>
 
-R_API RThreadCond *r_th_cond_new() {
+// XXX the windows implementation requires windows 2008 or higher
+
+R_API RThreadCond *r_th_cond_new(void) {
 	RThreadCond *cond = R_NEW0 (RThreadCond);
 	if (!cond) {
 		return NULL;
@@ -13,7 +15,7 @@ R_API RThreadCond *r_th_cond_new() {
 		return NULL;
 	}
 #elif __WINDOWS__
-	InitializeConditionVariable (&cond->cond);
+	r_w32_InitializeConditionVariable (&cond->cond);
 #endif
 	return cond;
 }
@@ -22,7 +24,7 @@ R_API void r_th_cond_signal(RThreadCond *cond) {
 #if HAVE_PTHREAD
 	pthread_cond_signal (&cond->cond);
 #elif __WINDOWS__
-	WakeConditionVariable (&cond->cond);
+	r_w32_WakeConditionVariable (&cond->cond);
 #endif
 }
 
@@ -30,7 +32,7 @@ R_API void r_th_cond_signal_all(RThreadCond *cond) {
 #if HAVE_PTHREAD
 	pthread_cond_broadcast (&cond->cond);
 #elif __WINDOWS__
-	WakeAllConditionVariable (&cond->cond);
+	r_w32_WakeAllConditionVariable (&cond->cond);
 #endif
 }
 
@@ -38,7 +40,7 @@ R_API void r_th_cond_wait(RThreadCond *cond, RThreadLock *lock) {
 #if HAVE_PTHREAD
 	pthread_cond_wait (&cond->cond, &lock->lock);
 #elif __WINDOWS__
-	SleepConditionVariableCS (&cond->cond, &lock->lock, INFINITE);
+	r_w32_SleepConditionVariableCS (&cond->cond, &lock->lock, INFINITE);
 #endif
 }
 
@@ -48,6 +50,8 @@ R_API void r_th_cond_free(RThreadCond *cond) {
 	}
 #if HAVE_PTHREAD
 	pthread_cond_destroy (&cond->cond);
+#elif __WINDOWS__
+	// TODO destroy the condition variable here
 #endif
 	free (cond);
 }
