@@ -23,7 +23,6 @@
 #include <grub/mm.h>
 #include <grub/misc.h>
 #include <grub/disk.h>
-#include <grub/dl.h>
 #include <grub/types.h>
 #include <grub/fshelp.h>
 #include <grub/charset.h>
@@ -160,9 +159,6 @@ struct grub_fshelp_node
   unsigned int dir_off;
 };
 
-static grub_dl_t my_mod;
-
-
 static char *
 load_sua (struct grub_iso9660_data *data, int sua_block, int sua_pos,
 	  int sua_size)
@@ -519,9 +515,9 @@ susp_iterate_dir (struct grub_iso9660_susp_entry *entry, void *closure)
 	 filename type is stored.  */
       grub_uint8_t *data = ENTRY_DATA(entry);
       if (data[0] & GRUB_ISO9660_RR_DOT)
-	filename = ".";
+	filename = strdup (".");
       else if (data[0] & GRUB_ISO9660_RR_DOTDOT)
-	filename = "..";
+	filename = strdup ("..");
       else
 	{
 	  int size = 1;
@@ -745,8 +741,6 @@ grub_iso9660_dir (grub_device_t device, const char *path,
   struct grub_fshelp_node *foundnode;
   struct grub_iso9660_dir_closure c;
 
-  grub_dl_ref (my_mod);
-
   data = grub_iso9660_mount (device->disk);
   if (! data)
     goto fail;
@@ -774,8 +768,6 @@ grub_iso9660_dir (grub_device_t device, const char *path,
  fail:
   grub_free (data);
 
-  grub_dl_unref (my_mod);
-
   return grub_errno;
 }
 
@@ -788,8 +780,6 @@ grub_iso9660_open (struct grub_file *file, const char *name)
   struct grub_fshelp_node rootnode;
   struct grub_fshelp_node *foundnode;
 
-  grub_dl_ref (my_mod);
-
   data = grub_iso9660_mount (file->device->disk);
   if (!data)
     goto fail;
@@ -815,8 +805,6 @@ grub_iso9660_open (struct grub_file *file, const char *name)
   return 0;
 
  fail:
-  grub_dl_unref (my_mod);
-
   grub_free (data);
 
   return grub_errno;
@@ -850,8 +838,6 @@ grub_iso9660_close (grub_file_t file)
 {
   grub_free (file->data);
 
-  grub_dl_unref (my_mod);
-
   return GRUB_ERR_NONE;
 }
 
@@ -884,8 +870,6 @@ grub_iso9660_uuid (grub_device_t device, char **uuid)
   struct grub_iso9660_data *data;
   grub_disk_t disk = device->disk;
 
-  grub_dl_ref (my_mod);
-
   data = grub_iso9660_mount (disk);
   if (data)
     {
@@ -925,8 +909,6 @@ grub_iso9660_uuid (grub_device_t device, char **uuid)
   else
     *uuid = NULL;
 
-	grub_dl_unref (my_mod);
-
   grub_free (data);
 
   return grub_errno;
