@@ -35,10 +35,10 @@
 #include <grub/mm.h>
 #include <grub/misc.h>
 #include <grub/disk.h>
-#include <grub/dl.h>
 #include <grub/types.h>
 #include <grub/fshelp.h>
 #include <r_types.h>
+#include <r_util/r_assert.h>
 
 #ifndef MIN
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
@@ -57,16 +57,6 @@
 #undef S_IFLNK
 #define S_IFLNK 0xA000
 
-static grub_dl_t my_mod;
-
-#define assert(boolean) real_assert (boolean, GRUB_FILE, __LINE__)
-static inline void
-real_assert (int boolean, const char *file, const int line)
-{
-  if (! boolean)
-    grub_printf ("Assertion failed at %s:%d\n", file, line);
-}
-
 enum grub_reiserfs_item_type
   {
     GRUB_REISERFS_STAT,
@@ -416,7 +406,7 @@ grub_reiserfs_set_key_type (struct grub_reiserfs_key *key,
       = ((key->u.v2.offset_type & grub_cpu_to_le64 (~0ULL >> 4))
          | grub_cpu_to_le64 ((grub_uint64_t) type << 60));
 
-  assert (grub_reiserfs_get_key_type (key) == grub_type);
+  r_warn_if_fail (grub_reiserfs_get_key_type (key) == grub_type);
 }
 
 /* -1 if key 1 if lower than key 2.
@@ -632,14 +622,14 @@ grub_reiserfs_get_item (struct grub_reiserfs_data *data,
 #endif
     }
 
-  assert (grub_errno == GRUB_ERR_NONE);
+  r_warn_if_fail (grub_errno == GRUB_ERR_NONE);
   grub_free (block_header);
   return GRUB_ERR_NONE;
 
  fail:
-  assert (grub_errno != GRUB_ERR_NONE);
+  r_warn_if_fail (grub_errno != GRUB_ERR_NONE);
   grub_free (block_header);
-  assert (grub_errno != GRUB_ERR_NONE);
+  r_warn_if_fail (grub_errno != GRUB_ERR_NONE);
   return grub_errno;
 }
 
@@ -976,11 +966,11 @@ grub_reiserfs_iterate_dir (grub_fshelp_node_t item,
   while (block_number);
 
  found:
-  assert (grub_errno == GRUB_ERR_NONE);
+  r_warn_if_fail (grub_errno == GRUB_ERR_NONE);
   grub_free (block_header);
   return ret;
  fail:
-  assert (grub_errno != GRUB_ERR_NONE);
+  r_warn_if_fail (grub_errno != GRUB_ERR_NONE);
   grub_free (block_header);
   return 0;
 }
@@ -999,7 +989,6 @@ grub_reiserfs_open (struct grub_file *file, const char *name)
   grub_uint32_t block_number;
   grub_uint16_t entry_version, block_size, entry_location;
 
-  grub_dl_ref (my_mod);
   data = grub_reiserfs_mount (file->device->disk);
   if (! data)
     goto fail;
@@ -1069,10 +1058,9 @@ grub_reiserfs_open (struct grub_file *file, const char *name)
   return GRUB_ERR_NONE;
 
  fail:
-  assert (grub_errno != GRUB_ERR_NONE);
+  r_warn_if_fail (grub_errno != GRUB_ERR_NONE);
   grub_free (found);
   grub_free (data);
-  grub_dl_unref (my_mod);
   return grub_errno;
 }
 
@@ -1267,7 +1255,6 @@ grub_reiserfs_close (grub_file_t file)
 
   grub_free (data);
   grub_free (node);
-  grub_dl_unref (my_mod);
   return GRUB_ERR_NONE;
 }
 
@@ -1306,7 +1293,6 @@ grub_reiserfs_dir (grub_device_t device, const char *path,
   struct grub_reiserfs_key root_key;
   struct grub_reiserfs_dir_closure c;
 
-  grub_dl_ref (my_mod);
   data = grub_reiserfs_mount (device->disk);
   if (! data)
     goto fail;
@@ -1330,12 +1316,10 @@ grub_reiserfs_dir (grub_device_t device, const char *path,
   c.closure = closure;
   grub_reiserfs_iterate_dir (found, iterate, &c);
   grub_free (data);
-  grub_dl_unref (my_mod);
   return GRUB_ERR_NONE;
 
  fail:
   grub_free (data);
-  grub_dl_unref (my_mod);
   return grub_errno;
 }
 
@@ -1362,8 +1346,6 @@ grub_reiserfs_uuid (grub_device_t device, char **uuid)
   struct grub_reiserfs_data *data;
   grub_disk_t disk = device->disk;
 
-  grub_dl_ref (my_mod);
-
   data = grub_reiserfs_mount (disk);
   if (data)
     {
@@ -1380,8 +1362,6 @@ grub_reiserfs_uuid (grub_device_t device, char **uuid)
   else
     *uuid = NULL;
 
-  grub_dl_unref (my_mod);
-
   grub_free (data);
 
   return grub_errno;
