@@ -96,13 +96,14 @@ static registers_t arm32[] = {
 	{"cpsr", 172, 4},
 	{"", 0, 0}};
 
-int qnxr_init (libqnxr_t *g) {
-	if (!g) return -1;
+int qnxr_init(libqnxr_t *g) {
+	r_return_val_if_fail (g, -1);
 	memset (g, 0, sizeof (libqnxr_t));
 	g->send_len = 0;
 	g->send_buff = (char *)calloc (DS_DATA_MAX_SIZE * 2, 1);
-	if (!g->send_buff)
+	if (!g->send_buff) {
 		return -1;
+	}
 	g->read_buff = (char *)calloc (DS_DATA_MAX_SIZE * 2, 1);
 	if (!g->read_buff) {
 		R_FREE (g->send_buff);
@@ -112,8 +113,10 @@ int qnxr_init (libqnxr_t *g) {
 	return 0;
 }
 
-int qnxr_set_architecture (libqnxr_t *g, ut8 architecture) {
-	if (!g) return -1;
+int qnxr_set_architecture(libqnxr_t *g, ut8 architecture) {
+	if (!g) {
+		return -1;
+	}
 	g->architecture = architecture;
 	switch (architecture) {
 	case ARCH_X86_32:
@@ -128,15 +131,15 @@ int qnxr_set_architecture (libqnxr_t *g, ut8 architecture) {
 	return 0;
 }
 
-int qnxr_cleanup (libqnxr_t *g) {
-	if (!g) return -1;
+int qnxr_cleanup(libqnxr_t *g) {
+	r_return_val_if_fail (g, -1);
 	free (g->send_buff);
 	g->send_len = 0;
 	free (g->read_buff);
 	return 0;
 }
 
-int qnxr_connect (libqnxr_t *g, const char *host, int port) {
+int qnxr_connect(libqnxr_t *g, const char *host, int port) {
 	char tmp[255];
 	int ret;
 	if (!g || !host || g->connected) return -1;
@@ -193,8 +196,8 @@ int qnxr_connect (libqnxr_t *g, const char *host, int port) {
 	return 0;
 }
 
-int qnxr_disconnect (libqnxr_t *g) {
-	if (!g) return -1;
+int qnxr_disconnect(libqnxr_t *g) {
+	r_return_val_if_fail (g, -1);
 
 	if (g->connected) {
 		nto_send_init (g, DStMsg_disconnect, 0, SET_CHANNEL_DEBUG);
@@ -210,6 +213,7 @@ int qnxr_disconnect (libqnxr_t *g) {
 }
 
 ptid_t qnxr_attach (libqnxr_t *g, pid_t pid) {
+	r_return_val_if_fail (g, null_ptid);
 
 	if (g->inferior_ptid.pid != pid) {
 		qnxr_disconnect (g);
@@ -234,14 +238,14 @@ ptid_t qnxr_attach (libqnxr_t *g, pid_t pid) {
 	return g->inferior_ptid;
 }
 
-ptid_t qnxr_run (libqnxr_t *g, const char *file, char **args, char **env) {
+ptid_t qnxr_run(libqnxr_t *g, const char *file, char **args, char **env) {
 	ut32 argc = 0;
 	ut32 envc = 0;
 
 	char **argv, *p;
 	int errors = 0;
 
-	if (!g) return null_ptid;
+	r_return_val_if_fail (g, null_ptid);
 
 	nto_send_init (g, DStMsg_env, DSMSG_ENV_CLEARENV, SET_CHANNEL_DEBUG);
 	nto_send (g, sizeof (DStMsg_env_t), 1);
@@ -308,7 +312,7 @@ ptid_t qnxr_run (libqnxr_t *g, const char *file, char **args, char **env) {
 	return null_ptid;
 }
 
-int qnxr_read_registers (libqnxr_t *g) {
+int qnxr_read_registers(libqnxr_t *g) {
 	int i = 0;
 	int len, rlen, regset;
 	int n = 0;
@@ -427,8 +431,8 @@ void qnxr_pidlist (libqnxr_t *g, void *ctx, pidlist_cb_t *cb) {
 	}
 }
 
-int qnxr_select (libqnxr_t *g, pid_t pid, int tid) {
-	if (!g) return 0;
+bool qnxr_select (libqnxr_t *g, pid_t pid, int tid) {
+	if (!g) return false;
 
 	/* TODO */
 	tid = 1;
@@ -441,17 +445,17 @@ int qnxr_select (libqnxr_t *g, pid_t pid, int tid) {
 
 	if (g->recv.pkt.hdr.cmd == DSrMsg_err) {
 		eprintf ("%s: failed to select %d\n", __func__, pid);
-		return 0;
+		return false;
 	}
 
-	return 1;
+	return true;
 }
 
-int qnxr_step (libqnxr_t *g, int thread_id) {
+int qnxr_step(libqnxr_t *g, int thread_id) {
 	return qnxr_send_vcont (g, 1, thread_id);
 }
 
-int qnxr_continue (libqnxr_t *g, int thread_id) {
+int qnxr_continue(libqnxr_t *g, int thread_id) {
 	return qnxr_send_vcont (g, 0, thread_id);
 }
 
@@ -496,7 +500,7 @@ int qnxr_write_reg (libqnxr_t *g, const char *name, char *value, int len) {
 	return 0;
 }
 
-int qnxr_send_vcont (libqnxr_t *g, int step, int thread_id) {
+int qnxr_send_vcont(libqnxr_t *g, int step, int thread_id) {
 	if (!g) return -1;
 
 	nto_send_init (g, DStMsg_run, step ? DSMSG_RUN_COUNT : DSMSG_RUN,
@@ -506,7 +510,7 @@ int qnxr_send_vcont (libqnxr_t *g, int step, int thread_id) {
 	return 0;
 }
 
-int qnxr_stop (libqnxr_t *g) {
+int qnxr_stop(libqnxr_t *g) {
 	if (!g) return 0;
 
 	eprintf ("%s: waiting for stop\n", __func__);
@@ -519,7 +523,7 @@ int qnxr_stop (libqnxr_t *g) {
 	return 1;
 }
 
-ptid_t qnxr_wait (libqnxr_t *g, pid_t pid) {
+ptid_t qnxr_wait(libqnxr_t *g, pid_t pid) {
 	if (!g || pid < 0) {
 		return null_ptid;
 	}
@@ -576,11 +580,11 @@ ptid_t qnxr_wait (libqnxr_t *g, pid_t pid) {
 	return returned_ptid;
 }
 
-int qnxr_set_bp (libqnxr_t *g, ut64 address, const char *conditions) {
+int qnxr_set_bp(libqnxr_t *g, ut64 address, const char *conditions) {
 	return _qnxr_set_bp (g, address, conditions, BREAKPOINT);
 }
 
-int qnxr_set_hwbp (libqnxr_t *g, ut64 address, const char *conditions) {
+int qnxr_set_hwbp(libqnxr_t *g, ut64 address, const char *conditions) {
 	return _qnxr_set_bp (g, address, conditions, HARDWARE_BREAKPOINT);
 }
 
