@@ -1,11 +1,13 @@
-#ifndef SDB_HT_P_H_
-#define SDB_HT_P_H_
+#ifndef SDB_HT_PP_H
+#define SDB_HT_PP_H
 
 /*
  * This header provides an hashtable HtPP that has void* as key and void* as
  * value. The API functions starts with "ht_pp_" and the types starts with "HtPP".
  */
+#undef HT_TYPE
 #define HT_TYPE 1
+//#include "sdbht.h"
 #include "ht_inc.h"
 
 SDB_API HtName_(Ht)* Ht_(new0)(void);
