@@ -5,45 +5,72 @@
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdbool.h>
+#include <inttypes.h>
+#include <stdio.h>
 
 #undef eprintf
-#define eprintf(x,y...) fprintf(stderr,x,##y)
+#define eprintf(...) fprintf(stderr,__VA_ARGS__)
 
+// Copied from https://gcc.gnu.org/wiki/Visibility
 #ifndef SDB_API
-#if defined(__GNUC__) && __GNUC__ >= 4
-#define SDB_API __attribute__((visibility("default")))
-#else
-#define SDB_API
-#endif
+	#undef SDB_IPI
+	#if defined _WIN32 || defined __CYGWIN__
+		#ifdef __GNUC__
+			#define SDB_API __attribute__ ((dllexport))
+		#else
+			#define SDB_API __declspec(dllexport) // Note: actually gcc seems to also supports this syntax.
+		#endif
+		#define SDB_IPI
+	#else
+	#if __GNUC__ >= 4
+		#define SDB_API __attribute__ ((visibility ("default")))
+		#define SDB_IPI  __attribute__ ((visibility ("hidden")))
+	#else
+		#define SDB_API
+		#define SDB_IPI
+	#endif
+	#endif
 #endif
 
 #if MINGW || __MINGW32__ || __MINGW64__
 #define __MINGW__ 1
 #endif
 
-#if __WIN32__ || __MINGW__ || __WINDOWS__
+#if __WIN32__ || __MINGW__ || __WINDOWS__ || _MSC_VER
 #define __SDB_WINDOWS__ 1
-#include <windows.h>
+#undef DIRSEP
 #define DIRSEP '\\'
+#undef lseek
+#define lseek _lseek
+#include <windows.h>
+#include <io.h>
+#if __MINGW32__
+#define ULLFMT "ll"
+#else
+#define ULLFMT "I64"
+#endif
+#undef HAVE_MMAN
+#define HAVE_MMAN 0
 #else
 // CYGWIN AND UNIX
 #define __SDB_WINDOWS__ 0
+#undef DIRSEP
 #define DIRSEP '/'
-#endif
-
-#include <inttypes.h>
-#if __CYGWIN__
-#define USE_MMAN 1
-#define ULLFMT "ll"
-#elif __SDB_WINDOWS__
-#define USE_MMAN 0
-#define ULLFMT "I64"
-#else
-#define ULLFMT "ll"
-#define USE_MMAN 1
-#endif
-
 #include <unistd.h>
+#undef HAVE_MMAN
+#define HAVE_MMAN 1
+#define ULLFMT "ll"
+#endif
+
+#if __wasi__
+#undef HAVE_MMAN
+#define HAVE_MMAN 0
+#endif
+
+#ifndef USE_MMAN
+#define USE_MMAN HAVE_MMAN
+#endif
 
 #ifndef UNUSED
 #  define UNUSED
@@ -62,8 +89,16 @@
 #define st64 long long
 #define boolt int
 // TODO: deprecate R_NEW
+#ifndef R_NEW
+//it means we are within sdb
 #define R_NEW(x) (x*)malloc(sizeof(x))
-#define R_NEW0(x) (x*)calloc(1,sizeof(x))
+#endif
+#ifndef R_NEW0
+#define R_NEW0(x) (x*)calloc(1, sizeof(x))
+#endif
+#ifndef R_FREE
+#define R_FREE(x) { free (x); x = NULL; }
+#endif
 #define UT32_MAX ((ut32)0xffffffff)
 #define UT64_MAX ((ut64)(0xffffffffffffffffLL))
 #endif
@@ -80,7 +115,7 @@
 #include "config.h"
 
 static inline int seek_set(int fd, off_t pos) {
-	return ((fd==-1) || (lseek (fd, (off_t) pos, SEEK_SET) == -1))? 0:1;
+	return ((fd == -1) || (lseek (fd, (off_t) pos, SEEK_SET) == -1))? 0:1;
 }
 
 static inline void ut32_pack(char s[4], ut32 u) {
