@@ -8,6 +8,10 @@
 #include <stdbool.h>
 #include <errno.h>
 
+#ifndef VERSION
+#define VERSION "1.2.0"
+#endif
+
 #ifdef S_API
 #undef S_API
 #endif
@@ -56,18 +60,8 @@
 #define snprintf _snprintf
 #endif
 
-#define VERSION "1.0"
-
 #define MAXIFL 128
 
-#ifndef HAVE_FORK
-#define HAVE_FORK 1
-#endif
-
-#ifndef HAVE_FORK
-#define HAVE_FORK 1
-#endif
-
 #ifndef DLL_LOCAL
 #ifdef _MSC_VER
 #define DLL_LOCAL
@@ -119,19 +113,19 @@ typedef struct SppBuf {
 #define PUT_CALLBACK(x) int x (Output *out, char *buf)
 #define IS_SPACE(x) ((x==' ')||(x=='\t')||(x=='\r')||(x=='\n'))
 
-struct Tag {
+typedef struct Tag {
 	const char *name;
 	TAG_CALLBACK((*callback));
-};
+} SppTag;
 
-struct Arg {
+typedef struct Arg {
 	const char *flag;
 	const char *desc;
 	int has_arg;
 	ARG_CALLBACK((*callback));
-};
+} SppArg;
 
-struct Proc {
+typedef struct Proc {
 	const char *name;
 	struct Tag **tags;
 	struct Arg **args;
@@ -146,15 +140,25 @@ struct Proc {
 	int default_echo;
 	SppState state;
 	SppBuf buf;
-};
+} SppProc;
+
+typedef struct spp_t {
+	SppProc *proc;
+	Output *out;
+} Spp;
 
 S_API int spp_file(const char *file, Output *out);
 S_API int spp_run(char *buf, Output *out);
 S_API void spp_eval(char *buf, Output *out);
+S_API void spp_proc_eval(SppProc *p, char *buf, Output *out);
 S_API void spp_io(FILE *in, Output *out);
 S_API void spp_proc_list(void);
 S_API void spp_proc_list_kw(void);
-S_API void spp_proc_set(struct Proc *p, char *arg, int fail);
+S_API void spp_proc_set(SppProc *p, const char *arg, int fail);
+
+S_API Spp *spp_new(SppProc *proc);
+S_API char *spp_parse(Spp *s, const char *input);
+S_API void spp_free(Spp *s);
 
 #if DEBUG
 #define D if (1)
@@ -162,4 +166,8 @@ S_API void spp_proc_set(struct Proc *p, char *arg, int fail);
 #define D if (0)
 #endif
 
+#ifndef HAVE_FORK
+#define HAVE_FORK 1
+#endif
+
 #endif
