@@ -3,6 +3,9 @@
 BUILD=1
 FLAGS=""
 PREFIX="/data/data/org.radare.radare2installer/radare2"
+MAKE=make
+gmake --help >/dev/null 2>&1
+[ $? = 0 ] && MAKE=gmake
 
 type pax
 [ $? != 0 ] && exit 1
@@ -31,7 +34,7 @@ case "$1" in
 #	FLAGS="-mlong-calls"
 #	export LDFLAGS="-fuse-ld=gold"
 	;;
-"arm")
+arm)
 	NDK_ARCH=arm
 	STATIC_BUILD=0
 	STRIP=arm-eabi-strip
@@ -41,13 +44,23 @@ arm64|aarch64)
 	STATIC_BUILD=0
 	STRIP=aarch64-linux-android-strip
 	;;
-"x86")
+x64|x86_64)
+	NDK_ARCH=x86_64
+	export NDK_ARCH
+	STATIC_BUILD=0
+	STRIP=strip
+	;;
+x86)
 	NDK_ARCH=x86
 	STATIC_BUILD=0
 	STRIP=strip
 	;;
+x64-static|static-x64)
+	NDK_ARCH=x86
+	STATIC_BUILD=1
+	;;
 arm64-static|static-arm64)
-	NDK_ARCH=arm64
+	NDK_ARCH=aarch64
 	STATIC_BUILD=1
 	;;
 arm-static|static-arm)
@@ -60,13 +73,13 @@ x86-static|static-x86)
 	;;
 mips-static|static-mips)
 	NDK_ARCH=mips
-	# XXX: by default we should build all libs as .a but link binary dinamically
+	# XXX: by default we should build all libs as .a but link binary dynamically
 	STATIC_BUILD=1
 	STRIP=mips-linux-android-strip
 	;;
 mips64-static|static-mips64)
 	NDK_ARCH=mips64
-	# XXX: by default we should build all libs as .a but link binary dinamically
+	# XXX: by default we should build all libs as .a but link binary dynamically
 	STATIC_BUILD=1
 	STRIP=mips64el-linux-android-strip
 	;;
@@ -76,7 +89,7 @@ local)
 	NDK_ARCH=local
 	;;
 ""|"-h")
-	echo "Usage: android-build.sh [local|arm|arm64|x86|mips|mips64][-static]"
+	echo "Usage: android-build.sh [local|arm|arm64|x86|x64|mips|mips64][-static]"
 	exit 1
 	;;
 *)
@@ -106,7 +119,7 @@ if [ "${BUILD}" = 1 ]; then
 	sleep 1
 
 	if [ 1 = 1 ]; then
-		make mrproper
+		${MAKE} mrproper
 		if [ $STATIC_BUILD = 1 ]; then
 			CFGFLAGS="--with-libr"
 		fi
@@ -115,11 +128,11 @@ if [ "${BUILD}" = 1 ]; then
 			--with-ostype=android \
 			--without-libuv \
 			--prefix=${PREFIX} ${CFGFLAGS}
-
+		cp -f plugins.android.cfg plugins.cfg
 		./configure --with-compiler=android --without-libuv \
 			--with-ostype=android \
 			--prefix=${PREFIX} ${CFGFLAGS} || exit 1
-		make -s -j 4 || exit 1
+		${MAKE} -s -j 4 || exit 1
 	fi
 fi
 rm -rf $D
@@ -128,15 +141,15 @@ mkdir -p $D
 HERE=${PWD}
 INSTALL_PROGRAM=`grep INSTALL_DATA config-user.mk|cut -d = -f 2`
 
-make install INSTALL_PROGRAM="${INSTALL_PROGRAM}" DESTDIR="$HERE/$D" || exit 1
+${MAKE} install INSTALL_PROGRAM="${INSTALL_PROGRAM}" DESTDIR="$HERE/$D" || exit 1
 
-make purge-dev DESTDIR=${PWD}/${D} STRIP="${STRIP}"
+${MAKE} purge-dev DESTDIR="${PWD}/${D}" STRIP="${STRIP}"
 #make purge-doc DESTDIR=${PWD}/${D} STRIP="${STRIP}"
 #rm -rf ${PWD}/${D}/share
 rm -rf ${PWD}/${D}/include
 rm -rf ${PWD}/${D}/lib/pkgconfig
 rm -rf ${PWD}/${D}/lib/libsdb.a
-rm -rf "${HERE}/${D}/${PREFIX}/lib"
+#rm -rf "${HERE}/${D}/${PREFIX}/lib"
 
 rm -rf "${HERE}/${D}/${PREFIX}/radare2" # r2pm
 rm -rf "${HERE}/${D}/${PREFIX}/bin/r2pm"
@@ -148,8 +161,10 @@ rm -rf "${HERE}/${D}/${PREFIX}/bin/r2pm"
 
 # use busybox style symlinkz
 cd binr/blob
-make STATIC_BUILD=1 || exit 1
-make install PREFIX="${PREFIX}" DESTDIR="${HERE}/${D}" || exit 1
+#${MAKE} || exit 1
+#CFLAGS=-static LDFLAGS=-static ${MAKE} -j4 || exit 1
+${MAKE} -j4 || exit 1
+${MAKE} install PREFIX="${PREFIX}" DESTDIR="${HERE}/${D}" || exit 1
 mkdir -p ${HERE}/${D}/${PREFIX}/projects
 :> ${HERE}/${D}/${PREFIX}/projects/.empty
 mkdir -p ${HERE}/${D}/${PREFIX}/tmp
