@@ -1,11 +1,17 @@
 #!/bin/sh
 
+
+set -x
 STOW=0
 fromscratch=1 # 1
-onlydebug=0
 onlymakedeb=0
 static=1
 
+
+gcc -v 2> /dev/null
+if [ $? = 0 ]; then
+	export HOST_CC=gcc
+fi
 if [ -z "${CPU}" ]; then
 	export CPU=arm64
 	#export CPU=armv7
@@ -19,7 +25,7 @@ export BUILD=1
 if [ ! -d sys/ios-include/mach/vm_behavior.h  ]; then
 (
 	cd sys && \
-	wget http://lolcathost.org/b/ios-include.tar.gz && \
+	wget -c https://lolcathost.org/b/ios-include.tar.gz && \
 	tar xzvf ios-include.tar.gz
 )
 fi
@@ -31,36 +37,36 @@ else
 PREFIX=/usr
 fi
 
+ROOT=dist/cydia/radare2/root
+
 makeDeb() {
 	make -C binr ios-sdk-sign
 	rm -rf /tmp/r2ios
 	make install DESTDIR=/tmp/r2ios
-	rm -rf /tmp/r2ios/${PREFIX}/share/radare2/*/www/enyo/node_modules
+	rm -rf /tmp/r2ios/${PREFIX}/share/radare2/*/www/*/node_modules
 	( cd /tmp/r2ios && tar czvf ../r2ios-${CPU}.tar.gz ./* )
-	rm -rf sys/cydia/radare2/root
-	mkdir -p sys/cydia/radare2/root
-	sudo tar xpzvf /tmp/r2ios-${CPU}.tar.gz -C sys/cydia/radare2/root
-	rm -f sys/cydia/radare2/root/${PREFIX}/lib/*.dSYM
-	rm -f sys/cydia/radare2/root/${PREFIX}/lib/*.a
-	rm -f sys/cydia/radare2/root/${PREFIX}/lib/*.dylib
+	rm -rf "${ROOT}"
+	mkdir -p "${ROOT}"
+	sudo tar xpzvf /tmp/r2ios-${CPU}.tar.gz -C "${ROOT}"
+	rm -f ${ROOT}/${PREFIX}/lib/*.{a,dylib,dSYM}
 	if [ "$static" = 1 ]; then
 	(
-		rm -f sys/cydia/radare2/root/${PREFIX}/bin/*
-		cp -f binr/blob/radare2 sys/cydia/radare2/root/${PREFIX}/bin
-		cd sys/cydia/radare2/root/${PREFIX}/bin
+		rm -f ${ROOT}/${PREFIX}/bin/*
+		cp -f binr/blob/radare2 "${ROOT}/${PREFIX}/bin"
+		cd ${ROOT}/${PREFIX}/bin
 		for a in r2 rabin2 rarun2 rasm2 ragg2 rahash2 rax2 rafind2 radiff2 ; do ln -fs radare2 $a ; done
 	)
 		echo "Signing radare2"
-		ldid2 -Sbinr/radare2/radare2_ios.xml sys/cydia/radare2/root/usr/bin/radare2
+		ldid2 -Sbinr/radare2/radare2_ios.xml ${ROOT}/usr/bin/radare2
 	else
-		for a in sys/cydia/radare2/root/usr/bin/* sys/cydia/radare2/root/usr/lib/*.dylib ; do
+		for a in "${ROOT}/usr/bin/"* "${ROOT}/usr/lib/"*.dylib ; do
 			echo "Signing $a"
 			ldid2 -Sbinr/radare2/radare2_ios.xml $a
 		done
 	fi
-if [ "${STOW}" = 1 ]; then
-	(
-		cd sys/cydia/radare2/root/
+	if [ "${STOW}" = 1 ]; then
+		(
+		cd "${ROOT}/"
 		mkdir -p usr/bin
 		# stow
 		echo "Stowing ${PREFIX} into /usr..."
@@ -73,11 +79,11 @@ if [ "${STOW}" = 1 ]; then
 				done
 			fi
 		done
-	)
-else
-	echo "No need to stow anything"
-fi
-	( cd sys/cydia/radare2 ; sudo make clean ; sudo make PACKAGE=${PACKAGE} )
+		)
+	else
+		echo "No need to stow anything"
+	fi
+	( cd dist/cydia/radare2 ; sudo make clean ; sudo make PACKAGE=${PACKAGE} )
 }
 
 if [ "$1" = makedeb ]; then
@@ -87,28 +93,25 @@ fi
 if [ $onlymakedeb = 1 ]; then
 	makeDeb
 else
+	RV=0
+	export CC="ios-sdk-gcc"
 	if [ $fromscratch = 1 ]; then
-		if [ $onlydebug = 1 ]; then
-			(cd libr/debug ; make clean)
-			RV=0
+		make clean
+		cp -f dist/plugins-cfg/plugins.ios.cfg plugins.cfg
+		if [ "$static" = 1 ]; then
+			./configure --prefix="${PREFIX}" --with-ostype=darwin --without-libuv \
+			--with-compiler=ios-sdk --target=arm-unknown-darwin --with-libr
 		else
-			make clean
-			cp plugins.ios.cfg plugins.cfg
-			if [ "$static" = 1 ]; then
-				./configure --prefix="${PREFIX}" --with-ostype=darwin --without-libuv \
-				--with-compiler=ios-sdk --target=arm-unknown-darwin --with-libr
-			else
-				./configure --prefix="${PREFIX}" --with-ostype=darwin --without-libuv \
-				--with-compiler=ios-sdk --target=arm-unknown-darwin
-			fi
-			RV=$?
+			./configure --prefix="${PREFIX}" --with-ostype=darwin --without-libuv \
+			--with-compiler=ios-sdk --target=arm-unknown-darwin
 		fi
-	else
-		RV=0
+		RV=$?
 	fi
 	if [ $RV = 0 ]; then
-		time make -j4
+		time make -j4 || exit 1
 		if [ "$static" = 1 ]; then
+			ls -l libr/util/libr_util.a || exit 1
+			ls -l libr/flag/libr_flag.a || exit 1
 			rm -f libr/*/*.dylib
 			(
 			cd binr ; make clean ; 
@@ -116,8 +119,6 @@ else
 			xcrun --sdk iphoneos strip radare2
 			)
 		fi
-		if [ $? = 0 ]; then
-			makeDeb
-		fi
+		[ $? = 0 ] && makeDeb
 	fi
 fi
